// MD_ASSESSOBJ.JS
//
// Author:	Poundboy
// Email:	poundboy@megapathdsl.net
// Version:	1.01
//
// Description:
//
// Collection of Assess functions/objects for Skunkworks.
//
//
//
// Misc Methods:
//
//  AssessItem(ItemAco)			Assess an item and returns the result as several global variables.
//  AssessCre(CreAco)			Assess a creature and returns the result as several global variables.
//  BuildDesc(Aco)			Build a one line description of an item.
//  SetDebug(flag)			Change debug flag.
//
//
// The following variables are defined automatically since it is very
// unlikely you would ever need more than one of this object:
//
// Assess = new AssessObj();
//
//
// Modification History
//
// Who	Date	  Version	Description
//
// PBoy	09/21/02  1.01		Initial version




// **************************************************************************************************
// Assess Context Constants
//

// Miscellaneous Variables
AssessLog	= new LogObj();		// Log object used to output to console/chat window
AssessDebug	= false;		// If set to true, will output debugging information
Assessh	 	= new AssessHandlers();	// Set up handlers global variables
AssessCreActive	= false;

// Define array for display brief text information from assess operations
assessSkid				= new Array();
assessSkid[skidAxe]			= "Axe";
assessSkid[skidBow]			= "Bow";
assessSkid[skidCrossbow]		= "XBow";
assessSkid[skidDagger]			= "Dagger";
assessSkid[skidMace]			= "Mace";
assessSkid[skidMeleeDefense]		= "Melee";
assessSkid[skidMissileDefense]		= "Missle";
assessSkid[skidSpear]			= "Spear";
assessSkid[skidStaff]			= "Staff";
assessSkid[skidSword]			= "Sword";
assessSkid[skidThrownWeapons]		= "TW";
assessSkid[skidUnarmedCombat]		= "UA";
assessSkid[skidArcaneLore]		= "Arcane";
assessSkid[skidMagicDefense]		= "MagicD";
assessSkid[skidManaConversion]		= "ManaC";
assessSkid[skidItemTinkering]		= "ItemTink";
assessSkid[skidAssessPerson]		= "AssPer";
assessSkid[skidDeception]		= "Decep";
assessSkid[skidHealing]			= "Healing";
assessSkid[skidJump]			= "Jump";
assessSkid[skidLockpick]		= "LockP";
assessSkid[skidRun]			= "Run";
assessSkid[skidAssessCreature]		= "AssCre";
assessSkid[skidWeaponTinkering]		= "WeapTink";
assessSkid[skidArmorTinkering]		= "ArmorTink";
assessSkid[skidMagicItemTinkering]	= "MagicTink";
assessSkid[skidCreatureEnchantment]	= "CreMagic";
assessSkid[skidItemEnchantment]		= "ItemMagic";
assessSkid[skidLifeMagic]		= "LifeMagic";
assessSkid[skidWarMagic]		= "WarMagic";
assessSkid[skidLeadership]		= "Leader";
assessSkid[skidLoyalty]			= "Loyal";
assessSkid[skidFletching]		= "Fletch";
assessSkid[skidAlchemy]			= "Alchemy";
assessSkid[skidCooking]			= "Cooking";

// Bane abbreviations
assessSpell = new Array();
assessSpell[skapi.SpellidFromSz("Blood Drinker I")]		= "Bd1";
assessSpell[skapi.SpellidFromSz("Blood Drinker II")]		= "Bd2";
assessSpell[skapi.SpellidFromSz("Blood Drinker III")]		= "Bd3";
assessSpell[skapi.SpellidFromSz("Blood Drinker IV")]		= "Bd4";
assessSpell[skapi.SpellidFromSz("Blood Drinker V")]		= "Bd5";
assessSpell[skapi.SpellidFromSz("Blood Drinker VI")]		= "Bd6";
assessSpell[skapi.SpellidFromSz("Swift Killer I")]		= "Sk1";
assessSpell[skapi.SpellidFromSz("Swift Killer II")]		= "Sk2";
assessSpell[skapi.SpellidFromSz("Swift Killer III")]		= "Sk3";
assessSpell[skapi.SpellidFromSz("Swift Killer IV")]		= "Sk4";
assessSpell[skapi.SpellidFromSz("Swift Killer V")]		= "Sk5";
assessSpell[skapi.SpellidFromSz("Swift Killer VI")]		= "Sk6";
assessSpell[skapi.SpellidFromSz("Heart Seeker I")]		= "Hs1";
assessSpell[skapi.SpellidFromSz("Heart Seeker II")]		= "Hs2";
assessSpell[skapi.SpellidFromSz("Heart Seeker III")]		= "Hs3";
assessSpell[skapi.SpellidFromSz("Heart Seeker IV")]		= "Hs4";
assessSpell[skapi.SpellidFromSz("Heart Seeker V")]		= "Hs5";
assessSpell[skapi.SpellidFromSz("Heart Seeker VI")]		= "Hs6";
assessSpell[skapi.SpellidFromSz("Defender I")]			= "Def1";
assessSpell[skapi.SpellidFromSz("Defender II")]			= "Def2";
assessSpell[skapi.SpellidFromSz("Defender III")]		= "Def3";
assessSpell[skapi.SpellidFromSz("Defender IV")]			= "Def4";
assessSpell[skapi.SpellidFromSz("Defender V")]			= "Def5";
assessSpell[skapi.SpellidFromSz("Defender VI")]			= "Def6";
assessSpell[skapi.SpellidFromSz("Impenetrability I")]		= "Impen1";
assessSpell[skapi.SpellidFromSz("Impenetrability II")]		= "Impen2";
assessSpell[skapi.SpellidFromSz("Impenetrability III")]		= "Impen3";
assessSpell[skapi.SpellidFromSz("Impenetrability IV")]		= "Impen4";
assessSpell[skapi.SpellidFromSz("Impenetrability V")]		= "Impen5";
assessSpell[skapi.SpellidFromSz("Impenetrability VI")]		= "Impen6";
assessSpell[skapi.SpellidFromSz("Acid Bane I")]			= "Acid1";
assessSpell[skapi.SpellidFromSz("Acid Bane II")]		= "Acid2";
assessSpell[skapi.SpellidFromSz("Acid Bane III")]		= "Acid3";
assessSpell[skapi.SpellidFromSz("Acid Bane IV")]		= "Acid4";
assessSpell[skapi.SpellidFromSz("Acid Bane V")]			= "Acid5";
assessSpell[skapi.SpellidFromSz("Acid Bane VI")]		= "Acid6";
assessSpell[skapi.SpellidFromSz("Blade Bane I")]		= "Blade1";
assessSpell[skapi.SpellidFromSz("Blade Bane II")]		= "Blade2";
assessSpell[skapi.SpellidFromSz("Blade Bane III")]		= "Blade3";
assessSpell[skapi.SpellidFromSz("Blade Bane IV")]		= "Blade4";
assessSpell[skapi.SpellidFromSz("Blade Bane V")]		= "Blade5";
assessSpell[skapi.SpellidFromSz("Blade Bane VI")]		= "Blade6";
assessSpell[skapi.SpellidFromSz("Bludgeon Bane I")]		= "Bludg1";
assessSpell[skapi.SpellidFromSz("Bludgeon Bane II")]		= "Bludg2";
assessSpell[skapi.SpellidFromSz("Bludgeon Bane III")]		= "Bludg3";
assessSpell[skapi.SpellidFromSz("Bludgeon Bane IV")]		= "Bludg4";
assessSpell[skapi.SpellidFromSz("Bludgeon Bane V")]		= "Bludg5";
assessSpell[skapi.SpellidFromSz("Bludgeon Bane VI")]		= "Bludg6";
assessSpell[skapi.SpellidFromSz("Flame Bane I")]		= "Flame1";
assessSpell[skapi.SpellidFromSz("Flame Bane II")]		= "Flame2";
assessSpell[skapi.SpellidFromSz("Flame Bane III")]		= "Flame3";
assessSpell[skapi.SpellidFromSz("Flame Bane IV")]		= "Flame4";
assessSpell[skapi.SpellidFromSz("Flame Bane V")]		= "Flame5";
assessSpell[skapi.SpellidFromSz("Flame Bane VI")]		= "Flame6";
assessSpell[skapi.SpellidFromSz("Frost Bane I")]		= "Frost1";
assessSpell[skapi.SpellidFromSz("Frost Bane II")]		= "Frost2";
assessSpell[skapi.SpellidFromSz("Frost Bane III")]		= "Frost3";
assessSpell[skapi.SpellidFromSz("Frost Bane IV")]		= "Frost4";
assessSpell[skapi.SpellidFromSz("Frost Bane V")]		= "Frost5";
assessSpell[skapi.SpellidFromSz("Frost Bane VI")]		= "Frost6";
assessSpell[skapi.SpellidFromSz("Lightning Bane I")]		= "Light1";
assessSpell[skapi.SpellidFromSz("Lightning Bane II")]		= "Light2";
assessSpell[skapi.SpellidFromSz("Lightning Bane III")]		= "Light3";
assessSpell[skapi.SpellidFromSz("Lightning Bane IV")]		= "Light4";
assessSpell[skapi.SpellidFromSz("Lightning Bane V")]		= "Light5";
assessSpell[skapi.SpellidFromSz("Lightning Bane VI")]		= "Light6";
assessSpell[skapi.SpellidFromSz("Piercing Bane I")]		= "Pierc1";
assessSpell[skapi.SpellidFromSz("Piercing Bane II")]		= "Pierc2";
assessSpell[skapi.SpellidFromSz("Piercing Bane III")]		= "Pierc3";
assessSpell[skapi.SpellidFromSz("Piercing Bane IV")]		= "Pierc4";
assessSpell[skapi.SpellidFromSz("Piercing Bane V")]		= "Pierc5";
assessSpell[skapi.SpellidFromSz("Piercing Bane VI")]		= "Pierc6";

// Attribute abbreviations
assessSpell[skapi.SpellidFromSz("Strength Self I")]		= "Strength1";
assessSpell[skapi.SpellidFromSz("Strength Self II")]		= "Strength2";
assessSpell[skapi.SpellidFromSz("Strength Self III")]		= "Strength3";
assessSpell[skapi.SpellidFromSz("Strength Self IV")]		= "Strength4";
assessSpell[skapi.SpellidFromSz("Strength Self V")]		= "Strength5";
assessSpell[skapi.SpellidFromSz("Strength Self VI")]		= "Strength6";
assessSpell[skapi.SpellidFromSz("Coordination Self I")]		= "Coord1";
assessSpell[skapi.SpellidFromSz("Coordination Self II")]	= "Coord2";
assessSpell[skapi.SpellidFromSz("Coordination Self III")]	= "Coord3";
assessSpell[skapi.SpellidFromSz("Coordination Self IV")]	= "Coord4";
assessSpell[skapi.SpellidFromSz("Coordination Self V")]		= "Coord5";
assessSpell[skapi.SpellidFromSz("Coordination Self VI")]	= "Coord6";
assessSpell[skapi.SpellidFromSz("Endurance Self I")]		= "Endur1";
assessSpell[skapi.SpellidFromSz("Endurance Self II")]		= "Endur2";
assessSpell[skapi.SpellidFromSz("Endurance Self III")]		= "Endur3";
assessSpell[skapi.SpellidFromSz("Endurance Self IV")]		= "Endur4";
assessSpell[skapi.SpellidFromSz("Endurance Self V")]		= "Endur5";
assessSpell[skapi.SpellidFromSz("Endurance Self VI")]		= "Endur6";
assessSpell[skapi.SpellidFromSz("Quickness Self I")]		= "Quick1";
assessSpell[skapi.SpellidFromSz("Quickness Self II")]		= "Quick2";
assessSpell[skapi.SpellidFromSz("Quickness Self III")]		= "Quick3";
assessSpell[skapi.SpellidFromSz("Quickness Self IV")]		= "Quick4";
assessSpell[skapi.SpellidFromSz("Quickness Self V")]		= "Quick5";
assessSpell[skapi.SpellidFromSz("Quickness Self VI")]		= "Quick6";
assessSpell[skapi.SpellidFromSz("Focus Self I")]		= "Focus1";
assessSpell[skapi.SpellidFromSz("Focus Self II")]		= "Focus2";
assessSpell[skapi.SpellidFromSz("Focus Self III")]		= "Focus3";
assessSpell[skapi.SpellidFromSz("Focus Self IV")]		= "Focus4";
assessSpell[skapi.SpellidFromSz("Focus Self V")]		= "Focus5";
assessSpell[skapi.SpellidFromSz("Focus Self VI")]		= "Focus6";
assessSpell[skapi.SpellidFromSz("Willpower Self I")]		= "Willpower1";
assessSpell[skapi.SpellidFromSz("Willpower Self II")]		= "Willpower2";
assessSpell[skapi.SpellidFromSz("Willpower Self III")]		= "Willpower3";
assessSpell[skapi.SpellidFromSz("Willpower Self IV")]		= "Willpower4";
assessSpell[skapi.SpellidFromSz("Willpower Self V")]		= "Willpower5";
assessSpell[skapi.SpellidFromSz("Willpower Self VI")]		= "Willpower6";

// Melee Skill abbreviations
assessSpell[skapi.SpellidFromSz("Axe Mastery Self I")]			= "Axe1";
assessSpell[skapi.SpellidFromSz("Axe Mastery Self II")]			= "Axe2";
assessSpell[skapi.SpellidFromSz("Axe Mastery Self III")]		= "Axe3";
assessSpell[skapi.SpellidFromSz("Axe Mastery Self IV")]			= "Axe4";
assessSpell[skapi.SpellidFromSz("Axe Mastery Self V")]			= "Axe5";
assessSpell[skapi.SpellidFromSz("Axe Mastery Self VI")]			= "Axe6";
assessSpell[skapi.SpellidFromSz("Bow Mastery Self I")]			= "Bow1";
assessSpell[skapi.SpellidFromSz("Bow Mastery Self II")]			= "Bow2";
assessSpell[skapi.SpellidFromSz("Bow Mastery Self III")]		= "Bow3";
assessSpell[skapi.SpellidFromSz("Bow Mastery Self IV")]			= "Bow4";
assessSpell[skapi.SpellidFromSz("Bow Mastery Self V")]			= "Bow5";
assessSpell[skapi.SpellidFromSz("Bow Mastery Self VI")]			= "Bow6";
assessSpell[skapi.SpellidFromSz("Crossbow Mastery Self I")]		= "XBow1";
assessSpell[skapi.SpellidFromSz("Crossbow Mastery Self II")]		= "XBow2";
assessSpell[skapi.SpellidFromSz("Crossbow Mastery Self III")]		= "XBow3";
assessSpell[skapi.SpellidFromSz("Crossbow Mastery Self IV")]		= "XBow4";
assessSpell[skapi.SpellidFromSz("Crossbow Mastery Self V")]		= "XBow5";
assessSpell[skapi.SpellidFromSz("Crossbow Mastery Self VI")]		= "XBow6";
assessSpell[skapi.SpellidFromSz("Dagger Mastery Self I")]		= "Dagger1";
assessSpell[skapi.SpellidFromSz("Dagger Mastery Self II")]		= "Dagger2";
assessSpell[skapi.SpellidFromSz("Dagger Mastery Self III")]		= "Dagger3";
assessSpell[skapi.SpellidFromSz("Dagger Mastery Self IV")]		= "Dagger4";
assessSpell[skapi.SpellidFromSz("Dagger Mastery Self V")]		= "Dagger5";
assessSpell[skapi.SpellidFromSz("Dagger Mastery Self VI")]		= "Dagger6";
assessSpell[skapi.SpellidFromSz("Mace Mastery Self I")]			= "Mace1";
assessSpell[skapi.SpellidFromSz("Mace Mastery Self II")]		= "Mace2";
assessSpell[skapi.SpellidFromSz("Mace Mastery Self III")]		= "Mace3";
assessSpell[skapi.SpellidFromSz("Mace Mastery Self IV")]		= "Mace4";
assessSpell[skapi.SpellidFromSz("Mace Mastery Self V")]			= "Mace5";
assessSpell[skapi.SpellidFromSz("Mace Mastery Self VI")]		= "Mace6";
assessSpell[skapi.SpellidFromSz("Spear Mastery Self I")]		= "Spear1";
assessSpell[skapi.SpellidFromSz("Spear Mastery Self II")]		= "Spear2";
assessSpell[skapi.SpellidFromSz("Spear Mastery Self III")]		= "Spear3";
assessSpell[skapi.SpellidFromSz("Spear Mastery Self IV")]		= "Spear4";
assessSpell[skapi.SpellidFromSz("Spear Mastery Self V")]		= "Spear5";
assessSpell[skapi.SpellidFromSz("Spear Mastery Self VI")]		= "Spear6";
assessSpell[skapi.SpellidFromSz("Staff Mastery Self I")]		= "Staff1";
assessSpell[skapi.SpellidFromSz("Staff Mastery Self II")]		= "Staff2";
assessSpell[skapi.SpellidFromSz("Staff Mastery Self III")]		= "Staff3";
assessSpell[skapi.SpellidFromSz("Staff Mastery Self IV")]		= "Staff4";
assessSpell[skapi.SpellidFromSz("Staff Mastery Self V")]		= "Staff5";
assessSpell[skapi.SpellidFromSz("Staff Mastery Self VI")]		= "Staff6";
assessSpell[skapi.SpellidFromSz("Sword Mastery Self I")]		= "Sword1";
assessSpell[skapi.SpellidFromSz("Sword Mastery Self II")]		= "Sword2";
assessSpell[skapi.SpellidFromSz("Sword Mastery Self III")]		= "Sword3";
assessSpell[skapi.SpellidFromSz("Sword Mastery Self IV")]		= "Sword4";
assessSpell[skapi.SpellidFromSz("Sword Mastery Self V")]		= "Sword5";
assessSpell[skapi.SpellidFromSz("Sword Mastery Self VI")]		= "Sword6";
assessSpell[skapi.SpellidFromSz("Thrown Weapons Mastery Self I")]   	= "TW1";
assessSpell[skapi.SpellidFromSz("Thrown Weapons Mastery Self II")]  	= "TW2";
assessSpell[skapi.SpellidFromSz("Thrown Weapons Mastery Self III")] 	= "TW3";
assessSpell[skapi.SpellidFromSz("Thrown Weapons Mastery Self IV")]  	= "TW4";
assessSpell[skapi.SpellidFromSz("Thrown Weapons Mastery Self V")]   	= "TW5";
assessSpell[skapi.SpellidFromSz("Thrown Weapons Mastery Self VI")]  	= "TW6";
assessSpell[skapi.SpellidFromSz("Unarmed Combat Mastery Self I")]   	= "UA1";
assessSpell[skapi.SpellidFromSz("Unarmed Combat Mastery Self II")]  	= "UA2";
assessSpell[skapi.SpellidFromSz("Unarmed Combat Mastery Self III")] 	= "UA3";
assessSpell[skapi.SpellidFromSz("Unarmed Combat Mastery Self IV")]  	= "UA4";
assessSpell[skapi.SpellidFromSz("Unarmed Combat Mastery Self V")]   	= "UA5";
assessSpell[skapi.SpellidFromSz("Unarmed Combat Mastery Self VI")]  	= "UA6";

// Misc skill abbreviations
assessSpell[skapi.SpellidFromSz("Arcane Enlightenment Self I")]		= "Arcane1"
assessSpell[skapi.SpellidFromSz("Arcane Enlightenment Self II")]	= "Arcane2"
assessSpell[skapi.SpellidFromSz("Arcane Enlightenment Self III")]	= "Arcane3"
assessSpell[skapi.SpellidFromSz("Arcane Enlightenment Self IV")]	= "Arcane4"
assessSpell[skapi.SpellidFromSz("Arcane Enlightenment Self V")]		= "Arcane5"
assessSpell[skapi.SpellidFromSz("Arcane Enlightenment Self VI")]	= "Arcane6"
assessSpell[skapi.SpellidFromSz("Armor Tinkering Expertise Self I")]	= "ArmorTink1";
assessSpell[skapi.SpellidFromSz("Armor Tinkering Expertise Self II")]	= "ArmorTink2";
assessSpell[skapi.SpellidFromSz("Armor Tinkering Expertise Self III")]	= "ArmorTink3";
assessSpell[skapi.SpellidFromSz("Armor Tinkering Expertise Self IV")]	= "ArmorTink4";
assessSpell[skapi.SpellidFromSz("Armor Tinkering Expertise Self V")]	= "ArmorTink5";
assessSpell[skapi.SpellidFromSz("Armor Tinkering Expertise Self VI")]	= "ArmorTink6";
assessSpell[skapi.SpellidFromSz("Creature Enchantment Mastery Self I")]	  = "CreMast1";
assessSpell[skapi.SpellidFromSz("Creature Enchantment Mastery Self II")]  = "CreMast2";
assessSpell[skapi.SpellidFromSz("Creature Enchantment Mastery Self III")] = "CreMast3";
assessSpell[skapi.SpellidFromSz("Creature Enchantment Mastery Self IV")]  = "CreMast4";
assessSpell[skapi.SpellidFromSz("Creature Enchantment Mastery Self V")]	  = "CreMast5";
assessSpell[skapi.SpellidFromSz("Creature Enchantment Mastery Self VI")]  = "CreMast6";
assessSpell[skapi.SpellidFromSz("Deception Mastery Self I")]		= "Decept1"
assessSpell[skapi.SpellidFromSz("Deception Mastery Self II")]		= "Decept2"
assessSpell[skapi.SpellidFromSz("Deception Mastery Self III")]		= "Decept3"
assessSpell[skapi.SpellidFromSz("Deception Mastery Self IV")]		= "Decept4"
assessSpell[skapi.SpellidFromSz("Deception Mastery Self V")]		= "Decept5"
assessSpell[skapi.SpellidFromSz("Deception Mastery Self VI")]		= "Decept6"
assessSpell[skapi.SpellidFromSz("Fealty Self I")]			= "Fealty1";
assessSpell[skapi.SpellidFromSz("Fealty Self II")]			= "Fealty2";
assessSpell[skapi.SpellidFromSz("Fealty Self III")]			= "Fealty3";
assessSpell[skapi.SpellidFromSz("Fealty Self IV")]			= "Fealty4";
assessSpell[skapi.SpellidFromSz("Fealty Self V")]			= "Fealty5";
assessSpell[skapi.SpellidFromSz("Fealty Self VI")]			= "Fealty6";
assessSpell[skapi.SpellidFromSz("Healing Mastery Self I")]		= "Heal1";
assessSpell[skapi.SpellidFromSz("Healing Mastery Self II")]		= "Heal2";
assessSpell[skapi.SpellidFromSz("Healing Mastery Self III")]		= "Heal3";
assessSpell[skapi.SpellidFromSz("Healing Mastery Self IV")]		= "Heal4";
assessSpell[skapi.SpellidFromSz("Healing Mastery Self V")]		= "Heal5";
assessSpell[skapi.SpellidFromSz("Healing Mastery Self VI")]		= "Heal6";
assessSpell[skapi.SpellidFromSz("Jumping Mastery Self I")]		= "Jump1";
assessSpell[skapi.SpellidFromSz("Jumping Mastery Self II")]		= "Jump2";
assessSpell[skapi.SpellidFromSz("Jumping Mastery Self III")]		= "Jump3";
assessSpell[skapi.SpellidFromSz("Jumping Mastery Self IV")]		= "Jump4";
assessSpell[skapi.SpellidFromSz("Jumping Mastery Self V")]		= "Jump5";
assessSpell[skapi.SpellidFromSz("Jumping Mastery Self VI")]		= "Jump6";
assessSpell[skapi.SpellidFromSz("Impregnability Self I")]		= "Impreg1"
assessSpell[skapi.SpellidFromSz("Impregnability Self II")]		= "Impreg2"
assessSpell[skapi.SpellidFromSz("Impregnability Self III")]		= "Impreg3"
assessSpell[skapi.SpellidFromSz("Impregnability Self IV")]		= "Impreg4"
assessSpell[skapi.SpellidFromSz("Impregnability Self V")]		= "Impreg5"
assessSpell[skapi.SpellidFromSz("Impregnability Self VI")]		= "Impreg6"
assessSpell[skapi.SpellidFromSz("Invulnerability Self I")]		= "Invuln1"
assessSpell[skapi.SpellidFromSz("Invulnerability Self II")]		= "Invuln2"
assessSpell[skapi.SpellidFromSz("Invulnerability Self III")]		= "Invuln3"
assessSpell[skapi.SpellidFromSz("Invulnerability Self IV")]		= "Invuln4"
assessSpell[skapi.SpellidFromSz("Invulnerability Self V")]		= "Invuln5"
assessSpell[skapi.SpellidFromSz("Invulnerability Self VI")]		= "Invuln6"
assessSpell[skapi.SpellidFromSz("Item Enchantment Mastery Self I")]	= "ItemMast1";
assessSpell[skapi.SpellidFromSz("Item Enchantment Mastery Self II")]	= "ItemMast2";
assessSpell[skapi.SpellidFromSz("Item Enchantment Mastery Self III")]	= "ItemMast3";
assessSpell[skapi.SpellidFromSz("Item Enchantment Mastery Self IV")]	= "ItemMast4";
assessSpell[skapi.SpellidFromSz("Item Enchantment Mastery Self V")]	= "ItemMast5";
assessSpell[skapi.SpellidFromSz("Item Enchantment Mastery Self VI")]	= "ItemMast6";
assessSpell[skapi.SpellidFromSz("Item Tinkering Expertise Self I")]	= "ItemTink1";
assessSpell[skapi.SpellidFromSz("Item Tinkering Expertise Self II")]	= "ItemTink2";
assessSpell[skapi.SpellidFromSz("Item Tinkering Expertise Self III")]	= "ItemTink3";
assessSpell[skapi.SpellidFromSz("Item Tinkering Expertise Self IV")]	= "ItemTink4";
assessSpell[skapi.SpellidFromSz("Item Tinkering Expertise Self V")]	= "ItemTink5";
assessSpell[skapi.SpellidFromSz("Item Tinkering Expertise Self VI")]	= "ItemTink6";
assessSpell[skapi.SpellidFromSz("Leadership Mastery Self I")]		= "Leader1";
assessSpell[skapi.SpellidFromSz("Leadership Mastery Self II")]		= "Leader2";
assessSpell[skapi.SpellidFromSz("Leadership Mastery Self III")]		= "Leader3";
assessSpell[skapi.SpellidFromSz("Leadership Mastery Self IV")]		= "Leader4";
assessSpell[skapi.SpellidFromSz("Leadership Mastery Self V")]		= "Leader5";
assessSpell[skapi.SpellidFromSz("Leadership Mastery Self VI")]		= "Leader6";
assessSpell[skapi.SpellidFromSz("Life Magic Mastery Self I")]		= "LifeMast1";
assessSpell[skapi.SpellidFromSz("Life Magic Mastery Self II")]		= "LifeMast2";
assessSpell[skapi.SpellidFromSz("Life Magic Mastery Self III")]		= "LifeMast3";
assessSpell[skapi.SpellidFromSz("Life Magic Mastery Self IV")]		= "LifeMast4";
assessSpell[skapi.SpellidFromSz("Life Magic Mastery Self V")]		= "LifeMast5";
assessSpell[skapi.SpellidFromSz("Life Magic Mastery Self VI")]		= "LifeMast6";
assessSpell[skapi.SpellidFromSz("Lockpick Mastery Self I")]		= "Lockpick1"
assessSpell[skapi.SpellidFromSz("Lockpick Mastery Self II")]		= "Lockpick2"
assessSpell[skapi.SpellidFromSz("Lockpick Mastery Self III")]		= "Lockpick3"
assessSpell[skapi.SpellidFromSz("Lockpick Mastery Self IV")]		= "Lockpick4"
assessSpell[skapi.SpellidFromSz("Lockpick Mastery Self V")]		= "Lockpick5"
assessSpell[skapi.SpellidFromSz("Lockpick Mastery Self VI")]		= "Lockpick6"
assessSpell[skapi.SpellidFromSz("Magic Item Tinkering Expertise Self I")]   = "MagicTink1";
assessSpell[skapi.SpellidFromSz("Magic Item Tinkering Expertise Self II")]  = "MagicTink2";
assessSpell[skapi.SpellidFromSz("Magic Item Tinkering Expertise Self III")] = "MagicTink3";
assessSpell[skapi.SpellidFromSz("Magic Item Tinkering Expertise Self IV")]  = "MagicTink4";
assessSpell[skapi.SpellidFromSz("Magic Item Tinkering Expertise Self V")]   = "MagicTink5";
assessSpell[skapi.SpellidFromSz("Magic Item Tinkering Expertise Self VI")]  = "MagicTink6";
assessSpell[skapi.SpellidFromSz("Magic Resistance Self I")]		= "MagicResist1"
assessSpell[skapi.SpellidFromSz("Magic Resistance Self II")]		= "MagicResist2"
assessSpell[skapi.SpellidFromSz("Magic Resistance Self III")]		= "MagicResist3"
assessSpell[skapi.SpellidFromSz("Magic Resistance Self IV")]		= "MagicResist4"
assessSpell[skapi.SpellidFromSz("Magic Resistance Self V")]		= "MagicResist5"
assessSpell[skapi.SpellidFromSz("Magic Resistance Self VI")]		= "MagicResist6"
assessSpell[skapi.SpellidFromSz("Mana Conversion Mastery Self I")]	= "ManaMast1";
assessSpell[skapi.SpellidFromSz("Mana Conversion Mastery Self II")]	= "ManaMast2";
assessSpell[skapi.SpellidFromSz("Mana Conversion Mastery Self III")]	= "ManaMast3";
assessSpell[skapi.SpellidFromSz("Mana Conversion Mastery Self IV")]	= "ManaMast4";
assessSpell[skapi.SpellidFromSz("Mana Conversion Mastery Self V")]	= "ManaMast5";
assessSpell[skapi.SpellidFromSz("Mana Conversion Mastery Self VI")]	= "ManaMast6";
assessSpell[skapi.SpellidFromSz("Person Attunement Self I")]		= "Person1"
assessSpell[skapi.SpellidFromSz("Person Attunement Self II")]		= "Person2"
assessSpell[skapi.SpellidFromSz("Person Attunement Self III")]		= "Person3"
assessSpell[skapi.SpellidFromSz("Person Attunement Self IV")]		= "Person4"
assessSpell[skapi.SpellidFromSz("Person Attunement Self V")]		= "Person5"
assessSpell[skapi.SpellidFromSz("Person Attunement Self VI")]		= "Person6"
assessSpell[skapi.SpellidFromSz("Sprint Self I")]			= "Sprint1"
assessSpell[skapi.SpellidFromSz("Sprint Self II")]			= "Sprint2"
assessSpell[skapi.SpellidFromSz("Sprint Self III")]			= "Sprint3"
assessSpell[skapi.SpellidFromSz("Sprint Self IV")]			= "Sprint4"
assessSpell[skapi.SpellidFromSz("Sprint Self V")]			= "Sprint5"
assessSpell[skapi.SpellidFromSz("Sprint Self VI")]			= "Sprint6"
assessSpell[skapi.SpellidFromSz("War Magic Mastery Self I")]		= "WarMast1";
assessSpell[skapi.SpellidFromSz("War Magic Mastery Self II")]		= "WarMast2";
assessSpell[skapi.SpellidFromSz("War Magic Mastery Self III")]		= "WarMast3";
assessSpell[skapi.SpellidFromSz("War Magic Mastery Self IV")]		= "WarMast4";
assessSpell[skapi.SpellidFromSz("War Magic Mastery Self V")]		= "WarMast5";
assessSpell[skapi.SpellidFromSz("War Magic Mastery Self VI")]		= "WarMast6";
assessSpell[skapi.SpellidFromSz("Weapon Tinkering Expertise Self I")]	= "WeapTink1";
assessSpell[skapi.SpellidFromSz("Weapon Tinkering Expertise Self II")]	= "WeapTink2";
assessSpell[skapi.SpellidFromSz("Weapon Tinkering Expertise Self III")]	= "WeapTink3";
assessSpell[skapi.SpellidFromSz("Weapon Tinkering Expertise Self IV")]	= "WeapTink4";
assessSpell[skapi.SpellidFromSz("Weapon Tinkering Expertise Self V")]	= "WeapTink5";
assessSpell[skapi.SpellidFromSz("Weapon Tinkering Expertise Self VI")]	= "WeapTink6";

// Life prots abbreviations
assessSpell[skapi.SpellidFromSz("Armor Self I")]			= "Armor1";
assessSpell[skapi.SpellidFromSz("Armor Self II")]			= "Armor2";
assessSpell[skapi.SpellidFromSz("Armor Self III")]			= "Armor3";
assessSpell[skapi.SpellidFromSz("Armor Self IV")]			= "Armor4";
assessSpell[skapi.SpellidFromSz("Armor Self V")]			= "Armor5";
assessSpell[skapi.SpellidFromSz("Armor Self VI")]			= "Armor6";
assessSpell[skapi.SpellidFromSz("Acid Protection Self I")]		= "AcidProt1"
assessSpell[skapi.SpellidFromSz("Acid Protection Self II")]		= "AcidProt2"
assessSpell[skapi.SpellidFromSz("Acid Protection Self III")]		= "AcidProt3"
assessSpell[skapi.SpellidFromSz("Acid Protection Self IV")]		= "AcidProt4"
assessSpell[skapi.SpellidFromSz("Acid Protection Self V")]		= "AcidProt5"
assessSpell[skapi.SpellidFromSz("Acid Protection Self VI")]		= "AcidProt6"
assessSpell[skapi.SpellidFromSz("Blade Protection Self I")]		= "BladeProt1"
assessSpell[skapi.SpellidFromSz("Blade Protection Self II")]		= "BladeProt2"
assessSpell[skapi.SpellidFromSz("Blade Protection Self III")]		= "BladeProt3"
assessSpell[skapi.SpellidFromSz("Blade Protection Self IV")]		= "BladeProt4"
assessSpell[skapi.SpellidFromSz("Blade Protection Self V")]		= "BladeProt5"
assessSpell[skapi.SpellidFromSz("Blade Protection Self VI")]		= "BladeProt6"
assessSpell[skapi.SpellidFromSz("Bludgeoning Protection Self I")]	= "BludgProt1"
assessSpell[skapi.SpellidFromSz("Bludgeoning Protection Self II")]	= "BludgProt2"
assessSpell[skapi.SpellidFromSz("Bludgeoning Protection Self III")]	= "BludgProt3"
assessSpell[skapi.SpellidFromSz("Bludgeoning Protection Self IV")]	= "BludgProt4"
assessSpell[skapi.SpellidFromSz("Bludgeoning Protection Self V")]	= "BludgProt5"
assessSpell[skapi.SpellidFromSz("Bludgeoning Protection Self VI")]	= "BludgProt6"
assessSpell[skapi.SpellidFromSz("Cold Protection Self I")]		= "ColdProt1"
assessSpell[skapi.SpellidFromSz("Cold Protection Self II")]		= "ColdProt2"
assessSpell[skapi.SpellidFromSz("Cold Protection Self III")]		= "ColdProt3"
assessSpell[skapi.SpellidFromSz("Cold Protection Self IV")]		= "ColdProt4"
assessSpell[skapi.SpellidFromSz("Cold Protection Self V")]		= "ColdProt5"
assessSpell[skapi.SpellidFromSz("Cold Protection Self VI")]		= "ColdProt6"
assessSpell[skapi.SpellidFromSz("Fire Protection Self I")]		= "FireProt1"
assessSpell[skapi.SpellidFromSz("Fire Protection Self II")]		= "FireProt2"
assessSpell[skapi.SpellidFromSz("Fire Protection Self III")]		= "FireProt3"
assessSpell[skapi.SpellidFromSz("Fire Protection Self IV")]		= "FireProt4"
assessSpell[skapi.SpellidFromSz("Fire Protection Self V")]		= "FireProt5"
assessSpell[skapi.SpellidFromSz("Fire Protection Self VI")]		= "FireProt6"
assessSpell[skapi.SpellidFromSz("Lightning Protection Self I")]		= "LightProt1"
assessSpell[skapi.SpellidFromSz("Lightning Protection Self II")]	= "LightProt2"
assessSpell[skapi.SpellidFromSz("Lightning Protection Self III")]	= "LightProt3"
assessSpell[skapi.SpellidFromSz("Lightning Protection Self IV")]	= "LightProt4"
assessSpell[skapi.SpellidFromSz("Lightning Protection Self V")]		= "LightProt5"
assessSpell[skapi.SpellidFromSz("Lightning Protection Self VI")]	= "LightProt6"
assessSpell[skapi.SpellidFromSz("Piercing Protection Self I")]		= "PiercProt1"
assessSpell[skapi.SpellidFromSz("Piercing Protection Self II")]		= "PiercProt2"
assessSpell[skapi.SpellidFromSz("Piercing Protection Self III")]	= "PiercProt3"
assessSpell[skapi.SpellidFromSz("Piercing Protection Self IV")]		= "PiercProt4"
assessSpell[skapi.SpellidFromSz("Piercing Protection Self V")]		= "PiercProt5"
assessSpell[skapi.SpellidFromSz("Piercing Protection Self VI")]		= "PiercProt6"

// Misc Life magic abbreviations
assessSpell[skapi.SpellidFromSz("Mana Renewal Self I")]			= "ManaRenew1";
assessSpell[skapi.SpellidFromSz("Mana Renewal Self II")]		= "ManaRenew2";
assessSpell[skapi.SpellidFromSz("Mana Renewal Self III")]		= "ManaRenew3";
assessSpell[skapi.SpellidFromSz("Mana Renewal Self IV")]		= "ManaRenew4";
assessSpell[skapi.SpellidFromSz("Mana Renewal Self V")]			= "ManaRenew5";
assessSpell[skapi.SpellidFromSz("Mana Renewal Self VI")]		= "ManaRenew6";
assessSpell[skapi.SpellidFromSz("Regeneration Self I")]			= "Regen1"
assessSpell[skapi.SpellidFromSz("Regeneration Self II")]		= "Regen2"
assessSpell[skapi.SpellidFromSz("Regeneration Self III")]		= "Regen3"
assessSpell[skapi.SpellidFromSz("Regeneration Self IV")]		= "Regen4"
assessSpell[skapi.SpellidFromSz("Regeneration Self V")]			= "Regen5"
assessSpell[skapi.SpellidFromSz("Regeneration Self VI")]		= "Regen6"
assessSpell[skapi.SpellidFromSz("Rejuvenation Self I")]			= "Rejuv1";
assessSpell[skapi.SpellidFromSz("Rejuvenation Self II")]		= "Rejuv2";
assessSpell[skapi.SpellidFromSz("Rejuvenation Self III")]		= "Rejuv3";
assessSpell[skapi.SpellidFromSz("Rejuvenation Self IV")]		= "Rejuv4";
assessSpell[skapi.SpellidFromSz("Rejuvenation Self V")]			= "Rejuv5";
assessSpell[skapi.SpellidFromSz("Rejuvenation Self VI")]		= "Rejuv6";

// War spell abbreviations
assessSpell[skapi.SpellidFromSz("Acid Stream I")]		= "Acid1";
assessSpell[skapi.SpellidFromSz("Acid Stream II")]		= "Acid2";
assessSpell[skapi.SpellidFromSz("Acid Stream III")]		= "Acid3";
assessSpell[skapi.SpellidFromSz("Acid Stream IV")]		= "Acid4";
assessSpell[skapi.SpellidFromSz("Acid Stream V")]		= "Acid5";
assessSpell[skapi.SpellidFromSz("Acid Stream VI")]		= "Acid6";
assessSpell[skapi.SpellidFromSz("Flame Bolt I")]		= "Flame1";
assessSpell[skapi.SpellidFromSz("Flame Bolt II")]		= "Flame2";
assessSpell[skapi.SpellidFromSz("Flame Bolt III")]		= "Flame3";
assessSpell[skapi.SpellidFromSz("Flame Bolt IV")]		= "Flame4";
assessSpell[skapi.SpellidFromSz("Flame Bolt V")]		= "Flame5";
assessSpell[skapi.SpellidFromSz("Flame Bolt VI")]		= "Flame6";
assessSpell[skapi.SpellidFromSz("Force Bolt I")]		= "Force1";
assessSpell[skapi.SpellidFromSz("Force Bolt II")]		= "Force2";
assessSpell[skapi.SpellidFromSz("Force Bolt III")]		= "Force3";
assessSpell[skapi.SpellidFromSz("Force Bolt IV")]		= "Force4";
assessSpell[skapi.SpellidFromSz("Force Bolt V")]		= "Force5";
assessSpell[skapi.SpellidFromSz("Force Bolt VI")]		= "Force6";
assessSpell[skapi.SpellidFromSz("Frost Bolt I")]		= "Frost1";
assessSpell[skapi.SpellidFromSz("Frost Bolt II")]		= "Frost2";
assessSpell[skapi.SpellidFromSz("Frost Bolt III")]		= "Frost3";
assessSpell[skapi.SpellidFromSz("Frost Bolt IV")]		= "Frost4";
assessSpell[skapi.SpellidFromSz("Frost Bolt V")]		= "Frost5";
assessSpell[skapi.SpellidFromSz("Frost Bolt VI")]		= "Frost6";
assessSpell[skapi.SpellidFromSz("Lightning Bolt I")]		= "Lightning1";
assessSpell[skapi.SpellidFromSz("Lightning Bolt II")]		= "Lightning2";
assessSpell[skapi.SpellidFromSz("Lightning Bolt III")]		= "Lightning3";
assessSpell[skapi.SpellidFromSz("Lightning Bolt IV")]		= "Lightning4";
assessSpell[skapi.SpellidFromSz("Lightning Bolt V")]		= "Lightning5";
assessSpell[skapi.SpellidFromSz("Lightning Bolt VI")]		= "Lightning6";
assessSpell[skapi.SpellidFromSz("Shock Wave I")]		= "Shock1";
assessSpell[skapi.SpellidFromSz("Shock Wave II")]		= "Shock2";
assessSpell[skapi.SpellidFromSz("Shock Wave III")]		= "Shock3";
assessSpell[skapi.SpellidFromSz("Shock Wave IV")]		= "Shock4";
assessSpell[skapi.SpellidFromSz("Shock Wave V")]		= "Shock5";
assessSpell[skapi.SpellidFromSz("Shock Wavet VI")]		= "Shock6";
assessSpell[skapi.SpellidFromSz("Whirling Blade I")]		= "Whirling1";
assessSpell[skapi.SpellidFromSz("Whirling Blade II")]		= "Whirling2";
assessSpell[skapi.SpellidFromSz("Whirling Blade III")]		= "Whirling3";
assessSpell[skapi.SpellidFromSz("Whirling Blade IV")]		= "Whirling4";
assessSpell[skapi.SpellidFromSz("Whirling Blade V")]		= "Whirling5";
assessSpell[skapi.SpellidFromSz("Whirling Blade VI")]		= "Whirling6";

// assessSpell[skapi.SpellidFromSz("
// assessSpell[skapi.SpellidFromSz("")]	= ""
// assessSpell[skapi.SpellidFromSz(" Self I")]		= "1"
// assessSpell[skapi.SpellidFromSz(" Self II")]		= "2"
// assessSpell[skapi.SpellidFromSz(" Self III")]	= "3"
// assessSpell[skapi.SpellidFromSz(" Self IV")]		= "4"
// assessSpell[skapi.SpellidFromSz(" Self V")]		= "5"
// assessSpell[skapi.SpellidFromSz(" Self VI")]		= "6"


// Private function to map armor protection numbers to standard game letter codes
function Assess_NumAlpha(Armor_Level) {
    if (Armor_Level >= 2.0) return "u";
    if (Armor_Level >= 1.6) return "e";
    if (Armor_Level >= 1.2) return "aa";
    if (Armor_Level >= 0.8) return "a";
    if (Armor_Level >= 0.4) return "ba";
    if (Armor_Level >= 0.1) return "p";
    return "n";
} // end function

function AssessHandlers_OnAssessCre(aco, fSuccess, cbi, cai, chi) {
    if (AssessDebug) AssessLog.OutD("Assess_OnAssessCre: Begin, Creature [" + aco.szName +"]" );

    CreAssess	= true;
    CreAco	= aco;
    CreStatus	= fSuccess;
    CreCbi	= cbi;
    CreCai	= cai;
    CreChi	= chi;

    if (AssessDebug) AssessLog.OutD("Assess_OnAssessCre: End" );
} //end func AssessHandlers_OnAssessCre

function AssessHandlers_OnAssessItem(Aco, fSuccess, Ibi, Iai, Iwi, Iei, Ipi) {
    if (AssessDebug) AssessLog.OutD("Assess_OnAssessItem: Begin, Item [" + Aco.szName +"]" );

    ItemAssess	= true;
    ItemAco	= Aco;
    ItemStatus	= fSuccess;
    ItemIbi	= Ibi;
    ItemIai	= Iai;
    ItemIwi	= Iwi;
    ItemIei	= Iei;
    ItemIpi	= Ipi;

    if (AssessDebug) AssessLog.OutD("Assess_OnAssessItem: End" );

} //end func AssessHandlers_OnAssessItem

function AssessHandlers() {
    this.OnAssessCreature = AssessHandlers_OnAssessCre;
    this.OnAssessItem 	  = AssessHandlers_OnAssessItem;
}

function AssessObj(debugflg) {
    if ((debugflg==undefined) || (debugflg==null)) debugflg = false;
    AssessDebug = debugflg;

    // Variables
    this.AssCreTimeout	= 2000;				// Amount of time to wait before assuming
    this.AssItemTimeout = 2000;				// assess attempt failed (in milliseconds)

    // Methods
    this.AssessCre	= Assess_AssessCre;		// Assess a creature
    this.AssessItem	= Assess_AssessItem;		// Assess an item
    this.BuildDesc	= Assess_BuildDesc;		// Build brief description of an object
    this.SetDebug	= Assess_SetDebug;		// Change debug flag, output location(s)

} // end object contructor

function Assess_SetDebug(Debugflag) {
    CastDebug = Debugflag;
    return true;
}

function Assess_AssessCre(CreAco) {
    if (AssessDebug) AssessLog.OutD("Assess_AssessCre: Begin" );

    // See if CreAco is defined
    if (CreAco == undefined) {
    	if (AssessDebug) AssessLog.OutD("Assess_AssessCre: CreAco not defined, aborting" );
    	return false;
    } // endif

    // See if CreAco is null
    if (CreAco == null) {
    	if (AssessDebug) AssessLog.OutD("Assess_AssessCre: CreAco null, aborting" );
    	return false;
    } // endif

    // Add handler for OnAssessCreature.  Only need briefly
    skapi.AddHandler(evidOnAssessCreature, Assessh );

    CreAssess = false;		// Clear ItemAssess success flag
    AssessCreActive = true;
    var CreAssStart = new Date();
    skapi.AssessAco(CreAco);	// Assess the creature

     // Now wait for ItemAssess flag to be set TRUE or the timeout value to be reached
    while (!CreAssess && ((new Date() - CreAssStart) < this.AssCreTimeout)) {
	skapi.WaitEvent(20, wemFullTimeout);	// Process all events in queue
    } // end while
    
    AssessCreActive = false;

    // If assess failed then try one more time
    if (!CreAssess) {
	var CreAssStart = new Date();
	skapi.AssessAco(CreAco);	// Assess the creature

	// Now wait for CreAssess flag to be set TRUE or the timeout value to be reached
	while (!CreAssess && ((new Date() - CreAssStart) < this.AssCreTimeout)) {
	    skapi.WaitEvent(20, wemFullTimeout);	// Process all events in queue
	} // end while
    } // endif

    // Remove handler for OnAssessCreature.
    skapi.RemoveHandler(evidOnAssessCreature, Assessh );

    if (CreAssess) {
    	if (AssessDebug) AssessLog.OutD("Assess_AssessCre: End, Success" );
    	return true;	// OnAssessCreate event was received
    } // endif

    // if we make it here then we failed to get the OnAssessCreate event
    if (AssessDebug) AssessLog.OutD("Assess_AssessCre: End, Failure" );
    return false;
} // end func Assess_AssessCre


function Assess_AssessItem(ItemAco) {
    if (AssessDebug) AssessLog.OutD("Assess_AssessItem: Begin" );

    // See if ItemAco is defined
    if (ItemAco == undefined) {
    	if (AssessDebug) AssessLog.OutD("Assess_AssessItem: ItemAco not defined, aborting" );
    	return false;
    } // endif

    // See if ItemAco is null
    if (ItemAco == null) {
    	if (AssessDebug) AssessLog.OutD("Assess_AssessItem: ItemAco null, aborting" );
    	return false;
    } // endif

    // Add handler for OnAssessItem.  Only need briefly
    skapi.AddHandler(evidOnAssessItem, Assessh );

    ItemAssess = false;			// Clear ItemAssess success flag
    var ItemAssStart = new Date();
    skapi.AssessAco(ItemAco);		// Assess the item

     // Now wait for ItemAssess flag to be set TRUE or the timeout value to be reached
    while (!ItemAssess && ((new Date() - ItemAssStart) < this.AssItemTimeout))
          skapi.WaitEvent(25, wemFullTimeout);	// Process all events in queue

    // Remove handler for OnAssessItem.
    skapi.RemoveHandler(evidOnAssessItem, Assessh );

    if (ItemAssess) {
    	if (AssessDebug) AssessLog.OutD("Assess_AssessItem: End, Success" );
    	return true;	// OnAssessItem event was received
    } // endif

    // if we make it here then we failed to get the OnAssessItem event
    if (AssessDebug) AssessLog.OutD("Assess_AssessItem: End, Failure" );
    return false;

} // end func Assess_AssessItem


// The following function builds a one line decription of an item
function Assess_BuildDesc(Aco) {
    if (AssessDebug) AssessLog.OutD("Assess_BuildDesc: Begin, Name [" + Aco.szName + "]");

    if ((Aco == null) || !Aco.fExists) return "";

    // If this is a scroll, don't need detailed description
    if ((Aco.mcm & mcmBooksPaper) && (Aco.szName.match(/Scroll of/))) return "";

    var Desc = "Wrk:" + Aco.workmanship + " Bur:" + Aco.burden + " Val:" + Aco.cpyValue;

    // If a gem, then we have all the info we need so return;
    if (Aco.mcm & mcmGems) return Desc;

    // Add basic information to description if available
    if ((Aco.oai) && (Aco.oai.ibi)) {
    	// If there is a wield requirement, add that in next
	if ((Aco.oai.ibi.skidWieldReq != null) && (Aco.oai.ibi.skidWieldReq != skidNil))
		Desc = Desc + " Wld:" + Aco.oai.ibi.lvlWieldReq + "/" + assessSkid[Aco.oai.ibi.skidWieldReq];

	// If a wand, add PvM/PvP bonuses and mana conversion modifier 
	if (Aco.eqm & eqmFocusWeapon) {
	    if (Aco.oai.iwi) {
		PvMBonus = (Math.round(Aco.oai.iwi.scalePvMElemBonus*1000)-1000)/10;
		if (PvMBonus > 0) Desc = Desc + " PvM:" + PvMBonus;
		PvPBonus = (Math.round(Aco.oai.iwi.scalePvPElemBonus*1000)-1000)/10;
		if (PvPBonus > 0) Desc = Desc + " PvP:" + PvPBonus;
	    } // endif

	    if (Aco.oai.ibi.fractManaConvMod > 0) {
		Desc = Desc + " MC:" + Util.Round(Aco.oai.ibi.fractManaConvMod*100,0);
	    } // endif
	} // endif
    } //endif

    // If this is armor, then add armor info
    if ((Aco.oai) && (Aco.oai.iai))
    	Desc = Desc + " AL:" + Aco.oai.iai.al + "/" +
    		Assess_NumAlpha(Aco.oai.iai.protSlashing) + "-" + Assess_NumAlpha(Aco.oai.iai.protPiercing) + "-" +
    		Assess_NumAlpha(Aco.oai.iai.protBludgeoning) + "-" + Assess_NumAlpha(Aco.oai.iai.protFire) + "-" +
    		Assess_NumAlpha(Aco.oai.iai.protCold) +	"-" + Assess_NumAlpha(Aco.oai.iai.protAcid) + "-" +
    		Assess_NumAlpha(Aco.oai.iai.protElectrical);

    // If this item has weapon info, then add info to description.  Remember that wands can now
    // have missle d and magic d bonuses.
    if ((Aco.oai) && (Aco.oai.iwi)) {
    	// If a ranged weapon then only need damage bonus, speed, and elemental bonus
    	if (Aco.eqm & eqmRangedWeapon) {
	    var DamBon = Math.round(Aco.oai.iwi.scaleDamageBonus*100)-100;
	    Desc = Desc + " Mod:" + DamBon + " Spd:" + Aco.oai.iwi.speed;
	    var ElemBon = Aco.oai.iwi.dhealthElemBonus;
	    if (ElemBon > 0)  Desc = Desc + " EB:" + ElemBon;
	} // endif
	
	// If a melee weapon then add dam bonus, att bonuses, and speed
	if (Aco.eqm & eqmWeapon) {
	    var MaxDam = Aco.oai.iwi.dhealth;
    	    var LowDam = MaxDam*(1-Aco.oai.iwi.scaleDamageRange);
    	    var AttBon = (Math.round(Aco.oai.iwi.scaleAttackBonus*1000)-1000)/10;
	    Desc = Desc + " Dam:" + Math.round(LowDam) + "-" + Math.round(MaxDam) + " " +
		          "Att:" + AttBon + " Spd:" + Aco.oai.iwi.speed;
	} // endif

    	// If item has defense bonus, then add it to description
    	if (Aco.oai.iwi.scaleDefenseBonus > 0) {
	    var DefBon = (Math.round(Aco.oai.iwi.scaleDefenseBonus*1000)-1000)/10;
	    Desc = Desc + " Def:" + DefBon;
	} // endif
    	
   	// If item has a missle d bonus then add it on to description
   	if (Aco.oai.iwi.scaleMissileDBonus > 0) {
	    var MisBon = (Math.round(Aco.oai.iwi.scaleMissileDBonus*1000)-1000)/10;
	    Desc = Desc + " Mis:" + Util.RoundFill(MisBon,1);
	} // endif

  	// If item has a magic d bonus then add it on to description
   	if (Aco.oai.iwi.scaleMagicDBonus > 0) {
	    var MagBon = (Math.round(Aco.oai.iwi.scaleMagicDBonus*1000)-1000)/10;
	    Desc = Desc + " Mag:" + Util.RoundFill(MagBon,1);
	} // endif

    } // endif

    // Add spell descriptions here if Iei defined
    if ((Aco.oai) && (Aco.oai.iei)) {
    	var Long  = "";		// Initialize list of spells we don't have abbreviations for
    	Desc = Desc + " Dif:" + Aco.oai.iei.difficulty;
    	if (Aco.oai.iei.rankReq > 0) Desc = Desc + " Rank:" + Aco.oai.iei.rankReq;
    	if ((Aco.oai.iei.szRaceReq != undefined) && (Aco.oai.iei.szRaceReq != null) && (Aco.oai.iei.szRaceReq != ""))
    	    Desc = Desc + " Race:" + Aco.oai.iei.szRaceReq.substring(0,4);
    	for (var x=0; x<Aco.oai.iei.cspellid; x++) {
	    var Id = Aco.oai.iei.rgspellid(x);
	    if ((assessSpell[Id] != undefined) && (assessSpell[Id] != null))
		Desc = Desc + " " + assessSpell[Id];
	    else
		var Long = Long + " " + skapi.SzFromSpellid(Id);
	} // end for
	Desc = Desc + Long;
	if ((Aco.oai.iei.skidReq != null) && (Aco.oai.iei.skidReq != skidNil))
	    Desc = Desc + " Req:" + Aco.oai.iei.sklvlReq + "/" + assessSkid[Aco.oai.iei.skidReq];
    } // endif

    if (AssessDebug) AssessLog.OutD("Assess_BuildDesc: End");
    return Desc;
} // endif

// Since unlikely to have more than one of these object, define an instance of each
Assess = new AssessObj(false);