// MD_COMPOBJ.JS
//
// Author:	Poundboy
// Email:	poundboy@megapathdsl.net
// Version:	1.01
//
// Description:
//
// Collection of component functions/objects for Skunkworks.
//
// The object contructor is as follows:
//
// myobj = new CompObj ( debug_flag, opm )
//
// where debug_flag is true or false, and opm indicates the destination of the debug output
//
// The following public methods are currently defined:
//
//   SetDebug()			Enables/Disables debugging messages
//
//   SplitTimeout(ms)		By default, the splitting routine will wait 5000 ms (5 secs)
//				for a pea to be split
//
//   DisplayList()		Used for debugging.  Displays contents of component list
//
//   SetMin(name,splitlvl,critlvl,warnlvl)
//				Sets pea split, warning, and critical levels for a component
//
// Requirements:
//
//	Requires the following libraries: MD_LOGOBJ.JS
//
//
// Modification History
//
// Who	Date	  Version	Description
//
// PB	07/07/02  1.01		Initial version
//

// Constants

compList = new Array (
		"Prismatic Taper", "Platinum Scarab", "Pyreal Scarab", "Gold Scarab",
		"Silver Scarab", "Copper Scarab", "Iron Scarab", "Lead Scarab", "Diamond Scarab",

		"Amaranth", "Bistort", "Comfrey", "Damiana", "Dragonsblood", "Eyebright",
		"Frankincense", "Ginseng", "Hawthorn", "Henbane", "Hyssop", "Mandrake",
		"Mugwort", "Myrrh", "Saffron", "Vervain", "Wormwood", "Yarrow",

		"Powdered Agate", "Powdered Amber", "Powdered Azurite", "Powdered Bloodstone",
		"Powdered Carnelian", "Powdered Hematite", "Powdered Lapis Lazuli", "Powdered Malachite",
		"Powdered Moonstone", "Powdered Onyx", "Powdered Quartz", "Powdered Turquoise",

		"Brimstone", "Cadmia", "Cinnabar", "Cobalt", "Colcothar", "Gypsum", "Quicksilver",
		"Realgar", "Stibnite", "Turpeth", "Verdigris", "Vitriol",

		"Alder Talisman", "Ashwood Talisman", "Birch Talisman", "Blackthorn Talisman",
		"Cedar Talisman", "Ebony Talisman", "Elder Talisman", "Hazel Talisman", "Hemlock Talisman",
		"Oak Talisman", "Poplar Talisman", "Rowan Talisman", "Willow Talisman", "Yew Talisman",

		"Blue Taper", "Brown Taper", "Green Taper", "Grey Taper", "Indigo Taper", "Orange Taper",
		"Pink Taper", "Red Taper", "Turquoise Taper", "Violet Taper", "White Taper", "Yellow Taper"   );


// Status values returned by Split method
compSuccess	= 1;		// Comp split was sucessful
compTimeout	= 2;		// Comp split timed out
compNoPea	= 3;		// No pea found to split
compNoTool	= 4;		// No splitting tool available

//
// Global variables
CompComplete	= false;		// Set to true by OnActionComplete event
CompDebug	= false;		// Set to true when Comp logging desired
CompLog		= new LogObj();		// Create log object for comp routines
CompH		= new CompHandlers();	// Handler object
CompHandReg	= false;		// Set to true when handlers registered

// Define functions to handle events from Skunkworks
function CompHandlers_OnActionComplete() {
    if (CompDebug) CompLog.OutD ( "Comp OnActionComplete called");
    CompComplete = true;
}

function CompHandlers() {
    this.OnActionComplete = CompHandlers_OnActionComplete;
}

function CompRec(Name, Enabled, SplitLvl, CritLvl, WarnLvl, BuyLvl) {
    this.Name    	= Name;		// Name of component
    this.Enabled	= Enabled;	// Do we want to check this comp?
    this.LastCheck	= 0;		// Last time component was checked
    this.Stock		= 0;		// Amount found last check
    this.Peas		= 0;		// Amount of peas found for this comp
    this.LastAlarm	= 0;		// Last time an alarm was sounded for
    					// this comp

    // Build pea name
    if ( Name.match(/Taper/) || Name.match(/Talisman/) || Name.match(/Scarab/) ) {
	var Words = Name.split(" ");
	this.PeaName = Words[0] + " Pea";
    } else {
	this.PeaName = Name + " Pea";
    } // endif

    if ((SplitLvl == undefined) || (SplitLvl == null)) SplitLvl = -1;
    this.SplitLvl = SplitLvl;	// Desired component level. Peas will be split to maintain this level

    // If specified, CritLvl defines threshold comp level where character will logout or go into defense mode
    if ((CritLvl == undefined) || (CritLvl == null)) CritLvl = -1;
    this.CritLvl = CritLvl;

    // If specified, WarnLvl defines threshold comp level where character will ask for more comps
    // via fellowship (if appropiate flag set).
    if ((WarnLvl == undefined) || (WarnLvl == null)) WarnLvl = -1;
    this.WarnLvl = WarnLvl;

    // If specified, BuyLvl defines amount of comp that should be in inventory after buying comps
    if ((BuyLvl == undefined) || (BuyLvl == null)) BuyLvl = -1;
    this.BuyLvl = BuyLvl;

} // end function

function CompObj(debugflg) {
    // Output debugging info to debug destination if debugflg set
    if ((debugflg==undefined) || (debugflg==null)) debugflg = false;
    CompDebug = debugflg;

    // If this is the first Comp object created, register the skunkworks handlers
    if (!(CompHandReg)) {
	skapi.AddHandler(evidOnActionComplete,CompH);
	CompHandReg = true;
    } // endif

    this.Index		= 0;	// Used to cycle though comp list when checking one comp at a time.
    this.SplitTimeout	= 5000;	// Default time (ms) to wait for pea split to occur
    				// before assuming pea split attempt failed.

    // Get ACO for pea splitting tool
    this.CompSplitter	= skapi.AcoFindInInv("Splitting Tool");
    if ((this.CompSplitter == undefined) || (this.CompSplitter == null)) {
    	CompLog.OutD ( "CompObj constructor: Can't find splitting tool");
    	this.CompSplitter = null;
    } // endif

    // Set up array for comps and populate it from compList array
    this.Info  = new Array();
    this.Count = compList.length;
    for (var x=0; x<this.Count; x++) this.Info[x] = new CompRec(compList[x], false);

    // Private Methods
    this.Split		= Comp_Split;

    // Public Methods
    this.SetDebug	= Comp_SetDebug;
    this.SetSplitTimeout= Comp_SetSplitTimeout;
    this.CheckAmounts	= Comp_CheckAmounts;
    this.Lookup		= Comp_Lookup;
    this.SetMin		= Comp_SetMin;
    this.Reset		= Comp_Reset;
    this.DisplayList	= Comp_DisplayList;	// Used for debugging.  Displays contents of
    						// complist
} // end object


function Comp_SetDebug(debugflag) {
    CompDebug = debugflag;
    return true;
} // end function

function Comp_Lookup(Name) {
    for (var x=0; x<this.Count; x++) if (Name == this.Info[x].Name) return x;
    return -1;
} // end func

function Comp_SetMin(Name, Enabled, SplitLvl, CritLvl, WarnLvl, BuyLvl) {
    // Lookup component on info list
    for (var x=0; x<this.Count; x++) if (Name == this.Info[x].Name) break;

    if (x == this.Count) {
        // Not found return error code
        return false;
    } // endif
    
    // If buy info not specified, then set it to -1
    if ((BuyLvl == undefined) || (BuyLvl == null)) BuyLvl = -1;

    // Match found, update info
    this.Info[x].Enabled  = Enabled;
    this.Info[x].SplitLvl = SplitLvl;
    this.Info[x].CritLvl  = CritLvl;
    this.Info[x].WarnLvl  = WarnLvl;
    this.Info[x].BuyLvl   = BuyLvl;

    return true;
} // end function

function Comp_SetSplitTimeout(value) {
    this.SplitTimeout = value;
    return true;
} // end function

function Comp_Reset() {
    for (var x=0; x<this.Count; x++) this.Info[x] = new CompRec(compList[x], false);
} // end func


// Used for debugging only
function Comp_DisplayList() {
    CompLog.OutD("Comp Count [" + this.Count + "]", true);
    for (var x=0; x<this.Info.length; x++) {
    	with (this.Info[x]) {
	    CompLog.OutD("Comp: " + Name + " Enabled: " + Enabled + " SplitLvl: " + SplitLvl +
	    		 " Stock: " + Stock + " Peas: " + Peas, true);
	} // end with
    } // end for
} // end function


// Function Comp_Split.  Splits a pea
//
// Returns the following values:
//    compSuccess	Comp split was sucessful
//    compTimeout	Comp split timed out
//    compNoPea		No pea found to split
//    compNoTool	No splitting tool available
//
function Comp_Split(Name) {
    if (CompDebug) CompLog.OutD("Comp_Split: Begin, Name [" + Name + "]");

    // Must be in peace mode to split peas
    Player.EnsureCombatMode(false);

    if (this.CompSplitter == null) {
    	if (CompDebug) CompLog.OutD("Comp_Split: No splitting tools found");
    	return compNoTool;
    } // endif

    // Build pea name
    if (Name.match(/Taper/) || Name.match(/Talisman/) || Name.match(/Scarab/) ) {
	var Words = Name.split(" ");
	var PeaName = Words[0] + " Pea";
    } else
	var PeaName = Name + " Pea";

    // See how many peas we have of this type
    var PeaAmount = skapi.CitemOfKindInInv(PeaName);
    if (PeaAmount == 0 ) {
	CompLog.OutD ( "Comp_Split: Out of peas for component [" + Name + "]");
	return compNoPea;
    } // endif

    if (CompDebug) CompLog.OutD("Comp_Split: Splitting pea [" + PeaName + "] ...");

    skapi.WaitEvent(0);		// Process events in queue (makes sure all OnActionComplete
    				// events are cleared out

    // Now use the splitting tool on the pea
    var SplitStart = new Date();
    CompComplete = false;		// Will be set true by OnActionComplete event
    this.CompSplitter.UseOnAco(skapi.AcoFindInInv(PeaName));

    // Now what for action to complete or timeout value to be reached
    while (!(CompComplete) && ((new Date()-SplitStart) < this.SplitTimeout))
	skapi.WaitEvent(20);

    if (CompComplete) {
    	if (CompDebug) CompLog.OutD("Comp_Split: End, Success");
	return compSuccess;
    } // endif

    if (CompDebug) CompLog.OutD("Comp_Split: Error splitting pea. Timeout");
    return compTimeout;
} // end func

// Loads the current amount of all comps in our list.  Returns true if all comp levels ok,
// false if we are below the limit on a comp
function Comp_CheckAmounts() {
    if (CompDebug) CompLog.OutD("Comp_CheckAmounts: Begin, CompCount [" + this.Info.length + "]");

    // Assume all comps are ok before check
    ReturnCode = true;

    // For each component
    for (var x=0; x<this.Count; x++) {
    	if (!this.Info[x].Enabled) continue;	// If comp not enabled, then skip it
	var Name    = this.Info[x].Name;
	var PeaName = this.Info[x].PeaName;
	var Req     = this.Info[x].SplitLvl;
	var Stock   = skapi.CitemOfKindInInv(Name);
	if ((Stock==undefined) || (Stock==null)) {
	    if (CompDebug) CompLog.OutD("Comp_CheckAmounts: Index [" + x + "] Stock undefined or null");
	    Stock = 0;
	} // endif

	var Peas = skapi.CitemOfKindInInv(PeaName);
	if ((Peas==undefined) || (Peas==null)) {
	    if (CompDebug) CompLog.OutD("Comp_CheckAmounts: Index [" + x + "] Peas undefined or null");
	    Peas = 0;
	} // endif

	// Update Comp.Info record
	this.Info[x].LastCheck = new Date();
	this.Info[x].Stock = Stock;
	this.Info[x].Peas  = Peas;

	// Now compute how much we need
	var Need  = Req - Stock;
	if (CompDebug) {
	    CompLog.OutD("Comp_CheckAmounts: Index [" + x + "] Name [" + Name  + "]");
	    CompLog.OutD("Comp_CheckAmounts: Req   [" + Req + "] Stock [" + Stock + "] Need [" + Need + "]");
	} //endif

	// If we are short on this comp then set return code indicating at least one comp is below
	// required minimum or its warning level
	if (Need > 0) {
	    ReturnCode = false;
	    if (CompDebug) CompLog.OutD("Comp_CheckAmounts: Comp [" + Name + "is low");
	} // endif

	// If we are below the warning level on this comp then set return code indicating
	// at least one comp is below required minimum or its warning level
	if (Stock < this.Info[x].WarnLvl) {
	    ReturnCode = false;
	    if (CompDebug) CompLog.OutD("Comp_CheckAmounts: Comp [" + Name + "below warning level");
	} // endif

    } // end for

    if (CompDebug) CompLog.OutD("Comp_CheckAmounts: End, Return code [" + ReturnCode + "]");
    return ReturnCode;
} // end funct Comp_CheckAmounts

Comp = new CompObj(false);