// MD_UTILOBJ.JS
//
// Author:	Poundboy
// Email:	poundboy@megapathdsl.net
// Version:	1.01
//
// Description:
//
// Collection of simple utility functions/objects for Skunkworks.
//
// The following object constructors are defined:
//
// myObj = new UtilObj();
// myObj = new ChatObj();
//
// Util Methods:
//
//  Deg2Rad			Convert degrees to radians
//  Rad2Deg			Convert radians to degrees
//  FormatDate(date_obj,msflag)	Returns a date object as a string. Add milliseconds to time if msflag true
//  FormatMinSec(interval)	Return a string 'x minutes y seconds' from a millisecond parameter
//  FormatTime(date_obj)	Returns a string time from a date object
//  FormatCoords(maploc,precision)
//				Returns string coordinates with 'precision' number of decimal places
//  Hex(number)			Returns hex equivalent of 'number' in string form.
//  Include(filename)		evaluates contents of 'filename'
//  Round(number,count)		Returns 'number' rounded to 'count' decimal places.
//  RoundFill(number,count)	Returns text representation of 'number' rounded to 'count' decimal places.
//				0s are added on at end if necessary.
//  Distance(maploc1,maploc2)	Returns distance between two maploc objects.
//  Pad(string,count)		Returns string padded with spaces so that string is a minimum
//				'count' characters long.
//  Width(string,inches)	Returns string padded with spaces to make it 'inches' long
//
// Chat Methods:
//
//  Screen(text)		Write 'text' to the screen
//  Say(text)			Writes text to the command inout area
//  Local(text)			Sends a local chat message
//  Tell(who,text)		Sends a @tell
//  Fellow(text)		Sends a @fellow
//  Patron(text)		Sends a @patron
//  Vassal(text)		Sends a @vassal
//  Covassal(text)		Sends a @covassal
//  Monarch(text)		Sends a @monarch
//  Emote(text)			Sends a @emote
//
//
// The following variables are defined automatically since it is very
// unlikely you would ever need more than one of each of these objects:
//
// Util = new UtilObj();
// Chat = new ChatObj();
//
//
// Modification History
//
// Who	Date	  Version	Description
//
// PBoy	07/07/02  1.01		Initial version


function UtilObj() {

    // Set up array that defines characters per inch for AC font. This
    // array enables us to make our columns line up.
    this.prop = new Array();
    this.prop["A"] =  6.5;
    this.prop["B"] =  8;
    this.prop["C"] =  32/4.5;
    this.prop["D"] =  32/4.5;
    this.prop["E"] =  32/3.5;
    this.prop["F"] =  32/3.5;
    this.prop["G"] =  39/6;
    this.prop["H"] =  39/6;
    this.prop["I"] =  16;
    this.prop["J"] =  16;
    this.prop["K"] =  32/4.5;
    this.prop["L"] =  8;
    this.prop["M"] =  32/6;
    this.prop["N"] =  38/6.5;
    this.prop["O"] =  6.5;
    this.prop["P"] =  8;
    this.prop["Q"] =  6.5;
    this.prop["R"] =  32/4.5;
    this.prop["S"] =  32/3.5;
    this.prop["T"] =  8;
    this.prop["U"] =  6.5;
    this.prop["V"] =  6.5;
    this.prop["W"] =  37/8;
    this.prop["X"] =  32/4.5;
    this.prop["Y"] =  32/4.5;
    this.prop["Z"] =  32/4.5;
    this.prop["a"] =  32/3.5;
    this.prop["b"] =  8;
    this.prop["c"] =  40/3.75;
    this.prop["d"] =  8;
    this.prop["e"] =  40/3.75;
    this.prop["f"] =  16;
    this.prop["g"] =  32/3.5;
    this.prop["h"] =  8;
    this.prop["i"] =  16;
    this.prop["j"] =  16;
    this.prop["k"] =  32/3.5;
    this.prop["l"] =  16;
    this.prop["m"] =  38/7;
    this.prop["n"] =  8;
    this.prop["o"] =  32/3.5;
    this.prop["p"] =  8;
    this.prop["q"] =  32/3.5;
    this.prop["r"] =  40/3.75;
    this.prop["s"] =  40/3.75;
    this.prop["t"] =  32/2.5;
    this.prop["u"] =  8;
    this.prop["v"] =  8;
    this.prop["w"] =  38/7;
    this.prop["x"] =  32/3.5;
    this.prop["y"] =  8;
    this.prop["z"] =  32/3.5;
    this.prop["0"] =  32/3.5;
    this.prop["1"] =  32/3.5;
    this.prop["2"] =  32/3.5;
    this.prop["3"] =  32/3.5;
    this.prop["4"] =  32/3.5;
    this.prop["5"] =  32/3.5;
    this.prop["6"] =  32/3.5;
    this.prop["7"] =  32/3.5;
    this.prop["8"] =  32/3.5;
    this.prop["9"] =  32/3.5;
    this.prop["`"] =  16;
    this.prop["~"] =  32/3.5;
    this.prop["!"] =  16;
    this.prop["@"] =  36/5;
    this.prop["#"] =  32/3.5;
    this.prop["$"] =  32/3.5;
    this.prop["%"] =  38/6.5;
    this.prop["^"] =  32/3.5;
    this.prop["&"] =  32/5;
    this.prop["*"] =  32/2.5;
    this.prop["("] =  16;
    this.prop[")"] =  16;
    this.prop["-"] =  32/2.5;
    this.prop["="] =  32/3.5;
    this.prop["_"] =  32/3.5;
    this.prop["+"] =  32/3.5;
    this.prop["["] =  32/2.5;
    this.prop["]"] =  32/2.5;
    this.prop["\\"]=  8;
    this.prop["{"] =  16;
    this.prop["}"] =  16;
    this.prop["|"] =  37/3.5
    this.prop[";"] =  40/1.875;
    this.prop["'"] =  40/1.875;
    this.prop[":"] =  40/1.875;
    this.prop["\""]=  32/2.5;
    this.prop[","] =  32/3.5;
    this.prop["."] =  40/1.875;
    this.prop["/"] =  16;
    this.prop["<"] =  32/3.5;
    this.prop[">"] =  32/3.5;
    this.prop["?"] =  32/2.5;
    this.prop[" "] =  16;

    // Methods
    this.Deg2Rad	= Util_Deg2Rad;		// Convert degrees to radians
    this.Rad2Deg	= Util_Rad2Deg;		// Convert radians to degrees
    this.Distance	= Util_Distance;	// Distance between two coordinate sets
    this.FormatDate	= Util_FormatDate;	// Returns a date object as a date/time string
    this.FormatMinSec	= Util_FormatMinSec;	// Return a string 'x minutes y seconds' from a millisecond parameter
    this.FormatTime	= Util_FormatTime;	// Returns a date object as a time string
    this.FormatCoords	= Util_FormatCoords;	// Return maploc as string coordinates
    this.Hex		= Util_Hex;		// Returns hex string of decimal number
    this.Include	= Util_Include;		// Read in and EVAL a file
    this.Round		= Util_Round;		// Round a number
    this.RoundFill	= Util_RoundFill;	// Round a number returning text with 0s filled
    						// in on end if necessary
    this.TestFont	= Util_TestFont;	// Print out character set so we can determine how a font will look
    this.Trim		= Util_Trim;		// Remove spaces from beginning and end of string
    this.Pad		= Util_Pad;		// Pads a string with spaces
    this.Width		= Util_Width;		// Appends spaces to a string to make it the
    						// specified length in screen inches.  Takes
    						// in account proportional size of each character.
} // end object contructor


function Util_TestFont(start,end) {
    var CharList = new Array("A", "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L",
			"M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y",
			"Z", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l",
			"m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y",
			"z", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "`", "~",
			"!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "-", "=", "_", 
			"+", "[", "]", "\\", "{", "}", "|", ";", "'", ":", "\"", ", ", ".",
			"/", "<", ">", "?", " ");

    if (end > (CharList.length - 1)) end = CharList.length - 1;
    
    for (var x=start-1; x<=end; x++) {
    	line = "|";
    	for (var y=0; y<40; y++) line += CharList[x];
    	line += "|";
    	Chat.Screen(line);
    } // end for
} // end func

function Util_Deg2Rad(degrees) {
    return (degrees/180)*Math.PI;
} // end func

function Util_Rad2Deg(radians) {
    return (radians/Math.PI)*180;
} // end func


function Util_Distance(maploc1, maploc2) {
    if ((maploc1 == undefined) || (maploc1 == null) || (maploc2 == undefined) || (maploc2 == null)) return 999;
    if (maploc1.fInDungeon != maploc2.fInDungeon) return 999;
    return maploc1.Dist2DToMaploc(maploc2);
} // end function

function Util_FormatCoords(maploc,precision) {
    if ((maploc == undefined) || (maploc == null)) return "[bad_maploc]";
    if ((precision == undefined) || (precision == null)) precision = 2;
    var lat_sign = "N";
    if (maploc.lat < 0) lat_sign = "S";
    var lng_sign = "E";
    if (maploc.lng < 0) lng_sign = "W";

    var out =	Util.RoundFill(Math.abs(maploc.lat),precision) + lat_sign + " " +
    		Util.RoundFill(Math.abs(maploc.lng),precision) + lng_sign;

    return out;
} // end function


function Util_FormatMinSec(ms) {
    var seconds = Math.floor(ms/1000);
    var minutes = Math.floor(seconds/60);
    var hours   = Math.floor(minutes/60);
    var modsecs = seconds - (minutes*60);
    var modmins = minutes - (hours*60);
    
    var RetString = hours + ":"; 
    if (hours < 10) RetString = "0" + RetString;

    if (modmins < 10) RetString = RetString + "0";
    RetString = RetString + modmins + ":";

    if (modsecs < 10) RetString = RetString + "0";
    RetString = RetString + modsecs

    return RetString;
} // end func


// Function to convert a dateobject to a formatted string
function Util_FormatTime(dateobj) {

    if (dateobj == undefined) dateobj = 0;
    if (dateobj==0) dateobj = new Date();

    var hour   = dateobj.getHours();	 	// Hours returned as 0-23
    var minute = dateobj.getMinutes();		// Minutes returned as 0-59
    var second = dateobj.getSeconds();		// Seconds returned as 0-59

    // 0 fill some of numbers
    if (hour<10)   hour   = "0" + hour;
    if (minute<10) minute = "0" + minute;
    if (second<10) second = "0" + second;

    return hour + ":" + minute + ":" + second;
} // end function


// Function to convert a dateobject to a formatted string
function Util_FormatDate(dateobj, ms_flag) {
    if (dateobj == undefined) dateobj = 0;
    if (dateobj==0) dateobj = new Date();
    if (ms_flag == undefined) ms_flag = true;

    var month  = dateobj.getMonth() + 1;	// Month returned as 0-11
    var day    = dateobj.getDate();		// Day returned as 1-31
    var hour   = dateobj.getHours();	 	// Hours returned as 0-23
    var minute = dateobj.getMinutes();		// Minutes returned as 0-59
    var second = dateobj.getSeconds();		// Seconds returned as 0-59
    var ms     = dateobj.getMilliseconds();	// Extract milliseconds

    // 0 fill some of numbers
    if (month<10)  month  = "0" + month;
    if (day<10)    day    = "0" + day;
    if (hour<10)   hour   = "0" + hour;
    if (minute<10) minute = "0" + minute;
    if (second<10) second = "0" + second;

    var outdate = month + "/" + day + " " + hour + ":" + minute + ":" + second;

    if (ms_flag) {
	if (ms<100) {
	    if (ms<10)
		ms     = "00" + ms;
	    else
		ms     = "0"  + ms;
	} // endif
	outdate = outdate + "." + ms;
    } // endif

    return outdate;
} // end function

// hex - Convert a number to hex
//
function Util_Hex(nbr) {
    // my old method -> return nbr.toString(16);

    // Thanks to David A. Down (Merlyn of Harvestion) for following hex conversion code

    // Return the unsigned hexadecimal string for a number
    var rbits= 0x10000 + (0xffff & nbr);
    var lbits= 0x10000 + (nbr >>> 16);
    return "0x"+lbits.toString(16).slice(1)+rbits.toString(16).slice(1);
} // end function Util_Hex

// Thanks to David A. Down (Merlyn of Harvestion) for following Include function.  Modified it
// to check for empty file
function Util_Include(name) { // Include the evaluated contents of a file
    var fso = new ActiveXObject("Scripting.FileSystemObject"); // For file functions
    if(!fso.FileExists(name)) return -1; // Does the data file exist?
    cf = fso.OpenTextFile(name,ForReading);
    if (!cf.AtEndOfStream) eval(cf.ReadAll());	// Evaluate the file contents
    cf.Close();
    return 0;
} // end funct

function Util_Round(value,count) {
    var power = Math.pow(10,count);
    return Math.round(value*power)/power;
} // endfunc

function Util_RoundFill(value,count) {
    // Chat.Screen("value [" + value + "] count [" + count + "]");
    if (count == 0) return Math.toString(this.Round(value,0));
    var zeros = "0";
    for (var x=0; x<count; x++) zeros += "0";
    var rounded = this.Round(value,count);
    var rounded_str = rounded + "";
    var dec_index = rounded_str.indexOf(".");
    if (dec_index < 0) rounded_str += ".";
    rounded_str += zeros;
    var dec_index = rounded_str.indexOf(".");
    var returnval = rounded_str.substring(0,dec_index+count+1);
    return returnval;
} // endfunc

// Trim all spaces from beginning and end of a string, and remove all tabs
function Util_Trim(text) {
    if ((text == undefined) || (text == null) || (text == "")) return "";
    var pattern = new RegExp ( "(^ +| +$|\t)", "g" );
    return text.replace (pattern, "");
} // end func

// Makes sure a text string is a minimum size.
function Util_Pad(text, size) {
     lots_spaces = "                                             ";

     if (text == "")
         var temp = lots_spaces;
     else
         var temp = text + lots_spaces;

     return temp.substring(0,size);
} // end function

function Util_Width(text,width) {
    // Calculate current size of string (in inches) based on proportional font used by AC
    // at 1024x768 screen size
    var textlen = 0;
    for (var x=0; x<(text.length); x++) {
    	var SubChar = text.substring(x,x+1);
    	var CPI     = this.prop[SubChar];
    	textlen = textlen + (1/CPI);
    } // endif

    var need = Math.round((width - textlen)*this.prop[" "]);
    var manyspaces = "";
    for (var x=0; x<need; x++) manyspaces = manyspaces + " ";
    return text+manyspaces;
} // end func

/// **********************************************************************

// Chat recipient masks:
//
// chrmLocal
// chrmEmote
// chrmFellowship
// chrmVassals
// chrmPatron
// chrmMonarch
// chrmCovassals
// chrmAllegiance
//

function ChatObj() {
    /// Methods
    this.Say	 = Chat_Say;
    this.Tell	 = Chat_Tell;
    this.Reply	 = Chat_Reply;
    this.Local	 = Chat_Local;
    this.Fellow  = Chat_Fellow;
    this.Patron	 = Chat_Patron;
    this.Screen	 = Chat_Screen;
    this.Vassal  = Chat_Vassal;
    this.Vassal  = Chat_Covassal;
    this.Monarch = Chat_Monarch;
    this.Emote	 = Chat_Emote;
} // end func

function Chat_Say(text) {
    // If we already have chat focus (ie the chat area is outlined in yellow), then get rid of it
    if (skapi.fInChatBuffer) {
    	skapi.Keys("{return}", kmoClick);
	skapi.Sleep(100);
    } // endif
    skapi.Keys("{return}", kmoClick);
    skapi.Sleep(100);
    skapi.PasteSz( text );
    skapi.Sleep(100);
    skapi.Keys("{return}", kmoClick);
} // end func

function Chat_Tell(who, text) {
    skapi.Tell(who, text);
} // end func

function Chat_Reply(text) {
    this.Say("@r " + text);
} // end func

function Chat_Local(text) {
    skapi.Chat(chrmLocal, text);
} // end func

function Chat_Fellow(text) {
    skapi.Chat(chrmFellowship, text);
} // end func

function Chat_Patron(text) {
    skapi.Chat(chrmPatron, text);
} // end func

function Chat_Vassal(text) {
    skapi.Chat(chrmVassals, text);
} // end func

function Chat_Covassal(text) {
    skapi.Chat(chrmCovassals, text);
} // end func

function Chat_Monarch(text) {
    skapi.Chat(chrmMonarch, text);
} // end func

function Chat_Emote(text) {
    skapi.Chat(chrmEmote, text);
} // end func

function Chat_Screen(text, textColor) {
    if ((textColor==undefined) || (textColor==null)) textColor = cmcPaleBlue;
    skapi.OutputLine(text, opmChatWnd, textColor);
} // end func

// *********************************************************************************

// Since unlikely to have more than one of these object, define an instance of each
Util = new UtilObj();
Chat = new ChatObj();