// WAR_EVENTS.JS
//
// Author:	Poundboy
// Version:	2.08
//
//
// Modification History:
//
// Who	  Date	    Version	Description
//
// PB	  09/13/02  1.01	Initial version.  Mostly based on old acscript drainbot
//
//
// Description:
//
//	The following file contains all the event functions for WarBot
//

// Event functions

// OnActionComplete
function MainHandlers_OnActionComplete() {
    if (debugEvent) Log.OutD("Main_OnActionComplete: Begin" );
    ActionComplete = true;
    if (debugEvent) Log.OutD("Main_OnActionComplete: End" );
} // end funct


function MainHandlers_OnAddToInventory(aco) {
    if (debugEvent||debugLoot||debugSalvage||debugAddInv)
	Log.OutD("Main_OnAddToInventory: Begin, Item [" + aco.szName + "] Oid [" + Util.Hex(aco.oid) + "]");

    // If we are looting, see if the item added to inventory is an item we are attempting
    // to pick up.  If so add it to the appropiate loot list.
    if (PickupActive || (State == stateLoot)) {
    	// Look for this object on the current corpse loot list
    	for (var x=0; x<LootCount; x++) {
	    if (LootList[x].Aco.oid == aco.oid) {
		// We have a match, add this item to the appropiate loot list.  Note we make sure
		// the item does not already exist on loot list due to a server-side bug that sometimes
		// duplicates addinventory events for the same item.
		if (debugLoot) Log.OutD("Main_OnAddToInventory: Adding to loot list");
		if (Loot.AddLoot(LootList[x].Aco, LootList[x].LootType, LootList[x].Desc)) {
		    if (debugLoot)
			Log.OutD("Main_OnAddToInventory: Item added to loot list [" + LootList[x].LootType + "]");
		} else {
		    if (debugLoot)
			Log.OutD("Main_OnAddToInventory: Duplicate item on loot list [" + aco.szName + "/" +
			    	  Util.Hex(aco.oid) + "]");
		    continue;
		} // endif

		// If this is the current item we are trying to pick up, then
		// set a flag so we stop waiting for the pickup
		if (PickupActive && (x==CurItem)) {
		    if (debugLoot) Log.OutD("Main_OnAddToInventory: Item matches current pickup attempt");
		    InventoryAdd = true;
		} // endif
	    } // endif
	} // endfor
    } // endif

    if (SalvageActive) {
	if (debugSalvage) {
	    Log.OutD("Main_OnAddToInventory: Salvage flag active");
	    Log.OutD("Main_OnAddToInventory: Added [" + aco.szName + "] Units [" + aco.cuseLeft + "]");
	} // endif
	InventoryAdd = true;
	// If item added was a full bag of salvage and autoMoveFull option is enabled, then
	// set flag indicating there are salvage bags to move out of main pack.  The next idle
	// period will take care of it.
	if (flagMoveFull && (aco.mcm & mcmSalvageBag) && (aco.cuseLeft == 100)) BagToMove = true;
    } // endif

    if (MoveActive) {
	if (debugSalvage) Log.OutD("Main_OnAddToInventory: Move flag active, setting InventoryAdd to TRUE.");
	InventoryAdd = true;
	AddInvAco = aco;
    } // endif

    // If we are sorting loot, then set flag indicating move is done
    if (LootSortMode) LootMoveDone = true;

    if (debugEvent||debugLoot) Log.OutD("Main_OnAddToInventory: End");
    return;
} // end func


function MainHandlers_OnAdjustStack(aco,citemPrev) {
    if (debugEvent||debugLoot)
	Log.OutD("Main_OnAdjustStack: Begin, Item [" + aco.szName + "/" + Util.Hex(aco.oid) +
		 "] Old size [" + citemPrev + "] New size [" + aco.citemStack + "]");

    // See if item adjusted is on our loot list.  If so we will have to adjust the
    // value total of this loot type
    if (Loot.LookupLoot(aco.oid)) {
	// Match found, adjust pyreal total
	if (debugLoot) Log.OutD("OnAdjustStack: Adjusting pyreal total for lootType [" + Loot.TypeIndex + "]");
	var CurValPerItem = aco.cpyValue / aco.citemStack;
	Loot.TypeInfo[Loot.TypeIndex].TotalValue += CurValPerItem*(aco.citemStack-citemPrev);
    } // endif

    if (PickupActive) {
     	if (debugLoot)
     	    Log.OutD("Main_OnAdjustStack: Current pickup [" + LootList[CurItem].Aco.szName +
     	    	     "] Oid [" + Util.Hex(LootList[CurItem].Aco.oid) + "]");
     	if (aco.szName == LootList[CurItem].Aco.szName) {
	    InventoryAdd = true;
	    InventoryMerged = true;
	    MergeAco = aco;
	    if (debugLoot) Log.OutD("Main_OnAdjustStack: InventoryAdd set to true");
	} else {
	    // False positive
	    if (debugLoot) Log.OutD("Main_OnAdjustStack: False positive");
	} //endif
    } // endif
    if (debugEvent||debugLoot) Log.OutD("Main_OnAdjustStack: End");
    return;
} // end func


function MainHandlers_OnApproachVendor(acoMerchant, mcmBuy, cpyMaxBuy, fractBuy, fractSell, coaco) {
    if (debugEvent) Log.OutD("Main_OnApproachVendor: Begin");
    VendorOpen	    = true;
    VendorAco	    = acoMerchant;
    VendorMcmBuy    = mcmBuy;
    VendorMaxBuy    = cpyMaxBuy;
    VendorFractBuy  = fractBuy;
    VendorFractSell = fractSell;
    ItemCoaco	    = coaco;
    if (debugEvent) Log.OutD("Main_OnApproachVendor: End");
    return;
} // end func


function MainHandlers_OnAssessCre(Aco, fSuccess, Cbi, Cai, Chi) {
    if (debugEvent||debugAssCre)
	Log.OutD("Main_OnAssessCre: Begin, Cre [" + Aco.szName + "] success [" +  fSuccess + "]");
    
    CreAssSuc = fSuccess;
    CreAssAco = Aco;
    CreAssCbi = Cbi;
    CreAssCai = Cai;
    CreAssChi = Chi;
    var Name = Aco.szName;
   
    // If assess failed, then no point going any further
    if (!CreAssSuc) {
	if (debugEvent||debugAssCre) Log.OutD("Main_OnAssessCre: End, Assess failed ");
	return;
    } // endif
 
    // Process creature info if event not a result of a creature assess call via the assess object.
    if (AssessCreActive) {
    	// Event will be processed via assess object so just exit
	if (debugEvent||debugAssCre) Log.OutD("Main_OnAssessCre: End, Assess object cre assess active.");
	return;
    } // endif
    
    // If this is not a creature, then abort
    if (!(Aco.ocm & ocmMonster)) {
	// We assessed another player, exit this routine
	if (debugEvent||debugAssCre)
	    Log.OutD("Main_OnAssessCre: End, Aco is not a creature, ocm [" + Util.Hex(Aco.ocm) + "]");
	return;
    } // endif
    
    // We have a successful access.  Figure out its creature index then check the database for accuracy.
    // If we are in the kill state then this access should be the current creature we are killing which
    // might save us a creature name lookup
    if ((State == stateKill) && (Name == Att.CreName)) {
    	if (debugEvent||debugAssCre) Log.OutD("Main_OnAssessCre: Assess is of creature we are attacking.");
	var Index = Att.CreIndex;
    } else {
    	if (debugEvent||debugAssCre) Log.OutD("Main_OnAssessCre: Looking up creature...");
    	var Index = Creature.NameToIndex(Name);
	if (Index == 0) {
	    if (debugEvent||debugAssCre) Log.OutD("OnAssessCre: Creature [" + Name + "] unknown.  Adding...");
	    Chat.Screen("Unknown creature '" + Name + "' detected, adding to database...", cmcYellow);
	    Index = Creature.AddUnknownCre(CreAssAco);
	    if (Index == 0) {
		// Couldn't add creature for some reason, abort
		return;
	    } // endif
	    var RaceCode = Creature.Info[Index].RaceCode;
	    if (debugEvent||debugAssCre)
		Log.OutD("OnAssessCre: Cre added to race [" + Creature.RaceInfo[RaceCode].RaceName + "/" + RaceCode + "]");
	    Chat.Screen("Creature added to race '" + Creature.RaceInfo[RaceCode].RaceName + "/" +
	    		 RaceCode + "'.", cmcYellow);

	    // Reset decal panel for creatures to make sure new creature displayed
	    skapi.SetControlProperty(szPanelCreature, "raceList3", "selected", 0);
	    skapi.SetControlProperty(szPanelCreature, "raceList3", "selected", 1);
	} // endif
    } // endif
    if (debugEvent||debugAssCre) Log.OutD("Main_OnAssessCre: Creature db index [" + Index + "]");

    if (Creature.Info[Index].Assessed) {
	if (debugEvent||debugAssCre) Log.OutD("Main_OnAssessCre: End, Creatured already assessed.");
	return;
    } // endif

    // Update creature record if necessary and log differences to file
    if (CreAssCbi && CreAssCai) {
	var NoHeader = true;
	with (Creature.Info[Index]) {
	    Assessed = true;
	    if (Level != CreAssCbi.lvl) {
		if (Creature.LogEnabled) {
		    if (NoHeader) {
		    	NoHeader= false;
		    	Creature.LogFile.WriteLineD(" ");
		    } // endif
		    if (debugEvent||debugAssCre)
			Log.OutD("Main_OnAssessCre: Cre [" + Name + "] level change, old [" +
				  Level + "] new [" + CreAssCbi.lvl + "]");
		    Creature.LogFile.WriteLineD("Cre [" + Name + "] level change, old [" +
						Level + "] new [" + CreAssCbi.lvl + "]");
		} // endif
		Level = CreAssCbi.lvl;
	    } // endif

	    if (Health != CreAssCbi.healthMax) {
		if (Creature.LogEnabled) {
		    if (NoHeader) {
		    	NoHeader= false;
		    	Creature.LogFile.WriteLineD(" ");
		    } // endif
		    if (debugEvent||debugAssCre)
			Log.OutD("Main_OnAssessCre: Cre [" + Name + "] health change, old [" +
				  Health + "] new [" + CreAssCbi.healthMax + "]");
		    Creature.LogFile.WriteLineD("Cre [" + Name + "] health change, old [" +
						Health + "] new [" + CreAssCbi.healthMax + "]");
		} // endif
		Health = CreAssCbi.healthMax;
	    } // endif

	    if (!BadRace && (RaceCode != CreAssCbi.species)) {
		if (Creature.LogEnabled) {
		    if (NoHeader) {
		    	NoHeader= false;
		    	Creature.LogFile.WriteLineD(" ");
		    } // endif
		    if (debugEvent||debugAssCre)
			Log.OutD("Main_OnAssessCre: Cre [" + Name + "] bad race, current [" +
				  RaceCode + "] should be [" + CreAssCbi.species + "]");
		    Creature.LogFile.WriteLineD("Cre [" + Name + "] bad race, current [" +
						RaceCode + "] should be [" + CreAssCbi.species + "]");
		} // endif
		BadRace = true;
	    } // endif
	    
	    if (Mana != CreAssCai.manaMax) {
	    	if (Creature.LogEnabled) {
		    if (NoHeader) {
		    	NoHeader= false;
		    	Creature.LogFile.WriteLineD(" ");
		    } // endif
		    if (debugEvent||debugAssCre)
			Log.OutD("Main_OnAssessCre: Cre [" + Name + "] mana change, old [" +
				  Mana + "] new [" + CreAssCai.manaMax + "]");
		    Creature.LogFile.WriteLineD("Cre [" + Name + "] mana change, old [" +
						Mana + "] new [" + CreAssCai.manaMax + "]");
		} // endif
	    	Mana = CreAssCai.manaMax;
	    } // endif
	   
	    if (Stamina != CreAssCai.staminaMax) {
	 	if (Creature.LogEnabled) {
		    if (NoHeader) {
		    	NoHeader= false;
		    	Creature.LogFile.WriteLineD(" ");
		    } // endif
		    if (debugEvent||debugAssCre)
			Log.OutD("Main_OnAssessCre: Cre [" + Name + "] stamina change, old [" +
				  Stamina + "] new [" + CreAssCai.staminaMax + "]");
		    Creature.LogFile.WriteLineD("Cre [" + Name + "] stamina change, old [" +
						Stamina + "] new [" + CreAssCai.staminaMax + "]");
		} // endif
	    	Stamina = CreAssCai.staminaMax;
	    } // endif
	} // end with
    } // endif

    if (debugEvent) Log.OutD("Main_OnAssessCre: End" );
} // end func 


    //if (Aco) {
    //	Chat.Screen(" ");
    //	Chat.Screen("Aco info...");
    //	if (Aco.oid)		Chat.Screen("oid : " +		Aco.oid );
    //	if (Aco.szName)		Chat.Screen("szName : " + 	Aco.szName );
    //	if (Aco.szPlural)	Chat.Screen("szPlural : " + 	Aco.szPlural );
    //	if (Aco.oty)		Chat.Screen("oty : " + 		Aco.oty );
    //	if (Aco.ocm)		Chat.Screen("ocm : " + 		Aco.ocm );
    //	if (Aco.olc)		Chat.Screen("olc : " + 		Aco.olc );
    //	if (Aco.mcm)		Chat.Screen("mcm : " + 		Aco.mcm );
    //	if (Aco.szMcm)		Chat.Screen("szMcm: " + 	Aco.szMcm );
    //	if (Aco.burden)		Chat.Screen("burden : " + 	Aco.burden );
    //	if (Aco.oidMonarch)	Chat.Screen("oidMonarch : " + 	Aco.oidMonarch );
    // } else {
    //	Chat.Screen(" ");
    //	Chat.Screen("Aco is null.");
    // } // endif

    // if (Cbi) {
    //	Chat.Screen(" ");
    //	Chat.Screen("Cbi info...");
    //	Chat.Screen("Lvl: " + Cbi.lvl);
    //	Chat.Screen("healthCur: " + Cbi.healthCur);
    //	Chat.Screen("healthMax: " + Cbi.healthMax);
    // } else {
    // 	Chat.Screen("Cbi is null");
    // } // endif

    // if (Cai) {
    //	Chat.Screen(" ");
    //	Chat.Screen("Cai info...");
    //	Chat.Screen("strength: " + Cai.strength);
    //	Chat.Screen("endur: " + Cai.endurance);
    //	Chat.Screen("quick: " + Cai.quickness);
    //	Chat.Screen("coord: " + Cai.coordination);
    //	Chat.Screen("focus: " + Cai.focus);
    //	Chat.Screen("self: " + Cai.self);
    //	Chat.Screen("stamCur: " + Cai.staminaCur);
    //	Chat.Screen("stamMax: " + Cai.staminaMax);
    //	Chat.Screen("manaCur: " + Cai.manaCur);
    //	Chat.Screen("manaMax: " + Cai.manaMax);
    // } else {
    //	Chat.Screen(" ");
    //	Chat.Screen("Cai is null");
    // } // endif  

    // if (Chi) {
    //	Chat.Screen(" ");
    //	Chat.Screen("Available Chi info...");
    //	if (Chi.rank)		Chat.Screen("rank: " +		Chi.rank);
    //	if (Chi.cfollower)	Chat.Screen("cfollower: " +	Chi.cfollower);
    //	if (Chi.fPK)		Chat.Screen("fPK: " +		Chi.fPK);
    //	if (Chi.szGender)	Chat.Screen("szGender: " + 	Chi.szGender);
    //	if (Chi.szRace)		Chat.Screen("szRace: " +	Chi.szRace);
    //	if (Chi.szProfession)	Chat.Screen("szProfession: " +	Chi.szProfession);
    //	if (Chi.szFellowship)	Chat.Screen("szFellowship: " +	Chi.szFellowship);
    //	if (Chi.szMonarch)	Chat.Screen("szMonarch: " +	Chi.szMonarch);
    //	if (Chi.szPatron)	Chat.Screen("szPatron: " +	Chi.szPatron);
    // } else {
    //	Chat.Screen(" ");
    //	Chat.Screen("Chi is null");
    // } // endif


function MainHandlers_OnAssessItem(Aco, fSuccess, Ibi, Iai, Iwi, Iei, Ipi) {
    if (debugEvent||debugLoot) Log.OutD("Main_OnAssessItem: Begin, Item [" + Aco.szName +"]" );
    AssessReceived = true;
    AssessAco = Aco;
    AssIbi = Ibi;
    AssIai = Iai;
    AssIwi = Iwi;
    AssIei = Iei;
    AssIpi = Ipi;

    // If this item is a corpse, see if corpse is on corpse loot list.  If on list, determine
    // if our kill and set flag.
    if (Aco.ocm & ocmMonsterCorpse) {
	var CorpseIndex = CorpseLookup(Aco);
	if (debugLoot) Log.OutD("Main_OnAssessItem: CorpseIndex [" + CorpseIndex + "]");
	if (CorpseIndex>=0) {
	    // Corpse found, see if we were the killers
	    CorpseList[CorpseIndex].AssessDone= true;
	    if ((Ibi) && (Ibi.szDescDetailed != undefined) && (Ibi.szDescDetailed != null) &&
	        (Ibi.szDescDetailed.indexOf(skapi.szName) >= 0)) {
		CorpseList[CorpseIndex].Ours = true;
		if (debugLoot) Log.OutD("Main_OnAssessItem: Our kill");
	    } else {
		CorpseList[CorpseIndex].Ours = false;
		if (debugLoot) Log.OutD("Main_OnAssessItem: Not our kill");
	    }
	} // endif
	return;
    } // endif

    // See if this item is on our loot list.  If so fill set the AssessDone flag for this item
    var LootIndex = LootLookup(Aco);
    if (debugLoot) Log.OutD("Main_OnAssessItem: LootIndex [" + LootIndex + "]");
    if (LootIndex>=0) LootList[LootIndex].AssessDone = true;

    if (debugEvent||debugLoot) Log.OutD("Main_OnAssessItem: End" );
} //end func MiscHandlers_OnAssessItem

function MainHandlers_OnChatBoxMessage(szMsg, cmc) {
    // Log.OutD("OnChatBoxMessage: szMsg [" + szMsg + "] cmc [" + cmc + "]");
    // szMsg = "+" + szMsg;	// Uncomment for admin alert testing
    var Info = szMsg.split(" ");
    // Log.OutD("OnChatBoxMessage: Info[0] = [" + Info[0] + "]");

    if (adminAbort && (Info[0].indexOf("+") >= 0)) {
    	if (State != stateAdmin) {
	    // Set state to Admin and save start time
	    Log.OutD("Main_OnChatBoxMessage: Admin broadcast chat detected, setting state to Admin...", true);
	    State = stateAdmin;
	    AdminStart = new Date();
	    FellowAbort = false;
	    if (State == stateGotoNext) {
		skapi.CancelNav();
		skapi.WaitEvent(1000, wemFullTimeout);
	    } // endif
	} // endif
    	if (ChatStart == 0) ChatStart = new Date();
    	NotifySupport("Admin: Broadcast chat from admin detected");
    	return;
    } // endif

    var MsgLen = szMsg.length - 1;
    var Msg    = szMsg.substr(0,MsgLen);

    if (manaStoneLimit < manaCnt) {
	// See if this is a mana too low message
	var Pattern = new RegExp ( "^Your .* is low on Mana.*$", "g" );	// detect mana low messages
	if (Pattern.test(Msg)) {
	    if (logCharge || debugCharge) Log.OutD("Main_OnChatBoxMessage: Low mana detected [" + Msg + "]" );
	    UseManaCharge = true;
	} // endif
    } // endif

} // end func

// OnChatBroadcast
function MainHandlers_OnChatBroadcast(acoSender, szMsg, cmc) {
    if (debugEvent) Log.OutD("Main_OnChatBroadcast: Begin, From [" + acoSender.szName + "] Msg [" + szMsg + "]");

    // See if this is an admin talking to us.  If so run special admin state if not already in it
    Name = acoSender.szName;
    if (flagAdminTest) Name = "+" + Name;

    if (adminAbort && (Name.indexOf("+") >= 0)) {
    	if (State != stateAdmin) {
	    // Set state to Admin and save start time
	    Log.OutD("Main_OnChatBroadcast: Admin broadcast chat detected, setting state to Admin...", true);
	    State = stateAdmin;
	    AdminStart = new Date();
	    FellowAbort = false;
	    if (State == stateGotoNext) {
		skapi.CancelNav();
		skapi.WaitEvent(1000, wemFullTimeout);
	    } // endif
	} // endif
    	NotifySupport("Admin: Broadcast chat from admin detected");
    	return;
    } // endif

    if (logChat) Log.OutD( "ChatBroad: " + Util.Pad(acoSender.szName,20)+ " " + szMsg );
} // end func

// OnChatLocal
function MainHandlers_OnChatLocal(acoSender, msg) {
    if (debugEvent||debugCmds)
	Log.OutD("Main_OnChatLocal: Begin, Id [" + Util.Hex(acoSender.oid) + "] Sender [" +
		  acoSender.szName + "] msg [" + msg + ")", true );

    if (logChat||debugCmds) Log.OutD( "OnChatLocal: Who [" + acoSender.szName + "] Msg [" + msg + "]" );

    // If chat is from us then...
    if (acoSender.szName == skapi.szName) {
	if (msg == "reset") {
	    RemoveInterface();
	    DisplayInterface();
	} // endif
	return;
    } // endif

    // See if this is an admin talking to us.  If so run special admin state if not already in it
    Name = acoSender.szName;
    if (flagAdminTest) Name = "+" + Name;

    if (adminAbort && (Name.indexOf("+") >= 0)) {
    	if (State != stateAdmin) {
	    // Set state to Admin and save start time
	    Log.OutD("Main_OnChatLocal: Admin local chat detected, setting state to Admin...", true);
	    State = stateAdmin;
	    AdminStart = new Date();
	    FellowAbort = false;
	    if (State == stateGotoNext) {
		skapi.CancelNav();
		skapi.WaitEvent(1000, wemFullTimeout);
	    } // endif
	} // endif
    	NotifySupport("Admin: Local chat from admin detected");
    	return;
    } // endif

    if (debugEvent||debugCmds) Log.OutD("Main_OnChatLocal: End");
} // end function MainHandlers_OnChatLocal

// OnChatServer
function MainHandlers_OnChatServer(text, code) {
    if (debugEvent) Log.OutD("Main_OnChatServer: Text [" + text + "] Type [" + code + "]" );

    // See if this message is a result of us trying to drain a creature.  If so record how much
    // we drained from the creature so we know if the next drain will be worthwhile.
    if (text.match(/gain/)) {
	if (text.match(/casting Drain Health/) || text.match(/casting Vitality Siphon/)) {
	    var String = " point"
	    var pos = text.indexOf(String);
	    if (pos!= 0)
	        Att.LastDrainHealth = eval(text.substring(9,pos));
	    else
	        Att.LastDrainHealth = 0;
	    if (logAttack||debugEvent) Log.OutD("Main_OnChatServer: Drain health result [" + Att.LastDrainHealth + "]");
	    return;
        } // endif

        if (text.match(/casting Drain Mana/) || text.match(/casting Essence Void/)) {
            var String = " point"
            var pos = text.indexOf(String);
            if (pos != 0)
                Att.LastDrainMana = eval(text.substring(9,pos));
            else
                Att.LastDrainMana = 0;
            if (logAttack||debugEvent) Log.OutD("Main_OnChatServer: Drain mana result [" + Att.LastDrainMana + "]");
            return;
        } // endif

        if (text.match(/casting Drain Stamina/) || text.match(/casting Vigor Siphon/)) {
            var String = " point"
            var pos = text.indexOf(String);
            if (pos != 0)
                AttLastDrainStam = eval(text.substring(9,pos));
            else
                AttLastDrainStam = 0;
            if (logAttack||debugEvent) Log.OutD("Main_OnChatServer: Drain Stamina Result [" + AttLastDrainStam + "]");
            return;
        } // endif
    } // endif

    if (RareMessage(text)) {
    	var logtext = "Rare message detected in OnChatServer, text [" + text + "]";
    	Log.OutD("OnChatServer: " + logtext);
    	ChatLog[ChatCnt] = Util.FormatDate(0,false) + logtext;
	ChatCnt++;
    } // endif

    // Following patterns will...
    var Pattern3 = new RegExp ( "^.* gives you .*$", "g" );	// detect gifts given to us
    var Pattern4 = new RegExp ( " gives you .*", "g" );		// determine the gift giver
    var Pattern5 = new RegExp ( "(.* gives you |.$)", "g" );	// extract the gift name

    if (Pattern3.test(text)) {
    	var Who = text.replace(Pattern4, "");
    	var Gift = text.replace(Pattern5, "");
    	if (logGifts||debugEvent) Log.OutD("Main_OnChatServer: Gift. Who [" + Who + "] Item [" + Gift + "]" );
    	return;
    } // endif

    if (logDebuffs) {
    	// Valid:
    	//    Virindi Observer cast Feeblemind Other VI on you
    	//
    	// Invalid:
    	//    Obsidian Golem tried to cast a spell on you, but was too far away
    	//    Virindi Observer cast Bafflement Other VI on you, surpassing Bafflement Other VI
    	//    You cast Revitalize Self VI and restore 86 points of your stamina.
    	//    You cast Fire Vulnerability Other VI on Tusker Guard
    	//    You cast Celcynd's Blessing on yourself, surpassing Celcynd's Blessing

	// This pattern will detect spells cast on you by others
	var Pattern6 = new RegExp(".* cast .* on you($|, surpassing.*$)", "g");

	// This pattern will remove everything except the spell name from the text
	var Pattern7 = new RegExp("(^.* cast | on you.*$)", "g");

	// This pattern will extract creature casting spell at us
	var Pattern8 = new RegExp(" cast .* on you.*$", "g");

	// If pattern6 matches message text, then it looks like someone just cast a spell on us
	if (Pattern6.test(text)) {
    	    // Extract spell name from message text
    	    var SpellName = text.replace(Pattern7, "");
    	    var CreName   = text.replace(Pattern8, "");
    	    Log.OutD("Main_OnChatServer: Debuff [" +  SpellName + "] cast by [" + CreName + "]");
    	    return;
	} // endif
    } // endif

    if (debugEvent) Log.OutD("Main_OnChatServer: End" );
} // end function MainHandlers_OnChatServer


function MainHandlers_OnCommand(szCmd) {
    if (debugEvent||debugCmds) Log.OutD("Main_OnCommand: Begin, Command [" + szCmd + "]", true);

    var Split = szCmd.split(" ");
    for (var x=0; x<Split.length; x++) {
	Split[x] = Util.Trim(Split[x]);
	if (debugCmds) Log.OutD("Main_OnChatLocal: Split # [" + x + "] Value [" + Split[x] + "]", true);
    } // endfor

    var Command = Split[0].toUpperCase();
    if (debugCmds) Log.OutD("Main_OnChatLocal: Command [" + Command + "]", true);

    if (debugEvent||debugCmds) Log.OutD("Main_OnCommand: End", true);
    return;
} // end function


// MainHandlers_OnDeathMessage
function MainHandlers_OnDeathMessage(msg) {
    if (debugEvent||debugAttack||debugDeathMsg) Log.OutD("Main_OnDeathMessage: Begin, Msg [" + msg + "]");
    // Chat.Screen("Main_OnDeathMessage: Begin, Msg [" + msg + "]");

    // Determine what kill attempt this message belongs to. There are 3 possibilities:
    //
    // 1) The message is from the previous creature we were attacking.  It is possible
    //    for an abort on a creature to happen while a war bolt is the air due to a
    //    critical condition detected (ie multiple creatures close to us) or the current
    //    creature went out of range but the war spell eventually hit the target past
    //    radar range.  We can detect these conditions because the message will always
    //    come within a few seconds of the start of the next kill attempt (before a war
    //    spell has a chance to finish casting) and most of the time the next selected
    //    creature will have a different name.  There is a slight chance of misidentifying
    //    this message if the deathmessage event comes after we have finished casting a
    //    war spell for the new target and the new target has the same name as the old one.
    //
    // 2) The message is from the primary creature selection for this kill attempt.
    //    The death message can only belong to the current creature if the current
    //    state is stateKill, the substate is stateWar, the currently selected creature's
    //    name is in the death message, and we have successfully cast a war spell for
    //	  this kill attempt
    //
    // 3) The message is from one of the secondary creature selections. If the primary and
    //    secondary creatures have different names, then it is easy to detect which one we
    //    killed.  If they are the same then use the variable AttCtx.LastTarget which tells
    //    us what target the last war spell was cast at.

    // If no attack record defined then user is probably killing creatures with macro paused
    if ((Att     == undefined) || (Att     == null) ||
        (Att.Aco == undefined) || (Att.Aco == null)) {
	return;
    } // endif

    // If this is the first kill, then set the StartTime to the current time.
    if (TotalKilled == 0) StartTime = new Date();

    TotalKilled++;
    AttLoc[AttLocCur].TotalKilled++;

    // See if this is the previous attack attempt.  If the death name matches the previous kill
    // attempt, and the current primary and secondary name is different or the last attempt
    // ended less than 3 seconds ago then this message is probably from the previous kill attempt.
    var CreNameExp = new RegExp(AttCtx.PrevCreName);
    if ((msg.match(CreNameExp)) &&
        ((Att.CreName != AttCtx.PrevCreName) || ((new Date()-AttCtx.LastDone) < 3000))) {
    	KillExp += AttCtx.PrevCreExp;
    	AttLoc[AttLocCur].TotalExp += AttCtx.PrevCreExp;
    	if (logKills||debugEvent||debugAttack||debugDeathMsg)
    	    Log.OutD("Main_OnDeathMessage: End, Seems to be previous selection, Exp [" + AttCtx.PrevCreExp +
		     "] Total kill exp [" + KillExp + "]");
	return;
    } // endif

    // See if this message is from the death of the primary target of the current kill attempt
    // (the most likely case).  This is the most important message to detect as it enables us
    // to rapidly switch to the next target.
    // Chat.Screen("Main_OnDeathMessage: Att.CreName [" + Att.CreName + "]");
    var CreNameExp = new RegExp(Att.CreName);
    if (msg.match(CreNameExp)) {
	AttCtx.LastKillOid  = Att.Oid;
	AttCtx.LastKillTime = new Date();
	AttCtx.PrevCreName  = Att.CreName;
	KillExp += Att.CreExp;
	AttLoc[AttLocCur].TotalExp += Att.CreExp;
	KillDone(); // Kill attempt is finished so save some info and set next state
	if (logKills||debugEvent||debugAttack||debugDeathMsg)
	    Log.OutD("Main_OnDeathMessage: End, Seems to be primary target, Exp [" + Att.CreExp +
		     "] Total kill exp [" + KillExp + "]");
	return;
    } // endif

    Log.OutD("Main_OnDeathMessage: End, Unable to identify death message [" + msg + "]");
    return;
} // end function MainHandlers_OnDeathMessage

// OnDeathSelf()
function MainHandlers_OnDeathSelf( acoKiller, msg) {
    Log.OutD("Main_OnDeathSelf: Player killed by id [" + Util.Hex(acoKiller.oid) + "] Name [" +
		acoKiller.szName + "] Msg [" + msg + "]", true);

   DeathStealthDisabled = false;
   if (Stealth) {
   	Log.OutD("Main_OnDeathSelf: Disabling stealth mode");
   	Chat.Screen("WARNING! Disabling stealth mode!");
	Stealth = false;	// Disable stealth so that we don't autolog if people at lifestone
	DeathStealthDisabled = true;
        skapi.SetControlProperty(szPanelControl, "btnStealth", "Text", "Enable Stealth");
    } // endif

    Dead = true;		// Set to true when we die.  Triggers special death processing
    DeathStart = new Date();

    // Clear retreat flags and cancel and active navigations, otherwise we may try to run
    // to the current location after death.
    RetreatActive = false;
    AdvanceReq = false;
    skapi.CancelNav();

    StateChange(stateDied,NoCheck);
    Log.OutD("Main_OnDeathSelf: End");
    return;
} // end func

// OnDisconnect
function MainHandlers_OnDisconnect() {
    Log.OutD("Main_OnDisconnect: Begin");
    Log.OutD("Main_OnDisconnect: Unexpected event, exiting macro...");
    StateChange(stateExit,NoCheck);
    RunFlag = 0;
    Log.OutD("Main_OnDisconnect: End");
} // end func

// OnEnd3D
function MainHandlers_OnEnd3D() {
    Log.OutD("Main_OnEnd3D: Begin");
    CharOnEnd3D = true;

    // If we are we are resetting the ac client, switching chars, logging off, or hiding from danger,
    // or just logging on then this event is expected so just return
    if ((State==stateLog) || (State==stateHide) || (State==statePaused)) {
    	Log.OutD("Main_OnEnd3D: End, Expected event. State [" + State + "/" + textState[State] + "]");
    	return;
    } // endif

    // Unexpected OnEnd3D message encountered, exit macro
    Log.OutD("Main_OnEnd3D: Unexpected event, exiting macro...");
    StateChange(stateExit,NoCheck);
    RunFlag = 0;
    Log.OutD("Main_OnEnd3D: End");
} // end func

function MainHandlers_OnEndPortalSelf() {
    if (debugEvent) Log.OutD("Main_OnEndPortalSelf:  Portal has ended");
    CharEndPortal = true;
} // end function

function MainHandlers_OnFellowInvite(szSender) {
    if (debugEvent) Log.OutD("Main_OnFellowInvite: Begin, Requester [" + szSender + "]");

    // If character reqeusting us to accept fellow is authorized, then accept fellow reqeust, else
    // reject it and send them a nasty message
    if (flagAcceptFellow || (Authorized(szSender))) {
    	Log.OutD("Character [" + szSender + "] is trying to fellow with us, accepting...", true);
    	skapi.Keys("Y");
    } else {
    	// Not authorized
    	Log.OutD("Character [" + szSender + "] is trying to fellow with us, rejecting...", true);
   	skapi.Keys("N");
    	// Chat.Tell(szSender, "Leave me alone");
    } // endif

    if (debugEvent) Log.OutD("Main_OnFellowInvite: End");
} // end function

function MainHandlers_OnFellowRecruit(fellowObj) {
    if (debugSupport)
	Chat.Screen("FellowRecruit: name [" + fellowObj.szName + "] healthCur [" + fellowObj.healthCur + "]");
    if (fellowObj.szName == skapi.szName) return;
    if (!(attHealFellow || attStamFellow)) return;

    // See if oid already exists on help list
    for (var x=0; x<HelpCnt; x++) {
	if (HelpList[x].Oid == fellowObj.oid) break;
    } // endfor
    if (debugSupport) Chat.Screen("FellowRecruit: HelpCnt [" + HelpCnt + "] x [" + x + "]");
    
    // If fellow members health or stamina is low then...
    if ((attHealFellow && ((fellowObj.healthMax  - fellowObj.healthCur)  >= supportHealth )) ||
        (attStamFellow && ((fellowObj.staminaMax - fellowObj.staminaCur) >= supportStamina))) {
	// Add to list if necessary
	if (debugSupport) Chat.Screen("Health or stam low");
	if (x == HelpCnt) {
	    HelpList[x] = new HelpObj(fellowObj.oid);
	    HelpCnt++;
	} // endif

	// Update current health stamina totals
	HelpList[x].MaxHealth  = fellowObj.healthMax;
	HelpList[x].Health     = fellowObj.healthCur;
	HelpList[x].MaxStamina = fellowObj.staminaMax;
	HelpList[x].Stamina    = fellowObj.staminaCur;
	if (debugSupport) {
	    Chat.Screen("FellowRecruit: HelpList[x].Health [" + HelpList[x].Health +
		        "] HelpList[x].MaxHealth [" + HelpList[x].MaxHealth + "]");
	    Chat.Screen("FellowRecruit: HelpList[x].Stamina [" + HelpList[x].Stamina +
		        "] HelpList[x].MaxStamina [" + HelpList[x].MaxStamina + "]");
	} // endif
    } else {
	// Stam and health ok, if member on help list then remove
	if (debugSupport) Chat.Screen("Healh and stam ok");
	if (x < HelpCnt) {
	    if (debugSupport) Chat.Screen("Removing OID from helplist");
	    HelpCnt--;
	    if (HelpCnt > x) HelpList[x] = HelpList[HelpCnt];
	} // endif
    } // endif
    if (debugSupport) Chat.Screen("end FellowRecruit");
} // end function


function MainHandlers_OnFellowUpdate(fellowObj) {
    if (debugSupport)
	Chat.Screen("FellowUpdate: name [" + fellowObj.szName + "] healthCur [" + fellowObj.healthCur + "]");
    if (fellowObj.szName == skapi.szName) return;
    if (!(attHealFellow || attStamFellow)) return;

    // See if oid already exists on help list
    for (var x=0; x<HelpCnt; x++) {
	if (HelpList[x].Oid == fellowObj.oid) break;
    } // endfor
    if (debugSupport) Chat.Screen("FellowUpdate: HelpCnt [" + HelpCnt + "] x [" + x + "]");
    
    // If fellow members health or stamina is low then...
    if ((attHealFellow && ((fellowObj.healthMax  - fellowObj.healthCur)  >= supportHealth )) ||
        (attStamFellow && ((fellowObj.staminaMax - fellowObj.staminaCur) >= supportStamina))) {
	// Add to list if necessary
	if (debugSupport) Chat.Screen("Health or stam low");
	if (x == HelpCnt) {
	    HelpList[x] = new HelpObj(fellowObj.oid);
	    HelpCnt++;
	} // endif

	// Update current health stamina totals
	HelpList[x].MaxHealth  = fellowObj.healthMax;
	HelpList[x].Health     = fellowObj.healthCur;
	HelpList[x].MaxStamina = fellowObj.staminaMax;
	HelpList[x].Stamina    = fellowObj.staminaCur;
	if (debugSupport) {
	    Chat.Screen("FellowUpdate: HelpList[x].Health [" + HelpList[x].Health +
		       "] HelpList[x].MaxHealth [" + HelpList[x].MaxHealth + "]");
	    Chat.Screen("FellowUpdate: HelpList[x].Stamina [" + HelpList[x].Stamina +
		    "] HelpList[x].MaxStamina [" + HelpList[x].MaxStamina + "]");
	} // endif
    } else {
	// Stam and health ok, if member on help list then remove
	if (debugSupport) Chat.Screen("Healh and stam ok");
	if (x < HelpCnt) {
	    if (debugSupport) Chat.Screen("Removing OID from helplist");
	    HelpCnt--;
	    if (HelpCnt > x) HelpList[x] = HelpList[HelpCnt];
	} // endif
    } // endif
    if (debugSupport) Chat.Screen("end FellowUpdate");
} // end function


function MainHandlers_OnLeaveVendor(acoMerchant) {
    if (debugEvent) Log.OutD("Main_OnLeaveVendor: Begin");
    VendorOpen = false;
    return;
    if (debugEvent) Log.OutD("Main_OnLeaveVendor: End");
} // end func


// OnMeleeDamageSelf
function MainHandlers_OnMeleeDamageSelf(name, damage, type, bpart) {
    if (debugEvent) Log.OutD("Main_OnMeleeDamageSelf: Attacker [" + name + "] Damage [" + damage +
			     "] Type [" + type + "] Body Part [" + bpart + "]" );

    LastAttTime      = new Date();
    LastMeleeDamTime = new Date();
    LastMeleeDamName = name;
    LastMeleeDamAmt  = damage;
    LastMeleeDamType = type;
    LastMeleeDamPart = bpart;

   // Log this attack to event file if we exceed the critical damage threshold.
    if (logCrits&&(damage>=critThresh))
        Log.OutD("Main_OnMeleeDamageSelf: Attacker [" + name + "] Damage [" + damage +
		 "] Type [" + Util.Hex(type) + "] Body Part [" + damBody[bpart] + "]");

    // If damage exceeds the retreat damage threshold, retreating is allowed, we are not already retreating, we
    // have not just died, the current attackloc has a retreat point, and we are not moving, then set flag that
    // causes us to retreat like a Frenchman.
    if ((damage >= retreatDamThresh) && flagRetreatCrit && !RetreatActive && !RetreatReq &&
        !Dead && (AttLoc[AttLocCur].RetreatLoc!=null) && (State != stateGotoNext) && (State != stateGotoCurrent)) {
	Log.OutD("Main_OnMeleeDamageSelf: Damage [" + damage + "] from [" + name +
		 "] exceeds retreat threshold, 'retreatReq' flag set.");
	Chat.Screen("Damage threshold exceeded, retreating...");
	RetreatReq = true;
    } // endif
    
    // If this damage amount exceeds our gem threshold, we can use stillness gems, there are creatures
    // near us, our health is low, we have not used a gem recently, and we have lifevulns on us, then
    // use a gem if we have one in inventory
    if ((damage>=gemCritThresh) && flagGem && Danger && (skapi.healthCur < gemHealthThresh)) {
	// See if last gem use was at least 'gemTimeInt' milliseconds ago
	if ((new Date()-CharLastGem) > gemTimeInt) {
	    // Last gem usage was long enough ago, now see if we have any life vulns on us
	    if (LifeVulnActive()) {
		// Ok, now see if we have any gems.  If so then use one
		var GemAco = skapi.AcoFindInInv("Gem of Stillness");
		if (GemAco != null) {
		    if (debugEvent||logVitals||debugVitals) Log.OutD("OnMeleeDamageSelf: Using gem of Stillness");
		    GemAco.Use();
		    CharLastGem = new Date();
		} else {
		    // Out of gems, report condition
		    if (debugEvent||logVitals||debugVitals) Log.OutD("OnMeleeDamageSelf: Out of gems");
		} // endif
	    } // endif
	} // endif
    } // endif

    if (!flagIgnoreAtt && (State != stateKill) && (State != stateSelect) &&
    	  (State != stateTarget) && (State != stateGotoNext) && 
    	  (State != stateGotoCurrent) && !(attSupportOnly && !attAlwaysVuln && !attAlwaysImperl)) {
	    Log.OutD("OnMeleeDamageSelf: Old state [" + State + "], changing state to select...");
	    StateChange(stateSelect);
    } // endif
} // end function

// OnMeleeEvadeSelf
function MainHandlers_OnMeleeEvadeSelf(Name) {
    if (debugEvent) Log.OutD("Main_OnMeleeEvadeSelf: Source [" + Name + "]");

    LastMeleeEvade = new Date();
    LastAttTime = new Date();

    if (!flagIgnoreAtt && (State != stateKill) && (State != stateSelect) && (State != stateTarget) &&
        (State != stateGotoNext) && (State != stateGotoCurrent) &&
        !(attSupportOnly && !attAlwaysVuln && !attAlwaysImperl)) {
	Log.OutD("OnMeleeEvadeSelf: Old state [" + State + "], changing state to select...");
	StateChange(stateSelect);
    } // endif
} // end function

function MainHandlers_OnMeleeLastAttacker(Aco) {
    if (debugEvent) Log.OutD("OnMeleeLastAttacker: Begin, Name [" + Aco.szName + "]");

    // See if this is a stale Aco
    if ((Aco == undefined) || (Aco == null) || !Aco.fExists || (Aco.szName == "")) {
	if (debugEvent) Log.OutD("OnMeleeLastAttacker: Stale ACO, ignoring event");
	return;
    } // endif

    // If the attacker is not a monster, then we ignore the attacker unless it
    // is a player and the attack player flag is set.  Lots of harmless actions
    // trigger this event like putting on jewelry with spells.
    if (Aco.ocm & ocmMonster) {
	if (debugEvent) Log.OutD("OnMeleeLastAttacker: Attacker is a monster, continuing...");
    } else {
	if (Aco.ocm & ocmPlayer) {
	    if (flagAttackPlayer) {
		// A PK player casting a beneficial spell on us will be targeted as the last attacker.  So 
		// normally we ignore this condition unless the attack player flag is set
		if (debugEvent) Log.OutD("OnMeleeLastAttacker: Flag set to attack players, continuing...");
	    } else {
		if (debugEvent) Log.OutD("OnMeleeLastAttacker: Flag set to ignore player attacks, exiting.");
		return;
	    } // endif
	} else {
	    if (debugEvent) Log.OutD("OnMeleeLastAttacker: Not a monster or player, exiting.");
	    return;
	} // endif
    } // endif   

    // If the attacking creature is the creature we are currently killing, then no sense setting it up
    // as the next target
    if ((State == stateKill) && (Aco.oid == Att.Oid)) {
	if (debugEvent) Log.OutD("OnMeleeLastAttacker: Attacker is creature we are currently killing, ignoring.");
	return;
    } // endif
    
    // If flag is set to ignore attacks, then exit
    if (flagIgnoreAtt) {
    	if (debugEvent) Log.OutD("OnMeleeLastAttacker: Flag set to ignore all attacks, exiting.");
    	return;
    } // endif

    // Set flag to use aco of last attacker as next target.  Note that we still give priority
    // to creatures close to us over last attacker which could be a creature far away throwing
    // clubs at us.
    SelUseAttackAco  = true;
    LastAttTime      = new Date(); 
    AttCtx.AttackAco = Aco;		// Save Aco of last attacker

    if ((State != stateKill) && (State != stateSelect) && (State != stateTarget) &&
        (State != stateGotoNext) && (State != stateGotoCurrent) &&
        !(attSupportOnly && !attAlwaysVuln && !attAlwaysImperl)) {
        Log.OutD("OnMeleeLastAttacker: State [" + State + "/" + textState[State] + "] Name/Oid [" + Aco.szName + "/" +
	     	  Util.Hex(Aco.oid) + "] just attacked us, setting SelUseAttackAco flag.");
	Log.OutD("OnMeleeLastAttacker: Changing state to Select...");
	StateChange(stateSelect);
    } // endif
    
    if (debugEvent) Log.OutD("OnMeleeLastAttacker: End");
    return;
} // end func MainHandlers_OnMeleeLastAttacker

function MainHandlers_OnNavStop(nsc) {
    if (debugEvent||debugMove) Log.OutD("Main_OnNavStop: Begin, Stop code [" + nsc + "]");
    MoveDone	= true;
    MoveCode	= nsc;
} // end func MainHandlers_OnNavStop

function MainHandlers_OnObjCre(aco) {
    if (debugEvent||debugLoot||debugEvade||debugMove)
	Log.OutD("Main_OnObjCre: Begin, Obj [" + aco.szName + "] Oid [" + Util.Hex(aco.oid) + "] created");

    // If user wants to alert on specific items, then see if we have match in item audible list
    if (itemAudAlert && !(aco.ocm & ocmMonsterCorpse)) {
    	var NameUp = aco.szName.toUpperCase();
	for (x=0; x<itemAudList.length; x++) {
	    Item = itemAudList[x].toUpperCase();
	    if (NameUp.indexOf(Item) >= 0) {
		var ObjLoc = aco.maploc;
		if ((ObjLoc != undefined) && (ObjLoc != null)) {
		    var OurLoc  = skapi.maplocCur;
		    var Heading = Util.Round(OurLoc.HeadToMaploc(ObjLoc),0);
		    var Distance = Util.RoundFill(Util.Distance(ObjLoc,OurLoc),2);
		    Chat.Screen(aco.szName + "  Head: " + Heading + " Dist: " + Distance + " Coords: " +
				Util.FormatCoords(ObjLoc,2), cmcYellow);
		    if ((new Date()-ItemLastAlert) > 2000) {
			alertAbortWav.Play();
			ItemLastAlert = new Date();
		    } // endif
		} // endif
	    } // endif
	} // end for
    } // endif

    // See if object created is coming towards us and is a known dangerous avoidable object (like
    // war spell or missle attack).  Since I don't know how to detect all the missile types of 
    // objects, what we will do is assume anything I can't identify is potentially harmful.
    var Name = aco.szName;
    if (flagWarAvoid && (aco.maploc != null) && 
    	(!(aco.ocm & (ocmPlayer | ocmMonster | ocmPlayerCorpse | ocmMonsterCorpse)))) {
    	if (debugEvade) Log.OutD("Main_OnObjCre: Unknown object detected, see if it will hit us");
    	// Object detected, see if it is heading in our direction
    	var OurLoc = skapi.maplocCur;
	var ObjLoc = aco.maploc;
	if (ObjLoc == null) {
	    if (debugEvade) Log.OutD("Main_OnObjCre: Object [" + Name + "] does not have a maploc, aborting check");
	    return;
	} // endif

	// If object created really close to us, then don't attempt to move
	var Distance = Util.Distance(ObjLoc,OurLoc);
	if (Distance < avoidMinDist) {
	    if (debugEvade) Log.OutD("Main_OnObjCre: War spell too close to avoid, aborting check");
	    return;
	} // endif

	// Ok we have all the data we need to calculate how close the object will come to us
	// and it's far enough away for us to move out of the way.  Now see it it will hit us.
	var HeadToUs = ObjLoc.HeadToMaploc(OurLoc);
	var Angle    = ObjLoc.head - HeadToUs;
	var Radians  = (Angle*2*Math.PI)/360;
	var Closest  = Distance*Math.sin(Radians);

	// Now see if the angles are close.  If not abort the check.  We do this because an object
	// heading away from us 180 degrees could generate a false positive.
	var Diff = Math.abs(Angle)
	if (Diff > 180) Diff=360-Diff;
	if (Diff>avoidMaxAngle) {
	    if (debugEvade)  Log.OutD("Main_OnObjCre: Angle diff [" + Diff + "] aborting check");
	    return;
	} // endif

	if (debugEvade) {
	    Log.OutD("Main_OnObjCre: Object      [" + Name		+ "] detected");
	    Log.OutD("Main_OnObjCre: Obj Coords  [" + Util.FormatCoords(ObjLoc,3) + "]");
	    Log.OutD("Main_OnObjCre: Heading     [" + ObjLoc.head	+ "]");
	    Log.OutD("Main_OnObjCre: Distance2Us [" + Distance		+ "]");
	    Log.OutD("Main_OnObjCre: Obj Head2Us [" + HeadToUs		+ "]");
	    Log.OutD("Main_OnObjCre: Our Heading [" + OurLoc.head	+ "]");
	    Log.OutD("Main_OnObjCre: AngleDiff   [" + Angle		+ "]");
	    Log.OutD("Main_OnObjCre: Radians     [" + Radians		+ "]");
	    Log.OutD("Main_OnObjCre: Closest     [" + Closest		+ "]");

	} // endif

	if (Math.abs(Closest) > avoidDangerThresh) {
	    if (debugEvade) Log.OutD("Main_OnObjCre: Object will miss us, aborting check.");
	    return;
	} else {
	    if (debugEvade) Log.OutD("Main_OnObjCre: Object will hit us.");
	} // endif

	// Ok object will most likely hit us, so try to move out of the way. If we are in the
	// middle of casting a spell and slidecasting is enabled then try to slidecastset, otherwise
	// set a flag so that the FSM will try to move us.  We don't do it here since we may be in
	// the middle of casting a spell when this event fires and any movement may abort the spell. 
	
	if (flagSlideCast && CastActive) {
	    if ((new Date()-LastSlideTime)>avoidSlideHold) {
		if (debugEvade) Log.OutD("Main_OnObjCre: We are casting a spell, trying to slidecast...");
		WarEvade(true);
		LastSlide = new Date();
	    } else {
	    	if (debugEvade) Log.OutD("Main_OnObjCre: Tried to slide too recently, ignoring war spell..");
	    } // endif
	} else {
	    if (debugEvade) Log.OutD("Main_OnObjCre: We are not casting a spell, setting EvadeWar flag.");
	    EvadeWar = true;
	} // endif
	return;
    } // endif

    // If object created is a monster, then no need to check any further so exit function
    if (aco.ocm & ocmMonster) {
	if (debugLoot||debugMove) Log.OutD("Main_OnObjCre:Monster [" + aco.szName + "] detected");
	
	// If we have the advance on close creature spawn option enabled for the current attack loc, then set
	// flag to advance to the next attack location if the spawn is close to our kill or retreat location
	if (AttLoc[AttLocCur].LocFlags & navFlagSpawnAdv) {
	    // See if close to kill location
	    var DistToKillLoc = Util.Distance(aco.maploc, AttLoc[AttLocCur].KillLoc);
	    if (DistToKillLoc < navSpawnDist) {
		SpawnAdvance = true;
		if (logMove||debugMove)
		    Log.OutDM("Main_OnObjCre", "Spawn detected near killloc, moving to next attack loc...", true);
		return;
	    } // endif

	    // If a retreat location is defined, then see if spawn is close to it.
	    if (AttLoc[AttLocCur].RetreatLoc != null) {
		var DistToRetreatLoc = Util.Distance(aco.maploc, AttLoc[AttLocCur].RetreatLoc);
		if (DistToRetreatLoc < navSpawnDist) {
		    SpawnAdvance = true;
		    if (logMove||debugMove)
		        Log.OutDM("Main_OnObjCre", "Spawn detected near retreatloc, moving to next attack loc...", true);
		    return;
		} // endif
	    } // endif
	} // endif
	return;
    } // endif

    // If flag is set to process items, then see if this object is a corpse or an item on a corpse
    if (flagLootCheck) {
	// If this object is a corpse and we have not previously looted it, then add it to the
	// list of corpses to check next Loot state.  We also don't add the corpse to the
	// corpselist if its burden is 6000 (which means corpse is empty).
	if ((aco.maploc != null) && (aco.oty & otyCorpse) && (aco.ocm & ocmMonsterCorpse) && (aco.burden>6000)) {
	    if (debugLoot) {
	        Log.OutD("Main_OnObjCre: Corpse [" + aco.szName + "] Ocm [" + Util.Hex(aco.ocm) +
			 "] Coords [" + Util.FormatCoords(aco.maploc,2)+ "]");
	        Log.OutD("Main_OnObjCre: Olc [" + Util.Hex(aco.olc) + "] Oty [" +
	    		 Util.Hex(aco.oty) + "] Eqm [" + Util.Hex(aco.eqm) + "]");
	    } // endif

	    // See if corpse was previously processed
	    for (var x=0; x<lootPrevCorpseMax; x++) {
	    	if (PrevCorpseList[x] == aco.oid) {
	    	    if (debugLoot) Log.OutD("Main_OnObjCre: End, Corpse on previous loot list, ignoring.");
	    	    return;
	    	} // endif
	    } // endfor

	    // See if corpse already on corpse list which could happen if we leave and then
	    // shortly re-enter an area.  If so, then skip this corpse
	    for (var x=0; x<CorpseCount; x++) {
	    	if (aco.oid == CorpseList[x].Oid) {
		    if (debugLoot) Log.OutD("Main_OnObjCre: End, Corpse already on current corpse list, ignoring.");
		    return;
	    	} // endif
	    } // endfor

	    // See if corpse is on skip list CorpseSkipList
	    for (var x=0; x<corpseSkipList.length; x++) {
	    	if (aco.szName.indexOf(corpseSkipList[x]) >= 0) {
	    	    if (debugLoot) Log.OutD("Main_OnObjCre: End, Corpse on corpse skip list, ignoring.");
	    	    return;
	    	} // endif
	    } // endfor

	    // Add corpse to CorpseList
	    CorpseList[CorpseCount] = new CorpseObj(aco);
	    if (lootCheckOwner) {
		CorpseList[CorpseCount].AssessCount = 1;
		CorpseList[CorpseCount].AssessTime = new Date();
		skapi.AssessAco(aco);
	    } else {
	    	CorpseList[CorpseCount].AssessReq = false;
	    } // endif
	    CorpseCount++;
	    if (debugLoot) Log.OutD("Main_OnObjCre: End, Corpse added to CorpseList.");
	    return;
	} // endif

	// See if this object is on item list
	var LootIndex = LootLookup(aco);
	if (LootIndex>=0) {
	    // Object on list
	    if (debugLoot)
		Log.OutD("Main_OnObjCre: Item [" + aco.szName + "/" + Util.Hex(aco.oid) + "] Index [" +
			 LootIndex + "] eqm [" + Util.Hex(aco.eqm) + "] found on corpse item list.");
	    LootList[LootIndex].ObjCreate = true;
	    if (NoAssessItem(aco)) {
	    	LootList[LootIndex].AssessReq = false;
	    	if (debugLoot) Log.OutD("Main_OnObjCre: Assess not required");
	     } else {
	    	LootList[LootIndex].AssessReq = true;
	    	if (debugLoot) Log.OutD("Main_OnObjCre: Assess required");
	    } // endif
	} // endif
    } // endif
    if (debugEvent||debugLoot||debugMove) Log.OutD("Main_OnObjCre: End");
} // end function MainHandlers_OnObjCre

// OnObjCrePlayer
function MainHandlers_OnObjCrePlayer(aco) {
    Log.OutD("Main_OnObjCrePlayer: Begin, Name [" + aco.szName + "]");

    var Name = aco.szName;
    if (flagAdminTest) Name = "+" + Name;	// Uncomment when testing

    // If player detected is ourselves, then we are done
    if (Name == skapi.szName) return;

    var ObjLoc = aco.maploc;
    var OurLoc = skapi.maplocCur;
    var Distance = Util.Distance(ObjLoc, OurLoc);

    // Log event if flag set
    if (logPlayers) {
    	Log.OutD("OnObjCrePlayer: Player detected [" + Name + "] Dist [" + Distance + "] Coords [" +
    		 Util.FormatCoords(ObjLoc,3) + "]");
   	Log.OutD("OnObjCrePlayer: Heading [" + Util.Round(ObjLoc.head,0) + "] OurCoords [" +
   		 Util.FormatCoords(OurLoc,3) + "] Head2Us [" + Util.Round(ObjLoc.HeadToMaploc(OurLoc),0) + "]");

   	// See how close player will come to us on current heading and report info
	var HeadToUs = ObjLoc.HeadToMaploc(OurLoc);
	var Angle    = ObjLoc.head - HeadToUs;
	var Radians  = (Angle*2*Math.PI)/360;
	var Closest  = Math.abs(Distance*Math.sin(Radians));
	Log.OutD("OnObjCrePlayer: Closest distance player will come to us [" + Util.Round(Closest,3) + "]");
    } // endif

    // Send info to chat screen if flag set
    if (flagReportPlayer) {
    	var OutLine = "Player detected: " + Util.Width(Name,1.7) + "   Dist: " + Util.Round(Distance,2);
    	if (Distance > 0.01)
	    OutLine = OutLine + "   Heading: " + Util.Round(OurLoc.HeadToMaploc(ObjLoc),0) +
				"   Coords: " + Util.FormatCoords(ObjLoc,2);
	Chat.Screen(OutLine);
    } // endif

    // See if this is an admin.  If so run special admin state if not already in it
    if (adminAbort && (State != stateAdmin) && (Name.indexOf("+") >= 0)) {
    	// Compute distance to admin.  If admin outside of radar range
    	// then just log, else try a little deception

	if (State == stateGotoNext) {
	    skapi.CancelNav();
	    skapi.WaitEvent(1000, wemFullTimeout);
	} // endif

    	if (Distance < 0.30) {
    	    // Set state to Admin and save start time
    	    Log.OutD("Main_OnObjCrePlayer: Admin detected in radar range, setting state to Admin...", true);
    	    State = stateAdmin;
    	    AdminStart = new Date();
    	    NotifySupport("Admin: Admin just materialized near me");
	} else {
	    // Admin detected outside of radar range, so log
   	    Log.OutD("Main_OnObjCrePlayer: Admin detected outside of radar range, logging...", true);
    	    State = stateLog;
	    NotifySupport("Admin: Admin materialized just outside of radar range");
	} // endif
	FellowAbort = false; // Cancel any abort requests from fellow members near us
	return;
    } // endif

    // If approaching player is on friends list, then return
    if (debugHide) Log.OutD("Main_OnObjCrePlayer: friendsList length [" + friendsList.length + "]");
    for (var loop=0; loop < friendsList.length ; loop++) {
    	if (debugHide) Log.OutD("Main_OnObjCrePlayer: friendsList entry [" + loop + "] Name [" + friendsList[loop] + "]");
	if (Name == friendsList[loop] ) {
	    Log.OutD("Main_OnObjCrePlayer: Player [" + Name + "] on friends list, aborted." );
	    return;
	} // endif
    } // endfor

    // If allegiance stealth flag is clear, then don't log if approaching player is member of our allegiance
    if (!flagAllegStealth) {
	if (debugHide) Log.OutD("Main_OnObjCrePlayer: Checking to see if player member of our allegiance...");
    	if (aco && (aco.oidMonarch == skapi.acoChar.oidMonarch)) {
    	    Log.OutD("Main_OnObjCrePlayer: Player [" + Name + "] monarch member, stealth aborted.");
    	    return;
    	} else {
    	    Log.OutD("Main_OnObjCrePlayer: Player [" + Name + "] not monarchy member.");
    	} // endif
    } // endif
    
    // Waiting for skunkworks enhancement before I can implement this
    // If approaching player a member of an allegiance on 'allegianceList', then don't autolog
    // if (debugHide) {
    //	Log.OutD("Main_OnObjCrePlayer: allegianceList length [" + allegianceList.length + "]");
    //	Log.OutD("Main_OnObjCrePlayer: players allegiance [" + aco.oidMonarch + "]");
    // } // endif
    // for (var loop=0; loop < allegianceList.length ; loop++) {
    //	if (debugHide)
    //    Log.OutD("Main_OnObjCrePlayer: allegianceList entry [" + loop + "] Name [" + allegianceList[loop] + "]");
    //   if (aco && (aco.szMonarch == allegianceList[loop])) {
    //    Log.OutD("Main_OnObjCrePlayer: Player [" + Name + "] on allegianceList list, aborted." );
    //     return;
    // } // endif
    // } // endfor

    // If stealth mode not enabled then just play warning sound if request and return
    if (!Stealth) {
    	if (alertPlayer) alertPlayerWav.Play();
	if (debugHide) Log.OutD("Main_OnObjCrePlayer: Stealth mode disabled. Aborting...");
	return;
    } // endif

    // If the macro is paused, we are already in stealth mode, we are logging out, or we have just
    // died then abort
    if (MacroPaused || (State == stateHide) || (State == stateLog) || (State == stateDied)) return;

    // If we make it here, then an unknown player was detected so go hide.  If spamPlayer flag
    // set, then notify fellow that unknown player detected by us
    if (spamPlayer) NotifySupport("WB: Unknown player detected [" + Name + "] hiding..." );
    if (!flagFellowSup) Chat.Fellow("WB: Unknown player detected [" + Name + "] hiding...")
    skapi.Logout(); 	// Initiate logout sequence.

    DetectedPlayer = Name;
    StateChange(stateHide,NoCheck);
    Log.OutD("Main_OnObjCrePlayer: End, Detected unknown player [" + Name + "] Hiding...", true);
    return;
} // end func


function MainHandlers_OnObjDestroy(Aco) {
    // Chat.Screen("Object name [" + Aco.szName + "] destroyed");
    if (Aco.oid == Att.Oid) {
    	Log.OutD("Main_OnObjDestroy: current creature we are attacking was destroyed");
    	Att.Destroyed = true;
    } // endif
} // end func


function MainHandlers_OnOpenContainer(ConAco) {
    if (debugEvent||debugLoot) Log.OutD("Main_OnOpenContainer: Begin, Name [" + ConAco.szName +
    					"] Item Count [" + ConAco.citemContents + "]");

    ContainerOpen  = true;
    LootCount = ConAco.citemContents;

    // Now add each item in corpse to LootList
    for (var x=0; x<ConAco.citemContents; x++) {
    	// Due to server bug, sometimes an item is duplicated, so make sure item is not
    	// already on loot list when adding.
    	for (var y=0; y<x; y++) {
	    // If this item matches a previously added item then skip it
	    if (LootList[y].Oid == ConAco.AcoFromIitem(x).oid) {
	    	// Duplicate item detected, skipping
	    	if (debugLoot)
		    Log.OutD("Main_OnOpenContainer: Duplicate item detected [" + ConAco.AcoFromIitem(x).szName + "/" +
		   		Util.Hex(ConAco.AcoFromIitem(x).oid + "]"));
		break;
	    } // endif
	} // endfor
	if (y<x) continue;	// Duplicate item
   	LootList[x] = new ItemObj(ConAco.AcoFromIitem(x));
    	if (debugLoot) Log.OutD("Main_OnOpenContainer: Index [" + x + "] Item [" + LootList[x].Aco.szName + "]");
    } // endfor

    if (debugEvent||debugLoot) Log.OutD("Main_OnOpenContainer: End");
} // end function

// OnPortalStormed
function MainHandlers_OnPortalStormed() {
    Log.OutD("Main_OnPortalStormed: Begin");
    StateChange(stateStormed);
    Log.OutD("Main_OnPortalStormed: End");
}

// OnPortalStormWarning
function MainHandlers_OnPortalStormWarning(severity) {
    if (debugEvent) Log.OutD("Main_OnPortalStormWarning:  Lvl (" + severity + ") warning received");

    // If severity is not an end portal storm code then process warning
    if (severity != pswStormEnded) {
	// If we are not already processing the portal storm warning, then set portal storm
	// warning flag.
	if (State !=stateReset) {
	    CharStormWarn = true;
	    Log.OutD("Main_OnPortalStormWarning: Portal storm detected. Severity [" + severity + "]" );
	} // endif
    } // endif

    if (debugEvent) Log.OutD("Main_OnPortalStormWarning: End");
} // end function

function MainHandlers_OnRemoveFromInventory(aco, acoContainer, iitem) {
    if (debugEvent||debugInvent||debugSalvage)
	Log.OutD("Main_OnRemoveFromInventory: Begin, item [" + aco.szName + "/" + Util.Hex(aco.oid) + "]");

    InventoryRemove	= true;
    RemoveAco		= aco;
    RemoveContainer	= acoContainer;

    // then see if this item is on any of our loot lists.  If so remove it.
    for (var x=0; x<lootTypeTotal; x++) {
	with (Loot.TypeInfo[x]) {
	    for (var y=0; y<ItemCount; y++) {
		if (ItemList[y].Aco.oid == RemoveAco.oid) {
		    // Match found, so remove it.  Order of list not important
		    // so just copy last entry to first and reduce count
		    if (debugEvent||debugInvent||debugSalvage)
			Log.OutD("Main_OnRemoveFromInventory: Name [" + aco.szName + "/" + Util.Hex(aco.oid) +
			    	 "] remove from loot list [" + Name + "] position ["+ y + "]");
		    if (debugSalvage)
			Log.OutD("Salvage: Name [" + aco.szName + "/" + Util.Hex(aco.oid) +
			    	 "] removed from loot list [" + Name + "]");
		    ItemCount--;
		    TotalValue -= ItemList[y].Aco.cpyValue;
		    if (ItemCount > y) ItemList[y] = ItemList[ItemCount];
		    return;
		} // endif
	    } // endfor
	} // end with
    } // end for

    if (debugEvent||debugInvent) Log.OutD("Main_OnRemoveFromInventory: End");
} // end function


function MainHandlers_OnSpellExpire(SpellId, Layer) {
    if (debugEvent) Log.OutD("Main_OnSpellExpire: Begin");
    // Since this event is only registered if 'logExpire' is set, just
    // print out the spell expiration event
    Name =  skapi.SzFromSpellid(SpellId);
    Log.OutD("OnSpellExpire: Id [" + SpellId + "] Layer [" + Layer + "] Name [" + Name + "]");
    if (debugEvent) Log.OutD("Main_OnSpellExpire: End");
} // end function MainHandlers_OnSpellExpire


function MainHandlers_OnSpellExpireSilent(SpellId, Layer) {
    if (debugEvent) Log.OutD("Main_OnSpellExpireSilent: Begin");
    // Since this event is only registered if 'logExpire' is set, just
    // print out the spell expiration event
    Name =  skapi.SzFromSpellid(SpellId);
    Log.OutD("OnSpellExpireSilent: Id [" + SpellId + "] Layer [" + Layer + "] Name [" + Name + "]");
    if (debugEvent) Log.OutD("Main_OnSpellExpireSilent: End");
} // end function MainHandlers_OnSpellExpireSilent


function MainHandlers_OnStatLevel(NewLvl) {
    Log.OutDM("OnStatLevel", "Congrats you cheater! Your new level is [" + NewLvl + "]", true);
} // end function MainHandlers_OnStatUnspentExp

function MainHandlers_OnStatSkillExp(skid, szSkill, diff, expTotal, dsklvl) {
    if (debugEvent) Log.OutD("Main_OnStatSkillExp: Begin");
    // If this is the first time we detected an exp change for this skill, then
    // just record the exp level and return
    if ((SkillExp[skid] == undefined) || (SkillExp[skid] == null)) {
    	SkillExp[skid] = expTotal;
    	return;
    } // endif

    // Only report skill exp changes above a specified threshold.  Allows us to filter
    // out small changes due to spell casting, but still permit us to detect larger
    // changes that enable us to calculate a creatures magic defense or attack skill
    if ((expTotal-SkillExp[skid]) > expSkillThresh)

    if (flagDispSkillExp) {
	Chat.Screen("StatSkillExp: " + szSkill + "   New [" + expTotal +
		    "] Diff [" + (expTotal-SkillExp[skid]) + "] Diff/2 [" + (expTotal-SkillExp[skid])/2 + "]");
    } // endif

    if (logSkillExp) {
	Log.OutD("StatSkillExp: " + szSkill + "   New [" + expTotal + "] Diff [" +
		 (expTotal-SkillExp[skid]) + "] Diff/2 [" + (expTotal-SkillExp[skid])/2 + "]");
    } // endif

    SkillExp[skid] = expTotal;
    if (debugEvent) Log.OutD("Main_OnStatSkillExp: End");
} // endif


function MainHandlers_OnStatUnspentExp(NewExp) {
    if (debugEvent) Log.OutD("Main_OnStatUnspentExp: Begin");
    if ((NewExp-UnspentExp) > expThresh) {
	if (logExp) Log.OutD( "UnspentExp: New [" + NewExp + "] Diff [" + (NewExp-UnspentExp) + "]");
	if (flagDispExp) Chat.Screen("UnspentExp: New [" + NewExp + "] Diff [" + (NewExp-UnspentExp) + "]" );
    } // endif
    UnspentExp = NewExp;
    if (debugEvent) Log.OutD("Main_OnStatUnspentExp: End");
} // end function MainHandlers_OnStatUnspentExp


function MainHandlers_OnTargetHealth(aco,fractHealth) {
    if (debugEvent||debugAttack)
	Log.OutD("Main_OnTargetHealth: Target [" + aco.szName + "] oid [" + Util.Hex(aco.oid) +
		 "] Health Fraction [" + fractHealth + "]");

    if ((Att.Aco != undefined) && (Att.Aco !=null)) {
    	// Make sure the oid of this Aco matches creature we have selected. Sometimes stale message
    	// from previous creature returned
    	if (aco.oid == Att.Oid) Att.CreHealth = fractHealth;
    } else {
    	if (debugEvent3||debugAttack) Log.OutD("Main_OnTargetHealth: Att.Aco undefined or null");
    } // endif
} // end function MainHandlers_OnTargetHealth


function ProcessAdminTell(Name) {
    if (ChatStart == 0) ChatStart = new Date();
    if (State != stateAdmin) {
	// Set state to Admin and save start time
	Log.OutD("ProcessAdminTell: Admin @tell detected, setting state to Admin...", true);
	State = stateAdmin;
	AdminStart = new Date();
	AdminName  = Name;
	FellowAbort = false;
	skapi.CancelNav();
	skapi.WaitEvent(1000, wemFullTimeout);
    } // endif
    Chat.Fellow("Admin: @tell from admin detected");
    NotifySupport("Admin: @tell from admin detected");
} // end function ProcessAdminTell


function ProcessTell(szSource, acoSender, acoReceiver, msg) {
    if (debugEvent) Log.OutD("ProcessTell: Source [" + szSource + "] Who [" + acoSender.szName + "] Rcv [" +
    			      acoReceiver.szName + "] Msg [" + msg + "]");

    var Name = acoSender.szName;
    if (flagAdminTest) Name = "+" + Name;	// for admin alert testing only

    // See if this is an admin sending us a message by search for a plus sign in the
    // name of the character who sent us the message.  If so then set state to stateAdmin.
    if (adminAbort && (Name.indexOf("+") >= 0)) {
	ChatLog[ChatCnt] = Util.FormatDate(0,false) + " @t from admin [" + acoSender.szName + "] Msg: " + msg;
	ChatCnt++;
	Log.OutD("ProcessTell: Admin, Source [" + szSource + "] Who [" + acoSender.szName + "] Msg [" + msg + "]");
    	ProcessAdminTell(Name);
    	return;
    } // endif

   // See if this is a monster or NPC talking to us.  If so then abort processing of tell
    if ((acoSender.ocm != undefined) && (acoSender.ocm != null)) {
        if (acoSender.ocm & ocmMonster) {
	    if (debugCmds) Log.OutD("ProcessTell: Creature talking to us, ignoring...");
	    return;
	}
 	if (acoSender.ocm & ocmNPC) {
	    if (debugCmds) Log.OutD("ProcessTell: NPC talking to us, ignoring...");
	    return;
	} // endif
    } // endif

    // Someone is talking to us so record it in our chat log unless it is a response from a command
    // we just issued, crap talk from a NPC, or a creature talking smack
    var NavData = false;
    var CmdResp = false;

    if (msg.match(/Comp:/) || msg.match(/WB:/) || msg.match(/Stats Reset:/)) {
	if (debugCmds) Log.OutD("ProcessTell: Command response, ignoring...");
	CmdResp = true;
    } // endif

    if (msg.match(/NAVBEGIN/) || msg.match(/NAVINFO/)) {
	if (debugCmds) Log.OutD("ProcessTell: Nav data, now recording...");
	NavData = true;
    } // endif

    if (!NavData && !CmdResp) {
	ChatLog[ChatCnt] = Util.FormatDate(0,false) + " @t from " + acoSender.szName + "  Msg: " + msg;
	ChatCnt++;
	if (logChat) Log.OutD("ProcessTell: Source [" + szSource + "] Who [" + acoSender.szName + "] Msg [" + msg + "]");
    } // endif

    // If enhancedTell flag is set, then consider all @tells as from an admin
    // unless the character sending the tell is on the tell friends list.
    if (enhancedTell && (State != stateAdmin)) {
	if (debugEvent) Log.OutD("Main_OnTell: tellList length [" + tellList.length + "]");
        for (var x=0; x < tellList.length ; x++) {
    	    if (debugHide) Log.OutD("Main_OnTell: tellList entry [" + x + "] Name [" + tellList[x] + "]");
	    if (Name == tellList[x] ) {
	        Log.OutD("Main_OnObjCrePlayer: Player [" + Name + "] on tell list, admin alert aborted." );
	        break;
	    } // endif
        } // endfor
        
        if (x == tellList.length) {
	    // Didn't recognized person who @tell'd us, assume it's an admin
	    Log.OutD("Main_OnTell: @tell from unrecognized char [" + Name + "]", ToChat);
	    Log.OutD("Main_OnTell: Msg [" + msg + "]", ToChat);
	    Log.OutD("Setting admin state...", ToChat);
	    ProcessAdminTell(Name);
	    return;
	} // endif
    } // endif

    // If we don't have admin commands enabled, then just return.  Sound alert if flag enabled
    if (!enableCommands) {
    	 if (alertTell) alertTellWav.Play();
    	 return;
    } // endif

    // Parse command string.  First split line into parameters and trim spaces/tables from each param
    var Split = msg.split(" ");
    for (var x=0; x<Split.length; x++) {
    	Split[x] = Util.Trim(Split[x]);
    	if (debugCmds) Log.OutD("ProcessTell: Split # [" + x + "] Value [" + Split[x] + "]");
    } // endif
    var Command = Split[0].toUpperCase();
    if (debugCmds) Log.OutD("ProcessTell: Command [" + Command + "] Param1 [" + Split[1] + "]");

    // See person who @tell'd us is on authorized to issue this command
    if ((Command == "FELLOW") &&
        (anybodyFellow || (passwordFellow && (Split[1].toUpperCase() == fellowPassword.toUpperCase())))) {
    	if (debugCmds) Log.OutD("ProcessTell: Authorized source for fellow command");
    } else {
    	// See of sender on 'authList'
	if (Authorized(acoSender.szName)) {
	    if (debugCmds) Log.OutD("ProcessTell: Authorized source");
	} else {
	    // Not authorized
	    if (flagAutoRespond && !MacroPaused) {
	        if (debugCmds) Log.OutD("ProcessTell: Unknown sender, replying with stock response...");
		Chat.Tell(acoSender.szName, tellResponse);
	    } else {
	        if (debugCmds) Log.OutD("ProcessTell: Unknown sender, ignoring @tell");
	    } // endif
	    if (alertTell) alertTellWav.Play();
	    return;
	} // endif
    } // endif

    // List attack locs if requested
    if (Command == "ATTLOCS") {
    	var OutLine = "";
	for (var x=1; x<=navMaxLoc; x++) {
	    if (AttLoc[x].KillLoc == null) break;
	    OutLine = OutLine + Util.FormatCoords(AttLoc[x].KillLoc,2) + "   ";
	} // endif
	Chat.Tell(acoSender.szName, "WB: " + OutLine);
	return;
    } // endif

    // Process any fellow requests
    if (Command == "FELLOW") {
    	// First make sure we are member of a fellow
	var cofellow = skapi.cofellow;
	if (cofellow.Count == 0) {
	    Chat.Tell(acoSender.szName, "WB: Can't fellow you. I'm not a member of a fellow.");
	    return;
	} // endif

	// Ok we are in a fellow so set flag to fellow next FSM loop and notify requester
	Log.OutD("Fellow request detected, requestor [" + Name + "]");
	FellowFlag = true;
	FellowChar = acoSender.szName;
	Chat.Tell(FellowChar, "WB: Request acknowledged, I will attempt to add you shortly. " +
			      "Make sure you are accepting fellow requests ");
	
	var Leader = cofellow.Item(0).szName;
    	if (Leader != skapi.szName) {
	    Chat.Tell(acoSender.szName, "WB: The leader of the fellowship is " + Leader +
	    				" but I'm going to assume the fellow is OPEN.");
	} // endif
	return;
    } // endif

    // LEADER command will report who is leader of fellow
    if (Command == "LEADER") {
    	Log.OutD("ProcessTell: LEADER command detected");
    	var cofellow = skapi.cofellow;
    	Chat.Tell(acoSender.szName,"WB: Leader of fellowship is [" + cofellow.Item(0).szName + "]");
    	return;
    } // endif

    // MEMBERS command will report all the members of the fellow
    if (Command == "MEMBERS") {
    	Log.OutD("ProcessTell: MEMBERS command detected");
    	var cofellow = skapi.cofellow;
    	var members = ""
	for ( var i = 0; i < cofellow.Count; i++) {
	    var fellow = cofellow.Item(i);
	    members = members + fellow.szName + "  ";
	} // endfor
	Chat.Tell(acoSender.szName,"WB: Fellow members: " + members );
   	return;
    } // endif

    // LOC command will report our location
   if (Command == "LOC") {
   	var OurLoc = skapi.maplocCur;
    	Log.OutD("ProcessTell: LOC command detected");
    	Chat.Tell(acoSender.szName,"WB: My location is [" + Util.FormatCoords(OurLoc,2) + "]");
   	return;
    } // endif

    // TIME command will report the time on our computer
    if (Command == "TIME") {
    	Log.OutD("ProcessTell: TIME command detected");
    	Chat.Tell(acoSender.szName, "WB: My computer's time is: " + new Date());
   	return;
    } // endif

    // COMP command reports levels of comps we are monitoring
    if (Command == "COMPS") {
	ReportComps(acoSender.szName);
   	return;
    } // endif

    // STATS command reports kill statistics/efficiency
    if (Command == "STATS") {
	LoadStats();
	Chat.Tell(acoSender.szName, "WB: Stats Reset: " + new Date(StartTime) + " Kills/Hr: " +
		  StatAveKillHour + "  Kill Exp/Hr: " +  StatEstExpHour + "  Act Exp/Hr: " + StatActExpHour );
   	return;
    } // endif

    // Version command reports current macro version
    if (Command == "IDENT") {
    	Chat.Tell(acoSender.szName, "WB: Running WarBot " + macroVersion);
   	return;
    } // endif

    // ACCEPT command causes us to press Y key to accept fellow requests
    if (Command == "ACCEPT") {
    	skapi.Keys("Y");
   	return;
    } // endif

   if (Command == "TOGGLE") {
   	var cofellow = skapi.cofellow;
	var Leader = cofellow.Item(0).szName;
    	if (Leader != skapi.szName) {
	    Chat.Tell(acoSender.szName, "WB: Can't toggle fellow OPEN/CLOSE status, leader of fellow is " + Leader);
	    return;
	} // endif

	Chat.Tell(acoSender.szName,"WB: TOGGLE command detected, toggling fellow OPEN/CLOSE status.");
	Log.OutD("ProcessTell: TOGGLE command detected, toggling fellow OPEN/CLOSE status.");
	Player.ToggleFellow();
   	return;
    } // endif

   // DISBAND command will cause us to disband our fellow IF we are the only one in it
   if ((Command == "DISBAND") && flagAllowDisband) {
   	var cofellow = skapi.cofellow;
	var Leader = cofellow.Item(0).szName;
    	if (Leader != skapi.szName) {
	    Chat.Tell(acoSender.szName, "WB: Can't disband, not leader of fellowship.  Leader is " + Leader);
	    return;
	} // endif

	Chat.Tell(acoSender.szName,"WB: DISBAND command detected, dissolving fellow shortly.");
	if (cofellow.Count > 1) {
	    var members = ""
	    for ( var i = 0; i < cofellow.Count; i++) {
		var fellow = cofellow.Item(i);
		members = members + fellow.szName + "  ";
	    } // endfor
	    Chat.Tell(acoSender.szName,"WB: Current fellow members: " + members );
	} // endif

	Log.OutD("ProcessTell: DISBAND command detected, dissolving fellow shortly...");
	CharDisbandFlag = true;
	CharDisbandReq  = acoSender.szName;
   	return;
    } // endif

    // SETEXC command will enable exclusion mode
    if (Command == "SETEXC") {
    	ExclMode = true;
	skapi.SetControlProperty(szPanelControl, "btnExclusion", "Text", "Disable ExclMode");
   	return;
    } // endif

    // CLEAREXC command will disable exclusion mode
    if (Command == "CLEAREXC") {
    	ExclMode = false;
	skapi.SetControlProperty(szPanelControl, "btnExclusion", "Text", "Enable ExclMode");
   	return;
    } // endif

    if (Command == "JOINGROUP") {
    	if (MoveGroup == 0) {
    	    Chat.Screen("We are not member of a movegroup, ignoring JOINGROUP request");
    	    Chat.Tell(acoSender.szName, "WB: I don't have a MoveGroup defined!");
    	    return;
    	} // endif

    	if (!ExclMode) {
    	    Chat.Screen("Enabling exclusion mode...");
    	    ExclMode = true;
	    skapi.SetControlProperty(szPanelControl, "btnExclusion", "Text", "Disable ExclMode");
	} // endif
    	SendNavInfo(acoSender.szName);
    	return;
    } // endif

    // See if someone is sending us navinfo
    if (Command == "NAVBEGIN") {
    	NavLoadActive = true;
    	Data = msg.substr(9);
    	Params = Data.split(",");
   	New_AttLocCnt = parseInt(Util.Trim(Params[0]));
    	New_AttLocCur = parseInt(Util.Trim(Params[1]));
   	New_MoveGroup = parseInt(Util.Trim(Params[2]));
   	New_Retreat1  = parseInt(Util.Trim(Params[3]));
   	New_Retreat2  = parseInt(Util.Trim(Params[4]));
   	New_Retreat3  = parseInt(Util.Trim(Params[5]));
   	// Chat.Screen("New_AttLocCnt : " + New_AttLocCnt);
   	// Chat.Screen("New_AttLocCur : " + New_AttLocCur);
   	// Chat.Screen("New_MoveGroup : " + New_MoveGroup);

   	// If masterway point defined, convert it to an internal maploc
   	if (Params.length == 7) {
   	    // Chat.Screen("New_MasterWay : " + Util.Trim(Params[3]));
	    New_MasterWayLoc = skapi.MaplocFromSz(Util.Trim(Params[3]));
	} else {
	    // Chat.Screen("New_MasterWay : null");
	    New_MasterWayLoc = null;
	} // endif

	// Initialize holding area for attack locations
	for (var i=1; i<=New_AttLocCnt; i++) New_AttLoc[i] = new AttLocRec();
	return;
    } // endif

    if (Command == "NAVINFO") {
    	if (!NavLoadActive) {
	    Chat.Screen("%NavLoad not active, ignoring navinfo @tell");
	    return;
	} // endif

    	Data = msg.substr(8);
    	Params = Data.split(",");
    	NumAttLocs = Params.length/5;
    	for (var i=0; i<NumAttLocs; i++) {
	    AttLocPos	   = parseInt(Params[(i*5)+0]);
	    New_KillLoc    = Params[(i*5)+1];
	    New_WayLoc     = Params[(i*5)+2];
	    New_RetreatLoc = Params[(i*5)+3];
	    New_LocFlags   = parseInt(Params[(i*5)+4]);
	    // Chat.Screen("AttLocPos      : " + AttLocPos      );
	    // Chat.Screen("New_KillLoc    : " + New_KillLoc    );
	    // Chat.Screen("New_WayLoc     : " + New_WayLoc     );
	    // Chat.Screen("New_RetreatLoc : " + New_RetreatLoc );
	    // Chat.Screen("New_LocFlags   : " + New_LocFlags   );
	    New_AttLoc[AttLocPos].KillLoc = skapi.MaplocFromSz(New_KillLoc);
	    if ((New_WayLoc != null) && (New_WayLoc != "")) {
		// Chat.Screen("Converting wayloc...");
		New_AttLoc[AttLocPos].WayLoc = skapi.MaplocFromSz(New_WayLoc);
	    } else {
	    	// Chat.Screen("Wayloc undefined");
		New_AttLoc[AttLocPos].WayLoc = null;
	    } // endif

	    if ((New_RetreatLoc != null) && (New_RetreatLoc != "")) {
	    	// Chat.Screen("Converting retreatloc...");
		New_AttLoc[AttLocPos].RetreatLoc = skapi.MaplocFromSz(New_RetreatLoc);
	    } else {
	    	// Chat.Screen("Retreatloc undefined");
		New_AttLoc[AttLocPos].RetreatLoc = null;
	    } // endif
	    
	    New_AttLoc[AttLocPos].LocFlags = New_LocFlags;
	} // endfor

	// Now see if all attlocs received, if any null then return from this function
	for (var i=1; i<=New_AttLocCnt; i++)
	    if (New_AttLoc[i].KillLoc == null) return;	// Not all attlocs received

	// All attlocs set, load them as the active navigation parameters
	Chat.Screen("Loading received NavInfo...");
	AttLocCnt	    = New_AttLocCnt;
	AttLocCur	    = New_AttLocCur;
	MoveGroup	    = New_MoveGroup;
	retreatAmtOne	    = New_Retreat1;
   	retreatAmtTwo	    = New_Retreat2;
   	retreatAmtThreePlus = New_Retreat3;
   	ClearMoveStats();
   	LastArriveTime = new Date();

   	// Update screen with new movement paramters
        skapi.SetControlProperty(szPanelConfig,   "retreatCnt1", "selected", retreatAmtOne-1       );
        skapi.SetControlProperty(szPanelConfig,   "retreatCnt2", "selected", retreatAmtTwo-1       );
        skapi.SetControlProperty(szPanelConfig,   "retreatCnt3", "selected", retreatAmtThreePlus-1 );
        skapi.SetControlProperty(szPanelConfig,   "moveGroup",	 "selected", MoveGroup             );
        skapi.SetControlProperty(szPanelMovement, "curAttLoc",   "selected", AttLocCur-1           );

	MasterWayLoc = New_MasterWayLoc;
	for (var i=1; i<=New_AttLocCnt; i++) AttLoc[i] = New_AttLoc[i];
	HomeLoc	     = AttLoc[AttLocCur].KillLoc;

	// We just joined a movegroup, enable exclusion mode and automove if necessary
	if (!ExclMode) {
    	    Chat.Screen("Enabling Exclusion mode...");
    	    ExclMode = true;
	    skapi.SetControlProperty(szPanelControl, "btnExclusion", "Text", "Disable ExclMode");
	} // endif
	if (!AutoMove) {
	    Chat.Screen("Enabling AutoMove...");
	    AutoMove = true;
	    skapi.SetControlProperty(szPanelControl, "btnAutoMove", "Text", "Disable AutoMove");
	} // endif
	Chat.Screen("New NavInfo loaded, joined MoveGroup " + MoveGroup + ".");
	return;
    } // endif

    // the @tell is not a command so assume someone is trying to talk to us. Sound alert if flag set
    if (alertTell) alertTellWav.Play();

    if (debugEvent) Log.OutD("ProcessTell: End");
} // end func ProcessTell

// OnTell
function MainHandlers_OnTell(acoSender, acoReceiver, msg) {
    Log.OutD("Main_OnTell: Begin, sender [" + acoSender.szName + "] msg [" + msg + "]");
    ProcessTell("TELL", acoSender, acoReceiver, msg);
    Log.OutD("Main_OnTell: End");
} // end func

// OnTellMisc
function MainHandlers_OnTellMisc(acoSender, acoReceiver, msg, cmc) {
    Log.OutD("Main_OnTellMisc: Begin, sender [" + acoSender.szName + "] msg [" + msg + "]");
    ProcessTell("MISC", acoSender, acoReceiver, msg);
    Log.OutD("Main_OnTellMisc: End");
} // end func

// OnTellServer
function MainHandlers_OnTellServer(acoSender, acoReceiver, msg) {
    Log.OutD("Main_OnTellServer: Begin, sender [" + acoSender.szName + "] msg [" + msg + "]");
    ProcessTell("SERVER", acoSender, acoReceiver, msg);
    Log.OutD("Main_OnTellServer: End");
} // end func

function MainHandlers_OnTellCovassal(szSender, szMsg) {
    if (debugEvent) Log.OutD("Main_OnTellCovassal: Begin");
    ChatLog[ChatCnt] = Util.FormatDate(0,false) + " @c from " + szSender + ",  Msg: " + szMsg;
    ChatCnt++;
    if (alertAlleg) alertAllegWav.Play();
    if (debugEvent) Log.OutD("Main_OnTellCovassal: End");
} // end func

function MainHandlers_OnTellFellowship( szSender, szMsg) {
    if ((szSender == null) || (szSender == "")) szSender = skapi.szName;
    if (debugEvent) Log.OutD("OnTellFellowship: Begin, Who [" + szSender + "] Msg [" + szMsg + "]" );
    if (logChat) Log.OutD("OnTellFellowship: Who [" + szSender + "] Msg [" + szMsg + "]");

    var Split = szMsg.split(" ");
    for (var x=0; x<Split.length; x++) {
    	Split[x] = Util.Trim(Split[x]);
    	if (debugCmds) Log.OutD("Main_OnTellFellowship: Split # [" + x + "] Value [" + Split[x] + "]");
    } // endif
    var Command = Split[0].toUpperCase();
    if (debugCmds) Log.OutD("Main_OnTellFellow: Command [" + Command + "]");

    // If we were the sender of the message, then ignore it, unless it is the STATS or RESET command
    if ((szSender == skapi.szName) && (Command != "STATS") && (Command != "RESET")) {
	if (debugCmds) Log.OutD("Main_OnTellFellow: Ignoring commands, it is from us");
    	return;
    } // endif

    // If we want to abort when someone else in our fellow detects an admin, then
    // write message to log and abort
    if (fellowAdminAbort && (State != stateAdmin) && (szMsg.indexOf("Admin:") >= 0)) {
	Log.OutD("Main_OnTellFellowShip: Admin detected by fellow member, logging after holddown...", true );
	if (alertAdmin) alertAdminWav.Play();
	FellowAbort = true;
	FellowWarnTime = new Date();
	return;
    } // endif

    // If this is a message broadcast automatically from someone in fellow running this
    // macro, then see if it is anything we need to look at
    if (Command == "WB:") {
	if (flagFellowStealth && (szMsg.match(/WB: Unknown player detected/))) {
    	    // Unknown player detected by member of fellow.  Log if flagFellowStealth set.
    	    var Pattern1 = new RegExp("(.*[\[]|[\]].*$)", "g");		// Extract the player name
    	    DetectedPlayer = szMsg.replace(Pattern1, "");
 	    Log.OutD("Main_OnTellFellow: End, Unknown player [" + DetectedPlayer +
	    	     "] detected by fellow member [" + szSender + "]. Hiding...", true);
    	    skapi.Logout(); 	// Initiate logout sequence.
	    StateChange(stateHide,NoCheck);
	    return;
	} // endif
    	if (debugCmds) Log.OutD("Main_OnTellFellow: End. WB broadcast, no processing necessary.");
    	return;
    } // endif

    // Process any fellow requests
    if (Command == "ATTLOCS") {
    	var OutLine = "";
	for (var x=1; x<=navMaxLoc; x++) {
	    if (AttLoc[x].KillLoc == null) break;
	    OutLine = OutLine + Util.FormatCoords(AttLoc[x].KillLoc,2) + "   ";
	} // endif
	Chat.Fellow("WB: " + OutLine);
	return;
    } // endif

    if ((Command == "MOVE") && (MoveGroup > 0) && (State != stateGotoNext) && !Dead) {
 	var Group  = parseInt(Split[1]);
	var NewLoc = parseInt(Split[2]);
	if ((Group  == undefined) || (Group  == null) || (Group  == NaN)) Group  = 0;
	if ((NewLoc == undefined) || (NewLoc == null) || (NewLoc == NaN)) NewLoc = 0;

	if ((Group<1) || (Group>navMaxGroups)) {
	    if (debugMove) Log.OutD("OnTellFellow: MOVE: Invalid group code detected [" + Split[1] + "]", true);
	    return;
	} // endif

	// If the movegroup this message is intended for is not ours, then ignore it
	if (Group != MoveGroup) {
	    if (debugMove) Log.OutD("OnTellFellow: Move group not ours, ignoring.");
	    return;
	} // endif

	if ((NewLoc<1) || (NewLoc>AttLocCnt)) {
	    if (debugMove) Log.OutD("OnTellFellow: MOVE: Invalid attloc detected [" + Split[2] + "]", true);
	    return;
	} // endif

	if (NewLoc == AttLocCur) {
	    if (debugMove) Log.OutD("OnTellFellow: MOVE, Already at attack loc # " + NewLoc);
	    return;
	} // endif

	// If we make it here then this move message is valid.  Save the new location even
	// if automove is disabled by low comps, so that we know where to go when the comp
	// situation is resolved (for example by someone giving us comps).
	MoveOverRide = NewLoc;
	if (!AutoMove) {
	    if (debugMove) Log.OutD("OnTellFellow: MOVE, Automove disabled.");
	    return;
	} // endif

	if (debugMove) Log.OutD("OnTellFellow: Moving to attack location # " + NewLoc);
	StateChange(stateGotoNext,NoCheck);
    } // endif

    if ((Command == "RETREAT") && (MoveGroup > 0) && (State != stateGotoNext) && !Dead) {
 	var Group  = parseInt(Split[1]);
	if ((Group  == undefined) || (Group  == null) || (Group  == NaN)) Group  = 0;

	if ((Group<1) || (Group>navMaxGroups)) {
	    if (debugMove) Log.OutD("OnTellFellow: RETREAT, Invalid group code detected [" + Split[1] + "]", true);
	    return;
	} // endif

	// If the movegroup this message is intended for is not ours, then ignore it
	if (Group != MoveGroup) {
	    if (debugMove) Log.OutD("OnTellFellow: RETREAT, Move group not ours, ignoring.");
	    return;
	} // endif

	if (RetreatActive) {
	    if (debugMove) Log.OutD("OnTellFellow: Already at or moving to retreat location.");
	    return;
	} // endif

	// If we make it here then the retreat message is valid.  Set flag to do the retreat
	// the next FSM cycle
	RetreatReq = true;

	if (debugMove) Log.OutD("OnTellFellow: Flag set to initiate move to retreat location.");
    } // endif

    if ((Command == "ADVANCE") && (MoveGroup > 0) && (State != stateGotoNext) && !Dead) {
 	var Group  = parseInt(Split[1]);
	if ((Group  == undefined) || (Group  == null) || (Group  == NaN)) Group  = 0;

	if ((Group<1) || (Group>navMaxGroups)) {
	    if (debugMove) Log.OutD("OnTellFellow: ADVANCE, Invalid group code detected [" + Split[1] + "]", true);
	    return;
	} // endif

	// If the movegroup this message is intended for is not ours, then ignore it
	if (Group != MoveGroup) {
	    if (debugMove) Log.OutD("OnTellFellow: ADVANCE, Move group not ours, ignoring.");
	    return;
	} // endif

	if (!RetreatActive) {
	    if (debugMove) Log.OutD("OnTellFellow: Already at or moving to kill location.");
	    return;
	} // endif

	// If we make it here then the advance message is valid.  Set flag to do the advance the next FSM cycle
	AdvanceReq = true;

	if (debugMove) Log.OutD("OnTellFellow: Flag set to initiate move to kill location.");
    } // endif

    if ((Command == "LOCK") && ExclMode) {
	var Oid = parseInt(Split[1]);
	var Prio = parseInt(Split[2]);
	if ((Oid  == undefined) || (Oid  == null) || (Oid  == NaN)) Oid  = 0;
	if ((Prio == undefined) || (Prio == null) || (Prio == NaN)) Prio = 0;

	if (debugExcl) Log.OutD("OnTellFellow: Lock Oid: " + Oid + "/" + Util.Hex(Oid) + "  Prio: " + Prio);

	// Now see if someone is attempting to lock the creature we are attacking.  This will
	// happen when two people try to attack the same creature at the same time.
	if (((State == stateTarget) && (SelId == Oid)) ||
	    ((State == stateKill)   && (Att.Oid == Oid))) {
	    if (AttemptPriority < Prio) {
		if (logExcl||debugExcl) Log.OutD("OnTellFellow: Kill attempt collision. We have lower prio, aborting...");
		if (State == stateTarget) {
		    SelAbort = true;
		    StateChange(stateSelect,NoCheck);
		} else {
		    // State is Kill, execute cleanup routine
		    KillDone();
		} // endif
	    } else {
		if (logExcl||debugExcl) Log.OutD("OnTellFellow: Kill collision.  We have higher prio, continuing...");
		return;	 // No need to lock creature since we already have the lock
	    } // endif
	} // endif

	ExclAddLock(Oid);

	// If debug information wanted, try to translate the Aco
	if (debugExcl) {
	    var Aco = skapi.AcoFromOid(Oid);
	    if ((Aco == undefined) || (Aco == null)) {
		Log.OutD("OnTellFellow: Unable to translate Lock OID");
	    } else {
		Log.OutD("OnTellFellow: Lock Oid Translation,  Name: " + Aco.szName);
	    } // endif
	} // endif
    } // endif

    if ((Command == "UNLOCK") && ExclMode) {
	var Oid      = parseInt(Split[1]);
	var YieldExp = parseInt(Split[2]);
	var VulnExp  = parseInt(Split[3]);
	var VulnLvl  = parseInt(Split[4]);
	if ((YieldExp == undefined) || (YieldExp == null) || (YieldExp == NaN)) YieldExp = 0;
	if ((VulnExp  == undefined) || (VulnExp  == null) || (VulnExp  == NaN)) VulnExp = 0;
	if ((VulnLvl  == undefined) || (VulnLvl  == null) || (VulnLvl  == NaN)) {
	    if (VulnExp == 0)
		VulnLvl = 0;
	    else
		VulnLvl = 6;
	} // endif

	if (debugExcl)
	    Log.OutD("OnTellFellow: UnLock Oid: " + Oid + "/" + Util.Hex(Oid) + "  " +
	    	     "Yield Offset: " + YieldExp + "  Vuln Offset: " + VulnExp);

	// If debug information wanted, try to translate the Aco
	if (debugExcl) {
	    var Aco = skapi.AcoFromOid(Oid);
	    if ((Aco == undefined) || (Aco == null)) {
		Log.OutD("OnTellFellow: Unable to translate Unlock OID");
	    } else {
		Log.OutD("OnTellFellow: Unlock Oid Translation,  Name: " + Aco.szName);
	    } // endif
	} // endif

	ExclDelLock(Oid, YieldExp, VulnExp, VulnLvl);
    } // endif

    if (Command == "CHECKLOC") {
	// Extract coords from command string and convert to internal maploc format
	var CoordsPre = Split[1] + " ";
	for (var x=2; x<Split.length; x++)
	    if (Split[x] != " ") {
	    	CoordsPre = CoordsPre + Split[x];
	    	break;
	    } // endif

	var Pattern1 = new RegExp(/^.*\[/);
	var Pattern2 = new RegExp(/\].*$/);
	var CoordsPost = CoordsPre.replace(Pattern1, "");
	var CoordsPost = CoordsPost.replace(Pattern2, "");
	if (debugCmds) Log.OutD("Main_OnTellFellow: Checkloc, CoordsPre [" + CoordsPre + "]  Post [" + CoordsPost + "]");

	var RequestorMapLoc = skapi.MaplocFromSz(CoordsPost);
	if (RequestorMapLoc == null) {
	    if (debugCmds) Log.OutD("Main_OnTellFellow: RequestorMapLoc is null.");
	    return;
	} // endif

	// Now compare the requestor's location with our attack locations
	var OutLine = "";
	var WarningFlag = false;
	for (var x=1; x<=navMaxLoc; x++) {
	    if (AttLoc[x].KillLoc == null) break;
	    var RequestorDist = Util.Distance(AttLoc[x].KillLoc,RequestorMapLoc);
	    if (RequestorDist < ((SelRange+0.01)*2)) {
		WarningFlag = true;
		Chat.Fellow("WB: AttLoc # " + x + "  " + Util.FormatCoords(AttLoc[x].KillLoc,2) + " (" +
	    			Util.RoundFill(RequestorDist,2) + ") OVERLAP");
	    } // endif
	} // endfor

	if (!WarningFlag) Chat.Fellow("WB: No overlap!");
	return;
    } // endif

    // LOC command will report our location
    if (Command == "LOC") {
   	var OurLoc = skapi.maplocCur;
    	Log.OutD("Fellow LOC command detected");
    	var OutLine = "WB: Location: " + Util.FormatCoords(OurLoc,2) +
    		      "   MoveGroup: " + MoveGroup +
    		      "   Nearby comacroers: " + PlayerNearCnt;
    	Chat.Fellow(OutLine);
   	return;
    } // endif

    // TIME command will report the time on our computer
    if (Command == "TIME") {
    	Log.OutD("Fellow TIME command detected");
    	Chat.Fellow("WB: My computer's time is " + new Date() );
   	return;
    } // endif

    // STATS command reports kill statistics/efficiency
    if ((Command == "STATS") && (Split.length == 1)) {
	LoadStats();
	Chat.Fellow("WB: Reset: " + new Date(StartTime) + " Kills/Hr: " + StatAveKillHour +
		    "  Kill Exp/Hr: " +  StatEstExpHour + "  Actual Exp/Hr: " + StatActExpHour );
   	return;
    } // endif
    
    // RESET commands resets stats.  First dump out current stats to log file
    if (Command == "RESET") {
    	Log.OutD("Stats reset by " + szSender, true );
    	LoadStats();
	Log.OutD("ElapsedTimeMin: " + ElapsedTimeMin);
	Log.OutD("Stats Reset: " + new Date(StartTime) + "   Kills: " + TotalKilled +
		 "  Kill Exp: " + KillExp + "  Actual Exp: " + StatActExp);
	Log.OutD("Kills/Hour: " + StatAveKillHour + "  Kill Exp/Hour: " +  StatEstExpHour +
		 "  Actual Exp/Hour: " + StatActExpHour );
    	ResetStats();
    } // endif

    // Version command reports current macro version
    if (Command == "IDENT") {
    	Chat.Fellow("WB: Running WarBot " + macroVersion);
   	return;
    } // endif

    if (debugEvent) Log.OutD("Main_OnTellFellowShip: End" );
} // end func


function MainHandlers_OnTellFollower(szSender, szMsg) {
    if (debugEvent) Log.OutD("Main_OnTellFollow: Begin");
    ChatLog[ChatCnt] = Util.FormatDate(0,false) + " @m from " + szSender + ",  Msg: " + szMsg;
    ChatCnt++;
    if (alertAlleg) alertAllegWav.Play();
    if (debugEvent) Log.OutD("Main_OnTellFollow: End");
} // end func


function MainHandlers_OnTellPatron(szSender, szMsg) {
    if (debugEvent) Log.OutD("Main_OnTellPatron: Begin");
    ChatLog[ChatCnt] = Util.FormatDate(0,false) + " @p from " + szSender + ",  Msg: " + szMsg;
    ChatCnt++;
    if (alertAlleg) alertAllegWav.Play();
    if (debugEvent) Log.OutD("Main_OnTellPatron: End");
} // end func


function MainHandlers_OnTellVassal(szSender, szMsg) {
    if (debugEvent) Log.OutD("Main_OnTellVassal: Begin");
    ChatLog[ChatCnt] = Util.FormatDate(0,false) + " @v from " + szSender + ",  Msg: " + szMsg;
    ChatCnt++;
    if (alertAlleg) alertAllegWav.Play();
    if (debugEvent) Log.OutD("Main_OnTellVassal: End");
} // end func


function MainHandlers_OnTradeAdd(aco, side) {
    // If the item being added is by our partner (side=2) rather then us (side=1), then add item to
    // aco collection of possible trade items
    if (side == 2) ItemCoaco.AddAco(aco);
} // end func


function MainHandlers_OnTradeEnd(arc) {
    TradeOpen = false;
} // end func


function MainHandlers_OnTradeReset(aco) {
     ItemCoaco.Clear();
} // end func


function MainHandlers_OnTradeStart(acoSender,acoReceiver) {
    TradeOpen  = true;
    ItemCoaco.Clear();
} // end func


// OnVitals
function MainHandlers_OnVitals( health, stam, mana) {
    if (debugEvent) Log.OutD("Main_OnVitals: Health [" + health + "] Stam [" + stam + "] Mana [" + mana + "]" );
} // end func


function MainHandlers() {
	this.OnActionComplete	  = MainHandlers_OnActionComplete;
	this.OnAddToInventory	  = MainHandlers_OnAddToInventory;
	this.OnAdjustStack	  = MainHandlers_OnAdjustStack;
	this.OnApproachVendor	  = MainHandlers_OnApproachVendor;
	this.OnAssessCreature	  = MainHandlers_OnAssessCre;
	this.OnAssessItem 	  = MainHandlers_OnAssessItem;
	this.OnChatBroadcast	  = MainHandlers_OnChatBroadcast;
	this.OnChatBoxMessage	  = MainHandlers_OnChatBoxMessage;
	this.OnChatLocal	  = MainHandlers_OnChatLocal;
	this.OnChatServer	  = MainHandlers_OnChatServer;
	this.OnCommand		  = MainHandlers_OnCommand;
	this.OnControlEvent	  = MainHandlers_OnControlEvent;
	this.OnDeathMessage	  = MainHandlers_OnDeathMessage;
	this.OnDeathSelf	  = MainHandlers_OnDeathSelf;
	this.OnDisconnect	  = MainHandlers_OnDisconnect;
	this.OnEnd3D		  = MainHandlers_OnEnd3D;
	this.OnEndPortalSelf 	  = MainHandlers_OnEndPortalSelf;
	this.OnFellowInvite	  = MainHandlers_OnFellowInvite;
	this.OnFellowRecruit	  = MainHandlers_OnFellowRecruit;
	this.OnFellowUpdate	  = MainHandlers_OnFellowUpdate;
	this.OnLeaveVendor	  = MainHandlers_OnLeaveVendor;
	this.OnMeleeDamageSelf	  = MainHandlers_OnMeleeDamageSelf;
	this.OnMeleeEvadeSelf	  = MainHandlers_OnMeleeEvadeSelf;
	this.OnMeleeLastAttacker  = MainHandlers_OnMeleeLastAttacker;
	this.OnNavStop		  = MainHandlers_OnNavStop;
	this.OnObjectCreate	  = MainHandlers_OnObjCre;
	this.OnObjectCreatePlayer = MainHandlers_OnObjCrePlayer;
	this.OnObjectDestroy	  = MainHandlers_OnObjDestroy;
	this.OnOpenContainer	  = MainHandlers_OnOpenContainer;
	this.OnPortalStormed	  = MainHandlers_OnPortalStormed;
	this.OnPortalStormWarning = MainHandlers_OnPortalStormWarning;
	this.OnRemoveFromInventory= MainHandlers_OnRemoveFromInventory;
	this.OnSpellExpire	  = MainHandlers_OnSpellExpire;
	this.OnSpellExpireSilent  = MainHandlers_OnSpellExpireSilent;
	this.OnStatLevel	  = MainHandlers_OnStatLevel;
	this.OnStatSkillExp	  = MainHandlers_OnStatSkillExp;
	this.OnStatUnspentExp	  = MainHandlers_OnStatUnspentExp;
	this.OnTargetHealth	  = MainHandlers_OnTargetHealth;
	this.OnTell		  = MainHandlers_OnTell;
	this.OnTellCovassal	  = MainHandlers_OnTellCovassal;
	this.OnTellFollower	  = MainHandlers_OnTellFollower;
	this.OnTellFellowship	  = MainHandlers_OnTellFellowship;
	this.OnTellMisc		  = MainHandlers_OnTellMisc;
	this.OnTellPatron	  = MainHandlers_OnTellPatron;
	this.OnTellServer	  = MainHandlers_OnTellServer;
	this.OnTellVassal	  = MainHandlers_OnTellVassal;
	this.OnTradeAdd		  = MainHandlers_OnTradeAdd;
	this.OnTradeEnd		  = MainHandlers_OnTradeEnd;
	this.OnTradeReset	  = MainHandlers_OnTradeReset;
	this.OnTradeStart	  = MainHandlers_OnTradeStart;
	this.OnVitals		  = MainHandlers_OnVitals;
} // end func

function fAddHandlers() {
    // Active handlers
    MainH = new MainHandlers();
    skapi.AddHandler( evidOnActionComplete,	MainH );
    skapi.AddHandler( evidOnAddToInventory,	MainH );
    skapi.AddHandler( evidOnAdjustStack,	MainH );
    skapi.AddHandler( evidOnApproachVendor,	MainH );
    skapi.AddHandler( evidOnAssessCreature,	MainH );
    skapi.AddHandler( evidOnAssessItem,		MainH );
    skapi.AddHandler( evidOnChatBroadcast,	MainH );
    skapi.AddHandler( evidOnChatLocal,		MainH );
    skapi.AddHandler( evidOnChatServer,		MainH );
    skapi.AddHandler( evidOnCommand,		MainH );
    skapi.AddHandler( evidOnControlEvent,	MainH );
    skapi.AddHandler( evidOnDeathMessage,	MainH );
    skapi.AddHandler( evidOnDeathSelf,		MainH );
    skapi.AddHandler( evidOnDisconnect,		MainH );
    skapi.AddHandler( evidOnEnd3D,		MainH );
    skapi.AddHandler( evidOnEndPortalSelf,	MainH );
    skapi.AddHandler( evidOnFellowInvite,	MainH );
    skapi.AddHandler( evidOnFellowRecruit,	MainH );
    skapi.AddHandler( evidOnFellowUpdate,	MainH );
    skapi.AddHandler( evidOnLeaveVendor,	MainH );
    skapi.AddHandler( evidOnNavStop,		MainH );
    skapi.AddHandler( evidOnObjectCreate,	MainH );
    skapi.AddHandler( evidOnObjectCreatePlayer,	MainH );
    skapi.AddHandler( evidOnObjectDestroy,	MainH );
    skapi.AddHandler( evidOnOpenContainer,	MainH );
    skapi.AddHandler( evidOnPortalStormed,	MainH );
    skapi.AddHandler( evidOnPortalStormWarning,	MainH );
    skapi.AddHandler( evidOnRemoveFromInventory,MainH );
    skapi.AddHandler( evidOnStatLevel,		MainH );
    skapi.AddHandler( evidOnStatSkillExp,	MainH );
    skapi.AddHandler( evidOnTargetHealth,	MainH );
    skapi.AddHandler( evidOnTell,		MainH );
    skapi.AddHandler( evidOnTellCovassal,	MainH );
    skapi.AddHandler( evidOnTellFellowship,	MainH );
    skapi.AddHandler( evidOnTellFollower,	MainH );
    skapi.AddHandler( evidOnTellMisc,		MainH );
    skapi.AddHandler( evidOnTellPatron,		MainH );
    skapi.AddHandler( evidOnTellServer,		MainH );
    skapi.AddHandler( evidOnTellVassal,		MainH );
    skapi.AddHandler( evidOnTradeAdd,		MainH );
    skapi.AddHandler( evidOnTradeEnd,		MainH );
    skapi.AddHandler( evidOnTradeReset,		MainH );
    skapi.AddHandler( evidOnTradeStart,		MainH );
    skapi.AddHandler( evidOnVitals,		MainH );

    // Add conditional handlers here
    if (adminChatBox || (manaStoneLimit < manaCnt)) {
	skapi.AddHandler( evidOnChatBoxMessage,		MainH );
    } // endif
    
    if (logExp) {
	skapi.AddHandler( evidOnStatUnspentExp,		MainH );
    } // endif

    if (logExpire) {
    	skapi.AddHandler( evidOnSpellExpire,		MainH );
    	skapi.AddHandler( evidOnSpellExpireSilent,	MainH );
    } // endif

    if (!PeaceMode || !flagIgnoreAtt) {
	skapi.AddHandler( evidOnMeleeDamageSelf,	MainH );
	skapi.AddHandler( evidOnMeleeEvadeSelf,		MainH );
	skapi.AddHandler( evidOnMeleeLastAttacker,	MainH );
    } // endif
  
} // end func

function fRemoveHandlers() {
    // Active handlers
    skapi.RemoveHandler( evidOnActionComplete,		MainH );
    skapi.RemoveHandler( evidOnAddToInventory,		MainH );
    skapi.RemoveHandler( evidOnAdjustStack,		MainH );
    skapi.RemoveHandler( evidOnApproachVendor,		MainH );
    skapi.RemoveHandler( evidOnAssessCreature,		MainH );
    skapi.RemoveHandler( evidOnAssessItem, 		MainH );
    skapi.RemoveHandler( evidOnChatBoxMessage,		MainH );
    skapi.RemoveHandler( evidOnChatBroadcast,		MainH );
    skapi.RemoveHandler( evidOnChatLocal,		MainH );
    skapi.RemoveHandler( evidOnChatServer,		MainH );
    skapi.RemoveHandler( evidOnCombatMode,		MainH );
    skapi.RemoveHandler( evidOnCommand,			MainH );
    skapi.RemoveHandler( evidOnControlEvent,		MainH );
    skapi.RemoveHandler( evidOnDeathMessage,		MainH );
    skapi.RemoveHandler( evidOnDeathSelf,		MainH );
    skapi.RemoveHandler( evidOnDisconnect,		MainH );
    skapi.RemoveHandler( evidOnEnd3D,			MainH );
    skapi.RemoveHandler( evidOnEndPortalSelf,		MainH );
    skapi.RemoveHandler( evidOnFellowInvite,		MainH );
    skapi.RemoveHandler( evidOnFellowUpdate,		MainH );
    skapi.RemoveHandler( evidOnLeaveVendor,		MainH );
    skapi.RemoveHandler( evidOnMeleeDamageSelf,		MainH );
    skapi.RemoveHandler( evidOnMeleeEvadeSelf,		MainH );
    skapi.RemoveHandler( evidOnMeleeLastAttacker,	MainH );
    skapi.RemoveHandler( evidOnNavStop,			MainH );
    skapi.RemoveHandler( evidOnObjectCreate,		MainH );
    skapi.RemoveHandler( evidOnObjectCreatePlayer,	MainH );
    skapi.RemoveHandler( evidOnObjectDestroy,		MainH );
    skapi.RemoveHandler( evidOnOpenContainer,		MainH );
    skapi.RemoveHandler( evidOnPortalStormed,		MainH );
    skapi.RemoveHandler( evidOnPortalStormWarning,	MainH );
    skapi.RemoveHandler( evidOnRemoveFromInventory,	MainH );
    skapi.RemoveHandler( evidOnStatLevel,		MainH );
    skapi.RemoveHandler( evidOnStatSkillExp,		MainH );
    skapi.RemoveHandler( evidOnTargetHealth,		MainH );
    skapi.RemoveHandler( evidOnTell,			MainH );
    skapi.RemoveHandler( evidOnTellCovassal,		MainH );
    skapi.RemoveHandler( evidOnTellFellowship,		MainH );
    skapi.RemoveHandler( evidOnTellFollower,		MainH );
    skapi.RemoveHandler( evidOnTellMisc,		MainH );
    skapi.RemoveHandler( evidOnTellPatron,		MainH );
    skapi.RemoveHandler( evidOnTellServer,		MainH );
    skapi.RemoveHandler( evidOnTellVassal,		MainH );
    skapi.RemoveHandler( evidOnTradeAdd,		MainH );
    skapi.RemoveHandler( evidOnTradeEnd,		MainH );
    skapi.RemoveHandler( evidOnTradeReset,		MainH );
    skapi.RemoveHandler( evidOnTradeStart,		MainH );
    skapi.RemoveHandler( evidOnVitals,			MainH );

    // Remove conditional handlers here
    if (adminChatBox) {
	skapi.RemoveHandler( evidOnChatBoxMessage,	MainH );
    } // endif

    if (logExp) {
	skapi.RemoveHandler( evidOnStatUnspentExp,	MainH );
    } // endif

    if (logExpire) {
    	skapi.RemoveHandler( evidOnSpellExpire,		MainH );
    	skapi.RemoveHandler( evidOnSpellExpireSilent,	MainH );
    } // endif
    
   if (!PeaceMode) {
	skapi.RemoveHandler( evidOnMeleeDamageSelf,	MainH );
	skapi.RemoveHandler( evidOnMeleeEvadeSelf,	MainH );
	skapi.RemoveHandler( evidOnMeleeLastAttacker,	MainH );
    } // endif

} // end func

// ***************************************************************************************************
                                                                                                                                                                                                                                                                                                                                                                                                         