// WAR_LOOT.JS
//
// Author:	Poundboy
// Version:	2.08
//
//
// Modification History:
//
// Who	  Date	    Version	Description
//
// PB	  09/18/03  1.04	Initial version.
//
//
// Description:
//
// Contains all constants and functions related to loot and salvage processing.
//

LootDebug		= false;
LootDebug2		= false;
LootDebugScreen		= false;

lootTypeJunk		=  0;
lootTypeItem		=  1;
lootTypeRare		=  2;
lootTypeMajor		=  3;
lootTypeMinor		=  4;
lootTypeArmor		=  5;
lootTypeWeaponM		=  6;
lootTypeWeapon		=  6;	// Needed because old config files may contain this value
lootTypeWeaponR		=  7;
lootTypeJewelry		=  8;
lootTypeClothing	=  9;
lootTypeWand		= 10;
lootTypeScroll		= 11;
lootTypeSalvage		= 12;
lootTypeUstSalv		= 13;
lootTypeValSalv		= 14;
lootTypeValuable	= 15;
lootTypeDeathItem	= 16;

lootTypeMax		= 16;
lootTypeTotal		= 17;

// Define beginning and ending ranges of good loot
lootTypeBegin		= 1;
lootTypeEnd		= 15;

// Combine point constants for salvaging
combineMaskVal = new Array();
combineMaskVal[1]	= 0x00000001;
combineMaskVal[2]	= 0x00000002;
combineMaskVal[3]	= 0x00000004;
combineMaskVal[4]	= 0x00000008;
combineMaskVal[5]	= 0x00000010;
combineMaskVal[6]	= 0x00000020;
combineMaskVal[7]	= 0x00000040;
combineMaskVal[8]	= 0x00000080;
combineMaskVal[9]	= 0x00000100;
combineMaskVal[10]	= 0x00000200;

// Material salvage type
salTypeImbue	= 1;
salTypeNonImbue	= 2;
salTypeUnknown	= 3;

salTypeText = new Array();
salTypeText[salTypeImbue]	= "Imbue";
salTypeText[salTypeNonImbue]	= "NonImbue";
salTypeText[salTypeUnknown]	= "Unknown";

// Wand and weapon andor codes
andorFlagText = new Array();
andorFlagText[0] = "AND";
andorFlagText[1] = "OR";

armorShield	= eqmShield;
armorHead	= eqmHead;
armorHands	= eqmHands;
armorFeet	= eqmFeet;
armorBody	= eqmChestOuter | eqmAbdomenOuter | eqmUpperArmsOuter |
		  eqmLowerArmsOuter | eqmUpperLegsOuter | eqmLowerLegsOuter;
armorExtremity	= armorHead | armorHands | armorFeet;
armorPerson	= armorBody | armorExtremity;
armorAll	= armorPerson | armorShield;

// Pick up items with the following names.  Thanks to David A. Down (Merlyn of Harvestion) for most of the
// following list (from AutoLoot)
lootItemList = new Array(
		"Pyreal Pea",
		"Gold Pea",
		"Silver Pea",
		"Copper Pea",
		"Iron Pea",
		"Lead Pea",
		"Pyreal Scarab",
		"Gold Scarab",
		"Silver Scarab",
		"Copper Scarab",
		"Iron Scarab",
		"Lead Scarab",
		"A Large Mnemosyne",
		"A Small Mnemosyne",
		"A Tiny Mnemosyne",
		"Ash Gromnie Tooth",
		"Aun Pendant",
		"Auroch Horn",
		"Banderling Scalp",
		"Banderling Aggressor Scalp",
		"Banderling Predator Scalp",
		"Banderling Antagonist Scalp",
		"Banderling Slayer Scalp",
		"Banderling Bone Ring",
		"Banderling Savage Arm",
		"Basalt Blade",
		"Black Rat Tail",
		"Black Rock",
		"Black Stone",
		"Blue Gem",
		"Blue Phyntos Wasp Wing",
		"Broken Virindi Mask",
		"Broken Virindi Profatrix Mask",
		"Bronze Armoredillo Spine",
		"Bronze Coil",
		"Bronze Gear from a Statue",
		"Bronze Gear",
		"Brown Beans",
		"Bunch of Nanners",
		"Canescent Mattekar Pelt",
		"Carenzi Burrower Pelt",
		"Carenzi Pouchling Pelt",
		"Carenzi Sentry Pelt",
		"Carenzi Stalker Pelt",
		"Caulnalain Key",
		"Copper Heart",
		"Coral Heart",
		"Coral Golem Heart",
		"Cracked Shard",
		"Dark Revenant Thighbone",
		"Dark Shard",
		"Dark Sliver",
		"Dark Speck",
		"Dark Tower",
		"Dark Towers",
		"Diamond Heart",
		"Diamond Powder",
		"Dire Mattakar Paw",
		"Doll Mask",
		"Dread Mattakar Paw",
		"Dread Mattekar Paw",
		"Drudge Charm",
		"Murk Drudge Charm",
		"Drudge Mystic Charm",
		"Gloom Drudge Charm",
		"Drudge Bloodletter Charm",
		"Peerless Drudge Charm",
		"Drudge Cabalist Charm",
		"Drudge Sage Charm",
		"Elaniwood Heart",
		"Essence of a Phantasm",
		"Eviscerator Head",
		"Fenmalain Key",
		"Fire Auroch Hide",
		"Fire Auroch Horn",
		"Glacial Heart",
		"Glowing Wisp Heart",
		"Gold Phyntos Wasp Wing",
		"Granite Heart",
		"Great Mattekar Horn",
		"Green Phyntos Wasp Wing",
		"Grey Armoredillo Spine",
		"Grievver Tibia",
		"Gromnie Hide",
		"Hoary Armoredillo Spine",
		"Hoary Mattakar Hide",
		"Hot Pepper",
		"Idol Gem",
		"Iron Heart",
		"Island Armoredillo Spine",
		"Ivory Gromnie Tooth",
		"Jade Gromnie Tooth",
		"Jungle Phyntos Wasp Wing",
		"Large Armoredillo Hide",
		"Large Lugian Sinew",
		"Large Sclavus Hide",
		"Large Shreth Hide",
		"Large Ursuin Hide",
		"Lich Skull",
		"Little Green Seeds",
		"Littoral Siraluun Claw",
		"Lugian Commander's Insignia",
		"Lugian Lieutenant's Insignia",
		"Lugian Sentinel's Insignia",
		"Mattekar Hide",
		"Mire Phyntos Wasp Wing",
		"Monougat",
		"Niffis Shell",
		"Obsidian Heart",
		"Olthoi Claw",
		"Olthoi Head",
		"Olthoi Ichor",
		"Oubliette",
		"Peppermint Stick",
		"Plant",
		"Plate Armoredillo Spine",
		"Platinum Heart",
		"Pulsating Wisp Heart",
		"Pyreal Heart",
		"Pyreal Mote",
		"Pyreal Nugget",
		"Pyreal Sliver",
		"Rabbit Carcass",
		"Rabid Carenzi Pelt",
		"Ravener Guts",
		"Red Jewel",
		"Red Phyntos Wasp Wing",
		"Reedshark Hide",
		"Slasher Reedshark Hide",
		"Reaper Reedshark Hide",
		"Sand Golem Heart",
		"Sandy Armoredillo Spine",
		"Sclavus Hide",
		"Seal",
		"Sharp Tusker Slave Tusk",
		"Shendolain Key",
		"Shore Armoredillo Spine",
		"Shreth Hide",
		"Skeleton's Skull",
		"Skull of High Acolyte",
		"Small Armoredillo Hide",
		"Small Bundle of Kithless Siraluun Feathers",
		"Small Lugian Sinew",
		"Small Niffis Shell",
		"Small Reedshark Hide",
		"Small Sclavus Hide",
		"Small Shard",
		"Small Ursuin Hide",
		"Strange Stick",
		"Sturdy Iron Key",
		"Sturdy Steel Key",
		"Swamp Gromnie Tooth",
		"Swamp Stone",
		"Telumiat Hollow Minion Essence",
		"Thick Gromnie Hide",
		"Thin Gromnie Hide",
		"Timber Siraluun Claw",
		"Tiny Shard",
		"Tree Trunk",
		"Twisted Dark Key",
		"Undead Femur Bone",
		"Undead Femur bone",
		"Undead Thighbone",
		"Ursuin Fang",
		"Ursuin Scalp",
		"Vapor Heart",
		"Vial of Organic Acid",
		"Virindi Directive Key",
		"Virindi Essense",
		"Virindi Implant",
		"Virindi Master Key",
		"Virindi Singularity Key",
		"White Jewel",
		"White Phyntos Wasp Wing",
		"Wood Heart",
		"Yellow Jewel" );

		// "A Lucky Gold Letter",
		// "A Lump of Coal",
		// "A Scarlet Red Letter",

// Make a list of the Major and Minor Spells.  Yes I could just search through all possible spells
// but I want them in a certain order.
majorSpellList = new Array(
		"Major Coordination", "Major Endurance", "Major Focus",
		"Major Quickness", "Major Strength", "Major Willpower",
		"Major Creature Enchantment Aptitude", "Major Item Enchantment Aptitude",
		"Major Life Magic Aptitude", "Major War Magic Aptitude",
		"Major Magic Resistance", "Major Mana Conversion Prowess",
		"Major Axe Aptitude", "Major Bow Aptitude", "Major Crossbow Aptitude", "Major Dagger Aptitude",
		"Major Mace Aptitude", "Major Spear Aptitude", "Major Staff Aptitude", "Major Sword Aptitude",
		"Major Thrown Aptitude", "Major Unarmed Aptitude", "Major Impregnability", "Major Invulnerability",
		"Major Armor Tinkering Expertise", "Major Item Tinkering Expertise",
		"Major Magic Item Tinkering Expertise", "Major Weapon Tinkering Expertise",
		"Major Salvaging Aptitude",
		"Major Alchemical Prowess", "Major Cooking Prowess", "Major Fletching Prowess",
		"Major Arcane Prowess", "Major Deception Prowess", "Major Fealty", "Major Healing Prowess",
		"Major Jumping Prowess", "Major Leadership", "Major Lockpick Prowess", "Major Monster Attunement",
		"Major Person Attunement", "Major Sprint",
		"Major Blood Thirst", "Major Defender", "Major Heart Thirst", "Major Swift Hunter",
		"Major Armor", "Major Acid Ward", "Major Bludgeoning Ward", "Major Flame Ward",
		"Major Piercing Ward", "Major Slashing Ward", "Major Frost Ward", "Major Storm Ward",
		"Major Impenetrability", "Major Acid Bane", "Major Bludgeoning Bane", "Major Flame Bane",
		"Major Frost Bane", "Major Piercing Bane", "Major Slashing Bane", "Major Storm Bane" );

minorSpellList = new Array(
		"Minor Coordination", "Minor Endurance", "Minor Focus",
		"Minor Quickness", "Minor Strength", "Minor Willpower",
		"Minor Creature Enchantment Aptitude", "Minor Item Enchantment Aptitude",
		"Minor Life Magic Aptitude", "Minor War Magic Aptitude",
		"Minor Magic Resistance", "Minor Mana Conversion Prowess",
		"Minor Axe Aptitude", "Minor Bow Aptitude", "Minor Crossbow Aptitude", "Minor Dagger Aptitude",
		"Minor Mace Aptitude", "Minor Spear Aptitude", "Minor Staff Aptitude", "Minor Sword Aptitude",
		"Minor Thrown Aptitude", "Minor Unarmed Aptitude", "Minor Impregnability", "Minor Invulnerability",
		"Minor Armor Tinkering Expertise", "Minor Item Tinkering Expertise",
		"Minor Magic Item Tinkering Expertise", "Minor Weapon Tinkering Expertise",
		"Minor Alchemical Prowess", "Minor Cooking Prowess", "Minor Fletching Prowess",
		"Minor Arcane Prowess", "Minor Deception Prowess", "Minor Fealty", "Minor Healing Prowess",
		"Minor Jumping Prowess", "Minor Leadership", "Minor Lockpick Prowess", "Minor Monster Attunement",
		"Minor Person Attunement", "Minor Sprint",
		"Minor Blood Thirst", "Minor Defender", "Minor Heart Thirst", "Minor Swift Hunter",
		"Minor Armor", "Minor Acid Ward", "Minor Bludgeoning Ward", "Minor Flame Ward",
		"Minor Frost Ward", "Minor Piercing Ward", "Minor Slashing Ward", "Minor Storm Ward",
		"Minor Impenetrability", "Minor Acid Bane", "Minor Bludgeoning Bane", "Minor Flame Bane",
		"Minor Frost Bane", "Minor Piercing Bane", "Minor Slashing Bane", "Minor Storm Bane" );

ItemMajorPat	= new RegExp("(^.*Major.*$)", "g" );	// Used to detect majors
ItemMinorPat	= new RegExp("(^.*Minor.*$)", "g" );	// Used to detect minors
ItemMinorBanePat= new RegExp("(^Minor.*Bane$)", "g" );	// Used to detect minor banes
ItemMinorWardPat= new RegExp("(^Minor.*Ward$)", "g" );	// Used to detect minor wards

// Flags set when special markers detected in loot config file
LOOT_BOF	= false;
LOOT_EOF	= false;


function MaterialRangeRec(Low,High) {
    this.Low  = Low;
    this.High = High;
} // end object

// Define filter for each item type.  Note at the moment the Rare type does not have a filter
// rec as we pick up either all of them or none.
function ItemFilterRec(Name) {
    this.Name		= Name;
    this.Enabled 	= false;
    this.Alert		= false;
} // endif

function MajorFilterRec(Name,SpellId) {
    this.Name		= Name;
    this.SpellId	= SpellId;
    this.Enabled	= true;		// By default pickup all majors
} // endif

function MinorFilterRec(Name,SpellId) {
    this.Name		= Name;
    this.SpellId	= SpellId;
    this.Enabled	= false;
    this.Eqm		= 0;		// Minor must be on the specified body location
} // endif

function ArmorFilterRec(Name, Eqm, CovFlag) {
    // Fixed variables that identify an armor item
    this.Name		= Name;
    this.Eqm		= Eqm;
    this.CovFlag	= CovFlag;

    // Variables settable by user
    this.Enabled	= false;
    this.MinAL		= 0;
    this.MaxWork	= 0;
    this.MaxBU		= 0;
    this.MaxValue	= 0;
    this.MinSpells	= 0;
    this.MinSplLvl	= 0;
} // endif

// Object that defines a melee weapon's loot collection profile
function WeaponMFilterRec(Name, Skid, Wield, AbsDamMax, AbsAttMax, AbsDefMax) {
    // Fixed variables that identify a weapon item
    this.Name		= Name;
    this.Skid		= Skid;
    this.Wield		= Wield;
    this.AbsDamMax	= AbsDamMax;
    this.AbsAttMax	= AbsAttMax;
    this.AbsDefMax	= AbsDefMax;

    // Variables settable by user
    this.Enabled	= false;
    this.MaxWork	= 0;
    this.MinDam		= 0;
    this.MinAtt		= 0;
    this.MinDef		= 0;
    this.AndOrFlag	= 0;
} // end func

// Object that defines a ranged weapon's (bow, xbow, atlatl) loot collection profile
function WeaponRFilterRec(Name, Skid, Wield, AbsDamMax, AbsDefMax, AbsElemMax) {
    // Fixed variables that identify a weapon item
    this.Name		= Name;
    this.Skid		= Skid;
    this.Wield		= Wield;
    this.AbsDamMax	= AbsDamMax;
    this.AbsDefMax	= AbsDefMax;
    this.AbsElemMax	= AbsElemMax;

    // Variables settable by user
    this.Enabled	= false;
    this.MaxWork	= 0;
    this.MinDam		= 0;
    this.MinDef		= 0;
    this.MinElem	= 0;
    this.AndOrFlag	= 0;
} // end func

function WandFilterRec(Name) {
    // Variables settable by user.
    this.Name		= Name;
    this.Enabled	= false;
    this.MinManaC	= 0;
    this.MinMeleeD	= 0;
    this.MinElem	= 0;
    this.AndOrFlag	= 0;
    this.AndOrFlag2	= 0;
} // endif

function SalvageFilterRec(MaterialId, Type, Name, Desc) {
    // Fixed variables that identify a salvage type
    this.Name		= Name;
    this.MaterialId	= MaterialId;
    this.Type		= Type;
    this.Desc		= Desc;
    this.Symbol		= "material" + Name;

    // Private variables used in salvaging and constructed from CombineMask property
    this.MaxWork	= 0;		// Maximum work to be salvaged
    this.WorkRange 	= new Array();	// For each workmanship level, holds the min and max work that will be
    					// will be combined into the same bag (uses MaterialRangeRec object)

    // Variables settable by user.
    this.Enabled	= false;
    this.ExcludeVal	= false;	// If set, then don't pick up this salvage type for selling purposes
    this.MinWork	= 0;		// Workmanship of loot item must be this before picking up.
    this.MaxBU		= 0;		// Maximum burden of loot. Item won't be picked up if burden > than this.
    this.SortPrio	= 0;		// Sort priority for each material.
    this.CombineMask	= combineMaskVal[10];
    					// Bit mask that defines the combine ranges.  Default is 1-10 which is done
} // end func				// by setting bit 10 and clearing all other bits.


function ValSalvFilterRec() {
    // Variables settable by user.
    this.MinRatio	= 0;		// Minimim value to work ratio before picking up item.  Ensures bags a good value
    this.MaxBagValue	= 100000;	// Maximum permitted value of salvage bag.  Needed because vendors don't accept
    					// really expensive bags.
} // endif

function JewelryFilterRec() {
    // Variables settable by user.
    this.MinSplLvl1	= 0;
    this.MinSplLvl2	= 0;
    this.MinSplLvl3	= 0;
} // endif

function ClothingFilterRec() {
    // Variables settable by user.
    this.MinSplLvl1	= 0;
    this.MinSplLvl2	= 0;
    this.MinSplLvl3	= 0;
} // endif

function ScrollFilterRec() {
    // Variables settable by user.
    this.MinLvl		= 7;
    this.MaxLvl		= 7;
    this.UnknownOnly	= false;
    this.LearnUnknown	= false;
} // endif

function ValuableFilterRec() {
    // Variables settable by user.
    this.MaxBU		= 0;
    this.MinValue	= 0;
    this.MaxTotal	= 0;
} // endif

function LootItemRec(Aco, Desc) {
    this.Oid  		= Aco.oid;	// oid of object.  Used to refresh aco if we logout/login
    this.Aco  		= Aco;		// Aco of loot object
    this.Desc 		= Desc;		// Detailed description of loot object generated from assess
    this.SalvageCount	= 0;		// # of times we tried to salvage this item if on salvage loot list
} // end func

function LootTypeRec(Name) {
    // Private properties
    this.Name		= Name;			// name of loot type
    this.Symbol		= "lootType" + Name;	// Symbolic name.  used when creating config files
    this.ItemList	= new Array();		// List of loot items (LootItemRec) of this loot type picked up
    this.ItemCount	= 0;			// Count of items on ItemList for this LootType
    this.TotalValue	= 0;			// Total value of this loot type picked up

    // Properties settable by user
    this.Enabled	= true;			// If true, then pickup this loot type
    this.MaxCount	= 100;			// Maximum # of this loot type that may be picked up
    this.MinSlots	= 10;			// Minimum # of slots that must be free in main pack
    						// before we will pick up this loot type
    
    // Death item processing for this loot type
    this.DiActive	= false;		// If true, DI processing possible for this loot type
    this.DiOrder	= 0;			// If active, index into DeathIndex array
    this.DiEnabled	= false;		// If true, DI processing enabled for this loot type
    this.DiMinValue	= 20000;		// Minimum value for a DI for this loot type
    this.DiMaxBU	= 250;			// Maximum burden for a DI for this loot type
    this.DiMaxWork	= 6;			// Maximum work for a DI for this loot type
 } // end function


function LootObj() {
    // Define some constants
    this.SalDefBur	= 250;			// Default maximum burden for salvage items

    // The following array defines one record for each loot type (weapons, armor, wands, salvage etc).
    this.TypeInfo	= new Array();    	// Holds LootTypeRec object.  Indexed by LootType* constants
    this.TypeCount	= lootTypeTotal;

    // The next set of variables holds the filters that define what we pick up.
    this.ItemFilter	= new Array();			// Items.  Array of ItemFilterRec
    this.MajorFilter	= new Array();			// Majors. Indexed by spell id. Array of MajorFilterRec
    this.MinorFilter	= new Array();			// Minors. Indexed by spell id. Array of MinorFilterRec
    this.ArmorFilter	= new Array();			// Armor. Array of ArmorFilterRec
    this.WeaponMFilter	= new Array();			// Melee weapons (UA, sword,  etc).
    this.WeaponRFilter	= new Array();			// Ranged weapons (bow, xbow, atlatl). 
    this.WandFilter	= new Array();			// Wands. Indexed by workmanship.  Array of WandFilterRec
    this.SalvageFilter	= new Array();			// Salvage.  Indexed by salvage id. Array of SalvageFilterRec
    this.ValSalvFilter  = new ValSalvFilterRec();	// Valuable salvage. Only one record needed.
    this.JewelryFilter	= new JewelryFilterRec();	// Jewelry. Only one record needed
    this.ClothingFilter	= new ClothingFilterRec();	// Clothing. Only one record needed
    this.ScrollFilter	= new ScrollFilterRec();	// Scrolls. Only one record needed
    this.ValuableFilter	= new ValuableFilterRec();	// Valuables (ie sellable stuff).  Only one record needed

    this.MajorIndex	= new Array();			// Translate spell id to MajorFilter offset
    this.MinorIndex	= new Array();			// Translate spell id to MinorFilter offset
    this.SalvageIndex	= new Array();			// Translate material id to SalvageFilter offset
    this.DeathIndex	= new Array();			// Translate type symbol to death list offset
    this.DeathCount	= 0;				// # of loot types that can have death items enabled
    this.FirstDI	= -1;				// Index of first active death item

    this.ItemOpts	= "";			// Decal option list of items
    this.MajorOpts	= "";			// Decal option list of majors (data value is spellid)
    this.MinorOpts	= "";			// Decal option list of minors (data value is spellid)
    this.ArmorOpts	= "";			// Decal option list of armor
    this.WeaponMOpts	= "";			// Decal option list of melee weapons
    this.WeaponROpts	= "";			// Decal option list of ranged weapons
    this.WandOpts	= "";			// Decal option list of wands
    this.SalvageOpts	= "";			// Decal option list of salvage (data value is material code)
    this.DeathItemOpts	= "";			// Decal option list of death item types

    this.SpellCount	= 0;
    this.SpellFreq	= new Array();

    // Next two properties hold the result from the most recent loot search
    this.TypeIndex	= 0;
    this.ItemIndex	= 0;
    this.AlertFlag	= false;		// If the last item we called the WhatType method on was alertable
    						// then this flag is set.
    this.FileName	= "";			// Last filename built from a save or restore operation
    this.OldWeaponAlert = false;		// Set to true when old weapon config info is found
    this.OldWandAlert	= false;		// Set to true when old wand config info is found

    // Methods
    this.Init		= Loot_Init;		// Initialize loot structures
    this.LoadDefaults	= Loot_LoadDefaults;	// Set everything to default values
    this.Load		= Loot_Load;		// Load a profile from a configuration file
    this.Save		= Loot_Save;		// Save a profile to a configuration file

    this.EnableType	= Loot_EnableType;	//
    this.DisableType	= Loot_DisableType;	//
    this.SetType	= Loot_SetType;	 	//
    this.ModType	= Loot_ModType;	 	//
    this.ShowType	= Loot_ShowType;	//
    this.ResetType	= Loot_ResetType;	//
    this.WhatType	= Loot_WhatType;	//

    this.CheckArmor	= Loot_CheckArmor;
    this.CheckWeaponM	= Loot_CheckWeaponM;
    this.CheckWeaponR	= Loot_CheckWeaponR;
    this.CheckWand	= Loot_CheckWand;
    this.CheckMinSpellLvls
    			= Loot_CheckMinSpellLvls
    this.LookupLoot	= Loot_LookupLoot;	// Look for an OID on a loot list
    this.AddLoot	= Loot_AddLoot;		// Add an item to a loot list
    this.RemoveLoot	= Loot_RemoveLoot;	// Remove an item from a loot list
    this.RefreshLoot	= Loot_RefreshLoot;	// Refresh loot lists due to a logoff/login
    this.ResetLoot	= Loot_ResetLoot;	// Zero out loot lists

    this.EnableItem	= Loot_EnableItem;	//
    this.DisableItem	= Loot_DisableItem;	//
    this.SetItem	= Loot_SetItem;		//
    this.ModItem	= Loot_ModItem;
    this.LookupItem	= Loot_LookupItem;	// Lookup an item by name, returning its index
    this.ShowItem	= Loot_ShowItem;
    this.ResetItem	= Loot_ResetItem;

    this.EnableMajor	= Loot_EnableMajor;
    this.DisableMajor	= Loot_DisableMajor;
    this.SetMajor	= Loot_SetMajor;
    this.ShowMajor	= Loot_ShowMajor;
    this.ResetMajor	= Loot_ResetMajor;

    this.EnableMinor	= Loot_EnableMinor;
    this.DisableMinor	= Loot_DisableMinor;
    this.SetMinor	= Loot_SetMinor;
    this.ModMinor	= Loot_ModMinor;
    this.ShowMinor	= Loot_ShowMinor;
    this.ResetMinor	= Loot_ResetMinor;

    this.EnableArmor	= Loot_EnableArmor;	//
    this.DisableArmor	= Loot_DisableArmor;	//
    this.AddArmor	= Loot_AddArmor;	//
    this.SetArmor	= Loot_SetArmor;	//
    this.ModArmor	= Loot_ModArmor;	//
    this.ShowArmor	= Loot_ShowArmor;	//
    this.ResetArmor	= Loot_ResetArmor;	//
    this.LookupArmor	= Loot_LookupArmor;	// Lookup an armor type by name, returning its index

    // Old SetWeapon method needed in case config file contains old settings.  This function
    // doesn't do anything but warn user his weapon data has been discarded
    this.SetWeapon	= Loot_SetWeapon;	// Set all weapon parameters

    this.EnableWeaponM	= Loot_EnableWeaponM;	// Enable a weapon to be picked up
    this.DisableWeaponM	= Loot_DisableWeaponM;	// Disable a weapon from pickup
    this.AddWeaponM	= Loot_AddWeaponM;	// Add a weapon to the weapon list
    this.SetWeaponM	= Loot_SetWeaponM;	// Set all weapon parameters
    this.ModWeaponM	= Loot_ModWeaponM;	// Set one or more weapon parameters
    this.ShowWeaponM	= Loot_ShowWeaponM;
    this.ResetWeaponM	= Loot_ResetWeaponM;
    this.LookupWeaponM	= Loot_LookupWeaponM;	// Lookup a weapon type by name, returning its index

    this.EnableWeaponR	= Loot_EnableWeaponR;	// Enable a weapon to be picked up
    this.DisableWeaponR	= Loot_DisableWeaponR;	// Disable a weapon from pickup
    this.AddWeaponR	= Loot_AddWeaponR;	// Add a weapon to the weapon list
    this.SetWeaponR	= Loot_SetWeaponR;	// Set all weapon parameters
    this.ModWeaponR	= Loot_ModWeaponR;	// Set one or more weapon parameters
    this.ShowWeaponR	= Loot_ShowWeaponR;
    this.ResetWeaponR	= Loot_ResetWeaponR;
    this.LookupWeaponR	= Loot_LookupWeaponR;	// Lookup a weapon type by name, returning its index

    this.EnableWand	= Loot_EnableWand;
    this.DisableWand	= Loot_DisableWand;
    this.SetWand	= Loot_SetWand;
    this.SetWand2	= Loot_SetWand2;	// MeleeD parameter added for wands
    this.SetWand3	= Loot_SetWand3;	// Elemental bonus parameter added for wands
    this.ModWand	= Loot_ModWand;
    this.ShowWand	= Loot_ShowWand;
    this.ResetWand	= Loot_ResetWand;

    this.EnableSalvage	= Loot_EnableSalvage;
    this.DisableSalvage	= Loot_DisableSalvage;
    this.AddSalvage	= Loot_AddSalvage;
    this.SetSalvage	= Loot_SetSalvage;
    this.SetSalvage2	= Loot_SetSalvage2;
    this.ModSalvage	= Loot_ModSalvage;
    this.ShowSalvage	= Loot_ShowSalvage;
    this.ResetSalvage	= Loot_ResetSalvage;
    this.SetWorkRange	= Loot_SetWorkRange;

    this.ModValSalv	= Loot_ModValSalv;
    this.ShowValSalv	= Loot_ShowValSalv;
    this.ResetValSalv	= Loot_ResetValSalv;

    this.ModJewelry	= Loot_ModJewelry;
    this.ShowJewelry	= Loot_ShowJewelry;
    this.ResetJewelry	= Loot_ResetJewelry;

    this.ModClothing	= Loot_ModClothing;
    this.ShowClothing	= Loot_ShowClothing;
    this.ResetClothing	= Loot_ResetClothing;

    this.ModScroll	= Loot_ModScroll;
    this.ShowScroll	= Loot_ShowScroll;
    this.ResetScroll	= Loot_ResetScroll;

    this.ModValuable	= Loot_ModValuable;
    this.ShowValuable	= Loot_ShowValuable;
    this.ResetValuable	= Loot_ResetValuable;

    this.EnableDI	= Loot_EnableDI;
    this.DisableDI	= Loot_DisableDI;
    this.SetDI		= Loot_SetDI;
    this.ShowDI		= Loot_ShowDI;
    this.ResetDI	= Loot_ResetDI;

} // end func


// CONFIGURATION METHODS **********************************************************

function Loot_Init(configFile) {
    if (LootDebug) Log.OutD("Loot_Init: Begin", LootDebugScreen);
    // Main LootType list
    this.TypeInfo[lootTypeJunk]		= new LootTypeRec("Junk");
    this.TypeInfo[lootTypeItem]		= new LootTypeRec("Item");
    this.TypeInfo[lootTypeRare]		= new LootTypeRec("Rare");
    this.TypeInfo[lootTypeMajor]	= new LootTypeRec("Major");
    this.TypeInfo[lootTypeMinor]	= new LootTypeRec("Minor");
    this.TypeInfo[lootTypeArmor]	= new LootTypeRec("Armor");
    this.TypeInfo[lootTypeWeaponM]	= new LootTypeRec("WeaponM");
    this.TypeInfo[lootTypeWeaponR]	= new LootTypeRec("WeaponR");
    this.TypeInfo[lootTypeJewelry]	= new LootTypeRec("Jewelry");
    this.TypeInfo[lootTypeClothing]	= new LootTypeRec("Clothing");
    this.TypeInfo[lootTypeWand]		= new LootTypeRec("Wand");
    this.TypeInfo[lootTypeScroll]	= new LootTypeRec("Scroll");
    this.TypeInfo[lootTypeSalvage]	= new LootTypeRec("Salvage");
    this.TypeInfo[lootTypeUstSalv]	= new LootTypeRec("UstSalv");
    this.TypeInfo[lootTypeValSalv]	= new LootTypeRec("ValSalv");
    this.TypeInfo[lootTypeValuable]	= new LootTypeRec("Valuable");
    this.TypeInfo[lootTypeDeathItem]	= new LootTypeRec("DeathItem");

    // Now mark as active loot types that allow death items
    this.TypeInfo[lootTypeArmor].DiActive	= true;
    this.TypeInfo[lootTypeWeaponM].DiActive	= true;
    this.TypeInfo[lootTypeWeaponR].DiActive	= true;
    this.TypeInfo[lootTypeJewelry].DiActive	= true;
    this.TypeInfo[lootTypeClothing].DiActive	= true;
    this.TypeInfo[lootTypeWand].DiActive	= true;
    
     // Build death item decal list and create array that maps list offset to loot symbol value
    for (var x=lootTypeBegin; x<lootTypeTotal; x++) {
    	if (this.TypeInfo[x].DiActive) {
    	    this.DeathItemOpts += " <option data='" + x + "' text='" + this.TypeInfo[x].Name + "' />\n";
    	    this.TypeInfo[x].DiOrder = this.DeathCount;
    	    this.DeathIndex[this.DeathCount] = x;
    	    this.DeathCount++;
    	    if (this.FirstDI < 0) this.FirstDI = x;
	} // endif
    } // end for

    // Items by name
    for (var x=0; x<lootItemList.length; x++)	{
    	var Name = lootItemList[x];
    	this.ItemFilter[x] = new ItemFilterRec(Name);
    	this.ItemOpts += " <option data='" + x + "' text=\"" + Name + "\" />\n";
    } // endfor

    // Majors
    for (var x=0; x<majorSpellList.length; x++)	{
    	var Name = majorSpellList[x];
    	var SpellId = skapi.SpellidFromSz(Name);
    	this.MajorFilter[x] = new MajorFilterRec(Name,SpellId);
    	this.MajorIndex[SpellId] = x;
    	this.MajorOpts += " <option data='" + x + "' text='" + Name + "'/>\n";
    } // endfor

    // Minors
    for (var x=0; x<minorSpellList.length; x++)	{
   	var Name = minorSpellList[x];
    	var SpellId = skapi.SpellidFromSz(Name);
    	this.MinorFilter[x] = new MinorFilterRec(Name,SpellId);
    	this.MinorIndex[SpellId] = x;
    	this.MinorOpts += " <option data='" + x + "' text='" + Name + "'/>\n";
    } // endfor

    // Armor
    this.AddArmor("Armor (Body)",	armorBody,	false);
    this.AddArmor("Armor (Head)",	armorHead, 	false);
    this.AddArmor("Armor (Hands)",	armorHands, 	false);
    this.AddArmor("Armor (Feet)",	armorFeet, 	false);
    this.AddArmor("Armor (Shield)",	armorShield, 	false);
    this.AddArmor("Cov Armor (Body)",	armorBody,	true );
    this.AddArmor("Cov Armor (Head)",	armorHead, 	true );
    this.AddArmor("Cov Armor (Hands)",	armorHands, 	true );
    this.AddArmor("Cov Armor (Feet)",	armorFeet, 	true );
    this.AddArmor("Cov Armor (Shield)",	armorShield, 	true );

    // Ranged Weapons (desc, skill, wield, max dam, max meleed, max elem)

    this.AddWeaponR("Bow - 360",	skidBow,	360,	130,	15,	12	);
    this.AddWeaponR("Bow - 335",	skidBow,	335,	130,	15,	8	);
    this.AddWeaponR("Bow - 315",	skidBow,	315,	130,	15,	4	);
    this.AddWeaponR("Bow - 290",	skidBow,	290,	130,	15,	0	);
    this.AddWeaponR("Bow - 270",	skidBow,	270,	120,	15,	0	);
    this.AddWeaponR("Bow - 250",	skidBow,	250,	110,	12,	0	);
    this.AddWeaponR("Bow - 0",		skidBow,	0,	100,	7,	0	);

    this.AddWeaponR("Crossbow - 360",	skidCrossbow,	360,	155,	15,	12	);
    this.AddWeaponR("Crossbow - 335",	skidCrossbow,	335,	155,	15,	8	);
    this.AddWeaponR("Crossbow - 315",	skidCrossbow,	315,	155,	15,	4	);
    this.AddWeaponR("Crossbow - 290",	skidCrossbow,	290,	155,	15,	0	);
    this.AddWeaponR("Crossbow - 270",	skidCrossbow,	270,	145,	15,	0	);
    this.AddWeaponR("Crossbow - 250",	skidCrossbow,	250,	140,	15,	0	);
    this.AddWeaponR("Crossbow - 0",	skidCrossbow,	0,	130,	7,	0	);

    this.AddWeaponR("Atlatl - 360", skidThrownWeapons,	360,	150,	15,	12	);
    this.AddWeaponR("Atlatl - 335", skidThrownWeapons,	335,	150,	15,	8	);
    this.AddWeaponR("Atlatl - 315", skidThrownWeapons,	315,	150,	15,	4	);
    this.AddWeaponR("Atlatl - 290", skidThrownWeapons,	290,	150,	15,	0	);
    this.AddWeaponR("Atlatl - 270", skidThrownWeapons,	270,	140,	15,	0	);
    this.AddWeaponR("Atlatl - 250", skidThrownWeapons,	250,	130,	15,	0	);
    this.AddWeaponR("Atlatl - 0",   skidThrownWeapons,	0,	110,	7,	0	);

    // Melee Weapons (desc, skill, wield, max dam, max attack, max meleed)

    this.AddWeaponM("Sword - 400",	skidSword,		400,	55,	15,	15	);
    this.AddWeaponM("Sword - 370",	skidSword,		370,	50,	15,	15	);
    this.AddWeaponM("Sword - 350",	skidSword,		350,	45,	15,	15	);
    this.AddWeaponM("Sword - 325",	skidSword,		325,	40,	15,	15	);
    this.AddWeaponM("Sword - 300",	skidSword,		300,	35,	15,	15	);
    this.AddWeaponM("Sword - 250",	skidSword,		250,	30,	15,	15	);
    this.AddWeaponM("Sword - 0",	skidSword,		0,	20,	11,	11	);

    this.AddWeaponM("UA - 400",		skidUnarmedCombat,  	400,	26,	15,	20	);
    this.AddWeaponM("UA - 370",		skidUnarmedCombat,  	370,	22,	15,	20	);
    this.AddWeaponM("UA - 350",		skidUnarmedCombat,  	350,	20,	15,	20	);
    this.AddWeaponM("UA - 325",		skidUnarmedCombat,  	325,	16,	15,	20	);
    this.AddWeaponM("UA - 300",		skidUnarmedCombat,  	300,	13,	15,	20	);
    this.AddWeaponM("UA - 250",		skidUnarmedCombat,  	250,	11,	15,	20	);
    this.AddWeaponM("UA - 0",		skidUnarmedCombat,  	0,	8,	7,	12	);

    this.AddWeaponM("Axe - 400",	skidAxe, 	    	400,	46,	15,	15	);
    this.AddWeaponM("Axe - 370",	skidAxe, 	    	370,	42,	15,	15	);
    this.AddWeaponM("Axe - 350",	skidAxe, 	    	350,	38,	15,	15	);
    this.AddWeaponM("Axe - 325",	skidAxe, 	    	325,	36,	15,	15	);
    this.AddWeaponM("Axe - 300",	skidAxe, 	    	300,	32,	15,	15	);
    this.AddWeaponM("Axe - 250",	skidAxe, 	    	250,	28,	15,	15	);
    this.AddWeaponM("Axe - 0",		skidAxe, 	    	0,	19,	7,	7	);

    this.AddWeaponM("Dagger - 400",	skidDagger, 	    	400,	26,	15,	15	);
    this.AddWeaponM("Dagger - 370",	skidDagger, 	    	370,	24,	15,	15	);
    this.AddWeaponM("Dagger - 350",	skidDagger, 	    	350,	20,	15,	15	);
    this.AddWeaponM("Dagger - 325",	skidDagger, 	    	325,	18,	15,	15	);
    this.AddWeaponM("Dagger - 300",	skidDagger, 	    	300,	16,	15,	15	);
    this.AddWeaponM("Dagger - 250",	skidDagger, 	    	250,	14,	15,	15	);
    this.AddWeaponM("Dagger - 0",	skidDagger, 	    	0,	12,	11,	11	);

    this.AddWeaponM("Mace - 400",	skidMace, 	    	400,	42,	15,	20	);
    this.AddWeaponM("Mace - 370",	skidMace, 	    	370,	38,	15,	20	);
    this.AddWeaponM("Mace - 350",	skidMace, 	    	350,	36,	15,	20	);
    this.AddWeaponM("Mace - 325",	skidMace, 	    	325,	34,	15,	20	);
    this.AddWeaponM("Mace - 300",	skidMace, 	    	300,	30,	15,	20	);
    this.AddWeaponM("Mace - 250",	skidMace, 	    	250,	26,	15,	20	);
    this.AddWeaponM("Mace - 0",		skidMace, 	    	0,	18,	7,	12	);

    this.AddWeaponM("Spear - 400",	skidSpear, 	    	400,	40,	15,	15	);
    this.AddWeaponM("Spear - 370",	skidSpear, 	    	370,	36,	15,	15	);
    this.AddWeaponM("Spear - 350",	skidSpear, 	    	350,	32,	15,	15	);
    this.AddWeaponM("Spear - 325",	skidSpear, 	    	325,	28,	15,	15	);
    this.AddWeaponM("Spear - 300",	skidSpear, 	    	300,	26,	15,	15	);
    this.AddWeaponM("Spear - 250",	skidSpear, 	    	250,	24,	15,	15	);
    this.AddWeaponM("Spear - 0",	skidSpear, 	    	0,	17,	7,	7	);

    this.AddWeaponM("Staff - 400",	skidStaff,	    	400,	26,	15,	15	);
    this.AddWeaponM("Staff - 370",	skidStaff, 	    	370,	24,	15,	15	);
    this.AddWeaponM("Staff - 350",	skidStaff, 	    	350,	20,	15,	15	);
    this.AddWeaponM("Staff - 325",	skidStaff, 	    	325,	18,	15,	15	);
    this.AddWeaponM("Staff - 300",	skidStaff,	    	300,	16,	15,	15	);
    this.AddWeaponM("Staff - 250",	skidStaff,	    	250,	14,	15,	15	);
    this.AddWeaponM("Staff - 0",	skidStaff,	    	0,	12,	7,	7	);

    // Wands
    for (var x=0; x<11; x++) {
    	var WandName = "Wand  (Work " + x + ")";
    	this.WandFilter[x] = new WandFilterRec(WandName);
    	if (x > 0) this.WandOpts += " <option data='" + x + "' text='" + WandName + "' />\n";
    } // end for

    this.AddSalvage(materialSteel,	    salTypeNonImbue,    "Steel",	 "Inc armor level by 20"	);
    this.AddSalvage(materialAlabaster,	    salTypeNonImbue,    "Alabaster",	 "Inc Pierce prot 0.2"		);
    this.AddSalvage(materialBronze,	    salTypeNonImbue,    "Bronze",	 "Inc Slash prot 0.2"		);
    this.AddSalvage(materialMarble,	    salTypeNonImbue,    "Marble",	 "Inc Bludg prot 0.2"		);
    this.AddSalvage(materialCeramic,	    salTypeNonImbue,    "Ceramic",	 "Inc Fire prot 0.4"		);
    this.AddSalvage(materialDilloHide,	    salTypeNonImbue,    "DilloHide",	 "Inc Acid prot 0.4"		);
    this.AddSalvage(materialReedsharkHide,  salTypeNonImbue,	"ReedsharkHide", "Inc Elec prot 0.4"		);
    this.AddSalvage(materialWool,	    salTypeNonImbue,    "Wool",		 "Inc Cold prot 0.4"		);
    this.AddSalvage(materialBrass,	    salTypeNonImbue,    "Brass",	 "Inc MeleeD Bonus by 1%"	);
    this.AddSalvage(materialGranite,	    salTypeNonImbue,    "Granite",	 "Dec Weapons Variance"		);
    this.AddSalvage(materialIron,	    salTypeNonImbue,    "Iron",		 "Inc Damage by 1"		);
    this.AddSalvage(materialMahogany,	    salTypeNonImbue,    "Mahogany",	 "Inc Dam Modifier by 4%"	);
    this.AddSalvage(materialOak,	    salTypeNonImbue,    "Oak",		 "Dec Speed by 5"		);
    this.AddSalvage(materialVelvet,	    salTypeNonImbue,    "Velvet",	 "Inc Attack Bonus by 1%"	);
    this.AddSalvage(materialGreenGarnet,    salTypeNonImbue,	"GreenGarnet",	 "Inc PvM Elem Bonus by 1%"	);
    this.AddSalvage(materialGold,	    salTypeNonImbue,    "Gold",		 "Raise value by 25%"		);
    this.AddSalvage(materialLinen,	    salTypeNonImbue,    "Linen",	 "Reduce burden by 25%"		);
    this.AddSalvage(materialPine,	    salTypeNonImbue,    "Pine",		 "Reduce value by 25%"		);
    this.AddSalvage(materialMoonstone,	    salTypeNonImbue,    "Moonstone",	 "Inc maximum mana by 250"	);
    this.AddSalvage(materialIvory,	    salTypeImbue,       "Ivory",	 "Make item givable"		);
    this.AddSalvage(materialLeather,	    salTypeImbue,	"Leather",	 "Make item retained"		);
    this.AddSalvage(materialBlackOpal,	    salTypeImbue,	"BlackOpal",	 "Critical Strike Imbue"	);
    this.AddSalvage(materialFireOpal,	    salTypeImbue,	"FireOpal",	 "Crippling Blow Imbue"		);
    this.AddSalvage(materialSunstone,	    salTypeImbue,	"Sunstone",	 "Armor Rending Imbue"		);
    this.AddSalvage(materialOpal,	    salTypeNonImbue,	"Opal", 	 "Inc Mana Conv Mod by 1%"	);
    this.AddSalvage(materialWhiteSapphire,  salTypeImbue,	"WhiteSapphire", "Bludgeon Resist Render"	);
    this.AddSalvage(materialBlackGarnet,    salTypeImbue,	"BlackGarnet",	 "Pierce Resist Render"		);
    this.AddSalvage(materialImperialTopaz,  salTypeImbue,	"ImperialTopaz", "Slash Resist Render"		);
    this.AddSalvage(materialEmerald,	    salTypeImbue,	"Emerald",	 "Acid Resist Render"		);
    this.AddSalvage(materialRedGarnet,	    salTypeImbue,	"RedGarnet",	 "Fire Resist Render"		);
    this.AddSalvage(materialAquamarine,	    salTypeImbue,	"Aquamarine",	 "Frost Resist Render"		);
    this.AddSalvage(materialJet,	    salTypeImbue,	"Jet",		 "Lightning Resist Render"	);
    this.AddSalvage(materialEbony,	    salTypeNonImbue,	"Ebony",	 "Race to Ghar"			);
    this.AddSalvage(materialTeak,	    salTypeNonImbue,	"Teak", 	 "Race to Aluvian"		);
    this.AddSalvage(materialPorcelain,	    salTypeNonImbue,	"Porcelain",	 "Race to Sho"			);
    this.AddSalvage(materialSilver,	    salTypeNonImbue,	"Silver",	 "MeleeD to Missle"		);
    this.AddSalvage(materialCopper,	    salTypeNonImbue,	"Copper",	 "Missle to MeleeD"		);
    this.AddSalvage(materialSilk,	    salTypeNonImbue,	"Silk", 	 "Remove Rank"			);
    this.AddSalvage(materialYellowTopaz,    salTypeImbue,	"YellowTopaz",	 "Missle D +1"			);
    this.AddSalvage(materialPeridot,	    salTypeImbue,	"Peridot",	 "Melee D +1"			);
    this.AddSalvage(materialZircon,	    salTypeImbue,	"Zircon",	 "Magic D +1"			);
    this.AddSalvage(materialLavenderJade,   salTypeImbue,	"LavenderJade",	 "Minor Mana Gain"		);
    this.AddSalvage(materialRedJade,	    salTypeImbue,	"RedJade",	 "Minor Health Gain"		);
    this.AddSalvage(materialCitrine,	    salTypeImbue,	"Citrine",	 "Minor Stamina Gain"		);
    this.AddSalvage(materialCarnelian,	    salTypeImbue,	"Carnelian",	 "Minor Strength"		);
    this.AddSalvage(materialRoseQuartz,	    salTypeImbue,	"RoseQuartz",	 "Minor Quick"			);
    this.AddSalvage(materialBloodstone,	    salTypeImbue,	"Bloodstone",	 "Minor Endurance"		);
    this.AddSalvage(materialSmokeyQuartz,   salTypeImbue,	"SmokeyQuartz",	 "Minor Coord"			);
    this.AddSalvage(materialAgate,	    salTypeImbue,	"Agate",	 "Minor Focus"			);
    this.AddSalvage(materialLapisLazuli,    salTypeImbue,	"LapisLazuli",	 "Minor Self"			);
    this.AddSalvage(materialMalachite,	    salTypeImbue,	"Malachite",	 "Warrior's Vigor"		);
    this.AddSalvage(materialHematite,	    salTypeImbue,	"Hematite",	 "Warrior's Vitality"		);
    this.AddSalvage(materialAzurite,	    salTypeImbue,	"Azurite",	 "Wizard's Intellect"		);
    this.AddSalvage(materialAmber,	    salTypeUnknown,	"Amber",	 "Unknown use"			);
    this.AddSalvage(materialAmethyst,	    salTypeUnknown,	"Amethyst",	 "Unknown use"			);
    this.AddSalvage(materialDiamond,	    salTypeUnknown,	"Diamond",	 "Unknown use"			);
    this.AddSalvage(materialGreenJade,	    salTypeUnknown,	"GreenJade",	 "Unknown use"			);
    this.AddSalvage(materialGromnieHide,    salTypeUnknown,	"GromnieHide",	 "Unknown use"			);
    this.AddSalvage(materialObsidian,	    salTypeUnknown,	"Obsidian",	 "Unknown use"			);
    this.AddSalvage(materialOnyx,	    salTypeUnknown,	"Onyx", 	 "Unknown use"			);
    this.AddSalvage(materialPyreal,	    salTypeUnknown,	"Pyreal",	 "Unknown use"			);
    this.AddSalvage(materialRuby,	    salTypeUnknown,	"Ruby", 	 "Unknown use"			);
    this.AddSalvage(materialSandstone,	    salTypeUnknown,	"Sandstone",	 "Unknown use"			);
    this.AddSalvage(materialSapphire,	    salTypeUnknown,	"Sapphire",	 "Unknown use"			);
    this.AddSalvage(materialSatin,	    salTypeUnknown,	"Satin",	 "Unknown use"			);
    this.AddSalvage(materialSerpentine,     salTypeUnknown,	"Serpentine",	 "Unknown use"			);
    this.AddSalvage(materialTigerEye,	    salTypeUnknown,	"TigerEye",	 "Unknown use"			);
    this.AddSalvage(materialTourmaline,	    salTypeUnknown,	"Tourmaline",	 "Unknown use"			);
    this.AddSalvage(materialTurquoise,	    salTypeUnknown,	"Turquoise",	 "Unknown use"			);
    this.AddSalvage(materialWhiteJade,	    salTypeUnknown,	"WhiteJade",	 "Unknown use"			);
    this.AddSalvage(materialWhiteQuartz,    salTypeUnknown,	"WhiteQuartz",	 "Unknown use"			);
    this.AddSalvage(materialYellowGarnet,   salTypeUnknown,	"YellowGarnet",	 "Unknown use"			);

    if (LootDebug) Log.OutD("Loot_Init: End", LootDebugScreen);
} // end func

// Default filter values for loot structures
function Loot_LoadDefaults() {
    if (LootDebug) Log.OutD("Loot_LoadDefaults: Begin", LootDebugScreen);
    this.ResetType();		// Reset loot types
    this.ResetItem();		// Reset items
    this.ResetMajor();		// Reset majors
    this.ResetMinor();		// Reset minors
    this.ResetArmor();		// Reset armor
    this.ResetWeaponM();	// Reset melee weapons
    this.ResetWeaponR();	// Reset ranged weapons
    this.ResetWand();		// Reset wands
    this.ResetSalvage();	// Reset salvage
    this.ResetValSalv();	// Valuable salvage
    this.ResetJewelry();	// Reset jewelry
    this.ResetClothing();	// Reset clothing
    this.ResetScroll();		// Reset scroll
    this.ResetValuable();	// Reset valuable
    this.ResetDI();		// Reset death items
    if (LootDebug) Log.OutD("Loot_LoadDefault: End", LootDebugScreen);
} // end func

function Loot_Save(fileNum) {
    if (LootDebug) LootLog.OutD("Loot_Save: Begin");

    // Build the output filename from the filenumber.  Note filenum 0 is a special filename.
    if ((fileNum == undefined) || (fileNum == null)) fileNum = 0;
    if (fileNum == 0) {
    	var Post = "def" + fileCfgExt;
    } else {
    	var Post = fileNum + fileCfgExt;
    	if (fileNum < 10) Post = "0" + Post;
    } // endif
    this.FileName = CharCfgDir + "\\" + fileLootCfg + Post;

    // Now create the file
    LootFile = Fso.OpenTextFile(this.FileName, ForWriting, true);	// Create config file

    // Write header record
    LootFile.WriteLine("// Loot config file created " + Util.FormatDate(0,false));
    LootFile.WriteLine("LOOT_BOF = true;")

    // First write type information.
    for (var x=1; x<this.TypeCount; x++) {
	with (this.TypeInfo[x]) {
	    LootFile.WriteLine("Loot.SetType( " + Symbol + ", " + Enabled + ", " + MaxCount + ", " + MinSlots + ");");
	} // end with
    } // end for

    // Write ItemFilter info.  Only need to write items that are enabled or we want to alert on since we set
    // everything disabled at macro start.
    for (var x=0; x<this.ItemFilter.length; x++) {
	with (this.ItemFilter[x]) {
	    if (Enabled || Alert) LootFile.WriteLine("Loot.SetItem(\"" + Name + "\", " + Enabled + ", " + Alert + ")");
	} // end with
    } // end for

    // Write MajorFilter info.  Only need to write items that are disabled since we enable all majors at startup
    for (var x=0; x<majorSpellList.length; x++) {
    	var Name = majorSpellList[x];
    	var SpellId = skapi.SpellidFromSz(Name);
    	var Index = this.MajorIndex[SpellId];
	with (this.MajorFilter[Index]) {
	    if (!Enabled) LootFile.WriteLine("Loot.SetMajor( '" + Name + "', " + Enabled + ")");
	} // end with
    } // end for

    // Write MinorFilter info. Only need to write items that are enabled or have non-zero eqm
    for (var x=0; x<minorSpellList.length; x++) {
   	var Name = minorSpellList[x];
    	var SpellId = skapi.SpellidFromSz(Name);
   	var Index = this.MinorIndex[SpellId];
	with (this.MinorFilter[Index]) {
	    if (Enabled || (Eqm > 0))
		LootFile.WriteLine("Loot.SetMinor( '" + Name + "', " + Enabled + ", " + Util.Hex(Eqm) + ")");
	} // end with
    } // end for

    // Write ArmorFilter info
    for (var x=0; x<this.ArmorFilter.length; x++) {
	with (this.ArmorFilter[x]) {
	    LootFile.WriteLine("Loot.SetArmor( '" + Name + "', " + Enabled + ", " + MinAL + ", " +  MaxWork + ", " +
	    			MaxBU + ", " + MaxValue + ", " + MinSpells + ", " + MinSplLvl + ")");
	} // end with
    } // end for

    // Write Melee WeaponFilter info
   for (var x=0; x<this.WeaponMFilter.length; x++) {
	with (this.WeaponMFilter[x]) {
	    LootFile.WriteLine("Loot.SetWeaponM( '" + Name + "', " + Enabled + ", " + MaxWork + ", " +
				MinDam + ", " + MinAtt + ", " + MinDef + ", " + AndOrFlag + ")");
	} // end with
    } // end for

    // Write Ranged WeaponFilter info
   for (var x=0; x<this.WeaponRFilter.length; x++) {
	with (this.WeaponRFilter[x]) {
	    LootFile.WriteLine("Loot.SetWeaponR( '" + Name + "', " + Enabled + ", " + MaxWork + ", " +
				MinDam + ", " + MinDef + ", " + MinElem + ", " + AndOrFlag + ")");
	} // end with
    } // end for

    // Write WandFilter info
    for (var x=1; x<11; x++) {
	with (this.WandFilter[x]) {
	    LootFile.WriteLine("Loot.SetWand3( " + x + ", " + Enabled + ", " + MinManaC + ", " + MinMeleeD + ", " + 
	    					MinElem + ", " + AndOrFlag + ", " + AndOrFlag2 + ")");
	} // end with
    } // end for

    // Write SalvageFilter info
    for (var x=0; x<=100; x++) {
        if ((this.SalvageFilter[x] != undefined) && (this.SalvageFilter[x] != null)) {
	    with (this.SalvageFilter[x]) {
		LootFile.WriteLine("Loot.SetSalvage2( " + Symbol + ", " + Enabled + ", " + MinWork + ", " + MaxBU +
				   ", " + SortPrio + ", " + Util.Hex(CombineMask) + ", " + ExcludeVal + ")");
	    } // end with
        } // endif
    } // end for

    // Write ValSalvFilter info
    with (this.ValSalvFilter)
	LootFile.WriteLine("Loot.ModValSalv( " + MinRatio + ", " +  MaxBagValue + ")" );

    // Write JewelryFilter info
    with (this.JewelryFilter)
	LootFile.WriteLine("Loot.ModJewelry( " + MinSplLvl1 + ", " +  MinSplLvl2 + ", " + MinSplLvl3 + ")" );

    // Write ClothingFilter info
    with (this.ClothingFilter)
	LootFile.WriteLine("Loot.ModClothing( " + MinSplLvl1 + ", " + MinSplLvl2 + ", " + MinSplLvl3 + ")" );

    // Write ScrollFilter info
    with (this.ScrollFilter)
	LootFile.WriteLine("Loot.ModScroll( " + MinLvl + ", " + MaxLvl + ", " + UnknownOnly + ", " + LearnUnknown + ")" );

    // Write ValuableFilter info
    with (this.ValuableFilter)
	LootFile.WriteLine("Loot.ModValuable( " + MaxBU + ", " + MinValue + ", " + MaxTotal + ")" );

    // Write death item info
    for (var x=lootTypeBegin; x<lootTypeTotal; x++) {
    	with (this.TypeInfo[x]) {
	    if (DiActive) {
		LootFile.WriteLine("Loot.SetDI( " + Symbol + ", " + DiEnabled + ", " + DiMinValue + ", " +
				   DiMaxBU + ", " + DiMaxWork + ")");
	    } // endif
	} // end with
    } // end for

    // All done, write EOF flag and close file
    LootFile.WriteLine("LOOT_EOF = true;")
    LootFile.Close();
    if (LootDebug) Log.OutD("Loot_Save: End", LootDebugScreen);
    return true;
} // endif

function Loot_Load(fileNum) {
    if (LootDebug) Log.OutD("Loot_Load: Begin", LootDebugScreen);

    // Build the output filename from the filenumber.  Note filenum 0 is a special filename.
    if ((fileNum == undefined) || (fileNum == null)) fileNum = 0;
    if (fileNum == 0) {
    	var Post = "def" + fileCfgExt;
    } else {
    	var Post = fileNum + fileCfgExt;
    	if (fileNum < 10) Post = "0" + Post;
    } // endif
    this.FileName = CharCfgDir + "\\" + fileLootCfg + Post;

    // See if file exists
    if(!Fso.FileExists(this.FileName)) return false;

    // Flags set from loot config file.  If not set then we have a problem with the config file
    LOOT_BOF = false;
    LOOT_EOF = false;

    // Read the config file
    Util.Include(this.FileName);

    if (!LOOT_EOF) {
	Chat.Screen("WARNING!!! End of file marker not detected in loot config file!", cmcYellow);
	Chat.Screen("WARNING!!! Loot file is either corrupt or did not write correctly last save operation!", cmcYellow);
	Chat.Screen("WARNING!!! Make sure your loot profiles are setup correctly!", cmcYellow);
    } // endif

    if (LootDebug) Log.OutD("Loot_Load: End", LootDebugScreen);
    return true;
} // endif


// LOOT TYPE METHODS ***********************************************************

function Loot_EnableType(TypeIndex) {
    this.TypeInfo[TypeIndex].Enabled = true;
} // end func

function Loot_DisableType(TypeIndex) {
    this.TypeInfo[TypeIndex].Enabled = false;
} // end func

function Loot_SetType(TypeIndex, Enabled, MaxCount, MinSlots) {
    this.TypeInfo[TypeIndex].Enabled  = Enabled;
    this.TypeInfo[TypeIndex].MaxCount = MaxCount;
    this.TypeInfo[TypeIndex].MinSlots = MinSlots;
} // end func

function Loot_ModType(TypeIndex, MaxCount, MinSlots) {
    if ((MaxCount == undefined) || (MaxCount == null)) MaxCount = -1;
    if ((MinSlots == undefined) || (MinSlots == null)) MinSlots = -1;
    if (MaxCount >= 0) this.TypeInfo[TypeIndex].MaxCount = MaxCount;
    if (MinSlots >= 0) this.TypeInfo[TypeIndex].MinSlots = MinSlots;
} // end func

function Loot_ResetType() {
   // Loot types:   SetType(type, pickup_flag, max_pickups, min_freeslots)
    this.SetType(lootTypeItem,		true,	100,	24 );
    this.SetType(lootTypeRare,		true,	100,	 0 );
    this.SetType(lootTypeMajor,		true,	100,	 0 );
    this.SetType(lootTypeMinor,		true,	100,	 4 );
    this.SetType(lootTypeArmor,		true,	100,	 8 );
    this.SetType(lootTypeWeaponM,	true,	100,	 8 );
    this.SetType(lootTypeWeaponR,	true,	100,	 8 );
    this.SetType(lootTypeJewelry,	true,	100,	 8 );
    this.SetType(lootTypeClothing,	true,	100,	 8 );
    this.SetType(lootTypeWand,		true,	100,	 8 );
    this.SetType(lootTypeScroll,	true,	100,	 8 );
    this.SetType(lootTypeSalvage,	true,	100,	12 );
    this.SetType(lootTypeUstSalv,	true,	100,	12 );
    this.SetType(lootTypeValSalv,	true,	100,	12 );
    this.SetType(lootTypeValuable,	true,	100,	12 );
} //end func

function Loot_ShowType(LootIndex) {
    if ((LootIndex==undefined) || (LootIndex==null)) LootIndex = 0;

    if (LootIndex==0) {
	var Begin = lootTypeBegin;
	var End   = lootTypeEnd;
    } else {
    	var Begin = LootIndex;
    	var End   = LootIndex;
    } // endif

    var Header = false;
    for (var x=Begin; x<=End; x++) {
    	// Show all loot types
	with (this.TypeInfo[x]) {
	    if (Enabled) {
		if (!Header) {
		    Header = true;
		    Chat.Screen(" ");
		    Chat.Screen(Util.Width("Type",1.2) + Util.Width("Enabled",1.0) +
		    	        Util.Width("MaxPickup",1.2) + "MinSlots");
		} // endif

		// Print info
		Chat.Screen(Util.Width(Name,1.2) + Util.Width(Enabled + " ",1.0) +
			    Util.Width(MaxCount + " ",1.2) + MinSlots);
	    } // endif
	} // end with
    } // endfor
    return Header;	// returns false if no items found, true otherwise
} // endfunc

// Check armor to see if it matches criteria.
function Loot_CheckArmor(ItemAco, ArmorIndex) {
    with (this.ArmorFilter[ArmorIndex]) {
	// Check AL level
	if (ItemAco.oai.iai.al < MinAL) return false;

	// Check workmanship
	if (ItemAco.workmanship > MaxWork) return false;

	// Check BU
	if (ItemAco.burden > MaxBU) return false;

	// Check value
	if (ItemAco.cpyValue > MaxValue) return false;

	// If minimum spell count is 0, then just return success since we just passed all the previous tests
	if (MinSpells == 0) return true;

	// Check spell count.  Note SpellCount is set in Loot_WhatType routine
	if (this.SpellCount < MinSpells) return false;

	// Check spell frequency.
	for (x=MinSplLvl; x<10; x++)
	if (this.SpellFreq[x] > 0) return true;
    } // end with

    // No spells found that meet minimum level requirement
    return false;
} // end function

function Loot_CheckWeaponM(ItemAco, WeaponIndex) {
    if (LootDebug) Log.OutD("Loot_CheckWeaponM: Begin, Item [" + ItemAco.szName + "] Index [" + WeaponIndex + "]");
    with (this.WeaponMFilter[WeaponIndex]) {
	var AttBon = (Math.round(ItemAco.oai.iwi.scaleAttackBonus*1000)-1000)/10;
	var DefBon = (Math.round(ItemAco.oai.iwi.scaleDefenseBonus*1000)-1000)/10;
	if (LootDebug)
	    Log.OutD("Loot_CheckWeaponM: AttBon [" + AttBon + "] DefBon [" + DefBon + "]");

	// Workmanship too high?
	if (ItemAco.workmanship > MaxWork) {
	    if (LootDebug) Log.OutD("Loot_CheckWeapon: Failed MaxWork test");
	    return false;
	} // endif

	// Check max damage
	if (ItemAco.oai.iwi.dhealth < MinDam) {
	    if (LootDebug) Log.OutD("Loot_CheckWeaponM: Failed Melee MinDam test");
	    return false;
	} // endif

	// Now check attack bonus and defense bonus.  Depending on value of AndOrFlag we
	// may need both to match or only one (0=AND, 1=OR)
	var Match = 2;

	// Check attack bonus
	if (AttBon < MinAtt) {
	    if (LootDebug) Log.OutD("Loot_CheckWeaponM: Failed MinAtt test");
	    Match--;
	} // endif

	// Check defense bonus
	if (DefBon < MinDef) {
	    if (LootDebug) Log.OutD("Loot_CheckWeaponM: Failed MinDef test");
	    Match--;
	} // endif

	// If none of the above matched, or one matched but the andorflag was set to 0 (AND), then reject weapon.
	if ((Match == 0) || ((Match == 1) && (AndOrFlag == 0))) {
	    if (LootDebug)
		Log.OutD("Loot_CheckWeaponM: Weapon rejected, Match [" + Match + "] AndOrFlag [" + AndOrFlag + "]");
	    return false;
	} // end if
    } // end with

    // Melee weapon passed all tests
    if (LootDebug) Log.OutD("Loot_CheckWeaponM: End, good melee weapon found.");
    return true;
} // end function

function Loot_CheckWeaponR(ItemAco, WeaponIndex) {
    if (LootDebug) Log.OutD("Loot_CheckWeapon: Begin, Item [" + ItemAco.szName + "] Index [" + WeaponIndex + "]");
    with (this.WeaponRFilter[WeaponIndex]) {
	var DamBon  =  Math.round(ItemAco.oai.iwi.scaleDamageBonus*100)-100;
	var DefBon  = (Math.round(ItemAco.oai.iwi.scaleDefenseBonus*1000)-1000)/10;
	var ElemBon = ItemAco.oai.iwi.dhealthElemBonus;
	if (LootDebug)
	    Log.OutD("Loot_CheckWeaponR: DamBon [" + DamBon + "] DefBon [" + DefBon + "] ElemBon [" + ElemBon + "]");

	// Workmanship too high?
	if (ItemAco.workmanship > MaxWork) {
	    if (LootDebug) Log.OutD("Loot_CheckWeaponR: Failed MaxWork test");
	    return false;
	} // endif

	// Reject if damage bonus too low
	if (DamBon < MinDam) {
	    if (LootDebug) Log.OutD("Loot_CheckWeaponR: Failed Ranged MinDam test");
	    return false;
	} // endif

	// Now check defense and elemental bonuses.  Depending on value of AndOrFlag we
	// may need both to match or only one (0=AND, 1=OR)
	var Match = 2;

	// Defense bonus too low?
	if (DefBon < MinDef) {
	    if (LootDebug) Log.OutD("Loot_CheckWeaponR: Failed MinDef test");
	     Match--;
	} // endif

	// Elemental bonus too low?
	if (ElemBon < MinElem) {
	    if (LootDebug) Log.OutD("Loot_CheckWeaponR: Failed MinElem test");
	     Match--;
	} // endif

	// If none of the above matched, or one matched but the andorflag was set to 0 (AND), then reject weapon.
	if ((Match == 0) || ((Match == 1) && (AndOrFlag == 0))) {
	    if (LootDebug)
		Log.OutD("Loot_CheckWeaponR: Weapon rejected, Match [" + Match + "] AndOrFlag [" + AndOrFlag + "]");
	    return false;
	} // end if
    } // end with

    // Ranged weapon passed all tests
    if (LootDebug) Log.OutD("Loot_CheckWeaponR: End, good ranged weapon found.");
    return true;
} // end function

function Loot_CheckWand(ItemAco, Work) {
    if (LootDebug) Log.OutD("Loot_CheckWand: Start");
    var ManaConvMod = ItemAco.oai.ibi.fractManaConvMod;
    var MeleeDMod = 0;
    var ElemBonus = 0;
    if (ItemAco.oai.iwi) {
	MeleeDMod = (Math.round(ItemAco.oai.iwi.scaleDefenseBonus*1000)-1000)/10;
	ElemBonus = (Math.round(ItemAco.oai.iwi.scalePvMElemBonus*1000)-1000)/10;
    } // endif

    if (LootDebug) Log.OutD("Loot_CheckWand: ManaConvMod [" + ManaConvMod +
			    "] MeleeDMod [" + MeleeDMod + "] ElemBonus [" + ElemBonus + "]");

    var MatchMana   = (ManaConvMod >= this.WandFilter[Work].MinManaC/100);
    var MatchMeleeD = (MeleeDMod   >= this.WandFilter[Work].MinMeleeD);
    var MatchElemBo = (ElemBonus   >= this.WandFilter[Work].MinElem);

    if (LootDebug) {
	Log.OutD("Loot_CheckWand: MatchMana [" + MatchMana +
		 "] MatchMeleeD [" + MatchMeleeD + "] MatchElemBo [" + MatchElemBo + "]");
	Log.OutD("Loot_CheckWand: AndOrFlag [" + AndOrFlag + "] AndOrFlag2 [" + AndOrFlag2 + "]");
    } // endif

    // First see if mana conversion / melee d passes test
    var ManaMeleeMatch = false;
    if (((this.WandFilter[Work].AndOrFlag == 0) && MatchMana && MatchMeleeD) ||
	((this.WandFilter[Work].AndOrFlag == 1) && (MatchMana || MatchMeleeD))) {
	// Passed
	ManaMeleeMatch = true;
    } // endif

    // Now compare above result with elemental bonus
    if (((this.WandFilter[Work].AndOrFlag2 == 0) && ManaMeleeMatch && MatchElemBo) ||
	((this.WandFilter[Work].AndOrFlag2 == 1) && (ManaMeleeMatch || MatchElemBo))) {
	// Passed
	if (LootDebug) Log.OutD("Loot_CheckWand: Wand passed");
	return true;
    } // endif

    // If we make it here, check failed
    if (LootDebug) Log.OutD("Loot_CheckWand: Wand failed");
    return false;
} // end function Loot_CheckWand

function Loot_CheckMinSpellLvls(MinSplLvl, MinSplLvl2, MinSplLvl3) {
    var TempFreq = new Array();
    for (var x=0; x<10; x++) TempFreq[x] = this.SpellFreq[x];

    if (MinSplLvl>0) {
	for (var x=MinSplLvl; x<10; x++) {
	    if (TempFreq[x] > 0) {
		TempFreq[x]--;	// Decrement freq count for this level
		break;
	    } // endif
	} // endfor
	// If no spell matches mininum specified in 'MinSplLvl', then check failed.
	if (x==10) return false;
    } // endif

   if (MinSplLvl2>0) {
	for (var x=MinSplLvl2; x<10; x++) {
	    if (TempFreq[x] > 0) {
		TempFreq[x]--;	// Decrement freq count for this level
		break;
	    } // endif
	} // endfor
	// If no spell matches mininum specified in 'MinSplLvl2', then check failed.
	if (x==10) return false;
    } // endif

   if (MinSplLvl3>0) {
	for (var x=MinSplLvl3; x<10; x++) {
	    if (TempFreq[x] > 0) {
		TempFreq[x]--;	// Decrement freq count for this level
		break;
	    } // endif
	} // endfor
	// If no spell matches mininum specified in 'MinSplLvl3', then check failed.
	if (x==10) return false;
    } // endif

    // If we make it here than all minimum spell requirements have been met, return true
    return true;
} //end func

function Loot_WhatType(ItemAco) {
    if (LootDebug) Log.OutD("Loot_WhatType: Begin, Item [" + ItemAco.szName + "] Eqm [" + Util.Hex(ItemAco.eqm) + "]");

    var Material = ItemAco.material;
    var Burden   = ItemAco.burden;
    var Work     = ItemAco.workmanship;
    var Value    = ItemAco.cpyValue;

    if (LootDebug)
    	Log.OutD("Loot_WhatType: Material [" + Material + "] Bur [" + Burden + "] Wrk [" + Work + "] Val [" + Value + "]");

    // See if the item name matches any entries on 'lootItemList'
    if (LootDebug) Log.OutD("Loot_WhatType: Checking name match...");
    this.AlertFlag = false;
    for (var x=0; x<this.ItemFilter.length; x++) {
	if (this.ItemFilter[x].Name == ItemAco.szName) {
	    // Found match, see if we want to pick it up
	    if (this.ItemFilter[x].Alert) this.AlertFlag = true;
	    if (this.ItemFilter[x].Enabled)  {
		if (LootDebug) Log.OutD("Loot_WhatType: Match found");
		return lootTypeItem;
	    } // endif
	} // endif
    } // endfor
    if (LootDebug) Log.OutD("Loot_WhatType: Name match not found");

    // See if item is a 'rare'.  At the moment (10/15/05) skunkworks doesn't
    // have a rare property on an item assess, so we look at the raw item assess
    // property which I think key 0x00010011 gives the rare #
    if (LootDebug) Log.OutD("Loot_WhatType: Checking RARE match...");
    if ((ItemAco.oai) && (ItemAco.oai.ibi)) {
    	if (ItemAco.oai.ibi.raw.Exists(rawRareKey)) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Rare found.");
	    this.AlertFlag = true;
	    return lootTypeRare;
	} // endif
    } // endif

    // Blank out spell count and frequency array
    this.SpellCount = 0;	// # of spells found on item
    for (var x=0; x<10; x++) this.SpellFreq[x] = 0;

    // If this is equipment and an assess was successfully completed on this item, then process the spell
    // information on this item (if it has any).  While looking at the spell info, check for majors and
    // minors if desired.
    if ((ItemAco.eqm != 0) && (ItemAco.oai) && (ItemAco.oai.iei)) {
    	if (LootDebug) Log.OutD("Loot_WhatType: Generating spell information...");
	// Loop through all the spells on this item
	this.SpellCount = ItemAco.oai.iei.cspellid;
	for (var x=0; x<ItemAco.oai.iei.cspellid; x++) {
	    var Id       = ItemAco.oai.iei.rgspellid(x);
	    var SpellObj = skapi.SpellInfoFromSpellid(Id);
	    var Name     = SpellObj.szName;

	    // Compute spell level from difficulty and update spell frequency array
	    var Lvl = Math.round(SpellObj.diff/50)+1;
	    this.SpellFreq[Lvl]++;
	    if (LootDebug) Log.OutD("Spell # [" + x + "] Id [" + Id + "] Lvl [" + Lvl + "] Name [" + Name + "]");

	    // Check for majors if desired
	    if (this.TypeInfo[lootTypeMajor].Enabled) {
	    	if (this.MajorIndex[Id] != undefined) {
		    var Offset = this.MajorIndex[Id];
		    if (this.MajorFilter[Offset].Enabled) {
			if (LootDebug) Log.OutD("Loot_WhatType: Major [" + Name + "] found");
			return lootTypeMajor;
		    } // endif
		} // endif
	    } // endif

	    // Check for minors if desired
	    if (this.TypeInfo[lootTypeMinor].Enabled) {
	    	if (this.MinorIndex[Id] != undefined) {
		    var Offset = this.MinorIndex[Id];
		    if (this.MinorFilter[Offset].Enabled && (this.MinorFilter[Offset].Eqm & ItemAco.eqm)) {
			if (LootDebug) Log.OutD("Loot_WhatType: Minor [" + Name + "] found");
			return lootTypeMinor;
		    } // endif
		} // endif
	    } // endif
	} // endfor
    } // endif

    // If this item is underclothes and flag is set to pick them up, then see if we meet the
    // minimum spell requirements.   If it's a boot (which has LowerLegsUnder set) then reject.
    if (this.TypeInfo[lootTypeClothing].Enabled && !(ItemAco.eqm & eqmFeet) &&
        (ItemAco.eqm & (eqmChestUnder | eqmAbdomenUnder | eqmUpperArmsUnder |
			eqmUpperLegsUnder | eqmLowerLegsUnder))) {
	if (this.CheckMinSpellLvls(this.ClothingFilter.MinSplLvl1, this.ClothingFilter.MinSplLvl2,
				   this.ClothingFilter.MinSplLvl3)) {
	    // Good clothing found.  See if we have hit maximum items for this type
	    if (Loot.TypeInfo[lootTypeClothing].ItemCount < Loot.TypeInfo[lootTypeClothing].MaxCount) {
		if (LootDebug) Log.OutD("Loot_WhatType: Good clothing found");
		return lootTypeClothing;
	    } else {
		if (LootDebug) Log.OutD("Loot_WhatType: Good clothing found but exceeding max item count!");
	    } // endif
	} // endif

	// Now see if this is a death item (if enabled)
	if (this.TypeInfo[lootTypeDeathItem].Enabled &&
	    (this.TypeInfo[lootTypeDeathItem].ItemCount < this.TypeInfo[lootTypeDeathItem].MaxCount) &&
	    this.TypeInfo[lootTypeClothing].DiActive &&
	    this.TypeInfo[lootTypeClothing].DiEnabled &&
	    (Value  >= this.TypeInfo[lootTypeClothing].DiMinValue) &&
	    (Burden <= this.TypeInfo[lootTypeClothing].DiMaxBU) &&
	    (Work   <= this.TypeInfo[lootTypeClothing].DiMaxWork)) {
	    // Found a clothing death item
	    return lootTypeDeathItem;
	} // endif
    } // endif

    // If we want to pickup armor, since item is armor, and armor assess information exists then check it
    if (this.TypeInfo[lootTypeArmor].Enabled && (ItemAco.eqm & armorAll) && (ItemAco.oai) && (ItemAco.oai.iai)) {
    	if (LootDebug) Log.OutD("Loot_WhatType: Armor found, examining...");
    	// See if item is covenant armor
    	if (ItemAco.szName.match(/Covenant/))
	    var CovArmor = true;
	else
	    var CovArmor = false;

	// Cycle through the armor filter entries looking for a match
	for (var x=0; x<this.ArmorFilter.length; x++)
	    if (this.ArmorFilter[x].Enabled &&
	        (this.ArmorFilter[x].Eqm & ItemAco.eqm) &&
	        (this.ArmorFilter[x].CovFlag == CovArmor)) break;

	// If match found, check armor item against criteria
	if (x < this.ArmorFilter.length) {
	    if (this.CheckArmor(ItemAco, x)) {
	    	// Good armor found.   See if we have hit maximum items for this type
		if (Loot.TypeInfo[lootTypeArmor].ItemCount < Loot.TypeInfo[lootTypeArmor].MaxCount) {
		    if (LootDebug) Log.OutD("Loot_WhatType: Good armor found");
		    return lootTypeArmor;
		} else {
		    if (LootDebug) Log.OutD("Loot_WhatType: Good armor found but exceeding max item count!");
		} // endif
	    } // endif
	} //endif
	
	// Now see if this is a death item (if enabled)
	if (this.TypeInfo[lootTypeDeathItem].Enabled &&
	    (this.TypeInfo[lootTypeDeathItem].ItemCount < this.TypeInfo[lootTypeDeathItem].MaxCount) &&
	    this.TypeInfo[lootTypeArmor].DiActive &&
	    this.TypeInfo[lootTypeArmor].DiEnabled &&
	    (Value  >= this.TypeInfo[lootTypeArmor].DiMinValue) &&
	    (Burden <= this.TypeInfo[lootTypeArmor].DiMaxBU) &&
	    (Work   <= this.TypeInfo[lootTypeArmor].DiMaxWork)) {
	    // Found a armor death item
	    return lootTypeDeathItem;
	} // endif
    } //endif

    // If we want to pickup melee weapons, this item is a melee weapon, and we have assess weapon info then check it
    if (this.TypeInfo[lootTypeWeaponM].Enabled && (ItemAco.eqm & eqmWeapon) &&
       (ItemAco.oai) && (ItemAco.oai.iwi) && (ItemAco.oai.ibi)) {

	// First determine weapon wield requirement
	var WieldReq = 0;
	if ((ItemAco.oai.ibi.skidWieldReq != null) && (ItemAco.oai.ibi.skidWieldReq != skidNil))
		WieldReq = ItemAco.oai.ibi.lvlWieldReq;

    	if (LootDebug) {
	    Log.OutD("Loot_WhatType: Melee weapon found, filter len [" + this.WeaponMFilter.length + "]");
	    Log.OutD("Loot_WhatType: Skid [" + ItemAco.oai.iwi.skid + "] wield [" + WieldReq + "]");
	} // endif

	// Cycle through the melee weapon filter entries looking for a match
	for (var x=0; x<this.WeaponMFilter.length; x++) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Index [" + x + "] Name [" + this.WeaponMFilter[x].Name +
	    			    "] Skid [" + this.WeaponMFilter[x].Skid +
	    			    "] Wield [" + this.WeaponMFilter[x].Wield + "]");
	    if (this.WeaponMFilter[x].Enabled &&
	        (this.WeaponMFilter[x].Skid == ItemAco.oai.iwi.skid) &&
		(this.WeaponMFilter[x].Wield == WieldReq)) break;
	} // endfor

	// If match found, check weapon item against criteria
	if (x < this.WeaponMFilter.length) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Melee weapon type [" + this.WeaponMFilter[x].Name + "] found.");
	    if (this.CheckWeaponM(ItemAco, x)) {
	    	// Good melee weapon found. See if we have hit maximum items for this type
	    	if (Loot.TypeInfo[lootTypeWeaponM].ItemCount < Loot.TypeInfo[lootTypeWeaponM].MaxCount) {
		    if (LootDebug) Log.OutD("Loot_WhatType: Good melee weapon found.");
		    return lootTypeWeaponM;
		} else {
		    if (LootDebug) Log.OutD("Loot_WhatType: Good melee weapon found but exceeding max item count!");
		} // endif
	    } // endif
	} else {
	    if (LootDebug) Log.OutD("Loot_WhatType: Unknown melee weapon type.");
	} //endif

	// Now see if this is a death item (if enabled)
	if (this.TypeInfo[lootTypeDeathItem].Enabled &&
	    (this.TypeInfo[lootTypeDeathItem].ItemCount < this.TypeInfo[lootTypeDeathItem].MaxCount) &&
	    this.TypeInfo[lootTypeWeaponM].DiActive &&
	    this.TypeInfo[lootTypeWeaponM].DiEnabled &&
	    (Value  >= this.TypeInfo[lootTypeWeaponM].DiMinValue) &&
	    (Burden <= this.TypeInfo[lootTypeWeaponM].DiMaxBU) &&
	    (Work   <= this.TypeInfo[lootTypeWeaponM].DiMaxWork)) {
	    // Found a melee weapon death item
	    return lootTypeDeathItem;
	} // endif
    } //endif

    // If we want to pickup ranged weapons, this item is a ranged weapon and we have assess weapon info then check it
    if (this.TypeInfo[lootTypeWeaponR].Enabled && (ItemAco.eqm & eqmRangedWeapon) &&
        (ItemAco.oai) && (ItemAco.oai.iwi) && (ItemAco.oai.ibi)) {

	// First determine weapon wield requirement
	var WieldReq = 0;
	if ((ItemAco.oai.ibi.skidWieldReq != null) && (ItemAco.oai.ibi.skidWieldReq != skidNil))
		WieldReq = ItemAco.oai.ibi.lvlWieldReq;

    	if (LootDebug) {
	    Log.OutD("Loot_WhatType: Ranged weapon found, filter len [" + this.WeaponRFilter.length + "]");
	    Log.OutD("Loot_WhatType: Skid [" + ItemAco.oai.iwi.skid + "] wield [" + WieldReq + "]");
	} // endif

	// Cycle through the weapon filter entries looking for a match
	for (var x=0; x<this.WeaponRFilter.length; x++) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Index [" + x + "] Name [" + this.WeaponRFilter[x].Name +
	    			    "] Skid [" + this.WeaponRFilter[x].Skid +
	    			    "] Wield [" + this.WeaponRFilter[x].Wield + "]");
	    if (this.WeaponRFilter[x].Enabled &&
	        (this.WeaponRFilter[x].Skid == ItemAco.oai.iwi.skid) &&
		(this.WeaponRFilter[x].Wield == WieldReq)) break;
	} // endfor

	// If match found, check weapon item against criteria
	if (x < this.WeaponRFilter.length) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Ranged weapon type [" + this.WeaponRFilter[x].Name + "] found.");
	    if (this.CheckWeaponR(ItemAco, x)) {
	    	// Good ranged weapon found. See if we have hit maximum items for this type
	    	if (Loot.TypeInfo[lootTypeWeaponR].ItemCount < Loot.TypeInfo[lootTypeWeaponR].MaxCount) {
		    if (LootDebug) Log.OutD("Loot_WhatType: Good ranged weapon found.");
		    return lootTypeWeaponR;
		} else {
		    if (LootDebug) Log.OutD("Loot_WhatType: Good ranged weapon found but exceeding max item count!");
		} // endif
	    } // endif
	} else {
	    if (LootDebug) Log.OutD("Loot_WhatType: Unknown ranged weapon type.");
	} //endif

	// Now see if this is a death item (if enabled)
	if (this.TypeInfo[lootTypeDeathItem].Enabled &&
	    (this.TypeInfo[lootTypeDeathItem].ItemCount < this.TypeInfo[lootTypeDeathItem].MaxCount) &&
	    this.TypeInfo[lootTypeWeaponR].DiActive &&
	    this.TypeInfo[lootTypeWeaponR].DiEnabled &&
	    (Value  >= this.TypeInfo[lootTypeWeaponR].DiMinValue) &&
	    (Burden <= this.TypeInfo[lootTypeWeaponR].DiMaxBU) &&
	    (Work   <= this.TypeInfo[lootTypeWeaponR].DiMaxWork)) {
	    // Found a ranged weapon death item
	    return lootTypeDeathItem;
	} // endif
    } //endif

    // If we want to pickup nice jewelry and this item is equipable in necklace,
    // bracelet, or ring slots, then check it out
    if (this.TypeInfo[lootTypeJewelry].Enabled && (ItemAco.eqm & (eqmNecklace|eqmBracelets|eqmRings))) {
	// See if at least one spell at the required level...
	if (this.CheckMinSpellLvls(this.JewelryFilter.MinSplLvl1, this.JewelryFilter.MinSplLvl2,
				   this.JewelryFilter.MinSplLvl3)) {
	    // Spell found at or above the required level, we have a winner
	    if (Loot.TypeInfo[lootTypeJewelry].ItemCount < Loot.TypeInfo[lootTypeJewelry].MaxCount) {
		if (LootDebug) Log.OutD("Loot_WhatType: Good jewelry found");
		return lootTypeJewelry;
	    } else {
		if (LootDebug) Log.OutD("Loot_WhatType: Good jewelry found but exceeding max item count!");
	    } // endif
	} // endif

	// Now see if this is a death item (if enabled)
	if (this.TypeInfo[lootTypeDeathItem].Enabled &&
	    (this.TypeInfo[lootTypeDeathItem].ItemCount < this.TypeInfo[lootTypeDeathItem].MaxCount) &&
	    this.TypeInfo[lootTypeJewelry].DiActive &&
	    this.TypeInfo[lootTypeJewelry].DiEnabled &&
	    (Value  >= this.TypeInfo[lootTypeJewelry].DiMinValue) &&
	    (Burden <= this.TypeInfo[lootTypeJewelry].DiMaxBU) &&
	    (Work   <= this.TypeInfo[lootTypeJewelry].DiMaxWork)) {
	    // Found a jewelry death item.
	    if (LootDebug) Log.OutD("Loot_WhatType: Jewelry death item found");
	    return lootTypeDeathItem;
	} // endif
    } // endif

    /* If picking up of wands is enabled, then check item to see if it is a good wand */
    if (this.TypeInfo[lootTypeWand].Enabled && (ItemAco.eqm & eqmFocusWeapon) && (ItemAco.oai) && (ItemAco.oai.ibi)) {
	if (LootDebug) Log.OutD("Loot_WhatType: Checking wand");

	if (!this.WandFilter[Work].Enabled) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Wand checking disabled for this workmanship");
	} else {
	    // Wand checking enabled for this workmanship
	    if (this.CheckWand(ItemAco,Work)) {
		// Found nice wand.  See if we have hit maximum items for this type
		if (Loot.TypeInfo[lootTypeWand].ItemCount < Loot.TypeInfo[lootTypeWand].MaxCount) {
		    if (LootDebug) Log.OutD("Loot_WhatType: Nice wand found");
		    return lootTypeWand;
		} else {
		    if (LootDebug) Log.OutD("Loot_WhatType: Nice wand found but exceeding max item count!");
		} // endif
	    } // endif
	} // endif

	// Now see if this is a death item (if enabled)
	if (this.TypeInfo[lootTypeDeathItem].Enabled &&
	    (this.TypeInfo[lootTypeDeathItem].ItemCount < this.TypeInfo[lootTypeDeathItem].MaxCount) &&
	    this.TypeInfo[lootTypeWand].DiActive &&
	    this.TypeInfo[lootTypeWand].DiEnabled &&
	    (Value  >= this.TypeInfo[lootTypeWand].DiMinValue) &&
	    (Burden <= this.TypeInfo[lootTypeWand].DiMaxBU) &&
	    (Work   <= this.TypeInfo[lootTypeWand].DiMaxWork)) {
	    // Found a wand death item
	    if (LootDebug) Log.OutD("Loot_WhatType: Wand death item found");
	    return lootTypeDeathItem;
	} // endif
    } // endif

    // If the item is a scroll, see if it matches are criteria for scroll pickup.
    if (this.TypeInfo[lootTypeScroll].Enabled && (ItemAco.mcm & mcmBooksPaper) &&
        (ItemAco.szName.match(/Scroll/))) {
	// See if scroll is within level restrictions and if we only want scrolls that are in our spell book
	var SpellId     = ItemAco.spellid;
	var SpellObj    = skapi.SpellInfoFromSpellid(SpellId);
	var SpellLvl    = Math.floor(SpellObj.diff/50)+1;
	var InSpellBook = skapi.FSpellidInSpellbook(SpellId);
	var OnLootList  = FindScrollOnLootList(ItemAco.szName);
	if (LootDebug) {
	    Log.OutD("Checking scroll [" + ItemAco.szName + "] diff [" + SpellObj.diff + "] Lvl [" +
		     SpellLvl + "] Inbook [" + InSpellBook + "] OnLootList [" + OnLootList + "]");
	    Log.OutD("FilterMin [" +  this.ScrollFilter.MinLvl + "] FilterMax [" + this.ScrollFilter.MaxLvl + "]");
	    Log.OutD("UnknownOnly [" + this.ScrollFilter.UnknownOnly + "]");
	} // endif
	if ((SpellLvl >= this.ScrollFilter.MinLvl) &&
	    (SpellLvl <= this.ScrollFilter.MaxLvl) &&
	    (!this.ScrollFilter.UnknownOnly || (this.ScrollFilter.UnknownOnly && !InSpellBook && !OnLootList))) {
	    if (LootDebug) Log.OutD("Loot_WhatType: Scroll found");
	    return lootTypeScroll;
	} // endif
    } // endif

    // If we want to look for tink salvage, see if this items workmanship is high enough for it's
    // material type and its burden is not too high.  Don't do burden check if autoUST enabled
    if (this.TypeInfo[lootTypeSalvage].Enabled) {
    	if (LootDebug) Log.OutD("Loot_WhatType: Checking for tink salvage");
	// See if this material is defined on our salvage list
	if ((this.SalvageIndex[Material] == undefined) || (this.SalvageIndex[Material] == null)) {
	    if (LootDebug) Log.OutD("Loot_WhatType: No filter record for this material, skipping tink salvage check.");
	} else {
	    // Material defined, see if material is enabled for pickup and item's workmanship is high enough. 
	    // Don't include bags of salvage in this category
	    var Offset = this.SalvageIndex[Material];
	    if (LootDebug) Log.OutD("Loot_WhatType: MinWork [" + this.SalvageFilter[Offset].MinWork + "]");
	    if ((this.SalvageFilter[Offset].Enabled) &&
	        (Work >= this.SalvageFilter[Offset].MinWork) &&
	        !(ItemAco.mcm & mcmSalvageBag)) {
	    	// Found a salvagable item.  If auto-usting enabled and materials workmanship is not too high,
	    	// then classify item as a to be usted salvage item and return.
	    	if (flagAutoUst && (Work <= this.SalvageFilter[Offset].MaxWork)) {
		    if (LootDebug) Log.OutD("Loot_WhatType: Ustable salvage found");
		    return lootTypeUstSalv;
		} // endif
	    	
	    	// Found a salvage item to potentially pick up.  Make sure items burden is not too high
	    	if (Burden > this.SalvageFilter[Offset].MaxBU) {
    	    	    if (LootDebug) Log.OutD("Loot_WhatType: Good tink salvage found but too heavy");
    	    	} else {
    	    	    // Good salvage item found.  Pickup if current item count less than max
		    if (Loot.TypeInfo[lootTypeSalvage].ItemCount < Loot.TypeInfo[lootTypeSalvage].MaxCount) {
			if (LootDebug) Log.OutD("Loot_WhatType: Good tink salvage found");
			return lootTypeSalvage;
		    } else {
		    	if (LootDebug) Log.OutD("Loot_WhatType: Good tink salvage found but exceeding max item count!");
		    } // endif
		} // endif
	    } // endif
	} // endif
	if (LootDebug) Log.OutD("Loot_WhatType: Not tink salvagable.");
    } // endif

    // If we want to look for valuable salvage, see if this items price to workmanship ratio is high enough
    // Note that USTing must be enabled before this category of loot will be picked up.
    if (this.TypeInfo[lootTypeValSalv].Enabled && flagAutoUst) {
    	if (LootDebug) Log.OutD("Loot_WhatType: Checking for valuable salvage");
	// See if this material is defined on our salvage list
	if ((this.SalvageIndex[Material] == undefined) || (this.SalvageIndex[Material] == null)) {
	    if (LootDebug) Log.OutD("Loot_WhatType: No filter record for this material, skipping val salvage check.");
	} else {
	    // Material defined, see if item's price to workmanship ratio is high enough.
	    // Don't include bags of salvage in this category or materials that are being picked
	    // up as tink salvage.
	    var Offset = this.SalvageIndex[Material];
	    if (LootDebug) Log.OutD("Loot_WhatType: Ratio [" + this.ValSalvFilter.MinRatio + "]");
	    if (!this.SalvageFilter[Offset].ExcludeVal && (Work>0) && 
	       ((Value/Work) >= this.ValSalvFilter.MinRatio) && !(ItemAco.mcm & mcmSalvageBag)) {
	    	// Found a good valuable salvage item
		if (LootDebug) Log.OutD("Loot_WhatType: Good valuable salvage found");
		return lootTypeValSalv;
	    } // endif
	} // endif
	if (LootDebug) Log.OutD("Loot_WhatType: Not val salvagable.");
    } // endif

    // Item failed all previous tests.  For last test we see if it is a valuable item
    if (this.TypeInfo[lootTypeValuable].Enabled) {
	if (LootDebug) Log.OutD("Loot_WhatType: Checking value...");
	// See if item above value minimum, below burden maximum, and is not a bag of salvage or pyreals
	if ((Value >= this.ValuableFilter.MinValue) && (Burden <= this.ValuableFilter.MaxBU) &&
	    !(ItemAco.mcm & (mcmSalvageBag|mcmPyreal))) {
	    // Found valuable loot
	    if (LootDebug) Log.OutD("Loot_WhatType: Expensive loot found");
	    return lootTypeValuable;
	} else {
	    if (LootDebug) Log.OutD("Loot_WhatType: Not valuable or too heavy");
	} // endif
    } // endif

    // If we make it here, then item is worthless to us.  Return junk item type.
    return lootTypeJunk;
} // end function

// LOOT METHODS ***********************************************************

// Look for an OID on a loot list
function Loot_LookupLoot(Oid, Type) {
    if ((Type==undefined) || (Type==null)) Type = -1;

    // If type specified, then only search that type, else search all lists.
    if (Type < 0) {
	Begin = 0;
	End = lootTypeMax;
    } else {
    	Begin = Type;
    	End = Type;
    } // endif

    for (var x=Begin; x<=End; x++) {
    	with (this.TypeInfo[x]) {
	    for (var y=0; y<ItemCount; y++) {
		if (Oid == ItemList[y].Oid) {
		    this.TypeIndex = x;
		    this.ItemIndex = y;
		    return true;
		} // endif
	    } // end for
	} // end with
    } // end for
    return false;
} // end func

// Add an item to a loot list
function Loot_AddLoot(Aco, Type, Desc) {
    if (LootDebug) Log.OutD("Loot_AddLoot: Begin, Name [" + Aco.szName + "] Type [" + Type +
		 	    "] Desc [" + Desc + "]", LootDebugScreen);

    if (this.LookupLoot(Aco.oid, Type)) {
    	if (LootDebug) Log.OutD("Loot_AddLoot: Duplicate", LootDebugScreen);
    	return false; 	// Item already on list
    } // endif

    with (this.TypeInfo[Type]) {
    	ItemList[ItemCount] = new LootItemRec(Aco, Desc);
    	ItemCount++;
    	TotalValue += Aco.cpyValue;
    } // end with
    return true;
} // end func

// Remove an item from a loot list
function Loot_RemoveLoot(Oid) {
    if (!(this.LookupLoot(Oid, Type))) return false;	// Item not found

    // Decrement list count and move the last item to the deleted items
    // spot unless it was at the end of the list
    with (this.TypeInfo[this.TypeIndex]) {
	ItemCount--;
	if (this.ItemIndex < ItemCount) ItemList[this.ItemIndex] = ItemList[ItemCount];
    } // end with
    return true;
} // end func

// Refresh loot lists due to a logoff/login
function Loot_RefreshLoot() {
    for (var x=0; x<lootTypeTotal; x++) {
    	with (this.TypeInfo[x]) {
	    var y=0;
	    while (y<ItemCount) {
	    	NewAco = skapi.AcoFromOid(ItemList[y].Oid);
	    	if (NewAco && NewAco.fInInventory) {
		    ItemList[y].Aco = NewAco;
		    y++;
		} else {
		    ItemCount--;
		    if (y<ItemCount) ItemList[y] = ItemList[ItemCount];
		} // endif
	    } // end while
	} // end with
    } // end for
} // end func

function Loot_ResetLoot() {
  // First clear out loot counters
    for (var x=0; x<lootTypeTotal; x++) {
	this.TypeInfo[x].ItemCount = 0;
	this.TypeInfo[x].TotalValue = 0;
    } // endfor
} // end func

// ITEM METHODS ***********************************************************

function Loot_EnableItem(Index) {
    if (LootDebug) Log.OutD("Loot_EnableItem: Begin, Index [" + Index + "]", LootDebugScreen);
    this.ItemFilter[Index].Enabled = true;
    if (LootDebug) Log.OutD("Loot_EnableItem: End", LootDebugScreen);
} // end func

function Loot_DisableItem(Index) {
    if (LootDebug) Log.OutD("Loot_DisableItem: Begin, Index [" + Index + "]", LootDebugScreen);
    this.ItemFilter[Index].Enabled = false;
    if (LootDebug) Log.OutD("Loot_DisableItem: End", LootDebugScreen);
} // end func

function Loot_SetItem(Name, Enabled, Alert) {
    if (LootDebug) Log.OutD("Loot_SetItem: Begin, Name [" + Name + "]", LootDebugScreen);
    var Index = this.LookupItem(Name);
    if (Index < 0) return false;
    this.ItemFilter[Index].Enabled = Enabled;
    this.ItemFilter[Index].Alert   = Alert;
    if (LootDebug) Log.OutD("Loot_SetItem: End", LootDebugScreen);
    return true;
} // end func

function Loot_ModItem(Index, Alert) {
    if (LootDebug) Log.OutD("Loot_ModItem: Begin, Index [" + Index + "]", LootDebugScreen);
    this.ItemFilter[Index].Alert = Alert;
    if (LootDebug) Log.OutD("Loot_ModItem: End", LootDebugScreen);
    return true;
} // end func

function Loot_LookupItem(Name) {
    if (LootDebug2) Log.OutD("Loot_LookupItem: Begin, Name [" + Name + "]", LootDebugScreen);
    for (var x=0; x<this.ItemFilter.length; x++) if (this.ItemFilter[x].Name == Name) {
    	if (LootDebug) Log.OutD("Loot_LookupItem: End, Found [" + x + "]", LootDebugScreen);
    	return x;
    } // endfor
    if (LootDebug2) Log.OutD("Loot_LookupItem: End, not found", LootDebugScreen);
    return -1;	  // not found
} // end func

function Loot_ShowItem(EnableOnly) {
    if ((EnableOnly==undefined) || (EnableOnly==null)) EnableOnly = false;
    if (LootDebug) Log.OutD("Loot_Showitem: Begin, EnableOnly [" + EnableOnly + "]", LootDebugScreen);

    var Header = false;
    for (var x=0; x<this.ItemFilter.length; x++) {
	with (this.ItemFilter[x]) {
	    if (EnableOnly && !Enabled && !Alert) continue;
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width("Item",4.0) + Util.Width("Enabled",1.0) + "Alert");
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(Name,4.0) + Util.Width(Enabled + " ",1.0) + Alert);
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowItem: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // endfunc

function Loot_ResetItem() {
    if (LootDebug) Log.OutD("Loot_ResetItem: Begin", LootDebugScreen);
    for (var x=0; x<this.ItemFilter.length; x++) {
	this.ItemFilter[x].Enabled = false;
	this.ItemFilter[x].Alert   = false;
    } // end for
    if (LootDebug) Log.OutD("Loot_ResetItem: End", LootDebugScreen);
} // end func

// MAJOR METHODS ***********************************************************

function Loot_EnableMajor(Index) {
    if (LootDebug) Log.OutD("Loot_EnableMajor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.MajorFilter[Index] = true;
    if (LootDebug) Log.OutD("Loot_EnableMajor: End", LootDebugScreen);
} // end func

function Loot_DisableMajor(Index) {
    if (LootDebug) Log.OutD("Loot_DisableMajor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.MajorFilter[Index] = false;
    if (LootDebug) Log.OutD("Loot_DisableMajor: End", LootDebugScreen);
} // end func

function Loot_SetMajor(Name, Enabled) {
    if (LootDebug) Log.OutD("Loot_SetMajor: Begin, Name [" + Name + "]", LootDebugScreen);
    var SpellId = skapi.SpellidFromSz(Name);
    if (this.MajorIndex[SpellId] == undefined) return false;
    var Index = this.MajorIndex[SpellId];
    this.MajorFilter[Index].Enabled = Enabled;
    if (LootDebug) Log.OutD("Loot_MajorItem: End", LootDebugScreen);
    return true;
} // end func

function Loot_ShowMajor(EnableOnly) {
    if ((EnableOnly==undefined) || (EnableOnly==null)) EnableOnly = false;
    if (LootDebug) Log.OutD("Loot_ShowMajor: Begin, EnableOnly [" + EnableOnly + "]", LootDebugScreen);

    var Header = false;
    for (var x=0; x<this.MajorFilter.length; x++) {
    	// Show major type
	with (this.MajorFilter[x]) {
	    if (EnableOnly && !Enabled) continue;
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width("Major",4.5) + Util.Width("Enabled",1.0));
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(Name,4.5) + Util.Width(Enabled + " ",1.0));
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowMajor: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // endfunc

function Loot_ResetMajor() {
    if (LootDebug) Log.OutD("Loot_ResetMajor: Begin", LootDebugScreen);
    for (var x=0; x<this.MajorFilter.length; x++) this.MajorFilter[x].Enabled = true;
    if (LootDebug) Log.OutD("Loot_ResetMajor: End", LootDebugScreen);
} // end func


// MINOR METHODS ***********************************************************

function Loot_EnableMinor(Index) {
    if (LootDebug) Log.OutD("Loot_EnableMinor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.MinorFilter[Index] = true;
    if (LootDebug) Log.OutD("Loot_EnableMinor: End", LootDebugScreen);
} // end func

function Loot_DisableMinor(Index) {
    if (LootDebug) Log.OutD("Loot_DisableMinor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.MinorFilter[Index] = false;
    if (LootDebug) Log.OutD("Loot_DisableMinor: End", LootDebugScreen);
} // end func

function Loot_SetMinor(Name, Enabled, Eqm) {
    if (LootDebug) Log.OutD("Loot_SetMinor: Begin, Name [" + Name + "]", LootDebugScreen);
    var SpellId = skapi.SpellidFromSz(Name);
    if (this.MinorIndex[SpellId] == undefined) return false;
    var Index = this.MinorIndex[SpellId];
    this.MinorFilter[Index].Enabled = Enabled;
    this.MinorFilter[Index].Eqm	    = Eqm;
    if (LootDebug) Log.OutD("Loot_SetMinor: End", LootDebugScreen);
    return true;
} // end func

function Loot_ModMinor(Index, Eqm) {
    if (LootDebug) Log.OutD("Loot_ModMinor: Begin, Index [" + Index + "]", LootDebugScreen);
    if ((Eqm==undefined) || (Eqm==null)) Eqm = -1;
    if (Eqm >= 0) this.MinorFilter[Index].Eqm = Eqm;
    if (LootDebug) Log.OutD("Loot_ModMinor: End", LootDebugScreen);
} // end func

function Loot_ShowMinor(Index) {
    if ((Index==undefined) || (Index==null)) Index = -1;
    if (LootDebug) Log.OutD("Loot_ShowMajor: Begin, Index [" + Index + "]", LootDebugScreen);

    if (Index<0) {
	var Begin = 0;
	var End   = this.MinorFilter.length-1;
    } else {
    	var Begin = Index;
    	var End   = Index;
    } // endif

    var Header = false;
    for (var x=Begin; x<=End; x++) {
    	// Show major type
	with (this.MinorFilter[x]) {
	    if (Enabled) {
		if (!Header) {
		    Header = true;
		    Chat.Screen(" ");
		    Chat.Screen(Util.Width("Minor",4.5) + Util.Width("Enabled",1.0) + "Eqm");
		} // endif

		// Print info. First convert eqm to test
		if (Eqm) {
		    EqmText = "";
		    if (Eqm & eqmHead)		  EqmText = EqmText + "hd  ";
		    if (Eqm & eqmHands)		  EqmText = EqmText + "ha  ";
		    if (Eqm & eqmFeet)		  EqmText = EqmText + "ft  ";
		    if (Eqm & eqmChestOuter)	  EqmText = EqmText + "ch  ";
		    if (Eqm & eqmUpperArmsOuter)  EqmText = EqmText + "ua  ";
		    if (Eqm & eqmLowerArmsOuter)  EqmText = EqmText + "la  ";
		    if (Eqm & eqmAbdomenOuter)	  EqmText = EqmText + "ab  ";
		    if (Eqm & eqmUpperLegsOuter)  EqmText = EqmText + "ul  ";
		    if (Eqm & eqmLowerLegsOuter)  EqmText = EqmText + "ll  ";
		} else {
		    EqmText = "None";
		} // endif

		Chat.Screen(Util.Width(Name,4.5) + Util.Width(Enabled + " ",1.0) + EqmText);
	    } // endif
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowMinor: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // endfunc

function Loot_ResetMinor() {
    if (LootDebug) Log.OutD("Loot_ResetMinor: Begin", LootDebugScreen);

    for (var x=0; x<this.MinorFilter.length; x++) {
    	this.MinorFilter[x].Enabled = false;
    	this.MinorFilter[x].Eqm     = 0;
    } // endfor

    // Magic skills
    this.SetMinor("Minor Creature Enchantment Aptitude", true,	armorExtremity	);
    // this.SetMinor("Minor Item Enchantment Aptitude",	 false,	0		);
    this.SetMinor("Minor Life Magic Aptitude",		 true,	armorExtremity	);
    this.SetMinor("Minor War Magic Aptitude",		 true,	armorExtremity	);
    // this.SetMinor("Minor Magic Resistance",		 false,	0		);
    // this.SetMinor("Minor Mana Conversion Prowess",	 false,	0		);

    // Melee skills
    this.SetMinor("Minor Axe Aptitude",		false,	armorExtremity	);
    this.SetMinor("Minor Bow Aptitude",		true,	armorExtremity	);
    this.SetMinor("Minor Crossbow Aptitude",	true,	armorExtremity	);
    // this.SetMinor("Minor Dagger Aptitude",	false,	0		);
    // this.SetMinor("Minor Mace Aptitude",	false,	0		);
    // this.SetMinor("Minor Spear Aptitude",	false,	0		);
    // this.SetMinor("Minor Staff Aptitude",	false,	0		);
    this.SetMinor("Minor Sword Aptitude",	true,	armorExtremity	);
    // this.SetMinor("Minor Thrown Aptitude",	false,	0		);
    this.SetMinor("Minor Unarmed Aptitude",	true,	armorExtremity	);
    // this.SetMinor("Minor Impregnability",	false,	0		);
    // this.SetMinor("Minor Invulnerability",	false,	0		);

    // this.SetMinor("Minor Armor Tinkering Expertise",	  false,	0		);
    // this.SetMinor("Minor Item Tinkering Expertise",	  false,	0		);
    // this.SetMinor("Minor Magic Item Tinkering Expertise", false,	0		);
    // this.SetMinor("Minor Weapon Tinkering Expertise",  false,	0		);
    // this.SetMinor("Minor Alchemical Prowess",	  false,	0		);
    // this.SetMinor("Minor Cooking Prowess",		  false,	0		);
    // this.SetMinor("Minor Fletching Prowess",		  false,	0		);

    // Miscellaneous
    // this.SetMinor("Minor Arcane Prowess",	false,	0	);
    // this.SetMinor("Minor Deception Prowess",	false,	0	);
    // this.SetMinor("Minor Fealty",		false,	0	);
    // this.SetMinor("Minor Healing Prowess",	false,	0	);
    // this.SetMinor("Minor Jumping Prowess",	false,	0	);
    // this.SetMinor("Minor Leadership",	false,	0	);
    // this.SetMinor("Minor Lockpick Prowess",	false,	0	);
    // this.SetMinor("Minor Monster Attunement",false,	0	);
    // this.SetMinor("Minor Person Attunement",	false,	0	);
    // this.SetMinor("Minor Sprint",		false,	0	);

    // Weapon modifiers.  These don't buy much by themselves.  A good weapon will be
    // picked up regardless, so disable these by default.
    // this.SetMinor("Minor Blood Thirst",	false,	0	);
    // this.SetMinor("Minor Defender",		false,	0	);
    // this.SetMinor("Minor Heart Thirst",	false,	0	);
    // this.SetMinor("Minor Swift Hunter",	false,	0	);

    // Wards and banes.  Mostly useless
    // this.SetMinor("Minor Armor", 		false,	0	);
    // this.SetMinor("Minor Acid Ward",		false,	0	);
    // this.SetMinor("Minor Bludgeoning Ward",	false,	0	);
    // this.SetMinor("Minor Flame Ward",	false,	0	);
    // this.SetMinor("Minor Frost Ward",	false,	0	);
    // this.SetMinor("Minor Piercing Ward",	false,	0	);
    // this.SetMinor("Minor Slashing Ward",	false,	0	);
    // this.SetMinor("Minor Storm Ward",	false,	0	);

    // this.SetMinor("Minor Impenetrability",	false,	0	);
    // this.SetMinor("Minor Acid Bane",		false,	0	);
    // this.SetMinor("Minor Bludgeoning Bane",	false,	0	);
    // this.SetMinor("Minor Flame Bane",	false,	0	);
    // this.SetMinor("Minor Frost Bane",	false,	0	);
    // this.SetMinor("Minor Piercing Bane",	false,	0	);
    // this.SetMinor("Minor Slashing Bane",	false,	0	);
    // this.SetMinor("Minor Storm Bane",	false,	0	);

    if (LootDebug) Log.OutD("Loot_ResetMinor: End", LootDebugScreen);
} // end func


// ARMOR METHODS ***********************************************************

function Loot_EnableArmor(Index) {
    if (LootDebug) Log.OutD("Loot_EnableArmor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.ArmorFilter[Index].Enabled = true;
    if (LootDebug) Log.OutD("Loot_EnableArmor: End", LootDebugScreen);
} // end func

function Loot_DisableArmor(Index) {
    if (LootDebug) Log.OutD("Loot_DisableArmor: Begin, Index [" + Index + "]", LootDebugScreen);
    this.ArmorFilter[Index].Enabled = false;
    if (LootDebug) Log.OutD("Loot_DisableArmor: End", LootDebugScreen);
} // end func

function Loot_AddArmor(Name, Eqm, CovFlag) {
    if (LootDebug) Log.OutD("Loot_AddArmor: Begin, Name [" + Name + "]", LootDebugScreen);
    if (this.LookupArmor(Name) >= 0) return false;	// Item already exists
    var NextSlot = this.ArmorFilter.length;
    this.ArmorFilter[NextSlot] = new ArmorFilterRec(Name, Eqm, CovFlag);
    this.ArmorOpts = this.ArmorOpts + " <option data='" + NextSlot + "' text='" + Name + "'/>\n";
    if (LootDebug) Log.OutD("Loot_AddArmor: End", LootDebugScreen);
    return true;
} // end func

function Loot_SetArmor(Name, Enabled, MinAL, MaxWork, MaxBU, MaxValue, MinSpells, MinSplLvl) {
    if (LootDebug) Log.OutD("Loot_SetArmor: Begin, Name [" + Name + "]", LootDebugScreen);
    var Index = this.LookupArmor(Name);
    if (Index < 0) return false;
    this.ArmorFilter[Index].Enabled	= Enabled;
    this.ArmorFilter[Index].MinAL	= MinAL;
    this.ArmorFilter[Index].MaxWork	= MaxWork;
    this.ArmorFilter[Index].MaxBU	= MaxBU;
    this.ArmorFilter[Index].MaxValue	= MaxValue;
    this.ArmorFilter[Index].MinSpells	= MinSpells;
    this.ArmorFilter[Index].MinSplLvl	= MinSplLvl;
    if (LootDebug) Log.OutD("Loot_SetArmor: End", LootDebugScreen);
    return true;
} // end func

function Loot_ModArmor(Index, MinAL, MaxWork, MaxBU, MaxValue, MinSpells, MinSplLvl) {
    if (LootDebug) Log.OutD("Loot_ModArmor: Begin, Index [" + Index + "]", LootDebugScreen);
    if ((MinAL     == undefined) || (MinAL     == null)) MinAL     = -1;
    if ((MaxWork   == undefined) || (MaxWork   == null)) MaxWork   = -1;
    if ((MaxBU     == undefined) || (MaxBU     == null)) MaxBU     = -1;
    if ((MaxValue  == undefined) || (MaxValue  == null)) MaxValue  = -1;
    if ((MinSpells == undefined) || (MinSpells == null)) MinSpells = -1;
    if ((MinSplLvl == undefined) || (MinSplLvl == null)) MinSplLvl = -1;

    if (MinAL     >= 0) this.ArmorFilter[Index].MinAL	   = MinAL;
    if (MaxWork   >= 0) this.ArmorFilter[Index].MaxWork	   = MaxWork;
    if (MaxBU     >= 0) this.ArmorFilter[Index].MaxBU	   = MaxBU;
    if (MaxValue  >= 0) this.ArmorFilter[Index].MaxValue   = MaxValue;
    if (MinSpells >= 0) this.ArmorFilter[Index].MinSpells  = MinSpells;
    if (MinSplLvl >= 0) this.ArmorFilter[Index].MinSplLvl  = MinSplLvl;
    if (LootDebug) Log.OutD("Loot_ModArmor: End", LootDebugScreen);
    return true;
} // end func

function Loot_ShowArmor(Index) {
    if ((Index==undefined) || (Index==null)) Index = -1;
    if (LootDebug) Log.OutD("Loot_ShowArmor: Begin, Index [" + Index + "]", LootDebugScreen);

    if (Index<0) {
	var Begin = 0;
	var End   = this.ArmorFilter.length-1;
    } else {
    	var Begin = Index;
    	var End   = Index;
    } // endif

    var Header = false;
    for (var x=Begin; x<=End; x++) {
	with (this.ArmorFilter[x]) {
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width(" ",3.2) + Util.Width("Min",0.8) + Util.Width("Max",0.8) +
			    Util.Width("Max",0.8) +  Util.Width("Max",0.8) + Util.Width("Min",0.8) + "MinSpl");
		Chat.Screen(Util.Width("Armor",2.0) + Util.Width("Enabled",1.2) + Util.Width("AL",0.8) +
			    Util.Width("BU",0.8) + Util.Width("Val",0.8) + Util.Width("Work",0.8) +
			    Util.Width("Spl",0.8) + "Lvl");
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(Name,2.2) + Util.Width(Enabled+" ",1.0) + Util.Width(MinAL+" ",0.8) +
			Util.Width(MaxBU+" ",0.8) + Util.Width(MaxValue+ " ",0.8) + Util.Width(MaxWork+" ",0.8) +
			Util.Width(MinSpells+" ",0.8) + MinSplLvl);
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowArmor: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // endfunc

function Loot_ResetArmor() {
    if (LootDebug) Log.OutD("Loot_ResetArmor: Begin", LootDebugScreen);
    // SetArmor(  Name,               Enabled, MinAL, MaxWork, MaxBU, MaxValue, MinSpells, MinSplLvl)
    this.SetArmor("Armor (Body)",	true,	225,	 5,	4000,	40000,	0,	6);
    this.SetArmor("Armor (Head)",	true,	295,	 5,	1000,	20000,	0,	6);
    this.SetArmor("Armor (Hands)",	true,	225,	 5,	1000,	20000,	0,	6);
    this.SetArmor("Armor (Feet)",	true,	225,	 5,	1000,	20000,	0,	6);
    this.SetArmor("Armor (Shield)",	true,	160,	 5,	2000,	20000,	0,	6);
    this.SetArmor("Cov Armor (Body)",	true,	350,	10,	4000,	40000,	0,	6);
    this.SetArmor("Cov Armor (Head)",	true,	350,	10,	1000,	20000,	0,	6);
    this.SetArmor("Cov Armor (Hands)",	true,	350,	10,	1000,	20000,	0,	6);
    this.SetArmor("Cov Armor (Feet)",	true,	350,	10,	1000,	20000,	0,	6);
    this.SetArmor("Cov Armor (Shield)",	true,	350,	10,	2000,	20000,	0,	6);
    if (LootDebug) Log.OutD("Loot_ResetArmor: End", LootDebugScreen);
} // end func

function Loot_LookupArmor(Name) {
    if (LootDebug2) Log.OutD("Loot_LookupArmor: Begin, Name [" + Name + "]", LootDebugScreen);
    for (var x=0; x<this.ArmorFilter.length; x++) if (this.ArmorFilter[x].Name == Name) {
   	if (LootDebug) Log.OutD("Loot_LookupArmor: End, Found [" + x + "]", LootDebugScreen);
    	return x;
    } // endfor
    if (LootDebug2) Log.OutD("Loot_LookupArmor: End, not found", LootDebugScreen);
    return -1;   // not found
} // end func


// WEAPON METHODS ***********************************************************

// Retired function that detects old weapon config function
function Loot_SetWeapon(Name, Enabled, MinDam, MinAtt, MinDef, MaxWork, MaxBU, MaxValue, MinSpells, MinSplLvl) {
    if (LootDebug) Log.OutD("Loot_SetWeapon: Begin, Name [" + Name + "] Enabled [" + Enabled +
    			    "] MinDam [" + MinDam + "]", LootDebugScreen);

    if (!this.OldWeaponAlert) {
	this.OldWeaponAlert = true;
	Chat.Screen("%Warning!!! Old weapon info found in loot config file.", cmcYellow);
	Chat.Screen("%Please redo melee and/or ranged weapon loot profiles through the decal panels.", cmcYellow);
    } /* endif */
} /* end function */

// Melee Weapons
function Loot_EnableWeaponM(Index) {
    if (LootDebug) Log.OutD("Loot_EnableWeaponM: Begin, Index [" + Index + "]", LootDebugScreen);
    this.WeaponMFilter[Index].Enabled = true;
    if (LootDebug) Log.OutD("Loot_EnableWeaponM: End", LootDebugScreen);
} // end func

function Loot_DisableWeaponM(Index) {
    if (LootDebug) Log.OutD("Loot_DisableWeaponM: Begin, Index [" + Index + "]", LootDebugScreen);
    this.WeaponMFilter[Index].Enabled = false;
    if (LootDebug) Log.OutD("Loot_DisableWeaponM: End", LootDebugScreen);
} // end func


function Loot_AddWeaponM(Name, Skid, Wield, AbsDamMax, AbsAttMax, AbsDefMax) {
    if (LootDebug) Log.OutD("Loot_AddWeaponM: Begin, Name [" + Name + "]", LootDebugScreen);
    if (this.LookupWeaponM(Name) >= 0) return false;	// Item already exists
    var NextSlot = this.WeaponMFilter.length;
    this.WeaponMFilter[NextSlot] = new WeaponMFilterRec(Name, Skid, Wield, AbsDamMax, AbsAttMax, AbsDefMax);
    this.WeaponMOpts = this.WeaponMOpts + " <option data='" + NextSlot + "' text='" + Name + "'/>\n";
    if (LootDebug) Log.OutD("Loot_AddWeaponM: End", LootDebugScreen);
    return true;
} // end func


function Loot_SetWeaponM(Name, Enabled, MaxWork, MinDam, MinAtt, MinDef, AndOrFlag ) {
    if (LootDebug) Log.OutD("Loot_SetWeaponM: Begin, Name [" + Name + "] Enabled [" + Enabled +
    			    "] MinDam [" + MinDam + "]", LootDebugScreen);
    var Index = this.LookupWeaponM(Name);
    if (Index < 0) {
    	Chat.Screen("%Config error, unknown weapon [" + Name + "]");
	return false;
    } // endif

    this.WeaponMFilter[Index].Enabled	= Enabled;
    this.WeaponMFilter[Index].MaxWork	= MaxWork;
    this.WeaponMFilter[Index].MinDam	= MinDam;
    this.WeaponMFilter[Index].MinAtt	= MinAtt;
    this.WeaponMFilter[Index].MinDef	= MinDef;
    this.WeaponMFilter[Index].AndOrFlag	= AndOrFlag;

    if (LootDebug) Log.OutD("Loot_SetWeaponM: End", LootDebugScreen);
    return true;
} //  end function

function Loot_ModWeaponM(Index, MaxWork, MinDam, MinAtt, MinDef, AndOrFlag) {
    if (LootDebug) Log.OutD("Loot_ModWeaponM: Begin, Index [" + Index + "]", LootDebugScreen);

    if ((MaxWork   == undefined) || (MaxWork   == null)) MaxWork   = -1;
    if ((MinDam    == undefined) || (MinDam    == null)) MinDam    = -1;
    if ((MinAtt    == undefined) || (MinAtt    == null)) MinAtt    = -1;
    if ((MinDef    == undefined) || (MinDef    == null)) MinDef    = -1;
    if ((AndOrFlag == undefined) || (AndOrFlag == null)) AndOrFlag = -1;

    if (MaxWork   >=0) this.WeaponMFilter[Index].MaxWork   = MaxWork;
    if (MinDam    >=0) this.WeaponMFilter[Index].MinDam    = MinDam;
    if (MinAtt    >=0) this.WeaponMFilter[Index].MinAtt    = MinAtt;
    if (MinDef    >=0) this.WeaponMFilter[Index].MinDef    = MinDef;
    if (AndOrFlag >=0) this.WeaponMFilter[Index].AndOrFlag = AndOrFlag;

    if (LootDebug) Log.OutD("Loot_ModWeaponM: End", LootDebugScreen);
} // end function

function Loot_ShowWeaponM(All) {
    if ((All == undefined) || (All == null)) All = false;
    if (LootDebug) Log.OutD("Loot_ShowWeaponM: Begin [" + All + "]", LootDebugScreen);

    var Header = false;
    for (var x=0; x<this.WeaponMFilter.length; x++) {
	with (this.WeaponMFilter[x]) {
 	    if (!All && !Enabled) continue;
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width(" ",1.8) + 
			    Util.Width("Max",0.7) +
			    Util.Width("Max",0.7) +
			    Util.Width("Min",0.6) +
			    Util.Width("Min",0.6) +
			    Util.Width("Min",0.6) +
			    "AndOr" );

		Chat.Screen(Util.Width("Weapon",1.8) +
			    Util.Width("Work",0.7) +
			    Util.Width("Dam",0.7) +
			    Util.Width("Dam",0.6) +
			    Util.Width("Att",0.6) +
			    Util.Width("Def",0.6) +
			    "Flag" );
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(Name,1.8) +
	    		Util.Width(MaxWork+" ",0.7) +
	    		Util.Width(AbsDamMax+" ",0.7) +
	    		Util.Width(MinDam+" ",0.6) +
	    		Util.Width(MinAtt+" ",0.6) +
	    		Util.Width(MinDef+" ",0.6) +
	    		andorFlagText[AndOrFlag]);
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowWeaponM: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // end func

function Loot_ResetWeaponM(skid) {
    if ((skid == undefined) || (skid == null)) skid = -1;
    if (LootDebug) Log.OutD("Loot_ResetWeaponM: Begin", LootDebugScreen);
 
    //					       Max Min   Min Min AndOr
    //                                  Enable Wrk Dam   Att Def Flag

    if ((skid == -1) || (skid == skidSword)) {
	this.SetWeaponM("Sword - 400",      true,  8,  43,   5,  5,  1 );	// 55 - 15 - 15
	this.SetWeaponM("Sword - 370",      true,  7,  43,   5,  5,  1 );	// 50 - 15 - 15
	this.SetWeaponM("Sword - 350",      true,  7,  43,   5,  5,  1 );	// 45 - 15 - 15
	this.SetWeaponM("Sword - 325",      false, 7,  38,   5,  5,  1 );	// 40 - 15 - 15
	this.SetWeaponM("Sword - 300",      false, 7,  33,   5,  5,  1 );	// 35 - 15 - 15
	this.SetWeaponM("Sword - 250",      false, 7,  28,   5,  5,  1 );	// 30 - 15 - 15
	this.SetWeaponM("Sword - 0",        false, 7,  18,   3,  3,  1 );	// 20 - 11 - 11
    } // endif

    if ((skid == -1) || (skid == skidUnarmedCombat)) {
	this.SetWeaponM("UA - 400",         true,  8,  20,   5,  5,  1 );	// 26 - 15 - 20
	this.SetWeaponM("UA - 370",         true,  7,  20,   5,  5,  1 );	// 22 - 15 - 20
	this.SetWeaponM("UA - 350",         false, 7,  18,   5,  5,  1 );	// 20 - 15 - 20
	this.SetWeaponM("UA - 325",         false, 7,  14,   5,  5,  1 );	// 16 - 15 - 20
	this.SetWeaponM("UA - 300",         false, 7,  11,   5,  5,  1 );	// 13 - 15 - 20
	this.SetWeaponM("UA - 250",         false, 7,   9,   5,  5,  1 );	// 11 - 15 - 20
	this.SetWeaponM("UA - 0",           false, 7,   6,   3,  3,  1 );	//  8 -  7 - 12
    } // endif

    if ((skid == -1) || (skid == skidAxe )) {
	this.SetWeaponM("Axe - 400",        true,  8,  40,   5,  5,  1 );   // 46 - 15 - 15
	this.SetWeaponM("Axe - 370",        true,  7,  40,   5,  5,  1 );   // 42 - 15 - 15
	this.SetWeaponM("Axe - 350",        false, 7,  36,   5,  5,  1 );   // 38 - 15 - 15
	this.SetWeaponM("Axe - 325",        false, 7,  34,   5,  5,  1 );   // 36 - 15 - 15
	this.SetWeaponM("Axe - 300",        false, 7,  30,   5,  5,  1 );   // 32 - 15 - 15
	this.SetWeaponM("Axe - 250",        false, 7,  26,   5,  5,  1 );   // 28 - 15 - 15
	this.SetWeaponM("Axe - 0",          false, 7,  17,   3,  3,  1 );   // 19 -  7 -  7
    } // end func

    if ((skid == -1) || (skid == skidDagger)) {
	this.SetWeaponM("Dagger - 400",     true,  7,  22,   5,  5,  1 );   // 26 - 15 - 15
	this.SetWeaponM("Dagger - 370",     true,  7,  22,   5,  5,  1 );   // 24 - 15 - 15
	this.SetWeaponM("Dagger - 350",     false, 7,  18,   5,  5,  1 );   // 20 - 15 - 15
	this.SetWeaponM("Dagger - 325",     false, 7,  16,   5,  5,  1 );   // 18 - 15 - 15
	this.SetWeaponM("Dagger - 300",     false, 7,  14,   5,  5,  1 );   // 16 - 15 - 15
	this.SetWeaponM("Dagger - 250",     false, 7,  12,   5,  5,  1 );   // 14 - 15 - 15
	this.SetWeaponM("Dagger - 0",       false, 7,  10,   5,  5,  1 );   // 12 - 11 - 11
    } // end func

    if ((skid == -1) || (skid == skidMace)) {
	this.SetWeaponM("Mace - 400",       true,  8,  36,   5,  5,  1 );	// 42 - 15 - 20
	this.SetWeaponM("Mace - 370",       true,  7,  36,   5,  5,  1 );	// 38 - 15 - 20
	this.SetWeaponM("Mace - 350",       false, 7,  34,   5,  5,  1 );	// 36 - 15 - 20
	this.SetWeaponM("Mace - 325",       false, 7,  32,   5,  5,  1 );	// 34 - 15 - 20
	this.SetWeaponM("Mace - 300",       false, 7,  28,   5,  5,  1 );	// 30 - 15 - 20
	this.SetWeaponM("Mace - 250",       false, 7,  24,   5,  5,  1 );	// 26 - 15 - 20
	this.SetWeaponM("Mace - 0",         false, 7,  16,   5,  5,  1 );	// 18 -  7 - 12
    } // end func

    if ((skid == -1) || (skid == skidSpear)) {
	this.SetWeaponM("Spear - 400",      true,  8,  34,   5,  5,  1 );   // 40 - 15 - 15
	this.SetWeaponM("Spear - 370",      true,  7,  34,   5,  5,  1 );   // 36 - 15 - 15
	this.SetWeaponM("Spear - 350",      false, 7,  30,   5,  5,  1 );   // 32 - 15 - 15
	this.SetWeaponM("Spear - 325",      false, 7,  26,   5,  5,  1 );   // 28 - 15 - 15
	this.SetWeaponM("Spear - 300",      false, 7,  24,   5,  5,  1 );   // 26 - 15 - 15
	this.SetWeaponM("Spear - 250",      false, 7,  22,   5,  5,  1 );   // 24 - 15 - 15
	this.SetWeaponM("Spear - 0",        false, 7,  15,   5,  5,  1 );   // 17 -  7 -  7
    } // end func

    if ((skid == -1) || (skid == skidStaff)) {
	this.SetWeaponM("Staff - 400",      true,  8,  22,   5,  5,  1 );   // 26 - 15 - 15
	this.SetWeaponM("Staff - 370",      true,  7,  22,   5,  5,  1 );   // 24 - 15 - 15
	this.SetWeaponM("Staff - 350",      false, 7,  18,   5,  5,  1 );   // 20 - 15 - 15
	this.SetWeaponM("Staff - 325",      false, 7,  16,   5,  5,  1 );   // 18 - 15 - 15
	this.SetWeaponM("Staff - 300",      false, 7,  14,   5,  5,  1 );   // 16 - 15 - 15
	this.SetWeaponM("Staff - 250",      false, 7,  12,   5,  5,  1 );   // 14 - 15 - 15
	this.SetWeaponM("Staff - 0",        false, 7,  10,   5,  5,  1 );   // 12 -  7 -  7
    } // end func

    if (LootDebug) Log.OutD("Loot_ResetWeaponM: End", LootDebugScreen);
} // end func

function Loot_LookupWeaponM(Name) {
    if (LootDebug2) Log.OutD("Loot_LookupWeaponM: Begin, Name [" + Name + "]", LootDebugScreen);
    for (var x=0; x<this.WeaponMFilter.length; x++) if (this.WeaponMFilter[x].Name == Name) {
  	if (LootDebug2) Log.OutD("Loot_LookupWeaponM: End, Found [" + x + "]", LootDebugScreen);
    	return x;
    } // endfor
    if (LootDebug2) Log.OutD("Loot_LookupWeaponM: End, not found", LootDebugScreen);
    return -1;	  // not found
} // end func


// Ranged Weapons
function Loot_EnableWeaponR(Index) {
    if (LootDebug) Log.OutD("Loot_EnableWeaponR: Begin, Index [" + Index + "]", LootDebugScreen);
    this.WeaponRFilter[Index].Enabled = true;
    if (LootDebug) Log.OutD("Loot_EnableWeaponR: End", LootDebugScreen);
} // end func

function Loot_DisableWeaponR(Index) {
    if (LootDebug) Log.OutD("Loot_DisableWeaponR: Begin, Index [" + Index + "]", LootDebugScreen);
    this.WeaponRFilter[Index].Enabled = false;
    if (LootDebug) Log.OutD("Loot_DisableWeaponR: End", LootDebugScreen);
} // end func

function Loot_AddWeaponR(Name, Skid, Wield, AbsDamMax, AbsDefMax, AbsElemMax) {
    if (LootDebug) Log.OutD("Loot_AddWeaponR: Begin, Name [" + Name + "]", LootDebugScreen);
    if (this.LookupWeaponR(Name) >= 0) return false;	// Item already exists
    var NextSlot = this.WeaponRFilter.length;
    this.WeaponRFilter[NextSlot] = new WeaponRFilterRec(Name, Skid, Wield, AbsDamMax, AbsDefMax, AbsElemMax);
    this.WeaponROpts = this.WeaponROpts + " <option data='" + NextSlot + "' text='" + Name + "'/>\n";
    if (LootDebug) Log.OutD("Loot_AddWeaponR: End", LootDebugScreen);
    return true;
} // end func

function Loot_SetWeaponR(Name, Enabled, MaxWork, MinDam, MinDef, MinElem, AndOrFlag ) {
    if (LootDebug) Log.OutD("Loot_SetWeaponR: Begin, Name [" + Name + "] Enabled [" + Enabled +
    			    "] MinDam [" + MinDam + "]", LootDebugScreen);
    var Index = this.LookupWeaponR(Name);
    if (Index < 0) {
    	Chat.Screen("%Config error, unknown weapon [" + Name + "]");
	return false;
    } // endif

    this.WeaponRFilter[Index].Enabled	= Enabled;
    this.WeaponRFilter[Index].MaxWork	= MaxWork;
    this.WeaponRFilter[Index].MinDam	= MinDam;
    this.WeaponRFilter[Index].MinDef	= MinDef;
    this.WeaponRFilter[Index].MinElem	= MinElem;
    this.WeaponRFilter[Index].AndOrFlag	= AndOrFlag;

    if (LootDebug) Log.OutD("Loot_SetWeaponR: End", LootDebugScreen);
    return true;
} //  end function

function Loot_ModWeaponR(Index, MaxWork, MinDam, MinDef, MinElem, AndOrFlag) {
    if (LootDebug) Log.OutD("Loot_ModWeaponR: Begin, Index [" + Index + "]", LootDebugScreen);

    if ((MaxWork   == undefined) || (MaxWork   == null)) MaxWork   = -1;
    if ((MinDam    == undefined) || (MinDam    == null)) MinDam    = -1;
    if ((MinDef    == undefined) || (MinDef    == null)) MinDef    = -1;
    if ((MinElem   == undefined) || (MinElem   == null)) MinElem   = -1;
    if ((AndOrFlag == undefined) || (AndOrFlag == null)) AndOrFlag = -1;

    if (MaxWork   >=0) this.WeaponRFilter[Index].MaxWork   = MaxWork;
    if (MinDam    >=0) this.WeaponRFilter[Index].MinDam    = MinDam;
    if (MinDef    >=0) this.WeaponRFilter[Index].MinDef    = MinDef;
    if (MinElem   >=0) this.WeaponRFilter[Index].MinElem   = MinElem;
    if (AndOrFlag >=0) this.WeaponRFilter[Index].AndOrFlag = AndOrFlag;

    if (LootDebug) Log.OutD("Loot_ModWeaponR: End", LootDebugScreen);
} // end function

function Loot_ShowWeaponR() {
    if (LootDebug) Log.OutD("Loot_ShowWeaponR: Begin", LootDebugScreen);

    var Header = false;
    for (var x=0; x<this.WeaponRFilter.length; x++) {
	with (this.WeaponRFilter[x]) {
 	    if (!Enabled) continue;
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width(" ",1.8) +
			    Util.Width("Max",0.7) +
			    Util.Width("Max",0.7) +
			    Util.Width("Min",0.6) +
			    Util.Width("Min",0.6) +
			    Util.Width("Min",0.6) +
			    "AndOr" );

		Chat.Screen(Util.Width("Weapon",1.8) +
			    Util.Width("Work",0.7) +
			    Util.Width("Dam",0.6) +
			    Util.Width("Dam",0.6) +
			    Util.Width("Def",0.6) +
			    Util.Width("Elem",0.6) +
			    "Flag" );
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(Name,1.8) +
			Util.Width(MaxWork+" ",0.7) +
			Util.Width(AbsDamMax+" ",0.6) +
	    		Util.Width(MinDam+" ",0.6) +
	    		Util.Width(MinDef+" ",0.6) +
	    		Util.Width(MinElem+" ",0.6) +
	    		andorFlagText[AndOrFlag]);
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowWeaponR: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // end func

function Loot_ResetWeaponR(skid) {
    if ((skid == undefined) || (skid == null)) skid = -1;
    if (LootDebug) Log.OutD("Loot_ResetWeaponR: Begin", LootDebugScreen);
 
    //					          Max  Min   Min Min  AndOr
    //                                     Enable Wrk  Dam   Def Elem Flag

    if ((skid == -1) || (skid == skidBow)) {
        this.SetWeaponR("Bow - 360",       true,  7,   125,  10,  6,  1  );	// 130 - 15 - 12
        this.SetWeaponR("Bow - 335",       true,  7,   125,  10,  6,  1  );	// 130 - 15 - 8
        this.SetWeaponR("Bow - 315",       true,  7,   125,  10,  0,  0  );	// 130 - 15 - 4
        this.SetWeaponR("Bow - 290",       true,  7,   125,  10,  0,  0  );	// 130 - 15 - 0
        this.SetWeaponR("Bow - 270",       false, 7,   120,  15,  0,  0  );	// 120 - 15 - 0
        this.SetWeaponR("Bow - 250",       false, 7,   110,  12,  0,  0  );	// 110 - 12 - 0
        this.SetWeaponR("Bow - 0",         false, 7,   100,  7,   0,  0  );	// 100 - 7  - 0
    } // endif

    if ((skid == -1) || (skid == skidCrossbow)) {
        this.SetWeaponR("Crossbow - 360",  true,  7,   150,  10,  6,  1  );	// 155 - 15 - 12
        this.SetWeaponR("Crossbow - 335",  true,  7,   150,  10,  6,  1  );	// 155 - 15 - 8
        this.SetWeaponR("Crossbow - 315",  true,  7,   150,  10,  0,  0  );	// 155 - 15 - 4
        this.SetWeaponR("Crossbow - 290",  true,  7,   150,  10,  0,  0  );	// 155 - 15 - 0
        this.SetWeaponR("Crossbow - 270",  false, 7,   145,  10,  0,  0  );	// 145 - 15 - 0
        this.SetWeaponR("Crossbow - 250",  false, 7,   140,  10,  0,  0  );	// 140 - 15 - 0
        this.SetWeaponR("Crossbow - 0",    false, 7,   130,  7,   0,  0  );	// 130 - 7  - 0
    } // endif

    if ((skid == -1) || (skid == skidThrownWeapons)) {
        this.SetWeaponR("Atlatl - 360",    true,  7,   150,  14,  8,  1  );	// 150 - 15 - 12
        this.SetWeaponR("Atlatl - 335",    true,  7,   150,  14,  8,  1  );	// 150 - 15 - 8
        this.SetWeaponR("Atlatl - 315",    true,  7,   150,  14,  0,  0  );	// 150 - 15 - 4
        this.SetWeaponR("Atlatl - 290",    true,  7,   150,  14,  0,  0  );	// 150 - 15 - 0
        this.SetWeaponR("Atlatl - 270",    false, 7,   140,  14,  0,  0  );	// 140 - 15 - 0
        this.SetWeaponR("Atlatl - 250",    false, 7,   130,  14,  0,  0  );	// 130 - 15 - 0
        this.SetWeaponR("Atlatl - 0",      false, 7,   110,  7,   0,  0  );	// 110 - 7  - 0
    } // endif

    if (LootDebug) Log.OutD("Loot_ResetWeaponR: End", LootDebugScreen);
} // end func

function Loot_LookupWeaponR(Name) {
    if (LootDebug2) Log.OutD("Loot_LookupWeaponR: Begin, Name [" + Name + "]", LootDebugScreen);
    for (var x=0; x<this.WeaponRFilter.length; x++) if (this.WeaponRFilter[x].Name == Name) {
  	if (LootDebug2) Log.OutD("Loot_LookupWeaponR: End, Found [" + x + "]", LootDebugScreen);
    	return x;
    } // endfor
    if (LootDebug2) Log.OutD("Loot_LookupWeaponR: End, not found", LootDebugScreen);
    return -1;	  // not found
} // end func


// WAND METHODS ***********************************************************

function Loot_EnableWand(Work) {
    this.WandFilter[Work].Enabled = true;
} // end func

function Loot_DisableWand(Work) {
    this.WandFilter[Work].Enabled = false;
} // end func

// Retired function that detects old weapon config function
function Loot_SetWand(Work, Enabled, MaxValue, MinManaC) {
    if (LootDebug) Log.OutD("Loot_SetWand: Begin, Name [" + Name + "] Enabled [" + Enabled +
    			    "] Work [" + Work + "]", LootDebugScreen);

    if (!this.OldWandAlert) {
	this.OldWandAlert = true;
	Chat.Screen("%Warning!!! Old wand info found in loot config file.", cmcYellow);
	Chat.Screen("%Please redo wand profiles through the decal panels.", cmcYellow);
    } /* endif */
    return true;
} /* end function */

function Loot_SetWand2(Work, Enabled, MaxValue, MinManaC, MinMeleeD, AndOrFlag) {
    if (LootDebug) Log.OutD("Loot_SetWand2: Begin, Name [" + Name + "] Enabled [" + Enabled +
    			    "] Work [" + Work + "]", LootDebugScreen);

    if (!this.OldWandAlert) {
	this.OldWandAlert = true;
	Chat.Screen("%Warning!!! Old wand info found in loot config file.", cmcYellow);
	Chat.Screen("%Please redo wand profiles through the decal panels.", cmcYellow);
    } /* endif */
    return true;
} // end func

function Loot_SetWand3(Work, Enabled, MinManaC, MinMeleeD, MinElem, AndOrFlag, AndOrFlag2) {
    if ((Work<1) || (Work>10)) return false;
    this.WandFilter[Work].Enabled    = Enabled;
    this.WandFilter[Work].MinManaC   = MinManaC;
    this.WandFilter[Work].MinMeleeD  = MinMeleeD;
    this.WandFilter[Work].MinElem    = MinElem;
    this.WandFilter[Work].AndOrFlag  = AndOrFlag;
    this.WandFilter[Work].AndOrFlag2 = AndOrFlag2;
    return true;
} // end func

function Loot_ModWand(Work, MinManaC, MinMeleeD, MinElem, AndOrFlag, AndOrFlag2) {
    if ((MinManaC   == undefined) || (MinManaC   == null)) MinManaC   = -1;
    if ((MinMeleeD  == undefined) || (MinMeleeD  == null)) MinMeleeD  = -1;
    if ((MinElem    == undefined) || (MinElem    == null)) MinElem    = -1;
    if ((AndOrFlag  == undefined) || (AndOrFlag  == null)) AndOrFlag  = -1;
    if ((AndOrFlag2 == undefined) || (AndOrFlag2 == null)) AndOrFlag2 = -1;

    if (MinManaC   >= 0) this.WandFilter[Work].MinManaC   = MinManaC;
    if (MinMeleeD  >= 0) this.WandFilter[Work].MinMeleeD  = MinMeleeD;
    if (MinElem    >= 0) this.WandFilter[Work].MinElem    = MinElem;
    if (AndOrFlag  >= 0) this.WandFilter[Work].AndOrFlag  = AndOrFlag;
    if (AndOrFlag2 >= 0) this.WandFilter[Work].AndOrFlag2 = AndOrFlag2;
} // end func

function Loot_ShowWand() {
    if (LootDebug) Log.OutD("Loot_ShowWand: Begin", LootDebugScreen);

    var Header = false;
    for (var x=1; x<11; x++) {
	with (this.WandFilter[x]) {
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width("Wand",1.8) +
			    Util.Width("Min",0.9) +
			    Util.Width("Min",0.9) +
			    "Elem");
			    
		Chat.Screen(Util.Width("(Work)",0.8) +
			    Util.Width("Enabled",1.0) +
			    Util.Width("ManaC",0.9) + 
			    Util.Width("MeleeD",0.9) +
			    Util.Width("Bonus",0.9) +
			    Util.Width("Match1", 0.9) +
			    "Match2");
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(x + " ",0.8) + 
	    		Util.Width(Enabled+" ",1.0) +
	    		Util.Width(MinManaC+" ",0.9) + 
	    		Util.Width(MinMeleeD+" ",0.9) +
	    		Util.Width(MinElem+" ",0.9) +
			Util.Width(andorFlagText[AndOrFlag]+" ",0.9) +
			andorFlagText[AndOrFlag2] );
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowWeapon: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // end func

function Loot_ResetWand() {
    // Loot_SetWand2(Work, Enabled, MinManaC, MinMeleeD, MinElem, AndOrFlag, AndOrFlag2)
    this.SetWand3(  1, true,  8, 12, 5, 1, 1 );
    this.SetWand3(  2, true,  8, 12, 5, 1, 1 );
    this.SetWand3(  3, true,  8, 12, 5, 1, 1 );
    this.SetWand3(  4, true,  8, 12, 5, 1, 1 );
    this.SetWand3(  5, true,  8, 12, 5, 1, 1 );
    this.SetWand3(  6, true,  8, 12, 5, 1, 1 );
    this.SetWand3(  7, true,  0, 13, 5, 0, 1 );
    this.SetWand3(  8, true,  0, 14, 5, 0, 1 );
    this.SetWand3(  9, true,  0, 15, 5, 0, 1 );
    this.SetWand3( 10, true,  0, 15, 5, 0, 1 );
} // end func


// SALVAGE METHODS ***********************************************************

function Loot_EnableSalvage(salIndex) {
    this.SalvageFilter[salIndex].Enabled = true;
} // end func

function Loot_DisableSalvage(salIndex) {
    this.SalvageFilter[salIndex].Enabled = false;
} // end func

function Loot_AddSalvage(MaterialId, SalType, Name, Desc) {
    var NextSlot = this.SalvageFilter.length;
    this.SalvageFilter[NextSlot] = new SalvageFilterRec(MaterialId, SalType, Name, Desc);
    this.SalvageIndex[MaterialId] = NextSlot;
    this.SalvageOpts = this.SalvageOpts + " <option data='" + NextSlot + "' text='" + Name + "'/>\n";
} // end func

function Loot_SetSalvage(MaterialId, Enabled, MinWork, MaxBU, SortPrio, CombineMask, ExcludeVal) {
    if (this.SalvageIndex[MaterialId] == undefined) {
	Chat.Screen("SetSalvage: Error, material id [" + MaterialId + "] unknown!");
    	return false;
    } // endif
    
    // Preserves previous behaviour of salvage points under new salvaging strategy
    for (var x=10; x>0; x--) {
	if (!(CombineMask & combineMaskVal[x])) {
	    CombineMask |= combineMaskVal[x];
	} else {
	    break;  // don't process after first set bit is found
	} // endif
    } // end for

    salIndex = this.SalvageIndex[MaterialId];
    this.SalvageFilter[salIndex].Enabled	= Enabled;
    this.SalvageFilter[salIndex].MinWork	= MinWork;
    this.SalvageFilter[salIndex].MaxWork	= 10;
    this.SalvageFilter[salIndex].MaxBU		= MaxBU;
    this.SalvageFilter[salIndex].SortPrio	= SortPrio;
    this.SalvageFilter[salIndex].CombineMask	= CombineMask;
    this.SalvageFilter[salIndex].ExcludeVal	= ExcludeVal;
    this.SetWorkRange(salIndex);
    return true;
} // end func


function Loot_SetSalvage2(MaterialId, Enabled, MinWork, MaxBU, SortPrio, CombineMask, ExcludeVal) {
    if (this.SalvageIndex[MaterialId] == undefined) {
	Chat.Screen("SetSalvage2: Error, material id [" + MaterialId + "] unknown!");
    	return false;
    } // endif

    // Get salvage index based on material type    
    salIndex = this.SalvageIndex[MaterialId];
    
    // Figure out maximum item work to salvage
    for (var x=10; x>0; x--) {
	if (CombineMask & combineMaskVal[x]) break;
    } // end for
    this.SalvageFilter[salIndex].MaxWork = x;

    this.SalvageFilter[salIndex].Enabled	= Enabled;
    this.SalvageFilter[salIndex].MinWork	= MinWork;
    this.SalvageFilter[salIndex].MaxBU		= MaxBU;
    this.SalvageFilter[salIndex].SortPrio	= SortPrio;
    this.SalvageFilter[salIndex].CombineMask	= CombineMask;
    this.SalvageFilter[salIndex].ExcludeVal	= ExcludeVal;
    this.SetWorkRange(salIndex);
    return true;
} // end func


function Loot_ModSalvage(salIndex, MinWork, MaxBU, SortPrio, CombineMask, ExcludeVal) {
    if ((MinWork == undefined) || (MinWork == null)) MinWork = -1;
    if ((MaxBU == undefined) || (MaxBU == null)) MaxBU = -1;
    if ((SortPrio == undefined) || (SortPrio == null)) SortPrio = -1;
    if ((CombineMask == undefined) || (CombineMask == null)) CombineMask = -1;

    if ((ExcludeVal != undefined) && (ExcludeVal != null)) {
    	this.SalvageFilter[salIndex].ExcludeVal = ExcludeVal;
    } // endif

    if (MinWork >= 0)	this.SalvageFilter[salIndex].MinWork	= MinWork;
    if (MaxBU  >= 0)	this.SalvageFilter[salIndex].MaxBU	= MaxBU;
    if (SortPrio >= 0)	this.SalvageFilter[salIndex].SortPrio	= SortPrio;
    if (CombineMask  >= 0) {
    	this.SalvageFilter[salIndex].CombineMask = CombineMask;
    	this.SetWorkRange(salIndex);
    } // endif
} // end func

function Loot_ResetSalvage() {
   // Armor Tinkering
    var defMask = combineMaskVal[7] | combineMaskVal[8] | combineMaskVal[9] | combineMaskVal[10];
    this.SetSalvage(materialSteel,	    true,	5, this.SalDefBur,   7200,  defMask);
    this.SetSalvage(materialAlabaster,	    true,	5, this.SalDefBur,   7100,  defMask);
    this.SetSalvage(materialBronze,	    true,	5, this.SalDefBur,   7000,  defMask);
    this.SetSalvage(materialMarble,	    true,	5, this.SalDefBur,   6900,  defMask);
    this.SetSalvage(materialCeramic,	    true,	5, this.SalDefBur,   6800,  defMask);
    this.SetSalvage(materialDilloHide,	    true,	5, this.SalDefBur,   6700,  defMask);
    this.SetSalvage(materialReedsharkHide,  true,	5, this.SalDefBur,   6600,  defMask);
    this.SetSalvage(materialWool,	    true,	5, this.SalDefBur,   6500,  defMask);

    // Weapon Tinkering
    this.SetSalvage(materialBrass,	    true,	5, this.SalDefBur,   6400,  defMask);
    this.SetSalvage(materialGranite,	    true,	4, this.SalDefBur,   6300,  defMask);
    this.SetSalvage(materialIron,	    true,	5, this.SalDefBur,   6200,  defMask);
    this.SetSalvage(materialMahogany,	    true,	4, this.SalDefBur,   6100,  defMask);
    this.SetSalvage(materialOak,	    true,	5, this.SalDefBur,   6000,  defMask);
    this.SetSalvage(materialVelvet,	    true,	5, this.SalDefBur,   5900,  defMask);
    this.SetSalvage(materialGreenGarnet,    true,	5, this.SalDefBur,   5850,  defMask);

    // Item Tinkering
    this.SetSalvage(materialGold,	    true,	5, this.SalDefBur,   5800,  defMask);
    this.SetSalvage(materialLinen,	    true,	5, this.SalDefBur,   5700,  defMask);
    this.SetSalvage(materialPine,	    true,	5, this.SalDefBur,   5600,  defMask);
    this.SetSalvage(materialMoonstone,	    true,	5, this.SalDefBur,   5500,  defMask);
    this.SetSalvage(materialIvory,	    true,	5, this.SalDefBur,   5400,  combineMaskVal[10]);
    this.SetSalvage(materialLeather,	    true,	5, this.SalDefBur,   5300,  combineMaskVal[10]);

    // Magic Item Tinkering
    this.SetSalvage(materialBlackOpal,	    true,	5, this.SalDefBur,   5200,  combineMaskVal[10]);
    this.SetSalvage(materialFireOpal,	    true,	5, this.SalDefBur,   5100,  combineMaskVal[10]);
    this.SetSalvage(materialSunstone,	    true,	5, this.SalDefBur,   5000,  combineMaskVal[10]);
    this.SetSalvage(materialOpal,	    true,	5, this.SalDefBur,   4900,  defMask);

    // Rendering
    this.SetSalvage(materialWhiteSapphire,  true,	5, this.SalDefBur,   4800,  combineMaskVal[10]);
    this.SetSalvage(materialBlackGarnet,    true,	4, this.SalDefBur,   4700,  combineMaskVal[10]);
    this.SetSalvage(materialImperialTopaz,  true,	5, this.SalDefBur,   4600,  combineMaskVal[10]);
    this.SetSalvage(materialEmerald,	    true,	5, this.SalDefBur,   4500,  combineMaskVal[10]);
    this.SetSalvage(materialRedGarnet,      true,	4, this.SalDefBur,   4400,  combineMaskVal[10]);
    this.SetSalvage(materialAquamarine,     true,	5, this.SalDefBur,   4300,  combineMaskVal[10]);
    this.SetSalvage(materialJet,	    true,	5, this.SalDefBur,   4200,  combineMaskVal[10]);

    // Requirements
    this.SetSalvage(materialEbony,	    true,	5, this.SalDefBur,   4100,  combineMaskVal[10]);
    this.SetSalvage(materialTeak,	    true,	5, this.SalDefBur,   4000,  combineMaskVal[10]);
    this.SetSalvage(materialPorcelain,	    true,	5, this.SalDefBur,   3900,  combineMaskVal[10]);
    this.SetSalvage(materialSilver,	    true,	5, this.SalDefBur,   3800,  combineMaskVal[10]);
    this.SetSalvage(materialCopper,	    true,	5, this.SalDefBur,   3700,  combineMaskVal[10]);
    this.SetSalvage(materialSilk,	    true,	5, this.SalDefBur,   3600,  combineMaskVal[10]);

    // Armor Imbues
    this.SetSalvage(materialYellowTopaz,    true,	5, this.SalDefBur,   3500,  combineMaskVal[10]);
    this.SetSalvage(materialPeridot,	    true,	5, this.SalDefBur,   3400,  combineMaskVal[10]);
    this.SetSalvage(materialZircon,	    true,	5, this.SalDefBur,   3300,  combineMaskVal[10]);

    // Jewelry Imbues
    this.SetSalvage(materialLavenderJade,   true,	5, this.SalDefBur,   3200,  combineMaskVal[10]);
    this.SetSalvage(materialRedJade,	    true,	5, this.SalDefBur,   3100,  combineMaskVal[10]);
    this.SetSalvage(materialCitrine,	    true,	5, this.SalDefBur,   3000,  combineMaskVal[10]);
    this.SetSalvage(materialCarnelian,	    true,	5, this.SalDefBur,   2900,  combineMaskVal[10]);
    this.SetSalvage(materialRoseQuartz,	    true,	5, this.SalDefBur,   2800,  combineMaskVal[10]);
    this.SetSalvage(materialBloodstone,	    true,	5, this.SalDefBur,   2700,  combineMaskVal[10]);
    this.SetSalvage(materialSmokeyQuartz,   true,	5, this.SalDefBur,   2600,  combineMaskVal[10]);
    this.SetSalvage(materialAgate,	    true,	5, this.SalDefBur,   2500,  combineMaskVal[10]);
    this.SetSalvage(materialLapisLazuli,    true,	5, this.SalDefBur,   2400,  combineMaskVal[10]);
    this.SetSalvage(materialMalachite,	    true,	5, this.SalDefBur,   2300,  combineMaskVal[10]);
    this.SetSalvage(materialHematite,	    true,	5, this.SalDefBur,   2200,  combineMaskVal[10]);
    this.SetSalvage(materialAzurite,	    true,	5, this.SalDefBur,   2100,  combineMaskVal[10]);

    // Unknown Use
    this.SetSalvage(materialAmber,	    false,	5, this.SalDefBur,   2000,  combineMaskVal[10]);
    this.SetSalvage(materialAmethyst,	    false,	5, this.SalDefBur,   1900,  combineMaskVal[10]);
    this.SetSalvage(materialDiamond,	    false,	5, this.SalDefBur,   1800,  combineMaskVal[10]);
    this.SetSalvage(materialGreenJade,	    false,	5, this.SalDefBur,   1600,  combineMaskVal[10]);
    this.SetSalvage(materialGromnieHide,    false,	5, this.SalDefBur,   1500,  combineMaskVal[10]);
    this.SetSalvage(materialObsidian,	    false,	5, this.SalDefBur,   1400,  combineMaskVal[10]);
    this.SetSalvage(materialOnyx,	    false,	5, this.SalDefBur,   1300,  combineMaskVal[10]);
    this.SetSalvage(materialPyreal,	    false,	5, this.SalDefBur,   1200,  combineMaskVal[10]);
    this.SetSalvage(materialRuby,	    false,	5, this.SalDefBur,   1100,  combineMaskVal[10]);
    this.SetSalvage(materialSandstone,	    false,	5, this.SalDefBur,   1000,  combineMaskVal[10]);
    this.SetSalvage(materialSapphire,	    false,	5, this.SalDefBur,    900,  combineMaskVal[10]);
    this.SetSalvage(materialSatin,	    false,	5, this.SalDefBur,    800,  combineMaskVal[10]);
    this.SetSalvage(materialSerpentine,	    false,	5, this.SalDefBur,    700,  combineMaskVal[10]);
    this.SetSalvage(materialTigerEye,	    false,	5, this.SalDefBur,    600,  combineMaskVal[10]);
    this.SetSalvage(materialTourmaline,	    false,	5, this.SalDefBur,    500,  combineMaskVal[10]);
    this.SetSalvage(materialTurquoise,	    false,	5, this.SalDefBur,    400,  combineMaskVal[10]);
    this.SetSalvage(materialWhiteJade,	    false,	5, this.SalDefBur,    300,  combineMaskVal[10]);
    this.SetSalvage(materialWhiteQuartz,    false,	5, this.SalDefBur,    200,  combineMaskVal[10]);
    this.SetSalvage(materialYellowGarnet,   false,	5, this.SalDefBur,    100,  combineMaskVal[10]);
} // end func

function Loot_ShowSalvage(EnableFlag) {
   if ((EnableFlag==undefined) || (EnableFlag==null)) EnableFlag = false;
    if (LootDebug) Log.OutD("Loot_ShowSalvage: Begin, EnableFlag [" + EnableFlag + "]", LootDebugScreen);

   var Header = false;
    for (var x=0; x<this.SalvageFilter.length; x++) {
	with (this.SalvageFilter[x]) {
 	    if (EnableFlag && !Enabled) continue;
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");

		Chat.Screen(Util.Width(" ",4.4) + "Ust");
		
		Chat.Screen(Util.Width(" ",2.8) +
			    Util.Width("Exclude",1.0) +
			    Util.Width("Min",0.6) +
			    Util.Width("Max",0.6) +
			    Util.Width("Max",0.6) +
			    Util.Width("Sort",0.6) +
			    "Combine" )

		Chat.Screen(Util.Width("Salvage",1.8) +
			    Util.Width("Enabled",1.0) +
			    Util.Width("ValSalv",1.0) +
			    Util.Width("Work",0.6) +
			    Util.Width("Work",0.6) +
			    Util.Width("BU",0.6) +
			    Util.Width("Prio",0.6) +
			    "Points" );
	    } // endif

 	    CombPoints = "";
	    for (var y=1; y<11; y++)
		if (CombineMask & combineMaskVal[y]) CombPoints += y + "  ";

	    // Print info
	    Chat.Screen(Util.Width(Name+" ",1.8)       + Util.Width(Enabled+" ",1.0)  +
	    		Util.Width(ExcludeVal+" ",1.0) + Util.Width(MinWork+" ",0.6)  +
	    		Util.Width(MaxWork+" ",0.6)    + Util.Width(MaxBU+" ",0.6)    +
	    		Util.Width(SortPrio+" ",0.6)   + CombPoints);
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowSalvage: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // end func

function Loot_SetWorkRange(salIndex) {
    with (this.SalvageFilter[salIndex]) {
	var LowIndex = 1;
	var HighIndex = 0;
	if (LootDebug2) Log.OutD("Loot_SetWorkRange: Combine mask: " + Util.Hex(CombineMask));
	for (var x=1; x<=salvageMaxWork; x++) {
	    if (LootDebug2) Log.OutD("Loot_SetWorkRange: ["+x+"] combineMaskVal[x]: "+Util.Hex(combineMaskVal[x]));
	    if (CombineMask & combineMaskVal[x]) {
	    	HighIndex = x;
	    	if (LootDebug2) Log.OutD("Loot_SetWorkRange: LowIndex ["+LowIndex+"] HighIndex ["+HighIndex+"]");
	    	for (var y=LowIndex; y<=HighIndex; y++) {
	    	    WorkRange[y] = new MaterialRangeRec(LowIndex, HighIndex+1);
	    	    if (LootDebug2) {
	    	    	Log.OutD("Loot_SetWorkRange: Work ["+y+"] Low ["+LowIndex+"] High [" + HighIndex + "]");
	    	    	Log.OutD("Loot_SetWorkRange: SalvageFilter [" + salIndex + "].WorkRange[" + y + "].Low [" +
	    	    		  WorkRange[y].Low + "]");
	    	    	Log.OutD("Loot_SetWorkRange: SalvageFilter [" + salIndex + "].WorkRange[" + y + "].High [" +
	    	    		  WorkRange[y].High + "]");
		    }// endif
	    	} // end for
	    	LowIndex = HighIndex + 1;
	    } // endif
	} // end for

	// NOTE!!! User must now explicitly specify combine points to max possible
	// value if salvaging for all work ranges desired.  This change was done
	// to enable user to salvage lower grade works automatically, but save the
	// higher work salvage for a character that has a good salvage bonus. So
	// now we specify the maximum salvage work above which a material will
	// not be USTed
	if (LootDebug2) Log.OutD("Loot_SetWorkRange: Max work set to HighIndex [" + HighIndex + "]");
	MaxWork = HighIndex;
	
	// Now define the records for the works above the highest specified.
	// for (var x=LowIndex; x<=salvageMaxWork; x++) {
	//    WorkRange[x] = new MaterialRangeRec(x,x+1);
	//    if (LootDebug2) {
	//	Log.OutD("Loot_SetWorkRange: SalvageFilter [" + salIndex + "].WorkRange[" + x + "].Low [" +
	//		  WorkRange[x].Low + "]");
	//	Log.OutD("Loot_SetWorkRange: SalvageFilter [" + salIndex + "].WorkRange[" + x + "].High [" +
	//		  WorkRange[x].High + "]");
	//    }// endif
	// } // endfor
    } // end with
} // end func


function Loot_EnableDI(Index) {
    this.TypeInfo[Index].DiEnabled	= true;
} // end func

function Loot_DisableDI(Index) {
    this.TypeInfo[Index].DiEnabled	= false;
} // end func

function Loot_SetDI(Index, DiEnabled, DiMinValue, DiMaxBU, DiMaxWork) {
    this.TypeInfo[Index].DiEnabled	= DiEnabled;
    this.TypeInfo[Index].DiMinValue	= DiMinValue;
    this.TypeInfo[Index].DiMaxBU	= DiMaxBU;
    this.TypeInfo[Index].DiMaxWork	= DiMaxWork;
} // end func

function Loot_ShowDI(EnabledOnly) {
    if ((EnabledOnly == undefined) || (EnabledOnly == null)) EnabledOnly = false;
    if (LootDebug) Log.OutD("Loot_ShowDI: Begin EnabledOnly [" + EnabledOnly + "]", LootDebugScreen);

    var Header = false;

    for (var x=lootTypeBegin; x<lootTypeTotal; x++) {
	with (this.TypeInfo[x]) {
	    if (!DiActive) continue;	// Only show active death item entries
 	    if (EnabledOnly && !DiEnabled) continue;
	    if (!Header) {
		Header = true;
		Chat.Screen(" ");
		Chat.Screen(Util.Width("Class",1.2)   + Util.Width("Enabled",1.0) +
			    Util.Width("Min Val",1.0) + Util.Width("Max BU",1.0) + "Max Work");
	    } // endif

	    // Print info
	    Chat.Screen(Util.Width(Name,1.2) + Util.Width(DiEnabled+" ",1.0) +
	    		Util.Width(DiMinValue+" ",1.0) + Util.Width(DiMaxBU+" ",1.0) + DiMaxWork);
	} // end with
    } // endfor
    if (LootDebug) Log.OutD("Loot_ShowDI: End, Header [" + Header + "]", LootDebugScreen);
    return Header;	// returns false if no items found, true otherwise
} // end func

function Loot_ResetDI() {
    for (var x=lootTypeBegin; x<lootTypeTotal; x++) {
    	with (this.TypeInfo[x]) {
	    DiEnabled	= false;
	    DiMinValue	= 20000;
	    DiMaxBU	= 250;
	    DiMaxWork   = 6;
	} // end with
    } // end for	
} // end func


// MISC FILTER MODIFICATION METHODS *******************************************

function Loot_ModValSalv(MinRatio, MaxBagValue) {
    if ((MinRatio    == undefined) || (MinRatio    == null)) MinRatio = -1;
    if ((MaxBagValue == undefined) || (MaxBagValue == null)) MaxBagValue = -1;
    if (MinRatio    >= 0) this.ValSalvFilter.MinRatio	 = MinRatio;
    if (MaxBagValue >= 0) this.ValSalvFilter.MaxBagValue = MaxBagValue;
} // end func

function Loot_ShowValSalv() {
} // end func

function Loot_ResetValSalv() {
    this.ModValSalv(0, 100000);
} // end func
    
function Loot_ModJewelry(MinSplLvl1, MinSplLvl2, MinSplLvl3) {
    if ((MinSplLvl1 == undefined) || (MinSplLvl1 == null)) MinSplLvl1 = -1;
    if ((MinSplLvl2 == undefined) || (MinSplLvl2 == null)) MinSplLvl2 = -1;
    if ((MinSplLvl3 == undefined) || (MinSplLvl3 == null)) MinSplLvl3 = -1;
    if (MinSplLvl1 >= 0) this.JewelryFilter.MinSplLvl1 = MinSplLvl1;
    if (MinSplLvl2 >= 0) this.JewelryFilter.MinSplLvl2 = MinSplLvl2;
    if (MinSplLvl3 >= 0) this.JewelryFilter.MinSplLvl3 = MinSplLvl3;
} // end func

function Loot_ShowJewelry() {
} // end func

function Loot_ResetJewelry() {
    this.ModJewelry(6, 6);
} // end func


function Loot_ModClothing(MinSplLvl1, MinSplLvl2, MinSplLvl3) {
    if ((MinSplLvl1 == undefined) || (MinSplLvl1 == null)) MinSplLvl1 = -1;
    if ((MinSplLvl2 == undefined) || (MinSplLvl2 == null)) MinSplLvl2 = -1;
    if ((MinSplLvl3 == undefined) || (MinSplLvl3 == null)) MinSplLvl3 = -1;

    if (MinSplLvl1 >= 0) this.ClothingFilter.MinSplLvl1 = MinSplLvl1;
    if (MinSplLvl2 >= 0) this.ClothingFilter.MinSplLvl2 = MinSplLvl2;
    if (MinSplLvl3 >= 0) this.ClothingFilter.MinSplLvl3 = MinSplLvl3;
} // end func

function Loot_ShowClothing() {
} // end func

function Loot_ResetClothing() {
    this.ModClothing(6, 6);
} // end func


function Loot_ModScroll(MinLvl, MaxLvl, UnknownOnly, LearnUnknown) {
    if ((MinLvl       == undefined) || (MinLvl       == null)) MinLvl = -1;
    if ((MaxLvl       == undefined) || (MaxLvl       == null)) MaxLvl = -1

    if (MinLvl >= 0) this.ScrollFilter.MinLvl = MinLvl;
    if (MaxLvl >= 0) this.ScrollFilter.MaxLvl = MaxLvl;
    if ((UnknownOnly  != undefined) && (UnknownOnly  != null)) this.ScrollFilter.UnknownOnly  = UnknownOnly;
    if ((LearnUnknown != undefined) && (LearnUnknown != null)) this.ScrollFilter.LearnUnknown = LearnUnknown;
} // end func

function Loot_ShowScroll() {
} // end func

function Loot_ResetScroll() {
    this.ModScroll(7, 7, false, false);
} // end func


function Loot_ModValuable(MaxBU, MinValue, MaxTotal) {
    if ((MaxBU    == undefined) || (MaxBU     == null)) MaxBU = -1;
    if ((MinValue == undefined) || (MinValue  == null)) MinValue  = -1;
    if ((MaxTotal == undefined) || (MaxTotal  == null)) MaxTotal  = -1;

    if (MaxBU    >= 0) this.ValuableFilter.MaxBU    = MaxBU;
    if (MinValue >= 0) this.ValuableFilter.MinValue = MinValue ;
    if (MaxTotal >= 0) this.ValuableFilter.MaxTotal = MaxTotal ;
} // end func

function Loot_ShowValuable() {
} // end func

function Loot_ResetValuable() {
    this.ModValuable(250, 8000, 9999999);
} // end func


Loot = new LootObj();


// For reference, here are all the equipment slots.
// var eqmNil                      =              0;	// No slot.
// var eqmHead                     =     0x00000001;
// var eqmChestUnder               =     0x00000002;
// var eqmAbdomenUnder             =     0x00000004;
// var eqmUpperArmsUnder           =     0x00000008;
// var eqmLowerArmsUnder           =     0x00000010;
// var eqmHands                    =     0x00000020;
// var eqmUpperLegsUnder           =     0x00000040;
// var eqmLowerLegsUnder           =     0x00000080;
// var eqmFeet                     =     0x00000100;
// var eqmChestOuter               =     0x00000200;
// var eqmAbdomenOuter             =     0x00000400;
// var eqmUpperArmsOuter           =     0x00000800;
// var eqmLowerArmsOuter           =     0x00001000;
// var eqmUpperLegsOuter           =     0x00002000;
// var eqmLowerLegsOuter           =     0x00004000;
// var eqmNecklace                 =     0x00008000;
// var eqmRBracelet                =     0x00010000;
// var eqmLBracelet                =     0x00020000;
// var eqmBracelets                =     0x00030000;	// Both bracelet slots.
// var eqmRRing                    =     0x00040000;
// var eqmLRing                    =     0x00080000;
// var eqmRings                    =     0x000C0000;	// Both ring slots.
// var eqmWeapon                   =     0x00100000;	// Melee weapon slot.
// var eqmShield                   =     0x00200000;
// var eqmRangedWeapon             =     0x00400000;	// Bow or crossbow slot.
// var eqmAmmo                     =     0x00800000;
// var eqmFocusWeapon              =     0x01000000;	// Wand, staff, or orb slot.
// var eqmAll                      =     0x01FFFFFF;	// All equipment slots.
         