// WAR_STATES.JS
//
// Author:	Poundboy
// Version:	2.13
//
//
//
// Description:
//
//	This module processes all the states for the Finite State Machine (FSM). See AAA_README.TXT and
//	CHANGE.TXT files for more details
//
//
// Modification History:
//
// Who	  Date	    Version	Description
//
// PB	  02/10/04  2.08	Initial version.
//

// After log on, this state is executed once
function DoState_Init() {
    if (debug||debugInit) Log.OutD("DoState_Init: Begin");

    // Disable messages generated from OnControlEvent.  Needed because when we set a choice box a event
    // is generated.  No event is generated when a text or checkbox is modified.
    flagVerbose = false;

    // Define command interface
    Log.OutDM("DoState_Init", "Displaying decal interface...", ToChat);
    DisplayInterface();

    // Initialize creature lock list (needed for exclusion mode)
    if (debugInit) Log.OutD("DoState_Init: Initializing lock database...");
    for (var x=0; x<selLockListSize; x++) SelLockList[x] = new LockObj(0);

    // If any items on loot lists then the macro just completed a logout/login sequence.
    // We need to rebuild the ACOs since they have been marked as invalid by skunkworks.
    Loot.RefreshLoot();

    // Initialize the critical and urgent creature amounts and retreat count.
    CriticalAmount = baseCriticalAmt;
    UrgentAmount   = baseUrgentAmt;
    PrevRetreatAmt = retreatAmtOne;
    RetreatAmount  = retreatAmtOne;

    // Record location at start of macro
    LastRecordedLoc = skapi.maplocCur;
    Log.OutD("DoState_Init: Start coords: [" + Util.FormatCoords(LastRecordedLoc,2) + "]");

    // If reduce burden flag set then see if strength spell active
    if (flagReduceBurden) {
   	if (debugInit) Log.OutD("DoState_Init: Checking for active strength spell...");
	for (var z=0; z < skapi.cospell.Count; z++) {
	    if (skapi.cospell(z).spellid == SpellObj_Strength.spellid) {
		if (debugInit||logBurden) Log.OutD("DoState_Init: Strength active, resetting expiration time...");
         	var Remain = skapi.cospell(z).csecRemain
            	var CurTime = new Date();
            	StrengthExp = CurTime.getTime() + (Remain*1000);
            	if (debugInit) Log.OutD("DoState_Init: New expiration time: " + new Date(StrengthExp));
		break;
	    } // endif
	} // end for
	if (debugInit && (z==skapi.cospell.Count)) Log.OutD("DoState_Init: Strength spell not active.");
    } // endif

    // Create fellowship if desired, and we are not already part of one
    if (creFellow) {
    	var cofellow = skapi.cofellow;
	if (cofellow.Count > 0) {
	    Log.OutDM("DoState_Init", "Fellow already exists, # members [" + cofellow.Count + "]", ToChat);
	} else {
	    Log.OutDM("DoState_Init", "Creating Fellow...", ToChat);
	    Player.CreateFellow(fellowName);
	} // endif

	// If autofellow enabled, then try to bring anybody in list into fellow
	if (autoFellow) {
	    if (debug) Log.OutDM("DoState_Init", "Autofellow check...",true);
	    for (var loop=0; loop < autoFellowList.length; loop++) {
	        var FellowChar = autoFellowList[loop];
	        Log.OutDM("DoState_Init", "Trying to fellow char: " + FellowChar, ToChat );
		Player.AddFellow(FellowChar);
	    } // end for
        } // endif
    } // endif

    // Clear No Attack List
    SelDNALCnt = 0;

    // If inactivity timer set, then set it.
    if (inactTimer>0) skapi.SetIdleTime(inactTimer);

    Log.OutDM("DoState_Init", "Startup sequence done, macro active!", ToChat);

    // Change state to IdleInit
    if (debug||debugInit) Log.OutD("DoState_Init: End, setting state to IdleInit");
    StateChange(stateIdleInit,NoCheck)
} // end function


function DoState_IdleInit() {
    if (debug||debugIdle) Log.OutD("DoState_IdleInit: Start");

    // Make sure we have latest event info
    skapi.WaitEvent(10);

    // If a death log time is set (by death routine), then see if it is time to log
    if ((DeathLogTime>0) && ((new Date()-DeathLogTime) > 0)) {
	Log.OutDM("State_IdleInit", "Death log time reached, logging character...", ToChat);
	StateChange(stateLog, NoCheck);
	return;
    } // endif

    // Rotate slightly to the left every once in a while so we don't log out from inactivity
    if ((new Date()-RotateLast) > rotateInt) {
	skapi.Keys(cmidTurnLeft, kmoDown);
	skapi.Sleep(250);
	skapi.Keys(cmidTurnLeft, kmoUp);
	RotateLast = new Date();
    } //endif

    // If creature close to us then cancel Idle state and deal with the danger,
    if (Danger) {
    	if (debug||debugIdle) Log.OutD("DoState_IdleInit: Danger detected, aborting idle state...");
    	StateChange(stateSelect);
    	Player.EnsureCombatMode(true);
    	return;
    } // endif

    // Now set the select range for creatures based on whether we are outdoors or in a dungeon.  This
    // is rechecked every Idle period.
    if (skapi.maplocCur.fInDungeon) {
    	if (SelRange != selRangeDung) {
	    Log.OutDM("DoState_IdleInit", "Select range set to in-dungeon distance of " + selRangeDung +
	    	      " map units.", ToChat);
	    SelRange = selRangeDung;
	} // endif
    } else {
   	if (SelRange != selRangeOut) {
	    Log.OutDM("DoState_IdleInit", "Select range set to outdoors distance of " + selRangeOut +
	    	      " map units.", ToChat);
	    SelRange = selRangeOut;
	} // endif
    } // endif

    // If we are not in retreat mode or component suspend mode and AutoHome is enabled, then
    // make sure we are near our kill location.  If we are too far away then don't move since
    // we may have died or manually moved character.
    if (flagAutoHome && !RetreatActive && !AttCtx.CompSusp) {
    	var KillLocDist = Util.Distance(AttLoc[AttLocCur].KillLoc,skapi.maplocCur);
    	if ((KillLocDist > navAutoMin) && (KillLocDist < navAutoMax)) {
    	    NavActive = true;
	    Chat.Screen("Too far away from KillLoc, going there..");
	    if (debugMove||logMove) Log.OutD("Main: Too far away from KillLoc, going there...");
	    GotoKillLoc(navTimeOut);
	    NavActive = false;
	    if (flagMoveDelay) {
		Chat.Screen("Arrived at kill location. Pausing for " + navWait + " ms...");
		if (debugMove||logMove)
		    Log.OutD("Main: Arrived at kill location. Pausing for " + navWait + " ms...");
		skapi.WaitEvent(navWait, wemFullTimeout);
	    } else {
		Chat.Screen("Arrived at kill location.");
		if (debugMove||logMove) Log.OutD("Main: Arrived at kill location.");
	    } // endif
	} // endif
    } // endif

    // If member of fellow, then count the number of fellow members near us.  The more macroers near us the
    // higher thresholds used for several functions like retreating, and some modes are enabled/disabled based
    // on the number of players near us (like exclusion mode).  Also, if healing or stam of fellow members
    // is enabled, see if any fellow members need a heal or stamina boost.
    PlayerExclCnt = 0;
    PlayerNearCnt = 0;
    var cofellow = skapi.cofellow;	// Get collection of fellow members
    // Chat.Screen("Cofellow count: " + cofellow.Count);
    if (cofellow.Count > 0) {
	for (var x=0; x<cofellow.Count; x++) {
	    var FellowObj = cofellow.Item(x);
	    if (FellowObj.szName == skapi.szName) continue;	// Don't check ourselves
	    var FellowAco = skapi.AcoFromOid(FellowObj.oid);
	    if (FellowAco == null) continue;			// if aco null then on to the next
	    if (FellowAco.maploc == null) continue;
	    var DistToPlayer = Util.Distance(FellowAco.maploc,skapi.maplocCur);
	    // Chat.Screen("Player [" + FellowObj.szName + "] dist [" + DistToPlayer + "]");

	    // Increase counters if fellow member close enough
	    if (DistToPlayer < coMacroerDist) PlayerNearCnt++;
	    if (DistToPlayer < autoExclDist)  PlayerExclCnt++;
	} // endfor
    } // endif

    // Compute the Critical and Urgent amounts.  When # of creatures near us exceeds these numbers special
    // processing occurs.  The more players that are macroing with us, the higher these numbers are set.
    CriticalAmount = baseCriticalAmt + PlayerNearCnt;
    UrgentAmount = baseUrgentAmt + PlayerNearCnt;

    // Compute the retreat amount.  We do this even if retreating disabled since it may
    // be re-enabled at any time.
    if (PlayerNearCnt == 0) {
	RetreatAmount = retreatAmtOne;
    } else {
    	if (PlayerNearCnt == 1)
	    RetreatAmount = retreatAmtTwo;
	else
	    RetreatAmount = retreatAmtThreePlus;
    } // endif

    // If retreating enabled and RetreatAmount has changed, then log information
    if (PrevRetreatAmt != RetreatAmount) {
    	if (flagRetreatCrit) {
    	    if (flagRepRetreat)
    	    	Chat.Screen("New retreat amount: " + RetreatAmount + "   Players near us: " + PlayerNearCnt);
	    if (logIdle) Log.OutD("DoState_IdleInit: New retreat amt: " + RetreatAmount +
	    			  ". Players near us: " + PlayerNearCnt);
	} // endif
    	PrevRetreatAmt = RetreatAmount;
    } // endif

    // Make sure we are part of a fellow if the creFellow variable is true.  If not a member,
    // create our own fellow.  If someone told us to drop the fellow recently, then give them
    // a little time (2 minutes) before we create one ourselves
    if (creFellow && !Player.MemberFellow()) {
        if ((new Date()-CharLastDisband) > (120*1000)) {
   	    if (debugIdle) Log.OutD("DoState_IdleInit: Fellow required and not member of one, creating...");
	    Log.OutDM("DoState_IdleInit", "Creating fellow [" + fellowName + "] ...", ToChat);
            Player.CreateFellow(fellowName);
            CharLastDisband = new Date();  	// If fellow attempt fails, this will cause a delay before next attempt
	} else {
 	    if (debugIdle) Log.OutD("DoState_IdleInit: Fellow required but waiting for timer to expire.");
	} // endif
    } // endif

    // If exclusion mode set but no fellow members near us (or fellow doesn't exist) then turn it off since it
    // is unnecessary now
    if (flagAutoExcl && ExclMode && (PlayerExclCnt == 0)) {
	// Turn off exclusion mode
	ExclMode = false;
	skapi.SetControlProperty(szPanelControl, "btnExclusion", "Text", "Enable ExclMode");
	if (!Player.MemberFellow()) {
	    Log.OutDM("DoState_IdleInit", "Not member of fellow, exclusion mode disabled", ToChat);
	} else {
	    Log.OutDM("DoState_IdleInit", "No nearby fellow members, exclusion mode disabled.", ToChat);
	} // endif
    } // endif

    // If exclusion mode not set but there are fellow members near us then turn it on
    if (flagAutoExcl && !ExclMode && (PlayerExclCnt > 0)) {
	// Turn on exclusion mode
	ExclMode = true;
	skapi.SetControlProperty(szPanelControl, "btnExclusion", "Text", "Disable ExclMode");
	Log.OutDM("DoState_IdleInit", "Another fellow member nearby, exclusion mode enabled.", ToChat);
    } // endif

    // If flag set to try to reduce burden then see if we are overburdened and
    // there is no active strength spell on us
    if (flagReduceBurden) {
	if (debugIdle) Log.OutD("DoState_IdleInit: Seeing if time to cast strength self spell...");
    	// If strength spell expired then
    	if ((new Date() - StrengthExp) > 0) {
    	    if (debugIdle) Log.OutD("DoState_IdleInit: Strength spell expired, seeing if we are overburdened...");
    	    // See if we are overburdened.  Get our current strength and 100% burden units
	    var skStrength = skapi.SkinfoFromAttr(attrStrength);
	    var CharCurStrength = skStrength.lvlCur;
	    var Burden100 = CharCurStrength * 150;

	    // Now see if we are overburdened
	    var OverBurden = skapi.burdenCur - Burden100;
	    if (OverBurden > 0) {
	    	// We are overburdened, log event and cast strength self spell
	    	if (debugIdle||logBurden)
		    Log.OutD("DoState_IdleInit: Overburdened by [" + OverBurden +
	    	    	     "] units, 100Burden [" + Burden100 + "] casting strength spell...");
	    	Cast.Cast(SpellObj_Strength, null, true, false);

	    	// If spell cast successful, update expiration date
	    	if (CastCode == castCodeSuccess) {
	    	    var CurTime = new Date();
	    	    StrengthExp = CurTime.getTime() + (SpellObj_Strength.csecDuration*1000);
	    	    if (debugIdle||logBurden)
			Log.OutD("DoState_IdleInit: Strength cast successful, exp [" + new Date(StrengthExp) + "]");
	    	} else {
		    if (debugIdle||logBurden) Log.OutD("DoState_IdleInit: Strength spell cast failed. " + SpellFailText());
	    	} // endif
	    } else {
	    	if (debugIdle) Log.OutD("DoState_IdleInit: We are not overburdened.");
	    } // endif
    	} // endif
    } // endif

    // If user wants to use one or more of the new tusker spit potions, then make sure they are active
    if (flagUseTuskerSpit || flagUseAmberApe || flagUseBoboStout) {
  	var TuskerSpitActive = false;
   	var AmberApeActive = false;
   	var BoboStoutActive = false;
	for (var x=0; x < skapi.cospell.Count; x++) {
	    var SpellObj = skapi.cospell(x);
	    var Remain = SpellObj.csecRemain;
	    var SpellId = SpellObj.spellid;
	    if (SpellId == spellKetnanEye) {
	    	// Chat.Screen("Ketnan's Eye found, remaining time: " + Remain);
		if (Remain > potionSafety) TuskerSpitActive = true;
	    } // endif

	    if (SpellId == spellBrighteye) {
	    	// Chat.Screen("Brighteye's Favor found, remaining time: " + Remain);
		if (Remain > potionSafety) AmberApeActive   = true;
	    } // endif

	    if (SpellId == spellBoboQuick) {
	    	// Chat.Screen("Bobo's Quickening found, remaining time: " + Remain);
		if (Remain > potionSafety) BoboStoutActive  = true;
	    } // endif
	} // endfor

	if (flagUseTuskerSpit && !(TuskerSpitActive)) {
	    // Find bottle of tusker spit ale and use it
	    PotionAco = skapi.AcoFindInInv("Tusker Spit Ale");
	    if (PotionAco != null) {
	    	Log.OutDM("DoState_IdleInit", "Using tusker spit ale potion...", ToChat);
	    	PotionAco.Use();
	    } // endif
	} // endif

	if (flagUseAmberApe && !(AmberApeActive)) {
	    // Find bottle of amber ape and use it
	    PotionAco = skapi.AcoFindInInv("Amber Ape");
	    if (PotionAco != null) {
	    	Log.OutDM("DoState_IdleInit", "Using amber ape potion...", ToChat);
		PotionAco.Use();
	    } // endif
	} // endif

	if (flagUseBoboStout && !(BoboStoutActive)) {
	    // Find bottle of bobo's stout and use it
	    PotionAco = skapi.AcoFindInInv("Bobo's Stout");
	    if (PotionAco != null) {
	    	Log.OutDM("DoState_IdleInit", "Using bobo's stout potion...", ToChat);
	    	PotionAco.Use();
	    } //endif
	} // endif
    } // endif

    // If we want to log hourly stats to event file, then see if its time
    if (logHourly && ((new Date()-LastHourly)>60*60*1000)) {
    	if (debugIdle) Log.OutD("DoState_IdleInit: Logging hourly stats...");
    	LastHourly = new Date();
    	Log.OutD( "Hourly: Begin");

    	// Print experience information
    	Log.OutD( "Hourly: Total experience   [" + skapi.expTotal  + "]");
    	Log.OutD( "Hourly: Unspent experience [" + UnspentExp + "]");
    	LoadStats();
	Log.OutD( "Hourly: ElapsedTimeMin: " + ElapsedTimeMin);
	Log.OutD( "Hourly: Stats Reset: " + new Date(StartTime));
	Log.OutD( "Hourly: Total Kills: " + TotalKilled);
	Log.OutD( "Hourly: Kill Exp: " + KillExp + "  Actual Exp: " + StatActExp);
	Log.OutD( "Hourly: Kills/Hour: " + StatAveKillHour + "  Kill Exp/Hour: " +  StatEstExpHour);
	Log.OutD( "Hourly: Actual Exp/Hour: " + StatActExpHour );

    	// Print pack space and burden info
 	var FreeSlots = skapi.acoChar.citemMax - skapi.acoChar.citemContents;
 	Log.OutD( "Hourly: Free Pack Slots [" + FreeSlots + "] Burden [" + skapi.burdenCur + "]");

    	// Print component information
    	Comp.CheckAmounts();
	for (var x=0; x<Comp.Count; x++) {
    	    with (Comp.Info[x]) {
    	    	if (Enabled)
		    Log.OutD("Hourly: Comp [" + Name + "] Stock [" + Stock + "] SplitLvl [" + SplitLvl +
			     "] Peas [" + Peas + "] CritLvl [" + CritLvl + "] WarnLvl [" + WarnLvl + "]");
	    } // end with
	} // end for

	// Now dump out sellable info
	Log.OutD("Hourly: " + Loot.TypeInfo[lootTypeValuable].ItemCount + " valuable items worth " +
    		  Loot.TypeInfo[lootTypeValuable].TotalValue + ".");
	BuildValSalList();
	Log.OutD("Hourly: " + ValBagColl.Count + " bags of valuable salvage worth " + ValBagTotal + ".");
	Log.OutD("Hourly: End");
    } // endif

    // Check our equipment.  Re need to do this regardless of the buff flag as this routine also
    // checks our wand imbue, ie the function returns the rending damage type of wand we are using.
    attWandElement = Buff.CheckEquip();

    //  When free pack space is low we temporarily disable looting to increase macro efficency
    var FreePackSlots = skapi.acoChar.citemMax - skapi.acoChar.citemContents;
    if ((FreePackSlots == 0) || (FreePackSlots <= lootDisableThresh)) {
	// Free pack space is zero, or equal to or below loot disable threshold, so
	// mark looting as being completed this pass.
	CorpseCheckDone = true;
    } else {
	CorpseCheckDone = false;
    } // endif

    // If no more free pack space, then disable USTing for this idle pass
    if (FreePackSlots == 0) {
	UstCheckDone = true;
    } else {
    	UstCheckDone = false;
    } // endif

    if (debugIdle) Log.OutD("DoState_IdleInit: Changing state to IDLE...");
    StateChange(stateIdle);
    if (debug||debugIdle) Log.OutD("DoState_IdleInit: End");
} // end function


function DoState_Idle() {
    if (debug||debugIdle)
      Log.OutD("DoState_Idle: Begin, UstCheckDone [" + UstCheckDone + "] CorpseCheckDone [" + CorpseCheckDone + "]");

    // If creature close to us, then cancel IDLE state and deal with the danger
    if (Danger) {
    	if (debug||debugIdle) Log.OutD("DoState_Idle: Danger detected, aborting idle state...");
    	StateChange(stateSelect);
    	Player.EnsureCombatMode(true);
    	return;
    } // endif

    // If pointer is busy, then wait a few seconds to see if pointer becomes
    // nonbusy.  If it doesn't clear up, then hit the escape key a couple times
    // in case we are 'stuck' doing an action.
    if (skapi.fPointerBusy) PossiblyResetToon();

    // Disband fellowship if requested
    if (CharDisbandFlag) {
	CharDisbandFlag = false;
	Chat.Tell(CharDisbandReq, "WB: DISBANDing fellowship...");
	Player.DisbandFellow();
	CharLastDisband = new Date();
    } // endif

    // Process any fellowship requests from other players
    if (FellowFlag) {
	FellowFlag = false;
	Log.OutD("DoState_Idle: Attempting to fellow " + FellowChar);
	Chat.Tell(FellowChar, "WB: Attempting to fellow you...");
	Player.AddFellow(FellowChar);
    } // endif

    // If we are checking spell component levels...
    if (flagCompCheck) {
    	if (debugIdle) Log.OutD("DoState_Idle: Comp checking enable, seeing it time to check...");
    	if ((new Date() - CompLastCheck) > compInterval) {
    	    if (debugIdle) Log.OutD("DoState_Idle: Checking comps...");
	    var AllOk = Comp.CheckAmounts();
	    if (AllOk) {
    		CompLastCheck = new Date();
    		// If killing previously suspended because of low comps, then resume killing since comps are ok now.
    		if (AttCtx.CompSusp) {
	            Log.OutDM("DoState_Idle", "Comps ok now, resuming killing...", ToChat);
	            AttCtx.CompSusp = false;
		    CompResume();
		} //endif
	    } else {
		// Some comp is not at desired level, go check each one
		CompStartCheck = new Date();	// Time component check started
		CompIndex = 0;			// Index # of first comp we will be checking
		AlarmSounded = false;
		StateChange(stateCheckComps);
		return;
	    } // endif
   	} else {
	    if (debugIdle) Log.OutD("DoState_Idle: Not time to check comps.");
	} // endif
    } // endif

   // See if any full bags of salvage need to be moved to side packs
    if (flagMoveFull && BagToMove) {
    	if (debugIdle) Log.OutD("DoState_Idle: Moving full bags of salvage...");
	MoveBags();
	BagToMove = false;
    } // endif

    // If we are running out of free pack space or the amount of salvage on the loot list is too high,
    // then force an UST operation before looting
    if (flagAutoUst && !UstCheckDone) {
	var FreePackSlots = skapi.acoChar.citemMax - skapi.acoChar.citemContents;
	var ToBeSalCount = Loot.TypeInfo[lootTypeUstSalv].ItemCount + Loot.TypeInfo[lootTypeValSalv].ItemCount;
	if ((ToBeSalCount >= lootMaxUstPend) || ((FreePackSlots <= lootForceUst) && (ToBeSalCount > 0))) {
            // Force an UST operation.  The call to SalvageInit should return TRUE except when pack
            // is full in which case USTing is disabled and the return code is FALSE
            var Success = SalvageInit();
            if (Success) {
		if (debugIdle) Log.OutD("DoState_Idle: Salvage to UST (1), setting state to SALVAGE...");
		StateChange(stateSalvage);
		return;
	    } // endif
	} // endif
    } //endif

    // If looting is enabled, loot routine has not finished this pass, and there are corpses to process,
    // then goto LOOT state
    if (flagLootCheck && !CorpseCheckDone && (CorpseCount > 0)) {
	if (debugIdle) Log.OutD("DoState_Idle: Corpses to process, setting state to LOOT...");
	LootStart = new Date();
	CurCorpse = 0;
	SubState = stateLootFindCorpse;
	StateChange(stateLoot);
	return;
    } // endif

    // If auto USTing is enabled and there are items on the ust or valuable salvage lists then go ust them
    if (flagAutoUst && !UstCheckDone &&
        ((Loot.TypeInfo[lootTypeUstSalv].ItemCount > 0) || (Loot.TypeInfo[lootTypeValSalv].ItemCount > 0))) {
	// Force an UST operation.  The call to SalvageInit should return TRUE except
        // when pack is full in which case USTing is disabled and the return code is FALSE
        var Success = SalvageInit();
        if (Success) {
	    if (debugIdle) Log.OutD("DoState_Idle: Salvage to UST (2), setting state to SALVAGE...");
	    StateChange(stateSalvage);
	    return;
	} // endif
    } // endif

    // If learn unknown spell option checked, and the HaveUnkScrolls flag is set, then go through
    // the list of scrolls on the loot list and learn any unknown ones.
    if (debugIdle) Log.OutD("DoState_Idle: LearnUnknown [" + Loot.ScrollFilter.LearnUnknown +
    			    "] HaveUnkScrolls [" + HaveUnkScrolls + "]");
    if ((Loot.ScrollFilter.LearnUnknown) && HaveUnkScrolls) {
    	if (Loot.TypeInfo[lootTypeScroll].ItemCount == 0) {
    	    if (debugIdle) Log.OutD("DoState_Idle: No scrolls to process.");
	    HaveUnkScrolls = false;
	} else {
	    if (debugIdle) Log.OutD("DoState_Idle: Changing state to stateScroll...");
	    ScrollStart = new Date();
    	    StateChange(stateScroll);
	    return;
	} // endif
    } // endif

    // All maintenance functions have completed.  If retreat active and we are not already flagged
    // to advance to the kill location, then see if time to go back to the primary kill location.
    // Don't advance if we just died or we have suspended killing due to low comps
    if (RetreatActive && !AdvanceReq && !Dead && !AttCtx.CompSusp) {
    	// If we have been at the retreat location for at least the holddown period, then go back to
    	// the kill location for this attloc.
    	if ((new Date()-RetreatLastAtt)>retreatHoldDown) {
	    // Set flag to do the advance next FSM cycle.
	    if (debugIdle) Log.OutD("DoState_Idle: Ending retreat mode, setting AdvanceReq = true");
	    AdvanceReq = true;
	} else {
	    if (debugIdle) Log.OutD("DoState_Idle: Cannot end retreat mode, holddown in effect");
	} // endif
    } //endif

    if (AdvanceReq && !Dead) {
	AdvanceReq = false;
	// Go back to kill location.  Don't need any elaborate checks since zero creatures should be
	// near us at the moment
	RetreatActive = false;
	NavActive = true;

	Chat.Screen("Moving to kill location...");
	if (debugIdle|debugMove||logMove) Log.OutD("DoState_Idle: Advancing to kill location");
	if (Player.MemberFellow() && (PlayerNearCnt > 0) && (MoveGroup>0)) Chat.Fellow("ADVANCE " + MoveGroup);

	// Now move back to the kill location
	GotoKillLoc(navTimeOut);

	// Move done
	NavActive = false;
	HomeLoc = AttLoc[AttLocCur].KillLoc;
	if (flagMoveDelay) {
	    Chat.Screen("Arrived at kill location. Pausing for " + navWait + " ms...");
	    if (debugIdle|debugMove||logMove)
		Log.OutD("DoState_Idle: Arrived at kill loc. Pausing for " + navWait + " ms...");
	    skapi.WaitEvent(navWait, wemFullTimeout);
	} else {
	    Chat.Screen("Arrived at kill location.");
	    if (debugIdle|debugMove||logMove) Log.OutD("DoState_Idle: Arrived at kill location.");
	} // endif
    } //endif

    if (debugIdle) Log.OutD("DoState_Idle: Changing state to SELECT...");
    StateChange(stateSelect);

    if (debug||debugIdle) Log.OutD("DoState_Idle: End");
} // end func


// Find a target
function DoState_Select() {
    if (debug||debugSelect) Log.OutD("DoState_Select: Begin");

    // If pointer is busy, then wait a few seconds to see if pointer becomes
    // nonbusy.  If it doesn't clear up, then hit the escape key a couple times
    // in case we are 'stuck' doing an action.
    if (skapi.fPointerBusy) PossiblyResetToon();

    // If we are in peace mode (never attack) or support mode (no war) and
    // don't wont to imperl or vuln, then we do not attack creatures so just
    // return to idle state
    if ((AttCtx.ManSelOid == 0) && (PeaceMode || (attSupportOnly && !attAlwaysVuln && !attAlwaysImperl))) {
    	StateChange(stateIdleInit);
    	return;
    } // endif

    // Clear event queue to make sure we have most recent information
    skapi.WaitEvent(0);

    // Clear current creature select aco
    SelAco = null;
    SelLocked = false;

    // See if user manually selected a creature to attack
    if (AttCtx.ManSelOid != 0) {
	if (debugSelect) Log.OutD("SelCreature: Checking manual select...");
	if (SelManOid()) {
	    if (debugSelect) Log.OutD("SelCreature: Using manual select.");
	} else {
	    // Unable to select manual selection
    	    if (debugSelect) Log.OutD("SelCreature: Unable to select manual select.");
    	    SelAco = null;
	} // endif
    } // endif

    // If special flag set that indicates we are being attacked (either magic or melee), and
    // there are no creatures within our danger radius, then try to select the last creature
    // who attacked us (ubless a creature was manually selected through decal panel.
    if ((SelAco == null) && SelUseAttackAco && !Danger && AttackMode) {
   	if (debugSelect) Log.OutD("SelCreature: Checking last attacker...");
    	if (SelLastAttacker()) {
    	    if (debugSelect) Log.OutD("SelCreature: Using last attacker.");
	} else {
	    // Unable to select last attacker
    	    if (debugSelect) Log.OutD("SelCreature: Unable to select last attacker.");
    	    SelAco = null;
	} // endif
    } // endif

    // If couldn't select a creature using last attacker info, try the standard way
    if (SelAco == null) {
	if (debugSelect) Log.OutD("SelCreature: Looking for target in creature table...");
	SelCreature();
    } // endif

    // If no target can be found then...
    if (SelAco == null) {
 	if (debugSelect||debugMove) Log.OutD("DoState_Select: Unable to find creature to kill");
	if (AttackMode && AutoMove && (AttLocCnt > 1) && !RetreatActive && !attSupportOnly &&
	    ((new Date()-MoveEnd)>navHoldDown)) {
	    if (debug||debugSelect||debugMove||logMove)
		Log.OutD("DoState_Select: Unable to find cre to kill, moving to next attloc (state set to GotoNext)...");
	    StateChange(stateGotoNext);
	} else {
	    if (debug||debugSelect||debugMove) Log.OutD("DoState_Select: Switching to idle state");
	    StateChange(stateIdleInit);
	} // endif
	return;
    } // endif

    // Target found so try to select it. First calculate a priority for this target attempt which is used to decide
    // who continues the kill attempt when two or more players try to lock the same creature at the same time.
    AttemptPriority = (attExclPrio*1000) + Math.round(Math.random()*1000);
    if (logSelect||debugSelect) {
	    Log.OutD("DoState_Select: Candidate found.  Creature [" + SelAco.szName + "/" + Util.Hex(SelId) +
		     "] SelIndex [" + SelIndex + "] AttemptPrio [" + AttemptPriority + "]");
	    Log.OutD("DoState_Select: SelPrio [" + SelPrio + "] SelLocked [" + SelLocked + "] SelDist [" + SelDist + "]");
    } // endif

    if (debugSelect) {
	with (Creature.Info[SelIndex]) {
	    Log.OutD("DoState_Select: Name [" + Name + "] RaceCode [" + RaceCode + "]");
	    Log.OutD("DoState_Select: RaceName [" + Creature.RaceInfo[RaceCode].RaceName +
		     "] Vuln [" + Vuln1 + "] Vuln Name [" + damName[Vuln1] + "]");
	    Log.OutD("DoState_Select: Health [" + Health + "] Mana [" + Mana + "] Stam [" + Stamina +
	             "] MagicD [" + MagicD + "] Level [" + Level + "]");
	    Log.OutD("DoState_Select: Exp [" + Exp + "] Prio [" + Priority + "] SelFlag [" + SelFlag +
	             "] AttFlg [" + AttFlag + "] YieldFlag [" + YieldFlag + "]");
	    Log.OutD("DoState_Select: VulnFlag [" + VulnFlag + "] DangerFlag [" + DangerFlag + "]");
	} // end width
    } // endif

    // Initialize attack object and set attack parameters from selection info
    Att		 = new AttCreRecord();
    Att.Aco	 = SelAco;
    Att.Oid	 = SelId;
    Att.CreIndex = SelIndex;
    Att.Locked	 = SelLocked;
    Att.CreDist	 = SelDist;
    Att.CreName	 = Att.Aco.szName;

    // Creature found, now try to target it.  Function will set state to stateTarget before
    // attempting to select
    if (debugSelect) Log.OutD("DoState_Select: Trying to target creature...");
    var Status = Target();
    if (Status) {
	// Target was successfull, now setup kill attempt.  Note state will be set to stateKill in
	// this routine.
	KillInit();
	
	// Make sure we are in magic mode
	if (debugSelect) Log.OutD("DoState_Select: Ensuring Combat Mode...");
	Player.EnsureCombatMode(true);
    } else {
    	// Couldn't target creature
	if (debugSelect) Log.OutD("DoState_Select: Couldn't target creature.");
    } // endif
    return;
} // end function


// Try to kill
function DoState_Kill() {
    if (debug||debugAttack) Log.OutD("DoState_Kill: Begin, SubState [" + textSubKill[SubState] + "]");

    // Clear out events on queue to make sure we have the most recent info
    skapi.WaitEvent(0);

    // See if we are getting too many TooShort spell casting errors.  If so abort kill attempt and put
    // creature on DNAL
    if ((CastFailCnt-Att.FailStart) > attFailMax) {
	if (flagReportAbort) Chat.Screen("Too many spell casting errors, aborting kill attempt.");
	if (debugAttack||logAttack) Log.OutD("DoState_Kill: Too many spell casting errors, aborting kill attempt");
	AttPrevAdd(Att.Oid);
	if (attDNAL) SelNoAttackAdd(Att.Oid);	// Put on DNAL if list enabled
	KillDone();				// Kill attempt is finished so save some info and set next state
	if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
	return;
    } // endif

    // If a OnObjectDestroy event was received for this creature, then abort current kill attempt
    if (Att.Destroyed) {
	if (flagReportAbort) Chat.Screen("Creature destroy event received, aborting kill attempt.");
	KillDone();
	return;
    } // endif

    // If the attack context abort flag is set, then abort killing this creature
    if (AttCtx.Abort) {
    	AttCtx.Abort = false;
	if (flagReportAbort) Chat.Screen("Abort flag set, ending kill attempt...");
	if (debugAttack||logAttack) Log.OutD("DoState_Kill: Abort flag set, ending kill attempt");
	AttPrevAdd(Att.Oid);
	KillDone();		// Kill attempt is finished so save some info and set next state
	if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
	return;
    } // endif

    // See if creature is still selected. If not, then end this kill attempt (most likely someone
    // else killed creature or it ran away). It usually takes 1-2 seconds for a killed creature to be
    // removed from the object table.
    var AcoSelected = skapi.acoSelected;
    if (!attSupportOnly && (AcoSelected == null)) {
    	if (flagReportAbort) Chat.Screen("Creature no longer selected, aborting kill attempt.");
	if (debugAttack||logAttack) Log.OutD("DoState_Kill: Creature no longer selected; aborting kill attempt");
	AttPrevAdd(Att.Oid);
	AttCtx.LastLostSel = Att.Oid;

	// Kill attempt is finished so save some info and set next state
	KillDone();

	// Now unlock creature if we are in exclusion mode, we are the holder of the lock, and the last
	// distance check indicated it was not close to us.   We do the distance check since it is more
	// likely for a creature to be no longer selected because our partner killed it then it moved
	// away us if the last distance check indicated it was very close to us.
	if (ExclMode && !Att.Locked && (Att.CreDist > criticalRadius)) UnlockTarget(Att.Aco);
	return;
    } // endif

    // See if a different creature got selected somehow (probably by human operator pressing keys). Abort if so.
    if (!attSupportOnly && (Att.Oid != AcoSelected.oid)) {
    	if (flagReportAbort) Chat.Screen("Different creature selected, aborting kill attempt.");
	if (debugAttack||logAttack) Log.OutD("DoState_Kill: Different creature now selected, aborting kill attempt");
	AttPrevAdd(Att.Oid);

	// Kill attempt is finished so save some info and set next state
	KillDone();

	if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
	return;
    } // endif

    // If aco no longer exists (for example someone else killed the creature), then end kill attempts
    if (!Att.Aco.fExists) {
	if (flagReportAbort) Chat.Screen("Creature aco no longer valid, aborting kill attempt.");
	if (debugAttack||logAttack) Log.OutD("Creature aco no longer valid, aborting kill attempt.");
	AttPrevAdd(Att.Oid);

	// Kill attempt is finished so save some info and set next state
	KillDone();

	if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
	return;
    } // endif

    // If maploc of creature invalid then it has gone out of range
    if (Att.Aco.maploc == null) {
	if (flagReportAbort) Chat.Screen("Creature maploc no longer valid, aborting kill attempt.");
	if (debugAttack||logAttack) Log.OutD("Creature maploc no longer valid, aborting kill attempt.");
	AttPrevAdd(Att.Oid);

	// Kill attempt is finished so save some info and set next state
	KillDone();

	if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
	return;
    } // endif

    // Re-compute creature distance to us
    Att.CreDist = Util.Distance(skapi.maplocCur, Att.Aco.maploc);
    if (logAttack||debugAttack)
	Log.OutD("DoState_Kill: Cre coords [" + Util.FormatCoords(Att.Aco.maploc,3) + "] range [" + Att.CreDist + "]");

    // If attack is taking too long then we are having trouble killing the creature so add it to the DNAL.
    // Attack timeout varies on creature distance to us
    if (Att.CreDist<selRealClose)
	Att.MaxTime = attMaxTimeCrit;
    else
	Att.MaxTime = attMaxTime;

    if (debugAttack) Log.OutD("DoState_Kill: Att.MaxTime set to [" + Att.MaxTime + "]");

    if ((new Date() - Att.StartTime) > Att.MaxTime) {
    	if (flagReportAbort) Chat.Screen("Taking too long to kill creature , aborting kill attempt.");
	if (debugAttack||logAttack)
	    Log.OutD("DoState_Kill: Timeout. Adding creature [" + Att.CreName + "] to DNAL and aborting attack.");
	if (attDNAL) SelNoAttackAdd(Att.Oid);
	KillDone();	// Kill attempt is finished so save some info and set next state
	if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
	return;
    } // endif

    // If we are in a critical state (ie multiple creatures very close to us), and the currently selected
    // creature is outside our danger radius, then see if there is a selectable creature within our danger
    // radius.  If so then abort current kill attempt so we will acquire the most dangerous creature close
    // to us.  If multiple macroers in same fellow using exclusion mode are close to each other, then there
    // may not be any valid targets for us closeby since they may be locked.
    if (Critical && (Att.CreDist>dangerRadius)) {
    	if (debugAttack) Log.OutD("DoState_Kill: Critical condition and current creature outside danger radius");
	skapi.WaitEvent(0);	    			// Clear event queue to make sure we have most recent info
	MonsterList = Acf.CoacoGet();			// Build new monster list
	var NewTarget = FindCloseCre(dangerRadius);	// Now try to find a valid creature in the danger range

	// If target inside danger radius found and it is not being attacked by another player, then
	// switch attack target
	if (NewTarget && !SelLocked) {
	    if (flagReportAbort) Chat.Screen("Critical condition, aborting kill attempt and switching to closer target.");
	    if (debugAttack||logKills)
		Log.OutD("DoState_Kill: Crit abort, Att.Oid [" + Util.Hex(Att.Oid) + "] Att.CreExp [" + Att.CreExp + "]");
	    AttCtx.PrevCreExp = Att.CreExp;	// Save experience value of previous creature.
	    AttCtx.PrevId     = Att.Oid;	// Save ID so we don't attempt to reselect this creature.  This is
						// needed since it sometimes takes 2 seconds or so for creature to be
						// cleared out the of object table.
	    AttPrevAdd(Att.Oid);		// Save any debuffs on this creature so we don't recast vulns/imperls
	    					// if this creature is selected again.

	    if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);	// unlock creature if exclusion mode active

	    // Calculate a priority for this target attempt.  Used to decide who continues
	    // the kill attempt when two or more players try to lock the same creature.
	    AttemptPriority = (attExclPrio*1000) + Math.round(Math.random()*1000);

	    // Initialize attack object and set attack parameters from selection info
	    Att		   = new AttCreRecord();
	    Att.Aco	   = SelAco;
	    Att.Oid	   = SelId;
	    Att.CreIndex   = SelIndex;
	    Att.Locked	   = SelLocked;
	    Att.CreDist	   = SelDist;
	    Att.CreName	   = Att.Aco.szName;
	    Att.CritSelect = true;

	    // Creature found, now try to target it.  Function will set next state to Idle if target
	    // not found and return false
	    var Status = Target();
	    if (Status) {
		// Target was successfull, now setup kill attempt.  State will be set to stateKill in this routine.
		KillInit();
	    } else {
		// Couldn't target creature
		if (debugAttack||logAttack||debugTarget) Log.OutD("DoState_Kill: Couldn't target creature.");
	    } // endif

	    return;
	} // endif
    } // endif

    // Now check to see if creature is still in war range.  If not abort kill attempt
    if (Att.CreDist > SelRange) {
    	if (flagReportAbort) Chat.Screen("Creature out of range, aborting kill attempt.");
	if (debugAttack||logAttack) Log.OutD("DoState_Kill: Cre [" + Att.CreName + "] out of range, aborting kill attempt");
	AttPrevAdd(Att.Oid);
	KillDone();	// Kill attempt is finished so save some info and set next state
	if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
	return;
    } // endif

    // If this creature has not been successfully assessed yet, we haven't issued an assess recently,
    // creature is unknown and unknown assessing enabled or creature is known and known assessing enabled,
    // and last time we assessed creature was further away, then assess it
    with (Creature.Info[Att.CreIndex]) {
	if (debugAttack||debugAssCre)
	    Log.OutD("DoState_Kill: Cre [" + Att.CreName + "] Assessed [" + Assessed + "] Unknown [" + Unknown + "]");
	if (!Assessed && ((new Date() - Att.LastAssessTime) > attAssessInt) &&
	    ((Unknown && flagAssessUnknown) || (!Unknown && flagAssessKnown)) &&
	    ((Att.CreDist <= attAssessDist) || ((Att.CreDist > attAssessDist) && (Att.CreDist < Att.LastAssessDist)))) {
	    // Do the assess. We don't wait for the result.
	    if (logAttack||debugAttack||debugAssCre) Log.OutD("DoState_Kill: Assessing creature.");
	    // Chat.Screen("Assessing current creature.");
	    Att.LastAssessDist = Att.CreDist;
	    Att.LastAssessTime = new Date();
	    skapi.AssessAco(Att.Aco);
	} else {
	    if (debugAttack||debugAssCre)
		Log.OutD("DoState_Kill: No assess needed, Att.CreDist [" + Att.CreDist + "] attAssessDist [" +
			 attAssessDist + "] attAssessInt [" + attAssessInt +
			 "] Att.LastAssessTime [" + new Date(Att.LastAssessTime) + "]");
	} // endif
    } // end with

    // If automatic stepup of war spells to level 7 is enabled, then increase the maximum bolt and
    // arc spell we can cast to lvl 7 if we are in an Urgent condition or if the current creature
    // is dangerous and close to us.  This increase will last for the duration of the current kill.
    if (attWarStepup && !Att.StepupActive && (Urgent || (Att.CreDangerFlag && (Att.CreDist < dangerRadius)))) {
    	Att.StepupActive = true;
    	if (debugAttack) Log.OutD("DoState_Kill: Auto stepup in effect, setting max bolt and arc to lvl 7.");

    	// Raise the max level for bolt and/or arc spells if we know them
    	if (Spell.Info[Att.Element].ArcMap[7]) {
	    Att.MaxArc = 7;
    	    if (debugAttack) Log.OutD("DoState_Kill: Max arc spell set to 7");
	} else {
	    if (debugAttack) Log.OutD("DoState_Kill: Lvl7 arc spell not known for element [" + damName[Att.Element] + "]");
	} // endif

    	if (Spell.Info[Att.Element].BoltMap[7]) {
	    Att.MaxBolt = 7;
   	    if (debugAttack) Log.OutD("DoState_Kill: Max bolt spell set to 7");
	} else {
	    if (debugAttack) Log.OutD("DoState_Kill: Lvl7 bolt spell not known for element [" + damName[Att.Element] + "]");
	} // endif
    } // endif

    // Set maximum level spell levels for war spells.   We also do this check every iteration of the
    // kill routine since debuffs may happen at any time. First set maximum level for war bolts
    for (var x=Att.MaxBolt; x>0; x--) if (CharCurWar >= attWarThresh[x]) break;
    Att.CurBolt = Spell.Info[Att.Element].BoltStepDown[x];

    // Now set maximum level for arcs...
    for (var x=Att.MaxArc; x>0; x--) if (CharCurWar >= attWarThresh[x]) break;
    Att.CurArc = Spell.Info[Att.Element].ArcStepDown[x];

    if (debugAttack) Log.OutD("DoState_Kill: Att.CurBolt [" + Att.CurBolt  + "]  Att.CurArc [" + Att.CurArc + "]" );

    // Compute health of currently selected creature
    var CreCurHealth = Att.CreMaxHealth*Att.CreHealth;

    // If creatures CURRENT health is above the minimum drain threshold, our health is below
    // ThreshDrainHealth, the last time we drained health from the creature we got back a decent
    // amount, we haven't failed to drain too many times and haven't failed recently, and the
    // creature is in range of the drain spell we are using, then cast drain health to replenish
    // health.  Also don't do drain if we are in the process of yielding the creature or drains
    // are disabled for this creature type
    if ((CreCurHealth >= minDrainHealth) &&
        (skapi.healthCur < ThreshDrainHealth) && (SubState != stateKillYield) &&
        (Att.DrainHealthFails < attMaxHealthFails) &&
        ((new Date()-Att.LastDrainFail)>attDrainFailHold) &&
        !Att.CreNoDrainHealth &&
        (Att.CreDist < SpellObj_DrainHealth.range) &&  (Att.LastDrainHealth > usefulDrainHealth)) {
	if (debugAttack||logAttack)
	    Log.OutD("DoState_Kill: Draining creatures health, Cur health [" + skapi.healthCur +
	    	     "] Last Drain Health [" + Att.LastDrainHealth + "]" );
	Cast.Cast(SpellObj_DrainHealth, Att.Aco, false, true);
	if (CastCode != castCodeSuccess) {
	    Att.DrainHealthFails++;
	    if (CastFail == castFailOutRange) Att.LastDrainFail = new Date();
	    if (debugAttack||logAttack) Log.OutD("DoState_Kill: Drain health failed. " + SpellFailText());
	} else {
	    if (debugAttack||logAttack) Log.OutD("DoState_Kill: Drain health succeeded, Arc [" + CastArc + "]");
	} // endif
	if (SafeMode) return; // if we need to drain health then return immediately so FSM can check health again
    } // endif

    // If creatures INITIAL mana is above the minimum drain threshold, our mana is below
    // ThreshDrainMana, the last time we drained mana from the creature we got back a decent,
    // amount, and the creature is in range of the drain spell we are using, then cast drain mana
    // to replenish mana.  Don't do drain if we are in the process of yielding the creature, the
    // Urgent flag is set, or mana drains disabled for this creature
    if ((Att.CreMaxMana >= minDrainMana) && (skapi.manaCur < ThreshDrainMana) &&
	(SubState != stateKillYield) && ((!Urgent && !Att.CreDangerFlag) || attAlwaysDrain) &&
	(Att.DrainManaFails < attMaxManaFails) &&
        ((new Date()-Att.LastDrainFail)>attDrainFailHold) &&
	!Att.CreNoDrainMana &&
        (Att.CreDist < SpellObj_DrainMana.range) && (Att.LastDrainMana > usefulDrainMana)) {
	if (debugAttack||logAttack)
	        Log.OutD("DoState_Kill: Draining creatures mana, Cur mana [" + skapi.manaCur +
	    	         "] Last Drain Mana [" + Att.LastDrainMana + "]" );
	Cast.Cast(SpellObj_DrainMana, Att.Aco, false, true);
	if (CastCode != castCodeSuccess) {
	    Att.DrainManaFails++;
	    if (CastFail == castFailOutRange) Att.LastDrainFail = new Date();
	    if (debugAttack||logAttack) Log.OutD("DoState_Kill: Drain mana failed. " + SpellFailText());
	} else {
	    if (debugAttack||logAttack) Log.OutD("DoState_Kill: Drain mana succeeded, Arc [" + CastArc + "]");
	} // endif
	if (SafeMode) return; // Let FSM loop so it can check health
    } // endif

    // If creatures INITIAL stamina is above the minimum drain threshold, our stamina is below
    // ThreshDrainStam, the last time we drained stamina from the creature we got back a decent
    // amount, and the creature is in range of the drain spell we are using, then cast drain stamina
    // to replenish stamina.  Also don't do drain if we are in the process of yielding the creature
    // or stamina drains disabled for this creature
    if ((Att.CreMaxStamina >= minDrainStamina) && (skapi.staminaCur < ThreshDrainStamina) &&
	(SubState != stateKillYield) && ((!Urgent && !Att.CreDangerFlag) || attAlwaysDrain) &&
	(Att.DrainStaminaFails < attMaxStaminaFails) &&
	((new Date()-Att.LastDrainFail)>attDrainFailHold) &&
	!Att.CreNoDrainStamina &&
	(Att.CreDist < SpellObj_DrainStamina.range) &&  (Att.LastDrainStamina > usefulDrainStamina)) {
	if (debugAttack||logAttack)
	    Log.OutD("DoState_Kill: Draining creatures stamina, Cur stamina [" + skapi.staminaCur +
	    	     "] Last Drain Stamina [" + Att.LastDrainStamina + "]" );
	Cast.Cast(SpellObj_DrainStamina, Att.Aco, false, true);
	if (CastCode != castCodeSuccess) {
	    Att.DrainStaminaFails++;
	    if (CastFail == castFailOutRange) Att.LastDrainFail = new Date();
	    if (debugAttack||logAttack) Log.OutD("DoState_Kill: Drain stamina failed. " + SpellFailText());
	} else {
	    if (debugAttack||logAttack) Log.OutD("DoState_Kill: Drain stamina succeeded, Arc [" + CastArc + "]");
	} // endif
	if (SafeMode) return; // Let FSM loop so it can check health
    } // endif

    // Clear out events on queue to make sure we have the most recent info
    skapi.WaitEvent(0);

    // Now process kill substates.  We use a while loop to potentially process multiple substates
    // in a single FSM iteration.  We keep processing substates until a spell has to be cast
    while (true) {
	if (debugAttack) Log.OutD("DoState_Kill: Top of while loop, SubState [" + SubState + "]");

	if (SubState == stateKillYield) {
	    if (debugAttack) Log.OutD("DoState_Kill: Processing substate KillYield");
	    if (Att.YieldFlag == attNever) {
	    	if (debugAttack) Log.OutD("DoState_Kill: YieldFlag set to Never, skipping yield.");
	    	SubState = stateKillImperl;
		continue;	// No spell cast so do next substate immediately
	    } // endif

	    // If Att.YieldFlag == attAuto, then only yield if creatures magic d is x higher than our
	    // life or war skill.  Note we do this before each yield attempt because debuff expirations
	    // could render this yield unnecessary and the yield attempt can happen multiple times
	    // if the target resists.
	    if (Att.YieldFlag == attAuto) {
	    	if (debugAttack) Log.OutD("DoState_Kill: YieldFlag= auto, cre MD [" + Att.CreMagicD + "]");
	    	// Compare creature's magic d to our magic skills.  If no vuln or imperl is necessary (due to
	    	// using imbued wand or previously vulning creature), then only do check against our war skill.
	    	if (debugAttack)
	    	    Log.OutD("DoState_Kill: Flag=Auto, attYieldThresh [" + attYieldThresh + "] CharCurWar [" +
	    	    	     CharCurWar + "] CharCurLife [" + CharCurLife + "] Cre Magic D [" + Att.CreMagicD + "]");
	    	if (((Att.CreMagicD-CharCurWar) < attYieldThresh) &&
	    	    (((Att.CreMagicD-CharCurLife) < attYieldThresh) || ((Att.VulnFlag == attNever) && !attAlwaysImperl))) {
	    	    if (debugAttack) Log.OutD("DoState_Kill: Yield not needed");
		    // Goto to next substate
		    SubState = stateKillImperl;
	    	    continue;	// No spell cast so do next substate immediately
	    	} // endif
	    } // endif

	    // Set maximum level for yield based on our current creature magic skill
	    // level, our maximum level allowed, and what yield spell levels we know.
	    for (var x=Att.MaxYield; x>0; x--) if (CharCurCre >= attDebuffThresh[x]) break;
	    if ((x<Att.MaxYield) && debugAttack)
		Log.OutD("DoState_Kill: Yield stepped down to [" + x + "] due to insufficient creature magic skill");
	    AttCurYield = Spell.YieldStepDown[x];
	    if (debugAttack) Log.OutD("DoState_Kill: AttCurYield [" + AttCurYield + "]" );
	    if (AttCurYield == 0) {
	    	Chat.Screen("WARNING.  No yield spell known at level " + AttCurYield + " or below, skipping yield.");
		if (debugAttack) Log.OutD("DoState_Kill: No yield spell known at AttCurYield or below, skipping yield.");
		SubState = stateKillImperl;
	    	continue;	// No spell cast so do next substate immediately
	    } // endif

	    // Yield currently selected creature
	    if (debugAttack||logAttack)
	        Log.OutD("DoState_Kill: Casting yield [" + skapi.SzFromSpellid(Damage.Yield[AttCurYield]) +
			"] Attempt [" + Att.YieldAttempts + "]");
	    Att.YieldSpell = skapi.SpellInfoFromSpellid(Damage.Yield[AttCurYield]);
	    Cast.Cast(Att.YieldSpell, Att.Aco, true, true);
	    if ((CastCode != castCodeSuccess) && (CastFail == castFailOutRange)) {
		if (debugAttack||logAttack)
		    Log.OutD("DoState_Kill: Yield Cre [" + Att.CreName + "] out of range, aborting kill attempt");
		AttPrevAdd(Att.Oid);
		OutOfRangeAdd(Att.Oid);
		KillDone();	// Kill attempt is finished so save some info and set next state
		if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
		return;
	    } // endif

	    // If this is the first yield attempt, sleep for a few milliseconds after yield.
	    // This allows player to rotate towards target before casting next spell
	    // if (Att.YieldAttempts == 0) skapi.WaitEvent(300, wemFullTimeout);

	    // If we failed the yield attempt, increment attempt counter and exit so we can try again
	    if ((CastCode != castCodeSuccess) && (Att.YieldAttempts < attMaxYields)) {
		Att.YieldAttempts++;
		if (debugAttack||logAttack) Log.OutD("DoState_Kill: Yield failed. " + SpellFailText());
		return;
	    } // endif

	    // If we make it here, then either cast was successful or we reached the limit for
	    // yield attempts. In either case just go to next stage.
	    if (CastCode != castCodeSuccess) {
		if (debugAttack||logAttack)
		    Log.OutD("DoState_Kill: Yield failed, retries exhausted [" + attMaxYields + "] " + SpellFailText());
	    } else {
		if (debugAttack||logAttack) Log.OutD("DoState_Kill: Yield succeeded");
		Att.YieldTime = new Date();
	    } // endif

	    SubState = stateKillImperl;
	    return;	// Spell was cast, so let FSM execute
	} // endif

	if (SubState == stateKillImperl) {
	    if (debugAttack) Log.OutD("DoState_Kill: Processing substate KillImperl");
	    if (!Att.ImperlFlag) {
	    	SubState = stateKillVuln;
		continue;	// No spell cast so do next substate immediately
	    } // endif

	    // Set maximum level for imperl based on our current life magic skill
	    // level, our maximum level allowed, and what imperl spell levels we know.
	    for (var x=Att.MaxImperl; x>0; x--) if (CharCurLife >= attDebuffThresh[x]) break;
	    if ((x<Att.MaxImperl) && debugAttack)
		Log.OutD("DoState_Kill: Imperl stepped down to [" + x + "] due to insufficient life magic skill");
	    AttCurImperl = Spell.ImperlStepDown[x];
	    if (debugAttack) Log.OutD("DoState_Kill: AttCurImperl [" + AttCurImperl + "]" );
	    if (AttCurImperl == 0) {
	    	Chat.Screen("WARNING. No imperl spell known at level " + AttCurImperl +
	    		    " or below, skipping imperl.", cmcYellow);
		if (debugAttack)
		    Log.OutD("DoState_Kill: No imperl spell known at AttCurImperl or below, skipping imperl.");
		SubState = stateKillVuln;
	    	continue;	// No spell cast so do next substate immediately
	    } // endif

	    // Imperl currently selected creature
	    if (debugAttack||logAttack)
	        Log.OutD("DoState_Kill: Casting imperl [" + skapi.SzFromSpellid(Damage.Imperl[AttCurImperl]) +
			"] Attempt [" + Att.ImperlAttempts + "]");
	    Att.ImperlSpell = skapi.SpellInfoFromSpellid(Damage.Imperl[AttCurImperl]);
	    Cast.Cast(Att.ImperlSpell, Att.Aco, true, true);
	    if ((CastCode != castCodeSuccess) && (CastFail == castFailOutRange)) {
		if (debugAttack||logAttack)
		    Log.OutD("DoState_Kill: Imperl Cre [" + Att.CreName + "] out of range, aborting kill attempt");
		AttPrevAdd(Att.Oid);
		OutOfRangeAdd(Att.Oid);
		KillDone();	// Kill attempt is finished so save some info and set next state
		if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
		return;
	    } // endif

	    // If we failed the imperl attempt, increment attempt counter and exit so we can try again
	    if ((CastCode != castCodeSuccess) && (Att.ImperlAttempts < attMaxImperls)) {
		Att.ImperlAttempts++;
		if (debugAttack||logAttack)
		    Log.OutD("DoState_Kill: Imperl failed. " + SpellFailText());
		return;
	    } // endif

	    // If we make it here, then either cast was successful or we reached the limit for
	    // imperl attempts. In either case just go to next stage.
	    if (CastCode != castCodeSuccess) {
		if (debugAttack||logAttack)
		    Log.OutD("DoState_Kill: Imperl failed, retries exhausted [" + attMaxImperls + "] " + SpellFailText());
	    } else {
		if (debugAttack||logAttack) Log.OutD("DoState_Kill: Imperl succeeded");
		Att.ImperlTime = new Date();
	    } // endif

	    SubState = stateKillVuln;
	    return;	// Spell was cast, so let FSM execute
	} // endif

	if (SubState == stateKillVuln) {
	    if (debugAttack) Log.OutD("DoState_Kill: Processing substate KillVuln");

	    if (Att.VulnFlag == attNever) {
	    	if (debugAttack) Log.OutD("DoState_Kill: VulnFlag set to Never, skipping vuln.");
	    	SubState = stateKillWar;
		continue;	// No spell cast so do next substate immediately
	    } // endif

	    // Set maximum level for vuln based on our current life magic skill level and our maximum level allowed.
	    for (var x=Att.MaxVuln; x>0; x--) if (CharCurLife >= attDebuffThresh[x]) break;
	    if ((x<Att.MaxVuln) && debugAttack)
		Log.OutD("DoState_Kill: Vuln stepped down to [" + x + "] due to insufficient life magic skill");
	    Att.CurVuln = Spell.Info[Att.Element].VulnStepDown[x];
	    if (Att.CurVuln == 0) {
	    	// This character doesn't know any vuln spells for this element type that it
	    	// can reliable cast, skipping vuln...
		if (debugAttack)
		    Log.OutD("DoState_Kill: No vuln available at or below level [" + x + "] for element [" +
		    	      Att.Element + "/" + damName[Att.Element ] + "]");
		SubState = stateKillWar;
		continue;	// No spell cast so do next substate immediately
	    } // endif

	    // If VulnFlag is set to Auto, then determine whether or not to vuln the creature based on its health
	    if (Att.VulnFlag == attAuto) {
		if (debugAttack) Log.OutD("DoState_Kill: Vuln Flag = auto, seeing if vuln necessary");
	    	if (!AttCheckVulnFlag(CreCurHealth)) {
		    // Vuln not required, set substate to war and return
		    if (debugAttack) Log.OutD("DoState_Kill: Vuln not required, switching substate to war");
		    SubState = stateKillWar;
		    continue;	// No spell cast so do next substate immediately
		} // endif
	    } // endif

	    // If we reach here, than we are going to cast a vuln.  If the highest spell we
	    // can cast is a lvl 7 spell, then see if we can cast a lower level vuln then the
	    // maximum (based on the health of the creature).  Note we will not do this if
	    // multiple creatures are near us or the currently selected creature is dangerous
	    // or has the 'NoStepDown' flag set.  This option can be enabled/disable through the
	    // config variables 'attVulnStepdown'
	    if (attVulnStepdown && (Att.CurVuln == 7) && !Urgent && !Att.CreNoStepDown && !Att.CreDangerFlag) {
	    	NextLowerVuln = Spell.Info[Att.Element].VulnStepDown[Att.CurVuln-1];
	    	if (NextLowerVuln > 0) {
		    if (EstimatedSpells(NextLowerVuln,CreCurHealth) <= EstimatedSpells(Att.CurVuln,CreCurHealth)) {
		    	// Same # of spells to kill creature, so step down vuln
		    	if (debugAttack)
			    Log.OutD("DoState_Kill: Stepping down vuln, Old: " + Att.CurVuln + " New: " + NextLowerVuln);
		    	Att.CurVuln = NextLowerVuln;
		    } else {
		    	// Stepdown would cause more was spells to be cast, skipping
		    	if (debugAttack)
			    Log.OutD("DoState_Kill: Stepdown vuln level [" + NextLowerVuln +
			    	     "] needs more spells then current level [" + Att.CurVuln + "]");
		    } // endif
		} else {
		    // This character doesn't have any vuln spells below current
		    if (debugAttack) Log.OutD("DoState_Kill: No vuln available below current, no stepdown possible");
		} // endif
	    } // endif

	    // Cast vuln
	    if (debugAttack||logAttack)
	        Log.OutD("DoState_Kill: Casting vuln [" + skapi.SzFromSpellid(Damage.Info[Att.Element].Vuln[Att.CurVuln]) +
	        	 "] Elem [" + Att.Element + "] Attempt [" + Att.VulnAttempts + "]");

	    Att.VulnSpell = skapi.SpellInfoFromSpellid(Damage.Info[Att.Element].Vuln[Att.CurVuln]);
	    Cast.Cast(Att.VulnSpell, Att.Aco, true, true);
	    if ((CastCode != castCodeSuccess) && (CastFail == castFailOutRange)) {
		if (debugAttack||logAttack)
		    Log.OutD("DoState_Kill: Vuln Cre [" + Att.CreName + "] out of range, aborting kill attempt");
		AttPrevAdd(Att.Oid);
		OutOfRangeAdd(Att.Oid);
		KillDone();	// Kill attempt is finished so save some info and set next state
		if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
		return;
	    } // endif

	    // If we failed the vuln attempt, increment attempt counter and exit so we can try again
	    if ((CastCode != castCodeSuccess) && (Att.VulnAttempts < attMaxVulns)) {
		Att.VulnAttempts++;
	        if (debugAttack||logAttack)
		    Log.OutD("DoState_Kill: Vuln [" + Att.VulnAttempts + "] failed. " + SpellFailText());
		return;
	    } // endif

	    // If we make it here, then either cast was successful or we reached the limit for vuln
	    // attempts. In either case just go to next stage.
	    if (CastCode == castCodeSuccess) {
		if (debugAttack||logAttack) Log.OutD("DoState_Kill: Vuln succeeded, Arc [" + CastArc + "]");
	    	Att.VulnTime   = new Date();
	    	Att.VulnActive = true;
	    	Att.VulnLevel  = Att.CurVuln;
	    } else {
	    	Att.VulnAttempts++;
		if (debugAttack||logAttack)
		    Log.OutD("DoState_Kill: Vuln [" + Att.VulnAttempts + "] failed, retries exhausted [" +
		    	     attMaxVulns + "] " + SpellFailText());
	    } // endif

	    if (debugAttack) Log.OutD("DoState_Kill: substate switch to war");
	    SubState = stateKillWar;
	    return;
	} // endif

	if (SubState == stateKillWar) {
	    if (debugAttack) Log.OutD("DoState_Kill(War): case stateKillWar");

	    // If we are in support mode, then just mark the creature as been
	    // processed (ie imped/vulned).  This will stop us from selecting
	    // it again.
	    if (attSupportOnly) {
		if (debugAttack) Log.OutD("DoState_Kill: support mode, ending kill");
		KillDone();
		return;
	    } // endif

	    // If ring spell conditions	are met	and the	current	creature is close to us, then
	    // use a ring spell	for the	war spell, else	use bolt or arc.
	    if (RingOk && (Att.CreDist<=attRingDist)) {
		WarSpellId = Damage.Info[Att.Element].Ring;
		if (debugAttack) Log.OutD("DoState_Kill: Ring spell being used, spellid [" + WarSpellId + "]");
	    } else {
		// Set maximum level for bolts.	 We do these checks before every war spell cast	since
		// debuffs may happen at any time.
		for (var x=Att.MaxBolt;	x>0; x--) if (CharCurWar >= attWarThresh[x]) break;
		Att.CurBolt = Spell.Info[Att.Element].BoltStepDown[x];
		if ((x<Att.MaxBolt) && debugAttack)
		    Log.OutD("DoState_Kill: Bolt stepped down to [" + Att.CurBolt + "] due to low war magic skill");

		// Set maximum level for arcs
		for (var x=Att.MaxArc; x>0; x--) if (CharCurWar	>= attWarThresh[x]) break;
		Att.CurArc = Spell.Info[Att.Element].ArcStepDown[x];
		if ((x<Att.MaxArc) && debugAttack)
		    Log.OutD("DoState_Kill: Arc	stepped	down to	[" + Att.CurArc	+ "] due to low	war magic skill");

		// Process all events to make sure we have the creatures most recent health info.
		// Since the OnTargetHealth events only	come every 2 or	3 seconds, the creatures
		// health is a slightly	stale most of the time.
		skapi.WaitEvent(0);
		var CreCurHealth = Att.CreMaxHealth*Att.CreHealth;

		// Determine what war spell to cast based on setting of	'Att.WarSpell' variable. Possible
		// values for this variable are:
		//   attSpellBolt   Use	Bolts only for war spells
		//   attSpellArc    Use	Arcs only for war spells
		//   attSpellAuto   Use	highest	spell we have for a particular damage type.  If	a tie
		//		    use	bolts if creature is moving and	far away from us, else use arcs
		if (Att.WarSpell == attSpellBolt) {
		    if (attWarStepdown && !Urgent && !Att.CreNoStepDown	&& !Att.CreDangerFlag)
			Att.CurBolt = StepDownBolt(CreCurHealth);
		    WarSpellId = Damage.Info[Att.Element].Bolt[Att.CurBolt];
		    if (debugAttack)
			Log.OutD("DoState_Kill(War): Bolt, Lvl [" + Att.CurBolt	+ "] Elem [" + Att.Element + "]");
		} // endif

		if (Att.WarSpell == attSpellArc) {
		    if (attWarStepdown && !Urgent && !Att.CreNoStepDown	&& !Att.CreDangerFlag)
			Att.CurArc = StepDownArc(CreCurHealth);
		    WarSpellId = Damage.Info[Att.Element].Arc[Att.CurArc];
		    if (debugAttack)
			Log.OutD("DoState_Kill(War): Arc, Lvl [" +	Att.CurArc + "]	Elem ["	+ Att.Element +	"]");
		} // endif

		if (Att.WarSpell == attSpellAuto) {
		    // If the bolt and arc spell max levels are	not equal, then	pick the higher	one
		    if (Att.CurBolt != Att.CurArc) {
			if (debugAttack) Log.OutD("DoState_Kill(War): Arc and bolt spell not equal, using higher spell.");
			if (Att.CurBolt	> Att.CurArc) {
			    if (debugAttack) Log.OutD("DoState_Kill(War): Using	bolt spell");
			    if (attWarStepdown && !Urgent && !Att.CreNoStepDown	&& !Att.CreDangerFlag)
				Att.CurBolt = StepDownBolt(CreCurHealth);
			    WarSpellId = Damage.Info[Att.Element].Bolt[Att.CurBolt];
			} else {
			    if (debugAttack) Log.OutD("DoState_Kill(War): Using	arc spell");
			    if (attWarStepdown && !Urgent && !Att.CreNoStepDown	&& !Att.CreDangerFlag)
				Att.CurArc = StepDownArc(CreCurHealth);
			    WarSpellId = Damage.Info[Att.Element].Arc[Att.CurArc];
			} // endif
		    } else {
			if (debugAttack) Log.OutD("DoState_Kill(War): Max arc and bolt spells equal");
			// Max bolt and	arc spell are equal so see if the creature has moved since last	check.
			// If so then set flag indicating creature is moving.  This affects what war spell we
			// will	use (arc or bolt). Once	we detect creature is moving, we do not	consider it not
			// moving until	it is in the same position for 'attMoveHoldDown' milliseconds.
			AttMoveDist = Util.Distance(Att.Aco.maploc,Att.PrevLoc);
			if (AttMoveDist	> attMoveThresh) {
			    // Creature	is moving
			    Att.Moving = true;
			    Att.LastMove = new Date();
			    if (debugAttack) Log.OutD("DoState_Kill(War): Creature moving");
			    Att.PrevLoc	= Att.Aco.maploc;
			} else {
			    // Creature	not moving.  If	creature hasn't	moved for 'attMoveHoldDown' millseconds
			    // then set	moving flag to false.  We need the hold	down period because if two checks
			    // happen very close together, we may not get the event that updates the creatures
			    // location	and we would falsely assume the	creature was not moving. The holddown
			    // time is very unlikely to	be used	since a	war spell cast is several seconds long.
			    if ((new Date()-Att.LastMove) > attMoveHoldDown) {
				// Set flag indicating we are not moving
				Att.Moving = false;
				if (debugAttack) Log.OutD("DoState_Kill(War): Creature not moving");
			    } else {
				// Creature not	moving but holddown period not expired
				if (debugAttack)
				    Log.OutD("DoState_Kill(War): Creature seems	to be not moving but holddown in effect");
			    } // endif
			} // endif

			if (debugAttack)
			    Log.OutD("DoState_Kill(War): SpellAuto, CurBolt [" + Att.CurBolt + "] CurArc [" + Att.CurArc +
				     "]	Att.Moving [" +	Att.Moving + "]	attArcRadius ["	+ attArcRadius +
				     "]	Att.CreDist [" + Att.CreDist + "]");

			// If creature is moving and outside our arc radius, then use a	bolt, else use an arc
			if (Att.Moving && (Att.CreDist>attArcRadius)) {
			    // Use a bolt
			    if (attWarStepdown && !Urgent && !Att.CreNoStepDown	&& !Att.CreDangerFlag)
				Att.CurBolt = StepDownBolt(CreCurHealth);
			    WarSpellId = Damage.Info[Att.Element].Bolt[Att.CurBolt];
			    if (debugAttack)
				Log.OutD("DoState_Kill:	AutoBolt, Lvl [" + Att.CurBolt + "] Elem [" + Att.Element + "]");
			} else {
			    if (attWarStepdown && !Urgent && !Att.CreNoStepDown	&& !Att.CreDangerFlag)
				Att.CurArc = StepDownArc(CreCurHealth);
			    WarSpellId = Damage.Info[Att.Element].Arc[Att.CurArc];
			    if (debugAttack)
				Log.OutD("DoState_Kill:	AutoArc, Lvl ["	+ Att.CurBolt +	"] Elem	[" + Att.Element + "]");
			} // endif
		    } // endif
		} // endif
	    } // endif

	    if (debugAttack||logAttack) Log.OutD("DoState_Kill: Casting war spell ["+skapi.SzFromSpellid(WarSpellId)+"]");
	    Cast.Cast(skapi.SpellInfoFromSpellid(WarSpellId), Att.Aco, false, true);
	    if ((CastCode != castCodeSuccess) && (CastFail == castFailOutRange)) {
	    	if (flagReportAbort) Chat.Screen("Creature out of range, aborting kill attempt.");
		if (debugAttack||logAttack)
		    Log.OutD("DoState_Kill: War, Creature [" + Att.CreName + "] out of range, aborting kill attempt");
		AttPrevAdd(Att.Oid);
		OutOfRangeAdd(Att.Oid);
		KillDone();	// Kill attempt is finished so save some info and set next state
		if (ExclMode && !Att.Locked) UnlockTarget(Att.Aco);
		return;
	    } // endif

	    if (CastCode != castCodeSuccess) {
		if (debugAttack||logAttack) Log.OutD("DoState_Kill: War failed. " + SpellFailText());
	    } else {
	    	Att.WarSpellCnt++;
	    	if (logAttack||debugAttack)
		    Log.OutD("DoState_Kill: War success, Arc [" + CastArc + "] WarSpellCnt [" +  Att.WarSpellCnt + "]");
	    } // endif

	    break;
	} // endif
    } // end while
} // End func DoStateKill


function DoState_CheckComps() {
    if (debug||debugComps) Log.OutD("DoState_CheckComps: Begin");

    // If creature close to us, then cancel comp checking and deal with the danger
    if (Danger) {
    	 if (debug||debugComps) Log.OutD("DoState_CheckComps: Danger detected, aborting check...");
    	StateChange(stateSelect);
    	Player.EnsureCombatMode(true);
    	return;
    } // endif

    // If we have reached the maximum amount of time permitted in this routine, then exit
    if ((new Date()-CompStartCheck) > compCheckMax) {
	if (debugComps) Log.OutD("DoState_CheckComps: Timeout, aborting CheckComps...");
	CompLastCheck = new Date();
	if (debug||debugComps) Log.OutD("DoState_CheckComps: Changing state to IDLE...");
	StateChange(stateIdle);
	return;
    } // endif

    // Find the next comp that is low
    if (debugComps) Log.OutD("DoState_CheckComps: CompIndex [" + CompIndex + "]");
    while ((CompIndex < Comp.Count) &&
           ((!Comp.Info[CompIndex].Enabled) ||
            ((Comp.Info[CompIndex].Stock >= Comp.Info[CompIndex].SplitLvl) &&
             (Comp.Info[CompIndex].Stock >= Comp.Info[CompIndex].CritLvl) &&
             (Comp.Info[CompIndex].Stock >= Comp.Info[CompIndex].WarnLvl))))
		CompIndex++;

    if (debugComps) Log.OutD("DoState_CheckComps: CompIndex [" + CompIndex + "] End [" + Comp.Count + "]");

    // If we have reached the end of the list then exit
    if (CompIndex == Comp.Count) {
	if (debugComps) Log.OutD("DoState_CheckComps: Done checking, changing state to IDLE...");
	CompLastCheck = new Date();
	StateChange(stateIdle);
	return;
    } // endif

    with (Comp.Info[CompIndex]) {
	if (flagSplitPeas && (Stock < SplitLvl)) {
	    // We found a component below the required level and pea splitting has been enabled so
	    // try to split peas.
	    if (debugComps)
		Log.OutD("DoState_CheckComps: Comp [" + Name + "/" + CompIndex + "] below split lvl [" + SplitLvl+ "]");

	    // Now try to split a pea
	    var Status = Comp.Split(Name);
	    if (Status == compSuccess) {
		// Split was successful, now see if we need more of this comp
		skapi.WaitEvent(200, wemFullTimeout);	// Make sure inventory information updated
		var NewAmount = skapi.CitemOfKindInInv(Name);
		if (Stock == NewAmount) {
		    // Component level didn't change even though status code indicated success.
		    // Let program continue to next section for error processing
		    if (debugComps) Log.OutD("DoState_CheckComps: Error, status success but comp lvl didnt change");
		} else {
		    // If we still don't have enough comps, then don't increment CompIndex which we do by
		    // returning from this function now.  This will cause the same component to be checked
		    // again
		    Comp.Info[CompIndex].Stock = NewAmount;
		    if (NewAmount >= Comp.Info[CompIndex].SplitLvl) {
			if (debugComps)
			    Log.OutD("DoState_CheckComps: Comp level above min, checking critical and warning levels");
		    } else {
			if (debugComps) Log.OutD("DoState_CheckComps: Comp still low, rechecking");
			return;
		    } // endif
		} // endif
	    } // endif
	} // endif

	// Now see if the critical level for the comp has been reached. If we are equal or below the
	// critical limit then either logout our toon or go into defense mode depending on the setting
	// of the variable 'flagCompLogOff'
	if (Stock < CritLvl) {
	    if (flagCompLogOff) {
	        Log.OutDM("DoState_CheckComps", "Comp " + Name + " is below critical level, logging...", ToChat);
		NotifySupport(Util.FormatTime() + " Comp " + Name + " is below critical threshold, logging...");
		StateChange(stateLog,NoCheck);
	    } else {
	    	if (!AttCtx.CompSusp) {
		    AttCtx.CompSusp = true;
		    Log.OutDM("DoState_CheckComps", "One or more comps below critical level, killing suspended.", ToChat);
		    NotifySupport(Util.FormatTime() + " One or more comps below critical level, killing suspended");

		    // Save current AttackMode and AutoMove values
		    SuspAttackMode = AttackMode
		    SuspAutoMove = AutoMove;

		    // Disable attacking if currently enabled
          	    if (AttackMode) {
          		AttackMode = false;
          		skapi.SetControlProperty(szPanelControl, "btnAttackMode", "Text", "Enable Attack");
		    } // endif

          	    // Also disable AutoMove if enabled
          	    if (AutoMove) {
          		AutoMove = false;
          		skapi.SetControlProperty(szPanelControl, "btnAutoMove", "Text", "Enable AutoMove");
          	    } // endif

          	    // Play audible alert if enabled
          	    if (alertComps && !AlarmSounded) {
          		alertCompsWav.Play();
          		AlarmSounded = true;	// Only play alarm once per comp check run
          	    } // endif

		    // Goto current attack locations retreat point if enabled and we are not close to it.
		    // This is useful when macroing in a group and need to get out of the kill zone.
		    if (flagCompRetreat && (AttLoc[AttLocCur].RetreatLoc != null)) {
		    	// Make sure retreat mode is cleared so we don't move back to the kill loc
		    	if (RetreatActive) {
			    RetreatActive = false;
			    Log.OutD("DoState_CheckComps: Retreat mode cancelled.");
			} //endif
		        var RetreatDist = Util.Distance(AttLoc[AttLocCur].RetreatLoc, skapi.maplocCur);
		        if (RetreatDist > 0.02) {
			    Log.OutDM("DoState_CheckComps", "Moving to retreat location...", ToChat);
			    NavActive = true;
			    GotoRetreatLoc(navTimeOut);
			    NavActive = false;
		        } // endif
		    } // endif
		} // endif

		// If an alarm hasn't been spammed recently for this comp, then alarm it
		if (new Date()-LastAlarm > compSpamInt) {
		    LastAlarm = new Date();
		    var Msg = Util.FormatCoords(skapi.maplocCur,2) + "  Comp " + Name +
	    		   " below critical level.  Remaining: " + Stock + "  Resume level: " +
	    		   Math.max(SplitLvl,WarnLvl);

		    // Spam comp low condition
		    Log.OutDM("DoState_CheckComps", Msg, ToChat);
		    NotifySupport(Util.FormatTime() + " " + Msg);
		} // endif

		// Check next comp
		CompIndex++;
	    } // endif
	    return;
	} // endif

	// If variable 'flagCompAssist' is true, then see if component level is below
	// its warning level.  If it is, then broadcast our shortage to the fellow.
	// Hopefully some helpful fellow member will resupply us.

	// If component below warning level, then spam message to fellowship and/or play audible alert
	if (Stock < WarnLvl) {
	    if (debugComps)
		Log.OutD("DoState_CheckComps: Comp [" + Name + "/" + CompIndex + "] below warnlvl [" +
			 WarnLvl + "] Critical Thresh [" + CritLvl + "]");

	    if (flagCompAssist) {
		if (logComp)
		    Log.OutD("DoState_CheckComps: Comp [" + Name + "/" + CompIndex + "] Amount [" + Stock +
		    	     "] is below warnlvl [" + WarnLvl + "] Log Thresh [" + CritLvl + "] spamming fellow...");
	        var OurLoc = skapi.maplocCur;
	        NotifySupport(Util_FormatTime() + " Comp request at location " + Util.FormatCoords(OurLoc,2) +
			      ".  Low on " + Name + ", Amount Left: " + Stock + "  Critical Thresh: " + CritLvl);
	    } // endif

	    if (alertComps) alertCompsWav.Play();
	    Chat.Screen("Low on " + Name + ", Amount Left: " + Stock + "  Critical Thresh: " + CritLvl);
	} // endif
    } // end with

    // Check next comp
    CompIndex++;

    if (debug||debugComps) Log.OutD("DoState_CheckComps: End");
    return true;
} // end function


function DoState_Loot() {
    if (debug||debugLoot) Log.OutD("DoState_Loot: Begin, SubState: "+textSubLoot[SubState]+", CorpseCount: "+CorpseCount);

    // If creature close to us, then cancel loot processing and deal with the danger
    if (Danger) {
    	if (debug||debugLoot) Log.OutD("DoState_Item: Danger detected, aborting loot processing...");
    	StateChange(stateSelect);
    	Player.EnsureCombatMode(true);
    	return;
    } // endif

    // Make sure all events processed
    skapi.WaitEvent(0);

    switch (SubState) {
	case stateLootFindCorpse:
	    // If we have spent too much time in the loot state, then abort so other states
	    // can be processed (like buffing)
	    if ((new Date()-LootStart)>lootMaxInt) {
	    	 if (debugLoot) Log.OutD("DoState_Loot: Time exceeded, aborting loot state...");
	    	 CorpseCheckDone = true;
	    	 StateChange(stateIdle);
	    	 return;
	    } // endif

	    // If there are no more corpses to process, then abort
	    if (CurCorpse >= CorpseCount) {
	     	 if (debugLoot) Log.OutD("DoState_Loot: All corpses processed");
	     	 SubState = stateLootDone;
	    	 break;
	    } // endif

	    // Calculate the number of free pack slots left in main pack
	    var FreePackSlots = skapi.acoChar.citemMax - skapi.acoChar.citemContents;

	    // If we are running out of free pack space or the amount of salvage on the loot list is too high,
	    // then force an UST operation before looting any more creature.
	    if (flagAutoUst && (FreePackSlots > 0)) {
		var ToBeSalCount = Loot.TypeInfo[lootTypeUstSalv].ItemCount + Loot.TypeInfo[lootTypeValSalv].ItemCount;
		if ((ToBeSalCount >= lootMaxUstPend) || ((FreePackSlots <= lootForceUst) && (ToBeSalCount > 0))) {
		    // Force an UST operation
		    var Success = SalvageInit();
		    if (Success) {
			if (debugIdle) Log.OutD("DoState_Loot: Salvage to UST, setting state to SALVAGE...");
			StateChange(stateSalvage);
			return;
		    } // endif
		} // endif
	    } // endif

	    //  When free pack space is low or zero we temporaily disable looting to increase macro efficency
	    if ((FreePackSlots == 0) || (FreePackSlots <= lootDisableThresh)) {
		// Free pack space is equal to or below loot auto disable threshold.
		Log.OutDM("DoState_Loot", "Free space below disable threshold, looting temporarily disabled.", ToChat);
		SubState = stateLootDone;
	    	break;
	    } // endif

	    while (CurCorpse<CorpseCount) {
		// See if current corpse valid by checking the Aco and also the back translated
		// Aco derived from the oid.  If corpse has decayed then remove it from list
		if ((CorpseList[CurCorpse].Aco == null) ||
		    !(CorpseList[CurCorpse].Aco.fExists) ||
		    (skapi.AcoFromOid(CorpseList[CurCorpse].Oid) == null)) {
		    // Remove from list.  Since we want to preserve the corpse order (so that older
		    // corpses are looted first), we must shift down the entries by 1
		    if (debugLoot) Log.OutD("DoState_Loot: Index [" + CurCorpse + "] decayed, discarding...");
		    CorpseCount--;
		    for (var x=CurCorpse; x<CorpseCount; x++) CorpseList[x] = CorpseList[x+1];
		    continue;
		} // endif

		if (debugLoot)
		    Log.OutD("DoState_Loot: Checking index [" + CurCorpse + "] Name [" +
			CorpseList[CurCorpse].Aco.szName + "] Oid [" + Util.Hex(CorpseList[CurCorpse].Oid) + "]");

		// See if corpse is in range.  For now we only loot corpses close to our current home location
		CorpseDist = Util.Distance(HomeLoc, CorpseList[CurCorpse].Aco.maploc);
		if (CorpseDist > lootRadius) {
		    if (debugLoot) Log.OutD("DoState_Loot: Corpse too far away, skipping for now...");
	       	    CurCorpse++;
	       	    continue;
		} // endif

		// See if assess event has fired for this corpse, if not we can't determine if it is ours
		if (CorpseList[CurCorpse].AssessReq && !CorpseList[CurCorpse].AssessDone) {
		    if (debugLoot) Log.OutD("DoState_Loot: Corpse not assessed yet");
		    // If assess timed out, then reissue it
		    if ((new Date()-CorpseList[CurCorpse].AssessTime) > assessTimeout) {
			CorpseList[CurCorpse].AssessCount++;
			skapi.AssessAco(CorpseList[CurCorpse].Aco);
		    } // endif
		    CurCorpse++;
		    continue;
		} // endif

		// If corpse is not ours then...
		if (!CorpseList[CurCorpse].Ours) {
		    // Discard it if we don't want to loot corpses we don't own
		    if (!flagLootAll) {
			if (debugLoot) Log.OutD("DoState_Loot: Corpse not ours, discarding...");
			CorpseCount--;
			for (var x=CurCorpse; x<CorpseCount; x++) CorpseList[x] = CorpseList[x+1];
			continue;
		    } // endif

		    // If corpse is less than 'corpseLockout' ms old, then don't try to loot it
		    // now. At the moment you can not loot someone elses corpse for 90 seconds.
		    // Don't do this check if flagLootWait is set to false, which you would want
		    // in a fellow sharing situation
		    if (flagLootWait && ((new Date()-CorpseList[CurCorpse].CreTime) < corpseLockout)) {
			if (debugLoot) Log.OutD("DoState_Loot: Corpse not ours and timer not expired, skipping...");
			CurCorpse++;
			continue;
		    } // endif
		} // endif

		// If we make it here than we found a candidate.  Ensure peace mode first
		if (debugLoot) Log.OutD("DoState_Loot(FindCorpse): Ensuring peace mode...");
		Player.EnsureCombatMode(false);

		// Before trying to open the corpse, see if we are too far away from it and if so
		// then move towards it.
		if (CorpseDist > corpseMoveThresh) {
		    if (debugLoot)
			Log.OutD("DoState_Loot(FindCorpse): Corpse current distance ["+CorpseDist+"]. Moving closer...");
		    MoveDone = false;
		    MoveStart = new Date();
 		    skapi.GoToMaploc(CorpseList[CurCorpse].Aco.maploc, corpseMovePrec);
		    while (!MoveDone && ((new Date()-MoveStart)<corpseMoveTimeout)) skapi.WaitEvent(20);
		    if (debugLoot) Log.OutD("DoState_Loot(FindCorpse): Move done.");
		} // endif

		// Now go try to open it.
		OpenAttempts = 0;
		SubState = stateLootOpenCorpse;
		return;
	    } // end while

	    // If we make it here than a candidate corpse was not found, so go to next state
	    if (debugLoot) Log.OutD("DoState_Loot(FindCorpse): No candidate corpse found, setting substate to Done");
	    SubState = stateLootDone;
	    break;

	case stateLootOpenCorpse:
	    // If open attempt exceeds maximum, then abort
	    OpenAttempts++;
	    if (OpenAttempts > openFailMax) {
	     	if (logLoot) Log.OutD("DoState_Loot(OpenCorpse): Max attempts exceeded, checking next corpse");
	     	CorpseCount--;
		for (var x=CurCorpse; x<CorpseCount; x++) CorpseList[x] = CorpseList[x+1];
		SubState = stateLootFindCorpse;
		break;
	    } else {
	    	if (debugLoot) Log.OutD("DoState_Loot(OpenCorpse): Open attempt [" + OpenAttempts + "]");
	    } // endif

	    if (debugLoot) Log.OutD("DoState_Loot(OpenCorpse): Ensuring peace mode...");
	    Player.EnsureCombatMode(false);

	    // Corpse is targeted so open it (by using it).
	    ContainerOpen = false;
	    PrevLoc = skapi.maplocCur;
	    LastLocCheck = new Date();
	    NotMoveCnt = 0;
	    LastMoveTime = new Date();
	    CorpseList[CurCorpse].Aco.Use();
	    OpenAttemptStart = new Date();
	    SubState = stateLootWaitOpen;
	    break;

	case stateLootWaitOpen:
	    if (!ContainerOpen) {
	    	if (debugLoot) Log.OutD("DoState_Loot(WaitOpen): Corpse not open yet");
	    	// If time for movement check...
	    	if ((new Date()-LastLocCheck)>openMoveInt) {
		    LastLocCheck = new Date();
	    	    // see if we are moving
	    	    if (Util.Distance(skapi.maplocCur,PrevLoc) > openMoveUnits) {
	    	        if (debugLoot) Log.OutD("DoState_Loot(WaitOpen): We are moving");
	    	        NotMoveCnt = 0;
	    	        LastMoveTime = new Date();
	    	    } else {
	    	        if (debugLoot) Log.OutD("DoState_Loot(WaitOpen): We are not moving");
	    	        NotMoveCnt++;
	    	    } // endif
	    	    PrevLoc = skapi.maplocCur;
	    	} // endif

	    	// If the holddown period has expired, then check to see if attempt has timed out
	    	if ((new Date()-OpenAttemptStart)>openMoveHoldDown) {
	    	    if (debugLoot) Log.OutD("DoState_Loot(WaitOpen): Checking for timeout...");
		    // If we have not moved recently then use shorter timeout value
		    if ((NotMoveCnt > 1) && ((new Date()-LastMoveTime) > openFailThresh)) {
			// Corpse not open and we haven't moved recently, retry open attempt..
			if (logLoot) Log.OutD("DoState_Loot(WaitOpen): Not moving and corpse open timer expired");
			SubState = stateLootOpenCorpse;
		    } else {
		        // Use longer movement timeout
		        if ((new Date()-OpenAttemptStart)>openMoveThresh) {
		            // Long open timer expired
		            if (logLoot) Log.OutD("DoState_Loot(WaitOpen): Moving and corpse open timer expired");
			    SubState = stateLootOpenCorpse;
		        } // endif
		    } // endif
	    	} // endif
	    	break;
	    } // endif

	    // If we make it here, then the corpse was opened (ie OpenContained event receive).
	    // Mark the corpse as being opened at least once and increment the corpse checked counter
	    if (!CorpseList[CurCorpse].Opened) {
	    	if (debugLoot) Log.OutD("DoState_Loot(WaitOpen): First time this corpse opened");
	    	CorpseList[CurCorpse].Opened = true;
	    	CorpseCheckCnt++;
		AttLoc[AttLocCur].TotalLooted++;
	    } // endif

	    if (LootCount == 0) {
	    	if (debugLoot) Log.OutD("DoState_Loot(WaitOpen): Container open but empty, aborting...");
	    	CorpseCount--;
		for (var x=CurCorpse; x<CorpseCount; x++) CorpseList[x] = CorpseList[x+1];
		SubState = stateLootFindCorpse;
		return;
	    } // endif

	    if (debugLoot) Log.OutD("DoState_Loot(WaitOpen): Container open, setting substate to LootPickup");
	    CorpseLootStart = new Date();
	    SubState = stateLootPickup;
	    AssessActive = false;	// No assess active yet
	    AssessCur = 0;		// Current item we are assessing
	    AssessesDone = false;	// Set to true when all assesses done
	    skapi.WaitEvent(pickupDelay, wemFullTimeout);  // Wait briefly to give corpse chance to fully open
	    break;

	case stateLootPickup:
	    if (debugLoot)
		Log.OutD("DoState_Loot(Pickup): AssessesDone [" + AssessesDone + "] AssessActive [" +
			 AssessActive + "] AssessCur [" + AssessCur + "]");

	    // See if time to pickup items from corpse has expired.  If so then we are done with this corpse.
	    if ((new Date()-CorpseLootStart)>corpseTimeout) {
		if (debugLoot) {
		    Log.OutD("DoState_Loot(Pickup): Pickup time expired.  Start [" + new Date(CorpseLootStart) + "]");
		    Log.OutD("DoState_Loot(Pickup): CorpseTimeout [" + corpseTimeout + "]  Processing next corpse...");
		} // endif
		CorpseCount--;
		for (var x=CurCorpse; x<CorpseCount; x++) CorpseList[x] = CorpseList[x+1];
		SubState = stateLootFindCorpse;
		break;
	    } else {
	    	if (debugLoot)
		    Log.OutD("DoState_Loot(Pickup): Pickup Start [" + new Date(CorpseLootStart) +
		    	     "] corpseTimeout [" + corpseTimeout + "]");
	    } // endif

	    // If there is an outstanding assess, then see if it is timed out or done
	    if (AssessActive) {
		if (debugLoot) Log.OutD("DoState_Loot(Pickup): Assess Active...");
		// See if assess completed successfully
		if (LootList[AssessCur].AssessDone) {
		    AssessActive = false;
		    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Assess completed.");
		    AssessCur++;
		    if (AssessCur==LootCount) AssessCur = 0;    // If at end of list, reset to beginning
		} else {
		    // See if assess timed out
		    if ((new Date()-LootList[AssessCur].AssessTime) > assessTimeout) {
			// Assess timed out.  If we have reached the assess limit then mark the
			// item as checked so we don't process this item (since an assess is required
			// to check this item)
			if (LootList[AssessCur].AssessCount >= assessLimit) {
			    LootList[AssessCur].Checked = true;
			    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Assess failed, item skipped.");
			} else {
			    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Assess failed.");
			} // endif
			AssessActive = false;
			AssessCur++;
			if (AssessCur==LootCount) AssessCur = 0;    // If at end of list, reset to beginning
		    } else {
			if (debugLoot) Log.OutD("DoState_Loot(Pickup): Waiting for assess.");
		    } // endif
		} // endif
	    } // endif

	    // If not all items assessed successfully, and no active assess, then...
	    if (!AssessesDone && !AssessActive) {
		// Find next item that needs assessing
		var Start = AssessCur;

		// Assume all items are assessed until will find one not assessed
		AssessesDone = true;

		// Find next item that needs accessing
		Done = false;
		while (!Done) {
		    // See if object create event has arrived.  If not then...
		    if (!LootList[AssessCur].ObjCreate) {
			// see if timer expired.  If not then continue with next item, ie we are still
			// waiting for the object create event to arrive
			if ((new Date()-LootList[AssessCur].CreTime) < objcreTimeout) {
			    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Waiting for object create...");
			    AssessesDone = false;
			    AssessCur++;
			    if (AssessCur == LootCount) AssessCur = 0;	// Wrap item pointer if at end of list
			    if (AssessCur == Start) Done = true;	// If we are back where we started
			    continue;
			} // endif
			// Timer expired.  Force an assess by marking object as created
			LootList[AssessCur].ObjCreate = true;
			if (debugLoot) Log.OutD("DoState_Loot(Pickup): Object create wait expired, assessing...");
		    } // endif

		    // If item not already checked or assessed, and an assess is needed then...
		    if ((!LootList[AssessCur].Checked) &&
		        (!LootList[AssessCur].AssessDone) &&
		        (LootList[AssessCur].AssessReq)) {
			// Assess item
			if (debugLoot)
			    Log.OutD("DoState_Loot(Pickup): Assessing item index [" + AssessCur + "] Item [" +
				     LootList[AssessCur].Aco.szName + "]");
			LootList[AssessCur].AssessCount++;
			LootList[AssessCur].AssessTime = new Date();
			skapi.AssessAco(LootList[AssessCur].Aco);
			AssessActive = true;
			AssessesDone = false;
			Done = true;
			continue;
		    } // endif

		    // Assess not required or finished, check next item
		    AssessCur++;
		    if (AssessCur == LootCount) AssessCur = 0;	// Wrap item pointer if at end of list
		    if (AssessCur == Start) Done = true;	// If we are back where we started, then we are done
		} // end while
	    } // end if

	    // Go through each item in corpse and see if any should be picked up.  We will potentially
	    // execute this section many times until all object create events arrive and all item
	    // assesses are done.
	    var Done = true;
	    for (CurItem=0; CurItem<LootCount; CurItem++) {
	    	if (debugLoot)
		    Log.OutD("DoState_Loot(Pickup): Item [" + LootList[CurItem].Aco.szName +
		    	     "] Checked ["   + LootList[CurItem].Checked +
		    	     "] ObjCreate [" + LootList[CurItem].ObjCreate +
		    	     "] AssReq ["    + LootList[CurItem].AssessReq +
		    	     "] AssDone ["   + LootList[CurItem].AssessDone + "]");

	    	// If item already checked then continue with next item
	    	if (LootList[CurItem].Checked) {
		    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Item already checked.");
		    continue;
		} // endif

	    	// If item requires an assess and assess not done then continue
	    	if ((LootList[CurItem].AssessReq) && (!LootList[CurItem].AssessDone)) {
		    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Item needs assess.");
		    Done = false;
		    continue;
		} // endif

		// If we reach here, then item is ready to be checked
		Done = false;
		break;
	    } // endfor

	    if (Done) {
	    	// All items in this corpse have been processed so go check next corpse.
	    	// First record this corpse's Oid so we don't re-process it again.
	    	PrevCorpseList[PrevCorpseNext] = CorpseList[CurCorpse].Oid;
	    	PrevCorpseNext++;
	    	if (PrevCorpseNext == lootPrevCorpseMax) PrevCorpseNext = 0;
		CorpseCount--;
		for (var x=CurCorpse; x<CorpseCount; x++) CorpseList[x] = CorpseList[x+1];
		SubState = stateLootFindCorpse;
		return;
	    } // endif

	    // See if any item ready to be checked
	    if (CurItem==LootCount) {
		if (debugLoot) Log.OutD("DoState_Loot(Pickup): No items ready to be checked, assesses outstanding.");
		return;
	    } // endif

	    // Ok, we have a item ready to go, mark it as checked then check it
	    LootList[CurItem].Checked = true;

	    // Build description of item
	    var ItemDesc = Assess.BuildDesc(LootList[CurItem].Aco);
	    LootList[CurItem].Desc = ItemDesc;
	    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Checking item [" + LootList[CurItem].Aco.szName + "/" +
				    Util.Hex(LootList[CurItem].Aco.oid) + "] Desc [" + ItemDesc + "]" );

	    // Now make sure this item is not inscribed, been tinked, or retained.  How is this possible
	    // you may ask?  Well warbot considers a corpse a corpse no matter if it's your character or
	    // a creature so we need these checks to make sure you don't salvage your own death items!
	    // We will mark these items as DeathItems and pick them up
	    var LootType = lootTypeJunk;
	    if ((LootList[CurItem].Aco.oai != undefined) && (LootList[CurItem].Aco.oai != null) &&
	        (LootList[CurItem].Aco.oai.ibi != undefined) && (LootList[CurItem].Aco.oai.ibi != null)) {
	        if (debugLoot) Log.OutD("DoState_Loot(Pickup): Checking for inscription/retained/imbued item...");
		if (((LootList[CurItem].Aco.oai.ibi.szInscription != null) &&
		     (LootList[CurItem].Aco.oai.ibi.szInscription != "")) ||
		    ((LootList[CurItem].Aco.oai.ibi.ctink != null) && (LootList[CurItem].Aco.oai.ibi.ctink > 0)) ||
		    (LootList[CurItem].Aco.oai.ibi.fRetained)) {
		    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Good item found, loot type set to DeathItem");
		    LootType = lootTypeDeathItem;
		    LootList[CurItem].LootType = LootType;
		} // endif
	    } // endif

	    // If above check didn't find an inscribed/retaned/imbued item, then determine it's loot type
	    if (LootType != lootTypeDeathItem) {
	    	// Figure out if this is interesting loot
	        if (debugLoot) Log.OutD("DoState_Loot(Pickup): Determing loot type...");
	    	LootType = Loot.WhatType(LootList[CurItem].Aco);
		LootList[CurItem].LootType = LootType;
		if (debugLoot)
		    Log.OutD("DoState_Loot(Pickup): LootType [" + LootType + "/" + Loot.TypeInfo[LootType].Name + "]");

		// If the Loot.WhatType method detected an alertable item, then notify appropiate parties
		if (Loot.AlertFlag) {
		    if (flagItemSpam) NotifySupport("WB: Item '" + LootList[CurItem].Aco.szName + "' found on " +
						    CorpseList[CurCorpse].Aco.szName + " @ " +
						    Util.FormatCoords(CorpseList[CurCorpse].Aco.maploc));
		    if (logItems)
			Log.OutD("Item: " + LootList[CurItem].Aco.szName + " found on " +
				 CorpseList[CurCorpse].Aco.szName + " @ " +
				 Util.FormatCoords(CorpseList[CurCorpse].Aco.maploc));
		    if (alertItem) {
			if (LootType == lootType) {
			    alertRareWav.Play();
			} else {
			    alertItemWav.Play();
			} // endif
		    } // endif
		} // endif
	    } // endif

	    if (LootType == lootTypeJunk) {
		if (debugLoot) Log.OutD("DoState_Loot(Pickup): Item not interesting");
		return;
	    } // endif

	    if ((LootType == lootTypeValuable) && (Loot.TypeInfo[LootType].TotalValue > Loot.ValuableFilter.MaxTotal)) {
		if (debugLoot) Log.OutD("DoState_Loot(Pickup): Valuable item pyreal total exceeded");
		return;
	    } // endif

	    // Item is good, pick it up if there is room in pack and we have not picked up too many of
	    // this type of item.  If item is stackable then pick up if at least one of this type is
	    // already in inventory (ie wont increase pack space)
	    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Interesting " + Loot.TypeInfo[LootType].Name + " found");

	    var FreeSlots = skapi.acoChar.citemMax - skapi.acoChar.citemContents;
	    if (debugLoot)
		    Log.OutD("DoState_Loot(Pickup): Max Slots [" + skapi.acoChar.citemMax + "] Used Slots [" +
			     skapi.acoChar.citemContents + "] Free Slots [" + FreeSlots + "]");

	    if (FreeSlots < Loot.TypeInfo[LootType].MinSlots) {
		// If item is stackable, see if item of same type already in our inventory.  If so then pick it up
		if (LootList[CurItem].Aco.citemMaxStack > 1) {
		    // Item is stackable
		    if (!FoundInMainPack(LootList[CurItem].Aco.szName)) {
		        if (debugLoot) Log.OutD("DoState_Loot(Pickup): Freeslots < MinSlots and item is stackable " +
		    				"but does exists in main pack");
		    	return;
		    } else {
		    	if (debugLoot) Log.OutD("DoState_Loot(Pickup): Freeslots < MinSlots but item is stackable " +
		    				"and exists in main pack");
		    } // endif
		} else {
		    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Not enough pack space left for this item type");
		    return;
		} // endif
	    } // endif

	    if (debugLoot)
		Log.OutD("DoState_Loot(Pickup): Picked up so far [" + Loot.TypeInfo[LootType].ItemCount +
			 "]. Maximum allowed [" + Loot.TypeInfo[LootType].MaxCount + "]");

	    // See if we are at the maximum of this item type to pick up.  DeathItem counts are checked elsewhere
	    // so don't do it again here.
	    if ((LootType != lootTypeDeathItem) &&
		(Loot.TypeInfo[LootType].ItemCount >= Loot.TypeInfo[LootType].MaxCount)) {
		// If item is stackable, see if item of same type already in our inventory.  If so then pick it up
		if (LootList[CurItem].Aco.citemMaxStack > 1) {
		    // Item is stackable
		    if (!FoundInMainPack(LootList[CurItem].Aco.szName)) {
		        if (debugLoot) Log.OutD("DoState_Loot(Pickup): ItemCount >= MaxCount and item is stackable " +
		    				"but does not exist in main pack");
		    	return;
		    } else {
		    	if (debugLoot) Log.OutD("DoState_Loot(Pickup): ItemCount >= MaxCount but item is stackable " +
		    				"and exists in main pack");
		    } // endif
		} else {
		    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Pickup limit reached for this item type");
		    return;
		} // endif
	    } // endif

	    // Ok seems we have a winner, setup the pickup attempt
	    if (logLoot||debugLoot) {
		Log.OutD("DoStateLoot(Pickup): Picking up [" + LootList[CurItem].Aco.szName + "/" +
			 Util.Hex(LootList[CurItem].Aco.oid) + "] Src [" + CorpseList[CurCorpse].Aco.szName + "]");
		Log.OutD("DoStateLoot(Pickup): Type [" + Loot.TypeInfo[LootType].Name + "] Desc [" +
			 LootList[CurItem].Desc + "]");
	    } // endif
	    PickupActive = true;
	    InventoryAdd = false;
	    InventoryMerged = false;
	    var PickupStart = new Date();

	    // Attempt the actual pickup
	    LootList[CurItem].Aco.MoveToPack(0,0);

	    // Wait until we receive an AddToInventory or AdjustStack event, or the timeout interval has been reached
	    while (!InventoryAdd && ((new Date()-PickupStart)<pickupFailInt)) skapi.WaitEvent(25);
	    PickupActive = false;

	    // Ok we either picked item up or we reached the timeout.  If item was picked up then...
	    if (InventoryAdd) {
		if (logLoot||debugLoot) Log.OutD("DoStateLoot(Pickup): Pickup successful.");
		if (flagReportLoot)
		    Chat.Screen("Looted (" + Loot.TypeInfo[LootType].Name + ") " +
		    		 LootList[CurItem].Aco.szName + " " + ItemDesc);

		// If this is a scroll and we have the learn unknown scrolls option checked,
		// then set flag to check scroll list next idle period for unknown spells.
		if ((LootType == lootTypeScroll) && (Loot.ScrollFilter.LearnUnknown) && !HaveUnkScrolls) {
		    HaveUnkScrolls = true;
		} // endif

		// Now add this aco to the loot list for this loot type.  These lists used to sort items in
		// pack and to sell them.  We don't do this if the item was merged into an existing stack.
		if (InventoryMerged) {
		    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Item merged with existing item in inventory.");
		} else {
		    // Item added to inventory.  Note the AddToInventory event adds the item to the loot list.
		    if (debugLoot) Log.OutD("DoState_Loot(Pickup): Item added to inventory.");
		} // endif
		skapi.WaitEvent(addInventDelay, wemFullTimeout);	// Give char time to put item in pack
	    } else {
		// Couldn't add item to inventory
		if (logLoot||debugLoot) Log.OutD("DoState_Loot(Pickup): Unable to pickup item.");
	    } // endif
	    break;

	case stateLootDone:
	    break;	// Process this state below
    } // end switch

    if (SubState == stateLootDone) {
    	CorpseCheckDone	= true;
	if (debug||debugLoot) Log.OutD("DoState_Loot: Switching state to IDLE...");
	StateChange(stateIdle);
    } // endif

    if (debug||debugLoot)
	Log.OutD("DoState_Loot: End, SubState [" + textSubLoot[SubState] + "] CorpseCount [" + CorpseCount + "]");
    return;
} // end function


function DoState_Scroll() {
    if (debug||debugScroll) Log.OutD("DoState_Scroll: Begin [" + ScrollIndex + "]");

    // If creature close to us, then cancel scroll processing and deal with the danger
    if (Danger) {
    	if (debug||debugScroll) Log.OutD("DoState_Scroll: Danger detected, aborting salvaging...");
    	StateChange(stateSelect);
    	Player.EnsureCombatMode(true);
    	return;
    } // endif

    // We start the scroll search from the beginning each time as the act of reading the scroll
    // destroys it which causes an adjustment to the loot list.
    var ScrollIndex = 0;
    while (ScrollIndex < Loot.TypeInfo[lootTypeScroll].ItemCount) {
	var ScrollAco = Loot.TypeInfo[lootTypeScroll].ItemList[ScrollIndex].Aco;
	if ((ScrollAco == undefined) || (ScrollAco == null) || !ScrollAco.fExists || !ScrollAco.fInInventory) continue;
	var InSpellBook = skapi.FSpellidInSpellbook(ScrollAco.spellid);
	if (!InSpellBook) break;
	ScrollIndex++;
    } // end while

    // If the scroll index is less then the scroll count, then we found an unlearned spell so go
    // learn it.  Otherwise we reached the end of the list without finding a spell that we do not
    // know so our state (we are done)
    if (ScrollIndex < Loot.TypeInfo[lootTypeScroll].ItemCount) {
	// Use the scroll (which learns it).  First reset flags that are set in events when scroll is destroyed
	ActionComplete = false;
	InventoryRemove	= false;
	var ReadStartTime = new Date();
	ScrollAco.Use();
	while (!ActionComplete && !InventoryRemove && ((new Date()-ReadStartTime) < scrollTimeout))
	    skapi.WaitEvent(25);

	if (ActionComplete && InventoryRemove) {
	    // Scroll learned successfully
	    if (logScroll || debugScroll)
		Log.OutD("DoState_Scroll: Successfully learned scroll [" + ScrollAco.szName + "]");
	} else {
	    // Action timed out, log event
	    if (logScroll || debugScroll)
		Log.OutD("DoState_Scroll: Timeout learning scroll [" + ScrollAco.szName + "]");
	} // endif
    } else {
    	// Gone through the scroll list, exit this state
   	HaveUnkScrolls = false;
    	StateChange(stateIdle);
    	if (debug||debugScroll) Log.OutD("DoState_Scroll: Switching state to IDLE...");
    } // endif

    if (debug||debugScroll) Log.OutD("DoState_Scroll: End [" + ScrollIndex + "]");
} // end func

// Salvage items.  The program defines two types of salvage which have different collection/combine rules:
// 1) Tink Salvage.  This is salvage destined to be applied to an item.
// 2) Valuable Salvage.  This is salvage that will be sold to a vendor for cash.
function DoState_Salvage() {
    if (debug||debugSalvage) Log.OutD("DoState_Salvage: Begin, Ust/Val Salvage count [" +
    		Loot.TypeInfo[lootTypeUstSalv].Count + "/" + Loot.TypeInfo[lootTypeValSalv].Count + "]");

    // If creature close to us, then cancel loot processing and deal with the danger
    if (Danger) {
    	if (debug||debugSalvage) Log.OutD("DoState_Salvage: Danger detected, aborting salvaging...");
    	StateChange(stateSelect);
    	Player.EnsureCombatMode(true);
    	return;
    } // endif

    var FreePackSlots = skapi.acoChar.citemMax - skapi.acoChar.citemContents;
    if (FreePackSlots == 0) {
        Log.OutDM("DoState_Salvage", "Pack is full, aborting.");
	UstCheckDone = true;
	StateChange(stateIdle);
	return;
    } // endif

    // If we have spent too much time in salvage state, then abort
    if ((new Date()-SalvageStart) > salvageMaxTime) {
    	Log.OutDM("DoState_Salvage", "Too long spent in salvage state, aborting.");
    	StateChange(stateIdle);
    	return;
    } // endif

    // Make sure UST tool in inventory
    UstAco = skapi.AcoFindInInv("Ust");
    if (!UstAco) {
	flagAutoUst = false;
	Log.OutDM("DoState_Salvage", "No UST tool found, disabling auto UST mode.", cmcYellow);
	skapi.SetControlProperty(szPanelConfig, "chkAutoUst", "checked", flagAutoUst );
	StateChange(stateIdle);
	return;
    } // endif

    // If no more salvage to UST, then move any full bags to side packs (if enabled) and then
    // set state to Idle and return
    if ((Loot.TypeInfo[lootTypeUstSalv].ItemCount == 0) && (Loot.TypeInfo[lootTypeValSalv].ItemCount == 0)) {
    	if (flagMoveFull&&BagToMove) {
	    MoveBags();
	    BagToMove = false;
	} // endif
	if (debug||debugSalvage) Log.OutD("DoState_Salvage: No more salvage, switching state to IDLE...");
	StateChange(stateIdle);
	return;
    } // endif

    // Process top item on one of the salvage loot lists (ust or val).  Only do one at a time to make
    // sure FSM executes timely so it can check for danger conditions.
    if (Loot.TypeInfo[lootTypeUstSalv].ItemCount > 0) {
    	if (debugSalvage) Log.OutD("DoState_Salvage: Usting top item on ustable tink salvage list...");
	var Success = Salvage_TinkItems();
    } else {
    	if (debugSalvage) Log.OutD("DoState_Salvage: Usting top item on ustable valuable salvage list...");
    	var Success = Salvage_ValItems();
    } //endif

    if (Success) {
    	// UST operation was successful
	if (debugSalvage) Log.OutD("DoState_Salvage: Ust operation succeeded");
    	UstFailCount = 0;
    } else {
    	// Increment Ust fail count and disabling USTing if we exceed the fail limit.
    	if (debugSalvage) Log.OutD("DoState_Salvage: Ust failed");
    	UstFailCount++;
    	if (UstFailCount>salvageMaxFails) {
	    // Exceeded consecutive fail limit, disable USTing
	    Chat.Screen("Exceeded Ust fail limit, disabling auto-Usting...");
	    Log.OutD("DoState_Salvage: Exceeded Ust fail limit, disabling auto-Usting...");
	    flagAutoUst = false;
	    skapi.SetControlProperty(szPanelConfig, "chkAutoUst", "checked", flagAutoUst);
	} // endif
    } // endif

    if (debug||debugSalvage) Log.OutD("DoState_Salvage: End, Ust/Val Salvage count [" +
    		Loot.TypeInfo[lootTypeUstSalv].Count + "/" + Loot.TypeInfo[lootTypeValSalv].Count + "]");
    return;
} // end function


function DoState_Paused() {
    Log.OutD("DoState_Paused: Begin");

    // Record logout time
    PauseStart = new Date();

    // Log out toon.  Don't play sound
    WB_PlayerLogOff(false);
    CorpseCount = 0;

    // Now pause until we detect that we are completely logged in
    while (skapi.plig != pligInWorld) {
    	if (debugLog) Log.OutD("DoState_Paused: plig = " + skapi.plig, false);
    	skapi.WaitEvent(100);
    } // endif

    Log.OutD("DoState_Paused: InWorld wait done (PLIG = " + skapi.plig + "), toon logged back in");

    // Update expiration times for buffs since they are absolute times but game uses relative times (ie
    // spells don't expire when character is logged)
    var ActualHideTime = new Date() - PauseStart;
    Buff.Extend(ActualHideTime);

    // Set state to INIT and continue
    StateChange(stateInit,NoCheck);

    Log.OutD("DoState_Paused: End");
} // end function


// Person detected in vicinity, log for a while
function DoState_Hide() {
    Log.OutD("DoState_Hide: Begin");
    // Don't use MD_PLAYEROBJ logoff function since we may need to integrate sound alert with logout action

    // If this is a landblock reset request by macroer, then wait a shorter interval to log back in
    if (LandReset) {
	Log.OutDM("DoState_Hide", "Landblock reset, waiting " + Util.Round(landblockInt/(1000*60),1) +
		  "minutes...", ToChat);
	HideTime = landblockInt;
	WB_PlayerLogOff(false);	// Log out toon.  Don't play sound
    } else {
	Log.OutDM("DoState_Hide", "Player [" + DetectedPlayer + "] detected.  " +
		 "Hiding for [" + Util.Round(hideInt/(1000*60),0) + "] minutes.", ToChat );
	HideTime = hideInt;
	WB_PlayerLogOff(true);	// Log out toon.  Play sound if enabled
    } // endif

    // Record logout time
    HideStart = new Date();
    CorpseCount = 0;

    // Just process events until hide time expires
    while ((new Date() - HideStart) < HideTime) skapi.WaitEvent(250);

    Log.OutD("State_Hide: Evasion Period Expired.  Logging back on and setting state to INIT...");
    StateChange(stateInit,NoCheck);

    Player.LogOn(CharSlot);	// Log back on

    // Update expiration times for buffs since they are absolute times but game uses relative times (ie
    // spells don't expire when character is logged)
    var ActualHideTime = new Date() - HideStart;
    Buff.Extend(ActualHideTime);
    LandReset = false;
    return;
} // end function DoState_Hide

function DoState_PortalStormed() {
    Log.OutD("DoState_PortalStormed: Begin");
    // For now just log off.  Eventually might try to get bot back to starting point by navigation or recall spells.
    Player.LogOff();
    RunFlag = false;	// Stop macro
    Log.OutD("DoState_PortalStormed: End");
} // end func


// Log off AC and stop macro
function DoState_Log() {
    Log.OutD("State_Log:  Logging character..." );
    Player.LogOff();
    RunFlag = false;	// Stop macro
} // end func


function DoState_GotoCurrent() {
    // Clear retreat active flag, otherwise if we were at the retreat location and the GotoCurrent button
    // was pressed, we would never retreat
    RetreatActive = false;

    // Go to current kill location
    MoveDone = false;
    MoveStart = new Date();
    skapi.GoToMaploc( AttLoc[AttLocCur].KillLoc, navPrecision );

    while ((State == stateGotoCurrent) && !MoveDone && ((new Date()-MoveStart)<navTimeOut)) {
	skapi.WaitEvent(20);
	if (flagRecordLoc) RecordLoc();
    } // endif

    if (State != stateGotoCurrent) {
	Log.OutDM("DoState_GotoCurrent", "State change detected exiting to FSM main loop...", ToChat);
	return;
    } // endif

    if (MoveDone) {
        if (debug||debugMove)
	    Log.OutDM("DoState_GotoCurrent", "Navigation done, status code [" + MoveCode + "/" +
		      textNscCode[MoveCode] + "]", ToChat);
	if (MoveCode == nscCanceled) Chat.Screen("Navigation aborted.");
	if (debug||debugMove) Log.OutD("DoState_GotoCurrent: Navigation aborted.");
    } else {
	Log.OutDM("DoState_GotoCurrent", "Move to current attack location timed out!", ToChat);
    } // endif

    // Set state to idle and resume FSM
    if (debug||debugMove) Log.OutD("Switching to idle state...");
    if (State == stateGotoCurrent) StateChange(stateIdleInit,NoCheck);

    // Now if enabled, wait a little bit to prevent spell cast failure
    if (flagMoveDelay) {
	Chat.Screen("Arrived at current attack location. Pausing for " + navWait + " ms...");
	skapi.WaitEvent(navWait, wemFullTimeout);
    } else {
	Chat.Screen("Arrived at current attack location.");
    } // endif
} // end function


function DoState_GotoNext() {
    if (debug||debugMove) Log.OutD("DoState_GotoNext: Begin");

    // Clear retreat active flag, otherwise if we were at the retreat location and the GotoNext button
    // was pressed, we would never retreat at the next attack location
    RetreatActive = false;

    // Now figure out the next location we should go to.  If the variable 'MoveOverRide' is set,
    // then goto the indicated attack location.  If not set, then just goto the next attack location
    // in sequence, wrapping to the beginning of the list when the end is reached
    if (MoveOverRide) {
	if (debugMove) Log.OutD("DoState_GotoNext: MoveOverRide set [" + MoveOverRide + "]", ToChat);
	if (AttLocCur == MoveOverRide) {
	    if (debugMove)
		Log.OutD("DoState_GotoNext: MoveOverRide [" + MoveOverRide + "] same as cur location, aborted.", ToChat);
	    MoveOverRide = 0;
	    StateChange(stateIdleInit,NoCheck);
	    return;
	} // endif
	var AttLocOld = AttLocCur;		// Save current attack location
	AttLocCur = MoveOverRide;
	MoveOverRide = 0;
    } else {
	// Set internal structures to point to next attloc in sequence
	if (debugMove) Log.OutD("DoState_GotoNext: Going to next attloc in sequence", ToChat);
	var AttLocOld = AttLocCur;		// Save current attack location
	AttLocCur++;
	if (AttLocCur > AttLocCnt) AttLocCur = 1;
    } // endif

    // Update some stats first
    var CurTime = new Date();

    AttLoc[AttLocOld].LastArrive = LastArriveTime;
    AttLoc[AttLocOld].LastDepart = CurTime;
    AttLoc[AttLocOld].LastInt    = CurTime.getTime() - LastArriveTime.getTime();
    AttLoc[AttLocOld].TotalInt	+= AttLoc[AttLocOld].LastInt;
    AttLoc[AttLocOld].TotalVisits++;

    if (debugStats) {
    	Log.OutD("DoState_GotoNext: AttLocOld [" + AttLocOld + "] AttLocCur [" + AttLocCur + "]");
    	Log.OutD("DoState_GotoNext: CurTime [" + CurTime + "] LastArriveTime [" + LastArriveTime + "]");
    	Log.OutD("DoState_GotoNext: LastInt [" + AttLoc[AttLocOld].LastInt + "/" +
    			Util.FormatMinSec(AttLoc[AttLocOld].LastInt) + "]");
    	Log.OutD("DoState_GotoNext: TotalInt [" + AttLoc[AttLocOld].TotalInt + "/" +
    			Util.FormatMinSec(AttLoc[AttLocOld].TotalInt) + "]");
    	Log.OutD("DoState_GotoNext: TotalVisits [" + AttLoc[AttLocOld].TotalVisits + "]");
    } // endif

    LastArriveTime = new Date();	// In case navigation aborted, at least have a time close to real start

    // Set decal control panel attack loc on movement panel to match new attack location
    skapi.SetControlProperty(szPanelMovement, "curAttLoc", "selected",  AttLocCur-1);
    skapi.SetControlProperty(szPanelMovement, "text2av", "text", AttLocCur);

    // Now see how far away from the attack location we are. If we are really far away then don't go there
    // since we may have died, may be manually moving character, or may have just loaded navigation info
    // far from final macro spot.
    var DistCheck = Util.Distance(AttLoc[AttLocCur].KillLoc, skapi.maplocCur);
    if (DistCheck > navNextMax) {
   	Log.OutDM("DoState_GotoNext", "Next kill location " + DistCheck +
   		  " map units away! Navigation aborted and automove disabled.", ToChat);
	if (AutoMove) {
	    AutoMove = false;
	    skapi.SetControlProperty(szPanelControl, "btnAutoMove",  "Text", "Enable AutoMove");
        } // endif

        // Now set state to idle and let FSM execute
    	StateChange(stateIdleInit,NoCheck);
    	if (debug||debugMove) Log.OutD("DoState_GotoNext: End, State set to idle");
   	return;
    } // endif

    // Log some info if enabled
    if (logMove) Log.OutD("DoState_GotoNext: Switching AttLoc, Old [" + AttLocOld + "] New [" + AttLocCur + "]");
    if (debugMove)
	Log.OutD("DoState_GotoNext: Switching AttLocs, Old [" + AttLocOld + "] New [" + AttLocCur + "]", ToChat);

    // Set new home location
    HomeLoc = AttLoc[AttLocCur].KillLoc;

    // If we are part of a move group, then broadcast the move to the group
    if (Player.MemberFellow() && (MoveGroup>0)) {
	if (debugMove) Log.OutD("DoState_GotoNext: Movegroup [" + MoveGroup + "] defined, spamming fellow...", ToChat);
	Chat.Fellow("MOVE " + MoveGroup + " " + AttLocCur);
    } // endif

    Chat.Screen("Moving to AttLoc #" + AttLocCur);
    if (debugMove||logMove) Log.OutD("DoState_GotoNext: Moving to AttLoc #" + AttLocCur);

    // If waypoint set for previous location, then go to it first
    if (AttLoc[AttLocOld].WayLoc != null) {
	if (debugMove)
	    Log.OutD("DoState_GotoNext: Moving to AttLoc " + AttLocOld + " waypoint at " +
	    	      Util.FormatCoords(AttLoc[AttLocOld].WayLoc,2), ToChat);
	MoveDone = false;
	MoveStart = new Date();
	skapi.GoToMaploc( AttLoc[AttLocOld].WayLoc, navPrecision );

	while ((State == stateGotoNext) && !MoveDone && ((new Date()-MoveStart)<navTimeOut)) {
	    skapi.WaitEvent(20);
	    if (flagRecordLoc) RecordLoc();
	} // endwhile

	if (State != stateGotoNext) {
	    Chat.Screen("State change detected, canceling navigation...");
	    if (debugMove||logMove)
		Log.OutD("DoState_GotoNext: State change [" + State + "/" + textState[State] +
			 "] detected exiting to FSM main loop...");

	    skapi.CancelNav();
	    MoveEnd = new Date();
	    return;
	} // endif

	if (MoveDone) {
	    MoveEnd = new Date();
	    if (debugMove)
		Log.OutD("DoState_GotoNext: Waypoint(from) navigation done, status code [" +
			MoveCode + "/" + textNscCode[MoveCode] + "]", ToChat);
	    if (MoveCode == nscCanceled) {
	    	Chat.Screen("Navigation aborted.");
	    	if (debugMove||logMove) Log.OutD("DoState_GotoNext: Navigation aborted.");
	    	// Now only set state to idle if we are still in the navigation state.  An event
	    	// could set the state to something else
    		if (State == stateGotoNext) {
    		    StateChange(stateIdleInit,NoCheck);
    		    if (debug||debugMove) Log.OutD("DoState_GotoNext: State set to idle");
    		} // endif
	    	return;
	    } // endif
	} else {
	    if (!debugMove) {
	    	Chat.Screen("Move to waypoint(from) timed out! Navigation aborted.");
	    	if (logMove) Log.OutD("DoState_GotoNext: Move to waypoint(from) timed out! Navigation aborted.");
	    } else {
		Log.OutD("DoState_GotoNext: Move to waypoint(from) timed out! Navigation aborted.", ToChat);
	    } // endif
	    skapi.CancelNav();
	    MoveEnd = new Date();
	    if (State == stateGotoNext) StateChange(stateIdleInit,NoCheck);
	    return;
	} // endif
    } // endif

    // See if master waypoint defined.  If so then go to it
    if (MasterWayLoc != null) {
	if (debugMove) Log.OutD("DoState_GotoNext: Moving to Mast Waypnt @ " + Util.FormatCoords(MasterWayLoc,2), ToChat);
	MoveDone = false;
	MoveStart = new Date();
	skapi.GoToMaploc( MasterWayLoc, navPrecision );

	while ((State == stateGotoNext) && !MoveDone && ((new Date()-MoveStart)<navTimeOut)) {
	    skapi.WaitEvent(20);
	    if (flagRecordLoc) RecordLoc();
	} // endwhile

	if (State != stateGotoNext) {
	    if (debugMove||logMove)
	    	Log.OutD("DoState_GotoNext: State change [" + State + "/" + textState[State] +
			 "] detected exiting to FSM main loop...");
	    Chat.Screen("State change detected, canceling navigation...");
	    skapi.CancelNav();
	    MoveEnd = new Date();
	    return;
	} // endif

	if (MoveDone) {
	    MoveEnd = new Date();
	    if (debugMove)
		Log.OutD("DoState_GotoNext", "Master waypoint navigation done, status code [" +
			 MoveCode + "/" + textNscCode[MoveCode] + "]", ToChat);
	    if (MoveCode == nscCanceled) {
	    	Chat.Screen("Navigation aborted.");
	    	if (debugMove||logMove) Log.OutD("DoState_GotoNext: Navigation aborted.");
	    	// Now only set state to idle if we are still in the navigation state.  An event
	    	// could set the state to something else
    		if (State == stateGotoNext) {
    		    StateChange(stateIdleInit,NoCheck);
    		    if (debug||debugMove) Log.OutD("DoState_GotoNext: State set to idle");
    		} // endif
	    	return;
	    } // endif
	} else {
	    if (!debugMove) {
	    	Chat.Screen("Move to master waypoint location timed out! Navigation aborted.");
	    	if (logMove) Log.OutD("DoState_GotoNext: Move to master waypoint timed out! Navigation aborted.");
	    } else {
		Log.OutD("DoState_GotoNext: Move to master waypoint location timed out! Navigation aborted.", ToChat);
	    } // endif
	    skapi.CancelNav();
	    MoveEnd = new Date();
	    if (State == stateGotoNext) StateChange(stateIdleInit,NoCheck);
	    return;
	} // endif
    } // endif

    // Now move to the new locations waypoint if defined
    if (AttLoc[AttLocCur].WayLoc != null) {
	if (debugMove)
	    Log.OutD("DoState_GotoNext: Moving to AttLoc " + AttLocCur + " waypoint at " +
	    	      Util.FormatCoords(AttLoc[AttLocCur].WayLoc,2), ToChat);
	MoveDone = false;
	MoveStart = new Date();
	skapi.GoToMaploc( AttLoc[AttLocCur].WayLoc, navPrecision );

	while ((State == stateGotoNext) && !MoveDone && ((new Date()-MoveStart)<navTimeOut)) {
	    skapi.WaitEvent(20);
	    if (flagRecordLoc) RecordLoc();
	} // endwhile

	if (State != stateGotoNext) {
	    if (debugMove||logMove)
		Log.OutD("DoState_GotoNext: State change [" + State + "/" + textState[State] +
			 "] detected exiting to FSM main loop...");
	    Chat.Screen("State change detected, canceling navigation...");
	    skapi.CancelNav();
	    MoveEnd = new Date();
	    return;
	} // endif

	if (MoveDone) {
	    MoveEnd = new Date();
	    if (debug||debugMove)
		Log.OutDM("DoState_GotoNext", "Waypoint(to) navigation done, status code [" +
			  MoveCode + "/" + textNscCode[MoveCode] + "]", ToChat);
	    if (MoveCode == nscCanceled) {
	    	Chat.Screen("Navigation aborted.");
	    	if (debug||debugMove) Log.OutD("DoState_GotoNext: Navigation aborted.");
	    	// Now only set state to idle if we are still in the navigation state.  An event
	    	// could set the state to something else
    		if (State == stateGotoNext) StateChange(stateIdleInit,NoCheck);
	    	return;
	    } // endif
	} else {
	    if (!debugMove) {
	    	Chat.Screen("Move to waypoint(to) timed out! Navigation aborted.");
	    	if (logMove) Log.OutD("DoState_GotoNext: Move to waypoint(to) timed out! Navigation aborted.");
	    } else {
		Log.OutD("DoState_GotoNext: Move to waypoint(to) timed out! Navigation aborted.", ToChat);
	    } // endif
	    skapi.CancelNav();
	    MoveEnd = new Date();
	    if (State == stateGotoNext) StateChange(stateIdleInit,NoCheck);
	    MoveEnd = new Date();
	    return;
	} // endif
    } // endif

    if (debug||debugMove)
	Log.OutDM("DoState_GotoNext", "Moving to # " + AttLocCur + " KillLoc, coords: " +
		  Util.FormatCoords(HomeLoc,3), ToChat);

    // Waypoint processing done, now go to kill location
    MoveDone = false;
    MoveStart = new Date();
    skapi.GoToMaploc(HomeLoc, navPrecision);

    while ((State == stateGotoNext) && !MoveDone && ((new Date()-MoveStart)<navTimeOut)) {
	skapi.WaitEvent(20);
	if (flagRecordLoc) RecordLoc();
    } // endwhile

    if (State != stateGotoNext) {
	if (debugMove||logMove)
	    Log.OutD("DoState_GotoNext: State change [" + State + "/" + textState[State] +
		     "] detected exiting to FSM main loop...");
	Chat.Screen("State change detected, canceling navigation...");
	skapi.CancelNav();
	MoveEnd = new Date();
	return;
    } // endif

    if (MoveDone) {
    	MoveEnd = new Date();
        if (debugMove)
	    Log.OutDM("DoState_GotoNext", "KillLoc navigation done, status code [" + MoveCode + "/" +
		      textNscCode[MoveCode] + "]", ToChat);
	if (MoveCode == nscCanceled) Chat.Screen("Navigation aborted.");
	if (debugMove) Log.OutD("DoState_GotoNext: Navigation aborted.");
    } else {
	if (!debugMove) {
	    Chat.Screen("Move to next attack location timed out! Navigation aborted.");
	    if (logMove) Log.OutD("DoState_GotoNext: Move to next attack location timed out! Navigation aborted.");
	} else {
	    Log.OutD("DoState_GotoNext: Move to to next attack location timed out! Navigation aborted.", ToChat);
	} // endif
	skapi.CancelNav();
	MoveEnd = new Date();
    } // endif

    // Timestamp arrival at new attack location
    LastArriveTime = new Date();

    // Set state to idle and resume FSM
    if (debug||debugMove) Log.OutD("DoState_GotoNext: Switching to IdleInit state...");
    if (State == stateGotoNext) StateChange(stateIdleInit,NoCheck);

    // Now wait a little bit to prevent spell cast failure if option enabled
    if (flagMoveDelay) {
	Chat.Screen("Arrived at next attack location. Pausing for " + navWait + " ms...");
	skapi.WaitEvent(navWait, wemFullTimeout);
    } else {
	Chat.Screen("Arrived at next attack location.");
    } // endif
} // endif


// We died, exit macro if desired, otherwise rebuff and wait for macroer to move us back to kill loc.  Optional
// timer can be set for autologging.
function DoState_Died() {
    Log.OutDM("State_Died", "We just died.", ToChat);

    // Sound audible alarm and spam death to fellow if enabled
    if (alertDeath) alertDeathWav.Play();
    NotifySupport("I just died.");

    // If flag set to log on death, or we have died too many times, then do it
    if (flagDeathLog||(DeathCount>=deathMax)) {
	Log.OutDM("State_Died", "We died, DeathCount [" + DeathCount + "] deathMax [" + deathMax + "] Logging...", ToChat);
	Player.LogOff();
	RunFlag = false;
	return;
    } // endif

    // If we make it here, then player doesn't want to log off immediately after death.  Wait a little
    // bit so that we exit portal space.  Eventually we will check PLIG to see when we materialize in world
    skapi.WaitEvent(deathWait, wemFullTimeout);

    // Reset cast cast count to 0 on all creature and life spells so randomize feature spreads out spells
    Buff.ResetCreLife();
    StrengthExp	= 0;		// Expire strength self spell

    // Clear retreat status, otherwise we may try to run to the current location after death.
    RetreatActive = false;
    AdvanceReq = false;

    // If player doesn't want to rebuff after death and buffing is enabled then disable it
    DeathBuffDisabled = false;
    if (!flagDeathRebuff && BuffMode) {
    	DeathBuffDisabled = true;
    	Log.OutDM("State_Died", "Buffing disabled", ToChat);
    	BuffMode = false;
	skapi.SetControlProperty(szPanelControl, "btnBuffMode", "checked", false);
    } // endif

    // If AutoMove or AutoHome enabled, disabled them so we don't go off running
    DeathAutoMoveDisabled = false;
    if (AutoMove) {
    	DeathAutoMoveDisabled = true;
        Log.OutDM("State_Died", "AutoMove disabled.", ToChat);
	AutoMove = false;
	skapi.SetControlProperty(szPanelControl, "btnAutoMove", "Text", "Enable AutoMove");
    } // endif

    // If AutoMove or AutoHome enabled, disabled them so we don't go off running
    DeathAutoHomeDisabled = false;
    if (flagAutoHome) {
    	DeathAutoHomeDisabled = true;
        Log.OutDM("State_Died", "AutoHome disabled.", ToChat);
	flagAutoHome = false;
	skapi.SetControlProperty(szPanelControl, "btnAutoHome",  "Text", "Enable AutoHome");
    } // endif

    // Enable peace mode (if not already enabled) to disabling attacking of creatures at lifestone.
    // We will still heal ourselves if attacked
    DeathPeaceModeEnabled = false;
    if (!PeaceMode) {
    	DeathPeaceModeEnabled = true;
        Log.OutDM("State_Died", "Peace mode enabled.", ToChat);
    	PeaceMode = true;
    	skapi.SetControlProperty(szPanelControl, "btnPeaceMode",  "Text", "Disable PeaceMode");
    } // endif

    // Record our current location so we know when macroer manually moved us.  When we detect movement
    // we restore any settings we changed (automove, autohome, buffing flag, peacemode etc)
    DeathPostLoc = skapi.maplocCur;

    // Now see if we should log out at a set time in the future.
    if (deathLogWait) {
    	var CurTime = new Date();
	DeathLogTime = CurTime.getTime() + deathLogWait;
	Log.OutDM("State_Died", "Macro will logout at " + new Date(DeathLogTime), ToChat);
    } // endif

    StateChange(stateIdleInit,NoCheck);
    return;
} // end func DoState_Died


// Log off AC and stop macro
function DoState_Admin() {
    Log.OutD("DoState_Admin: Begin");

    // Send @tell response to buy us a few seconds.  Hopefully we hear alert, otherwise we are going to be banned!
    if ((ChatStart > 0) && ((new Date()-ChatStart) > chatRespondInt)) {
    	ChatStart = -1;
	Log.OutD("DoState_Admin: Sending stock reply to @tell, hopefully we hear alert soon.", ToChat );
	Chat.Reply(chatRespondMsg);
	if (enhancedTell) {
	    skapi.WaitEvent(1800, wemFullTimeout);
	    if (AdminName.indexOf("+") >= 0) {
		Log.OutD("DoState_Admin: Sending @tell to [" + AdminName + "] in case reply screwed up.", ToChat );
		Chat.Tell(AdminName, chatRespondMsg);
	    } else {
		var PlusAdmin = "+" + AdminName;
		Log.OutD("DoState_Admin: Sending @tell+ to [" + PlusAdmin + "] in case reply screwed up.", ToChat );
		Chat.Tell(PlusAdmin, chatRespondMsg);
	    } // endif
	} // endif
    } // endif

    if (!alertAdmin || ((new Date()-AdminStart) > alertAdminInt)) {
	Log.OutD("DoState_Admin: Stopping macro...", ToChat );
	RunFlag = false;	// Stop macro
    } else {
	alertAdminWav.Play();
    } // endif

    Log.OutD("DoState_Admin: End" );
} // end func