// WAR_VAR.JS
//
// Author:	Poundboy
// Version:	2.08
//
//
// Modification History:
//
// Who	  Date	    Version	Description
//
// PB	  11/02/02  1.01	Initial version.
//
//
// Description:
//
//	This module contains all the variables and objects for the WarBot macro
//
//
// Macro context variables

State = stateInit;
		// Current state we are in. Possible values:
		// stateInit		Init macro context.  Done after every logon
		// stateIdle		Maintenance routines
		// stateBuff		Check buffs
		// stateSelect		Select a target
		// stateTarget		Target selection
		// stateKill		Try to kill
		// stateCheckCompsInit	Get ready to check comps
		// stateCheckComps	Check components
		// stateGotoCurrent	Goto current attack location
		// stateGotoNext	Goto next attack location
		// stateLootInit	Prepare to loot corpse(s)
		// stateLoot		Loot corpse(s)
		// stateSalvage		Salvage loot (not implemented yet!)
		// stateHide		Person detected in vicinity, log for x minutes
		// statePaused		Macro paused by logging off, will resume when manually logged back on
		// stateReset		Logoff/Logon to avoid portal storm
		// stateStormed		We were portal stormed
		// stateDied		We died, exit game and macro
		// stateLog		Log off AC and stop macro
		// stateExit		Stop Macro
		// stateAdmin		Admin detected, deal with him/her then log

SubState = substateNone;
		// Substate depending on primary state (State)
		//
		// stateKill:
		//
		// stateKillYield	Do the initial yield on the creature
		// stateKillVuln	Vuln creature is we are finishing off with war
 		// stateKillWar		War the creature to death
		// stateKillDone	Creature either died or ran awa
		//
		// stateLoot:
		//
		// stateLootFindCorpse	  Find a corpse to loot
		// stateLootOpenCorpse	  Open corpse
		// stateLootWaitOpen	  Wait for corpse to open
		// stateLootAssessItems	  Assess all items on corpse
		// stateLootPickup	  Pickup any items that match criteria
		// stateLootDone	  All done, go to next state

StartTime	= new Date();
Restricted 	= false;	// Set to true when spells necessary to macro operation are not defined.
				// Only certain looting/salvaging operations permitted.
RunFlag		= true; 	// Flag set to true when normal macro state processing should be done.
				// Set to false when macro should be terminated.


// **************************************************************************************************
//
// General  variables

// Initialize variables that determine when too many creatures are near us and special processing is required.
CriticalAmount	= 0;
UrgentAmount	= 0;

DetectedPlayer	= "";		// Name of any unknown player detected
CharConfigDir	= "";		// Configuration files directory for a specific character name

// **************************************************************************************************
//
// Scroll variables
TradeOpen	= false;	// Set to true when trade window open with another character
ItemCoaco	= skapi.CoacoNew();
ScrollCoaco	= skapi.CoacoNew();


// **************************************************************************************************
//
// Player Context variables

VitalsOK	= true;		// Health, mana, stamina are at acceptable levels
Dead		= false;	// Set to true when we die.  Triggers special death processing
DeathCount	= 0;		// # of times we have died
DeathStart	= 0;		// When we die, time of death recorded here
DeathLogTime	= 0;		// When we die, time to log set here
DeathPostLoc	= 0;		// When we die, record location we end up at (lifestone).  Used to determine
				// when to cancel special death processing.
StrengthExp	= 0;		// Time to cast next strength spell to reduce burden

UnspentExp	= 0;		// Keeps track of last unspent experience value received in
				// last OnStatUnspentExp event.  Used to track exp changes.

SkillExp	= new Array();	// Keeps track of experience changes for individual skills

KillExp		= 0;		// Estimated total exp of creatures we killed
TotalKilled	= 0;		// Total creatures killed

TotalExpStart	= 0;		// Total experience of character at macro start

CompLastCheck	= new Date();	// Time last spell component check was done

CharEndPortal	= false;	// Set when we come out of portal
CharStormWarn	= false;	// Set when portal storm detected
CharOnEnd3D 	= false;	// Set when OnEnd3D event detected
CharSlot	= -1;		// Set at macro startup.  Holds login screen slot # of current character
ForceSlot	= -1;		// Can be set in userconfig.dat to force a slot #.  Needed sometimes because
				// of bug in skunkworks.

CharDisbandFlag	= false;	// Set to true when someone wants us to disband our fellowship
CharLastDisband = 0;		// Last time someone told us to disband a fellow.  Used to determine when to
				// auto-create a fellow

CharLastGem	= 0;		// Last time we used a gem of stillness

CharCurWar	= 0;
CharCurLife	= 0;
CharCurCre	= 0;

LastJoinReq	= 0;		// Time last join movegroup was issued.
JoinTarget	= null;		// Character movegroup join was requested to

PlayerNearCnt	= 0;		// Count of co-macroers near us
PlayerExclCnt   = 0;

PauseStart	= 0;		// Last time macro was paused (via logout)
BuffStateStart  = 0;		// Time last buff run started

FellowFlag 	= false; 	// Set to True when authorized fellowship request detected
FellowChar	= "";		// Player to attempt fellowship with

EvadeWar 	= false;	// Set to true when war spell heading our way and we need to evade it.
PrevEvadeKey	= 0;		// Previous evade direction and time.  Needed so that we don't step
PrevEvadeTime	= 0;		// back into the first war spell when two come close together.
LastSlideTime	= 0;		// Last time we did a slide cast.

DangerPrev	= false;	// Keep track of previous state so we can log danger state changes
Danger		= false;	// Flag set to true when we are in a creature's attack range

CriticalPrev	= false;	// Keep track of previous state so we can log critical state changes
Critical	= false;	// Flag set to true when 2+ creatures within our critical radius

RingOkPrev	= false;	// Keep track of previous state so we can log ring state changes
RingOk		= false;	// Flag set to true when enough creatures close to us

UrgentPrev	= false;	// Keep track of previous state so we can log critical state changes
Urgent		= false;	// Flag set to true when 3+ creatures within our critical radius

AdminStart	= 0;		// Holds time admin first detected
ChatStart	= 0;		// Holds time last admin chat message received
FellowAbort	= false;	// Set to true when an admin or player fellowship warning received.
FellowWarnTime	= 0;		// Holds time of receipt of fellowship warning

LastRecordedLoc	= 0;

ChatLog		= new Array();	// Holds record of chat/tell messages received
ChatCnt		= 0;		// # of messages received so far.

AbortAction	= false;	// Set to true when we want to abort an action

LandReset	= false;	// When set to true, log for a short interval to reset landblock

DeathBuffDisabled	= false;
DeathAutoMoveDisabled	= false;
DeathAutoHomeDisabled	= false;
DeathPeaceModeEnabled   = false;
DeathStealthDisabled	= false;

UseManaCharge		= false;

function HelpObj(Oid) {
    this.Oid        = Oid;
    this.MaxHealth  = 0;
    this.Health     = 0;
    this.MaxStamina = 0;
    this.Stamina    = 0;
    // this.LastHeal = 0;
    // this.LastStam = 0;
} // end func

HelpList	= new Array();
HelpCnt		= 0;

// **************************************************************************************************
//
// Decal panel variables
BuffCrePrev	= 0;
BuffCreIndex	= 0;
BuffCreLast	= -1;

BuffLifePrev	= 0;
BuffLifeIndex	= 0;
BuffLifeLast	= -1;

BuffArmorPrev	= 0;
BuffArmorIndex	= 0;
BuffArmorLast	= -1;

LootTypePrev	= 0;
LootTypeIndex	= 0;
LootTypeLast	= -1;

BuffHermIndex	= 0;
BuffDefIndex	= 0;
BuffSpiritIndex	= 0;

LootMinorPrev	= 0;
LootMinorIndex	= 0;
LootMinorLast	= -1;

LootArmorPrev	= 0;
LootArmorIndex	= 0;
LootArmorLast	= -1;

LootWeaponPrev	= 0;
LootWeaponIndex	= 0;
LootWeaponLast	= -1;

LootRangedPrev	= 0;
LootRangedIndex	= 0;
LootRangedLast	= -1;

LootWandPrev	= 0;
LootWandIndex	= 0;
LootWandLast	= -1;

LootSalvPrev	= 0;
LootSalvIndex	= 0;
LootSalvLast	= -1;

// **************************************************************************************************
//
// Creature Selection Variables

SelFlag		= false;	// Whether or not a creature is currently selected
SelAco		= 0;		// Aco of creature selected
SelId		= 0;		// oid of creature selected
SelIndex	= 0;		// Index into creature database
SelDist		= 0;		// Distance to creature selected
SelPrio		= 0;		// Priority of currently selected creature
SelExp		= 0;		// Experience of currently selected creature
SelLocked	= false;	// Selected creature already locked
SelDangerFlag	= false;	// Danger flag of currently selected creature
SelRange	= 0;		// Creature selection range.
SelUseAttackAco	= false;	// If set, then use aco of last creature that attacked us when selecting
				// next creature to kill

AttemptPriority   = 0;		 // Semi-random priority set when creature selected.  When two people
				 // select same creature, one with lower priority backs off.

SelDNALCnt	= 0;		// # of of creatures on no-attack list.
SelDNAL		= new Array();	// List of creatures to not attack

SelOutRangeCnt	= 0;
SelOutRangeList	= new Array();

SelLockNext	= 0;		// Next entry to use in lock list
SelLockList	= new Array();	// List of creatures being attacked by macroers near us.  We check
				// this list before selecting a target to make sure someone else
				// is not attacking it already.

// **************************************************************************************************
//
// Attack context

attWandElement	= 0;		// If your current wand is resistance imbued, then this variable contains the
    				// element it is imbued with.  If set to 0, then wand not imbued

// Object that holds creature yield/vuln information for creatures whose kill attempt was aborted.
function AttPrevObj(Id) {
    this.Id	    = Id;	// Creature OID
    this.YieldExp   = 0;	// Yield expiration time
    this.ImperlExp  = 0;	// Imperl expiration time
    this.VulnExp    = 0;	// Vuln expiration time
    this.VulnLvl    = 0;	// Vuln level
} // end function

function AttDoneObj(Id, ExpTime) {
    this.Id	    = Id;	// Creature OID
    this.Aco        = skapi.AcoFromOid(Id);
    this.ExpTime    = ExpTime;	// Time creature is removed from done list, which matches
    				// the time the debuff will expire.
} // end func

function AttCreRecord() {
    // Context of current creature we are attacking (fixed values from creature db record or current creature's ACO)
    this.Aco		    = null;
    this.Oid		    = 0;
    this.HexOid		    = 0;
    this.CreIndex	    = 0;
    this.CreName	    = "";
    this.CreVuln1           = 0;
    this.CreVuln2           = 0;
    this.CreVuln3           = 0;
    this.CreMaxHealth       = 0;
    this.CreMaxMana	    = 0;
    this.CreMaxStamina	    = 0;
    this.CreMagicD	    = 0;
    this.CreExp		    = 0;
    this.CrePrio	    = 0;
    this.CreYieldFlag       = 0;
    this.CreVulnFlag        = 0;
    this.CreWarSpell        = 0;
    this.CreNoDrainHealth   = false;
    this.CreNoDrainStamina  = false;
    this.CreNoDrainMana     = false;
    this.CreNoStepDown      = false;
    this.CreDangerFlag	    = false;
    
    // If an OnObjectDestroy message is received for the current creature we are
    // killing, then abort kill attempt
    this.Destroyed          = false;

    // Variables to control attack **********************************************

    // Maximum spell level to use for different spell types based on the highest level
    // spell in our spellbook and the highest spell we are allowed to cast. Actual level
    // dependent on our current magic skill levels, spells we know, and and the health of the
    // creature.  Note since some of the max and min constants can be adjusted through the
    // command interface, we compute these values each creature kill attempt
    this.MaxBolt	    = 0;	// Maximum war bolt spell
    this.MinBolt	    = 0;	// Minimum war bolt spell.
    this.MaxArc		    = 0;	// Maximum war arc spell.
    this.MinArc		    = 0;	// Minimum war arc spell.
    this.MaxYield	    = 0;	// Maximum yield spell.
    this.MaxImperl	    = 0;	// Maximum imperl spell.
    this.MaxVuln	    = 0;	// Maximum vuln spell

    // Maximum level we can cast for each spell type.  These variables will be adjusted when
    // our magic skill levels and the health of the creature we are attacking are examined.
    this.CurVuln	    = 0;
    this.CurBolt	    = 0;
    this.CurArc		    = 0;

    // Next set of variables enable us to determine if a creature is moving.
    // May effect what war spell to use (bolt or arc)
    this.Moving		    = false;
    this.LastMove	    = new Date();		
    this.CurrLoc	    = 0;
    this.PrevLoc	    = 0;

    // Variables that keep track of how many time drain spells failed.  Eventually we will
    // stop draining when these counters reach a user settable threshold
    this.DrainHealthFails   = 0;		// # of times drain health failed
    this.DrainManaFails	    = 0;		// # of times drain mana failed
    this.DrainStaminaFails  = 0;		// # of times drain stamina failed
    this.LastDrainFail      = 0;		// Records time last drain failed.

    // Set variables that control when to give up using creatures mana/stam/health for replenishment.
    // Initially set to high value so at least one attempt is made.
    this.LastDrainMana	    = 100;		// Amount of mana received from last Drain Mana spell
    this.LastDrainStamina   = 100;		// Amount of stamina received from last Drain Stamina spell
    this.LastDrainHealth    = 100;		// Amount of health received from last Drain Health spell

    // The following variables used to store what time a yield/vuln/imperl was done and what spell was
    // used. If a creature is aborted for any reason (ie was not killed), then we save the time the
    // spell(s) will expire.  If creature is selected again then we don't need to recast the yield, vuln,
    // or imperl spell if the selection happens before the spells expire
    this.YieldTime	    = 0;		// Time yield was done
    this.YieldSpell	    = null;		// Spell used to yield
    this.YieldAttempts	    = 0;		// # of times we have attempted to yield the current creature
    this.VulnTime	    = 0;		// Time vuln was done
    this.VulnSpell	    = null;		// Spell used to vuln
    this.VulnAttempts	    = 0;		// # of times we have attempted to vuln the current creature
    this.ImperlTime	    = 0;		// Time imperl was done
    this.ImperlSpell	    = null;		// Spell used to imperl
    this.ImperlAttempts	    = 0;		// # of times we have attempted to imperl the current creature

    this.StartTime	    = new Date();	// Time current creature attack started
    this.MaxTime	    = 0;
    this.Locked		    = false;		// True if another person has this creature locked
    this.Element	    = 0;		// Element to use when attacking
    this.WarSpell	    = attSpellAuto;
    this.YieldFlag	    = attAuto;
    this.ImperlFlag	    = false;		// Set to true if imperl desired before every kill attempt
    this.VulnFlag	    = attAuto;
    this.VulnActive	    = false;		// Set to true when vuln cast or using same element as wand imbue.
    this.VulnLevel	    = 0;		// Set to level of vuln cast or estimated level of wand imbue.
    this.StepupActive	    = false;		//
    this.CreHealth          = 1;		// Current creature health on a 0 to 1 scale
    this.CreDist	    = 0;		// Last recorded distance of creature
    this.FailStart	    = 0;		// Count of spell failures at beginning of attack attempt
    this.WarSpellCnt	    = 0;		// # of successful war spell casts for this kill attempt
    
    // When certain decal panel options are selected, we periodically assess the creature we are attacking
    // until we get a successful assess.  This is done to either check the stats of a known creature, or
    // to find the stats of an unknown creature.  Note that if an assess fails, then the next assess will
    // not be done until the creature is closer then the distance where the last failed assess was done.
    this.LastAssessTime     = 0;		// Records time last assess was done.
    this.LastAssessDist	    = 999;		// Records distance of creature when last assess was done.

    // If creature was previously yielded, imperled, or vulned, then info is loaded here when
    // creature kill init done
    this.PrevYieldExp	    = 0;
    this.PrevImperlExp	    = 0;
    this.PrevVulnExp	    = 0;
    this.PrevVulnLvl	    = 0;
    
    // If this creature was the result of a critical switch, then the next variable will be set to true
    this.CritSelect	    = false;
} // end func

// Permanent attack context
function AttCtxRecord() {

    // Following two variables enable us to determine what spells we recently cast on creatures
    // so that we don't repeat a vuln on a creature if the same vuln was cast recently.
    this.PrevList	= new Array();	  // List of creatures that we cast spells on recently
    this.PrevCnt	= 0;		  // Count of creatures on AttCtx.PrevList
    
    this.DoneList	= new Array(200); // When in support mode, we need to keep track of the
    this.DoneCnt	= 0;		  // creatures we have vulned and/or imperled.  Not needed
    					  // in kill mode as creatures are removed from the object
    					  //  table after killing.

    this.Lvl7Override	= false;	 // When set to true (by decal command console), will cast lvl 7 war
    this.Lvl7OverStart	= 0;		 // spells for AttLvl7OverInt millseconds.
    this.Lvl7OverInt	= 0;		 // Set by decal command console (default is 15 minutes).

    // Miscellaneous
    this.AttackAco	= null;		// Set to ACO of last attacker
    this.LastKillOid	= 0;		// Oid of last creature killed
    this.LastKillTime	= 0;		// Time of last kill of selected creature
    this.LastDone	= 0;		// Records time last attack attempt ended.
    this.PrevCreExp	= 0;		// Experience of previously selected creature
    this.PrevCreName	= "xxxxx";	// Creature name of previous target
    this.PrevId		= 0;		// Identification of previous creature selected.  Used to determine if
					// event messages apply to previously selected creature.
    this.LastLostSel	= 0;		// Set when we lose selection on a creature that was near us.  Helps
				 	// us identify creatures killed by another person before they are
				 	// removed from the object table.
    this.ManSelOid	= 0;		// If nonzero, then user manually selected creature to attack
    this.Abort		= false;	// If set to true, then abort current creature				 	
    this.CompSusp	= false;	// Set to true when killing suspended because of low comps
} // end func

Att	= new AttCreRecord();		// Reset every time new creature selected.

AttCtx	= new AttCtxRecord();		// Permanent attack context.

// **************************************************************************************************
//
// Movement handling

MoveDone	= false;
MoveCode	= 0;
RetreatReq	= false;	// Set to true when a retreat is required
AdvancePending	= false;	// Set to true when advance needed but we want to do one buff/loot cycle
AdvanceReq	= false;	// Set to true when we should move back to KillLoc (from retreat loc)
RetreatActive	= false;
RetreatLastAtt	= 0;		// Last time a retreat was attempted
NavActive	= false;	// Set to true when a navigation is active
SpawnAdvance	= false;	// Set to true when a creature spawns near us and spawn advancing is enabled
MoveOverRide	= 0;		// When set, instead of going to the next attack location, we
				// go to the one indicated here
PrevRetreatAmt	= 0;
RetreatAmount	= 0;

function AttLocRec() {
    this.KillLoc	= null;		// Maploc where we kill creatures
    this.WayLoc		= null;		// Maploc we traverse when approaching/leaving KillLoc
    this.RetreatLoc	= null;		// Maploc where we retreat to when KillLoc is overrun by creatures
    this.LocFlags	= 0;		// Bitmask of flags for this location.
    
    // Now define some stats
    this.LastArrive	= 0;		// Arrival time from visit
    this.LastDepart	= 0;		// Departure time from last visit
    this.LastInt	= 0;		// How long we spent at this location last time we were here
    this.TotalVisits	= 0;		// # of times we have visited this location
    this.TotalRetreats	= 0;		// # of times we retreated from this location
    this.TotalInt	= 0;		// Total amount of time we have spent here
    this.TotalKilled	= 0;		// # of creatures killed at this location
    this.TotalExp	= 0;		// Total exp from all creatures kill here
    this.TotalLooted	= 0;		// Total # of creatures looted at this location
} // end function

AttLoc		= new Array();
AttLocCur	= 0;		// Array index of current attack position
AttLocCnt	= 0;		// # of attack locations defined
MoveStart	= 0;		// Last time we started last changed positions
MoveEnd		= 0;		// End time of last movement
LastArriveTime	= 0;		// Last time we arrived at a location
MasterWayLoc	= null;		// Maploc we traverse between AttLocs

// Temporary holding variables for nav info when being loaded from another character via @tells
NavLoadActive	= false;	// Set to true when nav load is active
New_AttLoc	= new Array();
New_AttLocCur	= 0;		// Array index of current attack position
New_AttLocCnt	= 0;		// # of attack locations defined
New_MoveGroup	= 0;
New_MasterWayLoc= null;		// Maploc we traverse between AttLocs

// **************************************************************************************************
//
// Idle Context

// Variables
LastHourly	= 0;		// Last time we logged hourly stats
RotateLast	= new Date();	// Last time we turned.  Used for chars that are leeching exp from fellow

MacroPaused	= false;	// Set by user commands to pause macro operation

CorpseCheckDone	= false;	// Set to true when no valid corpse found to loot during LOOT state
UstCheckDone	= false;	// Set to true when UST routine has completed for the current
				// idle state run.

SuspAttackMode	= false;	// When comps suspended, holds original value of AttackMode variables
SuspAutoMove	= false;	// When comps suspended, holds original value of AutoMove variable

// **************************************************************************************************
//
// Spell Casting Context

// Maximum level spells we will cast based on what is in our spell book and what the user set as
// the maximum in WAR_CONFIG.JS
CastMaxBolt	= new Array();	// Maximum bolt spell for each damage type
CastMaxArc	= new Array();	// Maximum arc spell for each damage type
CastMaxVuln	= new Array();	// Maximum vuln spell for each damage type
CastMaxYield	= 0;		// Maximum yield spell

// Spell level to use for various critical spells. Filled in later.
SpellLvlDrainHealth	= 0;
SpellLvlDrainStamina	= 0;
SpellLvlDrainMana	= 0;
SpellLvlStamHealth	= 0;
SpellLvlStamMana	= 0;
SpellLvlHeal		= 0;
SpellLvlRevitalize	= 0;
SpellLvlStrength	= 0;

SpellObj_DrainHealth	= null;
SpellObj_DrainStamina	= null;
SpellObj_DrainMana	= null;
SpellObj_StamHealth	= null;
SpellObj_StamMana	= null;
SpellObj_Heal		= null;
SpellObj_Revitalize	= null;
SpellObj_Strength	= null;

// **************************************************************************************************
//
// Inventory Management

VendorOpen	= false;
VendorAco	= null;
VendorMcmBuy	= 0;
VendorMaxBuy	= 0;
VendorFractBuy	= 0;
VendorFractSell	= 0;
AbortAction	= false;

// **************************************************************************************************
//
// Decal/Command Context

flagVerbose	= true;		// If set to true, send messages to chat area when decal controls are
				// detected.  Set to false when we automatically set controls through
				// the SetProperty method.

szPanelControl	= "WB Control";
szPanelConfig	= "WB Config";
szPanelMovement	= "WB Movement";
szPanelBuffs	= "WB Buffs";
szPanelLoot	= "WB Loot";
szPanelCreature	= "WB Creatures"; 

// **************************************************************************************************
//
// File context

Fso		= new ActiveXObject("Scripting.FileSystemObject")
ForWriting	= 2;

// **************************************************************************************************
//
// Item detection, notification, looting, and usting
CorpseList	= new Array();	// List of corpses to be opened up
CorpseCount	= 0;		// Count of corpses on CorpseList

PrevCorpseList	= new Array();	// List of corpse oids previously looted.
PrevCorpseNext	= 0;		// Next entry to use on PrevCorpseList.

UstAco		= null;		// ACO of UST

LootList	= new Array();	// Holds items found on a corpse
LootCount	= 0;		// Count of items found on corpse

CurItem		= -1;		// Current item we are looking at
PickupActive	= false;	// Set to true when trying to pickup item
SalvageActive	= false;	// Set to true when trying to salvage an item
MoveActive	= false;	// Set to true when we are trying to move items around in inventory
InventoryAdd	= false;	// Set to true when item added to inventory
BagToMove	= false;	// Set to true when at least one full bag of salvage needs to be moved to side pack
LootSortMode	= false;
LootMoveDone	= false;
HaveUnkScrolls	= false;	// Set to true when at least one unknown scroll on loot scroll list
CorpseDist	= 0;
UstFailCount	= 0;		// # of times UST operation failed.  When we reach the maximum allowed USTing is disabled.
CorpseCheckCnt	= 0;		// # of corpses looted

ItemLastAlert	= 0;


// Object that holds a corpse found through the OnObjectCreate event
function CorpseObj(Aco) {
    this.Aco		= Aco;
    this.Oid		= Aco.oid;
    this.CreTime	= new Date();
    this.AssessReq	= true;		// Needs to be assessed
    this.AssessDone	= false;	// Assess not completed yet
    this.AssessCount	= 0;		// # of times assess attempted
    this.AssessTime	= 0;		// Time last attempt done
    this.Ours		= true;		// Assume we own it for now
    this.Opened		= false;	// When opened for first time, set flag to true
    					// and increment corpse checked count
} // end function

// Object that holds an item found on a corpse
function ItemObj(Aco) {
    this.Aco		= Aco;
    this.Oid		= Aco.oid;
    this.CreTime	= new Date();	// When looting, time object detected from opencontainer event
    this.ObjCreate	= false;	// When looting, set to true when object create event received
    this.AssessReq	= true;		// Assume assess needed at first, will set to false if not necessary
    this.AssessDone	= false;	// Assess not completed yet
    this.AssessCount	= 0;		// # of times assess attempted
    this.AssessTime	= 0;		// Time last attempt done
    this.Checked	= false;	// Set to true when finished processing on this item
    this.Desc		= "";		// Description of item
    this.LootType	= 0;		// Type of loot
} // end function


function SelCreObj(Id) {
    this.Id = Id;		// Id of creature

    // Record name of creature if ACo exists
    var TempAco = skapi.AcoFromOid(Id);
    if ((TempAco != undefined) && (TempAco != null))
	this.Name = TempAco.szName;
    else
        this.Name = null;

    this.TimeAdd	= new Date();	// Time added to list
    this.TimeLast	= new Date();	// Last time entry updated
    this.Count		= 1;		// # of times added to list
} // end func


function SplRec() {
    // For each damage type we will compute the following info...
    this.MaxBolt	= 0;		// Max bolt spell we can cast
    this.MinBolt	= 0;		// Min bolt spell we can cast
    this.MaxArc		= 0;		// Max arc spell we can cast
    this.MinArc		= 0;		// Min arc spell we can cast
    this.MaxVuln	= 0;		// Max vuln spell we can cast
    this.MinVuln	= 0;		// Min vuln spell we can cast

    // Arrays used to determine what war and vuln spells we know. A zero indicates we do not
    // know the spell, otherwise the spell id is inserted
    this.BoltMap	= new Array();	// Bolt spells we know
    this.ArcMap		= new Array();	// Arc spells we know
    this.VulnMap	= new Array();	// Vuln spells we know

    // Arrays used by life/war stepdown routines to determine what vuln, bolt, or arc spell to use
    // based on our current magic skill level and min/max restrictions set by user
    this.BoltStepDown	= new Array();
    this.BoltStepUp	= new Array();
    this.ArcStepDown	= new Array();
    this.ArcStepUp	= new Array();
    this.VulnStepDown	= new Array();
} // end function SplRec


function SplObj() {
    if (debug) Log.OutD("SplObj: Begin");

    // Spell Info for each damage type.  Includes
    this.Info = new Array();
    for (var x=1; x<9; x++) this.Info[x] = new SplRec();

    // For each damage type, check what arc, bolt, and vuln spell we have in our spell
    // book. Also determine the lowest and highest spell level for arcs, bolts, and vulns
    // by going through the spellbook info for this char and seeing what the maximum spell
    // we have for each damage type.  The maximum spell is also contrained by the constants
    // 'maxWarSpell', 'maxYieldSpell', and 'maxVulnSpell'.

    for (var x=1; x<8; x++) {
	if (debug2) Log.OutD("SplObj: Checking damage type [" + damName[x] + "]");

    	// Check bolts
    	this.Info[x].BoltStepDown[0] = 0;
	for (var y=1; y<=spellMaxLvl; y++) {
    	    var SpellId = Damage.Info[x].Bolt[y];
    	    if ((Cast.SpellBook[SpellId] != undefined) && Cast.SpellBook[SpellId]) {
    		if (debug2) Log.OutD("SplObj: Bolt Lvl [" + y + "] in spell book");
		this.Info[x].BoltMap[y] = true;
		this.Info[x].MaxBolt = y;		// Update maximum bolt spell we can cast
		this.Info[x].BoltStepDown[y] = y;
		if (this.Info[x].MinBolt == 0) this.Info[x].MinBolt = y;	// Set minimum bolt spell we can cast
    	    } else {
    	        if (debug2) Log.OutD("SplObj: Bolt Lvl [" + y + "] not in spell book");
    	        this.Info[x].BoltMap[y] = false;
		this.Info[x].BoltStepDown[y] = this.Info[x].BoltStepDown[y-1];
    	    } // endif
    	} // end for

    	// Now setup bolt stepup structures
    	this.Info[x].BoltStepUp[spellMaxLvl+1] = 0;
    	for (var y=spellMaxLvl; y>0; y--) {
	    if (this.Info[x].BoltMap[y]) {
	    	this.Info[x].BoltStepUp[y] = y;
	    } else {
	    	this.Info[x].BoltStepUp[y] = this.Info[x].BoltStepUp[y+1];
	    } // endif
	} // endfor

    	// Check arcs
    	this.Info[x].ArcStepDown[0] = 0;
 	for (var y=1; y<=spellMaxLvl; y++) {
    	    var SpellId = Damage.Info[x].Arc[y];
    	    if ((Cast.SpellBook[SpellId] != undefined) && Cast.SpellBook[SpellId]) {
    		if (debug2) Log.OutD("SplObj: Arc Lvl [" + y + "] in spell book");
    		this.Info[x].ArcMap[y] = true;
    		this.Info[x].MaxArc = y;
    		this.Info[x].ArcStepDown[y] = y;
    		if (this.Info[x].MinArc == 0) this.Info[x].MinArc = y;	// Set minimum arc spell we can cast
    	    } else {
    	        if (debug2) Log.OutD("SplObj: Arc Lvl [" + y + "] not in spell book");
		this.Info[x].ArcMap[y] = false;
		this.Info[x].ArcStepDown[y] = this.Info[x].ArcStepDown[y-1];
    	    } // endif
    	} // end for

   	// Now setup arc stepup structures
    	this.Info[x].ArcStepUp[spellMaxLvl+1] = 0;
    	for (var y=spellMaxLvl; y>0; y--) {
	    if (this.Info[x].ArcMap[y]) {
	    	this.Info[x].ArcStepUp[y] = y;
	    } else {
	    	this.Info[x].ArcStepUp[y] = this.Info[x].ArcStepUp[y+1];
	    } // endif
	} // endfor

     	// Check vuln
     	this.Info[x].VulnStepDown[0] = 0;
	for (var y=1; y<=spellMaxLvl; y++) {
    	    var SpellId = Damage.Info[x].Vuln[y];
    	    if ((Cast.SpellBook[SpellId] != undefined) && Cast.SpellBook[SpellId]) {
    		if (debug2) Log.OutD("SplObj: Vuln Lvl [" + y + "] in spell book");
		this.Info[x].VulnMap[y] = true;
    		this.Info[x].MaxVuln = y;
    		this.Info[x].VulnStepDown[y] = y;
    		if (this.Info[x].MinVuln == 0) this.Info[x].MinVuln = y;   // Set minimum vuln spell we can cast
    	    } else {
    	        if (debug2) Log.OutD("SplObj: Vuln Lvl [" + y + "] not in spell book");
		this.Info[x].VulnMap[y] = false ;
		this.Info[x].VulnStepDown[y] = this.Info[x].VulnStepDown[y-1];
    	    } // endif
    	} // end for
    } // end for

    // Define yield structures
    this.MinYield = 0;
    this.YieldMap = new Array();	// Flags that indicate whether or not we know a yield spell for a lvl
    this.YieldStepDown = new Array();
    this.YieldStepDown[0] = 0;
    for (var y=1; y<=spellMaxLvl; y++) {
	var SpellId = Damage.Yield[y];
	if ((Cast.SpellBook[SpellId] != undefined) && Cast.SpellBook[SpellId]) {
	    if (debug2) Log.OutD("SplObj: Yield Lvl [" + y + "] in spell book");
	    this.YieldMap[y] = true;
	    this.YieldStepDown[y] = y;
	    this.MaxYield = y;	// Update maximum yield spell we can cast
	    if (this.MinYield == 0) this.MinYield = y;
	} else {
	    if (debug2) Log.OutD("SplObj: Yield Lvl [" + y + "] not in spell book");
	    this.YieldMap[y] = false;
	    this.YieldStepDown[y] = this.YieldStepDown[y-1];
	} // endif
    } // end for

    // Define imperl structures
    this.MinImperl = 0;
    this.ImperlMap = new Array();	// Flags that indicate whether or not we know an imperl spell for a lvl
    this.ImperlStepDown = new Array();
    this.ImperlStepDown[0] = 0;
    for (var y=1; y<=spellMaxLvl; y++) {
	var SpellId = Damage.Imperl[y];
	if ((Cast.SpellBook[SpellId] != undefined) && Cast.SpellBook[SpellId]) {
	    if (debug2) Log.OutD("SplObj: Imperl Lvl [" + y + "] in spell book");
	    this.ImperlMap[y] = true;
	    this.ImperlStepDown[y] = y;
	    this.MaxImperl = y;	// Update maximum imperl spell we can cast
	    if (this.MinImperl == 0) this.MinImperl = y;
	} else {
	    if (debug2) Log.OutD("SplObj: Imperl Lvl [" + y + "] not in spell book");
	    this.ImperlMap[y] = false;
	    this.ImperlStepDown[y] = this.ImperlStepDown[y-1];
	} // endif
    } // end for

    if (debug2) {
    	Log.OutD("CastInit: Min/Max War Bolts");
    	for (var x=1; x<8; x++) {
	    Log.OutD("SplObj: [" + damName[x] + "] Min bolt spell [" + this.Info[x].MinBolt +
	     	     "] Max bolt spell [" + this.Info[x].MaxBolt + "]");
	    for (var y=1; y<=spellMaxLvl; y++)
	    	Log.OutD("SplObj: " + y + "  Up: " + this.Info[x].BoltStepUp[y] +
	    				"  Down: " + this.Info[x].BoltStepDown[y]);
	} // end for


 	Log.OutD("SplObj: Min/Max War Arcs");
 	for (var x=1; x<8; x++) {
	    Log.OutD("SplObj: [" + damName[x] + "] Min arc spell [" + this.Info[x].MinArc +
	     	 "] Max arc spell [" + this.Info[x].MaxArc + "]");
	    for (var y=1; y<=spellMaxLvl; y++)
	    	Log.OutD("SplObj: " + y + "  Up: " + this.Info[x].ArcStepUp[y] +
	    				"  Down: " + this.Info[x].ArcStepDown[y]);
	} // end for

	Log.OutD("SplObj: Min/Max Life Vulns");
	for (var x=1; x<8; x++) {
 	    Log.OutD("SplObj: [" + damName[x] + "] Min vuln spell [" + this.Info[x].MinVuln +
	     	 "] Max vuln spell [" + this.Info[x].MaxVuln + "]");
	    for (var y=1; y<=spellMaxLvl; y++)
	    	Log.OutD("SplObj: " + y + "  Down: " + this.Info[x].VulnStepDown[y]);
	} // endfor

    	Log.OutD("SplObj: Min yield  [" +  this.MinYield  + "] Max yield  [" +this.MaxYield + "]");
	for (var y=1; y<=spellMaxLvl; y++)
	    Log.OutD("SplObj: " + y + "  Down: " + this.YieldStepDown[y]);

    	Log.OutD("SplObj: Min imperl [" +  this.MinImperl + "] Max imperl [" +this.MaxImperl + "]");
	for (var y=1; y<=spellMaxLvl; y++)
	    Log.OutD("SplObj: " + y + "  Down: " + this.ImperlStepDown[y]);
    } // endif
    debug2 = false;
    if (debug) Log.OutD("SplObj: End");
} // end object SplObj


function LockObj(Oid) {
    this.Oid = Oid;
    this.CreTime = new Date();
} // end function

function SortRec(Index, Units, Value) {
    this.Index	= Index;
    this.Units	= Units;
    this.Value	= Value;
} // end function

// Reset general config variables to program defaults
function GenVarInit() {

    // Gen 1 tab variables   ************************************************************
    flagLogBuffs	= true;		// If true, log item buffs to file.
    flagReduceBurden	= false;	// If true, cast strength spell when overburdened
    flagDeathLog    	= false;	// If true will immediately log on death, otherwise macro will continue and
    					// optionally log if 'deathLogWait' set below
    flagDeathRebuff	= true;		// If true and flagDeathLog false, will rebuff character after death
    flagAutoExcl	= false;	// If true, automatically turn exclusion mode on if members of fellow detected
    					// nearby, and turn off if no members detected
    flagWarAvoid	= true;		// If true, detect and avoid war spells
    flagFellowStealth	= false;	// If true, log when one member of fellow detects a player
    flagAllegStealth	= true;		// If false, then don't autolog when allegiance member comes into range
    flagAutoRespond 	= false;	// If true will respond to tells from unauthorized sources with stock
    					// reply defined below in var 'tellResponse'
    flagAllowDisband	= true;		// Allow authorized users to disband fellow
    flagAutoResume	= false;	// If true, automatically resume from paused state after 'autoResume' ms.
    flagAcceptFellow	= false;	// If true, allow people to pull us into a fellowship.  Otherwise they must be
    					// on the authorized list.
    flagGem 		= false;	// If true, then we will use a gem of stillness when we are taking heavy
    					// damage, have life vulns on us, and health/stamina/mana low. See 'gem'
    					// variables in war_config.js to see more restrictions on gem use.
    flagSlideCast	= true;		// If true, slide cast when war spell heading for us and we are casting a spell
    anybodyFellow	= false;	// If true, allow anybody to request fellow
    passwordFellow	= false;	// If true, allow users to specify a password to join fellow

    // Gen 2 tab variables    ************************************************************
    flagSelCheckZ	= false;
    flagAssessKnown	= false;
    flagAssessUnknown	= true;
    flagIgnoreAtt	= false;	// If true, don't respond to creature attacks directly
    manaStoneLimit	= manaCnt;	// Highest mana stone to use.  Default is NONE
    spellAugGems	= 0;		// # of spell aug gems used by this char.  Enables warbot to correctly compute
					// the expiration time of item spells.

    // The next set of variables control how we help out other people in our area
    attSupportOnly	= false;	// Don't kill creatures, just imperl/vuln/heal/stam
    attAlwaysImperl	= false;	// Always imperl creatures
    attAlwaysVuln	= false;	// Always vuln creatures
    attHealFellow	= false;	// Heal fellow members when their health is low
    attStamFellow	= false;	// Stam fellow members when their health is low
    flagBuffDanger	= true;		// If true, continue casting buffs while danger exists
    flagSelByDist	= false;	// If true, just select by distance, not by priority

    
    // Move tab variables     ************************************************************
    flagAutoHome	= true;		// If true, and we wander to far from our kill location, then go back to it.
    flagMoveDelay   	= false;	// If true, pause after navigations to prevent spell casting failures.
    flagRetreatCrit	= true;		// Move to retreat point if # of creatures within our danger radius
    					// equals or exceeds 'RetreatAmount' creatures
    flagMultRetreat 	= false;	// If true, retreating enabled when part of a movegroup with 2 or more members.
    flagHealthRetreat   = true;		// If true, when we do a big heal (stam to health), then move to retreat location
    flagRecordLoc 	= true;		// If true, record location changes that are more than 'locRecordThresh' map units
    					// from last recorded position
    retreatAmtOne	= 4;		// Retreat creature count when solo
    retreatAmtTwo	= 7;		// Retreat creature count when 2 people in same area
    retreatAmtThreePlus	= 10;		// Retreat creature count when 3 or more people in same area
    retreatDamThresh	= 50;		// If any one blow from a creature exceeds this, then retreat

    MoveGroup		= 0;		// When set, we look for movement messages from other members
					// of our movegroup and move when they do.

    // Info tab variables
    flagReportPlayer	= true;		// If true, display all characters detected to chat area
    flagReportLoot	= true;		// If true, display all loot picked up to chat area
    flagReportAbort 	= true;		// If true and we abort attacking a creature, display reason to chat area
    flagDispExp		= false;	// If true, display unspent exp changes
    flagDispSkillExp	= false;	// If true, display skill exp changes.
    flagItemSpam	= false;	// If true, notify Support when alertable item found on corpse
    flagFellowSup	= false;	// If true, send important macro events to fellow
    flagFriendSup	= false;	// If true, send important macro events to listed chars
    					// listed in the config array variable 'supportChars' defined below
    flagVerbose		= false;	// If true, display detailed info when enabling/disabling decal panel options


    // Loot tab variables   ***************************************************
    flagLootCheck   	= false;	// If set, we will check monster corpses for interesting items
    flagLootAll		= false;	// If true, loot all corpse.  If false only loot our kills
    flagAutoUst		= false;	// If true, we will automatically use UST on salvage
    flagLootWait	= true;		// If true, wait 90 seconds before looting other peoples kills,
    					// otherwise we may not be able to loot corpse.
    flagMoveFull	= false;	// If true, move full bags of salvage out of main pack (to any side pack).
    maxBuyNote		= 0;		// Maximum note we will buy during auto sell operation
    minBuyNote		= 0;		// Minimum note we will buy during auti sell operation

    // Startup tab variables   ***************************************************
    startBuffMode	= true;		// Initial value of BuffMode variable
    startAttackMode 	= false;	// Initial value of AttackMode variable
    startStealth	= false;	// Initial value of Stealth variable
    startAutoMove	= true;		// Initial value of AutoMove variable
    startExclMode	= false;	// Initial value of ExclMode variable
    startSafeMode	= true;		// Initial value of SafeMode variable
    startPeaceMode	= false;	// Initial value of PeaceMode variable
    creFellow		= false;	// Create fellowship at startup?


    // Spell 1 tab variables ************************************************************
    flagYieldDisable	= false;	// If true, then don't yield creatures ever
    flagYieldNoMagD	= false;	// If true, then yield when the creatures magicd is unknown.
    
    // Enable or disabled automatic stepdown of life and/or war spells.  Instead of casting the maximum
    // life and/or war spell we can cast, we will cast the minimum spell required to kill a creature.
    // For example if the maximum vuln and war spell we can cast is lvl 7, but the health of the creature
    // is only 150, then a lvl 6 vuln and war will kill the create 100% of the time (lvl 6 vuln adds 150%
    // to damage and minimum damage for lvl 6 war is 61 so vuln*minwar damage = 152).
    attVulnStepdown	= true;
    attWarStepdown	= true;

    // Enable or disable automatic stepup of war spells to level 7 when the number of creatures
    // within 'criticalRadius' equals or exceeds 'UrgentAmount'
    attWarStepup	= true;

    healStepUpThresh	= 0;		// When health falls below this level and our heal self spell is set
    					// to lvl 6, then cast a heal 7.  Reduces plat usage.

    // Variables related to the current wands rendering type.  If you set the wand element manually, use one of
    // following damage type constants: damBlade, damPierce, damBludgeon, damCold, damFire, damAcid, damElectric
    // For example: attWandElement = damFire;
    attUseWandImbue	= true;		// If set to true, then
    attMatchVuln2	= true;		// If set, then don't vuln a creature if its secondary vuln matches wand imbue
    attMatchVuln3	= true;		// If set, then don't vuln a creature if its tertiary vuln matches wand imbue

    // Set War spell to cast. Possible values for this variable are:
    //	attSpellBolt	Use Bolts only for war spells
    //	attSpellArc	Use Arcs only for war spells
    //	attSpellAuto	Use highest spell we have for a particular damage type unless creature is
    //			moving in which case we use a bolt spell.
    attSpell 	= attSpellAuto;

    // If the next variable is true, then ring spells will be used anytime enough creature are close to us
    attUseRingSpells	= false;

    // Spell 2 tab variables ************************************************************

    // Set maximum spell levels will can cast.  Note actual may be lower if max level spell is not in spellbook
    // or max spell required skill exceeds our current skill as defined in 'attWarSkill' or 'attDebuffSkill'.
    // War spells can be reduced in level if a creature can be killed with a lower level. Normally you
    // shouldn't have to touch these values unless for some reason you don't want to cast lvl 7 spells.
    maxWarSpell  	= 6;	// Set highest level war spell we will cast
    minWarSpell  	= 4;	// Set lowest level war spell we will cast
    maxYieldSpell 	= 6;	// Set highest level yield (creature) spell we will cast
    maxImperlSpell	= 6;	// Set highest level imperl spell we will cast
    maxVulnSpell 	= 6;	// Set highest level vuln (life) spell we will cast
    spellHeal	  	= 6;
    spellStamHealth	= 2;
    spellStamMana	= 6;
    spellRevitalize	= 6;
    spellStrength	= 6;
    spellDrainHealth	= 3;
    spellDrainStamina	= 3;
    spellDrainMana	= 3;
    
    flagUseTuskerSpit	= false; 	// If true, use tusker spit ale
    flagUseAmberApe	= false;	// If true, use amber ape
    flagUseBoboStout	= false;	// If true, use bobo's stout

    // Component tab variables  ****************************************************
    flagCompCheck	= true;		// If true, check component levels
    flagCompLogOff	= false;	// If true, log when a component is below its critical level,
    					// otherwise macro will go into defense mode and disable automove
    					// when critical level reached for any one component
    flagCompRetreat	= true;		// When true and CompLogOff = false, then when critical level reached goto
					// the current location's retreat point
    flagCompAssist	= false;	// If true, then broadcast comps we need to fellowship
    flagSplitPeas	= true;		// Controls whether we split peas or not

    // Alert tab variables   ********************************************************
    alertTell		= false;
    alertAlleg		= false;
    alertDeath		= true;		// When we die, play sound
    alertPlayer		= false;	// Play audible alert when player not on friends list detected
    alertStealth	= true;		// Play alert when stealth mode activated, ie we are hiding from someone
    alertComps		= true;
    alertItem		= true;
    alertAbort		= true;
    alertAdmin		= true;		// If true, sound alert when admin detected.

    // Debug tab variables ***********************************************************
    debug		= false;	// Will log basic debugging information for non-events
    debugEvent 		= false;	// Will log debug information for events
    debugState		= false;	// Show debug info related to FSM state changes.
    debugSelect		= false;	// Show debug information related to selecting creatures
    debugTarget		= false;	// Show debug info related to targeting creatures
    debugAttack		= false;	// Show debug information related to killing creatures
    debugMove   	= false;	// Show debug info related to movement between attack locations
    debugLoot		= false;	// If true, log information related to corpse looting
    debugSalvage	= false;	// If true, log information related to material salvaging
    debugIdle		= false;	// Show debug information related to IDLE state processing
    debugBuffs		= false;	// Show debugging information related to buffing
    debugCast		= false;	// Show debugging information related to casting spells

    // If the following variable is set, then only creatures whose experience points are greater
    // than 'selMinExp' will be selected.  Note that if a creature who gets too close to us
    // will be attacked regardless of the setting of this variable. Set this variable to 0 if you
    // don't want to do this check
    selMinExp	= 5000;


} // endif