// WARBOT.JS
//
// Author:	Poundboy
// Version:	2.12
//
//
//
// Description:
//
//	This module is the main code for the WarBot macro. See AAA_README.TXT and
//	CHANGE.TXT files for more details
//
//
// Modification History:
//
// Who	  Date	    Version	Description
//
// PB	  09/13/02  1.01	Initial version.  Mostly based on old AcScript drainbot
// PB	  02/10/04  2.08	Split out FSM state routines to WAR_STATES.JS
//

function main() {
    // Define log object for debugging information
    Log = new LogObj();
    
    if (!Fso.FolderExists(macroDir)) {
	// Moron didn't unzip files to correct folder.  Hopefully this check will be the end of the
	// 1000 posts I get every release concerning the script error in md_logobj.s line 154 path not found
	Log.OutD("You must unzip files to folder '" + macroDir + "' for the macro to work properly.", true);
	Log.OutD("Read the instructions and try again.", true);
	return;
    } // endif

    // Enable logging to file and write header
    Log.EnableFile(fileRunLog);
    Log.OutD(" ");
    Log.OutD(" ");
    Log.OutD(" ");
    Log.OutD(" ");
    Log.OutDM("Main", "Macro starting...");
    Log.OutD("Main: Macro version [" + macroVersion + "]");
    Log.OutD("Main: Skunkworks version [" + skapi.szVersion + "]");
    Log.OutD("Main: RunningOnNT [" + skapi.fRunningOnNT + "]");
    Log.OutD("Main: Character name [" + skapi.szName + "] World [" + skapi.szWorld + "]");
    Log.OutD("Main: Char lvl [" + skapi.lvl + "] CHOP [" + Util.Hex(skapi.chop) + "]");
    skWar  = skapi.SkinfoFromSkid(skidWarMagic);
    skLife = skapi.SkinfoFromSkid(skidLifeMagic);
    skCre  = skapi.SkinfoFromSkid(skidCreatureEnchantment);
    Log.OutD("Main: Base magic skills, War [" + skWar.lvlUnbuffed + "] Life [" + skLife.lvlUnbuffed +
    	     "] Cre [" + skCre.lvlUnbuffed + "]");

    if (skapi.szVersion<"3.3") {
	Log.OutDM("Main", "Skunkworks v3.3 or later required. Aborting...", ToChat);
	return;
    } // endif

    // Make sure our character is in the world
    if (!CheckGameState()) return;

    // Initialize default values in case no config file found
    GenVarInit();
    
    Log.OutDM("Main", "Loading creature database...");
    Creature.Init(fileRaces, fileCreatures, fileCreLog);	// Load databases from files

    // Make sure top level configuration directory exists
    if (!Fso.FolderExists(configDir)) {
	// Config directory doesn't exist, then create it
	Fso.CreateFolder(configDir);
	Log.OutDM("Main", "Configuration directory '" + configDir + "' created.", ToChat);
    } // endif
    
    // Make sure configuration directory exists for this character
    CharCfgDir = configDir + "\\" + skapi.szName;
    if (!Fso.FolderExists(CharCfgDir)) {
	// Character config directory doesn't exist, then create it
	Fso.CreateFolder(CharCfgDir);
	Log.OutDM("Main", "Configuration directory '" + CharCfgDir + "' created.", ToChat);
	Log.OutDM("Main", "Moving any config files in old location to new config folder...", ToChat);
	MoveOldCfgFiles();
	Log.OutDM("Main", "Files moved.", ToChat);
    } // endif

    Log.OutDM("Main", "Loading general config file...", ToChat);
    if (GenLoad(0)) {
	Log.OutDM("Main", "General config info loaded from " + GenFileName, ToChat);
    } else {
	Log.OutDM("Main", "WARNING!!! General config file " + GenFileName +
		  " not found! Default profile loaded!", ToChat, cmcYellow);
    } // endif
    
    Log.OutDM("Main", "Loading creature config file...", ToChat);
    if (CreLoad(0)) {
	Log.OutDM("Main", "Creature config info loaded from " + CreFileName, ToChat);
    } else {
	Log.OutDM("Main", "WARNING!!! Creature config file " + CreFileName +
		  " not found! Default profile loaded!", ToChat, cmcYellow);
    } // endif

    // Process any user defined variables
    var UserConfigFile = CharCfgDir + "\\" + fileUserCfg + fileCfgExt;
    if (Fso.FileExists(UserConfigFile)) {
    	// Read in user config file for this char
    	Log.OutDM("Main", "Loading user defined config from file '" + UserConfigFile + "'...", ToChat);
	Util.Include(UserConfigFile);
	Log.OutDM("Main", "User config loaded.", ToChat);
    } else {
    	CreateUserConfig(UserConfigFile);
    	Log.OutDM("Main", "User config file '" + UserConfigFile +
    		  "' does not exist! Template user config file created.", ToChat, cmcYellow);
    } // endif

    // Initialize creature database
    Log.OutDM("Main", "Applying user modifications to creature database...");
    CreInit();   // Modify database from user settings

    // Setup macro internal variables based on defaults/config info
    MacroInit();

    // Initialize spell casting context
    Log.OutDM("Main", "Initing spell casting context...");
    if (debugCast) Cast.SetDebug(true);
    CastInit();

    // Make sure we know all the spells needed for proper macro operation
    SpellCheck(true);

    // Initialize buff database
    if (debugBuffs) Buff.SetDebug(true);
    Log.OutDM("Main", "Loading buff database from file...");
    Buff.Init(fileBuffs);
    var LoadCount = Buff.Load(0);
    if (LoadCount > 0) {
        Log.OutDM("Main", LoadCount + " active buffs loaded from file " + Buff.FileName, ToChat);
    } else {
    	if (LoadCount < 0) {
	    Log.OutDM("Main", "WARNING!!! Default buff config file " + Buff.FileName + " not found!!!", ToChat, cmcYellow);
	    Log.OutDM("Main", "WARNING!!! You need to create a buff profile and save it through the buff panel.",
	    	      ToChat, cmcYellow);
	} // endif
	Log.OutDM("Main", "WARNING!!! No active user defined buffs found!  No buffs will be cast!!!", ToChat, cmcYellow);
    } // endif

    // See what equipment we are wearing.
    Log.OutDM("Main", "Checking equipment...", ToChat);
    // Buff.SetDebug(true);		// for testing

    // Load permanent equipment from default config file
    var EquipCount = Buff.LoadEquip(0);
    if (EquipCount < 0) {
    	// Equipment file does not exist, create a template one
    	Log.OutDM("Main", "Creating equipment template config file " + Buff.FileName + " ...", ToChat);
	CreateEquipConfig(Buff.FileName);
    } // endif
    
    // Make sure equipment is still in inventory and see what we are wearing.  If a wand is
    // equiped then the function returns the render type of the wand, 0 if not imbued.
    attWandElement = Buff.CheckEquip();

    // If item buff logging enabled then process item buff logfile to see if any buffs still active.	
    var ItemBuffLog = CharCfgDir + "\\" + fileItemBuffLog;
    if (flagLogBuffs) {
	Buff.ProcItemLog(ItemBuffLog)
	Buff.SetBuffLog(true,ItemBuffLog);
    } else {
    	Buff.SetBuffLog(false,ItemBuffLog);	// just set item buff filename in case we enable later.
    } // endif

    // Setup looting and salvaging structures
    Log.OutDM("Main", "Setting up looting structures...");
    Loot.Init();		// Initialize data structures in loot object
    Loot.LoadDefaults();	// Load defaults.
    // LootDebug = true;
    // LootDebugScreen = true;
    SalvageInit();		// Init salvaging structures

    // Load any user defined looting/salvaging parameters
    Log.OutDM("Main", "Loading loot profile...", ToChat);
    if (Loot.Load(0)) {
	Log.OutDM("Main", "Loot profile loaded from " + Loot.FileName, ToChat);
    } else {
	Log.OutDM("Main", "WARNING!!! Loot config file " + Loot.FileName +
		  " not found! Default loot profile loaded!", ToChat, cmcYellow);
    } // endif

    // Enable reception of Skunkwork events
    Log.OutDM("Main", "Enabling event handlers...");
    fAddHandlers();

    // Define pause delay after we switch from peace to war mode to ensure we don't
    // get 'Too Busy' error messages
    Player.SetModeWait(charModeDelay);	

    Log.OutDM("Main", "Building decal control panel schemas...", ToChat);
    DefineSchemas();

    // Setup current location as primary attack location
    Log.OutDM("Main", "Setting primary attack location to current coords...", ToChat);
    for (var x=1; x<=navMaxLoc; x++) AttLoc[x] = new AttLocRec();	// Init all possible attlocs
    AttLoc[1].KillLoc = skapi.maplocCur;
    AttLocCur = 1;
    AttLocCnt = 1;
    HomeLoc = AttLoc[AttLocCur].KillLoc;
    LastArriveTime = new Date();

    // Now warn the user if stealth mode or peacemode set
    if (PeaceMode) Chat.Screen("WARNING!!! PeaceMode enabled, no creatures will be attacked!", cmcYellow);
    if (!Stealth)  Chat.Screen("WARNING!!! Stealth mode disabled!", cmcYellow);

    // While we want macro to run, process the FSM (finite state machine)
    RunFlag = true;
    StateChange(stateInit,NoCheck);

    // While we want macro to run, process the FSM (finite state machine)
    while (RunFlag == true) {
	if (debug||debugState) {
	    var OutLine = "Main: FSM top of loop, State [" + State + "/" + textState[State] + "]  ";
	    if (State == stateKill) OutLine+= "SubState [" + SubState + "/" + textSubKill[SubState] + "]";
	    if (State == stateLoot) OutLine+= "SubState [" + SubState + "/" + textSubLoot[SubState] + "]";
	    Log.OutD(OutLine);
	} // endif

	// If macro paused, then spin until resume flag set
	if (MacroPaused) {
	    PauseMacro();
	} // endif

	// Process all events
	var EventCount = skapi.WaitEvent(0);
	if (EventCount == 0) {
	    // if no events processed, then wait a little bit so FSM doesn't go crazy.
	    // We don't wait if an event was received since something important may have happened.
	    skapi.WaitEvent(10, wemFullTimeout);
	} // endif

	// If flag is set to use mana charges on paperdoll, then look for a mana stone in inventory and use it
	if (UseManaCharge) {
	    UseManaCharge = false;
	    var StoneAco = FindManaStone();
	    if (StoneAco != null) {
		// Use stone on paper doll.  Wait a few milliseconds before using it in case we are just 
		// finishing up a spell
		Chat.Screen("You have an item low on mana, using " + StoneAco.szName);
		if (logCharge || debugCharge) Log.OutD("Main: You have an item low on mana.  Using " + StoneAco.szName);
		skapi.WaitEvent(500, wemFullTimeout);
		StoneAco.UseOnAco(skapi.acoChar);
	    } else {
		// Couldn't find mana stone, log event
		Chat.Screen("Warning! You have an item low on mana but you are out of mana stones!", cmcYellow);
		if (logCharge || debugCharge) Log.OutD("Main: Item low on mana but you are out of mana stones!");
	    } // endif
	} // endif

	// If location recording is enabled, write our current location to the log file if it is has
	// signicantly changed from the previous recorded location.
	if (flagRecordLoc) RecordLoc();

	// After logging on, state is set to this
        if (State == stateInit) DoState_Init();

        // Person detected in vicinity, log for a while
	if (State == stateHide) {
	    Log.OutD("Calling DoState_Hide...");
	    DoState_Hide();
	    continue;
	} // endif

	// If we don't know one or more spells necessary for macro operation, then the Restricted flag is set.
	// Only manual decal panel functions can be done when in this mode.
	if (Restricted) continue;

	// See if time to move to next attack location.  Note that if the spawn advance flag is set and
	// we are not already moving to the next location, then goto next attack loc.
	if (State == stateGotoNext) {
	    DoState_GotoNext();
	    continue;
	} else {
	    if (SpawnAdvance) {
	    	SpawnAdvance = false;
	    	StateChange(stateGotoNext);
	    	
	    	// If previous state change failed, then we in a critical condition so continue with FSM
	    	if (State != stateGotoNext) continue;

		// Before advancing, see if a big heal is necessary.
		if (skapi.healthCur < retreatBigHeal) {
		    if (debugVitals||logVitals)
			Log.OutD("Main:	Player Health [" + skapi.healthCur +
				 "] below retreat big heal thresh [" + retreatBigHeal +	"]");
	
		    if (skapi.staminaCur > threshBigHealStam) {
			if (debugVitals||logVitals) Log.OutD("Main: Casting Stam2Health");
			Cast.Cast(SpellObj_StamHealth, null, true, true);
			if (CastCode ==	castCodeSuccess) {
			    if (debugVitals||logVitals)	Log.OutD("Main:	Stam2Health success, Arc [" + CastArc +	"]");
			} else {
			    // Spell failed, 
			    Log.OutD("Main: Stam2Health	failed.	" + SpellFailText());
			} // endif
		    } else {
			if (debugVitals||logVitals) Log.OutD("CheckVitals: Not enough stamina to cast Stam2Health");
		    } // endif
		} // end if

	    	DoState_GotoNext();
	    	continue;
	    } // endif
	} // endif

	// If retreating is enabled, we are not currently retreating, another routine has not already
	// flagged the RetreatReq flag, we have not just died, and the current location has a retreat
	// point defined then see if we should go there.
	if (flagRetreatCrit && !RetreatActive && !RetreatReq && !Dead && (AttLoc[AttLocCur].RetreatLoc!=null)) {
	    // First see if we part of movegroup and there are co-macroers near us.  If so then
	    // only retreat if flag 'flagMultRetreat' is set to true
	    if (!((MoveGroup > 0) && (PlayerNearCnt > 0) && !flagMultRetreat)) {
	    	// See if we should retreat
	        var NearCreCnt = Player.CreatureCount(retreatRadius);
	        // If the # of creatures near us equals or exceeds the retreat threshold, and we haven't
	        // tried to retreat recently then goto the retreat point.
	        if ((NearCreCnt>=RetreatAmount) && ((new Date()-RetreatLastAtt)>retreatMinInt)) {
	    	    // Retreat like a Frenchman.  We set a flag and let the next section of code
	    	    // to do the retreat since other routines can set the RetreatReq flag
	    	    RetreatReq = true;
	    	    Chat.Screen("Creature count threshold exceeded, retreating...");
	        } // endif
	    } // endif
	} // endif

	// If Retreat Required flag is set, then run like a Frenchman to the retreat location
	if (RetreatReq && !Dead) {
	    // If players health below ThreshBigHeal, then cast stam-2-health (if enough stamina) before retreating
	    if (skapi.healthCur < retreatBigHeal) {
		if (debugVitals||logVitals)
		    Log.OutD("Main: Player Health [" + skapi.healthCur +
		    	     "] below retreat big heal thresh [" + retreatBigHeal + "]");

		if (skapi.staminaCur > threshBigHealStam) {
		    if (debugVitals||logVitals) Log.OutD("Main: Casting Stam2Health");
		    Cast.Cast(SpellObj_StamHealth, null, true, true);
		    if (CastCode == castCodeSuccess) {
			if (debugVitals||logVitals) Log.OutD("Main: Stam2Health success, Arc [" + CastArc + "]");
		    } else {
			// Spell failed, 
			Log.OutD("Main: Stam2Health failed. " + SpellFailText());
		    } // endif
		} else {
		    if (debugVitals||logVitals) Log.OutD("CheckVitals: Not enough stamina to cast Stam2Health");
		} // endif
	    } // end if
	    
	    // Now retreat
	    FrenchMode();
	} // endif

	// See if user wants to go to the current attack location
	if (State == stateGotoCurrent) {
	    DoState_GotoCurrent();
	    continue;
	} // endif

	// If someone saw an admin, then we don't log for a few seconds to make sure he didn't materialize
	// near us also.
	if (FellowAbort) {
	    if ((new Date() - FellowWarnTime) > fellowAdminDelay) {
	    	Log.OutDM("Main", "Fellow abort holddown timer expired, logging...", true);
		StateChange(stateLog,NoCheck);
	    } // endif
	} // endif

	// Process all events
	var EventCount = skapi.WaitEvent(0);

	if (logPlig && (skapi.plig != LastPlig)) {
	    Log.OutD("Main: Plig Change: Old [" + LastPlig + "] New [" + skapi.plig + "]");
	    LastPlig = skapi.plig;
	} // endif

	if ((State != stateExit) && (skapi.plig == pligNotRunning)) {
    	    Log.OutD("Main: Game not running, stopping macro", true);
    	    StateChange(stateExit,NoCheck);
    	} // endif

	// See if we should evade any war spells
	if (EvadeWar && (State != stateAdmin)) {
	    if (debugEvade) Log.OutD("FSM: War Spell Detected,  Evading...");
	    WarEvade(false);
	    EvadeWar = false;
	} // endif

	// See if any creatures close to us.  This routine will set global variables Danger and/or
	// Critical if creatures detected close to us.
	CheckDanger();

	// Since maximum health/stam/mana can change due to buffs/debuffs, set all variables
	// related to vitals every loop
	UpdateVitalVars();

	// Update current skill levels for war/life/creature magics which change often due to buffs/debuffs.
	// Needed so we can determine maximum spell to cast in a particular school.
	UpdateMagicSkills();

        // If we still want to run macro, we are not in a paused state, and
        // we are not dealing with an admin, then check vitals
        if ((RunFlag == true) && !MacroPaused && (State != stateAdmin)) {
            // if vital check detected an issue, then continue with FSM to check will occur quickly
            VitalsOK = CheckVitals();
            if (VitalsOK) {
		if (debugVitals) Log.OutD("FSM: Vitals ok");
	    } else {
	    	if (debugVitals) Log.OutD("FSM: Vitals condition detected");
	    	if (SafeMode || RetreatReq) continue;
	    } // endif

	    // See if we should evade any war spells
	    if (EvadeWar) {
		if (debugEvade) Log.OutD("FSM: War Spell Detected,  Evading...");
		WarEvade(false);
		EvadeWar = false;
	    } // endif
	} else {
	    VitalsOK = true;
	} // end if

	// If healing or staming of fellow members enabled, then see if anyone needs help.
	// Only do this when in attack mode or we would interrupt a navigation, loot, or
	// salavage attempt
	// Chat.Screen("HelpCnt [" + HelpCnt + "] State [" + State + "/" + textState[State] + "]");
	if ((HelpCnt > 0) &&
	    (attHealFellow || attStamFellow) &&
	    ((State == stateIdleInit) || (State == stateIdle) || (State == stateSelect) ||
	     (State == stateTarget) || (State == stateKill)))
	    	CheckFellow();

	// See if any creatures close to us.  This routine will set global variables Danger and/or
	// Critical if creatures detected close to us.
	CheckDanger();

	// If buffing enable, a buff is expired (or we don't know the next buff to expire),
	// and no creatures are near (or flag is set to ignore danger when buffing), and we
	// are a state where a spell cast wouldn't interrupt anything, then attempt a buff cast
	if (BuffMode &&
	    ((Buff.NextExp == 0) || (Buff.NextExp < new Date())) &&
	    (!Danger || flagBuffDanger) &&
	    ((State == stateIdleInit) || (State == stateIdle) || (State == stateSelect) ||
	     (State == stateTarget) || (State == stateKill))) {
	    // A buff is expired
	    if (skapi.manaCur<threshBuffMana) {
		if (debugBuffs) Log.OutD("Main: Mana too low to buff.");
	    } else {
		if (debugBuffs) Log.OutD("Main: Casting up to 1 buff...");
		// Chat.Screen("Casting up to one buff...");
		Buff.UpdateExpire();    // Update expiration times of creature and life buffs from active spell list
		var BuffCast = Buff.Cast(logBuffs, spellAugGems);
		if (Buff.NextExp == 0) {
		    // Chat.Screen("Next buff exp is unknown");
		} else {
		    // Chat.Screen("Next buff exp is [" + new Date(Buff.NextExp) + "]");
		} // endif
		if (BuffCast) {
		    if (debugIdle||debugBuffs) Log.OutD("Main: Buff cast attempted.");
		} else {
		    // No buffs were cast, 
		    if (debugBuffs) Log.OutD("DoState_Idle: No buffs to cast.");
		} // endif
	    } // endif
	} // endif

	if (debug) Log.OutD("FSM switch: Processing State [" + State + "]" );
	switch (State) {

	    case stateKill:		// Try to kill
		if (debug) Log.OutD("Calling DoState_Kill...");
		DoState_Kill();
		break;

	    case stateSelect:		// Find a target
		if (debug) Log.OutD("Calling DoState_Select...");
		DoState_Select();
 		break;
 
	    case stateIdle:		// Main maintenance function decision tree here
		if (debug) Log.OutD("Calling DoState_Idle...");
		DoState_Idle();
		break;

	    case stateIdleInit:		// Not attacking or selecting.  Maintenance functions done here
		if (debug) Log.OutD("Calling DoState_IdleInit...");
		DoState_IdleInit();
		break;

	    case stateCheckComps:	// Check components
		if (debug) Log.OutD("Calling DoState_CheckComps...");
		DoState_CheckComps();
		break;

	    case stateLoot:		// Pick up anything good
		if (debug) Log.OutD("Calling DoState_Loot...");
		DoState_Loot();
		break;

	    case stateSalvage:		// Use UST on salvage
		if (debug) Log.OutD("Calling DoState_Salvage...");
		DoState_Salvage();
		break;

	    case stateScroll:		// Learn any unknown scrolls
		if (debug) Log.OutD("Calling DoState_Scroll...");
		DoState_Scroll();
		break;

	    case statePaused:		// Pause macro by logging out.  Macro will resume when player
	    				// logs back in manually
		if (debug) Log.OutD("Calling DoState_Paused...");
		DoState_Paused();
		break;

	    case stateStormed:		// We have been portal stormed
		Log.OutD("Calling DoState_PortalStormed...");
		DoState_PortalStormed();
		break;

	    case stateDied:		// We died, exit game and macro
		Log.OutD("Calling DoState_Died...");
		DoState_Died();
		break;

	    case stateLog:		// Log off AC
		Log.OutD("Calling DoState_Log...");
		DoState_Log();
		break;

	    case stateExit:		// Stop macro
		if (alertAbort) {
		    for (var x=0; x<alertAbortCount; x++ ) {
			alertAbortWav.Play();
			skapi.Sleep(alertAbortInt);
		    } // end for
		} // endif
		RunFlag = 0;
		break;

	    case stateAdmin:		// Admin detected so do special admin procesing
		if (debug) Log.OutD("Calling DoState_Admin...");
		DoState_Admin();
		break;

	} // End switch
    } // end while

    // Any final logging/cleanup goes here
    Log.OutD("Macro shutting down...", true);

    fRemoveHandlers();
    RemoveInterface();
} // end main