// WAR_SUBS.JS
//
// Author:	Poundboy
// Version:	2.08
//
//
// Modification History:
//
// Who	  Date	    Version	Description
//
// PB	  11/01/02  1.01	Initial version.
//
//
// Description:
//
//	This module contains all the subroutines for the WarBot macro
//

function StateChange(NewState,NoCheck) {
    if ((NoCheck == undefined) || (NoCheck == null)) NoCheck = false;

    if (State == stateAdmin) {
	Log.OutD("StateChange: Current state locked (admin detected), desired state [" +
		 NewState + "/" + textState[NewState] + "]" );
	return false;
    } // endif

    if (!NoCheck) {
	if ((State == stateHide) || (State == stateDied) || (State == stateInit) || (State == stateLog) ||
	    (State == stateExit) || (State == stateGotoNext) || (State == stateGotoCurrent)) {
		Log.OutD("StateChange: Current State Locked, Current [" + State + "/" + textState[State] +
			 "] Desired state [" + NewState + "/" + textState[NewState] + "]" );
		return false;
	} // endif
    } // endif

    if (debug||debugState)
	Log.OutD("StateChange: Switching state from [" + State + "/" + textState[State] +
		 "] to [" + NewState + "/" + textState[NewState] + "] NoCheck [" + NoCheck + "]");

    State = NewState;
    return true;
} // end func


function CheckGameState() {
    // Determine game state
    Log.OutD("Main: Determining game state...");
    skapi.WaitEvent(0);
    if (skapi.plig == pligNotRunning) {
    	Log.OutDM("Main", "Game not running, please start AC before starting this macro", ToChat);
    	return false;
    } // endif

    skapi.WaitEvent(0);
    if (skapi.plig == pligAtLogin) {
    	Log.OutDM("Main", "Game at Login screen, logging in default char", ToChat);
    	Player.LogOn(accDefChar);
    } // endif

    skapi.WaitEvent(0);
    if (skapi.plig == pligInPortal) {
    	Log.OutDM("Main", "Macro started while char in portal space, waiting...", ToChat);
    	var wait_start = new Date();
   	while ((skapi.plig != pligInWorld) && ((new Date()-wait_start) < Player.LogonTimeout))
   		skapi.WaitEvent(10);
   	if (skapi.plig != pligInWorld) {
   	    Log.OutDM("Main", "Timeout waiting to come out of portal space, aborting macro", ToChat);
   	    return false;
   	} // endif
    } // endif

    skapi.WaitEvent(0);
    if (skapi.plig == pligInWorld) {
    	// Get current slot # of logged in character for use in auto login functions.
    	// If the ForceSlot variable is set, then use that for the slot number instead of
    	// determining it automatically (needed occasionally because of a skunkworks issue).
    	if (ForceSlot>=0) {
    	    CharSlot = ForceSlot;
    	} else {
	    CharSlot = Player.ToSlotNum(skapi.szName);
	} // endif
    	Log.OutDM("Main", "Character " + skapi.szName + " (Slot " + CharSlot + ") in world");
    } else {
    	Log.OutDM("Main", "Character not in world, aborting macro...", ToChat);
    	return false;
    } //endif

    // If we make it here, character is in the world (in-game)
    return true;
} // end func


function RareMessage(text) {
    if (rareDetectExp.test(text)) {
    	if (logRares) Log.OutD("RareMessage: Rare detected in text [" + text + "]");
    	alertRareWav.Play();
	return true;
    } // endif

    return false;  // Rare message not found in text
} // end func


function PossiblyResetToon() {
    // If players cursor (pointer) is busy, then press the escape key just in case we are
    // stuck doing an action.  During the idle state nothing should be going on.

    var WaitStart = new Date();
    while (skapi.fPointerBusy && ((new Date() - WaitStart) < busyWait)) skapi.WaitEvent(20, wemFullTimeout);

    // If pointer not busy, then return false (no action taken)
    if (!skapi.fPointerBusy) return false;

    // Pointer still busy, try to 'unstick' character   
    Log.OutDM("PossiblyResetToon", "Pointer busy, pressing escape key twice in case we are stuck...");
    skapi.Keys("{ESC}", kmoClick);
    skapi.WaitEvent(300, wemFullTimeout);
    skapi.Keys("{ESC}", kmoClick);
    skapi.WaitEvent(300, wemFullTimeout);
    Log.OutDM("PossiblyResetToon", "Unstuck attempt ended.");
    return true;
} // end func   


function FindScrollOnLootList(ScrollName) {
    var ScrollIndex = 0;
    while (ScrollIndex < Loot.TypeInfo[lootTypeScroll].ItemCount) {
	var ScrollAco = Loot.TypeInfo[lootTypeScroll].ItemList[ScrollIndex].Aco;
	if ((ScrollAco == undefined) || (ScrollAco == null) || !ScrollAco.fExists || !ScrollAco.fInInventory) continue;
	if (ScrollAco.szName == ScrollName) break;
	ScrollIndex++;
    } // end while

    // If the scroll index is less then the scroll count, then we found the scroll
    if (ScrollIndex < Loot.TypeInfo[lootTypeScroll].ItemCount) {
	return true;
    } // endif

    return false;
} // end func


// Move the passed Aco to the top of the desired pack
function MoveItem(Aco, PackNum, Check) {
    if ((Aco == null) || (!(Aco.fExists))) return false;  // If item doesn't exist anymore then abort move
    if (!Aco.fInInventory) return false;		  // If item not in our pack anymore then abort move
    if ((PackNum == undefined) || (PackNum == null)) PackNum = 0;
    if ((Check==undefined) || (Check==null)) Check = false;

    // If check enabled and item not in main pack then abort move
    if (Check && (Aco.acoContainer != skapi.acoChar)) return true;

    // Clear flag that when set indicates move done
    LootMoveDone = false;

    for (var x=0; x<lootMoveAttempts; x++) {
	// Set timer so we know when to give up
	LootMoveStart = new Date();
	LootSortMode = true;

	// Move item to top of main pack
	Aco.MoveToPack(PackNum, 0, false);

	// Wait until we receive the AddToEventory event or the timeout interval has been reached
	while (!LootMoveDone && !AbortAction && ((new Date()-LootMoveStart )<lootMoveTimeOut))
	    skapi.WaitEvent(20);

	LootSortMode = false;
	if (AbortAction) return false;

	// If loot moved successfully then return true
	if (LootMoveDone) return true;
    } // endfor

    // If we make it here, then move failed
    return false;
} // end function


// Give the source aco to the target aco
function GiveItem(SrcAco, TargetAco, Check) {
    if ((SrcAco == null) || (!(SrcAco.fExists))) return false;	 // If item doesn't exist anymore then abort give
    if (!SrcAco.fInInventory) return false;			 // If item not in our pack anymore then abort give
    if ((Check == undefined) || (Check == null)) Check = true;
    if (Check && (SrcAco.acoContainer != skapi.acoChar)) return true; // If item not in main pack then abort give

    // Clear flag that is set in OnRemoveFromInventory event when move done
    InventoryRemove = false;

    for (var x=0; x<lootGiveAttempts; x++) {
	// Set timer so we know when to give up
	LootGiveStart = new Date();

	// Give item to target
	SrcAco.MoveToStack(TargetAco);

	// Wait until we receive the AddToInventory event, the abort action flag is set,  or the
	// timeout interval has been reached
	while (!InventoryRemove && !AbortAction && ((new Date()-LootGiveStart )<lootGiveTimeOut))
	    skapi.WaitEvent(25);

   	// If abort button pressed, exit routine
    	if (AbortAction) return false;

	// If loot moved successfully then return true
	if (InventoryRemove) return true;
    } // endfor

    Log.OutD("GiveItem: Failed to give item " + SrcAco.szName + " to " + TargetAco.szName, true);
    return false;
} // end function


function GiveLoot(lootType) {
    // If no loot of this type, then abort
    if (Loot.TypeInfo[lootType].ItemCount == 0) {
	Chat.Screen("No loot found");
	return;
    } // endif

    // Now see if someone is selected
    var SelAco = skapi.acoSelected;
    if (SelAco == null) {
	Chat.Screen("No player selected to give loot");
	return;
    } // endif

    if (!(SelAco.ocm & ocmPlayer)) {
	Chat.Screen("Current selected object is not a player");
	return;
    } // endif

    if (SelAco.oid == skapi.acoChar.oid) {
	Chat.Screen("Can't give to ourself!");
	return;
    } // endif

    var PlayerDist = Util.Distance(skapi.maplocCur, SelAco.maploc);
    if (PlayerDist > giveMaxDist) {
	Chat.Screen("Current selected object is too far away, come within " + giveMaxDist + " map units");
	return;
    } // endif

    // Copy loot list to temporary array.  This isolates us from problems if someone manually
    // moves an item after the routine started.  This also permits the RemoveInventory event
    // to remove the item from the original list without fear of screwing us up.
    AbortAction = false;  // Clear abort flag
    skapi.WaitEvent(0);  // Clear out events
    var GiveColl = skapi.CoacoNew();
    for (var x=0; x<Loot.TypeInfo[lootType].ItemCount; x++) GiveColl.AddAco(Loot.TypeInfo[lootType].ItemList[x].Aco);

    // Now give all loot type to the currently selected target
    for (var x=0; x<GiveColl.Count; x++) {
	var Status = GiveItem(GiveColl.Item(x), SelAco);

	// See if abort button pressed
	if (AbortAction) return false;

	// Now wait a few milliseconds to make sure give attempts don't collide.
	skapi.WaitEvent(wemFullTimeout, lootGivePause);
    } // endfor

    return true;
} // endif

function SortbyMaterialWork() {
    this.CompareAcos = Compare_Acos;
} // end func

function Compare_Acos(aco1, aco2) {
    // First compare ACOs by sort priority
    var Index1 = Loot.SalvageIndex[aco1.material];
    var Index2 = Loot.SalvageIndex[aco2.material];
    if (Loot.SalvageFilter[Index1].SortPrio > Loot.SalvageFilter[Index2].SortPrio) return -1;
    if (Loot.SalvageFilter[Index1].SortPrio < Loot.SalvageFilter[Index2].SortPrio) return 1;

    // Same priority, now try material type
    if (aco1.material < aco2.material) return -1;
    if (aco1.material > aco2.material) return 1;

    // Sort priority and material the same, so last check is workmanship
    if (aco1.workmanship < aco2.workmanship) return -1;
    if (aco1.workmanship > aco2.workmanship) return 1;

    return 0;	/// Everything the same between the two ACOs. Return equal code.
} // end func

function SortLoot(sortall) {
    if (sortall == undefined) sortall = true;
    var Status = true;
    AbortAction = false;

    if (sortall && (Loot.TypeInfo[lootTypeJunk].ItemCount > 0)) {
	Chat.Screen("Moving junk to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeJunk].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeJunk].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeJunk].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (sortall && (Loot.TypeInfo[lootTypeValSalv].ItemCount > 0)) {
	Chat.Screen("Moving valuable salvage to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeValSalv].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeValSalv].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status)
		Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeValSalv].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (sortall && (Loot.TypeInfo[lootTypeUstSalv].ItemCount > 0)) {
	Chat.Screen("Moving ustable salvage to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeUstSalv].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeUstSalv].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status)
		Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeUstSalv].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (sortall && (Loot.TypeInfo[lootTypeSalvage].ItemCount > 0)) {
	Chat.Screen("Moving tinker salvage to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeSalvage].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeSalvage].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeSalvage].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (sortall && (Loot.TypeInfo[lootTypeValuable].ItemCount > 0)) {
	Chat.Screen("Moving valuable objects to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeValuable].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeValuable].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeValuable].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeDeathItem].ItemCount > 0) {
	Chat.Screen("Moving death items to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeDeathItem].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeDeathItem].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status)
		Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeDeathItem].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeItem].ItemCount > 0) {
	Chat.Screen("Moving items to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeItem].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeItem].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeItem].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

   if (Loot.TypeInfo[lootTypeScroll].ItemCount > 0) {
	Chat.Screen("Moving scrolls to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeScroll].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeScroll].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeItem].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeJewelry].ItemCount > 0) {
	Chat.Screen("Moving Jewelry to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeJewelry].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeJewelry].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeJewelry].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeClothing].ItemCount > 0) {
	Chat.Screen("Moving Clothing to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeClothing].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeClothing].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeClothing].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeWand].ItemCount > 0) {
	Chat.Screen("Moving Wands to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeWand].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeWand].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeWand].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeWeaponR].ItemCount > 0) {
	Chat.Screen("Moving ranged weapons to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeWeaponR].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeWeaponR].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status)
		Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeWeaponR].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeWeaponM].ItemCount > 0) {
	Chat.Screen("Moving melee weapons to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeWeaponM].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeWeaponM].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status)
		Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeWeaponM].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeArmor].ItemCount > 0) {
	Chat.Screen("Moving Armor to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeArmor].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeArmor].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeArmor].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeMinor].ItemCount > 0) {
	Chat.Screen("Moving Minors to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeMinor].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeMinor].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeMinor].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeMajor].ItemCount > 0) {
	Chat.Screen("Moving Majors to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeMajor].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeMajor].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeMajor].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    if (Loot.TypeInfo[lootTypeRare].ItemCount > 0) {
	Chat.Screen("Moving Rares to top of pack...");
	for (var x=0; x<Loot.TypeInfo[lootTypeRare].ItemCount; x++) {
	    Status = MoveItem(Loot.TypeInfo[lootTypeRare].ItemList[x].Aco, 0, true);
	    if (AbortAction) return false;
	    if (!Status) Chat.Screen("Unable to move item [" + Loot.TypeInfo[lootTypeRare].ItemList[x].Aco.szName + "]");
	} // endfor
    } // endif

    return true;
} // function


function SplitAco(Aco, Count) {
    skapi.SelectAco(Aco);
    skapi.WaitEvent(10,wemFullTimeout);
    skapi.SetStackCount(Count);

    // Now split the stack, returning the ACO of the newly split item
    MoveActive = true;
    InventoryAdd = false;
    var SplitStart = new Date();
    Aco.MoveToPack(0,0,false);

    // Wait for add inventory event so we can get the aco of the newly split item
    while (!InventoryAdd && ((new Date()-SplitStart)<splitFailInt)) skapi.WaitEvent(25);
    MoveActive = false;
    
    if (!InventoryAdd) return null;
    return AddInvAco;
} // endif


function SellLoot() {
    var SellValue  = 0;
    var FreeSlots  = skapi.acoChar.citemMax - skapi.acoChar.citemContents;
    var MaxPyreals = (FreeSlots-1)*10000;
    NoSpace = false;
    SellFound = false;
    AbortAction = false;

    for (var x=0; x<Loot.TypeInfo[lootTypeValuable].ItemCount; x++) {
    	skapi.WaitEvent(0);
	if (AbortAction) {
	    if (debugLoot) Log.OutD("SellLoot: Abort button pressed");
	    Chat.Screen("Aborting loot sell...");
	    AbortAction = false;
	    return false;
	} // endif

	var SellAco = Loot.TypeInfo[lootTypeValuable].ItemList[x].Aco;
	if (debugLoot) Log.OutD("SellLoot: Processing item [" + SellAco.szName + "]");

	// If item not in main container, then don't sell it
	if (SellAco.acoContainer != skapi.acoChar) {
	    if (debugLoot) Log.OutD("SellLoot: Item not in main pack, skipping...");
	    continue;
	} // endif

	if (!(SellAco.mcm & VendorMcmBuy)) {
	    if (debugLoot) Log.OutD("SellLoot: Vendor does not buy this item");
	    continue;
	} // endif
	
	if (SellAco.cpyValue > VendorMaxBuy) {
	    Chat.Screen("Item " + SellAco.szName + ", value " + SellAco.cpyValue +
	    		" pyreals is too expensive to sell at this vendor!");
	    continue;
	} // endif

	if ((SellValue+(SellAco.cpyValue*VendorFractBuy)) >= MaxPyreals) {
	    if (debugLoot) Log.OutD("SellLoot: Not enough pack space for pyreals");
	    NoSpace = true;
	    continue;
	} // endif

	// Add item to trade window
	SellFound = true;
	SellAco.MoveToStack(VendorAco);
	SellValue += SellAco.cpyValue*VendorFractBuy;
	MaxPyreals += 10000;
    } // end for
    
    // Sort valuable bag list by value
    ValBagColl.Sort(skapi.sortbyValue);
    
    for (var x=0; x<ValBagColl.Count; x++) {
    	skapi.WaitEvent(0);
	if (AbortAction) {
	    if (debugLoot) Log.OutD("SellLoot: Abort button pressed");
	    Chat.Screen("Aborting loot sell...");
	    AbortAction = false;
	    return false;
	} // endif

    	var SellAco = ValBagColl.Item(x);
	if (debugLoot) Log.OutD("SellLoot: Processing item [" + SellAco.szName + "]");

	if (!(SellAco.mcm & VendorMcmBuy)) {
	    if (debugInvent) Log.OutD("SellLoot: Vendor does not buy this item");
	    continue;
	} // endif

	if (SellAco.cpyValue > VendorMaxBuy) {
	    Chat.Screen("Item " + SellAco.szName + ", value " + SellAco.cpyValue +
	    		" pyreals is too expensive to sell at this vendor!");
	    continue;
	} // endif

	if ((SellValue+(SellAco.cpyValue*VendorFractBuy)) > MaxPyreals) {
	    if (debugLoot) Log.OutD("SellLoot: Not enough pack space for pyreals");
	    NoSpace = true;
	    continue;
	} // endif

	// If item in main container, then increase max pyreals by 10000 since a slot
	// will be freed up when this item is sold.
	if (SellAco.acoContainer == skapi.acoChar) {
	    if (debugInvent) Log.OutD("SellLoot: Item in main pack, increasing maxpyreals counter...");
	    MaxPyreals += 10000;
	} // endif

	// Add item to trade window
	SellFound = true;
	SellAco.MoveToStack(VendorAco);
	SellValue += SellAco.cpyValue*VendorFractBuy;
    } // end for

    // Sort pea list by value
    PeaListColl.Sort(skapi.sortbyValue);
    for (var x=0; x<PeaListColl.Count; x++) {
    	skapi.WaitEvent(0);
	if (AbortAction) {
	    if (debugLoot) Log.OutD("SellLoot: Abort button pressed");
	    Chat.Screen("Aborting loot sell...");
	    AbortAction = false;
	    return false;
	} // endif

    	var SellAco = PeaListColl.Item(x);
	if (debugLoot) Log.OutD("SellLoot: Processing item [" + SellAco.szName + "]");

	if (!(SellAco.mcm & VendorMcmBuy)) {
	    if (debugLoot) Log.OutD("SellLoot: Vendor does not buy peas");
	    break;
	} // endif

	// If pea not in main pack, then skip
	if (SellAco.acoContainer != skapi.acoChar) {
	    if (debugLoot) Log.OutD("SellLoot: Pea not in main pack");
	    continue;
	} // endif

	if (debugLoot) {
	    Log.OutD("SellLoot: MaxPyreals [" + MaxPyreals + "] SellValue [" + SellValue + "]");
	    Log.OutD("SellLoot: SellAco.cpyValue [" + SellAco.cpyValue + "] VendorFractBuy [" + VendorFractBuy + "]");
	} // endif
	
	if ((SellValue+(SellAco.cpyValue*VendorFractBuy)) > MaxPyreals) {
	    NoSpace = true;
	    if (debugLoot) Log.OutD("SellLoot: Not enough pack space for pyreals, trying to split stack.");
	    var StackCount = SellAco.citemStack;
	    if (StackCount == 1) {
		if (debugInvent) Log.OutD("SellLoot: Stack only has 1 item, too big to sell.");
		continue;
	    } // endif
	    var ValPerItem = (SellAco.cpyValue*VendorFractBuy) / StackCount;
	    var ValLeft = MaxPyreals - SellValue;
	    var SellCount = Math.floor(ValLeft/ValPerItem);
	    if (debugLoot) {
		Log.OutD("SellLoot: SellCount [" + SellCount + "] ValLeft [" + ValLeft + "]");
	    	Log.OutD("SellLoot: ValPerItem [" + ValPerItem + "] StackCount [" + StackCount + "]");
	    } // endif
	    if (SellCount == 0) {
		if (debugLoot) Log.OutD("SellLoot: Individual items too big to sell.");
		continue;
	    } // endif

	    if (debugLoot) Log.OutD("SellLoot: Adding " + SellCount + " items of stack to trade window...");

	    var NewAco = SplitAco(SellAco, SellCount);
	    if (NewAco != null) {
		SellFound = true;
		NewAco.MoveToStack(VendorAco);
		SellValue += (ValPerItem * SellCount);
		if (debugLoot) Log.OutD("SellLoot: Item added.");
	    } else {
		if (debugLoot) Log.OutD("SellLoot: Unable to split item.");
	    } // endif
	    continue;
	} // endif

	// If item in main container, then increase max pyreals by 10000 since a slot
	// will be freed up when this item is sold.
	if (SellAco.acoContainer == skapi.acoChar) {
	    if (debugLoot) Log.OutD("SellLoot: Item in main pack, increasing maxpyreals counter...");
	    MaxPyreals += 10000;
	} // endif

	// Add item to vendor window
	if (debugLoot) Log.OutD("SellLoot: Adding whole item to vendor window");
	SellFound = true;
	SellAco.MoveToStack(VendorAco);
	SellValue += SellAco.cpyValue*VendorFractBuy;
    } // end for
    return true;
} // end func


function ShowLoot() {
    BuildValSalList();
    
    // Get count and value of peas
    var PeaAcf = skapi.AcfNew();
    PeaAcf.mcm = mcmSpellComponents;
    PeaAcf.olc = olcInventory;
    PeaAcf.szName = "*Pea";
    var PeaColl = PeaAcf.CoacoGet(skapi.acoChar.coacoContents);  
    var PeaTotal = 0;
    var PeaCount = 0;
    for (var x=0; x<PeaColl.Count; x++) {
    	var PeaAco   = PeaColl.Item(x);
	PeaTotal += PeaAco.cpyValue;
	PeaCount += PeaAco.citemStack;
    } // endfor

    var Wid1 = 1.0;
    var Wid2 = 0.6;
    Chat.Screen(Util.Width("Junk",        Wid1) + ": " +
    		Util.Width(Loot.TypeInfo[lootTypeJunk].ItemCount+" ", Wid2) +
    		Util.Width("Salvage",     Wid1) + ": " + Loot.TypeInfo[lootTypeSalvage].ItemCount);
    Chat.Screen(Util.Width("UstSalv",     Wid1) + ": " +
    		Util.Width(Loot.TypeInfo[lootTypeUstSalv].ItemCount + " ", Wid2) +
    		Util.Width("ValSalv",     Wid1) + ": " + Loot.TypeInfo[lootTypeValSalv].ItemCount);
    Chat.Screen(Util.Width("Clothing",    Wid1) + ": " +
		Util.Width(Loot.TypeInfo[lootTypeClothing].ItemCount+" ", Wid2) +
    		Util.Width("Death Items", Wid1) + ": " + Loot.TypeInfo[lootTypeDeathItem].ItemCount);	
    Chat.Screen(Util.Width("Wands",       Wid1) + ": " +
    		Util.Width(Loot.TypeInfo[lootTypeWand].ItemCount+" ", Wid2) +
    		Util.Width("Jewelry",     Wid1) + ": " + Loot.TypeInfo[lootTypeJewelry].ItemCount);
    Chat.Screen(Util.Width("Armor",       Wid1) + ": " +
    		Util.Width(Loot.TypeInfo[lootTypeArmor].ItemCount+" ", Wid2) +
   		Util.Width("Scrolls",     Wid1) + ": " + Loot.TypeInfo[lootTypeScroll].ItemCount);
    Chat.Screen(Util.Width("MWeapons",    Wid1) + ": " +
		Util.Width(Loot.TypeInfo[lootTypeWeaponM].ItemCount+" ", Wid2) +
    		Util.Width("RWeapons",    Wid1) + ": " + Loot.TypeInfo[lootTypeWeaponR].ItemCount);
    Chat.Screen(Util.Width("Minors",      Wid1) + ": " +
    		Util.Width(Loot.TypeInfo[lootTypeMinor].ItemCount+" ", Wid2) +
    		Util.Width("Items",       Wid1) + ": " + Loot.TypeInfo[lootTypeItem].ItemCount);
    Chat.Screen(Util.Width("Majors",      Wid1) + ": " +
    		Util.Width(Loot.TypeInfo[lootTypeMajor].ItemCount+" ", Wid2) +
    		Util.Width("Rares",       Wid1) + ": " + Loot.TypeInfo[lootTypeRare].ItemCount);
    Chat.Screen(Loot.TypeInfo[lootTypeValuable].ItemCount + " valuable items worth " +
    		Loot.TypeInfo[lootTypeValuable].TotalValue + ".");
    Chat.Screen(ValBagColl.Count + " bags of valuable salvage worth " + ValBagTotal + ".");
    Chat.Screen(PeaCount + " peas worth " + PeaTotal + ".");
    Chat.Screen(CorpseCheckCnt + " corpses checked.");
    return;
} // end func


function ShowDetail() {
    var FoundSome = false;

    if (Loot.TypeInfo[lootTypeItem].ItemCount > 0) {
    	FoundSome = true;
        Chat.Screen("Item count: " + Loot.TypeInfo[lootTypeItem].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeItem].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeItem].ItemList[x].Aco.szName);
    } // endif

    if (Loot.TypeInfo[lootTypeDeathItem].ItemCount > 0) {
        FoundSome = true;
        Chat.Screen("Death Item count: " + Loot.TypeInfo[lootTypeDeathItem].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeDeathItem].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeDeathItem].ItemList[x].Aco.szName + " " +
        		Loot.TypeInfo[lootTypeDeathItem].ItemList[x].Desc);
    } // endif

   if (Loot.TypeInfo[lootTypeScroll].ItemCount > 0) {
    	FoundSome = true;
        Chat.Screen("Scroll count: " + Loot.TypeInfo[lootTypeScroll].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeScroll].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeScroll].ItemList[x].Aco.szName);
    } // endif

    if (Loot.TypeInfo[lootTypeClothing].ItemCount > 0) {
        FoundSome = true;
        Chat.Screen("Clothing count: " + Loot.TypeInfo[lootTypeClothing].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeClothing].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeClothing].ItemList[x].Aco.szName + " " +
        		Loot.TypeInfo[lootTypeClothing].ItemList[x].Desc);
    } // endif

    if (Loot.TypeInfo[lootTypeWeaponR].ItemCount > 0) {
    	FoundSome = true;
        Chat.Screen("Ranged weapon count: " + Loot.TypeInfo[lootTypeWeaponR].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeWeaponR].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeWeaponR].ItemList[x].Aco.szName + " " +
        		Loot.TypeInfo[lootTypeWeaponR].ItemList[x].Desc);
    } // endif

    if (Loot.TypeInfo[lootTypeWeaponM].ItemCount > 0) {
    	FoundSome = true;
        Chat.Screen("Melee weapon count: " + Loot.TypeInfo[lootTypeWeaponM].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeWeaponM].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeWeaponM].ItemList[x].Aco.szName + " " +
        		Loot.TypeInfo[lootTypeWeaponM].ItemList[x].Desc);
    } // endif

    if (Loot.TypeInfo[lootTypeWand].ItemCount > 0) {
    	FoundSome = true;
        Chat.Screen("Wand count: " + Loot.TypeInfo[lootTypeWand].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeWand].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeWand].ItemList[x].Aco.szName + " " +
        		Loot.TypeInfo[lootTypeWand].ItemList[x].Desc);
    } // endif

    if (Loot.TypeInfo[lootTypeJewelry].ItemCount > 0) {
    	FoundSome = true;
        Chat.Screen("Jewelry count: " + Loot.TypeInfo[lootTypeJewelry].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeJewelry].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeJewelry].ItemList[x].Aco.szName + " " +
        		Loot.TypeInfo[lootTypeJewelry].ItemList[x].Desc);
    } // endif

    if (Loot.TypeInfo[lootTypeArmor].ItemCount > 0) {
    	FoundSome = true;
        Chat.Screen("Armor count: " + Loot.TypeInfo[lootTypeArmor].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeArmor].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeArmor].ItemList[x].Aco.szName + " " +
        		Loot.TypeInfo[lootTypeArmor].ItemList[x].Desc);
    } // endif

    if (Loot.TypeInfo[lootTypeMinor].ItemCount > 0) {
    	FoundSome = true;
        Chat.Screen("Minor count: " + Loot.TypeInfo[lootTypeMinor].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeMinor].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeMinor].ItemList[x].Aco.szName + " " +
        		Loot.TypeInfo[lootTypeMinor].ItemList[x].Desc);
    } // endif

    if (Loot.TypeInfo[lootTypeMajor].ItemCount > 0) {
    	FoundSome = true;
        Chat.Screen("Major count: " + Loot.TypeInfo[lootTypeMajor].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeMajor].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeMajor].ItemList[x].Aco.szName + " " +
        		Loot.TypeInfo[lootTypeMajor].ItemList[x].Desc);
    } // endif
    
    if (Loot.TypeInfo[lootTypeRare].ItemCount > 0) {
    	FoundSome = true;
        Chat.Screen("Rare count: " + Loot.TypeInfo[lootTypeRare].ItemCount);
        for (var x=0; x<Loot.TypeInfo[lootTypeRare].ItemCount; x++)
            Chat.Screen("  " + Loot.TypeInfo[lootTypeRare].ItemList[x].Aco.szName + " " +
        		Loot.TypeInfo[lootTypeRare].ItemList[x].Desc);
    } // endif


    if (!FoundSome) {
    	Chat.Screen("No good loot.");
    } // endif
} // end func


function QuickSort(loBound, hiBound) {
    var pivot, loSwap, hiSwap, temp;

    // Two items to sort
    if (hiBound - loBound == 1) {
	if (SortList[loBound].Value > SortList[hiBound].Value) {
	    temp = SortList[loBound];
	    SortList[loBound] = SortList[hiBound];
	    SortList[hiBound] = temp;
	} // endif
	return;
    } // endif

    // Three or more items to sort
    pivot = SortList[parseInt((loBound + hiBound) / 2)];
    SortList[parseInt((loBound + hiBound) / 2)] = SortList[loBound];
    SortList[loBound] = pivot;
    loSwap = loBound + 1;
    hiSwap = hiBound;

    do {
	// Find the right loSwap
	while (loSwap <= hiSwap && SortList[loSwap].Value <= pivot.Value) loSwap++;

	// Find the right hiSwap
	while (SortList[hiSwap].Value > pivot.Value) hiSwap--;

	// Swap values if loSwap is less than hiSwap
	if (loSwap < hiSwap) {
	    temp = SortList[loSwap];
	    SortList[loSwap] = SortList[hiSwap];
	    SortList[hiSwap] = temp;
	} // endif
    } while (loSwap < hiSwap);

    SortList[loBound] = SortList[hiSwap];
    SortList[hiSwap] = pivot;

    // Recursively call function...
    // 2 or more items in first section		
    if (loBound < hiSwap - 1) QuickSort(loBound, hiSwap - 1);

    // 2 or more items in second section
    if (hiSwap + 1 < hiBound) QuickSort(hiSwap + 1, hiBound);
} // end func


function ShowValSalv() {
    // Display total units of valuable salvage for each material.
    // Enables you to easily see what you are picking up
    
    // Collect all salvage bags and sort by material type
    BuildValSalList();
    
    if (ValBagColl.Count == 0) {
    	Chat.Screen("No valuable salvage bags in inventory.");
    	return;
    } // endif
    // ValBagTotal
    
    var Material = -1;
    var Units = 0;
    var Value = 0;
    
    SortList = new Array();
    SortCnt  = 0;
    
    // Now sort bags of salvage by sort priority by material by workmanship.  I really only need
    // it by material, but this filter already existed and I was too lazy to make another one.
    ValBagColl.Sort(new SortbyMaterialWork);
    for (var x=ValBagColl.Count-1; x>=0; x--) {
    	var BagAco = ValBagColl.Item(x);
    	if (BagAco.material != Material) {
	    if (Material >= 0) {
	    	// Dump out previous material
	    	var Index = Loot.SalvageIndex[Material];
	    	SortList[SortCnt] = new SortRec(Index, Units, Value);
	    	SortCnt++;
	    	Units = 0;
		Value = 0;
	    } else {
	    	// Print heading
	    	
	    } // endif
	    Material = BagAco.material;
	} // endif
	Units += BagAco.cuseLeft;
	Value += BagAco.cpyValue;
    } // endif
    
    // Print out last entry
    var Index = Loot.SalvageIndex[Material];
    SortList[SortCnt] = new SortRec(Index, Units, Value);
    SortCnt++;

    // Sort records 
    QuickSort(0,SortCnt-1);

    // Print out array in reverse order
    Chat.Screen(Util.Width("Value",1.0) + Util.Width("Units",1.0) + Util.Width("Ratio",1.0) + "Material");
    for (var x=SortCnt-1; x>=0; x--) {
    	var Ratio = SortList[x].Value / SortList[x].Units;
	Chat.Screen(Util.Width(SortList[x].Value+" ",1.0) +
		    Util.Width(SortList[x].Units+" ",1.0) +
		    Util.Width(Util.Round(Ratio,0) +" ",1.0) +
		    Loot.SalvageFilter[SortList[x].Index].Name);
    } // endfor
    
    Chat.Screen(ValBagColl.Count + " bags of valuable salvage worth " + ValBagTotal + ".");
    return;
} // end func


function AssessLoot() {
    if (debugLoot) Log.OutD("AssessLoot: Begin");

    // First clear out loot counters
    Loot.ResetLoot();

    AbortAction = false;
    Chat.Screen("Gathering valid items from main pack...");

    // Load all valid items in main pack into LootList array.  Invalid
    // items include those with inscriptions and items protected by user
    LootCount = 0;
    var AssessNeeded = 0;
    for (var x=0; x<skapi.acoChar.citemContents; x++) {
	LootList[LootCount] = new ItemObj(skapi.AcoFromIPackIitem(0,x));
	if (debugLoot)
	    Log.OutD("AssessLoot: Pack Index [" + x + "] oid [" + Util.Hex(LootList[LootCount].Aco.oid) +
	    	     "] Item [" + LootList[LootCount].Aco.szName + "]");

	// See if item is protected by user.  If so then skip it
	if (NoLootItem(LootList[LootCount].Aco)) {
	    if (debugLoot) Log.OutD("AssessLoot: Skipping protected item [" + LootList[LootCount].Aco.szName + "]...");
	    continue;
	} // endif

	// See if item is a salvage bag.  If so then skip it
	if (LootList[LootCount].Aco.mcm & mcmSalvageBag) {
	    if (debugLoot) Log.OutD("AssessLoot: Skipping salvage bag [" + LootList[LootCount].Aco.szName + "]...");
	    continue;
	} // endif
	
	// See if item is money.  If so skip
	if (LootList[LootCount].Aco.mcm & (mcmPyreal | mcmTradeNotes)) {
	    if (debugLoot) Log.OutD("AssessLoot: Skipping money [" + LootList[LootCount].Aco.szName + "]...");
	    continue;
	} // endif

	// If we reach here then the item is ok so add it to list.
	// First determine if an assess is needed on the item.
	if (NoAssessItem(LootList[LootCount].Aco)) {
	    LootList[LootCount].AssessReq = false;
	    if (debugLoot) Log.OutD("AssessLoot: Found good item, no assess needed");
	} else {
	    LootList[LootCount].AssessReq = true;
	    AssessNeeded++;
	    if (debugLoot) Log.OutD("AssessLoot: Found good item, assess needed");
	} // endif
	LootCount++;
    } // end for

    Done = false;
    AssessSelf = false;
    Chat.Screen("Found " + LootCount + " items, need to assess " + AssessNeeded + " of them...");

    while (!Done) {
	Done = true;

	// If an assess failed, then we assess ourselves at start of run because of occasional access bug
	if (AssessSelf) {
	    AssessReceived = false;
	    AssessTime = new Date();
	    skapi.AssessAco(skapi.acoChar);
	    while ((!AssessReceived) && ((new Date()-AssessTime) < assessTimeout))
		skapi.WaitEvent(20, wemFullTimeout);
	    AssessSelf = false;
	} // endif

	for (var x=0; x<LootCount ; x++) {
	    if (debugLoot)
		Log.OutD("AssessLoot: Checking LootList Index [" + x + "] oid [" +
			 Util.Hex(LootList[x].Aco.oid) + "] Item [" + LootList[x].Aco.szName + "]");

	    if (!LootList[x].Aco.fExists) {
		if (debugLoot) Log.OutD("AssessLoot: Item no longer exists.");
		LootList[x].AssessReq = false;
		LootList[x].AssessDone = true;
	    } // endif

	    if (AbortAction) {
	    	if (debugLoot) Log.OutD("AssessLoot: Abort button pressed");
	    	Chat.Screen("Aborting re-assess...");
		AbortAction = false;
		return false;
	    } // endif

	    // If item assess not required then skip
	    if (!(LootList[x].AssessReq)) {
		if (debugLoot) Log.OutD("AssessLoot: Assess not required");
		continue;
	    } // endif

	    // If assess already done then skip
	    if (LootList[x].AssessDone) {
		if (debugLoot) Log.OutD("AssessLoot: Assess already done");
		continue;
	    } // endif

	    // Assess item
	    LootList[x].AssessTime = new Date();
	    LootList[x].AssessCount++;
	    if (debugLoot) Log.OutD("AssessLoot: Assessing item...");
	    skapi.AssessAco(LootList[x].Aco);

	    // Wait for item to be assessed
	    while ((LootList[x].AssessDone != true) &&
		   ((new Date()-LootList[x].AssessTime) < assessTimeout))
			skapi.WaitEvent(20, wemFullTimeout);

	    if (LootList[x].AssessDone == true) {
		if (debugLoot) Log.OutD("AssessLoot: Assess successful");
		// See if item has an inscription, is imbued, or is retained.  If so then skip it.
		if ((LootList[x].Aco.oai.ibi != undefined) && (LootList[x].Aco.oai.ibi != null)) {
		    if ((LootList[x].Aco.oai.ibi.szInscription != null) &&
		        (LootList[x].Aco.oai.ibi.szInscription != "")) {
			if (debugLoot) Log.OutD("AssessLoot: Item has inscription, skipping...");
			LootList[x].Checked = true;
		    } else {
			if ((LootList[x].Aco.oai.ibi.ctink != null) &&
			    (LootList[x].Aco.oai.ibi.ctink > 0)) {
			    if (debugLoot) Log.OutD("AssessLoot: Item has been tinked, skipping...");
			    LootList[x].Checked = true;
			} else {
			    if (LootList[x].Aco.oai.ibi.fRetained) {
			    	// Item is marked as retained, don't include in loot lists
			    	if (debugLoot) Log.OutD("AssessLoot: Item retained, skipping...");
			    	LootList[x].Checked = true;
			    } // endif
			} // endif
		    } // endif
		} // endif
	    } else {
		if (LootList[x].AssessCount < assessLimit) {
		if (debugLoot) Log.OutD("AssessLoot: Assess timeout");
		    AssessSelf = true;
		    Done = false;
		} else {
		    // Assess limit reached, skip this item
		    if (debugLoot) Log.OutD("AssessLoot: Assess timeout, skipping item");
		    LootList[x].AssessReq = false;
		} // endif
	    } // endif
	} // endfor
    } // end while

    Chat.Screen("Assesses done, categorizing items...");

    // Assesses done for all items (either success or failure), now categorize items
    for (var x=0; x<LootCount; x++) {
	if (debugLoot)
	    Log.OutD("AssessLoot: Index [" + x + "] Item [" + LootList[x].Aco.szName +
		     "] Checked [" + LootList[x].Checked + "] AssReq [" +
		     LootList[x].AssessReq + "] AssDone [" + LootList[x].AssessDone + "]");

	// Make sure all events processed so we don't lose any
	skapi.WaitEvent(0);

	// If item already checked, then continue with next item
	if (LootList[x].Checked) continue;

	// See what type of item it is and add it to the loot list for this loot type.
	var LootType = Loot.WhatType(LootList[x].Aco);
	var ItemDesc = Assess.BuildDesc(LootList[x].Aco);
	LootList[x].Desc = ItemDesc;
	if (debugLoot) Log.OutD("AssessLoot: LootType [" + LootType + "] ItemDesc [" + ItemDesc + "]");
	Loot.AddLoot(LootList[x].Aco, LootType, ItemDesc);
    } // endfor
    
    // In case user got rid of any items during reassess, make sure all loot items still exist in inventory
    Loot.RefreshLoot();

    Chat.Screen("Categorization complete.");
    if (debugLoot) Log.OutD("AssessLoot: End");
    return;
} // end func


function MacroInit() {
    BuffMode	= startBuffMode;
    AttackMode	= startAttackMode;
    Stealth	= startStealth;
    AutoMove	= startAutoMove;
    ExclMode	= startExclMode;
    SafeMode	= startSafeMode;
    PeaceMode	= startPeaceMode;

    // Save current player game state so we can detect changes
    LastPlig = skapi.plig;

    // Save experience at start of macro
    TotalExpStart = skapi.expTotal;

    // Now set the select range for creatures based on whether we are outdoors or in a dungeon.  This
    // is rechecked every Idle period.
    if (skapi.maplocCur.fInDungeon) {
    	if (SelRange != selRangeDung) {
	    Log.OutDM("MacroInit", "Select range set to in-dungeon distance of " + selRangeDung + " map units.", ToChat);
	    SelRange = selRangeDung;
	} // endif
    } else {
   	if (SelRange != selRangeOut) {
	    Log.OutDM("MacroInit", "Select range set to outdoors distance of " + selRangeOut + " map units.", ToChat);
	    SelRange = selRangeOut;
	} // endif
    } // endif

    // Initialize previous corpse list
    for (var x=0; x<lootPrevCorpseMax; x++) PrevCorpseList[x] = 0;

    CritSpellsInit();
} // end func


function CritSpellsInit() {
    // Define critical spells needed for macro operation
    SpellObj_DrainHealth	= skapi.SpellInfoFromSpellid(spellListDrainHealth[spellDrainHealth]);
    SpellObj_DrainStamina	= skapi.SpellInfoFromSpellid(spellListDrainStamina[spellDrainStamina]);
    SpellObj_DrainMana		= skapi.SpellInfoFromSpellid(spellListDrainMana[spellDrainMana]);
    SpellObj_StamHealth		= skapi.SpellInfoFromSpellid(spellListStamHealth[spellStamHealth]);
    SpellObj_StamMana		= skapi.SpellInfoFromSpellid(spellListStamMana[spellStamMana]);
    SpellObj_Heal		= skapi.SpellInfoFromSpellid(spellListHeal[spellHeal]);
    SpellObj_Heal7		= skapi.SpellInfoFromSpellid(spellListHeal[7]);
    SpellObj_Revitalize		= skapi.SpellInfoFromSpellid(spellListRevitalize[spellRevitalize]);
    SpellObj_Strength		= skapi.SpellInfoFromSpellid(spellListStrength[spellStrength]);
} // end func


function CastInit() {
    if (debug) Log.OutD("CastInit: Begin");

    Cast.BuildSpellBook();	// Make sure spellbook is built correctly
    Spell = new SplObj();	// Initialize spell object

    if (debug) Log.OutD("CastInit: End");
} // end function CastInit


function SalvageInit() {
    if (debug||debugLoot) Log.OutD("SalvageInit: Begin");

    // Get UST Aco
    UstAco = skapi.AcoFindInInv("Ust");
    if (flagAutoUst && !UstAco) {
    	flagAutoUst = false;
    	skapi.SetControlProperty(szPanelConfig, "chkAutoUst", "checked", flagAutoUst);
    	Log.OutD("SalvageInit: Auto-USTing disabled, no UST tool found");
    	Chat.Screen("Error: Auto-USTing disabled, no UST tool found");
    } // endif

   if (debug||debugLoot) Log.OutD("SalvageInit: End");
} // end function

function CreInit(CreInitDebug) {
    if ((CreInitDebug==undefined) || (CreInitDebug==null)) CreInitDebug = false;
    if (debug||CreInitDebug) Log.OutD("CreInit: Begin");

    // Define collection filter that returns all creatures within our attack range
    Acf = skapi.AcfNew();
    Acf.ocm = ocmMonster;

    var OneFound = false;
    if (selPrioList.length > 0) {
    	OneFound = true;
    	Log.OutDM("CreInit", "Obsolete variable selPrioList set in UserConfig.", ToChat, cmcYellow);
    } // endif
    	
    if (attSkipList.length > 0) {
     	OneFound = true;
    	Log.OutDM("CreInit", "Obsolete variable attSkipList set in UserConfig.", ToChat, cmcYellow);
    } // endif
  	
    if (attYieldList.length > 0) {
        OneFound = true;
    	Log.OutDM("CreInit", "Obsolete variable attYieldList set in UserConfig.", ToChat, cmcYellow);
    } // endif	
    	
    if (attVulnList.length > 0) {
    	OneFound = true;
    	Log.OutDM("CreInit", "Obsolete variable attVulnList set in UserConfig.", ToChat, cmcYellow);
    } // endif    	
    	
    if (attDangerList.length > 0) {
    	OneFound = true;
    	Log.OutDM("CreInit", "Obsolete variable attDangerList set in UserConfig.", ToChat, cmcYellow);
    } // endif    	
    	
    if (attNoStepDown.length > 0) {
    	OneFound = true;
    	Log.OutDM("CreInit", "Obsolete variable attNoStepDown set in UserConfig.", ToChat, cmcYellow);
    } // endif    	
    	
    if (attBoltList.length > 0) {
    	OneFound = true;
    	Log.OutDM("CreInit", "Obsolete variable attBoltList set in UserConfig.", ToChat, cmcYellow);
    } // endif    	
    	
    if (attNoHealthList.length > 0) {
    	OneFound = true;
    	Log.OutDM("CreInit", "Obsolete variable attNoHealthList set in UserConfig.", ToChat, cmcYellow);
    } // endif    	
    	
    if (attNoManaList.length > 0) {
    	OneFound = true;
    	Log.OutDM("CreInit", "Obsolete variable attNoManaList set in UserConfig.", ToChat, cmcYellow);
    } // endif    	
    	
    if (attNoStaminaList.length > 0) {
    	OneFound = true;
    	Log.OutDM("CreInit", "Obsolete variable attNoStaminaList set in UserConfig.", ToChat, cmcYellow);
    } // endif    
    
    if (OneFound) {
    	Log.OutDM("CreInit", "One or more obsolete creature variables set in UserConfig.", ToChat, cmcYellow);
    	Log.OutDM("CreInit", "This settings should now be done through the decal creature panel", ToChat, cmcYellow);
    	Log.OutDM("CreInit", "as these variables will be removed in a future release.", ToChat, cmcYellow);
    	Log.OutDM("CreInit", "The variables were still applied to the creature database so", ToChat, cmcYellow);
    	Log.OutDM("CreInit", "saving the current creature config will propagate these", ToChat, cmcYellow);
    	Log.OutDM("CreInit", "settings to the new creature file.", ToChat, cmcYellow);
    } // endif
    
    // Set priorities for creature on priority list.  Since default priority = creatures experience
    // points, assign higher numbers for creatures on this list.  Creatures at beginning of list get
    // higher priority then those at end.
    // Bump up priorities for creatures on 'selPrioList'.
    if (CreInitDebug) Log.OutD("CreInit: selPrioList length [" + selPrioList.length + "]");
    for (var x=0; x<selPrioList.length; x++) {
    	var CreIndex = Creature.NameToIndex(selPrioList[x]);
    	if (CreIndex == 0) {
    	    Log.OutD("CreInit: selPrioList, unknown creature [" + selPrioList[x] + "]",true);
    	} else {
    	    Creature.Info[CreIndex].Priority = (200000+selPrioList.length)-x;
    	    if (CreInitDebug)
    	        Log.OutD("CreInit: Creature [" + selPrioList[x] + "] New Prio [" +
    	    	      Creature.Info[CreIndex].Priority+ "]");
    	}
    } // endfor

    // For creatures inside of danger distance, don't select creatures on the attSkipList
    if (CreInitDebug) Log.OutD("CreInit: attSkipList length [" + attSkipList.length + "]");
    for (var x=0; x<attSkipList.length; x++) {
    	var CreIndex = Creature.NameToIndex(attSkipList[x]);
    	if (CreIndex == 0) {
    	    Log.OutD("CreInit: attSkipList, unknown creature [" + attSkipList[x] + "]", true);
    	} else {
    	    Creature.Info[CreIndex].AttFlag = attNever;
    	    if (CreInitDebug)
    	        Log.OutD("CreInit: Creature [" + attSkipList[x] + "] AttFlag = attNever [" +
    	    		 Creature.Info[CreIndex].AttFlag + "]");
    	} // endif
    } // endfor

    // For creatures on attYieldList, always yield them
    if (CreInitDebug) Log.OutD("CreInit: attYieldList length [" + attYieldList.length + "]");
    for (var x=0; x<attYieldList.length; x++) {
    	var CreIndex = Creature.NameToIndex(attYieldList[x]);
    	if (CreIndex == 0) {
    	    Log.OutD("CreInit: attYieldList, unknown creature [" + attYieldList[x] + "]", true);
    	} else {
    	    Creature.Info[CreIndex].YieldFlag = attAlways;
    	    if (CreInitDebug)
    	        Log.OutD("CreInit: Creature [" + attYieldList[x] + "] YieldFlag = attAlways [" +
    	    		 Creature.Info[CreIndex].YieldFlag + "]");
    	} // endif
    } // endfor

    // For creatures on attVulnList, always vuln them
    if (CreInitDebug) Log.OutD("CreInit: attVulnList length [" + attVulnList.length + "]");
    for (var x=0; x<attVulnList.length; x++) {
    	var CreIndex = Creature.NameToIndex(attVulnList[x]);
    	if (CreIndex == 0) {
    	    Log.OutD("CreInit: attVulnList, unknown creature [" + attVulnList[x] + "]", true);
    	} else {
    	    Creature.Info[CreIndex].VulnFlag = attAlways;
    	    if (CreInitDebug)
    	        Log.OutD("CreInit: Creature [" + attVulnList[x] + "] VulnFlag = attAlways [" +
    	    		 Creature.Info[CreIndex].VulnFlag+ "]");
    	} // endif
    } // endfor

    // For creatures on attDangerList, try to kill as fast as possible.  Drain mana/stam spells
    // are disabled for these creatures, and the stam to manas threshold is cut in half
    if (CreInitDebug) Log.OutD("CreInit: attDangerList length [" + attDangerList.length + "]");
    for (var x=0; x<attDangerList.length; x++) {
    	var CreIndex = Creature.NameToIndex(attDangerList[x]);
    	if (CreIndex == 0) {
    	    Log.OutD("CreInit: attDangerList, unknown creature [" + attDangerList[x] + "]");
    	    continue;
    	} // endif
    	Creature.Info[CreIndex].DangerFlag = true;
    	if (CreInitDebug)
    	    Log.OutD("CreInit: Creature [" + attDangerList[x] + "] DangerFlag = True [" +
    	    		 Creature.Info[CreIndex].DangerFlag + "]");
    } // endfor

    // We never try to stepdown war spells for creatures on the attNoStepDown list.  This is
    // useful for cretures that regain health fast
    if (CreInitDebug) Log.OutD("CreInit: attNoStepDown length [" + attNoStepDown.length + "]");
    for (var x=0; x<attNoStepDown.length; x++) {
    	var CreIndex = Creature.NameToIndex(attNoStepDown[x]);
    	if (CreIndex == 0) {
    	    Log.OutD("CreInit: attNoStepDown, unknown creature [" + attNoStepDown[x] + "]");
    	    continue;
    	} // endif
    	Creature.Info[CreIndex].NoStepDown = true;
    	if (CreInitDebug)
    	    Log.OutD("CreInit: Creature [" + attNoStepDown[x] + "] NoStepDown = True [" +
    	    		 Creature.Info[CreIndex].NoStepDown + "]");
    } // endfor

    // For creatures on attBoltList, always use bolts instead of arcs.  This is used
    // for creatures that run quickly towards us which means arcs almost always miss.
    if (CreInitDebug) Log.OutD("CreInit: attBoltList length [" + attBoltList.length + "]");
    for (var x=0; x<attBoltList.length; x++) {
    	var CreIndex = Creature.NameToIndex(attBoltList[x]);
    	if (CreIndex == 0) {
    	    Log.OutD("CreInit: attBoltList, unknown creature [" + attBoltList[x] + "]");
    	    continue;
    	} // endif
    	Creature.Info[CreIndex].WarSpell = attSpellBolt;
    	if (CreInitDebug)
    	    Log.OutD("CreInit: Cre [" + attBoltList[x] + "] WarSpell [" + Creature.Info[CreIndex].WarSpell + "]");
    } // endfor

    // Never try to drain health from creatures on this list.
    if (CreInitDebug) Log.OutD("CreInit: attNoHealthList length [" + attNoHealthList.length + "]");
    for (var x=0; x<attNoHealthList.length; x++) {
    	var CreIndex = Creature.NameToIndex(attNoHealthList[x]);
    	if (CreIndex == 0) {
    	    Log.OutD("CreInit: attNoHealthList, unknown creature [" + attNoHealthList[x] + "]");
    	    continue;
    	} // endif
    	Creature.Info[CreIndex].NoDrainHealth = true;
    	if (CreInitDebug)
    	    Log.OutD("CreInit: Creature [" + attNoHealthList[x] + "] .NoDrainHealth = true [" +
    	    		 Creature.Info[CreIndex].NoDrainHealth + "]");
    } // endfor

    // Never try to drain mana from creatures on this list.  For creatures that start out with a
    // high mana value but quickly deplete it casting spells
    if (CreInitDebug) Log.OutD("CreInit: attNoManaList length [" + attNoManaList.length + "]");
    for (var x=0; x<attNoManaList.length; x++) {
    	var CreIndex = Creature.NameToIndex(attNoManaList[x]);
    	if (CreIndex == 0) {
    	    Log.OutD("CreInit: attNoManaList, unknown creature [" + attNoManaList[x] + "]");
    	    continue;
    	} // endif
    	Creature.Info[CreIndex].NoDrainMana = true;
    	if (CreInitDebug)
    	    Log.OutD("CreInit: Cre [" + attNoManaList[x] + "] NoDrainMana [" + Creature.Info[CreIndex].NoDrainMana + "]");
    } // endfor

    // Never try to drain stamina from creatures on this list.
    if (CreInitDebug) Log.OutD("CreInit: attNoStaminaList length [" + attNoStaminaList.length + "]");
    for (var x=0; x<attNoStaminaList.length; x++) {
    	var CreIndex = Creature.NameToIndex(attNoStaminaList[x]);
    	if (CreIndex == 0) {
    	    Log.OutD("CreInit: attNoStaminaList, unknown creature [" + attNoStaminaList[x] + "]");
    	    continue;
    	} // endif
    	Creature.Info[CreIndex].NoDrainStamina = true;
    	if (CreInitDebug)
    	    Log.OutD("CreInit: Cre [" + attNoStaminaList[x] + "] NoDrainStamina [" +
    	    	     Creature.Info[CreIndex].NoDrainStamina + "]");
    } // endfor

    if (debug) Log.OutD("CreInit: End");
} // end function


function WarEvade(SlideFlag) {
    if (debug||debugEvade) Log.OutD("WarEvade: Begin, SlideFlag [" + SlideFlag + "]");

    // If previous evade happened very recently, then continue in same direction so we don't
    // run into previous war spell
    if ((new Date()-PrevEvadeTime) < avoidHoldDown) {
    	if (debugEvade) {
    	    Log.OutD("WarEvade: Previous evade just occured, continuing in same direction");
    	    if (PrevEvadeKey == cmidSideStepLeft)
    	        Log.OutD("WarEvade: Previous direction was left");
    	    else
    	      	Log.OutD("WarEvade: Previous direction was right");
    	} // endif
    	var KeyPress = PrevEvadeKey;
    } else {
    	// No recent evade, figure out direction that brings us closer to origin
	var OurLoc = skapi.maplocCur;
	var Head2Home = OurLoc.HeadToMaploc(HomeLoc);
	var Diff = Head2Home - OurLoc.head;
	if (Diff<0) Diff = Diff + 360;

	if (debugEvade) {
	    Log.OutD("WarEvade: OurLoc [" + Util.FormatCoords(OurLoc,3) + "] OurHead [" + OurLoc.head +
		     "] Head2Home [" + Head2Home + "]");
	    Log.OutD("WarEvade: HomeLoc [" + Util.FormatCoords(HomeLoc,3) + "] Diff [" + Diff + "]");
	} // endif

	if (Diff>180) {
    	    if (debugEvade) Log.OutD("WarEvade: Sidestepping Left!");
	    var KeyPress = cmidSideStepLeft;	// Sidestep Left
	} else {
    	    if (debugEvade) Log.OutD("WarEvade: Sidestepping Right!");
	    var KeyPress = cmidSideStepRight;	// Sidestep Right
	} // endif

    } // endif

    // If we are slidecasting hold down the sidestep and backwards key, else just sidestep
    if (SlideFlag) {
    	skapi.Keys(KeyPress, kmoDown);
    	skapi.Keys(cmidWalkBackwards, kmoDown);
	skapi.Sleep(avoidTimeSlide);
	skapi.Keys(cmidWalkBackwards, kmoUp);
	skapi.Keys(KeyPress, kmoUp);
    } else {
	skapi.Keys(KeyPress, kmoDown);
	skapi.Sleep(avoidTime);
	skapi.Keys(KeyPress, kmoUp);
    } // endif

    // Record what direction we went in case another war spell comes very quickly
    PrevEvadeTime = new Date();
    PrevEvadeKey  = KeyPress;

    if (debug||debugEvade) Log.OutD("WarEvade: End, Sidestep finished");
    return;
} // end function WarEvade


// Adds a creature to the previously attack list. Used to determine if any debuffs
// still active on such creatures if we attack them again.
function AttPrevAdd(Id) {
    // Log entry to this function (if desired)
    if (debug||debugPrev) Log.OutD("AttPrevAdd: Id [" + Util.Hex(Id) + "] AttCtx.PrevCnt [" + AttCtx.PrevCnt + "]" );

    // If no yield, imperl, or vuln spell was cast this kill attempt then no need to record anything.
    if ((Att.YieldTime == 0) && (Att.ImperlTime == 0) && (Att.VulnTime == 0) ) {
    	if (debugPrev) Log.OutD("AttPrevAdd: No yield, imperl, or vuln done");
    	return false;
    } // endif

    // Figure out expiration time for yield
    if (Att.YieldTime != 0) {
    	// Compute yield expiration time.  Subtract 30 seconds
    	var YieldExp = Att.YieldTime.getTime() + ((Att.YieldSpell.csecDuration-30)*1000);
    	if (debugPrev) Log.OutD("AttPrevAdd: Yield Exp Time [" + new Date(YieldExp) + "]");
    } else {
    	var YieldExp = 0;
    } // endif
 
    // Figure out expiration time for imperl
    if (Att.ImperlTime != 0) {
    	// Compute imperl expiration time.  Subtract 30 seconds
    	var ImperlExp = Att.ImperlTime.getTime() + ((Att.ImperlSpell.csecDuration-30)*1000);
    	if (debugPrev) Log.OutD("AttPrevAdd: Imperl Exp Time [" + new Date(ImperlExp) + "]");
    } else {
    	var ImperlExp = 0;
    } // endif

     // Figure out expiration time for vuln
    if (Att.VulnTime!=0) {
    	// Compute vuln expiration time.  Subtract 30 seconds
    	var VulnExp = Att.VulnTime.getTime() + ((Att.VulnSpell.csecDuration-30)*1000);
   	if (debugPrev) Log.OutD("AttPrevAdd: Vuln Exp Time [" + new Date(VulnExp) + "]");
    } else {
    	var VulnExp = 0;
    } // endif

    // Either a yield, imperl, or vuln was cast.  Figure out what slot number in list to use.
    // If creature already on the list then update its information, otherwise create a new
    // entry at the end.  While going though the list purge expired entries.
    var Index = -1;
    var x=0;
    while (x<AttCtx.PrevCnt) {

    	// If this id already on list, then reuse index
    	if (AttCtx.PrevList[x].Id == Id) {
	    Index = x;
	    break;
	} // endif

    	// Not a match, see if this entry has expired
	var RawTime = new Date();
	var CurTime = RawTime.getTime();
	if ((CurTime > AttCtx.PrevList[x].YieldExp) &&
	    (CurTime > AttCtx.PrevList[x].ImperlExp)  && 
	    (CurTime > AttCtx.PrevList[x].VulnExp)) {
	    // Entry has expired, decrement count of entries and move last entry to this one
	    if (debugPrev) Log.OutD("AttPrevAdd: Entry at index [" + x + "] expired, deleting...");
	    AttCtx.PrevCnt--;
	    if (AttCtx.PrevCnt > x) {
	    	AttCtx.PrevList[x] = AttCtx.PrevList[AttCtx.PrevCnt];
	    } // endif
	    continue;	// don't increment x as x is now a new entry
	} // endif

	// Entry not a match or expired, so look at next entry
	x++;
    } // end while

    // If didn't find creature's id, then add a new entry to the table
    if (Index == -1) {
    	if (debugPrev) Log.OutD("AttPrevAdd: Creature not in table, adding new entry");
	Index=AttCtx.PrevCnt;
	AttCtx.PrevList[Index] = new AttPrevObj(Id);
	AttCtx.PrevCnt++;
    } // endif

    // Now update expiration time/dates
    if (debugPrev) Log.OutD("AttPrevAdd: Updating info at index [" + Index + "]" );
    AttCtx.PrevList[Index].YieldExp	= YieldExp;
    AttCtx.PrevList[Index].ImperlExp	= ImperlExp;
    AttCtx.PrevList[Index].VulnExp	= VulnExp;
    AttCtx.PrevList[Index].VulnLvl	= Att.VulnLevel;

    if (debug||debugPrev) Log.OutD("AttPrevAdd: End, AttCtx.PrevCnt [" + AttCtx.PrevCnt + "]" );
} // end function

// See if a creature was attacked recently and still has attack debuffs (yield or vuln)
function AttPrevLookup(Id) {
    if (debug||debugPrev) Log.OutD("AttPrevLookup: Id [" + Util.Hex(Id) + "] AttCtx.PrevCnt [" + AttCtx.PrevCnt + "]");

    // Lookup id
    for (var x=0; x<AttCtx.PrevCnt; x++) {
    	if (debugPrev) {
    	    Log.OutD("Index [" + x + "] Id [" + Util.Hex(AttCtx.PrevList[x].Id) + "]");
    	    Log.OutD("AttCtx.PrevList[x].YieldExp  [" + new Date(AttCtx.PrevList[x].YieldExp)  + "]");
    	    Log.OutD("AttCtx.PrevList[x].ImperlExp [" + new Date(AttCtx.PrevList[x].ImperlExp) + "]");
    	    Log.OutD("AttCtx.PrevList[x].VulnExp   [" + new Date(AttCtx.PrevList[x].VulnExp)   + "]");
    	    Log.OutD("AttCtx.PrevList[x].VulnLvl   [" + new Date(AttCtx.PrevList[x].VulnLvl)   + "]");
    	} // endif
    	if (AttCtx.PrevList[x].Id == Id) {
	    // Match found
	    Att.PrevYieldExp  = AttCtx.PrevList[x].YieldExp;
	    Att.PrevImperlExp = AttCtx.PrevList[x].ImperlExp;
	    Att.PrevVulnExp   = AttCtx.PrevList[x].VulnExp;
	    Att.PrevVulnLvl   = AttCtx.PrevList[x].VulnLvl;
	    if (debugPrev) {
	    	Log.OutD("Att.PrevYieldExp  [" + Att.PrevYieldExp  + "]");
		Log.OutD("Att.PrevImperlExp [" + Att.PrevImperlExp + "]");
		Log.OutD("Att.PrevVulnExp   [" + Att.PrevVulnExp   + "]");
		Log.OutD("Att.PrevVulnLvl   [" + Att.PrevVulnLvl   + "]");
	    } // endif
	    if (debug||debugPrev) Log.OutD("AttPrevLookup: End, Match Found, Returning true");
	    return true;
	} // endif
    } // endfor

    // If we get here then creature was not yielded/vulned before
    if (debug||debugPrev) Log.OutD("AttPrevLookup: End, No match found, returning false");
    return false;
} // end function


// See if a creature was previously processed in support mode
function AttDoneLookup(Id) {
    // if (debug||debugDone)
	// Log.OutD("AttDoneLookup: Id [" + Util.Hex(Id) + "] AttCtx.DoneCnt [" + AttCtx.DoneCnt + "]");

    // Lookup id
    var RawTime = new Date();
    var CurTime = RawTime.getTime();
    for (var x=0; x<AttCtx.DoneCnt; x++) {
	// if (debugDone)
	//    Log.OutD("Index [" + x + "] .Id [" + Util.Hex(AttCtx.DoneList[x].Id) +
	//	     "] fExists [" + AttCtx.DoneList[x].Aco.fExists +
	//    	     "] .ExpTime [" + new Date(AttCtx.DoneList[x].ExpTime) + "]");

	if (AttCtx.DoneList[x].Id == Id) {
	    // Match found, see if entry expired
	    if (CurTime > AttCtx.DoneList[x].ExpTime) {
		// if (debug||debugDone) Log.OutD("AttDoneLookup: Match found but expired, returning false");
		return false;
	    } // endif

	    // if (debug||debugDone) Log.OutD("AttDoneLookup: End, Match Found, Returning true");
	    return true;
	} // endif
    } // endfor

    // If we get here then creature was not processed before in support mode
    // if (debug||debugDone) Log.OutD("AttDoneLookup: End, No match found, returning false");
    return false;
} // end function


// Adds a creature to the previously attack list. Used to determine if any debuffs
// still active on such creatures if we attack them again.
function AttDoneAdd(Id) {
    if (debug||debugDone)
	Log.OutD("AttDoneAdd: Id [" + Util.Hex(Id) + "] AttCtx.DoneCnt [" + AttCtx.DoneCnt + "]");

    // Calculate expiration time of Done entry.  
    var RawTime = new Date();
    var CurTime = RawTime.getTime();
    var ExpTime = 0;

    var Bonus = 1 + (spellAugGems*0.20);
    if (debugDone) Log.OutD("AttDoneAdd: spellAugGems [" + spellAugGems + "] Bonus [" + Bonus + "]");

    if (Att.ImperlTime != 0) {
    	// Compute imperl expiration time
    	ExpTime = Att.ImperlTime.getTime() + ((Att.ImperlSpell.csecDuration*Bonus)*1000);
    	if (debugDone) Log.OutD("AttDoneAdd: Using imperl Exp Time [" + new Date(ExpTime) + "]");
    } // endif

    if (Att.VulnTime != 0) {
    	// Compute vuln expiration time
    	var VulnExp = Att.VulnTime.getTime() + ((Att.VulnSpell.csecDuration*Bonus)*1000);
   	if (debugDone) Log.OutD("AttDoneAdd: Vuln Exp Time [" + new Date(VulnExp) + "]");
   	
   	// Now if an imperl wasn't done or the vuln will expire before the imperl, then
   	// use the vuln exp time as the DoneList expiration time.
   	if ((ExpTime == 0) || (VulnExp < ExpTime)) {
	    ExpTime = VulnExp;
	    if (debugDone) Log.OutD("AttDoneAdd: Using vuln exp Time [" + new Date(ExpTime) + "]");
	} // endif
    } // endif

    if (ExpTime == 0) {
	// Neither a vuln or imperl was cast, so don't need to add this entry
	if (debugDone) Log.OutD("AttDoneAdd: Neither a vuln or imperl was cast, aborting add.");
	return;
    } // endif

    if (debugDone) {
	Log.OutD("AttDoneAdd: Exp time for done entry will be [" +
	         new Date(ExpTime) + "] / [" + ExpTime + "]");
	Log.OutD("AttDoneAdd: CurTime [" + new Date(CurTime) + "] / [" + CurTime + "]");
    } // endif

    // Figure out what slot number in list to use.  If creature already on the list then
    // update its information, otherwise create a new entry at the end.  While going though
    // the list purge expired entries.
    var Index = -1;
    var x=0;
    while (x<AttCtx.DoneCnt) {
    	if (debugDone) {
	    var Exists = false;
	    if ((AttCtx.DoneList[x].Aco != null) && (AttCtx.DoneList[x].Aco.fExists)) Exists = true;
	    Log.OutD("AttDoneAdd: Index [" + x + "] id [" + Util.Hex(AttCtx.DoneList[x].Id) +
		     "] fExists [" + Exists + "]");
	    Log.OutD("AttDoneAdd: ExpTime [" + AttCtx.DoneList[x].ExpTime + 
		     "] / [" + new Date(AttCtx.DoneList[x].ExpTime) + "]");
	} // endif

    	// If this id already on list, then reuse index and just update the expiration time
    	if (AttCtx.DoneList[x].Id == Id) {
	    if (debugDone) Log.OutD("AttDoneAdd: Entry exists at index [" + x + "] updating exp time");
	    AttCtx.DoneList[x].ExpTime = ExpTime;
	    return;
	} // endif

    	// Not a match, see if this entry has expired
	if ((CurTime > AttCtx.DoneList[x].ExpTime) ||
	    (AttCtx.DoneList[x].Aco == null) ||
	    !(AttCtx.DoneList[x].Aco.fExists)) {
	    // Entry has expired or creature has been killed, decrement
	    // count of entries and move last entry to this one
	    if (debugDone)
		Log.OutD("AttDoneAdd: Entry [" + Util.Hex(AttCtx.DoneList[x].Id) +
			 "] at index [" + x + "] expired or doesn't exist, deleted.");
	    AttCtx.DoneCnt--;
	    if (AttCtx.DoneCnt > x) {
	    	AttCtx.DoneList[x] = AttCtx.DoneList[AttCtx.DoneCnt];
	    } // endif
	    continue;	// don't increment x as x is now a new entry
	} // endif

	// Entry not a match or expired, so look at next entry
	x++;
    } // end while

    // Didn't find creature's id, add a new entry to the table
    if (debugDone) Log.OutD("AttDoneAdd: Creature not in table, adding new entry at index [" + x + "]");
    AttCtx.DoneList[x] = new AttDoneObj(Id, ExpTime);
    AttCtx.DoneCnt++;
    if (debug||debugDone) Log.OutD("AttDoneAdd: End, AttCtx.DoneCnt [" + AttCtx.DoneCnt + "]" );
} // end function AttDoneAdd


function CheckFellow() {
    // First check health of everyone on list if enabled
    if (debugSupport) Chat.Screen("CheckFellow: HelpCnt [" + HelpCnt + "]");
    if (attHealFellow) {
    	if (debugSupport) Chat.Screen("CheckFellow: Checking health...");
	for (var x=0; x<HelpCnt; x++) {
	    if (debugSupport)
		Chat.Screen("CheckFellow: Index [" + x + "] .Health [" + HelpList[x].Health + " ] .MaxHealth [" +
	    		    HelpList[x].MaxHealth + "] supportHealth [" + supportHealth + "]");
	    if ((HelpList[x].MaxHealth-HelpList[x].Health) >= supportHealth) {   
		var FellowAco = skapi.AcoFromOid(HelpList[x].Oid);
		if (FellowAco == null) {
		    if (debugSupport) Chat.Screen("Aco null");
		    continue;       // if aco null then on to the next
		} // endif

		if (FellowAco.maploc == null) {
		    if (debugSupport) Chat.Screen("Maploc null");
		    continue;
		} // endif

		var DistToPlayer = Util.Distance(FellowAco.maploc,skapi.maplocCur);
		if (DistToPlayer <= supportDist) {
		    Chat.Screen("Healing needed for " + FellowAco.szName);   
		    var HealObj = skapi.SpellInfoFromSpellid(spellListHealOther[supportHealLvl]);
		    Cast.Cast(HealObj, FellowAco, false, true);
		    HelpList[x].Health = 999;	// set health so heal not attempted until stat update
		    return;
		} else {
		    if (debugSupport)
			Chat.Screen("Too far away, dist [" + DistToPlayer + "] max dist [" + supportDist + "]");
		} // endif
		break;
	    } // endif
	} // end for
    } // end if

     // Next check stamina of everyone on list if enabled
    if (attStamFellow) {
    	if (debugSupport) Chat.Screen("CheckFellow: Checking stamina...");
	for (var x=0; x<HelpCnt; x++) {
	    if (debugSupport)
		Chat.Screen("CheckFellow: Index [" + x + "] .Stamina [" + HelpList[x].Stamina + " ] .MaxStamina [" +
	    		    HelpList[x].MaxStamina + "] supportStamina [" + supportStamina + "]");
	    if ((HelpList[x].MaxStamina - HelpList[x].Stamina) >= supportStamina) {
		var FellowAco = skapi.AcoFromOid(HelpList[x].Oid);
		if (FellowAco == null) {
		    if (debugSupport) Chat.Screen("Aco null");
		    continue;       // if aco null then on to the next
		} // endif

		if (FellowAco.maploc == null) {
		    if (debugSupport) Chat.Screen("Maploc null");
		    continue;
		} // endif
		var DistToPlayer = Util.Distance(FellowAco.maploc,skapi.maplocCur);
		if (DistToPlayer <= supportDist) {
		    Chat.Screen("Stamina needed by " + FellowAco.szName); 
		    var StamObj = skapi.SpellInfoFromSpellid(spellListStamOther[supportStamLvl]);
		    Cast.Cast(StamObj, FellowAco, false, true);
		    HelpList[x].Stamina = 999;	// set stamina so stam not attempted until stat update
		    return;
		} // endif
	    } // endif
	} // end for
    } // end if
} // end function


// Function to add specified creature to the do not attack list
function OutOfRangeAdd(CreOid) {
    if (debug||debugAttack||debugRange) Log.OutD("OutOfRangeAdd: CreOid [" + Util.Hex(CreOid) + "]" );

    // Check for duplicate or expired item 
    var RetCode = false;
    var Dup = false; 
    for (var x=0; x<SelOutRangeCnt; x++) {
	if (SelOutRangeList[x].Id == CreOid) {
	    Dup = true;
	    SelOutRangeList[x].Count++;
	    SelOutRangeList[x].TimeLast = new Date();
	    if (debugAttack||debugRange) Log.OutD("OutOfRangeAdd: x [" + x + "] Id [" + Util.Hex(SelOutRangeList[x].Id) +
	    					  "] New count [" + SelOutRangeList[x].Count + "]" );
	    if (SelOutRangeList[x].Count > SelOutRangeMax) {
	    	Log.OutDM("OutOfRangeAdd", "Creature added to do not attack list");
	    	SelNoAttackAdd(CreOid);
	    	RetCode = true;
	    } // endif
	} // endif
	
	// See if entry expired.  If so remove it from list
	if ((new Date()-SelOutRangeList[x].TimeLast) > SelOutRangeExp) {
	    SelOutRangeCnt--;
	    SelOutRangeList[x] = SelOutRangeList[SelOutRangeCnt];
	    if (debugAttack||debugRange)
		Log.OutD("OutOfRangeAdd: Expired id [" + x + "/" + Util.Hex(SelOutRangeList[x].Id) +
			 "] List size now [" + SelOutRangeCnt + "]");
	} // endif
    } // endfor

    // If not a duplicate, then add a new entry to the list
    if (!Dup) {
	SelOutRangeList[SelOutRangeCnt] = new SelCreObj(CreOid);
	SelOutRangeCnt++;
	if (debugAttack||debugRange) Log.OutD("OutOfRangeAdd: New id, list size now [" + SelOutRangeCnt + "]");
    } // endif

    if (debug||debugAttack||debugRange) Log.OutD("OutOfRangeAdd: End, return code [" + RetCode + "]" );   
    return RetCode;
} // end func

// Function to add specified creature to the do not attack list
function SelNoAttackAdd(SelId) {
    if (debug) Log.OutD("SelNoAttackAdd: SelId [" + Util.Hex(SelId) + "]" );
    
    // Check for duplicate entry
    for (var x=0; x<SelDNALCnt; x++) {
    	if (SelDNAL[x].Id == SelId) return true;
    } // end for
    
    // Not a duplicate, add to list
    SelDNAL[SelDNALCnt] = new SelCreObj(SelId);
    SelDNALCnt++;
    return true;
} // end func

// Function to see if creature is on the do not attack list
function SelBadId(Ident) {
    if (debug) {
	Log.OutD("SelBadId: Ident [" + Util.Hex(Ident) + "]");
	Log.OutD("SelBadId: SelDNALCnt [" + SelDNALCnt + "]");
    } // endif

    for (index=0; index < SelDNALCnt; index++) {
	if (Ident == SelDNAL[index].Id) {
		if (debug2) Log.OutD("SelBadId: Match found");
		SelDNAL[index].LastIdTime = new Date();
		return true;
	} // endif
    } // end for

    if (debug) Log.OutD("SelBadId: Match not found");
    return false;
} // end function SelBadId


function TargetAco(Aco) {
    if (debug) Log.OutD("TargetAco: Begin");
    Att.CreHealth = 1;		// Assume creature at full health at start (update by OnTarget events)
    for (var loopcnt=0; loopcnt<50; loopcnt++) {
    	if ((Aco == null) || !Aco.fExists) {
    	    if (debugSelect) Log.OutD("TargetAco: Aco null or does not exist, aborting select attempt...");
    	    return false;
    	} // endif

    	// Select it
    	skapi.SelectAco(Aco);
    	skapi.WaitEvent(20, wemFullTimeout); // Pause a little bit

	var CurSelected = skapi.acoSelected;
    	if ((CurSelected != undefined) && (CurSelected != null) &&
    	    (CurSelected.fExists) && (CurSelected.oid == Aco.oid)) {
    	    SelFlag = true;
    	    if (debug||debugSelect||debugTarget) Log.OutD("TargetAco: End, Aco [" + Aco.szName + "] selected");
    	    return true;	 // Target is selected so we are all done
    	} // endif
    } // endfor

    // Target could not be selected
    if (debug||debugSelect||debugTarget) Log.OutD("TargetAco: End, failure");
    return false;
} // end function

function SelManOid() {
    // Convert manual select oid into an aco
    SelAco = skapi.AcoFromOid(AttCtx.ManSelOid);
    AttCtx.ManSelOid = 0;
 
    if (SelAco == null) {
	if (debugSelect||logSelect) Log.OutD("SelManOid: SelAco=null, rejected");
	return false;
    } // endif

    // If Aco does not exist anymore (ie it is stale), then reject
    if (!SelAco.fExists) {
    	if (debugSelect||logSelect) Log.OutD("SelManOid: SelAco.fExists = false, rejected");
    	return false;
    } // endif

    SelId    = SelAco.oid;
    SelIdHex = Util.Hex(SelId);
    SelName  = SelAco.szName;

    // Lookup this creature in database, add to database if unknown
    SelIndex = Creature.NameToIndex(SelName);
    if (SelIndex == 0) {
	if (logSelect||debugSelect)
	    Log.OutD("SelManOid: Creature [" + SelIdHex + "/" + SelName + "] unknown.  Adding...");
	Chat.Screen("Unknown creature '" + SelName + "' detected, adding to database...", cmcYellow);
	SelIndex = Creature.AddUnknownCre(SelAco);
	if (SelIndex == 0) {
	    // Couldn't add creature to database for some reason
	    Log.OutDM("SelManOid","Couldn't add creature to database, skipping...", true);
	    return false;
	} // endif

	var RaceCode = Creature.Info[SelIndex].RaceCode;
	Chat.Screen("Creature added to race '" + Creature.RaceInfo[RaceCode].RaceName +
	            "/" + RaceCode + "'.", cmcYellow);

	// Reset decal panel for creatures to make sure new creature displayed
	skapi.SetControlProperty(szPanelCreature, "raceList3", "selected", 0);
	skapi.SetControlProperty(szPanelCreature, "raceList3", "selected", 1);
    } // endif

    // Get our current location
    var OurLoc = skapi.maplocCur;

    SelPrio      = Creature.Info[SelIndex].Priority;
    SelDangerous = Creature.Info[SelIndex].DangerFlag;
    SelDist      = Util.Distance(SelAco.maploc,OurLoc);

    // Looks like a good creature to attack
    if (debugSelect||logSelect) Log.OutD("SelManOid: Creature [" + SelIdHex + "/" + SelName +"] selected");
    return true;
} // end func SelManOid

function SelLastAttacker() {
    // If last attacker is not a valid aco then reject
    SelUseAttackAco = false;
    if ((AttCtx.AttackAco == undefined) || (AttCtx.AttackAco == null)) {
	if (debugSelect||logSelect) Log.OutD("SelLastAttacker: AttCtx.AttackAco==null, rejected");
	return false;
    } // endif

    // If Aco does not exist anymore (ie it is stale), then reject
    if (!AttCtx.AttackAco.fExists) {
    	if (debugSelect||logSelect) Log.OutD("SelLastAttacker: AttCtx.AttackAco.fExists = false, rejected");
    	AttCtx.AttackAco = null;
    	return false;
    } // endif

    // See if last attack was too long ago, if so then reject
    if ((new Date()-LastAttTime)>attMaxLastAttInt) {
	if (debugSelect||logSelect) Log.OutD("SelLastAttacker: Last attack timed out, rejected");
	AttCtx.AttackAco = null;
	return false;
    } // endif

    // If we are in support mode and creature has already been imperled and/or vulned then reject
    if (attSupportOnly) {
    	if (AttDoneLookup(AttCtx.AttackAco.oid)) {
	    if (debugSelect||logSelect) Log.OutD("SelLastAttacker: Last attack already done, rejected");
	    AttCtx.AttackAco = null;
	    return false;
	} // endif
    } // endif

    SelAco   = AttCtx.AttackAco;
    SelId    = SelAco.oid;
    SelIdHex = Util.Hex(SelId);
    SelName  = SelAco.szName;

    // If exclusion mode enabled, see if creature is on lock list, ie someone else is killing it
    if (ExclMode && (ExclFindLock(SelId))) {
	if (debugSelect) Log.OutD("SelLastAttacker: Locked, rejected");
	AttCtx.AttackAco = null;
	return false;
    } // endif

    // Lookup this creature in database, add to database if unknown
    SelIndex = Creature.NameToIndex(SelName);
    if (SelIndex == 0) {
	if (logSelect||debugSelect)
	    Log.OutD("SelLastAttacker: Creature [" + SelIdHex + "/" + SelName + "] unknown.  Adding...");
	Chat.Screen("Unknown creature '" + SelName + "' detected, adding to database...", cmcYellow);
	SelIndex = Creature.AddUnknownCre(SelAco);
	if (SelIndex == 0) {
	    // Couldn't add creature to database for some reason
	    Log.OutDM("SelLastAttacker","Couldn't add creature to database, skipping...", true);
	    AttCtx.AttackAco = null;
	    return false;
	} // endif

	var RaceCode = Creature.Info[SelIndex].RaceCode;
	Chat.Screen("Creature added to race '" + Creature.RaceInfo[RaceCode].RaceName + "/" + RaceCode + "'.", cmcYellow);

	// Reset decal panel for creatures to make sure new creature displayed
	skapi.SetControlProperty(szPanelCreature, "raceList3", "selected", 0);
	skapi.SetControlProperty(szPanelCreature, "raceList3", "selected", 1);
    } // endif

    // If this is the previous creature we were attacking and we our within the holddown period,
    // then reject.  We do this because sometimes we get stale information from the OnMelee and
    // OnLastAttacker events.
    if ((AttCtx.PrevId == SelId) && ((new Date()-AttCtx.LastDone) < attPrevHoldDown)) {
	if (debugSelect == true) Log.OutD("SelLastAttacker: Creature was just attacked previously. Rejected");
	AttCtx.AttackAco = null;
	return false;
    } // endif

    // Get our current location
    var OurLoc = skapi.maplocCur;

    SelPrio      = Creature.Info[SelIndex].Priority;
    SelDangerous = Creature.Info[SelIndex].DangerFlag;
    SelDist      = Util.Distance(SelAco.maploc,OurLoc);

    // If we want to restrict selection of creatures by attitude, then check how far creature is above or below us
    if (flagSelCheckZ) {
    	if ((SelAco.maploc.z - OurLoc.z) > attMaxHighZDiff) {
	    if (debugSelect) Log.OutD("SelLastAttacker: Creature too far above us, rejected");
	    AttCtx.AttackAco = null;
	    return false;
	} // endif
   	if ((OurLoc.z - SelAco.maploc.z) > attMaxLowZDiff) {
	    if (debugSelect) Log.OutD("SelLastAttacker: Creature too far below us, rejected");
	    AttCtx.AttackAco = null;
	    return false;
	} // endif
    } // endif

    // See if creature within attack range
    if (SelDist>SelRange) {
	if (debugSelect) Log.OutD("SelLastAttacker: Cre [" + SelIdHex + "/" + SelName + "] out of range, rejected");
	AttCtx.AttackAco = null;
	return false;
    } // endif

    // if Monster's attack flag = never, reject
    if (Creature.Info[SelIndex].AttFlag == attNever) {
	if (debugSelect) Log.OutD("SelLastAttacker: Cre [" + SelIdHex + "/" + MonName + "] attflag = never. Rejected");
	AttCtx.AttackAco = null;
	return false;
    } // endif

    // if Monster's attack flag = auto, then check race attack setting
    if ((Creature.Info[SelIndex].AttFlag == attAuto) &&
	(Creature.RaceInfo[Creature.Info[SelIndex].RaceCode].AttFlag == attNever)) {
	if (debugSelect) Log.OutD("SelLastAttacker: Race [" + SelIdHex + "/" + SelName + "] attflag = never. Rejected");
	AttCtx.AttackAco = null;
	return false;
    } // endif

    // Looks like a good creature to attack
    if (debugSelect||logSelect)
	Log.OutD("SelLastAttacker: UseAtt.Aco: Creature [" + SelIdHex + "/" + SelName +"] last attacker");
    return true;
} // end function


// Find a creature close to us.   We attack anything close to us except for creatures with
// AttFlag == attNever.  Creatures with DangerFlag = true will be selected first, then the
// creature with the highest priority is checked, then distance from us is checked third
function FindCloseCre(MaxDist) {
    if (debug||debugSelect) Log.OutD("FindCloseCre: Begin");

    // Clear current creature select aco
    SelAco = null;
    SelLocked = false;

    // Get our current location
    var OurLoc = skapi.maplocCur;

    for (var index=0; index<MonsterList.Count; index++) {
	var MonAco   = MonsterList.Item(index);
	var MonId    = MonAco.oid;
	var MonIdHex = Util.Hex(MonId);
	var MonName  = MonAco.szName;
	if (debugSelect) Log.OutD("FindCloseCre: Examining [" + MonName + "] Id [" + MonIdHex + "]");
	if (!MonAco.fExists) {
	    // Aco does not exist anymore (ie is a stale entry)
	    if (debugSelect) Log.OutD("FindCloseCre: fExists = false. Rejected");
	} // endif

	// If monster on Do Not Attack List, then reject
	if (SelBadId(MonId)) {
	    if (debugSelect) Log.OutD("FindCloseCre: Creature [" + MonIdHex + "] on DNAL list. Rejected");
	    continue; // goto next creature
	} // endif

	if (attSupportOnly) {
    	    if (AttDoneLookup(MonId)) {
		if (debugSelect) Log.OutD("FindCloseCre: (support) already done, rejected");
		continue;
	    } // endif
	} // endif

	// If creature outside maximum distance, then reject.  First get monster location and distance
	var MonLoc  = MonAco.maploc;
	var MonDist = Util.Distance(MonLoc,OurLoc);
	if (MonDist > MaxDist) {
	    if (debugSelect) Log.OutD("FindCloseCre: Cre [" + MonIdHex + "/" + MonName + "] out of range. Rejected");
	    continue; // goto next creature
	} // endif

	// If we want to restrict selection of creatures by attitude, then check how far creature is above or below us
	if (flagSelCheckZ) {
	    if ((MonLoc.z - OurLoc.z) > attMaxHighZDiff) {
		if (debugSelect) Log.OutD("FindCloseCre: Creature too far above us, rejected");
		continue;
	    } // endif
	    if ((OurLoc.z - MonLoc.z) > attMaxLowZDiff) {
		if (debugSelect) Log.OutD("FindCloseCre: Creature too far below us, rejected");
		continue;
	    } // endif
	} // endif

	// See if creature is on lock list, ie someone else is killing it.  If the only
	// creatures we find are on the locked list then we pick one ie we help one
	// of our co-macroers out.
	if (ExclMode) {
	    if (ExclFindLock(MonId)) {
		MonLocked = true;
		if (debugSelect) Log.OutD("FindCloseCre: Creature locked.");
	    } else {
		MonLocked = false;
	    	if (debugSelect) Log.OutD("FindCloseCre: Creature not locked.");
	    } // endif
	} else {
	    MonLocked = false;
	    if (debugSelect) Log.OutD("FindCloseCre: Exclusion mode not set, creature not locked.");
	} // endif

	// If this is the previous creature we were attacking, then reject.  We do this because
	// sometimes it takes a few seconds for a killed creature to be removed from the object table.  If
	// the creature is locked though, then don't do this check since we may have just aborted our
	// kill attempt due to a lock collision.
	// if (debugSelect) {
	//    Log.OutD("FindCloseCre: MonId           [" + MonIdHex                  + "]");
	//    Log.OutD("FindCloseCre: MonLocked       [" + MonLocked                 + "]");
	//    Log.OutD("FindCloseCre: Current Time    [" + new Date()                + "]");
	//    Log.OutD("FindCloseCre: AttCtx.LastDone [" + new Date(AttCtx.LastDone) + "]");
	//    Log.OutD("FindCloseCre: AttCtx.PrevId   [" + Util.Hex(AttCtx.PrevId)   + "]");
	//    Log.OutD("FindCloseCre: attPrevHoldDown [" + attPrevHoldDown           + "]");
	// } // endif
 
	if ((MonId == AttCtx.PrevId) && !MonLocked && ((new Date()-AttCtx.LastDone) < attPrevHoldDown)) {
	    if (debugSelect) Log.OutD("FindCloseCre: Creature was just attacked previously. Rejected");
	    continue; // goto next creature
	} // endif

	// If this is the current creature we were attacking, then reject.  This case could happen when
	// we are in a critical condition and are looking for a closer target.
	if ((State==stateKill) && (MonId == Att.Oid) && !MonLocked) {
	    if (debugSelect) Log.OutD("FindCloseCre: Creature is being attacked now. Rejected");
	    continue; // goto next creature
	} // endif

	// See if this creature is in the database.  If not then add it in and use race record for vuln default.
	var MonIndex = Creature.NameToIndex(MonName);
	if (MonIndex == 0) {
	    if (logSelect||debugSelect)
		Log.OutD("FindCloseCre: Creature [" + MonIdHex + "/" + MonName + "] unknown.  Adding...");
	    Chat.Screen("Unknown creature '" + MonName + "' detected, adding to database...", cmcYellow);
	    MonIndex = Creature.AddUnknownCre(MonAco);
	    if (MonIndex == 0) {
		// Couldn't add creature to database for some reason
		Log.OutDM("FindCloseCre","Couldn't add creature to database, skipping...", true);
		continue;	// goto next creature
	    } // endif
	    var RaceCode = Creature.Info[MonIndex].RaceCode;
	    Chat.Screen("Creature added to race '" + Creature.RaceInfo[RaceCode].RaceName + "/" +
	    		 RaceCode + "'.", cmcYellow);

	    // Reset decal panel for creatures to make sure new creature displayed
	    skapi.SetControlProperty(szPanelCreature, "raceList3", "selected", 0);
	    skapi.SetControlProperty(szPanelCreature, "raceList3", "selected", 1);
	} // endif
	var MonPrio	 = Creature.Info[MonIndex].Priority;
	var MonDangerous = Creature.Info[MonIndex].DangerFlag;

	if (debugSelect) {
	    Log.OutD("FindCloseCre: MonIndex [" + MonIndex + "]  " +
	    	     "MonLoc [" + Util.FormatCoords(MonLoc,3) + "] MonDist [" + MonDist + "]");
	    with (Creature.Info[MonIndex]) {
		Log.OutD("FindCloseCre: Priority [" + Priority + "] SelFlag [" + SelFlag + "] AttFlag [" + AttFlag + "]");
		Log.OutD("FindCloseCre: YieldFlag [" + YieldFlag + "] VulnFlag [" + VulnFlag +
			 "] DangerFlag [" + DangerFlag + "]");
	    } // endwith
	} // endif

	// if Monster's attack flag = never, reject
	if (Creature.Info[MonIndex].AttFlag == attNever) {
	    if (debugSelect) Log.OutD("FindCloseCre: Cre [" + MonIdHex + "/" + MonName + "] attflag = never. Rejected");
	    continue; // goto next creature
	} // endif

	// if Monster's attack flag = auto, then check race attack setting
	if ((Creature.Info[MonIndex].AttFlag == attAuto) &&
	    (Creature.RaceInfo[Creature.Info[MonIndex].RaceCode].AttFlag == attNever)) {
	    if (debugSelect) Log.OutD("FindCloseCre: Race [" + MonIdHex + "/" + MonName + "] attflag = never. Rejected");
	    continue; // goto next creature
	} // endif

	// Monster passed all the self verification tests.  Now compare
	// it to best candidate so far to see if it is better
	if (SelAco == null) {
	    // No current selection so this one is now the leading candidate
	    if (debugSelect) Log.OutD("FindCloseCre: No prior candidate, new leader.");
	} else {
	    // If current leader is not locked, but this candidate is then reject it
	    if (!SelLocked && MonLocked) {
		if (debugSelect) Log.OutD("FindCloseCre: Candidate locked, current leader is not, rejected.");
		continue;
	    } // endif

	    // If current leader is locked, but candidate is not, then we have a new leader.
	    if (SelLocked && !MonLocked) {
		if (debugSelect) Log.OutD("FindCloseCre: Candidate not locked, current leader is, new leader.");
	    } else {
	        // If the creature we are examining is not dangerous, but the current winning
	        // candidate is, then reject it
	        if (!flagSelByDist && !MonDangerous && SelDangerous) {
		    if (debugSelect)
		        Log.OutD("FindCloseCre: Creature [" + MonIdHex + "/" + MonName +
	            	         "] not dangerous, previous leader is, rejected.");
		    continue;
	        } // endif

	        // If currently selected creature is dangerous but current leader is not, then
	        // we have a new leader
	        if (!flagSelByDist && MonDangerous && !SelDangerous) {
		    if (debugSelect)
		        Log.OutD("FindCloseCre: Creature [" + MonIdHex + "/" + MonName +
	            	             "] is dangerous, previous leader not, new leader.");
	        } else {
	    	    // Both have the same danger rating, check priority
	            if (!flagSelByDist && (MonPrio < SelPrio)) {
	                if (debugSelect)
	                    Log.OutD("FindCloseCre: Creature [" + MonIdHex + "/" + MonName +
	            	             "] has lower priority than current leader, rejected");
	                continue;
	            } // endif

	            if (!flagSelByDist && (MonPrio > SelPrio)) {
	                if (debugSelect)
	                    Log.OutD("FindCloseCre: Creature [" + MonIdHex + "/" + MonName +
	            	             "] has higher priority than current leader, new leader");
		    } else {
		        // Both have same priority, distance is final factor
		        if (MonDist >= SelDist) {
			    if (debugSelect)
			        Log.OutD("FindCloseCre: Creature [" + MonIdHex + "/" + MonName +
				         "] greater or equal distance then current leader, rejected");
			    continue; // goto next creature
			} else {
			    if (debugSelect)
				Log.OutD("FindCloseCre: Creature [" + MonIdHex + "/" + MonName +
				         "] distance less then current leader, new leader");
		        } // endif
	            } // endif
	        } // endif
	    } // endif
	} // endif

	// Passed all the tests, we have a new leader for the moment.
	if (debugSelect) Log.OutD("FindCloseCre: New leader");
	SelAco 	    	= MonAco;
	SelId	    	= MonId;
	SelPrio		= Creature.Info[MonIndex].Priority;
	SelDangerous	= Creature.Info[MonIndex].DangerFlag;
	SelExp		= Creature.Info[MonIndex].Exp;
	SelDist     	= MonDist;
	SelIndex   	= MonIndex;
	SelLocked	= MonLocked;
    } // endfor

    if (SelAco == null) {
    	if (debugSelect) Log.OutD("FindCloseCre: Unable to find creature to kill");
    	return false;	 // Couldn't find creature to kill
    } // endif

    if (debugSelect) Log.OutD("FindCloseCre: Candidate found [" + SelAco.szName + "] Id [" + Util.Hex(SelId)+ "]");
    return true;
} // end func


// Find a creature to kill.
function SelCreature() {
    if (debug||debugSelect) Log.OutD("SelCreature: Begin");

    // Get our current location
    var OurLoc = skapi.maplocCur;

    // Build monster list of all creatures within select range
    Acf.distMax = SelRange;
    MonsterList = Acf.CoacoGet();
    if (debugSelect) Log.OutD("SelCreature: Monster list generated, [" + MonsterList.Count + "] creatures found");

    // See if a creature is real close to us.   For creatures close to us, we select
    // dangerous creatures first, then by priority, then by distance
    if (FindCloseCre(selRealClose)) {
	if (debugSelect) Log.OutD("SelCreature: Found real close creature");
	return true;
    } else {
	if (debugSelect) Log.OutD("SelCreature: No close creature found");
    } // endif

    if (!AttackMode) {
	if (debugSelect) Log.OutD("SelCreature: Defend only mode active, aborting selection attempt.");
	return false;
    } // endif

   if (RetreatActive) {
	if (debugSelect) Log.OutD("SelCreature: Retreat mode active mode, aborting selection attempt.");
	return false;
    } // endif

    // Clear selection ACO
    SelAco = null;
    SelLocked = false;

    // No creatures found close to us.   For creatures far away we select by priority then by distance
    // Skip any creatures whose select flag is false or are on the Do Not Attack List (DNAL)
    for (var index=0; index<MonsterList.Count; index++) {
	var MonAco   = MonsterList.Item(index);
	var MonId    = MonAco.oid;
	var MonIdHex = Util.Hex(MonId);
	if (debugSelect) Log.OutD("SelCreature: Examining [" + MonAco.szName + "] Id [" + MonIdHex + "]");
	if (!MonAco.fExists) {
	    // Aco does not exist anymore (ie is a stale entry)
	    if (debugSelect) Log.OutD("SelCreature: fExists = false. Rejected");
	} // endif
	var MonName  = MonAco.szName;
	var MonIdent = MonName + "/" + MonIdHex;

	// Reject if monster outside of select range.  Although this shouldn't be necessary since
	// the monster collection is filtered on a maximum distance (which is the select range),
	// occasionally creatures outside the range are included in the collection.
	var MonLoc  = MonAco.maploc;
	var MonDist = Util.Distance(MonLoc,OurLoc);
	if (MonDist > SelRange) {
	    if (debugSelect) Log.OutD("SelCreature: Cre [" + MonIdent + "] not within select range. Rejected");
	    continue; // goto next creature
	} // endif

	// If we want to restrict selection of creatures by attitude, then check how far creature is above or below us
	if (flagSelCheckZ) {
	    if ((MonLoc.z - OurLoc.z) > attMaxHighZDiff) {
		if (debugSelect) Log.OutD("SelCreature: Creature too far above us, rejected");
		continue;
	    } // endif
	    if ((OurLoc.z - MonLoc.z) > attMaxLowZDiff) {
		if (debugSelect) Log.OutD("SelCreature: Creature too far below us, rejected");
		continue;
	    } // endif
	} // endif

	// If this is the previous creature we were attacking, then reject.  We do this because
	// sometimes it takes a few seconds for a killed creature to be removed from the object table
	if ((MonId == AttCtx.PrevId) && ((new Date()-AttCtx.LastDone) < attPrevHoldDown)) {
	    if (debugSelect) Log.OutD("SelCreature: Creature was just attacked previously. Rejected");
	    continue; // goto next creature
	} // endif

	if (attSupportOnly) {
    	    if (AttDoneLookup(MonId)) {
		if (debugSelect) Log.OutD("SelCreature: (support) already done, rejected");
		continue;
	    } // endif
	} // endif

	// See if this creature is in the database.  If not then add it in and use race record for vuln default.
	var MonIndex = Creature.NameToIndex(MonName);
	if (MonIndex == 0) {
	    if (debugSelect||logSelect)
		Log.OutD("SelCreature: Creature [" + MonIdHex + "/" + MonName + "] unknown.  Adding...");
	    Chat.Screen("Unknown creature '" + MonName + "' detected, adding to database...", cmcYellow);
	    MonIndex = Creature.AddUnknownCre(MonAco);
	    if (MonIndex == 0) {
		// Couldn't add creature to database for some reason
		Log.OutDM("SelCreature","Couldn't add creature to database, skipping...", true);
		continue;	// goto next creature
	    } // endif
	    var RaceCode = Creature.Info[MonIndex].RaceCode;
	    Chat.Screen("Creature added to race '" + Creature.RaceInfo[RaceCode].RaceName + "/" +
	    		 RaceCode + "'.", cmcYellow);

	    // Reset decal panel for creatures to make sure new creature displayed
	    skapi.SetControlProperty(szPanelCreature, "raceList3", "selected", 0);
	    skapi.SetControlProperty(szPanelCreature, "raceList3", "selected", 1);
	} // endif
	var MonPrio = Creature.Info[MonIndex].Priority;

	// See if creature is on lock list, ie someone else is killing it.  If so then reject it
	if (ExclMode && (ExclFindLock(MonId))) {
	    if (debugSelect) Log.OutD("SelCreature: Creature locked, rejected");
	    continue;
	} // endif

	if (debugSelect) {
	    Log.OutD("SelCreature: MonIndex [" + MonIndex + "]  " + "MonLoc [" + Util.FormatCoords(MonLoc,3) +
	    	     "] MonDist [" + MonDist + "]");
	    Log.OutD("SelCreature: OurLoc [" + Util.FormatCoords(OurLoc,3) + "] SelRange [" + SelRange + "]");
	    with (Creature.Info[MonIndex]) {
	      Log.OutD("SelCreature: Priority [" + Priority + "] SelFlag [" + SelFlag + "] AttFlag [" + AttFlag + "]");
	      Log.OutD("SelCreature: YieldFlag [" + YieldFlag + "] VulnFlag [" + VulnFlag + "] DangerFlag [" + DangerFlag +"]");
	    } // endwith
	} // endif

	// If monster on Do Not Attack List, then reject
	if (SelBadId(MonId)) {
	    if (debugSelect) Log.OutD("SelCreature: Creature [" + MonIdent + "] on DNAL list. Rejected");
	    continue; // goto next creature
	} // endif

	// if Monster's select flag = never, reject
	if (Creature.Info[MonIndex].SelFlag == attNever) {
	    if (debugSelect) Log.OutD("SelCreature: Creature [" + MonIdent + "] sel flag = never. Rejected");
	    continue; // goto next creature
	} // endif

	// If Creatures select flag = attAuto, and Monster's race record select flag = never, reject
	if ((Creature.Info[MonIndex].SelFlag == attAuto) &&
	    (Creature.RaceInfo[Creature.Info[MonIndex].RaceCode].SelFlag == attNever)) {
	    if (debugSelect) Log.OutD("SelCreature: Creature [" + MonIdent + "] race select flag = never. Rejected");
	    continue; // goto next creature
	} // endif

	// If monsters experience is less than minimum allowed, then reject
	if (Creature.Info[MonIndex].Exp < selMinExp) {
	    if (debugSelect) Log.OutD("SelCreature: Creature [" + MonIdent + "] exp less than minmum. Rejected");
	    continue; // goto next creature
	} // endif

	// Passed all the self verification tests, now compare this candidate to the current leader
	if (SelAco == null) {
	    // No current selection so this one is now the leading candidate
	    if (debugSelect) Log.OutD("SelCreature: No prior candidate, new leader.");
	} else {
	    // if Monster's priority is less than current best candidate, then reject
	    if (!flagSelByDist && (MonPrio < SelPrio)) {
	        if (debugSelect) Log.OutD("SelCreature: Creature [" + MonIdent + "] Prio < leader. Rejected");
	        continue; // goto next creature
	    } // endif

	    // If priorities are equal, then closer monster wins
	    if (flagSelByDist || (MonPrio == SelPrio)) {
	        if (MonDist > SelDist) {
		    if (debugSelect) Log.OutD("SelCreature: Cre [" + MonIdent +"] Prios =, but further away. Rejected");
		    continue; // goto next creature
	        } //endif
	    } else {
	        if (debugSelect) Log.OutD("SelCreature: Creature [" + MonIdent + "] Prio > current leader, new leader");
	    } //endif
	} // endif

	// Passed all the tests, we have a new leader for the moment.
	if (debugSelect) Log.OutD("SelCreature: New leader");
	SelAco 	    	= MonAco;
	SelId	    	= MonId;
	SelPrio		= Creature.Info[MonIndex].Priority;
	SelDangerous	= Creature.Info[MonIndex].DangerFlag;
	SelExp		= Creature.Info[MonIndex].Exp;
	SelDist     	= MonDist;
	SelIndex   	= MonIndex;
    } // endfor

    if (SelAco == null) {
    	if (debugSelect) Log.OutD("SelCreature: Unable to find creature to kill");
    	return false;	 // Couldn't find creature to kill
    } // endif

    if (debugSelect) Log.OutD("SelCreature: Candidate found [" + SelAco.szName + "] Id [" + Util.Hex(SelId)+ "]");
    return true;
} // end function


function FindSecondary() {

    var SelAtt = new AttCreRecord();
    var SelPrio = 0;

    Acf.distMax = SelRange;
    MonsterList = Acf.CoacoGet();
    PrimLoc = Att.Aco.maploc;

    for (var x=0; x<MonsterList.Count; x++) {
	var MonAco   = MonsterList.Item(x);
	var MonId    = MonAco.oid;
	var MonIdHex = Util.Hex(MonId);
	if (debugSelect) Log.OutD("FindSecondary: Examining [" + MonAco.szName + "] Id [" + MonIdHex + "]");
	if (!MonAco.fExists) {
	    // Aco does not exist anymore (ie is a stale entry)
	    if (debugSelect) Log.OutD("FindSecondary: fExists = false. Rejected");
	} // endif
	var MonName  = MonAco.szName;
	var MonIdent = MonName + "/" + MonIdHex;

	// Reject if this is the primary target
	if (Att.Oid == MonId) {
	    if (debugSelect) Log.OutD("FindSecondary: Cre [" + MonIdent + "] is the primary target. Rejected");
	    continue; // goto next creature
	} // endif

	// Reject if monster not close to primary creature
	// the monster collection is filtered on a maximum distance (which is the select range),
	// occasionally creatures outside the range are included in the collection.
	var MonLoc  = MonAco.maploc;
	var MonDist = Util.Distance(MonLoc,PrimLoc);
	if (MonDist > selSecMaxDist) {
	    if (debugSelect) Log.OutD("FindSecondary: Cre [" + MonIdent + "] not within select range. Rejected");
	    continue; // goto next creature
	} // endif

	// If this is the previous creature we were attacking, then reject.  We do this because
	// sometimes it takes a few seconds for a killed creature to be removed from the object table
	if ((MonId == AttCtx.PrevId) && ((new Date()-AttCtx.LastDone) < attPrevHoldDown)) {
	    if (debugSelect) Log.OutD("FindSecondary: Creature was just attacked previously. Rejected");
	    continue; // goto next creature
	} // endif

	if (attSupportOnly) {
    	    if (AttDoneLookup(MonId)) {
		if (debugSelect) Log.OutD("SelCreature: (support) already done, rejected");
		continue;
	    } // endif
	} // endif

	// See if this creature is in the database.  If not then skip it
	var MonIndex = Creature.NameToIndex(MonName);
	if (MonIndex == 0) {
	    if (debugSelect||logSelect) Log.OutD("FindSecondary: Creature [" + MonIdHex + "/" + MonName + "] unknown.");
	    continue;
	} // endif
	var MonPrio = Creature.Info[MonIndex].Priority;

	// If monster not the same race as primary target then reject
	if (Creature.Info[MonIndex].RaceCode != Creature.Info[Att.CreIndex].RaceCode) {
	    if (debugSelect) Log.OutD("FindSecondary: Creature [" + MonIdent + "] on DNAL list. Rejected");
	    continue; // goto next creature
	} // endif

	// If creature not vulnerable to the current wand rending type then abort
	if ((attWandElement != Creature.Info[MonIndex].Vuln1) &&
	   (attWandElement != Creature.Info[MonIndex].Vuln2) &&
	   (attWandElement != Creature.Info[MonIndex].Vuln3)) {
	    if (debugSelect)
		Log.OutD("FindSecondary: Creature [" + MonIdent + "] not vulnerable to wand imbue. Rejected");
	    continue;
	} // endif

	// If monster on Do Not Attack List, then reject
	if (SelBadId(MonId)) {
	    if (debugSelect) Log.OutD("FindSecondary: Creature [" + MonIdent + "] on DNAL list. Rejected");
	    continue; // goto next creature
	} // endif

	// if Monster's select flag = never, reject
	if (Creature.Info[MonIndex].SelFlag == attNever) {
	    if (debugSelect) Log.OutD("FindSecondary: Creature [" + MonIdent + "] sel flag = never. Rejected");
	    continue; // goto next creature
	} // endif

	// If monsters experience is less than minimum allowed, then reject
	if (Creature.Info[MonIndex].Exp < selMinExp) {
	    if (debugSelect) Log.OutD("FindSecondary: Creature [" + MonIdent + "] exp less than minmum. Rejected");
	    continue; // goto next creature
	} // endif

	// Passed all the self verification tests, now compare this candidate to the current leader.
	// All we care about is the priority of the monster
	if (SelAtt.Aco == null) {
	    // No current selection so this one is now the leading candidate
	    if (debugSelect) Log.OutD("FindSecondary: No prior candidate, new leader.");
	} else {
	    // if Monster's priority is less than current best candidate, then reject
	    if (MonPrio < SelPrio) {
	        if (debugSelect) Log.OutD("FindSecondary: Creature [" + MonIdent + "] Prio < leader. Rejected");
	        continue; // goto next creature
	    } // endif
	} // endif

	// Passed all the tests, we have a new leader for the moment.
	if (debugSelect) Log.OutD("FindSecondary: New leader");
	SelPrio	= Creature.Info[MonIndex].Priority;
	SelAtt.Aco	= MonAco;
	SelAtt.Oid	= MonId;
	SelAtt.CreIndex = MonIndex;
	SelAtt.CreDist	= MonDist;
	SelAtt.CreName	= MonAco.szName;
	SelAtt.CreExp   = Creature.Info[MonIndex].Exp;
    } // endfor

    if (SelAtt.Aco == null) {
    	if (debugSelect) Log.OutD("FindSecondary: Unable to find creature to kill");
    	return null;	 // Couldn't find creature to kill
    } // endif

    return SelAtt;
} // end func


// Select a target
function Target() {
    if (debug||debugTarget) Log.OutD("Target: Begin");

    // Set state to target and clear abort flag (set by events if we try to target a locked creature)
    StateChange(stateTarget);
    SelAbort = false;

    // Clear event queue to make sure we have most recent information
    if (debugTarget) Log.OutD("Target: Clearing event queue...");
    var EventCount = skapi.WaitEvent(0);
    if (debugTarget) Log.OutD("Target: Event queue cleared, [" + EventCount + "] events processed.");

    // See if abort flag set (through events) while trying to target creature
    if (SelAbort||(State!=stateTarget)) {
	if (debug||debugTarget||logTarget) Log.OutD("Target: Abort, new state [" + State + "/" + textState[State] + "]");
	return false;	// Don't change state.  Procedure who aborted us already did that
    } // endif

    // Now try to select the creature
    if (debugTarget) Log.OutD("Target: Trying to select creature...");
    var Status = TargetAco(SelAco);
    if (debugTarget) Log.OutD("Target: Select done, status [" + Status + "]");

    // See if abort flag set (through events) while trying to target creature
    if (SelAbort||(State!=stateTarget)) {
	if (debug||debugTarget||logTarget)
	    Log.OutD("Target: Target aborted, new state [" + State + "/" + textState[State] + "]");
	return false;	// Don't change state.  Procedure who aborted us already did that
    } // endif

    // If we successfully targeted the creature then...
    if (Status) {
	// If we are running in exclusion mode and target not already locked then lock it
	if (ExclMode) {
	    if (!SelLocked) {
		LockTarget(SelAco);
		if (debugTarget||logTarget) Log.OutD("Target: Target locked");
	    } else {
	    	if (debugTarget||logTarget) Log.OutD("Target: Target already locked, assisting co-macroer...");
	    } // endif
	} // endif
	if (debug||debugTarget||logTarget)
	    Log.OutD("Target: Creature [" + SelAco.szName + "/" + Util.Hex(SelId) + "] targeted, starting kill attempt");
	return true;
    } // endif

    // Creature could not be targeted. Add it to the DNAL if we want to maintain this list
    if (debug||debugTarget||logTarget) Log.OutD("Target: Unable to select ["+Util.Hex(SelAco.oid)+"/"+SelAco.szName+"]");
    if (attDNAL) {
    	SelNoAttackAdd(SelAco.oid);
	if (debugTarget||logTarget) Log.OutD("Target: Added to DNAL");
    } // endif

    // Set state to Select
    StateChange(stateSelect);
    return false;
} // end function


// Target selected, prepare for attack
function KillInit() {
    if (debug||debugAttack) Log.OutD("KillInit: Begin");

    // Set creature database attack parameters
    Att.CreVuln1	  = Creature.Info[Att.CreIndex].Vuln1;
    Att.CreVuln2	  = Creature.Info[Att.CreIndex].Vuln2;
    Att.CreVuln3	  = Creature.Info[Att.CreIndex].Vuln3;
    Att.CreMaxHealth	  = Creature.Info[Att.CreIndex].Health;
    Att.CreMaxMana	  = Creature.Info[Att.CreIndex].Mana;
    Att.CreMaxStamina	  = Creature.Info[Att.CreIndex].Stamina;
    Att.CreMagicD	  = Creature.Info[Att.CreIndex].MagicD;
    Att.CreExp		  = Creature.Info[Att.CreIndex].Exp;
    Att.CrePrio		  = Creature.Info[Att.CreIndex].Priority;
    Att.CreYieldFlag	  = Creature.Info[Att.CreIndex].YieldFlag;
    Att.CreVulnFlag	  = Creature.Info[Att.CreIndex].VulnFlag;
    Att.CreWarSpell	  = Creature.Info[Att.CreIndex].WarSpell;
    Att.CreNoDrainHealth  = Creature.Info[Att.CreIndex].NoDrainHealth;
    Att.CreNoDrainStamina = Creature.Info[Att.CreIndex].NoDrainStamina;
    Att.CreNoDrainMana	  = Creature.Info[Att.CreIndex].NoDrainMana;
    Att.CreNoStepDown	  = Creature.Info[Att.CreIndex].NoStepDown;
    Att.CreDangerFlag	  = Creature.Info[Att.CreIndex].DangerFlag;

    if (debugAttack) {
	Log.OutD("KillInit: Att.CreName           [" + Att.CreName           + "]");
	Log.OutD("KillInit: Att.Oid               [" + Util.Hex(Att.Oid)     + "]");
	Log.OutD("KillInit: Att.CreIndex          [" + Att.CreIndex          + "]");
	Log.OutD("KillInit: Att.CreDist           [" + Att.CreDist           + "]");
	Log.OutD("KillInit: Att.Locked            [" + Att.Locked            + "]");
	Log.OutD("KillInit: Att.CreVuln1          [" + Att.CreVuln1          + "]");
	Log.OutD("KillInit: Att.CreVuln2          [" + Att.CreVuln2          + "]");
	Log.OutD("KillInit: Att.CreVuln3          [" + Att.CreVuln3          + "]");
	Log.OutD("KillInit: Att.CreMaxHealth      [" + Att.CreMaxHealth      + "]");
	Log.OutD("KillInit: Att.CreMaxMana        [" + Att.CreMaxMana        + "]");
	Log.OutD("KillInit: Att.CreMaxStamina     [" + Att.CreMaxStamina     + "]");
	Log.OutD("KillInit: Att.CreMagicD	  [" + Att.CreMagicD         + "]");
	Log.OutD("KillInit: Att.CreExp            [" + Att.CreExp            + "]");
	Log.OutD("KillInit: Att.CrePrio           [" + Att.CrePrio           + "]");
	Log.OutD("KillInit: Att.CreYieldFlag      [" + Att.CreYieldFlag      + "]");
	Log.OutD("KillInit: Att.CreVulnFlag       [" + Att.CreVulnFlag       + "]");
	Log.OutD("KillInit: Att.CreWarSpell       [" + Att.CreWarSpell       + "]");
	Log.OutD("KillInit: Att.CreNoDrainHealth  [" + Att.CreNoDrainHealth  + "]");
	Log.OutD("KillInit: Att.CreNoDrainStamina [" + Att.CreNoDrainStamina + "]");
	Log.OutD("KillInit: Att.CreNoDrainMana    [" + Att.CreNoDrainMana    + "]");
	Log.OutD("KillInit: Att.CreNoStepDown     [" + Att.CreNoStepDown     + "]");
	Log.OutD("KillInit: Att.CreDangerFlag	  [" + Att.CreDangerFlag     + "]");
    } // endif

    // Initialize variables that figure out if creature is moving or not.
    // May effect what war spell to use (bolt or arc)
    Att.CurrLoc		= Att.Aco.maploc;
    Att.PrevLoc		= Att.Aco.maploc;

    // Copy creatures yield, imperl, and vuln flag to temporary variables.  We may set these to
    // attNever in certain cases (see below)
    Att.YieldFlag	= Att.CreYieldFlag;
    Att.VulnFlag	= Att.CreVulnFlag;
    Att.ImperlFlag	= attAlwaysImperl;
    
    // If flag is set to always vuln (useful in support situations), then force a vuln for this creature
    if (attAlwaysVuln) Att.VulnFlag = attAlways;
    
    // If support mode enabled and always vuln not checked, then 
    // no sense doing a vuln since we will never war it.
    if (attSupportOnly && !attAlwaysVuln) Att.VulnFlag = attNever;

    // Miscellaneous
    Att.FailStart	= CastFailCnt;	// Save count of spell failure resets at start of kill attempt
    AttCtx.LastLostSel	= 0;		// Id of last creature we lost lock on
    AttCtx.Abort	= false;

    // If we have not specified a war spell type for this creature, then use the global default
    if (Att.CreWarSpell == attSpellAuto) {
    	if (debugAttack) Log.OutD("KillInit: No war spell specified for this creature, using default");
    	Att.WarSpell = attSpell;
    } else {
    	Att.WarSpell = Att.CreWarSpell;
    } // endif

    // If user has disabling yielding globally, then disable it for this attack
    if (flagYieldDisable) {
	if (debugAttack) Log.OutD("KillInit: User disabled yielding.");
	Att.YieldFlag = attNever;
    } // endif

    // If resistance vuln skip enabled and we have a resistance wand equiped, then see
    // if our wand imbue matches any of the creatures vulnerabilities, otherwise use
    // the primary vuln as our attack element.  Note that if the creature's vuln flag
    // is always or never, then we don't do this check.  The flag must be 'auto' for
    // this to happen
    Att.Element = Att.CreVuln1;
    if (attUseWandImbue	&& (attWandElement > 0) && (Att.VulnFlag == attAuto)) {
    	// See if wand element matches primary, secondary, or tertiary vuln types. Primary
    	// vuln is always checked, secondary/Tertiary vulns only checked if enabled.
    	if ((attWandElement == Att.CreVuln1) ||
    	    (attMatchVuln2 && (attWandElement == Att.CreVuln2)) ||
    	    (attMatchVuln3 && (attWandElement == Att.CreVuln3))) {
	    // Match found. Disable vuln and set creatures element to wand imbue type. Also need to figure out
	    // the equivalent vuln level for the wand imbue so stepdown routines can estimate a damage value;
	    if (debugAttack) Log.OutD("KillInit: Creature vulnerable to wand imbue, disabling vuln");
	    Att.VulnFlag = attNever;
	    Att.VulnActive = true;	// Even though we did not cast a vuln, the wand imbue behaves the same
	    Att.Element = attWandElement;
	    var EstVulnRatio = CharCurWar/160;

	    // Damage.VulnInc = new Array(0,10,25,50,75,100,150,185); // 1.1, 1.25, 1.5, 1.75, 2.0, 2.5, 2.85
	    if (EstVulnRatio > 2.3) {
	    	Att.VulnLevel = 6;
	    } else {
	    	if (EstVulnRatio > 1.9) {
		    Att.VulnLevel = 5;
		} else {
		    if (EstVulnRatio > 1.65) {
		    	Att.VulnLevel = 4;
		    } else {
		    	if (EstVulnRatio > 1.4) {
			    Att.VulnLevel = 3;
			} else {
			    if (EstVulnRatio > 1.2)
			    	Att.VulnLevel = 2;
			    else
			    	Att.VulnLevel = 1;
			} // endif
		    } // endif
		} // endif
	    } // endif
	} // endif
    } // endif

    // Set maximum spell level to use for different spell types based on the highest level
    // spell in our spellbook and the highest spell we are allowed to cast. Actual level
    // dependent on our current magic skill levels, spells we know, and and the health of the
    // creature.  Note since some of the max and min constants can be adjusted through the
    // command interface, we compute these values each creature kill attempt
    Att.MaxBolt   = Spell.Info[Att.Element].BoltStepDown[maxWarSpell];	// Maximum war bolt spell
    Att.MinBolt   = Spell.Info[Att.Element].BoltStepUp[minWarSpell];	// Minimum war bolt spell.
    Att.MaxArc    = Spell.Info[Att.Element].ArcStepDown[maxWarSpell];	// Maximum war arc spell.
    Att.MinArc    = Spell.Info[Att.Element].ArcStepUp[minWarSpell];	// Minimum war arc spell.
    Att.MaxYield  = Spell.YieldStepDown[maxYieldSpell];			// Maximum yield spell.
    Att.MaxImperl = Spell.ImperlStepDown[maxImperlSpell];		// Maximum imperl spell.
    Att.MaxVuln   = Spell.Info[Att.Element].VulnStepDown[maxVulnSpell];	// Maximum vuln spell

    // If we don't know any war spells for this damage type within the mininum and
    // maximum war spell range set by the user then abort the macro.
    NoBolt = (Att.MaxBolt < minWarSpell);
    NoArc  = (Att.MaxArc < minWarSpell);
    if (((Att.WarSpell == attSpellBolt) && NoBolt) ||
	((Att.WarSpell == attSpellArc) && NoArc) ||
	((Att.WarSpell == attSpellAuto) && NoBolt && NoArc)) {
    	Log.OutD("You do not know any bolt or arc spells for element [" + damName[Att.Element] + " in the range", true);
    	Log.OutD("specified by the min [" + minWarSpell + "] and max [" + maxWarSpell + "] war spell levels.", true);
    	Log.OutD("Macro is aborting...", true);
	StateChange(stateExit,NoCheck);
    	return;
    } // endif

    // When enabled in the decal command console ,we will cast lvl 7 vuln and war spells for the specified interval
    if (AttCtx.Lvl7Override) {
    	if ((new Date()-AttCtx.Lvl7OverStart) > AttCtx.Lvl7OverInt) {
	    AttCtx.Lvl7Override = false;
	    if (debugAttack) Log.OutD("DoState_KillInit: AttCtx.Lvl7Override expired");
	} else {
	    if (debugAttack) Log.OutD("DoState_KillInit: Lvl7Override active, setting max vuln & war spells to lvl 7");
	    Att.MaxBolt  = Math.min(Spell.Info[Att.Element].MaxBolt, 7);
	    Att.MaxArc   = Math.min(Spell.Info[Att.Element].MaxArc,  7);
	    Att.MaxVuln  = Math.min(Spell.Info[Att.Element].MaxVuln, 7);
	} // endif
    } // endif

    // Assume we can cast the maximum level for each spell type for now.  These variables may be adjusted
    // when our magic skill levels and the health of the creature we are attacking are examined.
    Att.CurVuln   = Att.MaxVuln;
    Att.CurBolt   = Att.MaxBolt;
    Att.CurArc    = Att.MaxArc;

    if (debugAttack) {
	Log.OutD("KillInit: Att.Element   [" + Att.Element   + "]");
	Log.OutD("KillInit: Att.MaxBolt   [" + Att.MaxBolt   + "]");
	Log.OutD("KillInit: Att.MinBolt   [" + Att.MinBolt   + "]");
	Log.OutD("KillInit: Att.MaxArc    [" + Att.MaxArc    + "]");
	Log.OutD("KillInit: Att.MinArc    [" + Att.MinArc    + "]");
	Log.OutD("KillInit: Att.MaxVuln   [" + Att.MaxVuln   + "]");
	Log.OutD("KillInit: Att.MaxYield  [" + Att.MaxYield  + "]");
	Log.OutD("KillInit: Att.MaxImperl [" + Att.MaxImperl + "]");
    } // endif

    // If creature's magicd is unknown, then see if we should yield or not
    if ((Att.YieldFlag == attAuto) && (Att.CreMagicD == 0)) {
    	if (flagYieldNoMagD) {
	    // Yield desired in this case.  Set the creatures magicd to a high value to force the yield
	    if (debugAttack) Log.OutD("KillInit: Creature's magicd unknown, forcing yield.");
	    Att.CreMagicD = 9999;
	} else {
	    // Yield not wanted when magic d of creature not known
	    if (debugAttack) Log.OutD("KillInit: Creature's magicd unknown, yield disabled.");
	    Att.YieldFlag = attNever;
	} // endif
    } // endif

    // If creature being attacked is locked by another player, then don't yield, imperl, or vuln.
    // We let the other player take care of that.
    if (Att.Locked) {
	Att.YieldFlag  = attNever;
	Att.VulnFlag   = attNever;
	Att.ImperlFlag = false;
	if (debugAttack) Log.OutD("KillInit: Target locked, no yield/vuln will be done.");
    } // endif

    if ((Att.YieldFlag != attNever) || (Att.VulnFlag != attNever) || Att.ImperlFlag) {
	// See if this creature has an active yield, imperl, or vuln.  If true, then set either
	// Att.YieldFlag and/or Att.VulnFlag to attNever, and Att.ImperlFlag to false.  This situation
	// happens when we abort processing on a creature (several possible reasons for this) and then
	// reselect them within a few minutes (lvl 6 debuff lasts for 6.5 minutes)
 	if (AttPrevLookup(Att.Oid)) {
	    // See if yield still active
	    if ((Att.YieldFlag != attNever) && (Att.PrevYieldExp != 0)) {
	        if (new Date(Att.PrevYieldExp) > new Date()) {
		    Att.YieldFlag = attNever;
		    if (debugAttack) Log.OutD("KillInit: Previous yield active, no yield needed");
	        } else {
	    	    if (debugAttack) Log.OutD("KillInit: Previous yield expired");
	        } // endif
	    } else {
		if (debugAttack) Log.OutD("KillInit: No previous yield found");
	    } // endif

	    // See if imperl still active
	    if (Att.ImperlFlag && (Att.PrevYieldExp != 0)) {
	        if (new Date(Att.PrevYieldExp) > new Date()) {
		    Att.YieldFlag = attNever;
		    if (debugAttack) Log.OutD("KillInit: Previous yield active, no yield needed");
	        } else {
	    	    if (debugAttack) Log.OutD("KillInit: Previous yield expired");
	        } // endif
	    } else {
		if (debugAttack) Log.OutD("KillInit: No previous yield found");
	    } // endif

	    // See if vuln still active
	    if ((Att.VulnFlag != attNever)&& (Att.PrevVulnExp != 0)) {
	        if (new Date(Att.PrevVulnExp) > new Date()) {
		    Att.VulnFlag = attNever;
		    Att.VulnActive = true;
		    Att.VulnLevel  = Att.PrevVulnLvl;	// Restore vuln level so war spell stepdown routines
	    						// can estimate how much damage we will do
		    if (debugAttack)
			Log.OutD("DoState_KillInit: Previous vuln lvl [" + Att.VulnLevel + "] active, no need for vuln");
	        } else {
	    	    if (debugAttack) Log.OutD("KillInit: Previous vuln expired");
	        } // endif
	    } else {
	        if (debugAttack) Log.OutD("KillInit: No previous vuln found");
	    } // endif
 	} else {
    	    // No previous yield or vuln active
    	    if (debugAttack) Log.OutD("KillInit: No previous yield or vuln detected");
	} // endif
    } // endif

    // Kill setup finished.  Change state to Kill, compute the first thing to do and do the deed
    Att.StartTime = new Date();	// Time we started attack on currently selected creature.
    StateChange(stateKill);

    // Setup all done, go try to yield first
    SubState = stateKillYield;

    if (debug||debugAttack) Log.OutD("KillInit: End");
    return;
} // end function KillInit


function UpdateMagicSkills() {
    if (debug) Log.OutD("UpdateMagicSkills: Begin");
    skWar	= skapi.SkinfoFromSkid(skidWarMagic);
    skLife	= skapi.SkinfoFromSkid(skidLifeMagic);
    skCre	= skapi.SkinfoFromSkid(skidCreatureEnchantment);

    CharCurWar	= skWar.lvlCur;
    CharCurLife	= skLife.lvlCur;
    CharCurCre	= skCre.lvlCur;

    if (debug) Log.OutD("UpdateMagicSkills: End");
} // end function UpdateMagicSkills

function UpdateVitalVars() {
    if (debug) Log.OutD("UpdateVitalVars: Begin");

    // Get secondary attribute information
    skHealth  = skapi.SkinfoFromVital(vitalHealthMax);
    skStamina = skapi.SkinfoFromVital(vitalStaminaMax);
    skMana    = skapi.SkinfoFromVital(vitalManaMax);

    // Extract current vital maximums (includes debuffs/buffs)
    CharMaxHealth  = skHealth.lvlCur;
    CharMaxStamina = skStamina.lvlCur;
    CharMaxMana	   = skMana.lvlCur;

    // Compute vital recharge thresholds
    ThreshBigHeal = threshBigHeal;   		      	  // When health falls below this, then cast stam-to-health
    ThreshHeal	  = CharMaxHealth  - threshHealOffset; 	  // When health falls below this, then heal self
    ThreshMana	  = CharMaxMana    - threshManaOffset;	  // When mana falls below this, then stam-to-mana
    ThreshStamina = CharMaxStamina - threshStaminaOffset; // When stamina falls below this, then revitalize

    // Set Drain Health/Mana/Stamina thresholds.  When one of these gets too low and we are
    // attacking a creature, then drain its health if we are in range
    ThreshDrainStamina = CharMaxStamina - threshDrainStaminaOffset;
    ThreshDrainMana    = CharMaxMana    - threshDrainManaOffset;
    ThreshDrainHealth  = CharMaxHealth  - threshDrainHealthOffset;

    if (debug4) {
	Log.OutD("UpdateVitalVars: CharMaxHealth [" + CharMaxHealth +
				"] CharMaxMana [" + CharMaxMana +
				"] CharMaxStamina [" + CharMaxStamina + "]" );
	Log.OutD("UpdateVitalVars: ThreshHeal [" + ThreshHeal +
    				"] ThreshBigHeal [" + ThreshBigHeal +
    				"] ThreshMana [" + ThreshMana +
    				"] ThreshStamina [" + ThreshStamina + "]" );
    	Log.OutD("UpdateVitalVars: ThreshDrainStamina [" + ThreshDrainStamina +
    				"] ThreshDrainMana [" + ThreshDrainMana +
    				"] ThreshDrainHealth [" + ThreshDrainHealth + "]" );
    } // endif

    if (debug) Log.OutD("UpdateVitalVars: End");
} // end function

function LifeVulnActive() {
    if (debug||debugVitals) Log.OutD("VulnActive: Begin");

    // Assume no vuln active at first
    var Active = false;

    // For each damage type (8=imperl) ...
    for (x=1; x<9; x++) {
    	// for each spell level (1-7) ...
    	for (y=1; y<8; y++) {
	    for (var z=0; z < skapi.cospell.Count; z++) {
		if (skapi.cospell(z).spellid == Damage.Info[x].Vuln[y]) {
		    // Found an active vuln
		    if (debugVitals) Log.OutD("VulnActive: Vuln found [" + skapi.cospell(z).szName + "]");
		    Active = true;
		} // endif
	    } // end for
	} // end for
    } // end for

    if (debug||debugVitals) Log.OutD("VulnActive: End, Active [" + Active + "]");
    return Active;
} // end function

// Don't user MD_PLAYEROBJ logoff function since we may need to integrate sound alert
// with logout action

function WB_PlayerLogOff(PlaySound) {
    if (debugLog) Log.OutD("WB_PlayerLogOff: Begin");

    if (debugLog) Log.OutD("WB_PlayerLogOff: Sending logout shortcut...");
    var LogoffStart = new Date();	// Init timer for logoff attempt

    // Logout.  Although we may have already started a logout
    // attempt in the ObjPlayerCre event, we do it again just in case.
    // For a landblock reset this will be the first one
    skapi.Logout();
    
    // Sound alarm if desired
    if (alertStealth && PlaySound) alertStealthWav.Play();

    // Now make sure we are completely logged out
    if (debugLog) Log.OutD ("WB_PlayerLogOff: Waiting for char to logout...");
    while ((skapi.plig != pligAtLogin) && ((new Date()-LogoffStart) < logoutTimeout))
   	skapi.WaitEvent(40);

    if (debugLog) Log.OutD("WB_PlayerLogOff: AtLogin wait done");

    if (skapi.plig == pligAtLogin) {
	if (debugLog) Log.OutD("WB_PlayerLogOff: End, Toon successfully logged out");
	return true;
    } // endif

    if (debugLog) Log.OutD("WB_PlayerLogOff: End, Timeout" );
    return false;
} // end func


// Function returns True when vitals ok, false when some action was taken
function CheckVitals() {
    if (debug||debugVitals) Log.OutD("CheckVitals: Begin");

    if (flagCheckVitals == false) {
	if (debugVitals) Log.OutD("CheckVitals: CheckVitals disabled, aborting");
	return true;
    } // endif

    // If we are trying to loot a corpse or ust salvage, then hold off on vital management.
    if ((State==stateLoot) || (State==stateSalvage)) {
	if (debugVitals) Log.OutD("CheckVitals: Trying to loot corpses or salvage items, check aborted");
	return true;
    } // endif

    if (debugVitals) {
	Log.OutD("CheckVitals: Health  = " + skapi.healthCur  );
 	Log.OutD("CheckVitals: Stamina = " + skapi.staminaCur );
 	Log.OutD("CheckVitals: Mana    = " + skapi.manaCur    );
    } // endif

    // Clear out events which will make sure we have the most recent vitals info
    skapi.WaitEvent(0);

    if ((skapi.healthCur == 0) && (skapi.manaCur == 0) && (skapi.manaCur == 0)) {
    	if (debugVitals) Log.OutD("CheckVitals: All stats 0, aborting check");
    	return true;
    } // endif

    // If constant 'flagGem' is set to true, a creature is within our danger radius, and
    // health/mana/stamina are all low, then see if we should use a gem of stillness
    if (flagGem && Danger && (skapi.healthCur < gemHealthThresh) &&
        (skapi.manaCur <gemManaThresh) && (skapi.manaCur < gemStaminaThresh)) {
	// See if last gem use was at least 'gemTimeInt' milliseconds ago
	if ((new Date()-CharLastGem) > gemTimeInt) {
	    // Last gem usage was long enough ago, now see if we have any life vulns on us
	    if (LifeVulnActive()) {
		// Ok, now see if we have any gems.  If so then use one
		var GemAco = skapi.AcoFindInInv("Gem of Stillness");
		if (GemAco != null) {
		    if (logVitals||debugVitals) Log.OutD("CheckVitals: Using gem of Stillness");
		    GemAco.Use();
		    CharLastGem = new Date();
		} else {
		    // Out of gems, report condition
		    if (logVitals||debugVitals) Log.OutD("CheckVitals: Out of gems");
		} // endif
	    } // endif
	} // endif
    } // endif

    // If stamina is really low, then cast revitalize self no matter what our current health
    // or mana level is
    if (skapi.staminaCur < threshStaminaEmer) {
	if (debugVitals||logVitals)
	    Log.OutD("CheckVitals: Emer, Mana/Stam [" + skapi.manaCur + "/" + skapi.staminaCur + "] Casting Revit Self");
	Cast.Cast(SpellObj_Revitalize, null, true, true);
	if (CastCode != castCodeSuccess) {
	    Log.OutD("CheckVitals: Emer Revitalize failed. " + SpellFailText());
	} else {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Revit success, Arc [" + CastArc + "]");
	} // endif
    } // endif

    // If mana is really low, then cast stam to mana no matter what our current health or stamina level is
    if (skapi.manaCur < threshManaEmer) {
	if (debugVitals||logVitals)
	    Log.OutD("CheckVitals: EmerMana, Mana/Stam [" + skapi.manaCur + "/" + skapi.staminaCur + "] Casting Stam2Mana");
	Cast.Cast(SpellObj_StamMana, null, true, true);
	if (CastCode != castCodeSuccess) {
	    Log.OutD("CheckVitals: Emer Stam2Mana failed. " + SpellFailText());
	} else {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Stam2Mana success, Arc [" + CastArc + "]");
	} // endif
    } // endif

    // If players health below ThreshBigHeal, then cast stam-2-health (if enough stamina).
    // If retreating enabled, then just return because the retreat routine will cast stam2health
    // before retreating.
    if (skapi.healthCur < ThreshBigHeal) {
    	if (debugVitals||logVitals)
		Log.OutD("CheckVitals: Player Health [" + skapi.healthCur + "] below big heal thresh.");
    	// Set retreat required flag if low health retreating enabled, we are not currently retreating,
    	// and this attack location has a retreat point set
	if (flagRetreatCrit && flagHealthRetreat && !RetreatActive && !RetreatReq &&
	    !Dead && (AttLoc[AttLocCur].RetreatLoc!=null)) {
	    if (logVitals||debugVitals) Log.OutD("CheckVitals: Setting RetreatReq flag.");
	    Chat.Screen("Health low, retreating...");
	    RetreatReq = true;
	    return false;
	} // endif 
    
	if (skapi.staminaCur > threshBigHealStam) {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Casting Stam2Health");
	    Cast.Cast(SpellObj_StamHealth, null, true, true);
	    if (CastCode == castCodeSuccess) {
	    	if (debugVitals||logVitals) Log.OutD("CheckVitals: Stam2Health success, Arc [" + CastArc + "]");
		return false;
	    } // endif
	    // Spell failed, don't return back to FSM because a heal self spell can still be cast in next section
	    Log.OutD("CheckVitals: Stam2Health failed. " + SpellFailText());
	} else {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Not enough stamina to cast Stam2Health");
	} // endif
    } // end if

    // If player health below Heal_Thresh then heal ourselves.  Note when we are trying to
    // kill a creature we reduce ThreshHealth slightly
    if (((skapi.healthCur < ThreshHeal) && (State!=stateKill)) ||
     	((skapi.healthCur < (ThreshHeal-threshHealKillOffset)) && (State==stateKill))) {
	if (debugVitals||logVitals)
	    Log.OutD("CheckVitals: Player health [" + skapi.healthCur + "] below heal thresh, casting healself");
	    
	// If our heal spell is set to lvl 6 and our health is below the heal step up threshold, then use a
	// lvl 7 spell instead
	if ((spellHeal == 6) && (skapi.healthCur < healStepUpThresh)) {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Heal self stepped up to lvl 7 ");
	    Cast.Cast(SpellObj_Heal7, null, true, true);
	} else {
	    Cast.Cast(SpellObj_Heal, null, true, true);
	} // endif

	if (CastCode != castCodeSuccess) {
	    Log.OutD("CheckVitals: Heal self failed. " + SpellFailText());
	} else {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Heal self success, Arc [" + CastArc + "]");
	} // endif
	return false;
    } // endif

    // If mana is low and stamina high enough, then cast stam to mana.  Note when creatures
    // nearby or we are trying to kill a creature we reduce ThreshMana slightly
    if ((((skapi.manaCur < ThreshMana) && (!Danger&&(State!=stateKill))) ||
         ((skapi.manaCur < (ThreshMana-threshManaKillOffset)) && (Danger||(State==stateKill)))) &&
        (skapi.staminaCur > threshStaminaMin)) {
	if (debugVitals||logVitals)
	    Log.OutD("CheckVitals: Mana/Stam [" + skapi.manaCur + "/" + skapi.staminaCur + "] Casting Stam2Mana");
	Cast.Cast(SpellObj_StamMana, null, true, true);
	if (CastCode != castCodeSuccess) {
	    Log.OutD("CheckVitals: Stam2Mana failed. " + SpellFailText());
	} else {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Stam2Mana success, Arc [" + CastArc + "]");
	} // endif
	return false;
    } // endif

    // If player stamina below ThreshStamina then stam ourselves.  Note when creatures
    // nearby or we are trying to kill a creature we reduce ThreshStamina slightly.
   if (((skapi.staminaCur < ThreshStamina) && (!Danger&&(State!=stateKill))) ||
       ((skapi.staminaCur < (ThreshStamina-threshStamKillOffset)) && (Danger||(State==stateKill))))  {
	if (debugVitals||logVitals)
	    Log.OutD("CheckVitals: Mana/Stam [" + skapi.manaCur + "/" + skapi.staminaCur + "] Casting Revitalize Self");
	Cast.Cast(SpellObj_Revitalize, null, true, true);
	if (CastCode != castCodeSuccess) {
	    Log.OutD("CheckVitals: Revit failed. " + SpellFailText());
	} else {
	    if (debugVitals||logVitals) Log.OutD("CheckVitals: Revit success, Arc [" + CastArc + "]");
	} // endif
	return false;
    } // endif

    if (debug||debugVitals) Log.OutD("CheckVitals: End");
    return true;
} // end function


function CorpseLookup(Aco) {
    if (debug||debugLoot) Log.OutD("CorpseLookup: Begin, Aco Name [" + Aco.szName + "] Oid [" + Util.Hex(Aco.oid) + "]" );

    // See if Aco exists in corpse list
    for (x=0; x<CorpseCount; x++) {
    	if (CorpseList[x].Aco == null) continue;

    	if (CorpseList[x].Aco.oid == Aco.oid) {
	    if (debugLoot) Log.OutD("CorpseLookup: End, Aco found");
	    return x;
	} // endif
    } // endwhile

    if (debug||debugLoot) Log.OutD("CorpseLookup: End, Aco Not Found" );
    return -1;
} // end function


function LootLookup(Aco) {
    if (debug||debugLoot) Log.OutD("LootLookup: Begin, Name [" + Aco.szName + "] Oid [" + Util.Hex(Aco.oid) +
    				   "] Loot Cnt [" + LootCount + "]");

    // See if Aco exists on item list
    for (x=0; x<LootCount; x++) {
    	if (LootList[x].Aco == null) continue;

    	if (LootList[x].Aco.oid == Aco.oid) {
	    if (debugLoot) Log.OutD("LootLookup: End, Aco found");
	    return x;
	} // endif
    } // endwhile

    if (debug||debugLoot) Log.OutD("LootLookup: End, Aco Not Found" );
    return -1;
} // end function LootLookup

// The following function determines if an item should be processed as loot.
function NoLootItem(Aco) {
    for (var y=0; y<protectedItems.length; y++) {
	if (Aco.szName.indexOf(protectedItems[y]) >= 0) {
	    if (debugLoot) Log.OutD("NoLootItem: Item on protected list, skipping...");
	    return true;
	} // endif
    } // endfor

    // If item is on one of the no-assess lists, then skip it
    for (var y=0; y<noassessItems.length; y++) {
	if (Aco.szName == noassessItems[y]) {
	    if (debugLoot) Log.OutD("NoLootItem: Item on noassess list, skipping");
	    return true;
	} // endif
    } // endfor

   for (var y=0; y<noassessPartial.length; y++) {
	if (Aco.szName.indexOf(noassessPartial[y]) >= 0) {
	    if (debugLoot) Log.OutD("NoLootItem: Item on partial noassess list, skipping");
	    return true;
	} // endif
    } // endfor
} // end func

// The following function determines if an item needs to be assessed or not
function NoAssessItem(Aco) {
    if ((Aco.eqm == 0) || (Aco.mcm & lootNoAssessMask)) {
    	// Don't need to assess non-equipment, bags of salvage, gems, food etc.
    	if (debugLoot) Log.OutD("NoAsessItem: No assess needed on this item (mcm filter)");
    	return true;
    } // endif

    return false;	// item needs to be assessed
} // ends function


function ReportComps(Who) {
    if (debug) Log.OutD("ReportComps: Begin, Who [" + Who + "]" );

    // For each component we are tracking, report inventory
    var HeadingReq = true;
    for (var x=0; x<Comp.Info.length; x++) {
    	if (!Comp.Info[x].Enabled) continue;

    	if (HeadingReq) {
	    HeadingReq = false;
            OutLine = Util.Width("Comp",1.7) + Util.Width("Stock",0.7) + Util.Width("Warn",0.7) +
                      Util.Width("Crit",0.7) + "PeaSplit";
            if ((Who==undefined) || (Who=="")) {
                // report to ourselves
                Chat.Screen(OutLine)
            } else {
                // tell whoever asked
                Chat.Tell(Who,"WB: " + OutLine);
            } // endif
	} // endif

	var Name     = Comp.Info[x].Name;
	var Stock    = skapi.CitemOfKindInInv(Name);
	var WarnLvl  = Comp.Info[x].WarnLvl;
	var CritLvl  = Comp.Info[x].CritLvl;
	var SplitLvl = Comp.Info[x].SplitLvl;

	var OutLine = Util.Width(Name,1.7) + Util.Width(Stock+" ",0.7) +
		      Util.Width(WarnLvl+" ",0.7) + Util.Width(CritLvl+" ",0.7) + SplitLvl;

	if ((Who==undefined) || (Who=="")) {
	    // report to ourselves
	    Chat.Screen(OutLine)
	} else {
	    // tell whoever asked
	    Chat.Tell(Who,"WB: " + OutLine);
	} // endif
    } // endfor

    if (debug) Log.OutD("ReportComps: End" );
    return;
} // endif

function CheckDanger() {
    if (debug||debugDanger) Log.OutD("CheckDanger: Begin");

    // If we are in support mode and don't want to vuln or imperl, then no
    // sense checking danger condition as we won't attack the creature anyway.
    if (attSupportOnly && !attAlwaysVuln && !attAlwaysImperl) {
	Danger   = false;
	Critical = false;
	Urgent   = false;
	return;
    } // endif

    // Create aco collection filter
    var acf = skapi.AcfNew();
    acf.Reset();
    acf.ocm = ocmMonster;
    acf.distMax = Math.max(criticalRadius, dangerRadius);
    var MonList = acf.CoacoGet();

    // Calculate number of creatures near us
    var DangerCnt = 0;
    var CritCnt   = 0;
    var RingCnt   = 0;
    var OurLoc = skapi.maplocCur;
    
    for (var x=0; x<MonList.Count; x++) {
    	MonAco = MonList.Item(x);

    	// If creature was last one killed, is being attacked by a co-macroer, or is on
    	// the Do Not Attack List, then exclude from counts
    	if (MonAco.oid == AttCtx.LastKillOid) continue;
    	if (ExclMode && (ExclFindLock(AttCtx.LastLostSel)) && (MonAco.oid == AttCtx.LastLostSel)) continue;
    	if (SelBadId(MonAco.oid)) continue;
    	
    	var MonDist = Util.Distance(OurLoc, MonAco.maploc);
    	
    	if (MonDist<=dangerRadius) DangerCnt++;
    	if (MonDist<=criticalRadius) CritCnt++;
    	if (MonDist<=attRingDist) RingCnt++;

    } // end for
 
    if (debugDanger) {
	Log.OutD("CheckDanger: # of creatures found [" + MonList.Count + "]");
	Log.OutD("CheckDanger: DangerCnt [" + DangerCnt + "] CritCnt [" + CritCnt + "]");
    } // endif

    Danger = (DangerCnt >= dangerCreThresh);
    if (DangerPrev != Danger) {
        if (Danger) {
            if (debugSelect||debugAttack||logDanger||debugDanger)
		Log.OutD("CheckDanger: Danger condition detected, count [" + DangerCnt + "] dangerCreThresh [" +
			 dangerCreThresh + "]");
        } else {
            if (debugSelect||debugAttack||logDanger) Log.OutD("CheckDanger: Danger condition cleared");
        } // endif
        DangerPrev = Danger;
    } // endif

    // See if there are several creatures real close to us.  If so then try to kill the creatures
    // as fast as possible, ie don't do buffs, self revitalizes, and drain mana/stam spells.
    Critical = (CritCnt >= CriticalAmount);
    if (CriticalPrev != Critical) {
	if (Critical) {
	    if (debugSelect||debugAttack||logDanger||debugDanger)
		Log.OutD("CheckDanger: Crit condition detected, CritCnt[" + CritCnt + "] CriticalAmount [" +
			 CriticalAmount + "]");
	} else {
	    if (debugSelect||debugAttack||logDanger||debugDanger) Log.OutD("CheckDanger: Critical condition cleared");
	} // endif
            CriticalPrev = Critical;
    } // endif

    Urgent = (CritCnt >= UrgentAmount);
    if (UrgentPrev != Urgent) {
	if (Urgent) {
	    if (debugSelect||debugAttack||logDanger||debugDanger)
		Log.OutD("CheckDanger: Urgent condition detected [" + CritCnt + "]");
	} else {
	    if (debugSelect||debugAttack||logDanger||debugDanger) Log.OutD("CheckDanger: Urgent condition cleared");
	} // endif
        UrgentPrev = Urgent;
    } // endif

    RingOk = (attUseRingSpells && (RingCnt >= attRingThresh));
    if (RingOk != RingOkPrev) {
	if (RingOk) {
	    if (logRing||debugRing||debugAttack) Log.OutD("CheckDanger: Ring condition detected [" + RingCnt + "]");
	} else {
	    if (logRing||debugRing||debugAttack) Log.OutD("CheckDanger: Ring condition cleared.");
	} // endif
	RingOkPrev = RingOk;
    } // endif

    if (debugDanger) {
    	if (Danger)   Log.OutD("CheckDanger: Danger condition active!");
	if (Critical) Log.OutD("CheckDanger: Critical condition active!");
	if (Urgent)   Log.OutD("CheckDanger: Urgent condition active!");
    } // endif
    
    if (debug||debugDanger) Log.OutD("CheckDanger: End");
} // end function

function AttCheckVulnFlag(CreCurHealth) {
    if (debug||debugAttack) Log.OutD("AttCheckVulnFlag: Begin, CreCurHealth [" + CreCurHealth + "]");

    // See if a vuln is necessary.  If the health of our target creature is at a level
    // were it can be killed with one or two war spells, then no vuln necessary.  First
    // see if we can kill the creature with 1 spell
    if ((((Att.WarSpell == attSpellArc)  || (Att.WarSpell == attSpellAuto)) &&
         (CreCurHealth<=(Damage.WarBoltMin[Att.CurArc]+(Damage.WarBoltRange[Att.CurArc]*attOneWarWeight)))) ||
        (((Att.WarSpell == attSpellBolt) || (Att.WarSpell == attSpellAuto)) &&
	 (CreCurHealth<=(Damage.WarBoltMin[Att.CurBolt]+(Damage.WarBoltRange[Att.CurBolt]*attOneWarWeight))))) {
	// Creature's health is less than weighted damage we can cast with our highest war
	// spell.  If the weight value is 0 then we use the minimum value for the war spell
	// damage range and we will be 100% sure of killing creature with 1 war spell
	if (debugAttack) Log.OutD("AttCheckVulnFlag: Vuln not required, can kill with one war");
	return false;
    } // endif

    // Vuln required
    if (debug||debugAttack) Log.OutD("AttCheckVulnFlag: End, Vuln Required");
    return true;
} // end function

function LoadStats() {
    if (debug) Log.OutD("LoadStats: Begin" );

    var CurTime = new Date();
    ElapsedTimeSec  = (CurTime.getTime() - StartTime.getTime())/1000;
    ElapsedTimeMin  = Util.Round(ElapsedTimeSec/60,0);
    StatAveKillHour = Math.round((TotalKilled / ElapsedTimeSec) * 3600);
    StatEstExpHour  = Math.round((KillExp / ElapsedTimeSec) * 3600);
    StatActExp	    = skapi.expTotal-TotalExpStart;
    StatActExpHour  = Math.round((StatActExp / ElapsedTimeSec) * 3600);

    if (debug) Log.OutD("LoadStats: End" );
    return;
} // endif

function ResetStats() {
    StartTime = new Date();
    TotalExpStart = skapi.expTotal;
    TotalKilled	= 0;
    KillExp = 0;
} // end func


function ShowMoveStats() {
    
    for (var x=1; x<=AttLocCnt; x++) {
      with (AttLoc[x]) {
    	Chat.Screen(" ");
    	Chat.Screen("Location: " + x);

   	// For the current attack location we need to adjust the total time
    	// spent at the location by the time spent there for this visit
	var TimeInt = TotalInt;
    	if (x == AttLocCur) {
	    var CurTime = new Date();
	    var CurInt = CurTime.getTime() - LastArriveTime.getTime();
	    TimeInt += CurInt;
	    Chat.Screen("Current attack loc, arrived " + Util.FormatTime(LastArriveTime) +
	    		", been here " + Util.FormatMinSec(CurInt));
	} // endif

    	if (TimeInt == 0) {
	    Chat.Screen("Location not visited yet.");
	    continue;
	} // endif


    	Chat.Screen("Last Arrive: " + Util.FormatTime(LastArrive) + "  Depart: " + Util.FormatTime(LastDepart) +
    		    "  Int: " + Util.FormatMinSec(LastInt));
    	Chat.Screen("Total Visits: " + TotalVisits + "  Retreats: " + TotalRetreats +
    		    "  Int: " + Util.FormatMinSec(TimeInt));
    	Chat.Screen("Total Killed: " + TotalKilled + "  Exp: " + TotalExp + "  Looted: " + TotalLooted);

	// Print stat ratios
	var Secs = Math.round(TimeInt/1000);
	if (Secs == 0) {
	    var ExpHour  = 0;
	    var KillHour = 0;
	} else {
	    var ExpHour  = Math.round((TotalExp/Secs)*3600);
	    var KillHour = Math.round((TotalKilled/Secs)*3600);
	} // endif
	
	if (TotalKilled == 0)
	    var ExpKill = 0;
	else
	    var ExpKill = Math.round(TotalExp/TotalKilled);

    	Chat.Screen("Exp/Hour: " + ExpHour + "  Kill/Hour: " + KillHour + "  Exp/Kill: " + ExpKill);

      } // end with
    } // end for
} // end func

function ClearMoveStats() {
    for (var x=1; x<=AttLocCnt; x++) {
    	AttLoc[x].LastArrive    = 0;
    	AttLoc[x].LastDepart    = 0;
    	AttLoc[x].LastInt       = 0;
    	AttLoc[x].TotalVisits   = 0;
    	AttLoc[x].TotalRetreats = 0;
    	AttLoc[x].TotalInt      = 0;
    	AttLoc[x].TotalKilled   = 0;
    	AttLoc[x].TotalExp      = 0;
    	AttLoc[x].TotalLooted   = 0;
    } // end for
    LastArriveTime = new Date();
} // end func

function Authorized(Name) {
    if (debug) Log.OutD("Authorized: Begin" );
    // See if character 'Name' is an authorized person
    for (var index=0; index<authList.length; index++) {
	if (Name == authList[index]) {
	    if (debug) Log.OutD("Authorized: End, Success - On Authorized List" );
	    return true;
	} // endif
    } // endfor

    // See person is in our monarchy
    if (authAllegiance) {
    	// Get chars Aco from his/her name
    	var Aco = skapi.AcoFromSz(Name);
    	if (Aco && (Aco.oidMonarch == skapi.acoChar.oidMonarch)) {
    	    if (debug) Log.OutD("Authorized: End, Success - In monarchy" );
    	    return true;
    	} // endif
    } // endif

    if (debug) Log.OutD("Authorized: End, Failed" );
    return false;	// Not in our allegiance and not on the authorized list
} // end function

function EvalLoc() {
    if (debug||debugEval) Log.OutD("EvalLoc: Begin" );

    // Evaluate the current location for macroing.  Displays total # of creatures within kill
    // range (and their total experience points), and reports any hollow creatures nearby
    var EvalCount    = 0;
    var EvalExp      = 0;
    var EvalSelCount = 0;
    var EvalSelExp   = 0;

    Acf.distMax = SelRange;
    MonsterList = Acf.CoacoGet();
    for (var index=0; index<MonsterList.Count; index++) {
	var MonAco = MonsterList.Item(index);
	if (MonAco) {
	    var MonName  = MonAco.szName;
	    var MonIndex = Creature.NameToIndex(MonName);
	    if (MonIndex != 0) {
	    	var MonExp = Creature.Info[MonIndex].Exp;
	    	EvalCount++;
	    	EvalExp += MonExp;
		if (MonExp >= selMinExp) {
		    EvalSelCount++;
		    EvalSelExp += MonExp;
		} // endif

	    	if (debugEval)
		    Log.OutD("EvalLoc: Index [" + index + "] Cre [" + MonName + "] Exp [" + MonExp + "]", true);
	    } else {
	    	// Unknown monster
	        if (debugEval) Log.OutD("EvalLoc: Index [" + index + "] Cre [" + MonName + "] unknown");
	    } // endif
	} else {
	    if (debugEval) Log.OutD("Index [" + index + "] Aco is null");
	} // endif
    } // endfor

    // Compute average of creature
    if (EvalCount == 0)
	var EvalAve = 0;
    else
	var EvalAve = EvalExp / EvalCount;

    // Output creature statistics
    Chat.Screen("Found " + EvalCount + " creatures worth " + EvalExp + " experience points, average " +
    		Util.Round(EvalAve,0) + " points");
    if (debugEval)
	Log.OutD("Found " + EvalCount + " creatures worth " + EvalExp + " experience points, average " +
   		Util.Round(EvalAve,0) + " points");

    // If macroer has a minimum creature experience limit for attacking, then display monsters stats
    // for those creatures who exceed min exp requirement.
    if (selMinExp > 0) {
	if (EvalSelCount == 0)
	    var EvalSelAve = 0;
	else
	    var EvalSelAve = EvalSelExp / EvalSelCount;

	Chat.Screen("Minimum exp points for creature selection: " + selMinExp );
	Chat.Screen("Found " + EvalSelCount + " selectable creatures worth " + EvalSelExp +
		    " experience points, average " + Util.Round(EvalSelAve,0) + " points");
    } // endif

    return;
} // end function


function FindCreSummary() {
    if (debug||debugEval) Log.OutD("FindCreSummary: Begin" );

    // Find any creatures close by and report their distance, heading, and location
    var CreAcf = skapi.AcfNew();
    CreAcf.ocm = ocmMonster;
    CreAcf.distMax = SelRange;

    // Setup arrays to save info for each race type
    var RaceCount = new Array();
    var RaceExp   = new Array();
    for (var x=1; x<=Creature.RaceMax; x++) {
    	RaceCount[x] = 0;
    	RaceExp[x]   = 0;
    } // endfor

    // Get our location
    var OurLoc = skapi.maplocCur;

    // Build collection of all creatures
    var CreFound = false;
    var CreList = CreAcf.CoacoGet();
    CreList.Sort(skapi.sortbyName);
    for (var index=0; index<CreList.Count; index++) {
	var CreAco = CreList.Item(index);
	if (CreAco) {
	    var CreName = CreAco.szName;
	    var CreIndex = Creature.NameToIndex(CreName);
	    if (CreIndex != 0) {
	    	var CreFound = true;
	    	with (Creature.Info[CreIndex]) {
		    RaceCount[RaceCode]++;
		    RaceExp[RaceCode] += Creature.Info[CreIndex].Exp;
		} // end with
	    } else {
	    	Log.OutD("FindCreSummary: Unknown creature [" + CreName + "]");
	    } // endif
	} else {
	    if (debugEval) Log.OutD("FindCreatures:  Index [" + index + "] Aco is null");
	} // endif
    } // endfor

    if (!CreFound) {
    	Chat.Screen("No creatures found!");
    	return true;
    } // endif

    var TotalCount = 0;
    var TotalExp   = 0;
    Chat.Screen(" ");
    Chat.Screen(Util.Width("Race",1.8) + Util.Width("Count",1.0) + "Exp");
    for (var x=1; x<=Creature.RaceMax; x++) {
    	if (RaceCount[x] > 0) {
    	    TotalCount += RaceCount[x];
    	    TotalExp   += RaceExp[x];
	    Chat.Screen(Util.Width(Creature.RaceInfo[x].RaceName,1.8) + Util.Width(RaceCount[x]+" ",1.0) + RaceExp[x]);
	} // end if
    } // end for

    Chat.Screen(" ");
    Chat.Screen("Total of " + TotalCount + " creatures worth " + TotalExp + " exp points.");
    Chat.Screen(" ");
    return true;
} // end function


function FindCreDetail() {
    if (debug||debugEval) Log.OutD("FindCreDetail: Begin" );

    // Find any creatures close by and report their distance, heading, and location
    var CreAcf = skapi.AcfNew();
    CreAcf.ocm = ocmMonster;
    CreAcf.distMax = SelRange;
    var CreCount = 0;
    var TotalExp = 0;

    // Get our location
    var OurLoc = skapi.maplocCur;

    // Build collection of all creatures
    var CreList = CreAcf.CoacoGet();
    CreList.Sort(skapi.sortbyName);

    if (CreList.Count == 0) {
	Chat.Screen("No creatures nearby!");
    } else {
    	Chat.Screen(" ");
	Chat.Screen(Util.Width("Creature",2.5) + Util.Width("Exp",1.0) + Util.Width("Dist",0.8) + "Head");
    } // endif

    for (var index=0; index<CreList.Count; index++) {
	var CreAco = CreList.Item(index);
	if (CreAco) {
	    CreCount++;
	    var CreName = CreAco.szName;
	    var CreLoc  = CreAco.maploc;
	    var CreDist = Util.Distance(OurLoc, CreLoc);
	    var CreHead = OurLoc.HeadToMaploc(CreLoc);
	    var CreIndex = Creature.NameToIndex(CreName);
	    if (CreIndex != 0) {
	    	var CreExp = Creature.Info[CreIndex].Exp;
	    	TotalExp += CreExp;
	    } else {
		Log.OutD("FindCreDetail: Unknown creature [" + CreName + "]", true);
	    	var CreExp = 0;
	    } // endif

	    Chat.Screen(Util.Width(CreName,2.5) + Util.Width(CreExp+" ",1.0) +
			Util.Width(Util.RoundFill(CreDist,2),0.8) + Util.Round(CreHead,0));
	} else {
	    if (debugEval) Log.OutD("FindCreDetail:  Index [" + index + "] Aco is null");
	} // endif
    } // endfor

    Chat.Screen(" ");
    Chat.Screen("Found " +  CreCount + " creatures worth " + TotalExp + " exp points.");
    Chat.Screen(" ");
    if (debugEval) Log.OutD("FindCreDetail:  Found " +  CreCount + " creatures worth " + TotalExp + " exp points.");
    return;
} // end function


function ReqFellow() {
    var PlayerAcf = skapi.AcfNew();
    PlayerAcf.ocm = ocmPlayer;
    PlayerAcf.distMax = 0.10;
    var PlayerList = PlayerAcf.CoacoGet();
    var ClosestDist = 9999;
    var ClosestName = null;
    var OurLoc = skapi.maplocCur;

    for (var index=0; index<PlayerList.Count; index++) {
	var PlayerAco = PlayerList.Item(index);

	// If this Aco is us, then continue with next entry
	if (PlayerAco.szName == skapi.szName) continue;

	// If this entry is closer than current closest player, then we have a new winner
	var PlayerDist = Util.Distance(PlayerAco.maploc, OurLoc);
	if (PlayerDist < ClosestDist) {
	    ClosestName = PlayerAco.szName
	    ClosestDist = PlayerDist;
	} // endif
    } // endfor

    if (ClosestName == null) {
	Chat.Screen("No players found within 0.10 map units.");
    } else {
	Chat.Tell(ClosestName, "fellow");
    } // endif
    return;
} // end func

function JoinMoveGroup() {
    var PlayerAcf = skapi.AcfNew();
    PlayerAcf.ocm = ocmPlayer;
    PlayerAcf.distMax = 0.3;
    var PlayerList = PlayerAcf.CoacoGet();
    var ClosestDist = 9999;
    JoinTarget = null;
    var OurLoc = skapi.maplocCur;
    JoinTarget = null;

    for (var index=0; index<PlayerList.Count; index++) {
	var PlayerAco = PlayerList.Item(index);

	// If this Aco is us, then continue with next entry
	if (PlayerAco.szName == skapi.szName) continue;

	// If this entry is closer than current closest player, then we have a new winner
	var PlayerDist = Util.Distance(PlayerAco.maploc, OurLoc);
	if (PlayerDist < ClosestDist) {
	    JoinTarget = PlayerAco.szName
	    ClosestDist = PlayerDist;
	} // endif
    } // endfor

    if (JoinTarget == null) {
	Chat.Screen("No players found within 0.3 map units.");
	return;
    } // endif

    // Timestamp join request.  We only respond to movegroup info tells within 60 seconds of
    // a request.  This stops
    LastJoinReq	= new Date();
    Chat.Tell(JoinTarget, "JoinGroup");
    return;
} // end func

function ShowAttLoc(Index) {
    var CurrLoc = skapi.maplocCur;

    if (AttLoc[Index].KillLoc != null) {
    	var OutPart1 = Index + "   KL: " +  Util.FormatCoords(AttLoc[Index].KillLoc,2);
    	var Dist = Util.Distance(CurrLoc,AttLoc[Index].KillLoc);
    	if (Dist > 0.01)
	    OutPart1 = OutPart1 + "  " + Util.RoundFill(Dist,2) +
				  "  " + Util.Round(CurrLoc.HeadToMaploc(AttLoc[Index].KillLoc),0);
    } else {
	OutPart1 = Index + "   KL: Undefined";
    } // endif

   if (AttLoc[Index].WayLoc != null) {
   	var OutPart2 = "WP: " + Util.FormatCoords(AttLoc[Index].WayLoc,2);
    	var Dist = Util.Distance(CurrLoc,AttLoc[Index].WayLoc);
    	if (Dist > 0.01)
	    OutPart2 = OutPart2 + "  " + Util.RoundFill(Dist,2) +
				  "  " + Util.Round(CurrLoc.HeadToMaploc(AttLoc[Index].WayLoc),0);
    } else {
	var OutPart2 = "WP: Undefined";
    } // endif

    if (AttLoc[Index].RetreatLoc != null) {
   	var OutPart3 = "RL: " + Util.FormatCoords(AttLoc[Index].RetreatLoc,2);
    	var Dist = Util.Distance(CurrLoc,AttLoc[Index].RetreatLoc);
    	if (Dist > 0.01)
	    OutPart3 = OutPart3 + "  " + Util.RoundFill(Dist,2) +
				  "  " + Util.Round(CurrLoc.HeadToMaploc(AttLoc[Index].RetreatLoc),0);
    } else {
	var OutPart3 = "RL: Undefined";
    } // endif

    Chat.Screen(Util.Width(OutPart1,3.2) + Util.Width(OutPart2,3.0) + OutPart3)
} // end function


function NavShow() {
    var CurrLoc = skapi.maplocCur;
    var Wid  = 1.5;
    var Wid2 = 1.2;
    var Wid3 = 1.2;

    Chat.Screen(" ");

    // Print the first line
    var OutLine = "Current AttLoc: " + AttLocCur + "    CurrentCoords: " + Util.FormatCoords(CurrLoc,3) + "     ";
    if (MoveGroup == 0)
	OutLine+= "MoveGroup: None";
    else
	OutLine+= "MoveGroup: " + MoveGroup;

    Chat.Screen(OutLine);

    OutLine = "Retreat Count: " + RetreatAmount + "     ";

    if (MasterWayLoc != null) {
   	OutLine+= "Master WayPnt: " + Util.FormatCoords(MasterWayLoc,3) + "    ";
    	var Dist = Util.Distance(CurrLoc,MasterWayLoc);
    	if (Dist > 0.001)
	    OutLine += "Dist: " + Util.RoundFill(Dist,3) + "   Heading: " +
	    		Util.Round(CurrLoc.HeadToMaploc(MasterWayLoc),0);
    } else {
	OutLine+= "Master WayPnt: Undefined";
    } // endif

    Chat.Screen(OutLine);

    OutLine = "AutoMove: "
    if (AutoMove)
	OutLine+= "Enabled";
    else
	OutLine+= "Disabled";

    OutLine+= "    Retreating: ";
    if (flagRetreatCrit)
	OutLine+= "Enabled";
    else
	OutLine+= "Disabled";

    Chat.Screen(OutLine);

    for (var x=1; x<=AttLocCnt; x++) ShowAttLoc(x);
    return;
} // end func

function NavReset() {
    Chat.Screen("Clearing all attacklocs & waypoints.  Setting Attloc #1 to current coords...");
    for (var x=1; x<=navMaxLoc; x++) AttLoc[x] = new AttLocRec();
    MasterWayLoc = null;
    RetreatActive = false;
    AttLoc[1].KillLoc = skapi.maplocCur;
    AttLocCur = 1;
    AttLocCnt = 1;
    HomeLoc = AttLoc[1].KillLoc;
    LastArriveTime = new Date();
    ClearMoveStats();
    // Set decal control panel attack loc on movement panel to match new attack location
    skapi.SetControlProperty(szPanelMovement, "curAttLoc", "selected",  AttLocCur-1);
    skapi.SetControlProperty(szPanelMovement, "text2av", "text", AttLocCur);
    return;
} // end func


function NavLoad(filenum) {
    if ((filenum == undefined) || (filenum == null)) filenum = 1;
    if (filenum < 10) filenum = "0" + filenum;
    var FileName =  CharCfgDir + "\\" + fileNavCfg + filenum + fileCfgExt;

    // See if file exists
    if(!Fso.FileExists(FileName)) {
	Chat.Screen("Navigation file " + FileName + " not found!");
    	return;
    } // endif

    // Clear out current nav structures
    for (var x=1; x<=navMaxLoc; x++) AttLoc[x] = new AttLocRec();
    MasterWayLoc = null;

    // Now read config file in
    Chat.Screen("Restoring navigation information from file " + FileName + " ...");
    Util.Include(FileName);

    // Now figure out closest attack location from the restored info
    var CurrLoc = skapi.maplocCur;
    var ClosestDist = 9999;
    for (var x=1; x<=AttLocCnt; x++) {
	var NavDist = Util.Distance(CurrLoc, AttLoc[x].KillLoc);
	if (NavDist < ClosestDist) {
	    ClosestDist = NavDist;
	    AttLocCur = x;
	} // endif
    } // endfor
    HomeLoc = AttLoc[AttLocCur].KillLoc;
    LastArriveTime = new Date();
    skapi.SetControlProperty(szPanelMovement, "text2av", "text", AttLocCur);
    ClearMoveStats();
    DisplayAttLoc(AttLocCur);
    Chat.Screen("Navigation info restored.  Attack location set to AttLoc # " + AttLocCur );

    // Set decal control panel attack loc on movement panel to match new attack location
    skapi.SetControlProperty(szPanelMovement, "curAttLoc", "selected",  AttLocCur-1);

    // Now warn macroer if current attack location is not close to our current position
    if (ClosestDist > navWarnDist)
	Chat.Screen("WARNING! Current attack location is " + Util.Round(ClosestDist,2) + " map units away!");

    // Force state to IdleInit
    MoveEnd = new Date();
    StateChange(stateIdleInit,NoCheck);

    // All done
    return;
} // end func


function NavSave(filenum) {
    // Build the output filename from the filenumber
    if ((filenum == undefined) || (filenum == null)) filenum = 1;
    if (filenum < 10) filenum = "0" + filenum;
    var FileName =  CharCfgDir + "\\" + fileNavCfg + filenum + fileCfgExt;
    Chat.Screen("Saving navigation information to file " + FileName + " ...");

    // Now create the file
    NavFile = Fso.OpenTextFile(FileName, ForWriting, true);	// Create config file
    NavFile.WriteLine("// Navigation configuration data.  Be careful to preserve the syntax!");
    NavFile.WriteLine("AttLocCnt = " + AttLocCnt + ";");

    for (var x=1; x<=AttLocCnt; x++) {
	// Write KillLoc
	if (AttLoc[x].KillLoc != null) {
	    NavFile.WriteLine("AttLoc[" + x + "].KillLoc = skapi.MaplocFromSz(\"" +
			  Util.FormatCoords(AttLoc[x].KillLoc,3) + "\");");
	} else {
	    NavFile.WriteLine("AttLoc[" + x + "].KillLoc = skapi.MaplocFromSz(\"" +
			  Util.FormatCoords(skapi.maplocCur,3) + "\");");
	    Log.OutD("NavSave: Fatal Error, kill location " + x + " null. Using current location...", true);
	} // endif

	// If retreat location set, write it out
	if (AttLoc[x].RetreatLoc != null)
	    NavFile.WriteLine("AttLoc["+ x + "].RetreatLoc = skapi.MaplocFromSz(\"" +
			      Util.FormatCoords(AttLoc[x].RetreatLoc,3) + "\");");

	// If waypoint set, write it out
	if (AttLoc[x].WayLoc != null)
	    NavFile.WriteLine("AttLoc[" + x + "].WayLoc = skapi.MaplocFromSz(\"" +
			      Util.FormatCoords(AttLoc[x].WayLoc,3) + "\");");

	// Write out attloc flag value
	NavFile.WriteLine("AttLoc[" + x + "].LocFlags = " + AttLoc[x].LocFlags + ";");
    } // endfor

    // If master waypoint set, write it out
    if (MasterWayLoc != null)
	NavFile.WriteLine("MasterWayLoc = skapi.MaplocFromSz(\"" + Util.FormatCoords(MasterWayLoc,3) +
			  "\");");

    // All done, close file
    NavFile.Close();
    Chat.Screen("Navigation information saved.");
    return;
} // end func

function SendNavInfo(szTarget) {
    Line1 = "NAVBEGIN " + AttLocCnt + "," + AttLocCur + "," + MoveGroup + "," + retreatAmtOne + "," +
    	    retreatAmtTwo + "," + retreatAmtThreePlus;
    // If master waypoint set, add it to first line
    if (MasterWayLoc != null) Line1 = Line1 + "," + Util.FormatCoords(MasterWayLoc,3);
    Chat.Tell(szTarget, Line1);

    MaxPerLine = 3;
    Line = "NAVINFO ";
    LineCnt = 0;

    for (var x=1; x<=AttLocCnt; x++) {
    	// Add attack location to line
    	LineCnt++
    	if (LineCnt>1) Line = Line + ",";
    	Line = Line + x + "," + Util.FormatCoords(AttLoc[x].KillLoc,3) + ",";

	// If waypoint set, add it to line
	if (AttLoc[x].WayLoc != null) Line = Line + Util.FormatCoords(AttLoc[x].WayLoc,3);
	Line = Line + ",";

	// If retreat location set, add it to line
	if (AttLoc[x].RetreatLoc != null) Line = Line + Util.FormatCoords(AttLoc[x].RetreatLoc,3);
	Line = Line + ",";

	// Add flags for this attack loc to line
	Line = Line + AttLoc[x].LocFlags;

	if (LineCnt == MaxPerLine) {
	    Chat.Tell(szTarget, Line);
	    Line = "NAVINFO ";
	    LineCnt = 0;
	} // endif
    } // endfor

    if (LineCnt > 0) Chat.Tell(szTarget, Line);
} // end function

function EstimatedSpells(Vuln,CreCurHealth) {
    if (debug || debugAttack) Log.OutD("EstimatedSpells: Vuln lvl [" + Vuln + "] Cre health [" + CreCurHealth + "]");

    var Lvl = 0;
    if (Att.WarSpell == attSpellArc)  Lvl = Att.CurArc;
    if (Att.WarSpell == attSpellBolt) Lvl = Att.CurBolt;
    if (Att.WarSpell == attSpellAuto) Lvl = Math.max(Att.CurArc, Att.CurBolt);

    var EstDam = (Damage.WarBoltMin[Lvl] + (Damage.WarBoltRange[Lvl]*attVulnWeight))*(1+(Damage.VulnInc[Vuln]/100));
    var EstSpl = Math.ceil(CreCurHealth / EstDam);

    if (debug || debugAttack) Log.OutD("EstimatedSpells: End, EstDam [" + EstDam + "] EstSpells [" + EstSpl + "]");
    return EstSpl;
} // endif

function StepDownBolt(CreCurHealth) {
    if (debug || debugAttack) Log.OutD("StepDownBolt: Begin, Cre health [" + CreCurHealth + "]");

    // Calculate lowest spell that will kill the creature, ie don't cast a lvl n spell if
    // a lvl n-1 spell will do the job
    for (var x=minWarSpell; x<Att.CurBolt; x++) {
	// if we know this spell and the damage it would cause with the current
	// vuln is sufficient to kill, then lower the war spell down
	if (Spell.Info[Att.Element].BoltMap[x]) {
	    // We know the spell so calculate the estimated damage
	    EstDam = Damage.WarBoltMin[x] + (Damage.WarBoltRange[x] * attWarWeight);
	    if (Att.VulnActive) EstDam *= (1+(Damage.VulnInc[Att.VulnLevel]/100));
	    if (CreCurHealth < EstDam) {
	    	 if (debugAttack)
	    	     Log.OutD("StepDownBolt: End, Stepping down from lvl [" + Att.CurBolt + "] to [" + x +
	    	     	      "] Est Damage [" + EstDam + "]");
		 return x;
	    } // endif
	} // endif
    } // endfor

    if (debug || debugAttack) Log.OutD("StepDownBolt: End, no step down possible");
    return Att.CurBolt;
} // end function

function StepDownArc(CreCurHealth) {
    if (debug || debugAttack) Log.OutD("StepDownArc: Begin, Cre health [" + CreCurHealth + "]");
    // Calculate lowest spell that will kill the creature, ie don't cast a lvl n spell if
    // a lvl n-1 spell will do the job
    for (var x=minWarSpell; x<Att.CurArc; x++) {
	// if we know this spell and the damage it would cause with the current
	// vuln is sufficient to kill, then lower the war spell down
	if (Spell.Info[Att.Element].ArcMap[x]) {
	    // We know the spell so calculate the estimated damage
	    EstDam = Damage.WarBoltMin[x]+(Damage.WarBoltRange[x]*attWarWeight);
	    if (Att.VulnActive) EstDam *= (1+(Damage.VulnInc[Att.VulnLevel]/100));
	    if (CreCurHealth < EstDam) {
	    	 if (debugAttack)
	    	     Log.OutD("StepDownArc: End, Stepping down from lvl [" + Att.CurArc + "] to [" + x +
	    	     	      "] Est Damage [" + EstDam + "]");
		 return x;
	    } // endif
	} // endif
    } // endfor
    if (debug || debugAttack) Log.OutD("StepDownArc: End, no step down possible");
    return Att.CurArc;
} // end function

function SafeChatSay(text) {
    if (State != stateGotoNext)
	Chat.Say(text);
    else
        Chat.Screen("Local chat suppressed during navigation [" + text + "]");
} // end func

function LockTarget(Aco) {
    // Broadcast this creature selection to the fellow so no one else will attempt to
    // target this creature
    if (debugExcl)
	Log.OutD("LockTarget: Begin, Locking creature [" + Aco.szName + "] Oid [" + Aco.oid + "/" +
		 Util.Hex(Aco.oid) + "] attExclPrio [" + attExclPrio + "]  Attempt Prio [" + AttemptPriority + "]");

    Chat.Fellow("LOCK " + Aco.oid + " " + AttemptPriority );
} // end func

// When in exclusion mode, broadcast to fellow when we abort
function UnlockTarget(Aco) {
    if (debugExcl)
	Log.OutD("UnlockTarget: Unlocking cre [" + Aco.szName + "] Oid [" + Aco.oid + "/" + Util.Hex(Aco.oid) + "]");

    // Send message to fellow stating creature is no longer locked.  Send yield and vuln information
    // so somebody else doesn't try to reyield/revuln creature.  Note we send relative exp time
    // in milliseconds instead of absolute times since computer times will never be syncronized
    // between any two macroers
    OutLine = "UNLOCK " + Aco.oid;

    var CurTime = new Date();
    var YieldOffset = 0;
    var VulnOffset  = 0;
    var VulnLevel   = 0;

    // See if a yield was done.  If so send the exp time to fellow so they don't repeat it
    if (Att.YieldTime != 0) {
    	// Compute yield offset
    	var YieldExp = Att.YieldTime.getTime() + ((Att.YieldSpell.csecDuration-30)*1000);
    	YieldOffset = YieldExp - CurTime.getTime();
    	if (debugExcl) {
 	    Log.OutD("UnlockTarget: Yield cast time: " + new Date(Att.YieldTime));
	    Log.OutD("UnlockTarget: Yield exp Time: " + YieldExp + " [" + new Date(YieldExp) + "]");
	    Log.OutD("UnlockTarget: Yield Offset: " + YieldOffset);
	} // endif
	if (YieldOffset < 0) YieldOffset = 0;
    } else {
	if (debugExcl) Log.OutD("UnlockTarget: No yield done");
    } // endif
    OutLine = OutLine + " " + YieldOffset;

    // See if a yield was done.  If so send the exp time to fellow so they don't repeat it
    if (Att.VulnTime != 0) {
    	// Compute yield offset
    	var VulnExp = Att.VulnTime.getTime() + ((Att.VulnSpell.csecDuration-30)*1000);
    	VulnOffset = VulnExp - CurTime.getTime();
    	VulnLevel = Att.CurVuln;
    	if (debugExcl) {
 	    Log.OutD("UnlockTarget: Vuln cast time: " + new Date(Att.VulnTime));
	    Log.OutD("UnlockTarget: Vuln exp time: " + VulnExp + " [" + new Date(VulnExp) + "]");
	    Log.OutD("UnlockTarget: Vuln offset: " + VulnOffset);
	    Log.OutD("UnlockTarget: Vuln level: " + VulnLevel);
	} // endif
	if (VulnOffset < 0) VulnOffset = 0;
    } else {
	if (debugExcl) Log.OutD("UnlockTarget: No Vuln done");
    } // endif
    OutLine = OutLine + " " + VulnOffset + " " + VulnLevel;

    Chat.Fellow(OutLine);
    if (debugExcl||logExcl)
	Log.OutD("Target Unlocked: " + Aco.szName + "  Oid: " + Aco.oid + "/" + Util.Hex(Aco.oid) +
		 "  YieldOffset: " + YieldOffset + "  VulnOffset: " + VulnOffset + "  VulnLevel: " + VulnLevel );
} // end func


function ExclFindLock(Oid) {
    if (debugExcl) Log.OutD("ExclFindLock: Oid [" + Oid + "]");
    for (var x=0; x<selLockListSize; x++) {
    	if (SelLockList[x].Oid == Oid) {
	    if ((new Date()-SelLockList[x].CreTime) > selLockLimit) {
		if (debugExcl) Log.OutD("ExclFindLock: Lock Found, but expired");
	    	return false;
	    } else {
	    	if (debugExcl) Log.OutD("ExclFindLock: Lock Found");
	    	return true;
	    } // endif
	} // endif
    } // endfor
    if (debugExcl) Log.OutD("ExclFindLock: Lock not Found");
    return false;	// Lock not found
} // end func


function ExclAddLock(Oid) {
    if (debugExcl) Log.OutD("ExclAddLock: Begin, Oid [" + Oid + "]");

    // Make sure lock doesn't exist already
    for (var x=0; x<selLockListSize; x++) {
    	if (SelLockList[x].Oid == Oid) {
	    if (debugExcl) Log.OutD("ExclAddLock: Duplicate entry, updating lock creation time...");
	    SelLockList[x].CreTime = new Date();
	    return;
	} // endif
    } // endfor

    // Oid not found, add to list
    SelLockList[SelLockNext] = new LockObj(Oid);
    if (debugExcl) Log.OutD("ExclAddLock: End, Slot used [" + SelLockNext + "]");
    SelLockNext++;
    if (SelLockNext == selLockListSize) SelLockNext = 0;
} // end func


function ExclDelLock(Oid, YieldOffset, VulnOffset, VulnLvl) {
    if (debugExcl)
	Log.OutD("ExclDelLock: Begin, Oid [" + Oid + "] YieldOffset [" + YieldOffset + "]  VulnOffset [" +
		 VulnOffset + "] VulnLvl [" + VulnLvl + "]");

    // Remove lock from lock list
    for (var x=0; x<selLockListSize; x++) {
    	if (SelLockList[x].Oid == Oid) {
	    if (debugExcl) Log.OutD("ExclDelLock: Lock removed");
	    SelLockList[x].Oid == 0;
	    break;
	} // endif
    } // endfor

    if (x == selLockListSize) {
    	if (debugExcl) Log.OutD("ExclDelLock: Lock not found");
    } // endif

    // If a yield or vuln done, then add to previously yielded/vuln list in case we select this creature
    // in the future.  If no yield or vuln spell was cast this kill attempt then don't need to add to list
    // need to record anything.
    if ((YieldOffset == 0) && (VulnOffset == 0)) {
	if (debugExcl) Log.OutD("ExclDelLock: No yield or vuln active");
	return;
    } // endif

    // Either a yield or vuln was cast.  Calculate the absolute time the yield and vuln will expire
    var CurTime = new Date();
    var YieldExp = 0;
    var VulnExp = 0;
    if (YieldOffset > 0) YieldExp = CurTime.getTime() + YieldOffset;
    if (VulnOffset  > 0) VulnExp  = CurTime.getTime() + VulnOffset;
    if (debugExcl) {
	if (YieldExp > 0)
	    Log.OutD("ExclDelLock: YieldExp: " + YieldExp + " [" + new Date(YieldExp) + "]");
	else
	    Log.OutD("ExclDelLock: No Yield");

	if (VulnExp  > 0)
	    Log.OutD("ExclDelLock: VulnExp: " + VulnExp + " [" + new Date(VulnExp) + "]");
	else
	    Log.OutD("ExclDelLock: No Vuln");
    } // endif

    // Figure out what slot number in list to use.  If creature already on the list then update its
    // information, otherwise create a new entry
    var Index = -1;
    for (var x=0; x<AttCtx.PrevCnt; x++) {
    	if (AttCtx.PrevList[x].Id == Oid) {
	    // Match found
	    Index = x;
	    if (debugExcl) Log.OutD("ExclDelLock: Already on AttPrev list at index [" + Index + "]");
	    break;
	} // endif
    } // endfor

    // If didn't find creature's id, then add a new entry to the table
    if (Index == -1) {
    	if (debugExcl) Log.OutD("ExclDelLock: Creature not in table, adding new entry");
	Index=AttCtx.PrevCnt;
	AttCtx.PrevList[Index] = new AttPrevObj(Oid);
	AttCtx.PrevCnt++;
    } // endif

    // Now update expiration time/dates
    if (debugExcl) Log.OutD("ExclDelLock: Updating info at index [" + Index + "]");
    AttCtx.PrevList[Index].YieldExp	= YieldExp;
    AttCtx.PrevList[Index].VulnExp	= VulnExp;
    AttCtx.PrevList[Index].VulnLvl	= VulnLvl;

    if (debugExcl) Log.OutD("ExclDelLock: End");
} // end func


function GotoRetreatLoc(TimeOut) {
    if ((TimeOut == undefined) || (TimeOut == null)) TimeOut = 0;
    if (debug||debugMove) Log.OutD("GotoRetreatLoc: Begin, Timeout (ms): " + TimeOut );

    if (AttLoc[AttLocCur].RetreatLoc == null) {
	if (debugMove) Log.OutD("GotoRetreatLoc: No retreat location defined for this AttLoc, aborting...");
	return;
    } // endif

    if (debug||debugMove) Log.OutD("GotoRetreatLoc: Moving to retreat location...");
    MoveDone = false;
    MoveStart = new Date();
    skapi.GoToMaploc(AttLoc[AttLocCur].RetreatLoc, navPrecision );

    while (!MoveDone && ((new Date()-MoveStart)<TimeOut)) {
    	skapi.WaitEvent(20);
    	if (flagRecordLoc) RecordLoc();
    } // end while

    if (MoveDone) {
        if (debug||debugMove)
	    Log.OutD("GotoRetreatLoc:  Navigation done, status code [" + MoveCode + "/" + textNscCode[MoveCode] + "]");
	if (MoveCode == nscCanceled) {
	    Chat.Screen("Navigation aborted.");
	    return false;
	} // endif
	if (debugMove) Log.OutD("GotoRetreatLoc: Arrived at retreat location.");
	return true;
    } // endif

    // If we make it here, move timed out
    Log.OutD("GotoRetreatLoc: Move to retreat location timed out!");
    Chat.Screen("Navigation timed out.");
    skapi.CancelNav();
    return false;
} // end func

function GotoKillLoc(TimeOut) {
    if ((TimeOut == undefined) || (TimeOut == null)) TimeOut = 0;
    if (debug||debugMove) Log.OutD("GotoKillLoc: Begin, Timeout (ms): " + TimeOut );
    if (logMove||debug||debugMove) Log.OutD("GotoKillLoc: Moving to kill location...");
    MoveDone = false;
    MoveStart = new Date();
    skapi.GoToMaploc(AttLoc[AttLocCur].KillLoc, navPrecision );

    while (!MoveDone && ((new Date()-MoveStart)<navTimeOut)) {
    	skapi.WaitEvent(20);
    	if (flagRecordLoc) RecordLoc();
    } // endwhile

    if (MoveDone) {
        if (logMove||debug||debugMove) Log.OutD("GotoKillLoc: Nav done, status ["+MoveCode+"/"+textNscCode[MoveCode]+"]");
	if (MoveCode == nscCanceled) {
	    Chat.Screen("Navigation aborted.");
	    if (logMove||debugMove) Log.OutD("GotoKillLoc: Navigation aborted.");
	    return false;
	} // endif
	return true;
    } // endif

    // If we make it here, move timed out
    Chat.Screen("Navigation timed out.");
    Log.OutD("GotoKillLoc: Move to kill location timed out!");
    skapi.CancelNav();
    return false;
} // end func


function DeathReset() {
    Log.OutD("DeathReset: Begin");
    Dead = false;

    // Any parameters we temporarily disabled now restore
    if (DeathBuffDisabled) {
	DeathBuffDisabled = false;
	BuffMode = true;
	skapi.SetControlProperty(szPanelControl, "btnBuffMode", "checked", true);
    	Chat.Screen("Buffing re-enabled.");
    	Log.OutD("DeathReset: Buffing re-enabled");
    } // endif

    if (DeathAutoMoveDisabled) {
	DeathAutoMoveDisabled = false;
	AutoMove = true;
	skapi.SetControlProperty(szPanelControl, "btnAutoMove",  "Text", "Disable AutoMove");
   	Chat.Screen("AutoMove re-enabled.");
    	Log.OutD("DeathReset: AutoMove re-enabled");
    } // endif

    if (DeathAutoHomeDisabled) {
	DeathAutoHomeDisabled = false;
	flagAutoHome = true;
	skapi.SetControlProperty(szPanelControl, "btnAutoHome",  "Text", "Disable AutoHome");
   	Chat.Screen("AutoHome re-enabled.");
    	Log.OutD("DeathReset: AutoHome re-enabled");
    } // endif

    if (DeathPeaceModeEnabled) {
	DeathPeaceModeEnabled = false;
   	PeaceMode = false;
    	skapi.SetControlProperty(szPanelControl, "btnPeaceMode",  "Text", "Enable PeaceMode");
   	Chat.Screen("PeaceMode disabled.");
    	Log.OutD("DeathReset: PeaceMode disabled");
    } // endif

    // Reset death log time if set
    if (DeathLogTime>0) {
	DeathLogTime = 0;
	Chat.Screen("Death log time cleared.");
	Log.OutD("DeathReset: Death log time cleared.");
    } // endif

    // Reset death count if necessary
    if (DeathCount>0) {
	DeathCount = 0;
	Chat.Screen("Death count cleared.");
	Log.OutD("DeathReset: Death count cleared.");
    } // endif

} // end function DeathReset()

function RecordLoc() {
    if (skapi.plig != pligInWorld) return;
    var DistToLast = Util.Distance(LastRecordedLoc,skapi.maplocCur);
    if (DistToLast >= locRecordThresh) {
    	Log.OutD("RecordLoc: New Coords [" + Util.FormatCoords(skapi.maplocCur,2) + "]");
	LastRecordedLoc = skapi.maplocCur;
    } // endif
} // endif

function KillDone() {
    if (debug||debugAttack) Log.OutD("KillDone: Begin, Att.Oid [" + Util.Hex(Att.Oid) + "] Att.CreExp [" + Att.CreExp + "]");
    if (logKills)
	Log.OutD("KillDone: Kill attempt on [" + Att.Aco.szName + "] done, Att.Oid [" +
		 Util.Hex(Att.Oid) + "] Att.CreExp [" + Att.CreExp + "]");

    AttCtx.LastDone    = new Date();	// Save time this attack attempt ended.
    AttCtx.PrevCreName = Att.CreName;	// Save previous name in so we can detect death message for previous attack.
    AttCtx.PrevCreExp  = Att.CreExp;	// Save experience value.  Need this in case cre is killed after we deselect it.
    AttCtx.PrevId      = Att.Oid;	// Save ID so we don't attempt to reselect this creature.  This is needed since it
    					// sometimes takes 2 secs or so for creature to be cleared out the object table.
   
    // If we are in support mode and a imperl and/or vuln has been cast, then mark the creature as done
    if (attSupportOnly &&
	(!attAlwaysImperl || (attAlwaysImperl && Att.ImperlTime!=0)) &&
	(!attAlwaysVuln   || (attAlwaysVuln   && Att.VulnTime!=0))) {
	AttDoneAdd(Att.Oid);    //  add creature to the done list	    
    } // endif

    if (debugAttack) Log.OutD("KillDone: Next state set to select");
    StateChange(stateSelect);
    if (debug||debugAttack) Log.OutD("KillDone: End, new state [" + State + "/" + textState[State] + "]");
    return;
} // end func KillDone()


function CompResume() {
    if (debug|debugIdle) Log.OutD("CompResume: Begin");
   
    // If AttackMode or AutoMove was disabled, then turn them back on
    if (SuspAttackMode) {
	AttackMode = true;
	skapi.SetControlProperty(szPanelControl, "btnAttackMode", "Text", "Disable Attack");
    } // endif
	    
    if (SuspAutoMove) {
	AutoMove = true;
	skapi.SetControlProperty(szPanelControl, "btnAutoMove", "Text", "Disable AutoMove");
    } // endif

    // Now figure out what attack location to goto.  If we have received a MOVE message from a macro
    // in our movegroup then goto it, else we goto the attloc we were at when killing was suspended.
    if (MoveOverRide > 0) {
	Log.OutDM("CompResume", "Moving to AttLoc # " + MoveOverRide, ToChat);
	StateChange(stateGotoNext,NoCheck);
	return;
    } // endif

    // No move message was received, so go back to current attack location if we are not near it
    var AttDist = Util.Distance(AttLoc[AttLocCur].KillLoc, skapi.maplocCur)
    if (AttDist > 0.02) {
	Log.OutDM("CompResume", "Moving to current KillLoc (" + AttLocCur + ") ...", ToChat);
	NavActive = true;
	GotoKillLoc();
	NavActive = false;
    } else {
	Log.OutDM("CompResume", "No move necessary, distance to AttLoc [" + AttDist + "]", ToChat);
    } // endif

    if (debug|debugIdle) Log.OutD("CompResume: End");
} // end func


function SalvageInit() {
    if (debug||debugIdle||debugSalvage) Log.OutD("SalvageInit: Begin");

    var FreePackSlots = skapi.acoChar.citemMax - skapi.acoChar.citemContents;
    if (FreePackSlots == 0) {
   	Log.OutDM("DoState_SalvageInit", "USTing disabled due to full pack! Must be manually re-enabled!", cmcYellow);
   	flagAutoUst = false;
   	skapi.SetControlProperty(szPanelConfig, "chkAutoUst", "checked", false);
    	var Success = false;
    } else {
	// Clear out failure count, record start time, and set state to Salvage
	UstFailCount = 0;
	SalvageStart = new Date();
	var Success = true;
    } // endif
   if (debug||debugIdle||debugSalvage) Log.OutD("SalvageInit: End, Success [" + Success + "]");
   return Success;
} // end func


function SpellCheck(verbose) {
    if ((verbose==undefined) || (verbose==null)) verbose = false;
    if (debug) Log.OutD("SpellCheck: Begin");

    // Assume everything ok
    var ReturnCode = true;

    // Make sure all spells hardcoded in macro are defined correctly
    if (!Cast.Know(SpellObj_DrainHealth.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Drain Health Other not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!Cast.Know(SpellObj_DrainStamina.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Drain Stamina Other not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!Cast.Know(SpellObj_DrainMana.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Drain Mana Other not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!Cast.Know(SpellObj_StamHealth.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Stamina to Health Self not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!Cast.Know(SpellObj_StamMana.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Stamina to Mana Self not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!Cast.Know(SpellObj_Heal.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Heal Self not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!Cast.Know(SpellObj_Revitalize.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Revitalize Self not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!Cast.Know(SpellObj_Strength.spellid)) {
    	Log.OutDM("Spellcheck", "Spell Strength Self not defined.", ToChat, cmcYellow);
    	ReturnCode = false;
    } // endif

    if (!ReturnCode) {
	Restricted = true;
    	if (verbose) {
	    Log.OutDM("SpellCheck", "WARNING!!! One or more spells necessary for macro operation not defined!!!",
			ToChat, cmcYellow);
	    Log.OutDM("SpellCheck", "Macro operating in restricted mode (only some decal panel operations permitted).",
			ToChat, cmcYellow);
	    Log.OutDM("SpellCheck", "To fix this condition, select known spells on the S2 tab", ToChat, cmcYellow);
	    Log.OutDM("SpellCheck", "on the config panel for the spells listed above..", ToChat, cmcYellow);
	} // endif
    } // endif

    if (debug) Log.OutD("SpellCheck: End");
    return ReturnCode;
} // end func


// Function to process top item on salvage loot list.
function Salvage_TinkItems() {
    if (debug||debugSalvage) Log.OutD("Salvage_TinkItems: Begin");

    with (Loot.TypeInfo[lootTypeUstSalv]) {
	// If no items on salvage list then exit
	if (ItemCount == 0) {
	    if (debug||debugSalvage) Log.OutD("Salvage_TinkItems: End, No items on ust salvage list, exiting...");
	    return true;
	} // endif

	if (debugSalvage) Log.OutD("Salvage_TinkItems: Verifying first salvage item exists...");
	var FirstAco = ItemList[0].Aco;
	if ((FirstAco == undefined) || (FirstAco == null) || !FirstAco || !FirstAco.fExists || !FirstAco.fInInventory) {
    	    // Item does not exist anymore or is not in our inventory so take off salvage list
    	    // by copying the last item on the list to the first item position
    	    if (debugSalvage||logSalvage) Log.OutD("Salvage_TinkItems: Item does not exist, removed from list.");
	    ItemCount--;
	    if (ItemCount > 0) ItemList[0] = ItemList[ItemCount];
	    return true;
	} // endif

	var FirstWork = FirstAco.workmanship;
	var MaterialCode = FirstAco.material;
	var Index = Loot.SalvageIndex[MaterialCode];
	var Units = FirstWork;

	if (logSalvage||debugSalvage) {
	    Log.OutD("Salvage_TinkItems: 1st item [" + FirstAco.szName + "/" + Util.Hex(FirstAco.oid) + 
		     "] wrk [" + FirstWork + "] mat [" + MaterialCode + "] indx [" + Index + "] SalCount [" +
		     ItemList[0].SalvageCount + "]");
	} // endif

	// If we have attempted to salvage the first item too many times, then skip it.  There
	// is a rare bug that causes the macro to repeated try to place an item in the UST window
	// but the item pops out immediately, thus the item is never removed from the list and is
	// attempted to be salvaged repeatedly.
	if (ItemList[0].SalvageCount >= salvageMaxAtt) {
	    // Remove first item from salvage list
	    if (logSalvage||debugSalvage) Log.OutD("Salvage_TinkItems: Attempt max reached.  Removing first item.");
	    ItemCount--;
	    TotalValue -= ItemList[0].Aco.cpyValue;
	    if (ItemCount > 0) ItemList[0] = ItemList[ItemCount];
	    return true;
	} // endif

	// Make sure we are in peace mode
	if (debugSalvage) Log.OutD("Salvage_TinkItems: Ensuring peace mode...");
	Player.EnsureCombatMode(false);

	// Make sure UST window is open by using the UST
	if (debugSalvage) Log.OutD("Salvage_TinkItems: Ensuring UST window open...");
	UstAco.Use();

	// Put first salvage item on list into UST window
	if (logSalvage||debugSalvage) Log.OutD("Salvage_TinkItems: Adding first item to UST...");
	FirstAco.AddToUst();
	ItemList[0].SalvageCount++;
	skapi.WaitEvent(salvageDelay, wemFullTimeout);
    } // end with

    // See if we can find a bag of salvage to combine this with.  Always put at least one bag
    // in window if possible, but only put more than one if total units <= 100
    var MainPackAco = skapi.acoChar;
    var BagAdded = false;

    if (debugSalvage) {
    	Log.OutD("Salvage_TinkItems: Looking for bag of partial salvage of this type.");
    	Log.OutD("Salvage_TinkItems: # of items in main pack [" + MainPackAco.citemContents + "]");
    } // endif

    for (var x=0; x<MainPackAco.citemContents; x++) {
    	var PackItemAco = skapi.AcoFromIpackIitem(0,x);
	if ((PackItemAco == undefined) || (PackItemAco == null) || !PackItemAco ||
    	    !PackItemAco.fExists || !PackItemAco.fInInventory) continue;
    	if (debugSalExt) {
    	    Log.OutD("Salvage_TinkItems: Pack Offset [" + x + "] Item [" + PackItemAco.szName + "]");
    	    Log.OutD("Salvage_TinkItems: PackItemAco.mcm [" + Util.Hex(PackItemAco.mcm) +
			"] PackItemAco.material [" + PackItemAco.material + "]");
    	    Log.OutD("Salvage_TinkItems: PackItemAco.cuseLeft [" + PackItemAco.cuseLeft +
			"] PackItemAco.workmanship [" + PackItemAco.workmanship + "]");
    	    Log.OutD("Salvage_TinkItems: Loot.SalvageFilter[" + Index + "].WorkRange[" + FirstWork + "].Low [" +
			Loot.SalvageFilter[Index].WorkRange[FirstWork].Low + "]");
  	    Log.OutD("Salvage_TinkItems: Loot.SalvageFilter[" + Index + "].WorkRange[" + FirstWork + "].High [" +
    	    		Loot.SalvageFilter[Index].WorkRange[FirstWork].High + "]");
    	    Log.OutD("Salvage_TinkItems: BagAdded [" + BagAdded + "]");
	} // endif

  	// See if this item is a salvage bag, is the same material as the first item,
    	// is not a full bag, is the combine range as the first item
    	if ((PackItemAco.mcm & mcmSalvageBag) &&
	    (PackItemAco.material == MaterialCode) &&
	    (PackItemAco.cuseLeft < 100) &&
	    (PackItemAco.workmanship >= Loot.SalvageFilter[Index].WorkRange[FirstWork].Low) &&
	    (PackItemAco.workmanship < Loot.SalvageFilter[Index].WorkRange[FirstWork].High)) {
	    	// Found a bag of salvage.  If not first bag then make sure units don't exceed 100,
	    	// else add to UST window unconditionally
	    	if (BagAdded && ((Units+PackItemAco.cuseLeft)>100)) continue;
	    	if (logSalvage||debugSalvage)
		    Log.OutD("Salvage_TinkItems: Adding partial bag [" + PackItemAco.szName + "/" +
		    	     Util.Hex(PackItemAco.oid) + "] units [" + PackItemAco.cuseLeft +
		    	     "] work [" + Util.RoundFill(PackItemAco.workmanship,2) + "] to UST...");
		PackItemAco.AddToUst();
		skapi.WaitEvent(salvageDelay, wemFullTimeout);
		Units+=PackItemAco.cuseLeft;
		BagAdded = true;
		if (Units>=100) {
		    if (debugSalvage) Log.OutD("Salvage_TinkItems: Bag caused units to exceed 100");
		    break;
		} // endif
	} // endif
    } // endfor

    // If ustMultItems flag set to tue, then while the total units in window < 100, then see if
    // any more salvage items can be added to the UST window.
    if (ustMultItems) {
	if (debugSalvage) Log.OutD("Salvage_TinkItems: Trying to fit other salvage in window...");
	for (var x=1; x<Loot.TypeInfo[lootTypeUstSalv].ItemCount; x++) {
	    // If number of units in UST window equals or exceeds 100, then we are done
	    if (Units >= 100) {
		 if (debugSalvage) Log.OutD("Salvage_TinkItems: Units [" + Units + "] >= 100, no more items can fit");
		 break;
	    } // endif
    
	    // Get next item on salvage list
	    if (debugSalvage) Log.OutD("Salvage_TinkItems: Verifying index [" + x + "] salvage item exists...");
	    var SalvageAco = Loot.TypeInfo[lootTypeUstSalv].ItemList[x].Aco;
	    if ((SalvageAco == undefined) || (SalvageAco == null) || !SalvageAco ||
		!SalvageAco.fExists || !SalvageAco.fInInventory) {
		// Item does not exist anymore so skip
		if (logSalvage||debugSalvage) Log.OutD("Salvage_TinkItems: Item [" + x + "] does not exist, skipping...");
		continue;
	    } // endif
    
	    // If we have attempted to salvage this item too many times, then skip it.  There is
	    // a rare bug that causes the macro to repeated try to place an item in the UST window
	    // but the item pops out immediately, thus the item is never removed from the list and
	    // is attempted to be salvaged repeatedly.
	    if (Loot.TypeInfo[lootTypeUstSalv].ItemList[x].SalvageCount >= salvageMaxAtt) {
		if (logSalvage||debugSalvage) Log.OutD("Salvage_TinkItems: Salmax exceeded on index [" + x + "]");
		continue;
	    } // endif
    
	    var SalvageIndex = Loot.SalvageIndex[SalvageAco.material];
	    if (debugSalExt) {
		Log.OutD("Salvage_TinkItems: Salvage Offset [" + x + "] Item [" + SalvageAco.szName +
			 "/" + Util.Hex(SalvageAco.oid) + "]");
		Log.OutD("Salvage_TinkItems: SalvageAco.mcm [" + Util.Hex(SalvageAco.mcm) +
			 "] SalvageAco.material [" + SalvageAco.material + "]");
		Log.OutD("Salvage_TinkItems: SalvageAco.workmanship [" + SalvageAco.workmanship + "]");
		Log.OutD("Salvage_TinkItems: Loot.SalvageFilter[" + SalvageIndex + "].WorkRange[" + FirstWork +
			 "].Low [" + Loot.SalvageFilter[SalvageIndex].WorkRange[FirstWork].Low + "]");
		Log.OutD("Salvage_TinkItems: Loot.SalvageFilter[" + SalvageIndex + "].WorkRange[" + FirstWork +
			 "].High [" + Loot.SalvageFilter[SalvageIndex].WorkRange[FirstWork].High + "]");
	    } // endif
    
	    // See if this item is not a salvage bag, is not the same item as the first item, is
	    // the same material as the first item, and is in the combine range as the first item
	    if (!(SalvageAco.mcm & mcmSalvageBag) &&
		(SalvageAco.oid != FirstAco.oid) &&
		(SalvageAco.material == MaterialCode) &&
		(SalvageAco.workmanship >= Loot.SalvageFilter[SalvageIndex].WorkRange[FirstWork].Low) &&
		(SalvageAco.workmanship <  Loot.SalvageFilter[SalvageIndex].WorkRange[FirstWork].High)) {
		// Move this item to UST window and increase unit count
		if (logSalvage||debugSalvage)
		    Log.OutD("Salvage_TinkItems: Adding item [" + SalvageAco.szName + "/" + Util.Hex(SalvageAco.oid) +
			     "] Work [" + SalvageAco.workmanship + "] SalCount [" +
			     Loot.TypeInfo[lootTypeUstSalv].ItemList[x].SalvageCount+ "] to Ust...");
		SalvageAco.AddToUst();
		Units+=SalvageAco.workmanship;
		Loot.TypeInfo[lootTypeUstSalv].ItemList[x].SalvageCount++;
		skapi.WaitEvent(salvageDelay, wemFullTimeout);
	    } // endif
	} // endfor
    } // endif
    
    // Now press the salvage button
    var Success = Salvage_Button(Units);
    if (debug||debugSalvage) Log.OutD("Salvage_TinkItems: End, Success [" + Success + "]");
    return Success;
} // end func


function Salvage_ValItems() {
    if (debug||debugSalvage) Log.OutD("Salvage_ValItems: Begin");

    with (Loot.TypeInfo[lootTypeValSalv]) {
	// If no items on valuable salvage list then exit
	if (ItemCount == 0) {
	    if (debug||debugSalvage) Log.OutD("Salvage_ValItems: End, No items on salvage list, exiting...");
	    return true;
	} // endif

	if (debugSalvage) Log.OutD("Salvage_ValItems: Verify first salvage item exists...");
	var FirstAco = ItemList[0].Aco;
	if ((FirstAco == undefined) || (FirstAco == null) || !FirstAco || !FirstAco.fExists || !FirstAco.fInInventory) {
	    // Item does not exist anymore or is not in our inventory so take off salvage list
	    // by copying the last item on the list to the first item position
	    if (debugSalvage) Log.OutD("Salvage_ValItems: Item does not exist, removed from list.");
	    ItemCount--;
	    if (ItemCount > 0) ItemList[0] = ItemList[ItemCount];
	    return true;
	} // endif

	var FirstWork = FirstAco.workmanship;
	var MaterialCode = FirstAco.material;
	var Value = FirstAco.cpyValue;
	var UstValue = Value/2;
	var Units = FirstWork;
	var Index = Loot.SalvageIndex[MaterialCode];

	if (logSalvage||debugSalvage) {
	    Log.OutD("Salvage_ValItems: 1st item [" + FirstAco.szName + "/" + Util.Hex(FirstAco.oid) + 
		     "] wrk [" + FirstWork + "] mat [" + MaterialCode + "] indx [" + Index + "] SalCount [" +
		     ItemList[0].SalvageCount + "]");
	} // endif

	// If we have attempted to salvage the first item too many times, then skip it.  There
	// is a rare bug that causes the macro to repeated try to place an item in the UST window
	// but the item pops out immediately, thus the item is never removed from the list and is
	// attempted to be salvaged repeatedly.
	if (ItemList[0].SalvageCount >= salvageMaxAtt) {
	    // Remove first item from salvage list
	    if (debug||debugSalvage) Log.OutD("Salvage_ValItems: Removing first item from list");
	    ItemCount--;
	    TotalValue -= ItemList[0].Aco.cpyValue;
	    if (ItemCount > 0) ItemList[0] = ItemList[ItemCount];
	    return true;
	} // endif

	// Make sure we are in peace mode
	if (debugSalvage) Log.OutD("Salvage_ValItems: Ensuring peace mode...");
	Player.EnsureCombatMode(false);

	// Make sure UST window is open by using the UST
	if (debugSalvage) Log.OutD("Salvage_ValItems: Ensuring UST window open...");
	UstAco.Use();

	// Put first salvage item on list into UST window
	if (debugSalvage||logSalvage) Log.OutD("Salvage_ValItems: Adding 1st item to UST...");
	FirstAco.AddToUst();
	ItemList[0].SalvageCount++;
	skapi.WaitEvent(salvageDelay, wemFullTimeout);
    } // end with

    // See if we can find a bag of salvage to combine this with.  Always put at least one bag
    // in window if possible, but only put more than one if total units <= 100.  Never add a
    // bag if the resulting UST operation would make the bag exceed the maximum value limitation.
    var MainPackAco = skapi.acoChar;
    var BagAdded = false;

    if (debugSalvage) {
    	Log.OutD("Salvage_ValItems: Looking for bag of partial salvage of this type.");
    	Log.OutD("Salvage_ValItems: # of items in main pack [" + MainPackAco.citemContents + "]");
    } // endif

    for (var x=0; x<MainPackAco.citemContents; x++) {
    	var PackItemAco = skapi.AcoFromIpackIitem(0,x);
    	if ((PackItemAco == undefined) || (PackItemAco == null) || !PackItemAco ||
    	    !PackItemAco.fExists || !PackItemAco.fInInventory) continue;
    	if (debugSalExt) {
    	    Log.OutD("Salvage_ValItems: Pack Offset [" + x + "] Item [" + PackItemAco.szName + "]");
    	    Log.OutD("Salvage_ValItems: PackItemAco.mcm [" + Util.Hex(PackItemAco.mcm) +
			"] PackItemAco.material [" + PackItemAco.material + "]");
    	    Log.OutD("Salvage_ValItems: PackItemAco.cuseLeft [" + PackItemAco.cuseLeft +
			"] PackItemAco.workmanship [" + PackItemAco.workmanship + "]");
    	    Log.OutD("Salvage_ValItems: Value [" + PackItemAco.cpyValue + "] BagAdded [" + BagAdded + "]");
	} // endif

  	// See if this item is a salvage bag, is the same material as the first item, is below the
  	// minimum work pickup value if we are collecting this material for tinkering,
    	// is not a full bag, and the value of the bag + item in UST window does not
    	// exceed the maximum.
    	if ((PackItemAco.mcm & mcmSalvageBag) &&
    	    (PackItemAco.material == MaterialCode) &&
    	    (PackItemAco.cuseLeft < 100) &&
    	    (!Loot.SalvageFilter[Index].Enabled ||
    	     (Loot.SalvageFilter[Index].Enabled &&
    	      (PackItemAco.workmanship < Loot.SalvageFilter[Index].MinWork))) && 
	    ((PackItemAco.cpyValue + UstValue) <= Loot.ValSalvFilter.MaxBagValue)) {
	    	// Found a bag of salvage.  If not first bag then make sure units don't exceed 100,
	    	// else add to UST window unconditionally
	    	if (BagAdded && ((Units+PackItemAco.cuseLeft)>100)) continue;
	    	if (debugSalvage||logSalvage)
		    Log.OutD("Salvage_ValItems: Adding partial bag [" + PackItemAco.szName + "/" +
		    	     Util.Hex(PackItemAco.oid) + "] units [" + PackItemAco.cuseLeft +
		    	     "] work [" + Util.RoundFill(PackItemAco.workmanship,2) + "] to UST...");
		PackItemAco.AddToUst();
		skapi.WaitEvent(salvageDelay, wemFullTimeout);
		Units+=PackItemAco.cuseLeft;
		UstValue+=PackItemAco.cpyValue;
		BagAdded = true;
		if (Units>=100) {
		    if (debugSalvage) Log.OutD("Salvage_ValItems: Bag caused units to be >= 100");
		    break;
		} // endif
	} // endif
    } // endfor

    // If ustMultItems flag set to tue, then while total units in window < 100, then see if any
    // more salvage items can be added to the UST window.  Don't add an item if the resulting value
    // of the salvage bag would exceed the maximum allowed.
    if (ustMultItems) {
	if (debugSalvage) Log.OutD("Salvage_ValItems: Trying to fit other salvage in window...");
	for (var x=1; x<Loot.TypeInfo[lootTypeValSalv].ItemCount; x++) {
	    // If number of units in UST window equals or exceeds 100, then we are done
	    if (Units >= 100) {
		 if (debugSalvage) Log.OutD("Salvage_ValItems: Units [" + Units + "] >= 100, no more items can fit");
		 break;
	    } // endif
    
	    // Get next item on salvage list
	    if (debugSalvage) Log.OutD("Salvage_ValItems: Verifying index [" + x + "] salvage item exists...");
	    var SalvageAco = Loot.TypeInfo[lootTypeValSalv].ItemList[x].Aco;
	    if ((SalvageAco == undefined) || (SalvageAco == null) || !SalvageAco ||
		!SalvageAco.fExists || !SalvageAco.fInInventory) {
		// Item does not exist anymore so skip
		if (logSalvage||debugSalvage) Log.OutD("Salvage_ValItems: Item [" + x + "] does not exist, skipping...");
		continue;
	    } // endif
    
	    // If we have attempted to salvage this item too many times, then skip it.  There is
	    // a rare bug that causes the macro to repeated try to place an item in the UST window
	    // but the item pops out immediately, thus the item is never removed from the list and
	    // is attempted to be salvaged repeatedly.
	    if (Loot.TypeInfo[lootTypeValSalv].ItemList[x].SalvageCount >= salvageMaxAtt) {
		if (logSalvage||debugSalvage) Log.OutD("Salvage_ValItems: Salmax exceeded on index [" + x + "]");
		continue;
	    } // endif
    
	    // Check next item in salvage list
	    var SalvageIndex = Loot.SalvageIndex[SalvageAco.material];
	    if (debugSalExt) {
		Log.OutD("Salvage_ValItems: Salvage Offset [" + x + "] Item [" + SalvageAco.szName +
			 "/" + Util.Hex(SalvageAco.oid) + "]");
		Log.OutD("Salvage_ValItems: SalvageAco.mcm [" + Util.Hex(SalvageAco.mcm) +
			 "] SalvageAco.material [" + SalvageAco.material + "]");
		Log.OutD("Salvage_ValItems: SalvageAco.workmanship [" + SalvageAco.workmanship +
			 "] Value [" + SalvageAco.cpyValue + "]");
	    } // endif
    
	    // Make sure that this item is not a salvage bag, is not the same item as the first item, is
	    // the same material as the first item, and the resulting value of the salvage bag does
	    // not exceed the maximum allowed
	    if (!(SalvageAco.mcm & mcmSalvageBag) &&
		(SalvageAco.oid != FirstAco.oid) &&
		(SalvageAco.material == MaterialCode) &&
		(((SalvageAco.cpyValue/2)+UstValue) <= Loot.ValSalvFilter.MaxBagValue)) {
		// Move this item to UST window and increase unit/value count
		if (debugSalvage||logSalvage)
		    Log.OutD("Salvage_ValItems: Adding item [" + SalvageAco.szName + "/" + Util.Hex(SalvageAco.oid) +
			     "] Work [" + SalvageAco.workmanship + "] to Ust...");
		SalvageAco.AddToUst();
		Units+=SalvageAco.workmanship;
		UstValue+=(SalvageAco.cpyValue/2);
		Loot.TypeInfo[lootTypeValSalv].ItemList[x].SalvageCount++;
		skapi.WaitEvent(salvageDelay, wemFullTimeout);
	    } // endif
	} // endfor
    } // endif

   // Now press the salvage button
    var Success = Salvage_Button(Units);
    if (debug||debugSalvage) Log.OutD("Salvage_ValItems: End, Success [" + Success + "]");
    return Success;
} // end func

function Push_BuySell_Button() {
    var VertAdj = skapi.clrPixel(-321,-88) ? skapi.dypWindow/4-34 : 0; //
    var HorzOffset = skapi.clrPixel(-318,-184-VertAdj) ? -33 : -342;
    var VertOffset = -131 - VertAdj;
    Chat.Screen("VertAdj: " + VertAdj + "  HorzOffset: " + HorzOffset + "  VertOffset: " + VertOffset);
    skapi.MouseMove(HorzOffset,VertOffset);    // Move mouse to buy or sell button (same location)
    skapi.MouseButton(ibuttonLeft);
} // endif

function Salvage_Button(Units) {
    if (debug||debugSalvage) Log.OutD("Salvage_Button: Begin");
    // Determine location of salvage button and press it.  Note there are 4 possible
    // locations based on whether chat area expanded or not, and UI displayed or not.
    // First get vertical adjustment (math/coords borrowed from AutoLoot script by
    // David A. Down, Merlyn of Harvestion), then get the horizontal offset.
    // var VertAdj = skapi.clrPixel(-321,-88) ? skapi.dypWindow/4-34 : 0; //
    // var HorzOffset = skapi.clrPixel(-318,-184-VertAdj) ? -57 : -368;
    
    // Move mouse to salvage button
    // skapi.MouseMove(skapi.xpRight3D-36, skapi.ypBottom3D+54);    

    // Now press the left mouse button and wait for the OnAddToInventory event
    // which indicates the UST was successful.
    SalvageActive = true;
    InventoryAdd = false;
    var UstStart = new Date();
    if (debugSalvage) Log.OutD("Salvage_Button: Pressing salvage button...");
    // skapi.MouseButton(ibuttonLeft);
    // skapi.WaitEvent(salvageDelay, wemFullTimeout);
    skapi.Salvage();

    // Now wait for the inventory add event or the ust operation to timeout
    while (!InventoryAdd && ((new Date()-UstStart)<salvageUstWait)) skapi.WaitEvent(20, wemFullTimeout);

    SalvageActive = false;
    if (InventoryAdd) {
	if (debugSalvage||logSalvage) Log.OutD("Salvage_Button: USTing of [" + Units + "] units succeeded");
    } else {
    	// UST operation timed out, reset UST window by going into war mode
	if (debugSalvage||logSalvage)
	    Log.OutD("Salvage_TinkItems: USTing of [" + Units + "] units failed, resetting UST window...");
	skapi.WaitEvent(1000, wemFullTimeout);	// Wait one second.
	Player.EnsureCombatMode(true);		// Go into combat mode to close UST window.
	skapi.WaitEvent(1000, wemFullTimeout);	// Wait one second.
    } // endif

    if (debug||debugSalvage) Log.OutD("Salvage_Button: End, return code [" + InventoryAdd + "]");
    return InventoryAdd;
} // end function


function SortBags(SortFull) {
    if ((SortFull == undefined) || (SortFull == null)) SortFull = true;

    // Define filter and then let skunkworks build ACO list.
    AbortAction = false;
    var BagAcf = skapi.AcfNew();
    BagAcf.mcm = mcmSalvageBag;
    BagAcf.olc = olcInventory;

    if (SortFull) {
	var Desc = "full";
	BagAcf.cuseLeftMin = 100;
	BagAcf.cuseLeftMax = 100;
    } else {
	var Desc = "partial";
	BagAcf.cuseLeftMin = 1;
	BagAcf.cuseLeftMax = 99;
    } // endif

    // Now see if user has selected a side pack or an item in a side pack.  If so then
    // sort the contents of this pack instead if the main pack.
    var PackNum = 0;
    var SelAco = skapi.acoSelected;
    if ((SelAco != null) && (SelAco.fInInventory)) {
    	// Chat.Screen("Selected aco [" + SelAco.szName + "/" + Util.Hex(SelAco.oid) + "]");
    	// if (SelAco.acoContainer != null) Chat.Screen("Sel Container [" + Util.Hex(SelAco.acoContainer.oid) + "]");
    	// Compare item to each side pack aco
    	for (var x=1; x<skapi.cpack; x++) {
	    PackAco = skapi.AcoFromIpackIitem(x,-1);
    	    // Chat.Screen("Checking pack [" + PackAco.szName + "/" + Util.Hex(PackAco.oid) + "]");
	    // If item is one of our side packs, then match found
    	    if (SelAco.oid == PackAco.oid) break;
	    // If item is in one of the side packs, then match found
	    if ((SelAco.acoContainer != null) && (SelAco.acoContainer.oid == PackAco.oid)) break;
	} // endfor
	if (x<skapi.cpack) PackNum = x;
    } // endif

    // Now build the salvage bag collection using our filter. Only get items in one pack as computed above.
    // Chat.Screen("Computed pack num [" + PackNum + "]");
    if (PackNum == 0) PackAco = skapi.AcoFromIPackIitem(0,-1);
    var BagColl = BagAcf.CoacoGet(PackAco.coacoContents);

    if (BagColl.Count == 0) {
	Chat.Screen("No " + Desc + " bags of salvage found in pack # " + PackNum);
	return;
    } // endif

    Chat.Screen("Sorting " + BagColl.Count + " " + Desc + " bags of salvage in pack # " + PackNum);
    BagColl.Sort(new SortbyMaterialWork);

    // Move bags in reverse order so bags end up in increasing work order in main pack.
    var Count = 0;
    for (var x=BagColl.Count-1; x>=0; x--) {
    	var BagAco = BagColl.Item(x);
    	// If this bag is not a tinker salvage bag, then it's a valuable salvage bag
    	// so don't sort it.
    	var Index = Loot.SalvageIndex[BagAco.material];
    	if (!(Loot.SalvageFilter[Index].Enabled && (BagAco.workmanship >= Loot.SalvageFilter[Index].MinWork))) continue;

	// Good bag found, move it
	var Status = MoveItem(BagAco, PackNum);
	if (AbortAction) {
	    Chat.Screen("Sort aborted.");
	    return;
	} // endif
	if (Status)
	    Count++;
	else
	    Chat.Screen("Unable to move " + Desc + " bag [" + BagColl.Item(x).szName + "]");
    } // endfor
    Chat.Screen(Count + " " + Desc + " bags sorted.");
} // end function

// Following function moves salvage bags from main pack to any side pack
function MoveBags(ValOnly, Verbose) {
    if ((ValOnly == undefined) || (ValOnly == null)) ValOnly = false;
    if ((Verbose == undefined) || (Verbose == null)) Verbose = false;
    if (debug||debugSalvage) Log.OutD("MoveBags: Begin");

    var MainPackAco = skapi.acoChar;

    var BagAcf = skapi.AcfNew();
    BagAcf.mcm = mcmSalvageBag;
    BagAcf.olc = olcInventory;
    
    // If not all bags wanted, then just do full ones
    if (!ValOnly) {
	BagAcf.cuseLeftMin = 100;
	BagAcf.cuseLeftMax = 100;
    } // endif

    // Build bag collection from main pack only
    var BagColl = BagAcf.CoacoGet(skapi.acoChar.coacoContents);
    
    // Sort by reverse value so that most expensive bags are moved first
    BagColl.Sort(skapi.sortbyReverse(skapi.sortbyValue));

    // If no salvage found, just return
    if (BagColl.Count == 0) {
    	if (Verbose) Chat.Screen("No full bags of salvage in main pack.");
    	MoveCount = -1;
	return;
    } // endif
    
    // Now build array of free space in each side pack
    var SideFree = new Array();
    var HighSide = 0;
    for (var y=1; y < skapi.cpack; y++) {
	var PackAco = skapi.AcoFromIpackIitem(y,-1);
	SideFree[y] = PackAco.citemMax - PackAco.citemContents;
	if (SideFree[y] > 0) HighSide = y;
    } // endfor

    if (HighSide == 0) {
	if (Verbose) Chat.Screen("No room in side packs.");
	MoveCount = -1;
	return;
    } // endif

    // Move bags in collection to side pack
    MoveCount = 0;
    for (var x=0; x<BagColl.Count; x++) {
	if (HighSide == 0) {
	    if (Verbose) Chat.Screen("No more room in side packs.");
	    return;
	} // endif

	var BagAco = BagColl.Item(x);
   	// If valuable only flag set, then see if this bag is a tinker
   	// salvage bag (not a valuable salvage bag).  If so then skip it.
   	if (ValOnly) {
	    var Index = Loot.SalvageIndex[BagAco.material];
	    if (Loot.SalvageFilter[Index].Enabled && (BagAco.workmanship >= Loot.SalvageFilter[Index].MinWork)) continue;
	} // endif

	if (BagAco.acoContainer != MainPackAco) {
	    // With the filtering there should be no possible way that this bag is not
	    // in the main pack, but put this check in anyway because of a looping problem
	    Log.OutD("MoveBags: Not in main pack [" + BagAco.szName + "/" + Util.Hex(BagAco.oid) + "]");
	    continue;
	} // endif

	// Move this bag to a side pack.  Search from bottom side pack up to top
	if (debugSalvage) Log.OutD("MoveBags: Moving bag [" + BagAco.szName + "/" + Util.Hex(BagAco.oid) + "]");
	for (var y=HighSide; y>0; y--) {
	    if (SideFree[y] > 0) break;
	    HighSide = y - 1;	// If pack full, no sense checking it again for next bag of salvage
	} // endfor

	// If side pack space available, then move it there
	if (y>0) {
	    Log.OutD("MoveBags: Moving [" + BagAco.szName + "/" + Util.Hex(BagAco.oid) + "] to side pack [" + y + "]");
	    MoveActive = true;
	    var MoveStart = new Date();
	    BagAco.MoveToPack(y);
	    skapi.WaitEvent(salvageDelay, wemFullTimeout);
	    // Now wait for the inventory add event or the ust operation to timeout
	    while (!InventoryAdd && ((new Date()-MoveStart)<salvageMoveWait)) skapi.WaitEvent(20, wemFullTimeout);
	    MoveActive = false;
    	    if (InventoryAdd) {
    		// Move succeeded.  Wait a few more milliseconds so we don't get too busy messages
    		skapi.WaitEvent(100, wemFullTimeout);
    		SideFree[y]--;
    		MoveCount++;
    		if (debugSalvage) Log.OutD("MoveBags: Move succeeded");
    	    } else {
    		// Move failed
    		if (debugSalvage) Log.OutD("MoveBags: End, move failed");
    		return false;
    	    } // endif
	} else {
	    // No more space left in side packs so just return
	    if (debugSalvage) Log.OutD("MoveBags: End, no side pack space left");
	    return false;
	} // endif
    } // endfor
    if (debugSalvage) Log.OutD("MoveBags: End");
    return true;
} // end func


function GiveBags() {
    // Now see if someone is selected
    var SelAco = skapi.acoSelected;
    if (SelAco == null) {
	Chat.Screen("No player selected to give salvage.");
	return;
    } // endif

    if (!(SelAco.ocm & ocmPlayer)) {
	Chat.Screen("Current selected object is not a player");
	return;
    } // endif

    if (SelAco.oid == skapi.acoChar.oid) {
	Chat.Screen("Can't give to ourself!");
	return;
    } // endif

    var PlayerDist = Util.Distance(skapi.maplocCur, SelAco.maploc);
    if (PlayerDist > giveMaxDist) {
	Chat.Screen("Current selected object is too far away, come within " + giveMaxDist + " map units");
    } // endif

    var BagAcf = skapi.AcfNew();
    BagAcf.mcm = mcmSalvageBag;
    BagAcf.olc = olcInventory;
    BagAcf.cuseLeftMin = 100;
    BagAcf.cuseLeftMax = 100;

    // Now build the full salvage bag collection using our filter
    var BagColl = BagAcf.CoacoGet();

    if (BagColl.Count == 0) {
	Chat.Screen("No full bags of salvage found.");
	return;
    } // endif

    // Now sort bags of salvage by sort priority by material by workmanship
    // Chat.Screen("Sorting bags by material by workmanship...");
    BagColl.Sort(new SortbyMaterialWork);

    // Give bags to selected person.  Give in reverse order so bags end up
    // in increasing workmanship order in recipients pack.
    var Count = 0;
    var TinkCount = 0;
    AbortAction = false;

    for (var x=BagColl.Count-1; x>=0; x--) {
    	var BagAco   = BagColl.Item(x);
    	var Material = BagAco.material;
	var Work     = BagAco.workmanship;
    	var Index    = Loot.SalvageIndex[Material];
    	
    	// If this is a valuable salvage bag then don't give.
   	if (!(Loot.SalvageFilter[Index].Enabled && (Work >= Loot.SalvageFilter[Index].MinWork))) {
	    continue;
	} // endif
    	TinkCount++;

	// Now try to give bag to person selected
	var Status = GiveItem(BagAco, SelAco, false);
	if (AbortAction) {
	    Chat.Screen("Give Aborted!");
	    return;
	} // endif
	if (Status)
	    Count++;
	else
	    Chat.Screen("Unable to give bag [" + BagColl.Item(x) + "]");
    } // endfor
    
    if (TinkCount == 0) {
    	Chat.Screen("No tinker full bags of salvage found.");
    } else {
    	Chat.Screen(Count + " bags given.");
    } // endif
} // end func

function BuildValSalList() {
    // First get a collection of all bags of salvage in our inventory
    var BagAcf = skapi.AcfNew();
    BagAcf.mcm = mcmSalvageBag;
    BagAcf.olc = olcInventory;
    var BagColl = BagAcf.CoacoGet();
    
    // Next add all valuable salvage bags to a new collection and keep track of the total value
    ValBagColl = skapi.CoacoNew();
    ValBagTotal = 0;
    for (var x=0; x<BagColl.Count; x++) {
    	var BagAco   = BagColl.Item(x);
    	var Material = BagAco.material;
	var Work     = BagAco.workmanship;
    	var Index    = Loot.SalvageIndex[Material];
	if (Loot.SalvageFilter[Index].Enabled && (Work >= Loot.SalvageFilter[Index].MinWork)) {
	    // This bag is tinker salvage bag, don't add to collection
	    continue;
	} // endif
	ValBagColl.AddAco(BagAco);
	ValBagTotal += BagAco.cpyValue;
    } // endfor

    // All Done
    return true;
} // end func


function BuildPeaList() {
    var PeaAcf = skapi.AcfNew();
    PeaAcf.mcm = mcmSpellComponents;
    PeaAcf.olc = olcInventory;

    PeaListColl = skapi.CoacoNew();
    for (var x=0; x<peaName.length; x++) {
	PeaAcf.szName = peaName[x] + " Pea";
	var PeaColl = PeaAcf.CoacoGet(skapi.acoChar.coacoContents);
	for (var y=0; y<PeaColl.Count; y++) {
    	    var PeaAco = PeaColl.Item(y);
    	    PeaListColl.AddAco(PeaAco);
	} // endfor
    } // end for
} // end func


function GiveValSalvage(SelAco) {

    // Now sort bags of valuable salvage by value
    // Chat.Screen("Sorting bags by value...");
    ValBagColl.Sort(skapi.sortbyValue);

    // Give bags to selected person.  Give in reverse order so bags end up
    // in decreasing value in recipients pack.
    var Count = 0;
    AbortAction = false;

    for (var x=ValBagColl.Count-1; x>=0; x--) {
    	var BagAco = ValBagColl.Item(x);
    	
	// Now try to give bag to person selected
	var Status = GiveItem(BagAco, SelAco, false);
	if (AbortAction) return false;

	if (Status)
	    Count++;
	else
	    Chat.Screen("Unable to give bag [" + BagAco.szName + "]");
    } // endfor

    Chat.Screen(Count + " bags given.");
    return true;
} // end func


function GivePeas(SelAco) {
    // Now sort peas by value
    PeaListColl.Sort(skapi.sortbyValue);

    // Give bags to selected person.  Give in reverse order so bags end up
    // in decreasing value in recipients pack.
    var Count = 0;
    AbortAction = false;

    for (var x=PeaListColl.Count-1; x>=0; x--) {
    	var PeaAco = PeaListColl.Item(x);
    	
	// Now try to give pea to person selected
	var Status = GiveItem(PeaAco, SelAco, false);
	if (AbortAction) return false;

	if (Status) {
	    Count++;
	} else {
	    Chat.Screen("Unable to give pea [" + PeaAco.szName + "]");
	} // endif
    } // endfor

    Chat.Screen(Count + " peas given.");
    return true;
} // end func


function FoundInMainPack(Name) {
    // If an item called 'Name' exists in our main pack which has not reached its maximum
    // stack amount, then return true, else false;
    var NameAcf    = skapi.AcfNew();
    NameAcf.szName = Name;
    NameAcf.olc    = olcInventory;

    // Build collection from main pack only
    var NameColl = NameAcf.CoacoGet(skapi.acoChar.coacoContents);

    // If no salvage found, just return
    if (NameColl.Count == 0) {
	return false;
    } // endif

    // Go through each item of the same name looking for not full stack
    for (var x=0; x<NameColl.Count; x++) {
	var NameAco = NameColl.Item(x);
	if (NameAco.citemStack < NameAco.citemMaxStack) return true;
    } // endfor

    // Not addable stack found
    return false;
} // endif


function FindManaStone() {
    for (var x=manaStoneLimit; x<manaCnt; x++) {
	var Stone = skapi.AcoFindInInv(manaName[x] + " Mana Charge");
	if (Stone != null) return Stone;
    } // endfor
    return null;
} // end func


// This function allows macro to send support requests and/or notification messages
// to friendly chars through @tells and/or @fellow commands
function NotifySupport(request) {
    if (flagFellowSup && Player.MemberFellow) Chat.Fellow(request);
    if (flagFriendSup) {
    	for (var x=0; x<supportChars.length; x++) Chat.Tell(supportChars[x], request);
    } // endif
} // end function


function SpellFailText() {
    var Desc = "CC: " + CastCode + " CF: " + CastFail + "/" + castFailText[CastFail] + " CF2: " +
		CastFail2 + " A: " + CastArc + " FC: " + Cast.FailCount + " CFC: " + CastFailCnt;
    return Desc;
} // end func


// For debugging
function ListPrev() {
    Chat.Screen("PrevCorpseNext = [" + PrevCorpseNext + "]");
    for (var x=0; x<lootPrevCorpseMax; x++)
	if (PrevCorpseList[x] != 0) Chat.Screen("PrevCorpseList[" + x + "] = [" + PrevCorpseList[x] + "]");
} // endif


function PauseMacro() {
    if (debug||debugState) Log.OutD("Main: Pause Begin");
    Log.OutDM("Main", "Macro paused.", ToChat);

    var PauseStart = new Date();

    // Spin until resume command detected
    while (MacroPaused) {
    	skapi.WaitEvent(20);
    	if (flagAutoResume && ((new Date()-PauseStart) > autoResumeInt)) {
    	    Log.OutDM("Main", "Auto resume ending paused state...", ToChat);
    	    MacroPaused = false;
    	    skapi.SetControlProperty(szPanelControl, "btnPause", "Text", "Pause");
    	} // endif
    } // end while

    // If we get here then macro was resumed
    Log.OutDM("Main", "Macro resumed.", ToChat);

    // Pause flag cleared, resume macro.
    // Clear evade war flag so we don't move coming out of Paused state
    EvadeWar = false;

    // If we died recently then reset various parameters now that a human has manually intervened
    if (Dead) DeathReset();

    // Now warn macroer if current attack location is not close to our current position
    var HomeLocDist = Util.Distance(skapi.maplocCur, HomeLoc);
    if (HomeLocDist > navWarnDist) {
	Chat.Screen("WARNING!!! Current attack location is " + Util.Round(HomeLocDist,3) + " map units away!");
    } // endif

    if (PeaceMode) Chat.Screen("WARNING!!! PeaceMode enabled, no creatures will be attacked!", cmcYellow);
    if (!Stealth)  Chat.Screen("WARNING!!! Stealth mode disabled!", cmcYellow);
    if (!BuffMode) Chat.Screen("WARNING!!! Buffing disabled!", cmcYellow);
    if (debug||debugState) Log.OutD("Main: Pause End");
} // end func

function FrenchMode() {
    if (debugMove||logMove) Log.OutD("FrenchMode: Begin, Moving to retreat location.");

    // If we are too far away from the retreat location, then don't go.
    var TravelDist = Util.Distance(AttLoc[AttLocCur].RetreatLoc,skapi.maplocCur);
    if (TravelDist > retreatMaxDist) {
    	// Too far away.  Usually happens when we are killed and a fellow partner requests a retreat for our movegroup.
    	Chat.Screen("Retreat requested by we are too far away from retreat location. Aborted!");
    	if (logMove||debugMove) Log.OutD("FrenchMode: Too far [" + TravelDist + "] away from retreatloc, aborted.");
    	RetreatReq = false;
    	return;
    } // endif
    
    AttLoc[AttLocCur].TotalRetreats++;

    Chat.Screen("Moving to retreat location...");
    if (flagMultRetreat && Player.MemberFellow() && (PlayerNearCnt > 0) && (MoveGroup>0))
	Chat.Fellow("RETREAT " + MoveGroup);

    // Compute nav timeout based on distance we have to travel.
    var NavTimeOut = retreatTimeoutConst + ((TravelDist/0.1)*retreatTimeoutVar);
    if (debugMove) Log.OutD("FrenchMode: Retreat timeout [" + NavTimeOut + "]");

    // Init some variables related to move
    MoveDone = false;
    MoveOK = true;
    NoMoveChecked = false;	// Only need to check for no movement once
    MoveStart = new Date();
    StartLoc = skapi.maplocCur;

    // Now start the move to the retreat location
    NavActive = true;
    skapi.GoToMaploc(AttLoc[AttLocCur].RetreatLoc, navPrecision);

    // Wait for move to be successful or attempt to timeout.  Note the MoveDone variable
    // is also settable by the NavStop event
    while (!MoveDone) {
	// Read events.  Pause a bit so we don't eat up CPU
	skapi.WaitEvent(10);
	if (flagRecordLoc) RecordLoc();

	// See if navigation is taking too long.  Cancel attempt if so.
	if ((new Date()-MoveStart)>NavTimeOut) {
	    Chat.Screen("Navigation timed out, canceling nav...");
	    if (debugMove||logMove) Log.OutD("FrenchMode: Navigation timed out, canceling nav...");
	    MoveOK = false;
	    skapi.CancelNav();
	    MoveDone = true;
	} // endif

	// See if we are not moving anywhere.  This will happen if we are surrounded
	if (!NoMoveChecked && ((new Date()-MoveStart)>retreatNoMoveCheckTime)) {
	    NoMoveChecked = true;
	    var MovedDist = Util.Distance(StartLoc,skapi.maplocCur);
	    if (MovedDist < retreatNoMoveDist) {
		Chat.Screen("Not making any progress, canceling nav...");
		if (debugMove||logMove) Log.OutD("FrenchMode: Not making any progress, canceling nav...");
		MoveOK = false;
		skapi.CancelNav();
		MoveDone = true;
	    } // endif
	} // endif
    } // endwhile

    NavActive = false;
    RetreatLastAtt = new Date();

    if (MoveOK) {
	if (debugMove||logMove) Log.OutD("Main: Retreat done, status [" + MoveCode + "/" + textNscCode[MoveCode] + "]");
	if (MoveCode == nscCanceled) {
	    Chat.Screen("Navigation aborted.");
	    if (debugMove||logMove) Log.OutD("FrenchMode: Navigation aborted");
	} else {
	    HomeLoc = AttLoc[AttLocCur].RetreatLoc;
	    RetreatActive = true;
	    if (flagMoveDelay) {
		Chat.Screen("Arrived at retreat location. Pausing for " + navWait + " ms...");
		if (debugMove||logMove) Log.OutD("FrenchMode: Arrived at retreat loc. Pausing for " + navWait + " ms...");
		skapi.WaitEvent(navWait, wemFullTimeout);
	    } else {
		Chat.Screen("Arrived at retreat location.");
		Log.OutD("FrenchMode: Arrived at retreat location.");
	    } // endif
	} // endif
    } // endif
    
    RetreatReq = false;
    if (debugMove||logMove) Log.OutD("FrenchMode: End");
} // end func


function DisplayComp(CompIndex) {
    with (Comp.Info[CompIndex]) {
	skapi.SetControlProperty(szPanelConfig, "chkCompEnabled", "checked",	Enabled		);
	skapi.SetControlProperty(szPanelConfig, "textComp1v",     "text", 	WarnLvl		);
	skapi.SetControlProperty(szPanelConfig, "textComp2v",     "text", 	SplitLvl	);
	skapi.SetControlProperty(szPanelConfig, "textComp3v",     "text", 	CritLvl		);
	skapi.SetControlProperty(szPanelConfig, "textComp4v",     "text", 	BuyLvl		);
    } // end with
} // end func


function DisplayAttLoc(Index) {
    with (AttLoc[Index]) {
	if (LocFlags & navFlagSpawnAdv) {
	    skapi.SetControlProperty(szPanelMovement, "chkSpawnAdvance", "checked", true);
	} else {
	    skapi.SetControlProperty(szPanelMovement, "chkSpawnAdvance", "checked", false);
	} // end if
    } // end with
} // end func


function DisplayBuffCre(BuffIndex) {
    with (Buff.Info[BuffIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffCreEnab",    "checked",	Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffCreRandom",  "checked",	Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff2b", 	    "selected",	7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text2cv", 	    "text",	Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text2dv", 	    "text",	Priority   );
    } // end width
} // end func


function DisplayBuffLife(BuffIndex) {
    with (Buff.Info[BuffIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffLifeEnab",   "checked",	Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffLifeRandom", "checked",	Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff3b", 	    "selected",	7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text3cv", 	    "text",	Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text3dv", 	    "text",	Priority   );
    } // end width
} // end func


function DisplayBuffHerm() {
    with (Buff.Info[BuffHermIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffHermEnab",   "checked",	Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffHermRandom", "checked",	Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff1b", 	    "selected",	7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text1cv", 	    "text",	Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text1dv", 	    "text",	Priority   );
    } // end width
} // end func


function DisplayBuffDef() {
    with (Buff.Info[BuffDefIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffDefEnab",   "checked",	Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffDefRandom", "checked",	Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff5b", 	    "selected",	7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text5cv", 	    "text",	Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text5dv", 	    "text",	Priority   );
    } // end width
} // end func

function DisplayBuffSpirit() {
    with (Buff.Info[BuffSpiritIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffSpEnab",     "checked",	Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffSpRandom",   "checked",	Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff5f", 	    "selected",	7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text5gv", 	    "text",	Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text5hv", 	    "text",	Priority   );
    } // end width
} // end func

function DisplayBuffArmor(BuffIndex) {
    with (Buff.Info[BuffIndex]) {
	skapi.SetControlProperty(szPanelBuffs, "chkBuffArmorEnab",   "checked",	 Enabled    );
	skapi.SetControlProperty(szPanelBuffs, "chkBuffArmorRandom", "checked",	 Randomize  );
	skapi.SetControlProperty(szPanelBuffs, "choiceBuff4b", 	     "selected", 7-Level    );
	skapi.SetControlProperty(szPanelBuffs, "text4cv", 	     "text",	 Safety/1000);
	skapi.SetControlProperty(szPanelBuffs, "text4dv", 	     "text",	 Priority   );

	skapi.SetControlProperty(szPanelBuffs, "chkPaperDoll",	 "checked", (Eqm & eqmPaperDoll)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffHead",	 "checked", (Eqm & eqmHead)		);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffHands",	 "checked", (Eqm & eqmHands)		);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffFeet",	 "checked", (Eqm & eqmFeet)		);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffChest",	 "checked", (Eqm & eqmChestOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffUpArms",  "checked", (Eqm & eqmUpperArmsOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffLowArms", "checked", (Eqm & eqmLowerArmsOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffAbdomen", "checked", (Eqm & eqmAbdomenOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffUpLegs",	 "checked", (Eqm & eqmUpperLegsOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffLowLegs", "checked", (Eqm & eqmLowerLegsOuter)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffShirt",	 "checked", (Eqm & eqmChestUnder)	);
	skapi.SetControlProperty(szPanelBuffs, "chkBuffPants",	 "checked", (Eqm & eqmUpperLegsUnder)	);

    } // end width
} // end func


function DisplayLootType(typeIndex) {
    with (Loot.TypeInfo[typeIndex]) {
	skapi.SetControlProperty(szPanelLoot, "chkTypeEnab",	 "checked",	Enabled);
	skapi.SetControlProperty(szPanelLoot, "lootMaxPick", 	 "selected",	MaxCount);
	skapi.SetControlProperty(szPanelLoot, "lootMinSlot",	 "selected",	MinSlots);
    } // end with
} // end func

function DisplayLootMajor(majorIndex) {
    with (Loot.MajorFilter[majorIndex]) {
	skapi.SetControlProperty(szPanelLoot, "chkMajorEnab", "checked",	Enabled);
    } // end with
} // end func


function DisplayLootMinor(minorIndex) {
    with (Loot.MinorFilter[minorIndex]) {
	skapi.SetControlProperty(szPanelLoot, "chkMinorEnab",		"checked", Enabled);
	skapi.SetControlProperty(szPanelLoot, "chkMinorHead",		"checked", (Eqm & eqmHead)		);
	skapi.SetControlProperty(szPanelLoot, "chkMinorHands",		"checked", (Eqm & eqmHands)		);
	skapi.SetControlProperty(szPanelLoot, "chkMinorFeet",		"checked", (Eqm & eqmFeet)		);
	skapi.SetControlProperty(szPanelLoot, "chkMinorChest",		"checked", (Eqm & eqmChestOuter)	);
	skapi.SetControlProperty(szPanelLoot, "chkMinorUpArms",		"checked", (Eqm & eqmUpperArmsOuter)	);
	skapi.SetControlProperty(szPanelLoot, "chkMinorLowArms",	"checked", (Eqm & eqmLowerArmsOuter)	);
	skapi.SetControlProperty(szPanelLoot, "chkMinorAbdomen",	"checked", (Eqm & eqmAbdomenOuter)	);
	skapi.SetControlProperty(szPanelLoot, "chkMinorUpLegs",		"checked", (Eqm & eqmUpperLegsOuter)	);
	skapi.SetControlProperty(szPanelLoot, "chkMinorLowLegs",	"checked", (Eqm & eqmLowerLegsOuter)	);
	skapi.SetControlProperty(szPanelLoot, "chkMinorShirt",		"checked", (Eqm & eqmChestUnder)	);
	skapi.SetControlProperty(szPanelLoot, "chkMinorPants",		"checked", (Eqm & eqmUpperLegsUnder)	);
    } // end with
} // end func


function DisplayLootArmor(Index) {
    with (Loot.ArmorFilter[Index]) {
	skapi.SetControlProperty(szPanelLoot, "chkArmorEnab",		"checked",   Enabled     );
	skapi.SetControlProperty(szPanelLoot, "text3av", 		"text",	     MinAL       );
	skapi.SetControlProperty(szPanelLoot, "text3bv",		"text",	     MaxBU       );
	skapi.SetControlProperty(szPanelLoot, "text3cv",		"text",	     MaxValue    );
	skapi.SetControlProperty(szPanelLoot, "choiceLoot3d",		"selected",  MaxWork-1   );
	skapi.SetControlProperty(szPanelLoot, "choiceLoot3e",		"selected",  MinSpells   );
	skapi.SetControlProperty(szPanelLoot, "choiceLoot3f",		"selected",  7-MinSplLvl );
    } // end with
} // end func


function DisplayLootWeaponM(Index) {
    with (Loot.WeaponMFilter[Index]) {
	skapi.SetControlProperty(szPanelLoot, "textMWhead1v", 	  "text",	AbsDamMax   );
	skapi.SetControlProperty(szPanelLoot, "textMWhead2v", 	  "text",	AbsAttMax   );
	skapi.SetControlProperty(szPanelLoot, "textMWhead3v", 	  "text",	AbsDefMax   );
       	skapi.SetControlProperty(szPanelLoot, "chkMWenab",	  "checked",	Enabled     );
	skapi.SetControlProperty(szPanelLoot, "choiceLootMWwork", "selected",	MaxWork-1   );
	skapi.SetControlProperty(szPanelLoot, "choiceLootMWdam",  "selected",	55-MinDam   );
	skapi.SetControlProperty(szPanelLoot, "choiceLootMWatt",  "selected",	MinAtt	    );
	skapi.SetControlProperty(szPanelLoot, "choiceLootMWdef",  "selected",	MinDef	    );
	skapi.SetControlProperty(szPanelLoot, "choiceLootMWand",  "selected",	AndOrFlag   );
    } // end with
} // end func

function DisplayLootWeaponR(Index) {
    with (Loot.WeaponRFilter[Index]) {
	skapi.SetControlProperty(szPanelLoot, "textRWhead1v", 	  "text",	AbsDamMax  );
	skapi.SetControlProperty(szPanelLoot, "textRWhead2v", 	  "text",	AbsDefMax  );
	skapi.SetControlProperty(szPanelLoot, "textRWhead3v", 	  "text",	AbsElemMax );
       	skapi.SetControlProperty(szPanelLoot, "chkRWenab",	  "checked",	Enabled    );
	skapi.SetControlProperty(szPanelLoot, "choiceLootRWwork", "selected",	MaxWork-1  );
	skapi.SetControlProperty(szPanelLoot, "choiceLootRWdam",  "selected",	155-MinDam );
	skapi.SetControlProperty(szPanelLoot, "choiceLootRWdef",  "selected",	MinDef	   );
	skapi.SetControlProperty(szPanelLoot, "choiceLootRWelem", "selected",	MinElem    );
	skapi.SetControlProperty(szPanelLoot, "choiceLootRWand",  "selected",	AndOrFlag  );
    } // end with
} // end func

function DisplayLootSalvage(Index) {
    with (Loot.SalvageFilter[Index]) {
	skapi.SetControlProperty(szPanelLoot, "chkSalvEnab",	"checked",   Enabled	);
	skapi.SetControlProperty(szPanelLoot, "chkExcludeVal",  "checked",   ExcludeVal );
	skapi.SetControlProperty(szPanelLoot, "choiceLoot5a",	"selected",  MinWork-1	);
	skapi.SetControlProperty(szPanelLoot, "text5bv",	"text",	     MaxBU	);
	skapi.SetControlProperty(szPanelLoot, "text5cv",	"text",	     SortPrio	);

	for (var x=1; x<11; x++)
	    skapi.SetControlProperty(szPanelLoot, "chkSal"+x,	"checked", (CombineMask & combineMaskVal[x]) );

	skapi.SetControlProperty(szPanelLoot, "text5dv",	"text",	     salTypeText[Type]	);
	skapi.SetControlProperty(szPanelLoot, "text5ev",	"text",	     Desc	 	);
    } // end with
} // end func


function DisplayLootWand(Index) {
    with (Loot.WandFilter[Index]) {
       	skapi.SetControlProperty(szPanelLoot, "chkWandEnab",	"checked",   Enabled    );
	skapi.SetControlProperty(szPanelLoot, "choiceLoot6a",	"selected",  MinManaC   );
	skapi.SetControlProperty(szPanelLoot, "choiceLoot6b",	"selected",  MinMeleeD  );
	skapi.SetControlProperty(szPanelLoot, "choiceLoot6c",	"selected",  AndOrFlag  );
	skapi.SetControlProperty(szPanelLoot, "choiceLoot6d",	"selected",  MinElem    );
	skapi.SetControlProperty(szPanelLoot, "choiceLoot6e",	"selected",  AndOrFlag2 );
    } // end with
} // end func


function DisplayLootItem(Index) {
    with (Loot.ItemFilter[Index]) {
	skapi.SetControlProperty(szPanelLoot, "chkItemEnab",	"checked",   Enabled	);
	skapi.SetControlProperty(szPanelLoot, "chkItemAlert",	"checked",   Alert	);
    } // end with
} // end func


function DisplayLootJewelry() {
    skapi.SetControlProperty(szPanelLoot, "choiceLoot7b", "selected",  7-Loot.JewelryFilter.MinSplLvl1	);
    skapi.SetControlProperty(szPanelLoot, "choiceLoot7c", "selected",  7-Loot.JewelryFilter.MinSplLvl2	);
    skapi.SetControlProperty(szPanelLoot, "choiceLoot7d", "selected",  7-Loot.JewelryFilter.MinSplLvl3	);
} // end func


function DisplayLootClothing() {
    skapi.SetControlProperty(szPanelLoot, "choiceLoot7g", "selected",  7-Loot.ClothingFilter.MinSplLvl1	);
    skapi.SetControlProperty(szPanelLoot, "choiceLoot7h", "selected",  7-Loot.ClothingFilter.MinSplLvl2	);
    skapi.SetControlProperty(szPanelLoot, "choiceLoot7i", "selected",  7-Loot.ClothingFilter.MinSplLvl3	);
} // end func


function DisplayLootValSalv() {
    skapi.SetControlProperty(szPanelLoot, "text7lv",	"text",	   Loot.ValSalvFilter.MinRatio		);
    skapi.SetControlProperty(szPanelLoot, "text7mv",	"text",	   Loot.ValSalvFilter.MaxBagValue	);
} // end func


function DisplayLootScroll() {
    skapi.SetControlProperty(szPanelLoot, "choiceLoot8c", "selected",  7-Loot.ScrollFilter.MinLvl	);
    skapi.SetControlProperty(szPanelLoot, "choiceLoot8d", "selected",  7-Loot.ScrollFilter.MaxLvl	);
    skapi.SetControlProperty(szPanelLoot, "chkScrollUnk", "checked",   Loot.ScrollFilter.UnknownOnly	);
    skapi.SetControlProperty(szPanelLoot, "chkLearnUnk",  "checked",   Loot.ScrollFilter.LearnUnknown	);
} // end func


function DisplayLootValuable() {
    with (Loot.ValuableFilter) {
	skapi.SetControlProperty(szPanelLoot, "text8gv",	"text",	     MinValue  	);
	skapi.SetControlProperty(szPanelLoot, "text8hv",	"text",	     MaxBU	);
	skapi.SetControlProperty(szPanelLoot, "text8iv",	"text",	     MaxTotal	);
    } // end with
} // end func

function DisplayLootDI(Index) {
    with (Loot.TypeInfo[Index]) {
    	skapi.SetControlProperty(szPanelLoot, "chkDeathEnab",   "checked",   DiEnabled   );
	skapi.SetControlProperty(szPanelLoot, "text9av",	"text",	     DiMinValue  );
	skapi.SetControlProperty(szPanelLoot, "text9bv",	"text",	     DiMaxBU  	 );
	skapi.SetControlProperty(szPanelLoot, "choiceLoot9c",	"selected",  DiMaxWork-1 );
    } // end with
} // end func


function DisplayCreature(CreIndex) {
    // Chat.Screen("DisplayCreature: CreIndex = " + CreIndex);
    
    // Display specified creature info
    with (Creature.Info[CreIndex]) {
    	// Text fields
    	//
    	if (Health > 1)
	    skapi.SetControlProperty(szPanelCreature, "text3cv", 	"text",	    Health  	);
	else
	    skapi.SetControlProperty(szPanelCreature, "text3cv", 	"text",	    "?"  	);

	if (Mana > 1)
	    skapi.SetControlProperty(szPanelCreature, "text3dv",	"text",	    Mana  	);
	else
	    skapi.SetControlProperty(szPanelCreature, "text3dv",	"text",	    "?" 	);

	if (Stamina > 1)
	    skapi.SetControlProperty(szPanelCreature, "text3ev",	"text",	    Stamina  	);
	else
	    skapi.SetControlProperty(szPanelCreature, "text3ev",	"text",	    "?" 	);

	if (Level > 1)
	    skapi.SetControlProperty(szPanelCreature, "text3fv",	"text",	    Level  	);
	else
	    skapi.SetControlProperty(szPanelCreature, "text3fv",	"text",	    "?"  	);

	if (Exp > 1)
	    skapi.SetControlProperty(szPanelCreature, "text3gv",	"text",	    Exp  	);
	else
	    skapi.SetControlProperty(szPanelCreature, "text3gv",	"text",	    "?"  	);
	
	if (MagicD > 1)
	    skapi.SetControlProperty(szPanelCreature, "text3hv",	"text",	    MagicD  	);
	else
	    skapi.SetControlProperty(szPanelCreature, "text3hv",	"text",	    "?"  	);
 
	skapi.SetControlProperty(szPanelCreature, "vuln1Choice",   "selected",	Vuln1-1		);
	skapi.SetControlProperty(szPanelCreature, "vuln2Choice",   "selected",	Vuln2		);
	skapi.SetControlProperty(szPanelCreature, "vuln3Choice",   "selected",	Vuln3		);

	skapi.SetControlProperty(szPanelCreature, "text3lv",	   "text",	Priority 	);
	
	skapi.SetControlProperty(szPanelCreature, "selChoice",     "selected",	SelFlag		);
	skapi.SetControlProperty(szPanelCreature, "attChoice",     "selected",	AttFlag		);
	skapi.SetControlProperty(szPanelCreature, "yieldChoice",   "selected",	YieldFlag	);
	skapi.SetControlProperty(szPanelCreature, "vulnChoice",    "selected",	VulnFlag	);
	skapi.SetControlProperty(szPanelCreature, "warChoice",     "selected",	WarSpell	);

	skapi.SetControlProperty(szPanelCreature, "chkNoDrainH",   "checked",	NoDrainHealth	);
	skapi.SetControlProperty(szPanelCreature, "chkNoDrainS",   "checked",	NoDrainStamina	);
	skapi.SetControlProperty(szPanelCreature, "chkNoDrainM",   "checked",	NoDrainMana	);
	skapi.SetControlProperty(szPanelCreature, "chkNoStepDown", "checked",	NoStepDown	);
	skapi.SetControlProperty(szPanelCreature, "chkDangerFlag", "checked",	DangerFlag	);
    } // end with
} // end func


function BlankCreature() {
    skapi.SetControlProperty(szPanelCreature, "text3cv", 	"text",	    "?"  	);
    skapi.SetControlProperty(szPanelCreature, "text3dv",	"text",	    "?" 	);
    skapi.SetControlProperty(szPanelCreature, "text3ev",	"text",	    "?" 	);
    skapi.SetControlProperty(szPanelCreature, "text3fv",	"text",	    "?"  	);
    skapi.SetControlProperty(szPanelCreature, "text3gv",	"text",	    "?"  	);
    skapi.SetControlProperty(szPanelCreature, "text3hv",	"text",	    "?"  	);
    skapi.SetControlProperty(szPanelCreature, "text3lv",	"text",	    "?"		);
    skapi.SetControlProperty(szPanelCreature, "chkNoDrainH",    "checked",  false	);
    skapi.SetControlProperty(szPanelCreature, "chkNoDrainS",    "checked",  false	);
    skapi.SetControlProperty(szPanelCreature, "chkNoDrainM",    "checked",  false	);
    skapi.SetControlProperty(szPanelCreature, "chkNoStepDown",  "checked",  false	);
    skapi.SetControlProperty(szPanelCreature, "chkDangerFlag",  "checked",  false	);
} // end func


function DisplayRace(RaceIndex) {
    // Chat.Screen("DisplayRace: RaceIndex = " + RaceIndex);

    // Display specified creature info
    with (Creature.RaceInfo[RaceIndex]) {
	skapi.SetControlProperty(szPanelCreature, "vulnDefChoice", "selected",	DefVuln-1	);
	skapi.SetControlProperty(szPanelCreature, "selRChoice",    "selected",	SelFlag		);
	skapi.SetControlProperty(szPanelCreature, "attRChoice",    "selected",	AttFlag		);
    } // end with
} // end func


function DisplayLootType(typeIndex) {
    with (Loot.TypeInfo[typeIndex]) {
	skapi.SetControlProperty(szPanelLoot, "chkTypeEnab",	 "checked",	Enabled);
	skapi.SetControlProperty(szPanelLoot, "lootMaxPick", 	 "selected",	MaxCount);
	skapi.SetControlProperty(szPanelLoot, "lootMinSlot",	 "selected",	MinSlots);
    } // end with
} // end func


function ShowRaceAll(OnlyDisabled) {
    if ((OnlyDisabled == undefined) || (OnlyDisabled == null)) OnlyDisabled = false;

    Chat.Screen(" ");
    var DisplayHeader = true;

    for (var x=0; x<Creature.RaceDecalCnt; x++) {
	var RaceIndex = Creature.RaceDecal[x];
	with (Creature.RaceInfo[RaceIndex]) {
	    if (OnlyDisabled) {
		if ((SelFlag != attNever) && (AttFlag != attNever)) continue;
	    } //endif
	    if (DisplayHeader) {
		Chat.Screen(Util.Width("Race Name", 2.7) + Util.Width("Def Vuln", 1.0) +
			    Util.Width("Sel Flag", 1.0)  + Util.Width("Att Flag", 1.0));
	    	DisplayHeader = false;
	    } // endif
	    Chat.Screen(Util.Width(RaceName, 2.7) + Util.Width(damName[DefVuln], 1.0) +
			Util.Width(AttText[SelFlag], 1.0)  + Util.Width(AttText[AttFlag], 1.0));
	} // end with
    } // end for
    if (DisplayHeader) {
    	if (OnlyDisabled)
	    Chat.Screen("No disabled creatures found!");
	else
	    Chat.Screen("No races found! Something is really wrong here.");
    } // endif
} // end func

function GenLoad(fileNum) {
    if (debug) Log.OutD("GenLoad: Begin");

   // Build the input filename from the filenumber.  Note filenum 0 is a special filename.
    if ((fileNum == undefined) || (fileNum == null)) fileNum = 0;
    if (fileNum == 0) {
    	GenFileName = CharCfgDir + "\\" + fileGenCfg + "def" + fileCfgExt;
    } else {
    	var Post = fileNum + fileCfgExt;
    	if (fileNum < 10) Post = "0" + Post;
    	GenFileName = CharCfgDir + "\\" + fileGenCfg + Post;
    } // endif

    // See if file exists
    if(!Fso.FileExists(GenFileName)) return false;

    // Now read config file
    Util.Include(GenFileName);
    return true;
} // end func

function GenSave(fileNum) {
    if (debug) BuffLog.OutD("GenSave: Begin");

    // Build the output filename from the filenumber.  Note filenum 0 is a special filename.
    if ((fileNum == undefined) || (fileNum == null)) fileNum = 0;
    if (fileNum == 0) {
    	GenFileName = CharCfgDir + "\\" + fileGenCfg + "def" + fileCfgExt;
    } else {
    	var Post = fileNum + fileCfgExt;
    	if (fileNum < 10) Post = "0" + Post;
    	GenFileName = CharCfgDir + "\\" + fileGenCfg + Post;
    } // endif

    GenFile = Fso.OpenTextFile(GenFileName, ForWriting, true);	// Create config file

    GenFile.WriteLine("// General configuration file created " + Util.FormatDate(0,false) + " by " + skapi.szName);
    GenFile.WriteLine("GEN_BOF = true;");

    // Gen 1 tab variables
    GenFile.WriteLine("flagLogBuffs        = " + flagLogBuffs        + ";");
    GenFile.WriteLine("flagReduceBurden    = " + flagReduceBurden    + ";");
    GenFile.WriteLine("flagDeathLog        = " + flagDeathLog        + ";");
    GenFile.WriteLine("flagDeathRebuff     = " + flagDeathRebuff     + ";");
    GenFile.WriteLine("flagAutoExcl        = " + flagAutoExcl        + ";");
    GenFile.WriteLine("flagWarAvoid        = " + flagWarAvoid        + ";");
    GenFile.WriteLine("flagFellowStealth   = " + flagFellowStealth   + ";");
    GenFile.WriteLine("flagAllegStealth    = " + flagAllegStealth    + ";");
    GenFile.WriteLine("flagAutoRespond     = " + flagAutoRespond     + ";");
    GenFile.WriteLine("flagAllowDisband    = " + flagAllowDisband    + ";");
    GenFile.WriteLine("flagAutoResume      = " + flagAutoResume      + ";");
    GenFile.WriteLine("flagAcceptFellow    = " + flagAcceptFellow    + ";");
    GenFile.WriteLine("flagGem             = " + flagGem             + ";");
    GenFile.WriteLine("flagSlideCast       = " + flagSlideCast       + ";");
    GenFile.WriteLine("anybodyFellow       = " + anybodyFellow       + ";");
    GenFile.WriteLine("passwordFellow      = " + passwordFellow      + ";");
 
    // Gen 2 tab variables
    GenFile.WriteLine("flagSelCheckZ       = " + flagSelCheckZ       + ";");
    GenFile.WriteLine("flagAssessKnown     = " + flagAssessKnown     + ";");
    GenFile.WriteLine("flagAssessUnknown   = " + flagAssessUnknown   + ";");
    GenFile.WriteLine("flagIgnoreAtt       = " + flagIgnoreAtt       + ";");
    GenFile.WriteLine("manaStoneLimit      = " + manaStoneLimit      + ";");
    GenFile.WriteLine("spellAugGems        = " + spellAugGems        + ";");
    GenFile.WriteLine("attSupportOnly      = " + attSupportOnly      + ";");
    GenFile.WriteLine("attAlwaysVuln       = " + attAlwaysVuln       + ";");
    GenFile.WriteLine("attAlwaysImperl     = " + attAlwaysImperl     + ";");
    GenFile.WriteLine("attHealFellow       = " + attHealFellow       + ";");
    GenFile.WriteLine("attStamFellow       = " + attStamFellow       + ";");
    GenFile.WriteLine("flagBuffDanger      = " + flagBuffDanger      + ";");
    GenFile.WriteLine("flagSelByDist       = " + flagSelByDist       + ";"); 

    // Move tab variables  
    GenFile.WriteLine("flagAutoHome        = " + flagAutoHome        + ";");
    GenFile.WriteLine("flagMoveDelay       = " + flagMoveDelay       + ";");  
    GenFile.WriteLine("flagRetreatCrit     = " + flagRetreatCrit     + ";");
    GenFile.WriteLine("flagMultRetreat     = " + flagMultRetreat     + ";");
    GenFile.WriteLine("flagHealthRetreat   = " + flagHealthRetreat   + ";");
    GenFile.WriteLine("flagRecordLoc       = " + flagRecordLoc       + ";");
    GenFile.WriteLine("retreatAmtOne       = " + retreatAmtOne       + ";");
    GenFile.WriteLine("retreatAmtTwo       = " + retreatAmtTwo       + ";");
    GenFile.WriteLine("retreatAmtThreePlus = " + retreatAmtThreePlus + ";");
    GenFile.WriteLine("retreatDamThresh    = " + retreatDamThresh    + ";");
    GenFile.WriteLine("MoveGroup           = " + MoveGroup	     + ";");

    // Info tab variables
    GenFile.WriteLine("flagReportPlayer    = " + flagReportPlayer    + ";");
    GenFile.WriteLine("flagReportLoot      = " + flagReportLoot      + ";");
    GenFile.WriteLine("flagReportAbort     = " + flagReportAbort     + ";");
    GenFile.WriteLine("flagDispExp         = " + flagDispExp         + ";");
    GenFile.WriteLine("flagDispSkillExp    = " + flagDispSkillExp    + ";");
    GenFile.WriteLine("flagItemSpam        = " + flagItemSpam        + ";");
    GenFile.WriteLine("flagFellowSup       = " + flagFellowSup       + ";");
    GenFile.WriteLine("flagFriendSup       = " + flagFriendSup       + ";");
    GenFile.WriteLine("flagVerbose         = " + flagVerbose         + ";");
    
    // Loot tab variables
    GenFile.WriteLine("flagLootCheck       = " + flagLootCheck       + ";");
    GenFile.WriteLine("flagLootAll         = " + flagLootAll         + ";");
    GenFile.WriteLine("flagLootWait        = " + flagLootWait        + ";");
    GenFile.WriteLine("flagAutoUst         = " + flagAutoUst         + ";");
    GenFile.WriteLine("flagMoveFull        = " + flagMoveFull        + ";");
    GenFile.WriteLine("maxBuyNote          = " + maxBuyNote          + ";");
    GenFile.WriteLine("minBuyNote          = " + minBuyNote          + ";");

    // Startup tab variables
    GenFile.WriteLine("startBuffMode       = " + startBuffMode       + ";");
    GenFile.WriteLine("startAttackMode     = " + startAttackMode     + ";");
    GenFile.WriteLine("startStealth        = " + startStealth        + ";");
    GenFile.WriteLine("startAutoMove       = " + startAutoMove       + ";");
    GenFile.WriteLine("startExclMode       = " + startExclMode       + ";");
    GenFile.WriteLine("startSafeMode       = " + startSafeMode       + ";");
    GenFile.WriteLine("startPeaceMode      = " + startPeaceMode      + ";");
    GenFile.WriteLine("creFellow           = " + creFellow           + ";");

    // Spell 1 tab variables
    GenFile.WriteLine("flagYieldDisable    = " + flagYieldDisable    + ";");
    GenFile.WriteLine("flagYieldNoMagD     = " + flagYieldNoMagD     + ";");
    GenFile.WriteLine("attVulnStepdown     = " + attVulnStepdown     + ";");
    GenFile.WriteLine("attWarStepdown      = " + attWarStepdown      + ";");
    GenFile.WriteLine("attWarStepup        = " + attWarStepup        + ";");
    GenFile.WriteLine("healStepUpThresh    = " + healStepUpThresh    + ";");
    GenFile.WriteLine("attUseWandImbue     = " + attUseWandImbue     + ";");
    GenFile.WriteLine("attMatchVuln2       = " + attMatchVuln2       + ";");
    GenFile.WriteLine("attMatchVuln3       = " + attMatchVuln3       + ";");
    GenFile.WriteLine("attSpell            = " + attSpell            + ";");
    GenFile.WriteLine("attUseRingSpells    = " + attUseRingSpells    + ";");

    // Spell 2 tab variables
    GenFile.WriteLine("maxWarSpell         = " + maxWarSpell         + ";");
    GenFile.WriteLine("minWarSpell         = " + minWarSpell         + ";");
    GenFile.WriteLine("maxYieldSpell       = " + maxYieldSpell       + ";");
    GenFile.WriteLine("maxImperlSpell      = " + maxImperlSpell      + ";");
    GenFile.WriteLine("maxVulnSpell        = " + maxVulnSpell        + ";");
    GenFile.WriteLine("spellHeal           = " + spellHeal           + ";");
    GenFile.WriteLine("spellStamHealth     = " + spellStamHealth     + ";");
    GenFile.WriteLine("spellStamMana       = " + spellStamMana       + ";");
    GenFile.WriteLine("spellRevitalize     = " + spellRevitalize     + ";");
    GenFile.WriteLine("spellStrength       = " + spellStrength       + ";");
    GenFile.WriteLine("spellDrainHealth    = " + spellDrainHealth    + ";");
    GenFile.WriteLine("spellDrainStamina   = " + spellDrainStamina   + ";");
    GenFile.WriteLine("spellDrainMana      = " + spellDrainMana      + ";");
 
    GenFile.WriteLine("flagUseTuskerSpit   = " + flagUseTuskerSpit   + ";");
    GenFile.WriteLine("flagUseAmberApe     = " + flagUseAmberApe     + ";");
    GenFile.WriteLine("flagUseBoboStout    = " + flagUseBoboStout    + ";");

    // Component tab variables
    GenFile.WriteLine("flagCompCheck       = " + flagCompCheck       + ";");
    GenFile.WriteLine("flagCompLogOff      = " + flagCompLogOff      + ";");
    GenFile.WriteLine("flagCompAssist      = " + flagCompAssist      + ";");
    GenFile.WriteLine("flagCompRetreat     = " + flagCompRetreat     + ";");
    GenFile.WriteLine("flagSplitPeas       = " + flagSplitPeas       + ";");
    for (var x=0; x<Comp.Info.length; x++) {
    	with (Comp.Info[x]) {
	    if (Enabled || (SplitLvl != -1) || (CritLvl != -1) || (WarnLvl != -1)) {
		GenFile.WriteLine("Comp.SetMin('" + Name + "', " + Enabled + ", " + SplitLvl + ", " +
	    			   CritLvl + ", " + WarnLvl + ", " + BuyLvl + ");")
	    } // endif
	} // end with
    } // end for

    // Alert tab variables
    GenFile.WriteLine("alertTell           = " + alertTell           + ";");
    GenFile.WriteLine("alertAlleg          = " + alertAlleg          + ";");
    GenFile.WriteLine("alertDeath          = " + alertDeath          + ";");
    GenFile.WriteLine("alertPlayer         = " + alertPlayer         + ";");
    GenFile.WriteLine("alertStealth        = " + alertStealth        + ";");
    GenFile.WriteLine("alertComps          = " + alertComps          + ";");
    GenFile.WriteLine("alertItem           = " + alertItem           + ";");
    GenFile.WriteLine("alertAbort          = " + alertAbort          + ";");
    GenFile.WriteLine("alertAdmin          = " + alertAdmin          + ";");

   // Debug tab variables
    GenFile.WriteLine("debug               = " + debug               + ";");
    GenFile.WriteLine("debugEvent          = " + debugEvent          + ";");
    GenFile.WriteLine("debugState          = " + debugState          + ";");
    GenFile.WriteLine("debugSelect         = " + debugSelect         + ";");
    GenFile.WriteLine("debugTarget         = " + debugTarget         + ";");
    GenFile.WriteLine("debugAttack         = " + debugAttack         + ";");
    GenFile.WriteLine("debugMove           = " + debugMove           + ";");
    GenFile.WriteLine("debugLoot           = " + debugLoot           + ";");
    GenFile.WriteLine("debugSalvage        = " + debugSalvage        + ";");
    GenFile.WriteLine("debugIdle           = " + debugIdle           + ";");
    GenFile.WriteLine("debugBuffs          = " + debugBuffs          + ";");
    GenFile.WriteLine("debugCast           = " + debugCast           + ";");

    // Write End Of File marker
    GenFile.WriteLine("GEN_EOF = true;");

    // All done, close file and exit
    GenFile.Close();
    return true;
} // end func


function CreLoad(fileNum) {
    if (debug) Log.OutD("CreLoad: Begin");

    // Build the input filename from the filenumber.  Note filenum 0 is a special filename.
    if ((fileNum == undefined) || (fileNum == null)) fileNum = 0;
    if (fileNum == 0) {
    	CreFileName = CharCfgDir + "\\" + fileCreCfg + "def" + fileCfgExt;
    } else {
    	var Post = fileNum + fileCfgExt;
    	if (fileNum < 10) Post = "0" + Post;
    	CreFileName = CharCfgDir + "\\" + fileCreCfg + Post;
    } // endif

    // See if file exists
    if(!Fso.FileExists(CreFileName)) return false;

    // Now read config file
    Util.Include(CreFileName);
    return true;
} // end func

function CreSave(fileNum) {
    if (debug) BuffLog.OutD("GenSave: Begin");

    // Build the output filename from the filenumber.  Note filenum 0 is a special filename.
    if ((fileNum == undefined) || (fileNum == null)) fileNum = 0;
    if (fileNum == 0) {
    	CreFileName = CharCfgDir + "\\" + fileCreCfg + "def" + fileCfgExt;
    } else {
    	var Post = fileNum + fileCfgExt;
    	if (fileNum < 10) Post = "0" + Post;
    	CreFileName = CharCfgDir + "\\" + fileCreCfg + Post;
    } // endif

    CreFile = Fso.OpenTextFile(CreFileName, ForWriting, true);	// Create config file

    CreFile.WriteLine("// Creature configuration file created " + Util.FormatDate(0,false) + " by " + skapi.szName);
    CreFile.WriteLine("CRE_BOF = true;");

    // Save creature related variables
    CreFile.WriteLine("selMinExp           = " + selMinExp           + ";");

    // Save Race info for any record not equal to warbot defaults
    for (var x=0; x<Creature.RaceDecalCnt; x++) {
    	var RaceIdx = Creature.RaceDecal[x];
    	with (Creature.RaceInfo[RaceIdx]) {
	    if ((OrigDefVuln != DefVuln) || (OrigSelFlag != SelFlag) || (OrigAttFlag != AttFlag))
		CreFile.WriteLine("Creature.SetRace2(\"" + RaceName + "\", " +
					DefVuln + ", " + SelFlag + ", " + AttFlag + ");" );
	} // end with
    } // end for

    // Save creature info
    for (var x=1; x<Creature.CreCnt; x++) {
	with (Creature.Info[x]) {
    	    if ((OrigVuln1 != Vuln1 ) || (OrigVuln2 != Vuln2 ) || (OrigVuln3 != Vuln3 ) ||
		(OrigPriority != Priority ) || (OrigSelFlag != SelFlag ) || (OrigAttFlag != AttFlag ) ||
		(OrigYieldFlag != YieldFlag ) || (OrigVulnFlag != VulnFlag ) || (OrigWarSpell != WarSpell ) ||
		(OrigNoDrainHealth != NoDrainHealth ) || (OrigNoDrainStamina != NoDrainStamina ) ||
		(OrigNoDrainMana != NoDrainMana ) || (OrigNoStepDown != NoStepDown ) || (OrigDangerFlag != DangerFlag ))
		CreFile.WriteLine("Creature.SetCre2(\"" + Name + "\", " +
					Vuln1 + ", " + Vuln2 + ", " + Vuln3 + ", " +  Priority + ", " + 
					SelFlag + ", " + AttFlag + ", " + YieldFlag + ", " + VulnFlag + ", " + 
					WarSpell + ", " + NoDrainHealth + ", " + NoDrainStamina + ", " + 
					NoDrainMana + ", " + NoStepDown + ", " + DangerFlag + ");" );
	} // endwith
    } // end for

    // Write End Of File marker
    CreFile.WriteLine("CRE_EOF = true;");

    // All done, close file and exit
    CreFile.Close();
    return true;
} // end func


function CreateUserConfig(fileName) {
    UserFile = Fso.OpenTextFile(fileName, ForWriting, true);	// Create config file
    UserFile.WriteLine('// User Configuration File');
    UserFile.WriteLine('//');
    UserFile.WriteLine('// If you want to modify a variable in WAR_CONFIG.JS, then copy the variable');
    UserFile.WriteLine('// to this file.  If you modify the variable directly in WAR_CONFIG.JS then');
    UserFile.WriteLine('// you will lose the modification the next upgrade. ');
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Some common variable modifications are listed below.');
    UserFile.WriteLine(' ');
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// Protected items will never be salvaged or sold');
    UserFile.WriteLine('// protectedItems = new Array();');
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// People on this list can give your bot commands');
    UserFile.WriteLine('// authList = new Array("Boy Gary", "Crackhead Bob", "Robin");');
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// Chars on the friends list will be ignored by the stealth routine');
    UserFile.WriteLine('// friendsList = new Array("Boy Gary", "Crackhead Bob", "Robin");');
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// Chars who belong to an allegiance on allegianceList will be ignored by the stealth routine');
    UserFile.WriteLine('// allegianceList = new Array("Psycho Killers", "Natural Born Killers");');
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// Creatures on the attYieldList will always be yielded');
    UserFile.WriteLine('// attYieldList= new Array("Virindi Consul", "Platinum Golem");');
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// Creatures on the attVulnList will always be vulned.  Only comes into play');
    UserFile.WriteLine('// when using a RR wand and you want to make sure difficult creatures are vulned.');
    UserFile.WriteLine('// attVulnList  = new Array("Platinum Golem", "Assailer");');
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// Never stepdown life or war spells for creatures on this list.');
    UserFile.WriteLine('// Useful for creatures that regain health quickly.');
    UserFile.WriteLine('// attNoStepDown = new Array();');
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// Creatures listed in the following array will be killed as fast as'                    );
    UserFile.WriteLine('// possible. Drain stam/mana spells will be disabled, and the threshold for'             );
    UserFile.WriteLine('// stam to mana will be reduced.  If creature moves within our danger radius,'           );
    UserFile.WriteLine('// then stepup maximum war spell to 7s, ie use highest spell we can reliable'            );
    UserFile.WriteLine('// cast.  Order in this list is unimportant.'                                            );
    UserFile.WriteLine('// attDangerList = new Array('                                                           );
    UserFile.WriteLine('//            "Assailer", "Rampager", "Sotiris Hollow Minion",'                          );
    UserFile.WriteLine('//            "Telumiat Hollow Minion", "Terebrous Hollow Minion",'                      );
    UserFile.WriteLine('//            "Virindi Profatrix", "Virindi Inquisitor", "Drudge Bloodletter",'          );
    UserFile.WriteLine('//            "Shadow Wraith", "Shadow Phantom", "Shadow Spectre", "Abyssal Shadow",'    );
    UserFile.WriteLine('//            "Platinum Golem", "Tempest Wisp", "Entropy Wisp", "Olthoi Swarm Matron");' );
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// When macroing with a partner, one person should reverse order of selection'   );
    UserFile.WriteLine('// to minimize kill attempt collisions (ie going for same creature)."'           );
    UserFile.WriteLine('// selPrioList = new Array('                                                     );
    UserFile.WriteLine('//            "Pandemonium Shadow", "Paroxysm Shadow", "Maelstrom Shadow",'      );
    UserFile.WriteLine('//            "Sinister Shadow", "Tenebrous Shadow", "Abyssal Shadow",'          );
    UserFile.WriteLine('//            "Shadow Spectre", "Shadow Phantom", "Shadow Wraith",'              );
    UserFile.WriteLine('//            "Olthoi Warrior", "Olthoi Mutilator", "Rampager", "Assailer",'     );
    UserFile.WriteLine('//            "Terebrous Hollow Minion", "Telumiat Hollow Minion",'              );
    UserFile.WriteLine('//            "Pyreal Golem", "Platinum Golem", "Vapor Golem", "Plasma Golem");' );
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// Time to hide when an unknown player is detected (in milliseconds)' );
    UserFile.WriteLine('// hideInt = 10*60*1000; // Hide for 10 minutes' );
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// When an unknown player sends you a tell, respond with this if option enabled');
    UserFile.WriteLine('// tellResponse	= "Unrecognized command. Valid commands are SUMMON PRIMARY and SUMMON SECONDARY";');
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// Fellowship name if create fellow at startup option enabled' );
    UserFile.WriteLine('// fellowName = "Exp";' );
    UserFile.WriteLine(' ');
    UserFile.WriteLine('// Default password for fellow' );
    UserFile.WriteLine('// fellowPassword = "boygary";' );
    UserFile.WriteLine('//');

    UserFile.Close();
    return;
} // end function


function CreateEquipConfig(fileName) {
    Chat.Screen(" fileName [" + fileName + "]");
    UserFile = Fso.OpenTextFile(fileName, ForWriting, true);	// Create config file
    UserFile.WriteLine('// Equipment Configuration File');
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Put any items in here you always want buffed whether you are wearing it');
    UserFile.WriteLine('// or not.  Typical use is to buff weapons or extra wands.  Note the burden');
    UserFile.WriteLine('// and value fields are used to uniquely identift an item when more than');
    UserFile.WriteLine('// one item matches an item name.  They are not required but if they are');
    UserFile.WriteLine('// not used and more than one item matches a name, then both items will be');
    UserFile.WriteLine('// buffed.');
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Each line sets one spell, so use multiple lines if multiple spells required.');
    UserFile.WriteLine('//');
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Format: Item Name, Value, Burden, Spell Name (from Buff.csv file), Level,');
    UserFile.WriteLine('//          Priority, Safety, Randomize');
    UserFile.WriteLine('//');
    UserFile.WriteLine('// For example:');
    UserFile.WriteLine('//');
    UserFile.WriteLine('// Pyreal Flaming Sword, 2834, 453, Blood Drinker, 7, 100, 180, false');
    UserFile.WriteLine('// Pyreal Flaming Sword, 2834, 453, Defender, 7, 100, 180, false');
    UserFile.WriteLine('//');
    UserFile.Close();
    return;
} // endif

function MoveOldCfgFiles() {
    // First move old default files which had character names in them to file DEF
    if (Fso.FileExists(fileOldGenCfgPre  + skapi.szName + fileOldGenCfgPost))
	Fso.MoveFile(fileOldGenCfgPre  + skapi.szName + fileOldGenCfgPost,  CharCfgDir + "\\" + fileGenCfg  + "def.dat");
	
    if (Fso.FileExists(fileOldBuffCfgPre  + skapi.szName + fileOldBuffCfgPost))
	Fso.MoveFile(fileOldBuffCfgPre + skapi.szName + fileOldBuffCfgPost, CharCfgDir + "\\" + fileBuffCfg + "def.dat");
	
    if (Fso.FileExists(fileOldLootCfgPre  + skapi.szName + fileOldLootCfgPost))
	Fso.MoveFile(fileOldLootCfgPre + skapi.szName + fileOldLootCfgPost, CharCfgDir + "\\" + fileLootCfg + "def.dat");
	
    if (Fso.FileExists(fileOldUserCfgPre  + skapi.szName + fileOldUserCfgPost))
	Fso.MoveFile(fileOldUserCfgPre + skapi.szName + fileOldUserCfgPost, CharCfgDir + "\\" + fileUserCfg + ".dat");

    for (var x=1; x<=navMaxFiles; x++) {
	if (x<10) {
	    var infile  = fileOldNavCfgPre + "0" + x + fileOldNavCfgPost;
	    var outfile = CharCfgDir + "\\" + fileNavCfg + "0" + x + fileCfgExt;
	} else {
	    var infile  = fileOldNavCfgPre + x + fileOldNavCfgPost;
	    var outfile = CharCfgDir + "\\" + fileNavCfg + x + fileCfgExt;
	} // endif
	if (Fso.FileExists(infile)) Fso.CopyFile(infile, outfile);
    } // end for

    for (var x=1; x<=genMaxFiles; x++) {
	if (x<10) {
	    var infile  = fileOldGenCfgPre + "0" + x + fileOldGenCfgPost;
	    var outfile = CharCfgDir + "\\" + fileGenCfg + "0" + x + fileCfgExt;
	} else {
	    var infile  = fileOldGenCfgPre + x + fileOldGenCfgPost;
	    var outfile = CharCfgDir + "\\" + fileGenCfg + x + fileCfgExt;
	} // endif
	if (Fso.FileExists(infile)) Fso.CopyFile(infile, outfile);
    } // end for

    for (var x=1; x<=lootMaxFiles; x++) {
	if (x<10) {
	    var infile  = fileOldLootCfgPre + "0" + x + fileOldLootCfgPost;
	    var outfile = CharCfgDir + "\\" + fileLootCfg + "0" + x + fileCfgExt;
	} else {
	    var infile  = fileOldLootCfgPre + x + fileOldLootCfgPost;
	    var outfile = CharCfgDir + "\\" + fileLootCfg + x + fileCfgExt;
	} // endif
	    if (Fso.FileExists(infile)) Fso.CopyFile(infile, outfile);
    } // end for

    for (var x=1; x<=buffMaxFiles; x++) {
	if (x<10) {
	    var infile  = fileOldBuffCfgPre + "0" + x + fileOldBuffCfgPost;
	    var outfile = CharCfgDir + "\\" + fileBuffCfg + "0" + x + fileCfgExt;
	} else {
	    var infile  = fileOldBuffCfgPre + x + fileOldBuffCfgPost;
	    var outfile = CharCfgDir + "\\" + fileBuffCfg + x + fileCfgExt;
	} // endif
	if (Fso.FileExists(infile)) Fso.CopyFile(infile, outfile);
    } // end for
} // end func